#! /bin/sh
#
#  Controller script to run multiple CIAO Smoke Tests.
#  This script finds and executes the individual test scripts
#  Results are monitored and a global result is presented. 
#
#  The expectation is that this script sits with the 
#  individual test scripts.
#
# ----------------------------------------------------------------------
get_id_string()
{
  ID=$1

  case $ID in 
    chips*)
       IDString="CXC Chips Modules";;
    crates*)
       IDString="CXC Crates library";;
    dm*)
       IDString="CXC DataModel library";;
    tools*)
       IDString="CIAO Tools";;
    transform*)
       IDString="CXC Transform library";;
    ui*)
       IDString="CXC User Infrastructure";;
    *)
       IDString="All";;
  esac
}
# ----------------------------------------------------------------------
THISDIR=`dirname $0`; export THISDIR
IDString=""

# Check for optional argument to specify a subset of
# tests to run by ID pattern match.
ID=""
if test $# -gt 0
then
  ID=$1
fi
get_id_string $ID

# Generate a list of tests to run
#  (specifically exclude certain files )
TESTLIST=`ls ${THISDIR}/${ID}*.sh | egrep -v "run_|runtime|_template|setup"`

#
# Print out a little banner
#
echo ""
echo "*************************************************************"
echo "${IDString} smoke tests.."

#
# Run each test
#
anybad=0
for tfile in $TESTLIST
do
  # execute the test script
  eval $tfile

  # check results
  if test $? != 0
  then
    anybad=1
  fi

done

#
# Set global results..
#
grade="FAIL"
if test $anybad = 0
then
  grade="PASS"
fi

#
# Show global results and close banner
#
echo ""
echo "${IDString} smoke tests completed:  $grade "
echo "*************************************************************"

exit $anybad
