# 
#  Copyright (C) 2007  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


from pycrates import CrateDict
from pycrates import CrateSubspaceData
import pycrates.io as io

class CrateSubspace:

    def __init__(self):
        self.__clear__()


    def __clear__(self):
        self.name = ""
        self.__ssdatalist = CrateDict()
        

    def __str__(self):
        return self.__repr__()


    def __repr__(self):
        retstr = ""
        for sskey in (self.__ssdatalist.keys()):
            retstr += str(self.__ssdatalist[sskey]) + "\n"
        return retstr

        
    def add_subspace_data(self, data):
        """
        Appends input subspace data item to the subspace dictionary.
        """

        if isinstance(data, CrateSubspaceData):
            self.__ssdatalist[data.name] = data


    def get_subspace_datadict( self ):
        """
        Returns the dictionary of subspace data items.
        """
        return self.__ssdatalist


    def get_subspace_data( self, item ):
        """
        Retrieves the specified subspace data item (either a string or a number). 
        """
        if item is None: 
            return
        
        item_str = None

        if isinstance(item, str):
            for name in self.__ssdatalist.keys():
                if name.lower() == item.lower():
                    item_str = name
                    break

        elif isinstance(item, int):
            if item > 0 and item < self.__ssdatalist.nkeys():
                for ii in range( self.__ssdatalist.nkeys() ):
                    if ii == item:
                        item_str = name
                        break
        else:
            raise IndexError("Cannot find subspace item " + str(item) )

        return self.__ssdatalist[item_str]


    def get_subspace_ndata(self):
        """
        Returns the total number of data items in the subspace component.
        """
        return self.__ssdatalist.nkeys()
