require ("pysl");
require ("transform");
py_import ("pycrates");

private define _crates_push ( arg ) { return arg; }

%Check for return. Push tuples on the list individually
private define check_return ()
{
  variable ret_val=NULL;
  if (_stkdepth () > 0){
     ret_val = ();
     if (py_call(py.type,ret_val) == py.tuple){
        variable len = py_call (py.len (ret_val))-1;
        variable ii;
        _for (0, len, 1){
            ii = ();
           _crates_push (ret_val[ii]);
        }
     }
  }
  return ret_val;
}

%Create valid types for python
private define _crates_check_type (value);
private define _crates_check_type (value)
{
  if (typeof (value) == Char_Type)
     value = int (value);
  return value;
}
%Turn a structure into a list
private define _crates_struct_to_list (s);
private define _crates_struct_to_list (s)
{
  variable sl_list = {};
  variable name, value;
  variable prefix = NULL;
  if (_NARGS > 1)
     prefix = ();
  try{
     %Go through each element in the structure and make it a list
     foreach (get_struct_field_names (s)){
        name = ();
        value = get_struct_field (s, name);
        if (prefix != NULL)
           name = prefix + "." + name;
        %Expand out sub structures
        if (is_struct_type (value)){
           variable tmp_list = _crates_struct_to_list (name,value);
           variable ii;
           %Concatinate the two lists
           _for (length (tmp_list)-1, 0, -1){
              ii = ();
              list_insert (sl_list, tmp_list[ii], 0);
           }
        }
        else{
           value = _crates_check_type (value);
           list_insert (sl_list,value,0);
           list_insert (sl_list,name,0);
        }
     }
  }
  catch RunTimeError:
     %Return NULL if this was not a structure
     return NULL;
  return sl_list;
}
#if (_slang_version >= 20101)
%Return all the qualifiers in a struct
private define _crates_get_qualifiers (kwstruct);
private define _crates_get_qualifiers (kwstruct)
{
  if( kwstruct == NULL )
    return NULL;
  variable ii, name;
  variable kwnames = get_struct_field_names(kwstruct);
  variable kwargs = Struct_Type[2 * length(kwnames)];
  ii = 0;
  foreach (kwnames)
  {
     name = ();
     kwargs[ii] = struct{value=name};
     ii = ii + 1;
     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};
     ii = ii + 1;
   }
 return kwargs;
 }
#endif
%Crates HLUI Functions
define BUFFER_LENGTH () { return py.pycrates.BUFFER_LENGTH; }

define Crate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.Crate, __push_args (f_args));
        else
        py_call(py.pycrates.Crate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.Crate);
       else
         py_call(py.pycrates.Crate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define CrateData ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.CrateData, __push_args (f_args));
        else
        py_call(py.pycrates.CrateData, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.CrateData);
       else
         py_call(py.pycrates.CrateData, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define CrateDataset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.CrateDataset, __push_args (f_args));
        else
        py_call(py.pycrates.CrateDataset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.CrateDataset);
       else
         py_call(py.pycrates.CrateDataset, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define CrateDict ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.CrateDict, __push_args (f_args));
        else
        py_call(py.pycrates.CrateDict, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.CrateDict);
       else
         py_call(py.pycrates.CrateDict, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define CrateKey ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.CrateKey, __push_args (f_args));
        else
        py_call(py.pycrates.CrateKey, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.CrateKey);
       else
         py_call(py.pycrates.CrateKey, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define DT_BIT () { return py.pycrates.DT_BIT; }

define DT_CHAR () { return py.pycrates.DT_CHAR; }

define DT_INT2 () { return py.pycrates.DT_INT2; }

define DT_INT4 () { return py.pycrates.DT_INT4; }

define DT_LOGICAL () { return py.pycrates.DT_LOGICAL; }

define DT_PTR () { return py.pycrates.DT_PTR; }

define DT_REAL4 () { return py.pycrates.DT_REAL4; }

define DT_REAL8 () { return py.pycrates.DT_REAL8; }

define DT_UINT1 () { return py.pycrates.DT_UINT1; }

define DT_UINT2 () { return py.pycrates.DT_UINT2; }

define DT_UINT4 () { return py.pycrates.DT_UINT4; }

define DT_UNK () { return py.pycrates.DT_UNK; }

define DT_VOID () { return py.pycrates.DT_VOID; }

define IMAGECrate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.IMAGECrate, __push_args (f_args));
        else
        py_call(py.pycrates.IMAGECrate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.IMAGECrate);
       else
         py_call(py.pycrates.IMAGECrate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define MAX_LENGTH () { return py.pycrates.MAX_LENGTH; }

define PHACrateDataset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.PHACrateDataset, __push_args (f_args));
        else
        py_call(py.pycrates.PHACrateDataset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.PHACrateDataset);
       else
         py_call(py.pycrates.PHACrateDataset, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define PySwigIterator ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.PySwigIterator, __push_args (f_args));
        else
        py_call(py.pycrates.PySwigIterator, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.PySwigIterator);
       else
         py_call(py.pycrates.PySwigIterator, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define REGULAR () { return py.pycrates.REGULAR; }

define RMFCrateDataset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.RMFCrateDataset, __push_args (f_args));
        else
        py_call(py.pycrates.RMFCrateDataset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.RMFCrateDataset);
       else
         py_call(py.pycrates.RMFCrateDataset, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define TABLECrate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.TABLECrate, __push_args (f_args));
        else
        py_call(py.pycrates.TABLECrate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.TABLECrate);
       else
         py_call(py.pycrates.TABLECrate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define UserDict ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.UserDict, __push_args (f_args));
        else
        py_call(py.pycrates.UserDict, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.UserDict);
       else
         py_call(py.pycrates.UserDict, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define VIRTUAL () { return py.pycrates.VIRTUAL; }

define add_col ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.add_col, __push_args (f_args));
        else
        py_call(py.pycrates.add_col, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.add_col);
       else
         py_call(py.pycrates.add_col, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define add_crate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.add_crate, __push_args (f_args));
        else
        py_call(py.pycrates.add_crate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.add_crate);
       else
         py_call(py.pycrates.add_crate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define add_key ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.add_key, __push_args (f_args));
        else
        py_call(py.pycrates.add_key, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.add_key);
       else
         py_call(py.pycrates.add_key, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define add_piximg ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.add_piximg, __push_args (f_args));
        else
        py_call(py.pycrates.add_piximg, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.add_piximg);
       else
         py_call(py.pycrates.add_piximg, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define array ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.array, __push_args (f_args));
        else
        py_call(py.pycrates.array, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.array);
       else
         py_call(py.pycrates.array, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define check_crate_modified ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.check_crate_modified, __push_args (f_args));
        else
        py_call(py.pycrates.check_crate_modified, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.check_crate_modified);
       else
         py_call(py.pycrates.check_crate_modified, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define check_cratedata_modified ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.check_cratedata_modified, __push_args (f_args));
        else
        py_call(py.pycrates.check_cratedata_modified, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.check_cratedata_modified);
       else
         py_call(py.pycrates.check_cratedata_modified, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define col_exists ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.col_exists, __push_args (f_args));
        else
        py_call(py.pycrates.col_exists, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.col_exists);
       else
         py_call(py.pycrates.col_exists, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define convert_row_to_pha1 ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.convert_row_to_pha1, __push_args (f_args));
        else
        py_call(py.pycrates.convert_row_to_pha1, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.convert_row_to_pha1);
       else
         py_call(py.pycrates.convert_row_to_pha1, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define copy_colvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.copy_colvals, __push_args (f_args));
        else
        _py_call(py.pycrates.copy_colvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.copy_colvals);
       else
         _py_call(py.pycrates.copy_colvals, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define copy_piximgvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.copy_piximgvals, __push_args (f_args));
        else
        _py_call(py.pycrates.copy_piximgvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.copy_piximgvals);
       else
         _py_call(py.pycrates.copy_piximgvals, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define delete_col ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.delete_col, __push_args (f_args));
        else
        py_call(py.pycrates.delete_col, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.delete_col);
       else
         py_call(py.pycrates.delete_col, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define delete_crate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.delete_crate, __push_args (f_args));
        else
        py_call(py.pycrates.delete_crate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.delete_crate);
       else
         py_call(py.pycrates.delete_crate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define delete_key ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.delete_key, __push_args (f_args));
        else
        py_call(py.pycrates.delete_key, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.delete_key);
       else
         py_call(py.pycrates.delete_key, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define delete_piximg ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.delete_piximg, __push_args (f_args));
        else
        py_call(py.pycrates.delete_piximg, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.delete_piximg);
       else
         py_call(py.pycrates.delete_piximg, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define dmFAILURE () { return py.pycrates.dmFAILURE; }

define dmSUCCESS () { return py.pycrates.dmSUCCESS; }

define get_axis_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_axis_transform, __push_args (f_args));
        else
        _py_call(py.pycrates.get_axis_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_axis_transform);
       else
         _py_call(py.pycrates.get_axis_transform, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_col ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_col, __push_args (f_args));
        else
        py_call(py.pycrates.get_col, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_col);
       else
         py_call(py.pycrates.get_col, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_col_names ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_col_names, __push_args (f_args));
        else
        _py_call(py.pycrates.get_col_names, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_col_names);
       else
         _py_call(py.pycrates.get_col_names, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_colvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_colvals, __push_args (f_args));
        else
        _py_call(py.pycrates.get_colvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_colvals);
       else
         _py_call(py.pycrates.get_colvals, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_crate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_crate, __push_args (f_args));
        else
        py_call(py.pycrates.get_crate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_crate);
       else
         py_call(py.pycrates.get_crate, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_crate_checksum ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_crate_checksum, __push_args (f_args));
        else
        py_call(py.pycrates.get_crate_checksum, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_crate_checksum);
       else
         py_call(py.pycrates.get_crate_checksum, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_crate_item_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_crate_item_type, __push_args (f_args));
        else
        _py_call(py.pycrates.get_crate_item_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_crate_item_type);
       else
         _py_call(py.pycrates.get_crate_item_type, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_crate_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_crate_type, __push_args (f_args));
        else
        _py_call(py.pycrates.get_crate_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_crate_type);
       else
         _py_call(py.pycrates.get_crate_type, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_cratedata_checksum ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_cratedata_checksum, __push_args (f_args));
        else
        py_call(py.pycrates.get_cratedata_checksum, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_cratedata_checksum);
       else
         py_call(py.pycrates.get_cratedata_checksum, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_dmType ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_dmType, __push_args (f_args));
        else
        py_call(py.pycrates.get_dmType, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_dmType);
       else
         py_call(py.pycrates.get_dmType, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_dmType_str ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_dmType_str, __push_args (f_args));
        else
        py_call(py.pycrates.get_dmType_str, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_dmType_str);
       else
         py_call(py.pycrates.get_dmType_str, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_key ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_key, __push_args (f_args));
        else
        py_call(py.pycrates.get_key, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_key);
       else
         py_call(py.pycrates.get_key, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_key_names ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_key_names, __push_args (f_args));
        else
        _py_call(py.pycrates.get_key_names, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_key_names);
       else
         _py_call(py.pycrates.get_key_names, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_keyval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_keyval, __push_args (f_args));
        else
        _py_call(py.pycrates.get_keyval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_keyval);
       else
         _py_call(py.pycrates.get_keyval, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_number_cols ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_number_cols, __push_args (f_args));
        else
        py_call(py.pycrates.get_number_cols, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_number_cols);
       else
         py_call(py.pycrates.get_number_cols, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_number_rows ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_number_rows, __push_args (f_args));
        else
        py_call(py.pycrates.get_number_rows, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_number_rows);
       else
         py_call(py.pycrates.get_number_rows, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_piximg ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_piximg, __push_args (f_args));
        else
        py_call(py.pycrates.get_piximg, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_piximg);
       else
         py_call(py.pycrates.get_piximg, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_piximg_shape ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_piximg_shape, __push_args (f_args));
        else
        py_call(py.pycrates.get_piximg_shape, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_piximg_shape);
       else
         py_call(py.pycrates.get_piximg_shape, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_piximgvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_piximgvals, __push_args (f_args));
        else
        _py_call(py.pycrates.get_piximgvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_piximgvals);
       else
         _py_call(py.pycrates.get_piximgvals, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_pyType ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_pyType, __push_args (f_args));
        else
        py_call(py.pycrates.get_pyType, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_pyType);
       else
         py_call(py.pycrates.get_pyType, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.get_transform, __push_args (f_args));
        else
        py_call(py.pycrates.get_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.get_transform);
       else
         py_call(py.pycrates.get_transform, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define get_transform_matrix ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.get_transform_matrix, __push_args (f_args));
        else
        _py_call(py.pycrates.get_transform_matrix, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.get_transform_matrix);
       else
         _py_call(py.pycrates.get_transform_matrix, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define is_pha ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.is_pha, __push_args (f_args));
        else
        py_call(py.pycrates.is_pha, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.is_pha);
       else
         py_call(py.pycrates.is_pha, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define is_pha_type1 ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.is_pha_type1, __push_args (f_args));
        else
        py_call(py.pycrates.is_pha_type1, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.is_pha_type1);
       else
         py_call(py.pycrates.is_pha_type1, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define is_pha_type2 ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.is_pha_type2, __push_args (f_args));
        else
        py_call(py.pycrates.is_pha_type2, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.is_pha_type2);
       else
         py_call(py.pycrates.is_pha_type2, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define is_rmf ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.is_rmf, __push_args (f_args));
        else
        py_call(py.pycrates.is_rmf, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.is_rmf);
       else
         py_call(py.pycrates.is_rmf, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define is_virtual ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.is_virtual, __push_args (f_args));
        else
        py_call(py.pycrates.is_virtual, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.is_virtual);
       else
         py_call(py.pycrates.is_virtual, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define key_exists ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.key_exists, __push_args (f_args));
        else
        py_call(py.pycrates.key_exists, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.key_exists);
       else
         py_call(py.pycrates.key_exists, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define print_axis_names ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.print_axis_names, __push_args (f_args));
        else
        py_call(py.pycrates.print_axis_names, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.print_axis_names);
       else
         py_call(py.pycrates.print_axis_names, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define print_col_names ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.print_col_names, __push_args (f_args));
        else
        py_call(py.pycrates.print_col_names, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.print_col_names);
       else
         py_call(py.pycrates.print_col_names, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define print_key_names ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.print_key_names, __push_args (f_args));
        else
        py_call(py.pycrates.print_key_names, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.print_key_names);
       else
         py_call(py.pycrates.print_key_names, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define read_file ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.read_file, __push_args (f_args));
        else
        _py_call(py.pycrates.read_file, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.read_file);
       else
         _py_call(py.pycrates.read_file, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define read_pha ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.read_pha, __push_args (f_args));
        else
        _py_call(py.pycrates.read_pha, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.read_pha);
       else
         _py_call(py.pycrates.read_pha, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define read_rmf ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pycrates.read_rmf, __push_args (f_args));
        else
        _py_call(py.pycrates.read_rmf, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pycrates.read_rmf);
       else
         _py_call(py.pycrates.read_rmf, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define set_colvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.set_colvals, __push_args (f_args));
        else
        py_call(py.pycrates.set_colvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.set_colvals);
       else
         py_call(py.pycrates.set_colvals, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define set_keyval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.set_keyval, __push_args (f_args));
        else
        py_call(py.pycrates.set_keyval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.set_keyval);
       else
         py_call(py.pycrates.set_keyval, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define set_piximgvals ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.set_piximgvals, __push_args (f_args));
        else
        py_call(py.pycrates.set_piximgvals, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.set_piximgvals);
       else
         py_call(py.pycrates.set_piximgvals, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define update_all_cratedata_checksums ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.update_all_cratedata_checksums, __push_args (f_args));
        else
        py_call(py.pycrates.update_all_cratedata_checksums, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.update_all_cratedata_checksums);
       else
         py_call(py.pycrates.update_all_cratedata_checksums, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define update_crate_checksum ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.update_crate_checksum, __push_args (f_args));
        else
        py_call(py.pycrates.update_crate_checksum, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.update_crate_checksum);
       else
         py_call(py.pycrates.update_crate_checksum, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define update_cratedata_checksum ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.update_cratedata_checksum, __push_args (f_args));
        else
        py_call(py.pycrates.update_cratedata_checksum, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.update_cratedata_checksum);
       else
         py_call(py.pycrates.update_cratedata_checksum, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define write_file ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.write_file, __push_args (f_args));
        else
        py_call(py.pycrates.write_file, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.write_file);
       else
         py_call(py.pycrates.write_file, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define write_pha ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.write_pha, __push_args (f_args));
        else
        py_call(py.pycrates.write_pha, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.write_pha);
       else
         py_call(py.pycrates.write_pha, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define write_pha1_files ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.write_pha1_files, __push_args (f_args));
        else
        py_call(py.pycrates.write_pha1_files, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.write_pha1_files);
       else
         py_call(py.pycrates.write_pha1_files, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define write_rmf ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.write_rmf, __push_args (f_args));
        else
        py_call(py.pycrates.write_rmf, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.write_rmf);
       else
         py_call(py.pycrates.write_rmf, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

define write_row_as_pha1 ()
{
#if (_slang_version >= 20101)
    variable kwargs = _crates_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _crates_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _crates_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pycrates.write_row_as_pha1, __push_args (f_args));
        else
        py_call(py.pycrates.write_row_as_pha1, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pycrates.write_row_as_pha1);
       else
         py_call(py.pycrates.write_row_as_pha1, PY_KW,  __push_args(kwargs));
     }
   }
   catch IOError:
   {
     print("IO Error");
     return(NULL);
   }
   catch RunTimeError:
   {
     print("Runtime Error");
     return(NULL);
   }
}

$1 = path_concat (path_dirname (__FILE__), "help/crates_hlui.hlp");
if (NULL != stat_file ($1))
  add_doc_file ($1);
provide ("crates")