# 
#  Copyright (C) 2007  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


import pycrates
import pytransform

f = open ('crates.sl', 'w')

# Use hand edited slang help file
try:
   f_help = open ('crates_hlui.hlp', 'w')
except:
   print "crates_hlui.hlp NOT open. Cannot update help file"
   f_help = None

# add in the docstring converted help.
#if (f_help):
#base_help = open('chipsfuns_base.hlp','r')
#for line in base_help:
#   f_help.write(line)   
#base_help.close()

# These are functions that need to return NULL.
# Any function with this name has the ability to return NULL
ret_funcs = ['get_crate_type',      \
             'get_crate_item_type', \
             'get_col_names',       \
             'get_colvals',         \
             'copy_colvals',        \
             'get_key_names',       \
             'get_keyval',          \
             'read_file',           \
             'read_arf',            \
             'read_rmf',            \
             'read_pha',            \
             'get_axis_transform',  \
             'get_transform_matrix',\
             'copy_piximgvals',     \
             'get_piximgvals',      ] 

# These are subsets of the above function that we don't
# want to return NULL. Specify the return function name
# in the dictionary and any string in the list associated'
# with that name will not allow for NULL return
no_ret_funcs = {}

# slang usage errors for                                      
usage_err = {};

# Define a subroutine which writes the definions for several
# helper functions to the output file.
def write_helper_functions ():

   # Define a push routine that will push values onto the stack
   f.write ('private define _crates_push ( arg ) { return arg; }\n\n')
   
   # Create generic run function
   #   Check to see if there's anything on the stack and return it
   #   If the return was a tuple, parse it out and return all
   #    the return values
   f.write ('%Check for return. Push tuples on the list individually\n')
   f.write ('private define check_return ()\n{\n')
   f.write ('  variable ret_val=NULL;\n');
   f.write ('  if (_stkdepth () > 0){\n') 
   f.write ('     ret_val = ();\n');
   f.write ('     if (py_call(py.type,ret_val) == py.tuple){\n')
   f.write ('        variable len = py_call (py.len (ret_val))-1;\n');
   f.write ('        variable ii;\n');
   f.write ('        _for (0, len, 1){\n');
   f.write ('            ii = ();\n');
   f.write ('           _crates_push (ret_val[ii]);\n');
   f.write ('        }\n');
   f.write ('     }\n');
   f.write ('  }\n');
   f.write ('  return ret_val;\n}\n\n');
   
   # Create valid types for python
   f.write ('%Create valid types for python\n');
   f.write ('private define _crates_check_type (value);\n');
   f.write ('private define _crates_check_type (value)\n');
   f.write ('{\n');
   f.write ('  if (typeof (value) == Char_Type)\n')
   f.write ('     value = int (value);\n')
   f.write ('  return value;\n')
   f.write ('}\n')
   
   # Define function to turn slang structures into lists
   f.write ('%Turn a structure into a list\n');
   f.write ('private define _crates_struct_to_list (s);\n');
   f.write ('private define _crates_struct_to_list (s)\n');
   f.write ('{\n');
   f.write ('  variable sl_list = {};\n')
   f.write ('  variable name, value;\n')
   f.write ('  variable prefix = NULL;\n')
   f.write ('  if (_NARGS > 1)\n')
   f.write ('     prefix = ();\n')
   f.write ('  try{\n')
   f.write ('     %Go through each element in the structure and make it a list\n')
   f.write ('     foreach (get_struct_field_names (s)){\n');
   f.write ('        name = ();\n');
   f.write ('        value = get_struct_field (s, name);\n');
   f.write ('        if (prefix != NULL)\n')
   f.write ('           name = prefix + "." + name;\n')
   f.write ('        %Expand out sub structures\n')
   f.write ('        if (is_struct_type (value)){\n')
   f.write ('           variable tmp_list = _crates_struct_to_list (name,value);\n')
   f.write ('           variable ii;\n')
   f.write ('           %Concatinate the two lists\n')
   f.write ('           _for (length (tmp_list)-1, 0, -1){\n')
   f.write ('              ii = ();\n')
   f.write ('              list_insert (sl_list, tmp_list[ii], 0);\n')
   f.write ('           }\n')
   f.write ('        }\n')
   f.write ('        else{\n')
   f.write ('           value = _crates_check_type (value);\n')
   f.write ('           list_insert (sl_list,value,0);\n')
   f.write ('           list_insert (sl_list,name,0);\n')
   f.write ('        }\n')
   f.write ('     }\n');
   f.write ('  }\n');
   f.write ('  catch RunTimeError:\n')
   f.write ('     %Return NULL if this was not a structure\n')
   f.write ('     return NULL;\n');
   f.write ('  return sl_list;\n');
   f.write ('}\n');

   f.write ('#if (_slang_version >= 20101)\n')
   f.write ('%Return all the qualifiers in a struct\n');
   f.write ('private define _crates_get_qualifiers (kwstruct);\n')
   f.write ('private define _crates_get_qualifiers (kwstruct)\n')
   f.write ('{\n');
   f.write ('  if( kwstruct == NULL )\n')
   f.write ('    return NULL;\n')
   f.write ('  variable ii, name;\n')
   f.write ('  variable kwnames = get_struct_field_names(kwstruct);\n')
   f.write ('  variable kwargs = Struct_Type[2 * length(kwnames)];\n')
   f.write ('  ii = 0;\n')
   f.write ('  foreach (kwnames)\n')
   f.write ('  {\n')
   f.write ('     name = ();\n')
   f.write ('     kwargs[ii] = struct{value=name};\n')
   f.write ('     ii = ii + 1;\n')
   f.write ('     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};\n')
   f.write ('     ii = ii + 1;\n')
   f.write ('   }\n')
   f.write (' return kwargs;\n')
   f.write (' }\n')
   f.write ('#endif')

def create_slang_func (module, func_list):
   hlist = []

   for name in func_list:
      # Make sure the function is callable or a defined value (dmSHORT)
      try:
          func = getattr(pycrates,name)
          if not callable (func) and \
                 type (func) != int and \
                 type (func) != str and \
                 type (func) != float:
              func = None
      except:
          func = None

      # screen out uncallables or 'private' functions (_<func>)
      if not name.startswith ('_') and not name.startswith ('new_instance') and func != None:

         if type (func) == int or \
            type (func) == str or \
            type (func) == float:
             # Simple wrap for defined values.. just make python call.
             f.write ('define '+name+' () {');
             f.write (' return py.'+module.__name__+'.'+name+'; }\n\n')
         else:
             f.write ('define '+name+' ()\n{\n');
         
             # Check for qualifiers/keywords
             f.write ('#if (_slang_version >= 20101)\n')
             f.write ('    variable kwargs = _crates_get_qualifiers(__qualifiers());\n')
             f.write ('#else\n')
             f.write ('    variable kwargs = NULL;\n')
             f.write ('#endif\n')
                     
             # usage_err functions have special return values
#             if (usage_err.has_key (name)):
             f.write ('  variable err;\n');
             f.write ('  try (err){\n');
             f.write ('     if (_NARGS != 0){\n');
             f.write ('        variable f_args = __pop_args (_NARGS);\n');
             # Convert slang structure to slang list. Python cant
             # parse the names of structures. This could be done
             # in python calling back to slang for parsing. However,
             # it would give more control over when the parsing occurs
             f.write ('        variable ii, list_conv;\n')
             f.write ('        _for (0, length (f_args)-1,1){\n')
             f.write ('           ii = ();\n');
             f.write ('           list_conv = _crates_struct_to_list (f_args[ii].value);\n');
             f.write ('           if (list_conv != NULL)\n');
             f.write ('              f_args[ii].value = list_conv;\n');
             f.write ('           else\n')
             f.write ('              f_args[ii].value = _crates_check_type (f_args[ii].value);\n');
             f.write ('        }\n');


                             
             # see if the function is expected to return something
             ret_found = False
             for ret_name in ret_funcs:
                 if name.find (ret_name) != -1:
                     ret_found = True
         
                     # exlude any functions that have return function name
                     # in it but is not supposed to actually return NULL
                     if no_ret_funcs.has_key (ret_name):
                         for no_ret_name in no_ret_funcs[ret_name]:
                             if name.find (no_ret_name) != -1:
                                 ret_found = False
         
             # Check for qualifiers/keywords
             f.write ('        if( kwargs == NULL )\n')
             if ret_found:
                 f.write ('         _py_call')
             else:
                 f.write ('         py_call')
         
             f.write ('(py.'+module.__name__+'.'+name+', __push_args (f_args));\n');
             f.write ('        else\n')
             if ret_found:
                 f.write ('        _py_call')
             else:
                 f.write ('        py_call')
         
             f.write ('(py.'+module.__name__+'.'+name+', __push_args (f_args), PY_KW,  __push_args(kwargs));\n')
             f.write ('     }\n');
             f.write ('     else{\n');
             f.write ('       if ( kwargs == NULL )\n')
             if ret_found:
                 f.write ('        _py_call')
             else:
                 f.write ('        py_call')
             f.write ('(py.'+module.__name__+'.'+name+');\n');
             f.write ('       else\n')
             if ret_found:
                 f.write ('         _py_call')
             else:
                 f.write ('         py_call')
             f.write ('(py.'+module.__name__+'.'+name+', PY_KW,  __push_args(kwargs));\n');
             f.write ('     }\n')
         
             # Check to see if there's anything on the stack and return it
             if (ret_found):
                 f.write ('     return check_return ();\n');
             f.write ('   }\n');
	     f.write ('   catch IOError:\n')
	     f.write ('   {\n')
	     f.write ('     print("IO Error");\n')
	     f.write ('     return(NULL);\n')
	     f.write ('   }\n')
	     f.write ('   catch RunTimeError:\n')
	     f.write ('   {\n')
	     f.write ('     print("Runtime Error");\n')
	     f.write ('     return(NULL);\n')
	     f.write ('   }\n')


             if (usage_err.has_key (name)):
                 f.write ('   catch TypeMismatchError:\n');
                 f.write ('   {\n');
                 f.write ('      throw TypeMismatchError, "Incorrect usage. Try - '+usage_err[name] + '", err.object;\n');
                 f.write ('   };\n');
             f.write ('}\n\n');
         
         #Create slang help file
         if type (func) != int and \
            type (func) != str and \
            type (func) != float and \
            func.__doc__ != None and \
            f_help:
               f_help.write (name + '\n\n')
               f_help.write (' ' + func.__doc__)
               f_help.write ('\n\n--------------------------------------------------------------\n\n')
               hlist.append(name)

   # Write generic help for crates containing function list
   if (f_help):
       f_help.write ('crates\n')
       f_help.write ('  High Level User Interface to the Crates software library\n')
       f_help.write ('    Function List:\n')
       for name in hlist:
           f_help.write ('      '+ name + '\n')
       f_help.write ('\n\n--------------------------------------------------------------\n\n')

# ----------------------------------------------------------------------
def create_func_list ():

   func_list = []
   crate_funcs = dir(pycrates)
   trans_funcs = dir(pytransform)

   for name in crate_funcs:
      if name.startswith ('_') or \
         name.endswith ('_swigregister') or \
         name.startswith ('new_instance'):
         # Screens 'private' functions ( _<func> )
         func = None
      else:        
         try:
            func = getattr(pycrates,name)

            # Check that item is callable or a defined values (e.g. dmSHORT)
            if not callable (func) and \
                   type (func) != int and \
                   type (func) != str and \
                   type (func) != float:
                func = None
         except:
            func = None

         # Check for items which are from the Transform library
         # We dont want to create wrappers for them here.
         if func != None:
            try:
                trfunc = getattr(pytransform,name)
                # Allow shared defines and helper functions.
                if not callable (trfunc) or \
                        name.startswith ('PySwig') or \
                       name.startswith ('get_dmType') or \
                       name.startswith ('get_pyType'):
                    trfunc = None
            except:
                trfunc = None

            if trfunc == None:
                func_list.append(name)

   return func_list


# **********************************************************************
# Import pysl and pycrates
f.write ('require ("pysl");\n')
f.write ('require ("transform");\n')
f.write ('py_import ("pycrates");\n\n')

# Write helper function definitions.
write_helper_functions ()

# Put Crates HLUI functions into slang
f.write ('\n%Crates HLUI Functions\n')

# create function list
flist = create_func_list()

# create slang functions for all functions in pycrates
create_slang_func (pycrates, flist )

#create access to help file
f.write ('$1 = path_concat (path_dirname (__FILE__), "help/crates_hlui.hlp");\n')
f.write ('if (NULL != stat_file ($1))\n')
f.write ('  add_doc_file ($1);\n')

#provide crates
f.write ('provide ("crates")')

