/*                                                                
**  Copyright (C) 2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* H*****************************************************************
 *
 * FILE NAME:  pyparamio.h
 *
 * DEVELOPMENT: tools
 *
 * DESCRIPTION:
 *
 * header info for paramio tool ( prototypes and macros )
 *
 *
 * REVISION HISTORY:
 *
 * Ref. No.         Date
   ----------       -----
   0.1              21May2007 	File Created
H***************************************************************** */
#include "Python.h"
#include "structmember.h"       /* Provides declarations used to handle attributes */
#include "parameter.h"
#include <string.h>
#include <stdlib.h>


/*
 * Constants Defined
 */
#define PF_MAX_LINE_LEN 		1024

/* * * * * * * * * * * * * * * * * * * * * * * *
 * Create Python datatypes
 * * * * * * * * * * * * * * * * * * * * * * * */

typedef struct {
	/* Note that there is no semicolon after the PyObject_HEAD macro;
	 * one is included in the macro definition.
   */
	PyObject_HEAD
	paramfile fptr;
} Param_File;

staticforward PyTypeObject Param_File_Type;

/* * * * * * * * * * * * * * * * * * * * * * * *
 * Function Declarations
 * * * * * * * * * * * * * * * * * * * * * * * */

/*
 * This function will open a parameter file, returning a file pointer
 * which can be used with the other paramio functions.
 */
static PyObject *_paramopen(PyObject *self,	/*i: Used by Python */
                            PyObject *args	/*i: Python tuple of the arguments */
                            );

/*
 * This function will close the parameter file associated with the given
 * Param_File_Type variable.
 */
static PyObject *_paramclose(PyObject *self,	/*i: Used by Python */
                             PyObject *args	 /*i: Python tuple of the arguments */
                             );

/*
 * This function will set parameters.
 */
static PyObject *_pset(PyObject *self,	/*i: Used by Python */
                       PyObject *args	 /*i: Python tuple of the arguments */
                       );

/*
 * This function will get the value of the specified parameter(s) from the
 * given parameter file.
 */
static PyObject *_pget(PyObject *self,	/*i: Used by Python */
                       PyObject *args	 /*i: Python tuple of the arguments */
                       );

/*
 * This function will get parameter values.
 */
static PyObject *_pquery(PyObject *self,	/*i: Used by Python */
                         PyObject *args	 /*i: Python tuple of the arguments */
                         );

/*
 * This function will find out whether parameters and parameter files
 * are accessible
 */
static PyObject *_paccess(PyObject *self,	/*i: Used by Python */
                          PyObject *args	 /*i: Python tuple of the arguments */
                          );

/*
 * This function will return the location of the parameter file for
 * the given argument.
 */
static PyObject *_paramgetpath(PyObject *self,	/*i: Used by Python */
                               PyObject *args	 /*i: Python tuple of the arguments */
                               );

/*
 * This function will restore the parameter values in the specified
 * parameter file to the system defaults.
 */
static PyObject *_punlearn(PyObject *self,	/*i: Used by Python */
                           PyObject *args	 /*i: Python tuple of the arguments */
                           );

/*
 * This function will read/write individual parameter values.
 */
static PyObject *_pgeti(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will provide read and write access and typecast the
 * variable into the given type.
 */
static PyObject *_pputi(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will read/write individual parameter values.
 */
static PyObject *_pgetb(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will provide read and write access and typecast the
 * variable into the given type.
 */
static PyObject *_pputb(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will read/write individual parameter values.
 */
static PyObject *_pgetd(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will provide read and write access and typecast the
 * variable into the given type.
 */
static PyObject *_pputd(PyObject *self,	/*i: Used by Python */
                        PyObject *args	 /*i: Python tuple of the arguments */
                        );

/*
 * This function will read/write individual parameter values.
 */
static PyObject *_pgetstr(PyObject *self,	/*i: Used by Python */
                          PyObject *args	 /*i: Python tuple of the arguments */
                          );

/*
 * This function will provide read and write access and typecast the
 * variable into the given type.
 */
static PyObject *_pputstr(PyObject *self,	/*i: Used by Python */
                          PyObject *args	 /*i: Python tuple of the arguments */
                          );
/*
 *
 * This returns a list of parameter names
*/

static PyObject *_plist(PyObject *self,	/*i: Used by Python */
                          PyObject *args	 /*i: Python tuple of the arguments */
                          );
