/*                                                                
**  Copyright (C) 2002-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*****************************************************************/
/*                                                               */
/*   dmdslib.c                                                   */
/*                                                               */
/*   private dataset functions                                   */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmlibP.h"
#include <unistd.h>



/***************************************************************/
/*                                                             */
/* Function:   dmp_dataset_open_mode                           */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*               const char *ddsname                           */
/*               const char *hint                              */
/*               const char *mode                              */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*            dmDataset *ds                                    */
/*                                                             */
/***************************************************************/
dmDataset* dmp_dataset_open_mode( const char* ddsname,
                              const char* hint, const char* mode )
{
  dmKernel kernel;
  dmDataset *ds = NULL;

  char   name_str[CARD_SIZE];
  char  *name   = NULL;
  char **list   = NULL;
  char  *dsname = NULL;
  char  *value  = NULL;

  long   nitems    = 0;
  long   i         = 0;
  long   kernel_id = 0;


  if ( !ddsname )
    return NULL;

  dmInit(0,NULL);

  /* Strip any DM syntax from the filename */
  dsname = dmu_extract_filename( ddsname );

  /* See if there is a hint defining the kernel */
  if ( hint )
  {
    list = cs_parse_comma_list( hint, &nitems );
    name = &name_str[0];

    for ( i = 0; i < nitems; i++ )
    {
      value = dmu_parse_token( list[i], '=', &name, CARD_SIZE );

      if ( cs_uceq( name, "KERNEL" ))
        kernel_id = dmkGetNamedKernel( value );
    }
    ar_free_cp( list, nitems );
  }
 

  /* Call the kernel to open the dataset */
  if ( cs_eq( mode, "W" ))
  {
    if ( !kernel_id )
      kernel_id = dmkGetTypedKernel( "W" );

    kernel = dmkKernelNo( kernel_id ); 
    ds = kernel->DatasetCreate( dsname, hint );

    if ( ds )
      ds->kernel = kernel;
  }
  else
  {
    if ( !kernel_id )
      ds = dmk_dataset_open( dsname, hint, mode );
    else
    {
      kernel = dmkKernelNo( kernel_id );
      ds = kernel->DatasetOpen( dsname, hint, mode );

      if ( ds )
        ds->kernel = kernel;
    }

    /* if open-update failed, try open readonly */
    if ( !ds && cs_eq( mode, "RW" ))
    {
      if ( !kernel_id )
        ds = dmk_dataset_open( dsname, hint, "R" );
      else
      {
        ds = kernel->DatasetOpen( dsname, hint, "R" );

        if ( ds )
          ds->kernel = kernel;
      }
    }
  }

  if ( ds )
    dmu_err_clear();

  free( dsname );
  return ds;
}


