/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/* dmcomment.c                                                   */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmlibP.h"

/***************************************************************/
/* Function:  dmBlockWriteComment                              */
/*                                                             */
/* Description:                                                */
/*     Write comment key value to current key position         */
/*                                                             */
/* Input parameters:                                           */
/*       dmBlock      *block    - block pointer                */
/*       const char   *tag      - key tag                      */
/*       const char   *comment  - comment string               */
/*                                                             */
/* Output parameters:                                          */
/*    none                                                     */
/*                                                             */
/* Returns:                                                    */
/*     dmBool  UT_TRUE or UT_FALSE - write successful flag     */
/*                                                             */
/***************************************************************/
dmBool dmBlockWriteComment( dmBlock* block, const char* tag, const char* comment )
{
  dmKernel kernel; 

  if ( !block )
    return UT_FALSE;

  kernel = block->ds->kernel;
  dmi_block_set_update( block );
  return kernel->WriteComment( block, block->keyno, tag, comment );
}


/***************************************************************/
/* Function:  dmBlockReadComment                               */
/*                                                             */
/* Description:                                                */
/*     Read comment key value at current key position          */
/*                                                             */
/* Input parameters:                                           */
/*       const dmBlock *block    - block pointer               */
/*                                                             */
/* Output parameters:                                          */
/*       const char   **tag      - key tag                     */
/*       const char   **comment  - comment string              */
/*                                                             */
/* Returns:                                                    */
/*     dmBool  UT_TRUE or UT_FALSE - read successful flag      */
/*                                                             */
/***************************************************************/
dmBool dmBlockReadComment( const dmBlock* block, char** tag, char** comment )
{
  dmKernel kernel; 

  if ( !block )
    return UT_FALSE;

  kernel = block->ds->kernel;
  return kernel->ReadComment( (dmBlock*)block, block->keyno, tag, comment );
}
