/*                                                                
**  Copyright (C) 2002-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmibuffer.c                                                 */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmi.h"
#include <limits.h>
#define XDEBUG 1

extern long dmi_filter_rows_simple( dmBuffer* buffer, void* data );
extern long dmi_filter_rows_with_nulltest( dmBuffer* buffer, void* data );

void dmi_buffer_print_hdr( dmBuffer* buffer,  char* buf );



/***************************************************************/
/*                                                             */
/* Function:   dmi_block_add_col                               */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*           dmBlock      *buffer                              */
/*           dmDescriptor *dd                                  */
/*           long          flag                                */
/*                                                             */
/*  Output parameters:                                         */
/*          none                                               */
/*                                                             */
/*  Returns:    none                                           */
/*                                                             */
/*                                                             */
/***************************************************************/
void dmi_block_add_col( dmBlock* block, dmDescriptor* dd, long flag )
{

  if ( !block || !block->buffer )
    return;

  dmi_buffer_add_col( block->buffer, dd, flag );
}


/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_add_col                            */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBuffer     *buffer                               */
/*          dmDescriptor *dd                                   */
/*          long          flag                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:              none                                 */
/*                                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_add_col( dmBuffer* buffer, dmDescriptor* dd, long flag )
{
 dmDescriptor *old_dd = NULL;

 long colno     = 0;
 long old_size  = 0;
 long nvals     = 0;
 long pad       = 0;
 long vpad      = 0;
 long used      = 0;
 long n         = 0;
 long vn        = 0;
 long atom_size = 0;
 long size      = 0;


/*
 * IMPLEMENT case of FLAG_RESULT here: need to find operands
 * and flag them. Test case: dmlist evt2.fits[cols ra] data
 */


  dd->flag |= flag;
  colno = dmi_descriptor_match( dd, buffer->cols, buffer->ncols );

  if ( colno <= 0 )
  {
    n = buffer->ncols;

    /* Handle case of image col create - we just deleted cols[0] */

    while( n>0 && buffer->cols[n-1] == NULL)
    {
      --n;
      buffer->ncols--;
    }

    n = buffer->ncols;

    if ( dd->flag & DM_FLAG_UNBUF )
    {
      size = 0;
      nvals = 1;
      pad = 0;
      vpad = 0;
    }
    else
    {
      size = dmi_descriptor_get_bytes( dd );

      if ( dd->type == DT_CHAR || dd->type == DT_BIT )
        atom_size = UT_SZ_C;
      else
        atom_size = size;

      nvals  = dmi_descriptor_get_nvals( dd );

      if ( n >= 1 )
      {
        old_dd  = buffer->cols[n-1];

        if ( old_dd )
        {
          old_size = dmi_descriptor_get_bytes( old_dd );

          if ( old_dd->type == DT_CHAR || old_dd->type == DT_BIT )
            old_size = UT_SZ_C;

          pad = ut_sys_align( buffer->total, atom_size, old_size );
        }

        if ( dd->flag & DM_FLAG_VISIBLE )
        {
          /* The visible padding (i.e. what the padding would be if all
           * hidden columns were removed from the buffer) may be different
           * than the "real" padding, so we need to handle it separately.
           * It's used to calculate the visible offset, which is needed
           * for GetRow()/PutRow() and friends. */

          vn = n;

          /* Step backward through the column list until we find a visible
           * column. */

          while ( vn > 0 && ( NULL != ( old_dd = buffer->cols[vn-1] ) ) &&
                 !( old_dd->flag & DM_FLAG_VISIBLE ) )
            --vn;

          if ( vn > 0 && old_dd )
          {
            old_size = dmi_descriptor_get_bytes( old_dd );

            if ( old_dd->type == DT_CHAR || old_dd->type == DT_BIT )
              old_size = UT_SZ_C;

            vpad = ut_sys_align( buffer->vtotal, atom_size, old_size );
          }
        }
      }
    }

    used = nvals * size;

    if ( n >= buffer->nmax )
    {
      buffer->nmax = 2 * n + 10;
      buffer->cols = realloc( buffer->cols, buffer->nmax *UT_SZ_P );
      buffer->offsets= realloc( buffer->offsets, buffer->nmax  *UT_SZ_I );
      buffer->voffsets = realloc( buffer->voffsets, buffer->nmax  *UT_SZ_I );
      buffer->sizes = realloc( buffer->sizes, buffer->nmax *UT_SZ_I );
    }

    buffer->resync = UT_TRUE; 

    buffer->cols[n] = dd;
    buffer->sizes[n] = used;
    dd->bufno = n+1;

    /* Mark vector components as being results, also set cpt buf colnos */
    if ( dd->dim > 1 )
    {
      long i = 0;
      dmDescriptor *ddp = NULL;
      int result = dd->flag & DM_FLAG_RESULT;
  
      for ( i = 0; i < dd->dim; i++ )
      {
        ddp = dd->cpts[i];

        if ( ddp )
        {
          ddp->bufno = n+1;

          if ( result )
            ddp->flag |= DM_FLAG_RESULT;
        }
      }
    }

    /* We carry two different row sizes: for the first row, which may have
     * a binned image at the end; and for subsequent rows which don't. 
     */

    if ( dd->flag & DM_FLAG_BINNED )
    {
      buffer->offsets[n] = buffer->imtotal;
      buffer->imtotal += used + pad;   
    }
    else
    {
      buffer->offsets[n] = buffer->total + pad;
      buffer->total += used + pad;

      if ( dd->flag & DM_FLAG_VISIBLE )
      {
        buffer->voffsets[n] = buffer->vtotal + vpad;
        buffer->vtotal += used + vpad;  /* Visible bytes */
      }
      else
        {
          /* Hidden columns don't have a visible offset.  Would -1 be a better
           * value? */
          buffer->voffsets[n] = 0;
        }
    }

    buffer->ncols++;

    /* If dd is a variable-length array, we need a hidden column to hold the
     * length of its rows */
    if ( dd->varlen )
    {
      dmi_buffer_add_varlen_length_col( dd->block, dd );
    }
  }
}


/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_eval_size                          */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/* Calculate the size of the DM row buffer given a requested   */
/* size and a kernel buffer size.                              */
/* First calculate a minimum size: if default size is already  */
/* set (by a previous call, or by a user hint) use that.       */
/* Otherwise use the global default; if the ksize is non zero, */
/* round up to a multiple of ksize (ksize is the kernel buffer */
/* size).                                                      */ 
/*                                                             */
/* For the buffer size, use the larger of the minimum size and */
/* the requested size, but (if readonly) no larger than the    */
/* total size of the file.                                     */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          long          insize                               */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:         long size                                 */
/*                                                             */
/*                                                             */
/***************************************************************/
long dmi_buffer_eval_size( dmBlock* block, long insize )
{
  dmBuffer *buffer = NULL;

  long     maxmeg  = 0;
  long     size    = 0;
  long     ksize   = 0;
  long     default_size   = 0;

  double   test_size = 0;
  unsigned long max_size  = 0;

  buffer = block->buffer;

  /* 
    If default size is unset, set it to a multiple of the kernel buffer size.
    That way if the data is unfiltered, we gain some efficiency. 
  */

  ksize = buffer->kernel_rows;
  default_size = buffer->default_size;

  if ( default_size == 0 || default_size ==  dm_buffersize )
  {
    default_size = dm_buffersize;

    if ( ksize > 0 ) 
    {
      default_size = (default_size / ksize ) * ksize;

      if ( default_size < dm_buffersize )
        default_size += ksize;
    }

    buffer->default_size = default_size;
  }

  size = buffer->default_size;

  if ( insize > size )
    size = insize;

  /* If block is writeable, we don't know what nrows will end up being,
     so keep the buffer large. If not, then buffer needs to be 
     at most nrows. */

  if ( !dmi_block_writeable( block ) && size > block->nrows )
    size = block->nrows;  /* Raw nrows */

  /* Allow at most 1/4 of maxmeg for the table buffer */

  maxmeg = dmi_block_get_maxmeg( block ) / 4;

  /* If buffer->total is known already and too big, make nrows smaller */
  /* need to be careful about wrapping test_size here....(Bug #8796)   */
  /* 11/07/2008: encountered a file (CALDB4) where size*total wraps    */
  /*             the unsigned long test_size.                          */
  /*             Changing datatype of test_size to double              */

  test_size = (double)size * (double)(buffer->total);
  max_size = maxmeg * 1000000;

  if ( ( test_size > (double)max_size ) || 
       ( test_size < (double)(buffer->total)) )
  {
    size = max_size / buffer->total;
  }

  /* At least one row */
  if ( size < 1 )
    size = 1;

  /* If a maximum buffer size has been set, respect it (unless the
   * maximum size is less than the requested size) */

  if ( buffer->max_size > 0 )
  {
    if ( ( buffer->max_size < size ) && ( buffer->max_size >= insize ) )
      size = buffer->max_size;
  }

  return size;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_free                                 */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmi_buffer_free( dmBuffer* buffer )
{
  long index_flag = 0;

  if ( !buffer )
    return;

  free( buffer->contiguous );
  buffer->contiguous = NULL;

  free( buffer->contig_start );
  buffer->contig_start = NULL;

  free( buffer->data );
  buffer->data = NULL;

  free( buffer->index );
  buffer->index = NULL;

  free( buffer->vdata );
  buffer->vdata = NULL;

  free( buffer->offsets );
  buffer->offsets = NULL;

  free( buffer->voffsets );
  buffer->voffsets = NULL;

  free( buffer->cols );
  buffer->cols = NULL;

  free( buffer->sizes );
  buffer->sizes = NULL;

  dmi_execute_stack_free( buffer->execute1 );
  dmi_execute_stack_free( buffer->execute2 );
  buffer->execute1 = NULL;
  buffer->execute2 = NULL;

  if ( buffer->subspace )
  {
    dmi_subspace_free( buffer->subspace, NULL );
    buffer->subspace = NULL;
  }

  if ( buffer->offset_index )
  {
    for ( index_flag = 0; index_flag < DMBI_N; index_flag++ )
       free( buffer->offset_index[index_flag] );
 
    free( buffer->offset_index);
    buffer->offset_index = NULL; 
 }

  if ( buffer->col_index )
  {
    for ( index_flag = 0; index_flag < DMBI_N; index_flag++ )
      free( buffer->col_index[index_flag] );

    free( buffer->col_index );
    buffer->col_index = NULL;
  }

  free( buffer->n_index );
  buffer->n_index = NULL;
 
  free ( buffer );
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_alloc                                */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            none                                             */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            dmBuffer *buffer                                 */
/*                                                             */
/***************************************************************/
dmBuffer* dmi_buffer_alloc( void )
{
  dmBuffer *buffer = NULL;

  buffer = (dmBuffer*)calloc( 1, sizeof( dmBuffer ) );

  buffer->offsets      = NULL;
  buffer->voffsets     = NULL;
  buffer->sizes        = NULL;
  buffer->nmax         = 0;
  buffer->ncols        = 0;
  buffer->cols         =  NULL;
  buffer->update       =  UT_FALSE;
  buffer->total        = 0;
  buffer->col_ordered  = UT_TRUE;
  buffer->rrow1        = 0;
  buffer->rrow2        = 0;
  buffer->default_size = dm_buffersize;
  buffer->max_size     = 0;
  buffer->nrows        = 0;
  buffer->data         = NULL;

  return buffer;
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_list                                 */
/*                                                             */
/* Description:                                                */
/*     Look at hidden entries: block->cols can be extended?    */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*            dmBuffer *buffer                                 */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_list( dmBlock* block, dmBuffer* buffer )
{
  dmDescriptor *dd = NULL;

  long  i = 0;
  char  cbuf[120];


  for ( i = 1; i <= buffer->ncols; i++ )
  {
    dd = block->cols[i];
    sprintf( cbuf, "%ld %-20s %4ld", i, dd->name, buffer->offsets[i] );
    printf( "%s\n", cbuf );
  }
}

/* Helper function to return buffer offset for printing */
static long brel( long a, long b );

/***************************************************************/
/*                                                             */
/* Function:   brel                                            */
/*                                                             */
/* Description:                                                */
/*     Helper function to return buffer offset for printing    */
/*                                                             */
/*  Input parameters:                                          */
/*            long  a                                          */
/*            long  b                                          */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            static long value                                */
/*                                                             */
/***************************************************************/
static long brel( long a, long b )
{
  if ( a > 0 )
    return a-b+1;
  else
    return 0;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_status                               */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*                                                             */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_status( dmBlock* block )
{
 dmBuffer *buffer = NULL;
 long      one    = 1;

  buffer = block->buffer;

  if ( !buffer)
    return;

  printf( "--------------------------------\n" );
  printf( "DM Table Buffer for %s:%s\n", block->ds->name, block->name );
  printf( "                     Row       Buffer Pos\n" );
  printf( "Block raw rows       R %ld:%ld           \n", one, block->nrows );
  printf( "Buffer length                  B %ld:%ld\n",  one, buffer->nrows );
  printf( "Buffer row filter    R %ld:%ld          \n", buffer->minrow, buffer->maxrow );
  printf( "Buffer raw rows      R %ld:%ld B %ld:%ld \n", buffer->rrow1, buffer->rrow2, one, brel( buffer->rrow2, buffer->rrow1 ));
  printf( "Buffer top           R    :%ld B    :%ld \n", buffer->rrow_top, brel( buffer->rrow_top, buffer->rrow1 ));
  printf( "Buffer end           R    :%ld B    :%ld \n", buffer->rrow_end, brel( buffer->rrow_end, buffer->rrow1 ));

  if ( buffer->soffset < buffer->nrows )
    printf( "Buffer raw segment:  R %ld:%ld B %ld:%ld \n",  buffer->rrow_seg1, buffer->rrow_seg2, buffer->soffset+1, buffer->soffset+(buffer->rrow_seg2-buffer->rrow_seg1+1) );
 else
   printf( "Buffer raw segment:  R %ld:%ld  \n",  buffer->rrow_seg1, buffer->rrow_seg2 );

 if ( block->nvrows > 0 )
   printf( "Block filtered rows  V %ld:%ld\n", one, block->nvrows );
 else
   printf( "Block filtered rows  V 0:0\n" );
   printf( "Buffer target rows   V %ld:%ld\n", buffer->vrow_go1, buffer->vrow_go2 );
   printf( "Buffer filtered rows V %ld:%ld\n",  buffer->vrow1, buffer->vrow2 );
   printf( "Buffer last row used V %ld\n", buffer->lastrow );
   printf( "Buffer current row   V %ld\n",  buffer->vrow );
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_print                                */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*                                                             */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_print( dmBlock* block )
{
#if 1
{
  long i = 0;
  char xbuf[120];
  char status[80];
  dmDescriptor *dd = NULL;
  dmBuffer     *buffer = NULL;

  dmi_buffer_status( block );
  buffer = block->buffer;

  if ( !buffer)
    return;

  printf( "   Entry                        Size         Offset     Flags\n" );

  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];
    sprintf( xbuf, "%3ld %-20s %12ld %12ld", i+1, dmi_descriptor_get_name( dd ), buffer->sizes[i],
             buffer->offsets[i] );

    dmi_dd_print_flag( dd, status );
    printf( "%s %s\n", xbuf, status );
  }
}
#endif
  printf( "--------------------------------\n" );
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_print_vb                             */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*            long      vr1                                    */
/*            long      vr2                                    */
/*                                                             */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_print_vb( dmBlock* block, long vr1, long vr2  )
{
  dmBuffer *buffer = NULL;
  char      buf[4096];

  long vrow_offset = 0;
  long rrow_offset = 0;

  buffer = block->buffer;

  if ( !buffer)
    return;

  printf( " \n" );
  printf( "Virtual Buffer\n" );
  printf( " \n" );

  for ( vrow_offset = vr1-1; vrow_offset < vr2; vrow_offset++ )
  {
    rrow_offset = buffer->index[vrow_offset];
    dmi_buffer_print_row( buffer, buffer->rrow1 + rrow_offset, buf );
    printf( "%4ld %6ld %4ld | %s\n", vrow_offset+1, rrow_offset+1, buffer->contiguous[vrow_offset], buf );
  }
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_print_data                           */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*                                                             */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_print_data( dmBlock* block )
{
  dmBuffer *buffer = NULL;
  char      buf[4096];
  long      rrowno = 0; 

  buffer = block->buffer;

  if ( !buffer)
    return;

  printf( "Buffer:\n" );
  dmi_buffer_print_hdr( buffer, buf );
  printf( "%s\n", buf );

  for ( rrowno = buffer->rrow1; rrowno <= buffer->lastrow; rrowno++ )
  {
    dmi_buffer_print_row( buffer, rrowno, buf );
    printf( "%s\n", buf );
  }
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_print_row                            */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*            long      rrowno                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            char *buf                                        */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_print_row( dmBuffer* buffer, long rrowno, char* buf )
{
  long i           = 0;
  long tab         = 0;
  long m           = 0;
  long tab_size    = 10;
  long nitems      = 0;
  long rrow_offset = 0;
  long size        = 0;
  long item        = 0;

  dmDescriptor *dd =  NULL;
  dmDataType type;

  char xbuf[CARD_SIZE];

  void *ptr  = NULL;
  void *iptr = NULL;


  cs_copy( " ", buf );

  if ( !buffer)
    return;

  rrow_offset = rrowno-buffer->rrow1;

  if ( rrowno <= 0 || rrowno >   buffer->nrows )
    return;

  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];

    if ( ! (dd->flag & DM_FLAG_BINNED ))
    {
      nitems = dd->size * dd->dim;  

      if ( nitems > 10 )
        nitems = 10;

      type = dd->type;
      size = dmi_descriptor_get_bytes( dd );
      ptr = (char*)buffer->data + dmi_buffer_offset( buffer, i, rrow_offset );

      for( item = 0; item < nitems; item++ )
      {
        iptr = (char*)ptr + item * size;

        if ( type == DT_CHAR ) 
          ar_mix_print_generic( &iptr, type, xbuf, CARD_SIZE );
        else   
          ar_mix_print_generic( iptr, type, xbuf, CARD_SIZE );

        m = strlen( buf );

        while( m < tab )
        {
          buf[m++] = ' ';
        }

        buf[m] = 0;
        tab += tab_size;

        strcat( buf, xbuf );
        strcat( buf, " " );
      }
    }
  }
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_print_hdr                            */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            char *buf                                        */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_print_hdr( dmBuffer* buffer,  char* buf )
{
  dmDescriptor *dd = NULL;
  dmDataType    type;

  long i        = 0;
  long nitems   = 0;
  long size     = 0;
  long tab_size = 10;
  long m        = 0;
  long tab      = 0;

  cs_copy( " ", buf );

  if ( !buffer)
    return;

  tab = 0;
 
  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];

    if ( ! (dd->flag & DM_FLAG_BINNED ))
    {
      nitems = dd->size * dd->dim;  

      if ( nitems > 10 )
        nitems = 10;

      type = dd->type;
      size = dmi_descriptor_get_bytes( dd );
   
      m = strlen( buf );

      while( m < tab )
      {
        buf[m++] = ' ';
      }

      buf[m] = 0;
      tab += nitems * tab_size;
      strcat( buf, dd->name );
    }
  }
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_offset                               */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*            long      field_offset                           */
/*            long      row_offset                             */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            long offset                                      */
/*                                                             */
/***************************************************************/
long dmi_buffer_offset( dmBuffer* buffer, long field_offset, long row_offset )
{
   return buffer->offsets[field_offset] * buffer->nrows
               + row_offset  * buffer->sizes[field_offset];
#if ROW_ORDERED

   return buffer->total * row_offset + buffer->offsets[field_offset];
#endif
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_get_col_offset                       */
/*                                                             */
/* Description:                                                */
/*             Need to handle components...                    */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer     *buffer                             */
/*            dmDescriptor *dd                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            long offset                                      */
/*                                                             */
/***************************************************************/
long dmi_buffer_get_col_offset( dmBuffer* buffer, dmDescriptor* dd )
{
  (void) buffer;
  return dd->bufno-1;
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_get_cpt_offset                       */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer     *buffer                             */
/*            dmDescriptor *dd                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            long offset                                      */
/*                                                             */
/***************************************************************/
long dmi_buffer_get_cpt_offset( dmBuffer* buffer, dmDescriptor* dd )
{
  long i = 0;
  long j = 0;

  dmDescriptor *vector = NULL;

  for ( i = 0; i < buffer->ncols; i++ )
  {
    vector = buffer->cols[i];

    if ( vector == dd )
      return 0;

    if ( vector && vector->dim > 1 )
    {
      for ( j = 0; j < vector->dim; j++ )
      {
        if ( vector->cpts[j] == dd )
          return j;
      }
    }
  }
  return -1; 
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_bin_map_offsets                             */
/*                                                             */
/* Description:                                                */
/*      Convert subspace to physical coords always             */
/*      The image copying and binning process:                 */
/*                                                             */
/*       Define rectangular parent image subspace              */
/*       Intersect with binning range                          */
/*       Apply region filter(s)                                */
/*       Map to binning pixel                                  */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmDescriptor *parent_image                       */
/*            dmDescriptor *image                              */
/*                                                             */
/*  Output parameters:                                         */
/*            dmBinSpec    *binspec                            */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_bin_map_offsets( dmBinSpec* binspec, dmDescriptor* parent_image, dmDescriptor* image )
{
  dmBlock      *block = NULL;
  dmDescriptor *dd    = NULL;
  dmBinAxis    *axis  = NULL;

  double crpix[dmMAXDIM];
  double crval[dmMAXDIM];
  double cdelt[dmMAXDIM];
  double tmin[dmMAXDIM];
  double tmax[dmMAXDIM];
  double ltmin[dmMAXDIM];
  double ltmax[dmMAXDIM];

  double step = 1.0;

  long n2           = image->naxes;
  long axis_offset1 = 0;
  long axis_offset2 = 0;
  long dim          = 0;
  long group_offset = 0;
  long offset       = 0;
  long vcpt         = 0;
  long n1           = 0;
  long axisno       = 0;

  int is_logical    = UT_FALSE;


  if ( parent_image )
    n1 = parent_image->naxes;
  else
    n1 = n2;

  block = (dmBlock*)image->block;


  binspec->offsets  = calloc( n2, UT_SZ_I );
  binspec->vcpts    = calloc( n2, UT_SZ_I );
  binspec->groups   = calloc( n2, UT_SZ_I );
  binspec->axes     = calloc( n2, UT_SZ_I );
  binspec->axis_map = calloc( n2, UT_SZ_I );
  binspec->min      = calloc( n2, UT_SZ_D );
  binspec->max      = calloc( n2, UT_SZ_D );
  binspec->step     = calloc( n2, UT_SZ_D );
  binspec->start    = calloc( n1, UT_SZ_I );


  for ( group_offset = 0; group_offset < binspec->n; group_offset++ )
  {
    axis = &(binspec->axis[group_offset]);
    dd = axis->dd;
    dim = dd->vdim;
    is_logical = ( dd->flag & DM_FLAG_LOGICAL );

    if ( parent_image )
      offset = dmi_descriptor_get_group_axis( parent_image, dd->no, 1 ); 
    else
      offset = dd->bufno-1;

    if ( offset < 0 )
    {
      sprintf( dm_errbuf, "Binning axis %s not in buffer", dd->name );
      dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
      return;
    }

    /* Calculate filtered-image logical bounds from physical bounds */

    for ( vcpt = 0; vcpt < dd->vdim; vcpt++ )
    {
      tmin[vcpt]= axis->vals[vcpt][0];
      tmax[vcpt]= axis->vals[vcpt][1];
    }

    dmi_coord_get_transform_d( dd, crpix, crval, cdelt, dim );

    if ( !is_logical )
    {
      dmi_coord_invert_d( dd, tmin, ltmin );
      dmi_coord_invert_d( dd, tmax, ltmax );
    }
    else
    {
      for ( vcpt = 0; vcpt < dd->vdim; vcpt++ )
      {
        ltmin[vcpt]= tmin[vcpt];
        ltmax[vcpt]= tmax[vcpt];
      }
    }

    /* Here, axis_offset is an offset into the new image axis list.                           */
    /* offset+vcpt is the offset into the parent table buffer column list or image axis list. */
    /* binspec->offsets records the mapping.                                                  */

    for ( vcpt = 0; vcpt < dd->vdim; vcpt++ )
    {
      axisno       = offset + vcpt;
      axis_offset1 = axisno-1;

      binspec->offsets[axis_offset2] = offset;
      binspec->vcpts[axis_offset2]   = vcpt;
      binspec->groups[axis_offset2]  = group_offset;

      if ( parent_image )
      {
        step = fabs( cdelt[vcpt] );
        binspec->axis_map[axis_offset2] = offset + vcpt;
        binspec->min[axis_offset2] = (long)(ltmin[vcpt]+0.5+ltmin[vcpt]*FLT_EPSILON) + binspec->image_mins[axis_offset1] - 1;

        /* Need the following FLT_EPSILON because ltmax is intended as             */
        /* "up to but not including" whereas binspec->max is "up to and including" */

        binspec->max[axis_offset2] = (long)(ltmax[vcpt]+0.5-ltmax[vcpt]*FLT_EPSILON) + binspec->image_mins[axis_offset1] - 1;   
        /* Intersect image filter with bin filter. Note that min and max are now relative to */
	/* unfiltered image bounds.                                                          */

        if ( binspec->max[axis_offset2] < binspec->image_maxes[axis_offset1] )
        {
          binspec->image_maxes[axis_offset1] = binspec->max[axis_offset2];
        }

        if ( binspec->image_maxes[axis_offset1] < 1 )
          binspec->image_maxes[axis_offset1] = 1;

        if ( binspec->min[axis_offset2] > binspec->image_mins[axis_offset1] )
          binspec->image_mins[axis_offset1] = binspec->min[axis_offset2];

        if ( binspec->image_mins[axis_offset1] > parent_image->axes[axis_offset1] )
          binspec->image_mins[axis_offset1] = parent_image->axes[axis_offset1];

        binspec->start[axis_offset1] = binspec->image_mins[axis_offset1] - binspec->min[axis_offset2];
        binspec->axes[axis_offset2] = binspec->max[axis_offset2] - binspec->min[axis_offset2]+1;
      }
      else
      {
        binspec->min[axis_offset2] = axis->vals[vcpt][0];
        binspec->max[axis_offset2] = axis->vals[vcpt][1];
      }

      /* New logical step size relative to old logical step size */

      binspec->step[axis_offset2] = axis->vals[vcpt][2] / step;
      ++axis_offset2;
    }
  }
  
  binspec->weights = dmi_calc_image_weights( image->axes, image->naxes );

  /* Case of filter only */
  if ( binspec->n == 0 && image->naxes > 0 )
  {
    for( axis_offset1 = 0; axis_offset1 < image->naxes; axis_offset1++ )
    {     
      binspec->axis_map[axis_offset1] = axis_offset1+1;
      binspec->step[axis_offset1] = 1.0;
    }

    axis_offset2 = n2;
  }

  if( axis_offset2 != n2 )
  {
    sprintf( dm_errbuf, "Binning axes mismatch NAXES=%ld Sum(dd dim)=%ld", n2, axis_offset2 );
    dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
    return;
  }
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_calc_image_buffer_size                      */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            long        *taxes                               */
/*            long         naxes                               */
/*            long         pnaxes                              */
/*            dmBinSpec   *binspec                             */
/*            dmDataType   type                                */
/*            long         default_size                        */
/*            int          force_buffered                      */
/*            int         *is_buffered                         */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            long image_buffer_size                           */
/*                                                             */
/***************************************************************/
long dmi_calc_image_buffer_size( long* taxes, long naxes, long pnaxes,
           dmBinSpec* binspec, dmDataType type, long default_size,
           int force_buffered, int* is_buffered )
{
  long   i         = 0;
  long   byte_size = 0;
  long   image_buffer_size = 1;
  dmBool can_buffer        = UT_TRUE;

  for ( i = 0; i <  pnaxes; i++ )
    image_buffer_size *= taxes[i];

  /*
   * Check whether we can buffer the image
   *
   * FIXME:  This is a quick fix.  What we really need to do is fix the
   * buffering algorithm so that these cases work.  Some other time...
   */

  if ( ( naxes != pnaxes ) || ( naxes > 2 ) )
  {
    /* Number of axes has changed or is >2; can't buffer */
    can_buffer = UT_FALSE;
  }
  else
    if ( binspec && binspec->axis_map )
    {
      for ( i = 0; i < naxes; i++ )
      {
        if ( binspec->axis_map[i] != i + 1 )
        {
          /* Order of axes has changed; can't buffer */
          can_buffer = UT_FALSE;
          break;
        }
      }
    }

  byte_size = ar_mix_type_size( type );

  if ( can_buffer && ( image_buffer_size * byte_size > default_size || force_buffered ) )
  {
    *is_buffered = 1;
    image_buffer_size /=taxes[pnaxes-1];
  }
  else
  {
    *is_buffered = 0;
  }
 
  return image_buffer_size;
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_calc_image_buffer_size                      */
/*                                                             */
/* Description:                                                */
/*     This routine used by PutRow; since writing, vrow = rrow */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock   *block                                 */
/*            void      *row                                   */
/*            long       vrowno                                */
//*                                                            */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_copy_row_to_buffer( dmBlock* block, void* row, long vrowno )
{
  dmBuffer     *buffer    = block->buffer;
  dmDescriptor *dd        = NULL;
  dmDescriptor *length_dd = NULL;

  char *iptr =  NULL;
  char *optr =  NULL;

  long rrow_offset = 0;
  long i           = 0;
  long nitems      = 0;
  long size        = 0;

  rrow_offset = vrowno - buffer->rrow1;

#if 0
  /* If we had a row-oriented buffer, we could do this... */
  iptr = (char*)buffer->data + dmi_buffer_offset( buffer, 0, rrow_offset );
  nbytes = buffer->vtotal;
  memcpy( iptr, row, nbytes );
#endif

  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];

    if ( !( dd->flag & DM_FLAG_BINNED ) && ( dd->flag & DM_FLAG_VISIBLE ) )
    { 
      nitems = dd->size * dd->dim;  
      size = dmi_descriptor_get_bytes( dd );
      iptr = (char*)buffer->data + dmi_buffer_offset( buffer, i, rrow_offset );

      /* The code calling dmTablePutRow() knows about visible columns only, */
      /* so use voffsets rather than offsets                                */
 
      optr = (char*)row          + buffer->voffsets[i];

      memcpy( iptr, optr, size * nitems );

      /* This is a dumb way to write a variable-length array row, but we'll */
      /* allow it and just set the row length to the maximum                */

      if ( dmi_descriptor_is_varlen( dd ) )
      {
        length_dd = dmi_buffer_get_varlen_length_col( block, dd );

        if ( length_dd )
        {
          dmi_buffer_write_varlen_length( block, dd, length_dd, (void*)iptr, dmi_descriptor_get_array_size( dd ) );
        }
        else
        {
          sprintf( dm_errbuf, "Error in buffering variable-length array descriptor %s; all row-length information will be discarded", dd->name );
          dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
        }
      }
    }
  }

  if ( vrowno > buffer->lastrow )
    buffer->lastrow = vrowno;

  if ( buffer->vrow1 == 0 )
    buffer->vrow1 = buffer->rrow1;

  buffer->update = UT_TRUE;
  dmi_block_set_update( block );
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_copy_buffer_to_row                          */
/*                                                             */
/* Description:                                                */
/*    Replaced by dmp routine                                  */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock   *block                                 */
/*            void      *row                                   */
/*            long       rowno                                 */
//*                                                            */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_copy_buffer_to_row( dmBlock* block, void* row, long rowno )
{
  dmBuffer     *buffer = block->buffer;
  dmDescriptor *dd     = NULL;
  char         *iptr   = NULL;
  char         *optr   = NULL;

  long rrow_offset = 0;
  long i           = 0;
  long nitems      = 0;
  long size        = 0;


  rrow_offset = rowno - buffer->rrow1;

  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];

    if ( !( dd->flag & DM_FLAG_BINNED ) && ( dd->flag & DM_FLAG_VISIBLE ) )
    { 
      nitems = dd->size * dd->dim;  
      size = dmi_descriptor_get_bytes( dd );
      iptr = (char*)buffer->data + dmi_buffer_offset( buffer, i, rrow_offset );

      /* The calling code knows about visible columns only, so use voffsets */
      /* rather than offsets                                                */

      optr = (char*)row          + buffer->voffsets[i];

      memcpy( optr, iptr, size * nitems );
    }
  }
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_calc_image                           */
/*                                                             */
/* Description:                                                */
/*    Accumulate the image.                                    */ 
/*    The design of this routine is driven by the fact that    */
/*    the binning columns and the output image may all have    */
/*    different data types, and we don't want to have lots of  */
/*    switch statements inside the innermost row loop.         */
/*    So, we do things in stages a whole buffer at a time.     */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock      *block                              */
/*            dmDescriptor *image                              */
/*            long          nvrows_total                       */
//*                                                            */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_calc_image( dmBlock* block, dmDescriptor* image, long nvrows_total )
{
  dmBuffer     *buffer = block->buffer;
  dmDescriptor *weight_dd     = NULL;
  dmDescriptor *logical_image = NULL;
  dmDescriptor *dd            = NULL;
  dmBinSpec    *binspec       = NULL;

  void     *weight_data = NULL;
  void     *ptr   = NULL;
  float    *fptr  = NULL;
  double   *dptr  = NULL;
  double    tmin  = 0.0;
  double    tmax  = 0.0;
  double    tstep = 0.0;
  double    value = 0.0;
  double    pixel = 0.0;

  long  *lptr         = NULL;
  long  *pixel_offset = NULL;

  long   weight_col_offset = 0;
  long   weight_cpt_offset = 0;
  long   weight_size       = 0; /* Size of uncasted weight data */
  long   cast_weight_size  = 0; /* Size of casted weight data */
  long   weight            = 0;
  long   col_offset        = 0;
  long   size              = 0;
  long   offset            = 0;
  long   n                 = 0;
  long   buffer_row_offset = 0;
  long   rrow_offset       = 0;
  long   axis_offset       = 0;
  long   vcpt              = 0;
  long   group_offset      = 0;
  long   dvrows            = 0;
  long   vrow_offset       = 0;

  int   *state = NULL; 
  short *sptr  = NULL;



  /* First, cache the information about each binning axis     */
  /* for efficiency later.                                    */
  /* This is done on the first buffer of rows passed through. */

  n = image->naxes;

  logical_image = dmi_coord_get_parent( image );
  binspec       = dmi_descriptor_get_aux( logical_image );

  if ( !binspec->offsets )
    dmi_bin_map_offsets( binspec, NULL, image );


  /*  Now, loop through the filtered rows calculating which */
  /*  image pixel they correspond to.                       */
 
  dvrows = 1;
  state = calloc( nvrows_total, UT_SZ_I );
  pixel_offset = calloc( nvrows_total, UT_SZ_I );

  for ( vrow_offset =0; vrow_offset < nvrows_total; vrow_offset ++ )
  {
    state[vrow_offset] = 1;
  }
 
  for ( axis_offset = 0; axis_offset < n; axis_offset++ )
  {
    group_offset = binspec->groups[axis_offset];
    col_offset = binspec->offsets[axis_offset];

    if ( col_offset < 0 )
    {
      sprintf( dm_errbuf, "Binning axis %ld not in buffer", axis_offset+1 );
      dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
      return;
    }

    size = buffer->sizes[col_offset];
    dd = buffer->cols[col_offset];
    vcpt = binspec->vcpts[axis_offset];
    tmin = binspec->min[axis_offset];
    tmax = binspec->max[axis_offset];
    tstep = binspec->step[axis_offset];
    weight = binspec->weights[axis_offset];

    if ( binspec->weight_dd )
    {
      /* Get binning weights from weight_dd column in buffer */

      weight_dd = binspec->weight_dd;
      weight_size = ar_mix_type_size( weight_dd->type );
      cast_weight_size = ar_mix_type_size( image->type );
      weight_col_offset = weight_dd->bufno - 1;

      /* Use component offset only if weight_dd is a vector component */
      /* and its parent is in the buffer */

      if ( weight_dd->cptno > 0 && weight_dd->parent && weight_dd->parent->bufno > 0 )
        weight_cpt_offset = weight_dd->cptno - 1;

      weight_data = calloc( nvrows_total, cast_weight_size );
    }

    for ( vrow_offset = 0; vrow_offset < nvrows_total; vrow_offset ++ )
    {
      if ( buffer->filtered )
        rrow_offset = buffer->index[vrow_offset];
      else
        rrow_offset =vrow_offset;

      /* Work with this set of contiguous good rows */  
      ptr = (char*)buffer->data + dmi_buffer_offset( buffer, col_offset, rrow_offset );

      switch( dd->type )
      {
        case DT_REAL8:
          value = *((double*)ptr+vcpt);
          break;
        case DT_REAL4:
          value = (double)*((float*)ptr+vcpt);
          break;
        case DT_INT4:
          value = (double)*((long*)ptr+vcpt);
          break;
        case DT_INT2:
          value = (double)*((short*)ptr+vcpt);
          break;
        case DT_UINT4:
          value = (double)*((unsigned long*)ptr+vcpt);
          break;
        case DT_UINT2:
          value = (double)*((unsigned short*)ptr+vcpt);
          break;
        case DT_UINT1:
          value = (double)*((unsigned char*)ptr+vcpt);
          break;
        default:
          offset = 0.0;
          break;
      }

      if ( value >= tmin && value < tmax )
      {
        pixel = ( value - tmin ) / tstep;
        pixel_offset[vrow_offset] += ((long)floor( pixel )) * weight;

        if ( weight_dd )
        {
          /* Get casted weight data for this row */
          ptr = (char*)buffer->data + dmi_buffer_offset( buffer, weight_col_offset,
 	          rrow_offset ) + weight_size*weight_cpt_offset;
          ar_mix_numeric_cast( ptr, weight_dd->type, image->type,
                              (char*)weight_data + cast_weight_size*vrow_offset );
        }
      }
      else
      {
        /* If pixel is out of range on any axis, we lose it */
        state[vrow_offset] = 0;
      }

    } // end for vrow_offset...
  } // end for axis_offxet...


  /* Now we switch for each image output datatype, accumulating the image */ 
  /* looping over rows using the pixel values saved in the above step.    */

  switch( image->type )
  {
    case DT_INT2:
      sptr = buffer->vdata;

      for ( buffer_row_offset = 0; buffer_row_offset < nvrows_total; buffer_row_offset++ )
      {
        if ( state[buffer_row_offset] )
        {
          offset = pixel_offset[buffer_row_offset];

          if ( sptr[offset] == DT_INT2_MAX )
          {
            sprintf( dm_errbuf, "Binning will overflow i2 datatype, try [opt type=i4]" );
            dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
          }

          if ( !weight_dd )
            sptr[offset]++;
          else
            sptr[offset] += ((short*)weight_data)[buffer_row_offset];
        }
      }
      break;

    case DT_INT4:
      lptr = buffer->vdata;

      for ( buffer_row_offset = 0; buffer_row_offset < nvrows_total; buffer_row_offset++ )
      {
        if ( state[buffer_row_offset] )
        {
          offset = pixel_offset[buffer_row_offset];

          if ( lptr[offset] == DT_INT4_MAX )
          {
            sprintf( dm_errbuf, "Binning will overflow i4 datatype, try [opt type=r4]" );
            dmu_err_set( DM_ERR_INTERNAL, dm_errbuf );
          }

          if ( !weight_dd )
            lptr[offset]++;
          else
            lptr[offset] += ((long*)weight_data)[buffer_row_offset];
        }
      }
      break;

    case DT_REAL4:
      fptr = buffer->vdata;

      for ( buffer_row_offset = 0; buffer_row_offset < nvrows_total; buffer_row_offset++ )
      {
        if ( state[buffer_row_offset] )
        {
          offset = pixel_offset[buffer_row_offset];

          if ( !weight_dd )
            fptr[offset]++;
          else
            fptr[offset] += ((float*)weight_data)[buffer_row_offset];
        }
      }
      break;

    case DT_REAL8:
      dptr = buffer->vdata;

      for ( buffer_row_offset = 0; buffer_row_offset < nvrows_total; buffer_row_offset++ )
      {
        if ( state[buffer_row_offset] )
        {
          offset = pixel_offset[buffer_row_offset];

          if ( !weight_dd )
            dptr[offset]++;
          else
            dptr[offset] += ((double*)weight_data)[buffer_row_offset];
        }
      }
      break;

    default: 
      break;
  } // end switch

  free( state );
  free( pixel_offset );  
  free( weight_data );
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_binspec_free_cache                          */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBinspec *binspec                               */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_binspec_free_cache( dmBinSpec* binspec )
{
  free( binspec->min );
  binspec->min = NULL;

  free( binspec->max ); 
  binspec->max = NULL;

  free( binspec->axes );
  binspec->axes =NULL;

  free( binspec->offsets );
  binspec->offsets = NULL;

  free( binspec->step );
  binspec->step = NULL;

  free( binspec->start );
  binspec->start = NULL;

  free( binspec->axis_map );
  binspec->axis_map = NULL;

  free( binspec->image_mins );
  free( binspec->image_maxes );
  binspec->image_mins = NULL;
  binspec->image_maxes = NULL;
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_get_sub_dde                          */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmSubspace   *rts                                */
/*            dmDescriptor *dd                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            dmDescriptor *sub_dd                             */
/*                                                             */
/***************************************************************/
dmDescriptor* dmi_buffer_get_sub_dd( dmSubspace* rts, dmDescriptor* dd )
{
  long  rno = 0;
  dmDescriptor *sub_dd = NULL;
 
  if ( rts )
  {
    for ( rno = 0; rno < rts->ncols; rno++ )
    {
      sub_dd = rts->cols[rno];

      if ( sub_dd && sub_dd->parent == dd )
        return sub_dd;
    }     
  }
  return NULL;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_index_flag_name                      */
/*                                                             */
/* Description:                                                */
/*       For debug                                             */
/*                                                             */
/* Each descriptor has a flag - a bitmask describing what kind */
/* of descriptor it is. This routine maps these descriptor     */
/* flags to buffer index flags, which describe how to handle   */
/* buffering of the descriptor when reading rows from the      */
/* block.                                                      */
/*                                                             */
/* DMBI_FCALC if the descriptor is the result of a calculation */
/*   but is needed in filtering.                               */
/* DMBI_FREAD if the descriptor is needed for the result of a  */
/*   calculation that is needed in filtering.                  */
/* DMBI_FILT if dd is a raw column being filtered              */
/* DMBI_READ if dd is a column that needs reading but not      */
/*   filtering                                                 */
/* DMBI_BINNED if dd is a binned image in memory               */
/* DMBI_OTHER if dd is something that needs to be calculated at*/
/*    the end, etc.                                            */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            long  flag                                       */
/*                                                             */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            char *name                                       */
/*                                                             */
/***************************************************************/
char* dmi_buffer_index_flag_name( long flag )
{
 static char* names[] = {  "FILT", "FCALC", "FREAD", "READ", "BINNED", "OTHER", "N" };
 return names[flag];
}

/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_index_flag                           */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            long   flag                                      */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            long index_flag                                  */
/*                                                             */
/***************************************************************/
long dmi_buffer_index_flag( long flag )
{
  long index_flag = -1;

  if ( flag & DM_FLAG_UNBUF )
  {
    index_flag = DMBI_OTHER;
  }
  else if ( flag &  DM_FLAG_BINNED  )
  {
    index_flag = DMBI_BINNED;
  }
  else if ( flag &  DM_FLAG_FILT  )
  {  
    if ( flag & DM_FLAG_RESULT )
      index_flag = DMBI_FCALC;
    else
      index_flag = DMBI_FILT;
  }
  else if ( flag & DM_FLAG_FILT_OPERAND )
  {
    index_flag = DMBI_FREAD;
  }
  else if ( flag & DM_FLAG_RESULT )
  {
    index_flag = DMBI_OTHER;
  }
  else if( flag & ( DM_FLAG_VISIBLE | DM_FLAG_RAW | DM_FLAG_OPERAND ) )
  {
    index_flag = DMBI_READ;
  }
  else
  {
    index_flag = DMBI_OTHER;
  }

 return index_flag;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_row_filter_extremes                  */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*  The row filter can be several components.                  */
/*  The range of rows we need is the union of all the component*/
/*   ranges.  Then we intersect that result with the initial   */
/*   range passed to us.                                       */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmDescriptor *sub_dd                             */
/*            long          ncpts                              */
/*                                                             */
/*  Output parameters:                                         */
/*            long         *minrow                             */
/*            long         *maxrow                             */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_row_filter_extremes( dmDescriptor* sub_dd, long ncpts,
                                     long* minrow, long* maxrow )
{
  dmFilter    *filter = NULL;
  dmRangeList *range  = NULL;

  long row1  = 0;
  long row2  = 0;
  long n     = 0;
  long cpt   = 0;
  long trow1 = 0;
  long trow2 = 0;


  if ( !sub_dd )
    return;

  row2 = *minrow-1;
  row1 = *maxrow;

  for ( cpt = 1; cpt <= ncpts; cpt++ )
  {
    filter = dmi_subspace_col_get_filter( sub_dd, cpt );

    if ( filter )   
      range = &filter->ranges[0];
    else
      range = NULL;

    if ( range && range->n > 0 )
    {
      /* make sure ranges are sorted before assuming where the min/max are */
      if ( !range->sorted ) 
        dmi_rangelist_reduce( range, DT_REAL8 );

      trow1 = ((double*)range->mins)[0];
      trow2 = ((double*)range->maxes)[range->n-1];

      if ( trow1 < row1 )
        row1 = trow1;

      if ( trow2 > row2 )
        row2 = trow2;

      ++n;  
    }
  }

  /* Use n to hedge against there being no filters */
  if ( n && row1 > *minrow )
    *minrow = row1;

  if ( n && row2 < *maxrow )
    *maxrow = row2;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_virtual_buffer_reset                        */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBlock  *block                                  */
/*                                                             */
/*  Output parameters:                                         */
/*            dmBuffer *buffer                                 */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_virtual_buffer_reset( dmBlock* block,  dmBuffer* buffer )
{
  dmDescriptor *sub_dd = NULL;
  dmDescriptor *dd     = NULL;
  dmSubspace   *rts    = NULL;
  dmFilter     *filter = NULL;
  dmRTSCache   *cache  = NULL;
  dmBool        testnull = dmFALSE;
  dmBool        isnull   = dmFALSE;
 
  long *crossindex = NULL;
  long  i          = 0;
  long  index_flag = 0;
  long  id         = 0;
  long  ncpts      = 1;
  long  cpt        = 0;
  long  n          = 0;
  long  nf         = 0; /* index to the filtered descriptors */
  long  dim        = 0;
  long  offset     = 0;


  if ( !buffer )
    return;

  buffer->minrow = 1;

  if ( dmi_block_writeable( block ))
  {
    if ( block->type == dmIMAGE )
      buffer->maxrow = 1;
    else
      buffer->maxrow = LONG_MAX;
  }
  else
    buffer->maxrow = block->nrows;

  rts = buffer->subspace;

  if ( rts )
  {
    ncpts = rts->ncpts;

    if ( rts->cache )
      dmi_subspace_free_cache( rts );

    rts->cache = calloc( 1, sizeof( dmRTSCache ));
    cache = rts->cache;
    cache->accept = calloc( ncpts, UT_SZ_Q );
  }
  else
    cache = NULL;

  /* Count number of filtered columns */
  crossindex           = calloc( buffer->ncols, UT_SZ_I );
  buffer->n_index      = calloc( DMBI_N, UT_SZ_I );
  buffer->col_index    = calloc( DMBI_N, UT_SZ_P );
  buffer->offset_index = calloc( DMBI_N, UT_SZ_P );

  for ( i = 0; i < buffer->ncols; i++ )
  {
    dd = buffer->cols[i];

    /* Map descriptor flags to buffer flags */
    index_flag = dmi_buffer_index_flag( dd->flag );
    crossindex[i] = index_flag;
    buffer->n_index[index_flag]++;
  }

  if ( cache )
  {
    /* allocate the lists for optimized filtering */
    cache->n_filt      = buffer->n_index[DMBI_FCALC] + buffer->n_index[DMBI_FILT];
    cache->ddlist      = calloc( cache->n_filt, UT_SZ_P );
    cache->col_offsets = calloc( cache->n_filt, UT_SZ_I );
    cache->sub_ddlist  = calloc( cache->n_filt, UT_SZ_P );
    cache->filter_list = calloc( cache->n_filt, UT_SZ_P );
    cache->null        = UT_FALSE;
  }

  testnull = UT_FALSE;

  /* Allocate space for each index */
  for ( index_flag = 0; index_flag < DMBI_N; index_flag++ )
  {
    n = buffer->n_index[index_flag];

    if ( n > 0 )
    {
      buffer->col_index[index_flag] = calloc( n, UT_SZ_P );
      buffer->offset_index[index_flag] = calloc( n, UT_SZ_I );
    }

    /* Clear the counter */
    buffer->n_index[index_flag] = 0;
  }

  nf = 0;

  /* Fill the indexes */
  for ( id = 0; id < buffer->ncols; id++ )
  {
    index_flag = crossindex[id];
    n = buffer->n_index[index_flag]++;
    dd = buffer->cols[id];
    buffer->col_index[index_flag][n] = dd;
    buffer->offset_index[index_flag][n] = id;

    /* If a filtered column, cache the pointer to the filter */
    if ( index_flag == DMBI_FILT || index_flag == DMBI_FCALC )
    {
      buffer->filtered = UT_TRUE;
      sub_dd = dmi_buffer_get_sub_dd( rts, dd );
      cache->ddlist[nf] = dd;
      cache->sub_ddlist[nf] = sub_dd;
      cache->col_offsets[nf] = id;
      cache->filter_list[nf] = calloc( ncpts, UT_SZ_P );
      isnull = UT_TRUE;

      for ( cpt = 1; cpt <= ncpts; cpt++ )
      {
        filter = dmi_subspace_col_get_filter( cache->sub_ddlist[nf], cpt );
        cache->filter_list[nf][cpt-1] = filter;
        dim = filter->dim;

        for ( offset = 0; offset < dim; offset++ )
        {
          /* Do we need to test for null/NaN values when filtering? */
          if ( filter->ranges[offset].incnull )
            testnull = UT_TRUE;
        }

        if ( !filter->null )
          isnull = UT_FALSE;
      }

      /* there is a filter that is null over all components, */
      /* therefore the entire filter is null                 */

      if ( isnull )
        cache->null = UT_TRUE;
  
      ++nf;

      /* Optimize row filtering by caching row range */
      if ( dd->flag & DM_FLAG_ROW )
      {
        dmi_buffer_row_filter_extremes( sub_dd, ncpts, &buffer->minrow, &buffer->maxrow );
        dmi_buffer_add_execute_func( &buffer->execute1, DM_EXEC_ROW, &id, 1 );
      }
      else
        if ( dd->flag & DM_FLAG_COORD && index_flag == DMBI_FCALC )
        {
          dmi_buffer_add_execute_coord( buffer, &buffer->execute1, id );
        }
    }
    else
      if ( index_flag == DMBI_OTHER )
      {
        if ( dd->flag & DM_FLAG_UNBUF )
        {
          continue;
        }
        else
          if ( dd->flag & DM_FLAG_ROW )
          {
            dmi_buffer_add_execute_func( &buffer->execute2, DM_EXEC_ROW, &id, 1 );
          }
	  else
            if ( dd->flag & DM_FLAG_COORD )
            {
              dmi_buffer_add_execute_coord( buffer, &buffer->execute2, id );
            }
      }

    if ( index_flag == DMBI_BINNED )
    {
      if ( buffer->image )
        dmu_err_set( DM_ERR_INTERNAL, "More than one binned image" );

      buffer->image = dd;
    }
  }

  free( crossindex );

  if ( !testnull )
    buffer->filter_rows = dmi_filter_rows_simple;
  else
    buffer->filter_rows = dmi_filter_rows_with_nulltest;
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_set_row_vals                         */
/*                                                             */
/* Description:                                                */
/*      Set values into the raw row descriptor                 */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            long   rrowno1                                   */
/*            long   rrowno2                                   */
/*                                                             */
/*  Output parameters:                                         */
/*            long  *values                                    */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_set_row_vals( long* values, long rrowno1, long rrowno2 )
{
  long rrowno = 0;

  for ( rrowno = rrowno1; rrowno <= rrowno2; rrowno++ )
  {
    values[rrowno-rrowno1] = rrowno;
  }
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_filter_rows                                 */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*                                                             */
/*  Output parameters:                                         */
/*            void  *data                                      */
/*                                                             */
/*  Returns:                                                   */
/*            long status                                      */
/*                                                             */
/***************************************************************/
long dmi_filter_rows( dmBuffer* buffer, void* data )
{
  return buffer->filter_rows( buffer, data );
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_image_rebin_segment                         */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmDescriptor  *image                             */
/*            void          *image_buffer                      */
/*            long           image_buffer-size                 */
/*            long          *toffset                           */
/*            long          *llpix                             */
/*            long          *urpix                             */
/*            double        *tstep                             */
/*            long           onaxe                             */
/*                                                             */
/*  Output parameters:                                         */
/*            void  *data                                      */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_image_rebin_segment( dmDescriptor* image, void* image_buffer,
                              long image_buffer_size, long* toffset,
                              long* llpix, long* urpix, double* tstep,
                              long onaxes, void* data )
{
  float  *fptr1      = NULL;
  float  *fptr2      = NULL;
  double *dptr1      = NULL;
  double *dptr2      = NULL;
  double  axis_value = 0;

  long *lptr1      = NULL;
  long *lptr2      = NULL;
  long *weight     = NULL;
  long *opixel     = NULL;
  long inc_axis    = 0;
  long offset      = 0;
  long new_offset  = 0;
  long axis_offset = 0;
  long maxis       = 0;
  long nvals       = 0;
  long lnull       = 0;

  short *sptr1 = NULL;
  short *sptr2 = NULL;
  short  snull = 0;

  nvals  = dmi_descriptor_get_nvals( image );
  weight = dmi_calc_image_weights( image->axes, image->naxes );
  opixel = malloc( onaxes * UT_SZ_I );

  for ( inc_axis = 0; inc_axis < onaxes; inc_axis++ )
    opixel[inc_axis] = llpix[inc_axis];

  /* Rebin into new image. The image buffer goes from tlmin:tlmax */

  switch( image->type )
  {
    case DT_INT2:
      sptr2= (short*)data;
      sptr1= (short*)image_buffer;

      if ( image->null )
        snull = *(short*)image->null;

      for ( offset = 0; offset < image_buffer_size; offset++ )
      {
        new_offset = 0;

        for ( axis_offset = 0; axis_offset < image->naxes; axis_offset++ )
        {
          maxis = toffset[axis_offset];
          axis_value = ((double)opixel[maxis-1] + 0.5) / tstep[axis_offset];
          new_offset += (long)(floor(axis_value)) * weight[axis_offset];
        }

       /* Copy value - what if null? */

#if 0
printf( "%6ld %6ld NO %6ld OP %6ld %6ld NP %10.2f %10.2f\n", offset, image_buffer_size, new_offset, opixel[0], opixel[1], opixel[0] / tstep[0], opixel[1] / tstep[1] );
#endif
 
       if ( new_offset >= 0 && new_offset < nvals )
       {
         if ( !image->null )
           sptr2[new_offset] += sptr1[offset];     
         else
         {
           if ( sptr1[offset] == snull )
           {
             if ( sptr2[new_offset] == 0 )
               sptr2[new_offset] = snull;
           }
           else
             if ( sptr2[new_offset] == snull )
             {
               if ( sptr1[offset] )
                 sptr2[new_offset] = sptr1[offset];
             }
             else
               sptr2[new_offset] += sptr1[offset];   
         }
       }


#if DEBUG
printf( "COPY OFFSET %ld TO %ld\n", offset, new_offset );
#endif

       /* Increment old pixel */
       opixel[0]++;
       inc_axis = 0;

       while( opixel[inc_axis] >= urpix[inc_axis] && inc_axis < onaxes - 1 )
       {
         opixel[inc_axis] = llpix[inc_axis];
         ++inc_axis;
         opixel[inc_axis]++;
       }
     }
     break;

   case DT_INT4:
     lptr1 = (long*)image_buffer;
     lptr2 = (long*)data;

     if ( image->null )
       lnull = *(long*)image->null;

     for ( offset = 0; offset < image_buffer_size; offset++ )
     {
       inc_axis = 0;
       new_offset = 0;

       for ( axis_offset = 0; axis_offset < image->naxes; axis_offset++ )
       {
         maxis = toffset[axis_offset];
         axis_value = ((double)opixel[maxis-1] + 0.5) / tstep[axis_offset];
         new_offset += (long)(floor(axis_value)) * weight[axis_offset];
       }

       opixel[0]++;

       while( opixel[inc_axis] >= urpix[inc_axis] && inc_axis < onaxes - 1)
       {
         opixel[inc_axis] = llpix[inc_axis];
         inc_axis++;
         opixel[inc_axis]++;
       }

       if ( new_offset >= 0 && new_offset < nvals )
       {
         if ( !image->null )
           lptr2[new_offset] += lptr1[offset];
         else
         {
           if ( lptr1[offset] == lnull )
           {
             if ( lptr2[new_offset] == 0 )
	       lptr2[new_offset] = lnull;
           }
           else
             if ( lptr2[new_offset] == lnull )
             {
               if ( lptr1[offset] )
	         lptr2[new_offset] = lptr1[offset];
             }
             else
               lptr2[new_offset] += lptr1[offset];   
         }
       }
     }
     break;

    case DT_REAL4:
      fptr1 = (float*)image_buffer;
      fptr2 = (float*)data;

      for ( offset = 0; offset < image_buffer_size; offset++ )
      {
        inc_axis = 0;
        new_offset = 0;

        for ( axis_offset = 0; axis_offset < image->naxes; axis_offset++ )
        {
          maxis = toffset[axis_offset];
          axis_value = ((double)opixel[maxis-1] + 0.5) / tstep[axis_offset];
          new_offset += (long)(floor(axis_value)) * weight[axis_offset];
        }

        opixel[0]++;

        while( opixel[inc_axis] >= urpix[inc_axis] && inc_axis < onaxes - 1 )
        {
          opixel[inc_axis] = llpix[inc_axis];
          ++inc_axis;
          opixel[inc_axis]++;
        }

        if ( new_offset >= 0 && new_offset < nvals )
          fptr2[new_offset] += fptr1[offset];
      }
      break;

    case DT_REAL8:
      dptr1 = (double*)image_buffer;
      dptr2 = (double*)data;

      for ( offset = 0; offset < image_buffer_size; offset++ )
      {
        inc_axis = 0;
        new_offset = 0;

        for ( axis_offset = 0; axis_offset < image->naxes; axis_offset++ )
        {
          maxis = toffset[axis_offset];
          axis_value = ((double)opixel[maxis-1] + 0.5) / tstep[axis_offset];
          new_offset += (long)(floor(axis_value)) * weight[axis_offset];
        }

        opixel[0]++;

        while( opixel[inc_axis] >= urpix[inc_axis] && inc_axis < onaxes - 1 )
        {
          opixel[inc_axis] = llpix[inc_axis];
          ++inc_axis;
          opixel[inc_axis]++;
        }

        if ( new_offset >= 0 && new_offset < nvals )
          dptr2[new_offset] += dptr1[offset];
      }
      break;

    default:   
      break;
  } // end switch

  free( weight );
  free( opixel );
}


/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_copu_to_vdata                        */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer                                 */
/*            long      nvrows_total                           */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            int status                                       */
/*                                                             */
/***************************************************************/
int dmi_buffer_copy_to_vdata( dmBuffer*  buffer, long nvrows_total )
{
  if ( !buffer->vdata )
    buffer->vdata = calloc( buffer->total * buffer->nrows, UT_SZ_C );

  return  dmi_buffer_copy( buffer, buffer, buffer->data, buffer->vdata, nvrows_total );
}



/***************************************************************/
/*                                                             */
/* Function:   dmi_buffer_copu                                 */
/*                                                             */
/* Description:                                                */
/*  Copy data from one buffer to another, explicitly giving the*/
/*  buffer data pointers. Could be enhanced to support         */
/*  row-oriented buffers.                                      */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*            dmBuffer *buffer1                                */
/*            dmBuffer *buffer2                                */
/*            void     *data1                                  */
/*            void     *data2                                  */
/*            long      nvrows_total                           */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            int status                                       */
/*                                                             */
/***************************************************************/
int dmi_buffer_copy( dmBuffer* buffer1, dmBuffer* buffer2, void* data1,
                     void* data2, long nvrows_total )
{
  dmDescriptor *dd = NULL;

  char *iptr = NULL;
  char *optr = NULL;

  long  dvrows       = 0;
  long  vrow_offset  = 0;         /* Row offset into virtual buffer */
  long  rrow_offset  = 0;  /* Row offset into raw buffer */
  long  col_offset   = 0;
  long  contig_start = 0;
  long  nbytes       = 0;

  for ( vrow_offset =0; vrow_offset < nvrows_total; vrow_offset += dvrows )
  {
    if ( buffer1->filtered )
    {
      rrow_offset = buffer1->index[vrow_offset];
      contig_start = buffer1->contig_start[vrow_offset];
      dvrows = buffer1->contiguous[contig_start] - ( vrow_offset - contig_start );

#if DEBUG
printf( "VRO = %ld RRO=%ld CONTIG START = %ld CONTIG = %ld DVR = %ld\n", vrow_offset, rrow_offset, contig_start,  buffer1->contiguous[contig_start], dvrows );
#endif

    }
    else
    {
      rrow_offset =0;
      dvrows = nvrows_total;
    }

    if ( dvrows == 0 )
    {
      dmu_err_set( DM_ERR_INTERNAL, "SYNC ERROR IN BUFFER" );
      return UT_FALSE;
    }

    /* Copy dvrows from data to vdata */
    /* Assumes col_order */

    for( col_offset = 0; col_offset < buffer1->ncols; col_offset++ )
    {
      dd = buffer1->cols[col_offset];

      if ( ( dd->flag & DM_FLAG_VISIBLE ) || ( dd->flag & DM_FLAG_AUX_COLUMN ) )
      {
        if ( dd->flag & DM_FLAG_UNBUF )
        {
          printf( "UNBUF %s\n", dd->name );
        }
        else
        {
          iptr = (char*)data1  + dmi_buffer_offset( buffer1, col_offset, rrow_offset );
          optr = (char*)data2 +  dmi_buffer_offset( buffer2, col_offset, vrow_offset );

          /* Note that this copies all vector components */
          nbytes = dvrows * buffer1->sizes[col_offset];

#if DEBUG
printf( "DATA2 OFFSET = %ld:%ld %ld DATA1 OFFSET = %ld:%ld %ld R2=%ld R1=%ld CO=%ld DVR=%ld SIZE=%ld INVD=%d\n", optr-(char*)data2, optr-(char*)data2+nbytes-1,
optr+nbytes-1, iptr-(char*)data1, iptr-(char*)data1+nbytes-1, iptr+nbytes-1, vrow_offset, rrow_offset, col_offset, dvrows, buffer1->sizes[col_offset], buffer1->in_vdata );
#endif

          memcpy( optr, iptr, nbytes );
        }
      }
    }
  }  

 return UT_TRUE;
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_resync                             */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          dmBuffer     *buffer                               */
/*          long          nvrows                               */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*               long     buffer_nrows                         */
/*                                                             */
/***************************************************************/
long dmi_buffer_resync( dmBlock* block, dmBuffer* buffer, long nvrows )
{
  long buffer_size  = 0;
  long buffer_nrows = 0;

  free( buffer->data );
  free( buffer->index );
  free( buffer->contig_start );
  free( buffer->contiguous );

  buffer_nrows  = dmi_buffer_eval_size( block, nvrows );
  buffer->nrows = buffer_nrows;
  buffer_size   = buffer->total * buffer_nrows;

  // Code Change  September 29, 2008  Enh 11292
  // Allocate data buffer only if size > 0

  if ( buffer_size > 0 )
    buffer->data = calloc( buffer_size, UT_SZ_C );
  else
    buffer->data = NULL;


  /* Use vdata for either filtered rows or binned image */

  free( buffer->vdata );

  if ( buffer->imtotal > 0 )
    buffer->vdata = calloc( buffer->imtotal, UT_SZ_C );
  else
    buffer->vdata = NULL;

  buffer->index        = calloc( buffer_nrows, UT_SZ_I );
  buffer->contiguous   = calloc( buffer_nrows, UT_SZ_I );
  buffer->contig_start = calloc( buffer_nrows, UT_SZ_I );

  buffer->rrow1 = 0;
  buffer->rrow2 = 0;
  buffer->vrow1 = 0;
  buffer->vrow2 = 0;
  buffer->lastrow = 0;
  buffer->resync  = UT_FALSE;

  /* Note that vrow_go1 and vrow_go2 must be left unchanged */
  return buffer_nrows;
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_get_varlen_length_col_name         */
/*                                                             */
/* Description:                                                */
/*   If varlen_dd->name is "NAME", then the string returned    */
/*   (i.e. the name of varlen_dd's row-length column) is       */
/*   "#NAME_rowlen"                                            */
/*   The string allocated by this function must be freed by    */
/*   the caller                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmDescriptor *varlen_dd                            */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*               char *name                                    */
/*                                                             */
/***************************************************************/
static char* dmi_buffer_get_varlen_length_col_name( dmDescriptor* varlen_dd )
{
  char *name = NULL;
  char  name_prefix[] = "#";
  char  name_suffix[] = "_rowlen";
  long  name_len = 0;

  if ( !varlen_dd || !varlen_dd->name || !(*(varlen_dd->name)) )
    return NULL;

  name_len = strlen( name_prefix ) + strlen ( varlen_dd->name ) + strlen( name_suffix ) + 1;
  name = malloc( name_len * UT_SZ_C );
  sprintf( name, "%s%s%s", name_prefix, varlen_dd->name, name_suffix );

  return name;
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_add_varlen_length_col              */
/*                                                             */
/* Description:                                                */
/*   Create a column in the buffer to hold row lengths for     */
/*   variable-length array descriptor varlen_dd.  This column  */
/*   is hidden; it exists only in the buffer and will never be */
/*   flushed with the block.                                   */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          dmDescriptor *varlen_dd                            */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*               none                                          */
/*                                                             */
/***************************************************************/
void dmi_buffer_add_varlen_length_col( dmBlock* block, dmDescriptor* varlen_dd )
{
  dmDescriptor *length_dd = NULL;
  char *name = NULL;

  if ( !block || !block->buffer || !varlen_dd || !varlen_dd->varlen ||
       varlen_dd->dim != 1 || varlen_dd->cptno != 0 || varlen_dd->naxes != 1 )
  {
    /* As far as the DM is concerned, this descriptor can't be variable-length. */
    /* However, the kernel may still need to know that it's supposed to be      */
    /* variable-length.  (For example, the FITS kernel needs to know this in    */
    /* order to flush the column correctly.)  Therefore, *don't* unset the      */
    /* varlen flag.  Functions that handle varlen arrays will be able to tell   */
    /* that the length_dd is missing by checking if                             */
    /* dmi_buffer_get_varlen_length_col() returns NULL.                         */

     return;
  }

  /* Create row-length column */

  name = dmi_buffer_get_varlen_length_col_name( varlen_dd );

  if ( !name )
  {
    /* Can't have a nameless row-length column; something went wrong */
    return;
  }

  length_dd = dmi_descriptor_alloc( block, dmCOLUMN, 0, name, DT_INT4, NULL,
	 			    NULL, 1, dmVALUE, 1, NULL, 0 );

  free( name );

  /* Add row-length column to buffer */
  dmi_buffer_add_col( block->buffer, length_dd, DM_FLAG_AUX_COLUMN );
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_get_varlen_length_col              */
/*                                                             */
/* Description:                                                */
/*   Return the row-length column created in the buffer for    */
/*   variable-length array descriptor varlen_dd.  If it can't  */
/*   be found, return NULL.                                    */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          dmDescriptor *varlen_dd                            */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*               dmDescriptor *length_dd                       */
/*                                                             */
/***************************************************************/
dmDescriptor* dmi_buffer_get_varlen_length_col( dmBlock* block, dmDescriptor* varlen_dd )
{
  dmBuffer     *buffer    = NULL;
  dmDescriptor *length_dd = NULL;
  char         *name      = NULL;

  if ( !block || !block->buffer || !varlen_dd )
    return NULL;
 
  buffer = block->buffer;
  name = dmi_buffer_get_varlen_length_col_name( varlen_dd );
  length_dd = dmi_descriptor_list_search_descriptor( (const dmDescriptor**)buffer->cols, buffer->ncols, name );
  free( name );

  /* If length_dd is NULL (i.e. it wasn't found), let caller decide how to */
  /* deal with it */

  return length_dd;
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_calc_row_offset_from_ptr           */
/*                                                             */
/* Description:                                                */
/*   Given "data", a pointer to somewhere in the data array    */
/*   for "dd" in the buffer, return the row offset             */
/*   corresponding to the location to which "data" points.     */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBuffer     *buffer                               */
/*          dmDescriptor *dd                                   */
/*          void         *data                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*            long offset                                      */
/*                                                             */
/***************************************************************/
static long dmi_buffer_calc_row_offset_from_ptr( dmBuffer* buffer, dmDescriptor* dd, void* data )
{
  char *ptr =  NULL;
  char *ptr0 = NULL;  /* Pointer to dd data for row offset of zero */
  long  col_offset = 0;

  if ( !buffer || !dd || !data )
    return 0;

  ptr = (char*)data;
  col_offset = dd->bufno - 1;
  ptr0 = (char*)buffer->data + dmi_buffer_offset( buffer, col_offset, 0 );

  return ( ( ptr - ptr0 ) / buffer->sizes[col_offset] );
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_write_varlen_length                */
/*                                                             */
/* Description:                                                */
/*   Store the length of a variable-length array row in the    */
/*   buffer.                                                   */
/*   "dd" is the variable-length array column.                 */
/*   "length_dd" is the hidden buffer column that holds row    */
/*       lengths for "dd". It must have been created using     */
/*       dmi_buffer_add_varlen_length_col().                   */
/*   "dd_ptr" is the location in the buffer of dd's data for   */
/*       this row. It's used to extract the current row offset.*/
/*   "length" is the length value to write.                    */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          dmDescriptor *dd                                   */
/*          dmDescriptor *length_dd                            */
/*          void         *dd_ptr                               */
/*          long          length                               */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*            none                                             */
/*                                                             */
/***************************************************************/
void dmi_buffer_write_varlen_length( dmBlock* block, dmDescriptor* dd,
                                     dmDescriptor* length_dd,
                                     void* dd_ptr, long length )
{
  dmBuffer *buffer     = NULL;
  char     *ptr        = NULL;
  long      col_offset = 0;
  long      row_offset = 0;

  if ( !block || !block->buffer || !dd || !length_dd || !dd_ptr )
    return;

  buffer = block->buffer;
  col_offset = length_dd->bufno - 1;

   row_offset = dmi_buffer_calc_row_offset_from_ptr( buffer, dd, dd_ptr );
  ptr = (char*)buffer->data + dmi_buffer_offset( buffer, col_offset, row_offset );

  /* Counting on length_dd to hold a dmLONG here */
  *((long*)ptr) = length;
}



/***************************************************************/
/*                                                             */
/* Function:     dmi_buffer_read_varlen_length                 */
/*                                                             */
/* Description:                                                */
/*   Read the length of a variable-length array row from the   */
/*   buffer. If an error occurs, the function will return -1.  */
/*                                                             */
/*   "dd" is the variable-length array column.                 */
/*   "length_dd" is the hidden buffer column that holds row    */
/*        lengths for "dd". It must have been created using    */
/*         dmi_buffer_add_varlen_length_col().                 */
/*   "dd_ptr" is the location in the buffer of dd's data for   */
/*         this row.  It's used to extract the current row     */
/*         offset.                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*          dmBlock      *block                                */
/*          dmDescriptor *dd                                   */
/*          dmDescriptor *length_dd                            */
/*          void         *dd_ptr                               */
/*                                                             */
/*  Output parameters:                                         */
/*                        none                                 */
/*                                                             */
/*  Returns:                                                   */
/*            long length                                      */
/*                                                             */
/***************************************************************/
long dmi_buffer_read_varlen_length( dmBlock* block, dmDescriptor* dd,
                                    dmDescriptor* length_dd, void* dd_ptr )
{
  dmBuffer *buffer = NULL;
  void     *data   = NULL;
  char      *ptr   = NULL; 
  long col_offset  = 0;
  long row_offset  = 0;

  if ( !block || !block->buffer || !dd || !length_dd || !dd_ptr )
    return -1;
 
  buffer = block->buffer;

  /* Not sure this is necessary, but it doesn't seem to hurt */
  if ( buffer->in_vdata )
    data = buffer->vdata;
  else
    data = buffer->data;

  col_offset = length_dd->bufno - 1;
  row_offset = dmi_buffer_calc_row_offset_from_ptr( buffer, dd, dd_ptr );
  ptr = (char*)data + dmi_buffer_offset( buffer, col_offset, row_offset );

  /* Counting on length_dd to hold a dmLONG here */
  return *((long*)ptr);
}


