/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "ascdm.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef CARD_SIZE
#define CARD_SIZE 80
#endif

extern char* cs_dup( const char* );
extern void cs_copy_siz( const char* source, char* dest, long siz );
extern dmErrCode dmGetVectorArray( dmDescriptor* dd, dmDataType type, void* vals, long dim, long start, long nvals);


#define DM_DOUBLE_PRECISION 13
#define DM_FLOAT_PRECISION  8;
void printCols( dmBlock* block );
void getCompositeName( dmDescriptor* dd, char* val, long vlen );

int main( int argc, char* argv[] )
{
 dmDataset* ds;
 dmBlock* blk;

 dmBool tval;

 char *nohead_simple_file;
 char *simple_file;
 char *sm_file;
 char *dtf_file;

 if ( argc < 5 )
 {
  printf( "astest: Args are not correct.\n" );
  exit (EXIT_FAILURE);
 }
 simple_file = argv[1];
 nohead_simple_file = argv[2];
 sm_file = argv[3];
 dtf_file = argv[4];
 blk = dmTableOpen(simple_file);
 if ( !blk )
 {
  printf( "Block is Null\n" );
 }
 printf( "Printing RAW Cols ------------------------------------\n" );
 printCols( blk );
 dmTableClose( blk );

 blk = dmTableOpen(sm_file);
 if ( !blk )
 {
  printf( "SIMPLE Block is Null\n" );
 }
 printf( "Printing SIMPLE Dataset cols --------------------------------\n" );
 printCols( blk );
 
 dmTableClose( blk );

 blk = dmTableOpen(nohead_simple_file);
 if ( !blk )
 {
  printf( "Block is Null\n" );
 }
 printf( "Printing RAW NOHEAD Dataset cols ----------------------\n" );
 printCols( blk );

 dmTableClose( blk ); 

 blk = dmTableOpen( dtf_file );
 if ( !blk )
 {
  printf( "DTF Block is NULL\n" );
 }
 printf( "Printing DTF Cols ----------------------------------------\n" );
 printCols( blk );
 dmTableClose( blk );


 /* test all of the kernel interface functions */
 printf( "\n\n" );
 printf( "Testing ASCII kernel functions ---------------------------\n" );
 tval = dmDatasetAccess(simple_file, "R");
 printf( "dmDatasetAccess( %s, \"R\"): %d\n", simple_file, tval );
 ds = dmDatasetOpen( simple_file );
 tval = ds != NULL;
 printf( "dmDatasetOpen( %s ): %d\n", simple_file, tval );
 dmDatasetPrint( ds, "BLOCKS" );
 dmDatasetClose( ds );
 ds = dmDatasetOpen( sm_file );
 tval = ds != NULL;
 printf( "dmDatasetOpen( %s ): %d\n", sm_file, tval );
 dmDatasetPrint( ds, "BLOCKS" );
 dmDatasetClose( ds );
 ds = dmDatasetOpen( dtf_file );
 tval = ds != NULL;
 printf( "dmDatasetOpen( %s ): %d\n", dtf_file, tval );
 dmDatasetPrint( ds, "BLOCKS" );
 dmDatasetClose( ds );

 return 0;
}

void printCols( dmBlock* block )
{
 dmDescriptor** cols = NULL;

 long i;
 long ncols;
 char buf[CARD_SIZE];
 char buf2[CARD_SIZE];
 long namelen = 10;
 long typelen = 10;
 long nolen = 5;
 long len;
 long* namelens = NULL;
 long* nolens = NULL;
 long* typelens = NULL;
 char** colnames = NULL;
 char** typenames = NULL;
 
 if ( !block ) return;
 ncols = dmTableGetNoCols( block );
 if ( ncols > 0 ) 
 {
  cols = (dmDescriptor**)malloc( sizeof(dmDescriptor*)*ncols );
  namelens = (long*)malloc( sizeof(long)*ncols );
  nolens = (long*)malloc( sizeof(long)*ncols );
  colnames = (char**)malloc( sizeof(char*)*ncols );
  typenames = (char**)malloc( sizeof(char*)*ncols );
  typelens = (long*)malloc( sizeof(long)*ncols );
  typelen = 0;
  nolen = 0;
  namelen = 0;
 }
 for ( i = 0; i < ncols; i++ )
 {
  cols[i] = dmTableOpenColumnNo( block, i+1 );
  getCompositeName( cols[i], buf, CARD_SIZE );
  colnames[i] = cs_dup( buf );
  namelens[i] = strlen(buf);
  typenames[i] = dmGetDataTypeName( cols[i] );
  /* for array and bit columns... */
  len = dmDescriptorGetLength( cols[i] );
  if ( len > 0 )
  {
   strcpy( buf, typenames[i] );
   free( typenames[i] );
   sprintf( buf2, "%s[%ld]", buf, len );
   typenames[i] = cs_dup( buf2 );
  }
  typelens[i] = strlen( typenames[i] );
  sprintf( buf, "%ld", i+1 );
  nolens[i] = strlen(buf);
 }

 for ( i = 0; i < ncols; i++ )
 {
  if ( nolens[i] > nolen ) nolen = nolens[i];
  if ( namelens[i] > namelen ) namelen = namelens[i];
  if ( typelens[i] > typelen ) typelen = typelens[i];
 }

 nolen +=5;
 namelen += 2;
 typelen += 2;

 printf( "# %-*s%-*s%-*s\n", (int)nolen, "ColNo", (int)namelen, "Name", (int)typelen, "Type" );
 for ( i = 0; i < ncols; i++ )
 {
  printf( "  %-*ld%-*s%-*s\n", (int)nolen, i+1, (int)namelen, colnames[i], (int)typelen, typenames[i] );
 }

 for ( i = 0; i < ncols; i++ )
 {
  free( colnames[i] );
  free( typenames[i] );
 }
 if ( colnames ) free( colnames );
 if ( typenames ) free( typenames );
 if ( cols ) free( cols );
 if ( namelens ) free( namelens );
 if ( nolens ) free( nolens );
}

/* unfinished */
#if 0
void printHeader( dmBlock* block )
{
 char name[CARD_SIZE];
 char buf[CARD_SIZE];
 char unit[CARD_SIZE];
 long nkeys;
 long keyno;

 dmDescriptor* key;
 dmDataType type;

 nkeys = dmBlockGetNoKeys( block );
 dmBlockMoveToKeyNo( block, 0 );
 
 for ( keyno = 1; keyno <= nkeys; keyno++ )
 {
  key = dmBlockGetKey( block, keyno );
  if ( key ) 
  {
   type = dmGetDataType( key );
   getCompositeName( key, name, CARD_SIZE );
   dmGetUnit( key, unit, CARD_SIZE );
  }
 }
}
#endif

void getCompositeName( dmDescriptor* dd, char* val, long vlen )
{
 char buf[CARD_SIZE];
 char buf2[CARD_SIZE];
 long edim;
 long naxes;
 long* axes;
 long j;

 dmDescriptor* cpt_dd;

 dmGetName( dd, buf, CARD_SIZE );
 edim = dmGetElementDim( dd );
 naxes = dmGetArrayDimensions( dd, &axes );
 if ( edim > 1 )
 {
  strcat( buf, "(" );
  for ( j = 1; j <= edim; j++ )
  {
   cpt_dd = dmGetCpt( dd, j );
   dmGetName( cpt_dd, buf2, CARD_SIZE );
   if ( j > 1 )
   {
    strcat( buf, "," );
   }
   strcat( buf, buf2 );
  }
  strcat( buf, ")" );
 }
 if ( naxes > 0 )
 {
  for ( j = 0; j < naxes; j++ )
  {
   sprintf( buf2, "[%ld]", axes[j] );
   strcat( buf, buf2 );
  }
  free( axes ); 
 }
 cs_copy_siz( buf, val, vlen );
}

/* unfinished */
#if 0
long getScalarValueString( dmDescriptor* dd, long cptno, long cellno, char* val, long vlen )
{
 double tmp;
 char buf[CARD_SIZE];
 char* cval = (char*)&tmp;
 long *lval = (long*)&tmp;
 unsigned long* ulval = (unsigned long*)&tmp;
 short* sval = (short*)&tmp;
 unsigned short* usval = (unsigned short*)&tmp;
 dmBool* bval = (dmBool*)&tmp;
 long dim;
 
 dmDataType type;

 type = dmGetDataType( dd );
 dim = dmGetElementDim( dd );
 dmGetVectorArray( dd, type, &tmp, dim, cellno, 1 );
 switch (type)
 {
 case dmSHORT:
  break;
 case dmUSHORT:
  break;
 case dmLONG:
  break;
 case dmULONG:
  break;
 case dmBIT:
  break;
 case dmTEXT:
  break;
 case dmDOUBLE:
  break;
 case dmFLOAT:
  break;
 case dmBOOL:
  break;
 default:
  break;
 }
}
#endif


