/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "dmtestlib.h"


void dmTablePrintRow( dmBlock* block, void* row )
{
 long ncols;
 long i;
 dmDescriptor** cols;
 dmDescriptor* dd;
 dmDescriptor* cpt_dd;
 void* ptr;
 long dim;
 long j;
 long offset;
 
 
 cols = dmTableOpenColumnList( block, &ncols );
 for ( i = 0, offset=0; i < ncols; i++ )
 {
  dd = cols[i];
  dim = dmGetElementDim( dd );
  for ( j = 0; j < dim; j++, offset++ )
  {
   cpt_dd = dmGetCpt( dd, j+1 );
   ptr = dmColumnGetRowPointer( cpt_dd, row );
   /*   ptr = dmTableGetColPtr( block, row, offset+1 ); */
   dmDescriptorPrintValue( cpt_dd, ptr );
   printf( "     " );
  }
 }
 printf( "\n" );
}

long dmGetDataTypeSize( dmDataType type )
{
 return ar_mix_type_size( type );
}

long dmGetSize( dmDescriptor* dd )
{
 long asize = dmGetArraySize( dd );
 long dim = dmGetElementDim( dd );
 long typesize = ar_mix_type_size( dmGetDataType(dd) );
 return asize * dim * typesize;
}

void dmDescriptorPrintValue( dmDescriptor* dd, void* data )
{
 long ncpts;
 long i;
 long size;
 dmDescriptor* cpt_dd;
 void* ptr = data;
 ncpts = dmGetElementDim( dd );
 for ( i = 0 ; i < ncpts; i++ )
 {
  cpt_dd = dmGetCpt( dd, i+1 );
  dmDescriptorPrintScalarValue( cpt_dd, ptr );
  printf(" ");
  size = dmGetSize( cpt_dd );
  ptr = (char*)ptr + size;
 }
}

void dmDescriptorPrintScalarValue( dmDescriptor* dd, void* data )
{
 dmDataType type;
 long len;
 char *byte_string;
 type = dmGetDataType( dd );
 switch(type)
 {
 case dmFLOAT:
  printf( "%.6f", *(float*)data );
  break;
 case dmDOUBLE:
  printf( "%.6f", *(double*)data );
  break;
 case dmLONG:
  printf( "%ld", *(long*)data );
  break;
 case dmSHORT:
  printf( "%d", *(short*)data );
  break;
 case dmULONG:
  printf( "%lu", *(unsigned long*)data );
  break;
 case dmUSHORT:
  printf( "%d", *(unsigned short*)data );
  break;
 case dmBYTE:
  printf( "%c", *(unsigned char*)data );
  break;
 case dmBOOL:
  if ( *(int*)data )
   printf( "TRUE" );
  else
   printf( "FALSE" );
  break;
 case dmBIT:
  len = dmDescriptorGetLength( dd );
  byte_string = calloc( len+1, sizeof(char) );
  dmu_map_bits( (unsigned char*)data, len, byte_string );
  printf( byte_string );
  free( byte_string );
  break;
 default:
  break;
 }
}

void dmFreeArray( dmDescriptor* dd, void* data, long numRows )
{
 long ii;
 dmDataType type;
 

 type = dmGetDataType( dd );
 if ( type == DT_CHAR )
 {
  for ( ii=0; ii < numRows; ii++ )
   free( ((char**)data)[ii] );
  free( data );
 }
 else
  free( data );
}

long dmGetScalars( dmDescriptor* dd, void* data, dmDataType type, long firstRow, long nrows )
{
 long maxlen;
 switch( type )
 {
 case DT_INT2:
  return dmGetScalars_s( dd, data, firstRow, nrows );

 case DT_INT4:
  return dmGetScalars_l( dd, data, firstRow, nrows );

 case DT_REAL4:
  return dmGetScalars_f( dd, data, firstRow, nrows );

 case DT_REAL8:
  return dmGetScalars_d( dd, data, firstRow, nrows );

 case DT_LOGICAL:
  return dmGetScalars_q( dd, data, firstRow, nrows );

 case DT_CHAR:
  maxlen = dmDescriptorGetLength( dd );
  return dmGetScalars_c( dd, data, maxlen, firstRow, nrows );

 case DT_UINT1:
  return dmGetScalars_ub( dd, data, firstRow, nrows );

 case DT_UINT2:
  return dmGetScalars_us( dd, data, firstRow, nrows );

 case DT_UINT4:
  return dmGetScalars_ul( dd, data, firstRow, nrows );

 case DT_BIT:
  return dmGetScalars_bit( dd, data, firstRow, nrows );

 default:
  return 0;
 }
}

void* dmAllocArray( dmDescriptor* dd, long nvals )
{
 void* data = NULL;
 dmDataType type;
 long len;
 long ii;
 long size;

 type = dmGetDataType( dd );
 if ( type == DT_CHAR )
 {
  len = dmDescriptorGetLength( dd );
  data = malloc( nvals * sizeof(char*) );
  for ( ii=0; ii < nvals; ii++ )
   ((char**)data)[ii] = calloc( len, sizeof(char) );
 }
 else
 {
  size = dmGetSize( dd );
  data = calloc( size*nvals, sizeof(char) );
 }
 return data;
}

