/*                                                                
**  Copyright (C) 2004,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "ascdm.h"
#include <stdlib.h>
#include <stdio.h>

extern char* cs_dup( const char* );

int main( int argc, char* argv[] )
{
 dmBlock* block;
 dmBlock* block2;
 dmDescriptor* dd;
 dmDescriptor** new_dds;
 dmDescriptor** new_dds2;

 dmErrCode status;

 char *dtf_file;
 char **vnames;

 char *new_colnames[] = { "time", "X", "Y" };
 char *units[] = { "s", "mm", "mm" };
 char *descs[] = { "time col", "x col", "y col" };
 dmDataType new_types[] = { DT_REAL8, DT_INT4, DT_INT4 };
 long slens[] = { 0, 0, 0 };
 
 
 long new_ncols = 3;
 long ii;

 if ( argc < 2 )
 {
  printf( "dtftest: args are not correct.\n" );
  exit ( EXIT_FAILURE );
 }
 dtf_file = argv[1];

 /* now try writing to the files */
 block = dmTableOpenUpdate( dtf_file );
 if ( !block )
 {
  printf( "DTF Block is NULL\n" );
 }

 dd = dmTableOpenColumn( block, "TIME" );
 dmTableSetRow( block, 4 );
 dmSetScalar_d( dd, 10.135 );
 dmTableNextRow( block );
 dmSetScalar_d( dd, 19.789102 );
 dd = dmKeyOpen( block, "FAST" );
 if ( !dd )
 {
  printf( "Key FAST is NULL\n" );
 }
 status = dmDescriptorDelete( dd );
 if ( status != dmSUCCESS )
 {
  printf( "Failed to delete key FAST\n" );
 }
 dd = dmKeyWrite_d( block, "RYAND", 20000.0005, "s", "Ryan's test key value" );
 if ( dd == NULL )
 {
  printf( "Failed to write key RYAND\n" );
 }
 dd = dmKeyWrite_d( block, "RYAND", 10000.1111, "km", "Ryan's second test key value" );
 if ( dd == NULL )
 {
  printf( "Failed to write key RYAND the second time\n" );
 }
 dd= dmKeyWrite_c( block, "RYANS", "BLAH BLAH BLAH", "foo", "Ryan's string keyword value" );
 if ( dd == NULL )
 {
  printf( "Failed to write a string valued keyword.\n" );
 }
 dd = dmColumnCreate( block, "NEW_COL", DT_REAL8, 0, "s", "My new column" );
 if ( dd == NULL )
 {
  printf( "Failed to create new column: NEW_COL.\n" );
 }
 vnames = calloc( 2, sizeof(char*) );
 vnames[0] = cs_dup( "NEWVEC1" );
 vnames[1] = cs_dup( "NEWVEC2" );
 dd = dmColumnCreateVector( block, "NEW_VECTOR", DT_REAL8, 0, NULL, NULL, vnames, 2 );
 free( vnames[0] );
 free( vnames[1] );
 free( vnames );
 if ( !dd )
 {
  printf( "Failed to create new vector column: NEW_VECTOR.\n" );
 }
 dd = dmColumnCreateArray( block, "NEW_ARRAY", DT_REAL8, 0, NULL, NULL, 5 );
 if ( !dd )
 {
  printf( "Failed to create new array column: NEW_ARRAY.\n" );
 }
 status = dmBlockWriteComment( block, dmCOMMENT, "This is ryan's first comment." );
 if ( status != dmSUCCESS )
 {
  printf( "Write comment failed!.\n" );
 }
 status = dmBlockWriteComment( block, dmCOMMENT, "Here's another!" );
 if ( status != dmSUCCESS )
 {
  printf( "Write comment 2 failed!.\n" );
 }
 status = dmBlockWriteComment( block, dmHISTORY, "How 'bout a little HISTORY?" );
 if ( status != dmSUCCESS )
 {
  printf( "Write history failed!.\n" );
 }
 dmTableClose( block );

 dmKernelSetCreate( "text" );
 dmKernelSetOption( "dtf" );
 block = dmDatasetCreateTable( NULL, "/tmp/RYANTEST" );
 if ( !block )
 {
  printf( "Couldn't create new table.\n" );
 }
 new_dds = dmTableCreateColumns( block, new_colnames, new_types, slens, units, descs, new_ncols );
 if ( !new_dds )
 {
  printf( "Couldn't create new columns.\n" );
 }
 for ( ii=0; ii <= 10; ii++ )
 {
  dmSetScalar_d( new_dds[0], 1234.567 + ii );
  dmSetScalar_l( new_dds[1], 1234567 + ii );
  dmSetScalar_l( new_dds[2], 456789 + ii );
  dmTablePutRow( block, NULL );
 }

 block2 = dmDatasetCreateTable( NULL, "/tmp/ANOTHER" );
 if ( !block2 )
 {
  printf( "Couldn't create second block!\n" );
 }
 new_dds2 = dmTableCreateColumns( block2, new_colnames, new_types, slens, units, descs, new_ncols );
 if ( !new_dds2 )
 {
  printf( "Couldn't create new columns in second block.\n" );
 }
 for ( ii=0; ii <= 10; ii++ )
 {
  dmSetScalar_d( new_dds2[0], 4321.567 - ii );
  dmSetScalar_l( new_dds2[1], 7654321 - ii );
  dmSetScalar_l( new_dds2[2], 987654 - ii );
  dmTablePutRow( block2, NULL );
 } 

 for ( ii=0; ii <= 10; ii++ )
 {
  dmSetScalar_d( new_dds[0], 1234.567 + ii );
  dmSetScalar_l( new_dds[1], 1234567 + ii );
  dmSetScalar_l( new_dds[2], 456789 + ii );
  dmTablePutRow( block, NULL );
 }

 for ( ii=0; ii <= 10; ii++ )
 {
  dmSetScalar_d( new_dds2[0], 4321.567 - ii );
  dmSetScalar_l( new_dds2[1], 7654321 - ii );
  dmSetScalar_l( new_dds2[2], 987654 - ii );
  dmTablePutRow( block2, NULL );
 } 
 
  dmDatasetClose( dmBlockGetDataset( block ) );
  dmDatasetClose( dmBlockGetDataset( block2 ) );
 /*
  dmDatasetDelete( dmBlockGetDataset( block ) );
  dmDatasetDelete( dmBlockGetDataset( block2 ) );
 */
 return 0;
}
