/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdio.h>
#include "dmtestlib.h"
#include "dmulib.h"

/* This program is for testing dmGetScalars functionality.
 * since dmGetScalars isn't use by any major DM tools, but
 * dmGetScalars operates very differently on vector columns
 * than dmGetVectors().
 */

int main( int argc, char* argv[] )
{
 char name[1024];
 char* filename;
 dmBlock* table;
 long numRows;
 long numCols;
 dmDescriptor* dd;
 dmDescriptor* cpt_dd;
 long jj;
 void* data;
 void* ptr;
 dmDataType type;
 long dim;
 long size;
 long cptno;
 long colno;


 if ( argc < 2 )
 {
  printf( "USAGE:\n\tgetscalars <infile>\n" );
  exit (0);
 }

 filename = cs_dup( argv[1] );
 table = dmTableOpen( filename );
 if ( !table )
 {
  printf( "ERROR: Could not open file %s\n", filename );
 }
 numRows = dmTableGetNoRows( table );
 numCols = dmTableGetNoCols( table );
 
 for ( colno=1; colno <= numCols; colno++ )
 {
  dd = dmTableOpenColumnNo( table, colno );
  dim = dmGetElementDim( dd );
  for ( cptno = 1; cptno <= dim; cptno++ )
  {
   cpt_dd = dmGetCpt( dd, cptno );
   type = dmGetDataType( cpt_dd );
   data = dmAllocArray( cpt_dd, numRows );
   dmGetScalars( cpt_dd, data, type, 1, numRows );

   dmGetName( cpt_dd, name, 1024 );
   printf( "Col Name: %s\n", name );
   size = dmGetSize( cpt_dd );
   ptr = data;
   for ( jj=0; jj < numRows; jj++ )
   {
    dmDescriptorPrintValue( cpt_dd, ptr );
    printf( "\n" );
    ptr = (char*)ptr + size;
   }
   dmFreeArray( cpt_dd, data, numRows );
   printf( "\n" );
  }
 }

 return 0;
}


