/*                                                                
**  Copyright (C) 2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdio.h>
#include "dmtestlib.h"
#include "dmulib.h"

char* cs_dup( const char* str );
extern void dmu_map_bits( unsigned char* buf, long n, char* bits );


int main( int argc, char* argv[] )
{
  void* row      = NULL;
  char* filename = NULL;
  char* spec     = NULL;

  dmBlock* table      = NULL;
  dmRowFilter* filter = NULL;

  int npass;
  char errbuf[1024];

  if ( argc < 3 )
  {
    printf( "USAGE:\n\trowfilter <infile> <vspec>\n" );
    exit (0);
  }

  filename = cs_dup( argv[1] );
  spec = cs_dup( argv[2] );

  table = dmTableOpen( filename );
  if ( !table )
  {
    printf( "ERROR: Could not open file %s\n", filename );
    exit(1);
  }

  row = dmTableAllocRow( table );
  filter = dmTableAllocRowFilter( table, spec );
  if ( !filter )
  {
    printf( "ERROR: Could not allocate row filter.\n" );
    if ( dmGetError() != dmSUCCESS )
    {
      dmGetErrorMessage( errbuf, 1024 );
      printf( "%s\n", errbuf );
    }
    exit (1);
  }

  npass = 0;
  while( dmTableGetRow( table, row ) != dmNOMOREROWS )
  {
    if ( dmTableFilterRow( table, filter, row ))
    {
      printf( "PASSED\n" );
      npass++;
    }
    else
      printf( "FILTERED\n" );

    dmTablePrintRow( table, row );
  }
 
  printf( "TOTAL PASSED: %d\n", npass );
  free( row );
  dmRowFilterFree( filter );
 
  free( filename );
  free( spec );

  return 0;
}
