/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "ascdm.h"
#include <stdlib.h>
#include <stdio.h>

extern char* cs_dup( const char* );

int main( int argc, char* argv[] )
{
 dmBlock* block;
 dmDescriptor* dd;
 double vals1[] = { 11.385, 12.385 };
 double vals2[] = { 13.385, 14.385 };
 dmErrCode status;
 
 char *sm_file;
 char** vnames;
 if ( argc < 2 )
 {
  printf( "smtest: args are not correct.\n" );
  exit ( EXIT_FAILURE );
 }
 sm_file = argv[1];
 

 block = dmTableOpenUpdate( sm_file );
 if ( !block )
 {
  printf( "Failed to open table %s.\n", sm_file );
 }
 dd = dmTableOpenColumn( block, "POS" );
 dmTableSetRow( block, 1 );
 dmSetVector_d( dd, vals1, 2 );
 dmTableNextRow( block );
 dmSetVector_d( dd, vals2, 2 );
 dmColumnCreate( block, "NEW_COL", DT_REAL8, 0, "s", "My new column" );
 if ( dd == NULL )
  printf( "Failed to create new column: NEW_COL.\n" );
 vnames = calloc( 2, sizeof(char*) );
 vnames[0] = cs_dup( "NEWVEC1" );
 vnames[1] = cs_dup( "NEWVEC2" );
 dd = dmColumnCreateVector( block, "NEW_VECTOR", DT_REAL8, 0, NULL, NULL, vnames, 2 );
 free( vnames[0] );
 free( vnames[1] );
 free( vnames );
 if ( !dd )
 {
  printf( "Failed to create new vector column: NEW_VECTOR.\n" );
 }
 dd = dmColumnCreateArray( block, "NEW_ARRAY", DT_REAL8, 0, NULL, NULL, 5 );
 if ( !dd )
 {
  printf( "Failed to create new array column: NEW_ARRAY.\n" );
 }
 status = dmBlockWriteComment( block, dmCOMMENT, "This is ryan's first comment." );
 if ( status != dmSUCCESS )
 {
  printf( "Write comment failed!.\n" );
 }
 status = dmBlockWriteComment( block, dmCOMMENT, "Here's another!" );
 if ( status != dmSUCCESS )
 {
  printf( "Write comment 2 failed!.\n" );
 }
 status = dmBlockWriteComment( block, dmHISTORY, "How 'bout a little HISTORY?" );
 if ( status != dmSUCCESS )
 {
  printf( "Write history failed!.\n" );
 }
 dmTableClose( block );

 return 0;
}
