$RM = find_inpath( 'rm', $ENV{'PATH'}, ':' );
# 
#  Copyright (C) 2004,2007  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

$DIFF = find_inpath( 'diff', $ENV{'PATH'}, ':' );
$CP = find_inpath( 'cp', $ENV{'PATH'}, ':' );
$CAT = find_inpath( 'cat', $ENV{'PATH'}, ':' );
$DMDIFF = find_inpath( 'dmdiff', $ENV{'PATH'}, ':' );
$DMLIST = find_inpath( 'dmlist', $ENV{'PATH'}, ':' );
$DMIMGCALC = find_inpath( 'dmimgcalc', $ENV{'PATH'}, ':' );
$CAT = find_inpath( 'cat', $ENV{'PATH'}, ':' );
$SED = find_inpath( 'sed', $ENV{'PATH'}, ':' );
$GREP = find_inpath( 'grep', $ENV{'PATH'}, ':' );

use strict;
use vars '$DIFF', '$CP', '$CAT', '$DMDIFF', '$DMLIST', '$DMIMGCALC',
'$RM', '$CAT', '$SED', '$GREP';

#------------------------------------------------------------
#  Function:  parse_args
#
#  Description:
#    Create an associative array of arguments and their
#    associated values (if any)
#
#------------------------------------------------------------
sub parse_args {
   my ( $argsref ) = @_;
   my @arglist = @$argsref;
   my %arghsh;
   my ($ii, $jj);
   
   if ( scalar @arglist >= 0 )
   {
      for ( $ii = 0; $ii <= $#arglist; $ii++ )
      {
	 if ( $arglist[$ii] =~ /^-/ )
	 {
	    # A new argument tag
	    my $tag = $arglist[$ii];
	    my $argval = "";
	    
	    # Check for associated values
	    for ($jj = $ii + 1; $jj <= $#arglist;  $jj++)
	    {
	       # Check for next argument tag
	       if ( $arglist[$jj] =~ /^-/ ){ last; }
	       
	       if ( $jj == $ii+1 )
	       {
		  $argval = $arglist[$jj];
	       }
	       else
	       {
		  $argval = " $arglist[$jj]";
	       }
	    }
	    $ii = $jj - 1;
	    
	    $arghsh{"$tag"} = $argval;
	 } # end if New Arg. Tag
      } # end argument loop
   }
   return(%arghsh);
}


sub find_inpath {
   my ($file, $pathStr, $delim);
   if(scalar @_ == 2) {
      ($file, $pathStr) = @_;
      $delim = ' ';
   } else {
      ($file, $pathStr, $delim) = @_;
   }
   
   my @pathArr = split(/$delim/, $pathStr);
   foreach my $path (@pathArr) {
      if(-e "$path/$file") {
         return "$path/$file";
      }
   }
   return "";
}

######################################################################
# Subroutine 
# dmdiff_test OUTDIR SAVDIR testid block logfile diff_file
# Perform the dmdiff based test. 
# $1: OUTDIR
# $2: SAVDIR
# $3: testid
# $4: block
# $5: logfile
# $6: diff_file
# Returns 0 -> Failure, 1 -> Pass
sub dmdiff_test
{
   my ($outfile, $savfile, $block, $tolfile, $logfile, $diff_file) = @_;
   if ( ! -e $outfile )
   {
      return "ERROR: outfile does not exist: $outfile\n";
   }
   if ( ! -e $savfile )
   {
      return "ERROR: savfile does not exist: $savfile\n";
   }
   my $comstr = "$DMDIFF \"${outfile}\[$block\]\" \"${savfile}\[$block\]\" tol=$tolfile verbose=1 > $diff_file 2>>$logfile";
   my $ec = system($comstr);
   my $msg = "";
   if ( $ec == 0 )
   {
      return "";
   }
   else 
   {
      return "ERROR: MISMATCH in $outfile\[$block\]\n";
   }
}

######################################################################
# Subroutine 
# dmimgcalc_test OUTDIR SAVDIR testid logfile diff_file
# Perform the dmimgcalc based test.  This test only compares on block.
# $1: OUTDIR
# $2: SAVDIR
# $3: testid
# $4: block
# $5: logfile
# $6: diff_file
# Returns 0 -> Failure, 1 -> Pass
sub dmimgcalc_test
{
    my ($outfile, $savfile, $block, $tolfile, $logfile, $diff_file) = @_;
    my $err_msg = "";
    if ( ! -e $outfile )
    {
       $err_msg .= "ERROR: outfile does not exist: $outfile\n";
       return $err_msg;
    }
    if ( ! -e $savfile )
    {
       $err_msg .= "ERROR: savfile does not exist: $savfile\n";
       return $err_msg;
    }

    my $comstr = "$DMDIFF \"$outfile\[$block\]\" \"$savfile\[$block\]\" tol=$tolfile verbose=1 > $diff_file 2>>$logfile";
    my $ec = system($comstr);
    unless ( $ec == 0 )
    {
       $err_msg .= "ERROR: IMAGE HEADER MISMATCH in $outfile\[$block\]\n";
    }
    $comstr = "$DMIMGCALC \"$outfile\[$block\]\" \"$savfile\[$block\]\" none tst verbose=0 2>>$logfile";
    $ec = system($comstr);
    unless ( $ec == 0 )
    {
       $err_msg .= "ERROR: IMAGE DATA MISMATCH in $outfile\[$block\]\n";
    }
    return $err_msg;
}

######################################################################
# Subroutine 
# dmlist_test OUTDIR SAVDIR testid logfile diff_file extra
# Perform the dmlist based test.  This test works accross all blocks.
# $1: OUTDIR
# $2: SAVDIR
# $3: testid
# $4: logfile
# $5: diff_file
# $6: extra cmd line ie. "rows=blah cells=blah"
# Returns 0 -> Failure, 1 -> Pass
sub dmlist_test
{
   my ($outfile, $outfile_filt, $savfile, $savfile_filt, $tolkeysref, $logfile, $diff_file, $extra) = @_;
   my $msg = "";
   if ( ! -e $outfile )
   {
      return "ERROR: outfile does not exist: $outfile\n";
   }
   if ( ! -e $savfile )
   {
      return "ERROR: savfile does not exist: $savfile\n";
   }
   my $comstr = "$DMLIST infile=$outfile opt=data,cols,header,array,clean,raw,all outfile=${outfile_filt}_keytmp $extra >> $logfile 2>&1";
   system($comstr);
   $comstr = "$DMLIST infile=$savfile opt=data,cols,header,array,clean,raw,all outfile=${savfile_filt}_keytmp $extra >> $logfile 2>&1";
   system($comstr);
   keyfilter( "${outfile_filt}_keytmp", $outfile_filt, $tolkeysref );
   keyfilter( "${savfile_filt}_keytmp", $savfile_filt, $tolkeysref );
   system("$RM -f ${outfile_filt}_keytmp");
   system("$RM -f ${savfile_filt}_keytmp");
   $msg .= zerotest( $outfile_filt );
   $comstr = "$DIFF $outfile_filt $savfile_filt > $diff_file 2>>$logfile";
   my $ec = system($comstr);
   unless ( $ec == 0 )
   {
      $msg .= "ERROR: MISMATCH in $outfile\n";
   }

   return $msg;
}

######################################################################
# Subroutine 
# dmlist_header_test outfile outfile_filt savfile savfile_filt tolkeysref logfile diff_file
# Same as dmlist_test, except it only lists and compares the headers.
# Returns 0 -> Failure, 1 -> Pass
sub dmlist_header_test
{
   my ($outfile, $outfile_filt, $savfile, $savfile_filt, $tolkeysref, $logfile, $diff_file) = @_;
   my $msg = "";
   if ( ! -e $outfile )
   {
      return "ERROR: outfile does not exist: $outfile\n";
   }
   if ( ! -e $savfile )
   {
      return "ERROR: savfile does not exist: $savfile\n";
   }
   my $comstr = "$DMLIST infile=$outfile opt=header,clean,raw,all outfile=${outfile_filt}_keytmp >> $logfile 2>&1";
   system($comstr);
   $comstr = "$DMLIST infile=$savfile opt=header,clean,raw,all outfile=${savfile_filt}_keytmp >> $logfile 2>&1";
   system($comstr);
   keyfilter( "${outfile_filt}_keytmp", $outfile_filt, $tolkeysref );
   keyfilter( "${savfile_filt}_keytmp", $savfile_filt, $tolkeysref );
   system("$RM -f ${outfile_filt}_keytmp");
   system("$RM -f ${savfile_filt}_keytmp");
   $msg .= zerotest( $outfile_filt );
   $comstr = "$DIFF $outfile_filt $savfile_filt > $diff_file 2>>$logfile";
   my $ec = system($comstr);
   unless ( $ec == 0 )
   {
      $msg .= "ERROR: MISMATCH in $outfile\n";
   }

   return $msg;
}

sub diff_test
{
   my ($outfile, $outfile_filt, $savfile, $savfile_filt, $wordsref, $logfile, $diff_file) = @_;
   if ( ! -e $outfile )
   {
      return "ERROR: outfile does not exist: $outfile\n";
   }
   if ( ! -e $savfile )
   {
      return "ERROR: savfile does not exist: $savfile\n";
   }
   my $rc;
   my $msg;
   
   wordfilter( $outfile, $outfile_filt, $wordsref );
   wordfilter( $savfile, $savfile_filt, $wordsref );

   $msg .= zerotest( $outfile_filt );
   my $comstr = "$DIFF $outfile_filt $savfile_filt > $diff_file 2>> $logfile";
   $rc = system($comstr);
   unless ( $rc == 0 )
   {
      $msg .= "ERROR: MISMATCH in $outfile_filt\n";
   }
   return $msg;
}

sub wordfilter
{
   my ($infile, $outfile, $wordsref) = @_;
   my %words = %$wordsref;
   my @wordlist = keys %words;
   open INPUT, "<$infile" or return;
   open OUTPUT, ">$outfile" or return;
   my $line;
   my $newline;
   while ( $line = <INPUT> )
   {
      foreach my $word (@wordlist)
      {
	 $line =~ s/$word/$words{$word}/g;
      }
      print OUTPUT $line;
   }
   close OUTPUT;
}

sub keyfilter
{
   my ($infile, $outfile, $tolkeysref) = @_;

   my @tolkeys = @$tolkeysref;
   my $comstr = "";
   if ( scalar @tolkeys < 1 )
   {
      $comstr = "$CP $infile $outfile";
   }
   else
   {
      my $tolstr = "";
      foreach my $tolkey ( @tolkeys )
      {
	 $tolstr .= "s/${tolkey}/Dataset/g;";
      }
      $comstr = "$CAT $infile | $SED -e \'$tolstr\' | $GREP -v Dataset > $outfile";
   }
   system($comstr);
}

# parse a dmdiff tolerance file to find keywords to exclude in a test
sub parse_tolfile
{
   my ($tolfile) = @_;
   my @tolkeys = ( );

   open TOLERANCE, "<$tolfile" or return \@tolkeys;
   
   while( my $line = <TOLERANCE> )
   {
      if ( $line =~ /^\!/ )
      {
	 chop $line;
	 my $tolkey = substr($line, 1);
	 push( @tolkeys, $tolkey );
      }
   }
   return \@tolkeys;
}

sub zerotest
{
   my ( $filename ) = @_;
   my $msg;
   if ( -s $filename )
   {
      $msg = "";
   }
   else
   {
      $msg =  "ERROR; file $filename is of zero length\n";
   }
   return $msg;
}

# Print the minitest header
sub dt_print_mt_header
{
   my ( $id, $semantic, $logfile, $printfile, $catagory ) = @_;

   if ( $catagory eq "" ){ $catagory = "DataModel"; }

   my $date = `date +%d%b%Y_%H:%M:%S`;
   chop($date);
   print $logfile "

STARTING $catagory Test \#$id ( $semantic ) $date
-----------------------------------------------------------------------------

";
   print $printfile "

STARTING $catagory Test \#$id ( $semantic )
-----------------------------------------------------------------------------

";
}

# Print the minitest footer
sub dt_print_mt_footer
{
   my ( $num, $semantic, $status, $msg, $time_user, $time_sys, $logfile, $printfile, $catagory ) = @_;
   
   my $line = "";
   if ( $catagory eq "" ){ $catagory = "DataModel"; }

   print $logfile "FINISHED $catagory Test \#$num ( $semantic )\n";
   print $printfile "FINISHED $catagory Test \#$num ( $semantic )\n";
   $line = "$status";
   if ( $status == 0 )
   {
      $line = "PASSED";
   }
   else
   {
      $line = "FAILED";
   }
   if ( $msg ne "" )
   {
      $line .= " : $msg\n";
   }
   print $logfile "$line\n";
   print $printfile "$line\n";
   print $logfile 
"TOTAL CPU TIME: USER: $time_user s.  SYS: $time_sys s.
-----------------------------------------------------------------------------

";
   print $printfile 
"TOTAL CPU TIME: USER: $time_user s.  SYS: $time_sys s.
-----------------------------------------------------------------------------

";
}

# Print per-test header
sub dt_print_test_header
{
   my ( $num, $comstr, $logfile, $printfile, $kernel ) = @_;

   print $logfile "RUNNING $kernel TEST $num ...\n\n";
   print $printfile "RUNNING $kernel TEST $num ...\n\n";
   print $logfile "$comstr\n";
   print $printfile "$comstr\n";
}

# Print per-test footer
sub dt_print_test_footer
{
   my ( $num, $status, $err_msg, $time_user, $time_sys, $logfile, $printfile ) = @_;

   my $isok;
   if ( $status == 0 )
   {
      $isok = "OK";
   }
   else
   {
      $isok = "NOT-OK";
   }
   print $logfile "TEST $num: $isok\n";
   print $printfile "TEST $num: $isok\n";
   if ( $err_msg ne "" )
   {
      print $logfile "$err_msg\n";
      print $printfile "$err_msg\n";
   }
   print $logfile 
"CPU TIME:  USER: $time_user s.  SYS: $time_sys s.

";
   print $printfile 
"CPU TIME:  USER: $time_user s.  SYS: $time_sys s.

";
}

# Runs a command
# Returns:
#    1. the exit code of the command
#    2. the CPU user time taken by the exec'd process
#    3. the CPU system time taken by the exec'd process
sub dt_run_test
{
   use Benchmark;
   my ( $comstr ) = @_;
   my $command = sprintf "system( q($comstr) )";

   my $t = timeit( 1, $command );
   
   return ( $?, $t->[3], $t->[4] );
}

sub dt_print_web_tbl
{
   my ( $logfile, $outfile ) = @_;

   print $outfile "
<table border cellspacing=2 cellpadding=2>
";
   my $test_title;
   my $test_os;
   my $test_ascdsver;
   my $test_ct;
   my $test_numpassed;
   my $test_numfailed;
   my $test_totaluser;
   my $test_totalsys;
   my @test_logs;
   my @test_ids;
   my @test_semantics;
   my @test_status;
   my @test_timeuser;
   my @test_timesys;

   my $test_logsref;
   my $test_idsref;
   my $test_semanticsref;
   my $test_statusref;
   my $test_timeuserref;
   my $test_timesysref;

   my ($ar1ref, $ar2ref) = dt_parse_log_info( $logfile );
   if ( $ar1ref == undef || $ar2ref == undef ) 
   {
      return;
   }
   ( $test_title, $test_os, $test_ascdsver, $test_ct, $test_numpassed, $test_numfailed, $test_totaluser, $test_totalsys ) = @$ar1ref;
   ( $test_idsref, $test_logsref, $test_semanticsref, $test_statusref, $test_timeuserref, $test_timesysref ) = @$ar2ref;
   @test_logs = @$test_logsref;
   @test_ids = @$test_idsref;
   @test_semantics = @$test_semanticsref;
   @test_status = @$test_statusref;
   @test_timeuser = @$test_timeuserref;
   @test_timesys = @$test_timesysref;

   my $percent_success = ($test_numpassed / ( $test_numpassed + $test_numfailed )) * 100;
   $percent_success = sprintf("%2.2f", $percent_success);
   my $numtests = scalar @test_ids;
   print $outfile
"
<tr><td><strong>Title</strong></td><td colspan=2>$test_title</td></tr>
<tr><td><strong>OS</strong></td><td colspan=2>$test_os</td></tr>
<tr><td><strong>ASCDS Version</strong></td><td colspan=2>$test_ascdsver</td></tr>
<tr><td><strong>Creation Time</strong></td><td colspan=2>$test_ct</td></tr>
<tr><td><strong>Percent Success</strong></td><td colspan=2>$percent_success % ${test_numpassed}/${numtests}</td></tr>
<tr><td><strong>Test Log</strong></td><td colspan=2><a href=\"$logfile\">log</a></td></tr>
";
   for ( my $ii=0; $ii<=$#test_ids; $ii++ )
   {
      my $color;
      my $testlog;
      if ( $test_status[$ii] eq "PASSED" )
      {
	 $color = "#00cc00";
      }
      else
      {
	 $color = "#cc0000";
      }
      if ( -e $test_logs[$ii] )
      {
	 $testlog = "<a href=\"file://$test_logs[$ii]\">$test_ids[$ii] ($test_semantics[$ii])</a>";
      }
      else
      {
	 $testlog = "$test_ids[$ii] ($test_semantics[$ii])";
      }
	 print $outfile "
<tr>
<td>$testlog</td>
<td bgcolor=\"$color\"><strong>$test_status[$ii]</strong></td>
<td>$test_timeuser[$ii] : $test_timesys[$ii]</td>
</tr>
";
   }
   print $outfile "</table>\n";
}

sub dt_gen_web_rpt
{
   my ( $logfile, $outfile ) = @_;

   open HTMLOUT, ">$outfile" or return;
   print HTMLOUT 
"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">
<html><head>
<title>\"DataModel Regression Test Results\"</title>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">
</head><body BGCOLOR=\"#FFFFFF\">
<H2><STRONG>DataModel Regression Test Results</STRONG></H2>\n";
   dt_print_web_tbl( $logfile, *HTMLOUT );
   print HTMLOUT "
</body>
</html>
";
   close HTMLOUT;
}

sub dt_parse_log_info
{
   my ($logfile) = @_;

   open LOGIN, "<$logfile" or return ( undef, undef );
   
   my $test_title = "";
   my $test_os;
   my $test_ascdsver;
   my $test_ct;
   my $test_numpassed;
   my $test_numfailed;
   my $test_totaluser;
   my $test_totalsys;
   my @test_logs;
   my @test_ids;
   my @test_semantics;
   my @test_status;
   my @test_timeuser;
   my @test_timesys;

   my $line = <LOGIN>;
   ( $test_ct ) = ( $line =~ /(\d{2}[a-zA-Z]{3}\d{4}_\d{2}:\d{2}:\d{2})$/ );
   $line = <LOGIN>;
   while ($line && ($line !~ /ASCDS VERSION/))
   {
      $line = <LOGIN>;
   }
   ( $test_ascdsver ) = ( $line =~ /\s*ASCDS VERSION\s*:\s*(\S+.*)/ );
   while ( $line && $line !~ /PLATFORM/ )
   {
      $line = <LOGIN>;
   }
   ( $test_os ) = ( $line =~ /\s*PLATFORM\s*:\s*(\S+.*)/ );
   my $cur_id = 0;
   my $cur_semantic;
   my $cur_status;
   my $cur_log;
   my $cur_timeuser;
   my $cur_timesys;
   while ( $line = <LOGIN> )
   {
      if ( $line =~ /^Running test / )
      {
	 my $abnormal = 0;
	 ( $cur_id, $cur_semantic ) = ( $line =~ /^Running test (\d+) \((\w+)\)/ );
	 push( @test_ids, $cur_id );
	 push( @test_semantics, $cur_semantic );
	 while ( $line && $line !~ /Log at / )
	 {
	    $line = <LOGIN>;
	 }
	 ( $cur_log ) = ( $line =~ /Log at\s*(\S*)/ );
	 push( @test_logs, $cur_log );
	 while ( $line && $line !~ /-------*/ )
	 {
	    $line = <LOGIN>;
	    if ( $line =~ /^(PASSED)|(FAILED)/ )
	    {
	       $abnormal = 1;
	       last;
	    }
	 }
	 while ( $line && $line !~ /-------*/ && ! $abnormal)
	 {
	    $line = <LOGIN>;
	 }
	 while ( $line && $line !~ /(PASSED)|(FAILED)/ )
	 {
	    $line = <LOGIN>;
	 }
	 ( $cur_status ) = ( $line =~ /((PASSED)|(FAILED))/ );
	 push( @test_status, $cur_status );
	 while ( $line && $line !~ /CPU TIME:/ )
	 {
	    $line = <LOGIN>;
	 }
	 ( $cur_timeuser, $cur_timesys ) = ( $line =~ /CPU TIME:\s*USER:\s*(\S*)\s*s.\s*SYS:\s*(\S*)/ );
	 push( @test_timeuser, $cur_timeuser );
	 push( @test_timesys, $cur_timesys );	 
      }
      elsif( $line =~ /FINISHED DATAMODEL TESTS/ )
      {
	 $line = <LOGIN>;
	 ( $test_numpassed ) = ( $line =~ /PASSED:\s*(\d*)/ );
	 $line = <LOGIN>;
	 ( $test_numfailed ) = ( $line =~ /FAILED:\s*(\d*)/ );
	 $line = <LOGIN>;
	 ( $test_totaluser, $test_totalsys ) = ( $line =~ /^TOTAL CPU TIME:\s*USER:\s*(\S*) s.\s*SYS:\s*(\S*)/ );
      }
   }
   return ( [ $test_title, $test_os, $test_ascdsver, $test_ct, $test_numpassed, $test_numfailed, $test_totaluser, $test_totalsys ], [ \@test_ids, \@test_logs, \@test_semantics, \@test_status, \@test_timeuser, \@test_timesys ] );
}
1;
