/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdlib.h>
#include <stdio.h>
#include "ascdm.h"

#define MAX_LEN 256

void checkDMError( void );

int main(int argc, char **argv) {
   dmDataset* ds;

   dmBlock *image;
   dmDescriptor *data;
   long axes[2] = { 2, 2 };
   long naxes = 2;
   long ll[2] = { 1, 1 };
   long ur[2] = { 2, 2 };

   dmBlock *table;
   dmDescriptor **cols;
   char *col_names[] = { "USHORT", "ULONG", "FLOAT_SCALED", "INT_SCALED" };
   dmDataType col_types[4] = { dmUSHORT, dmULONG, dmSHORT, dmSHORT };
   long col_lens[4] = { 0, 0, 0, 0 };
   char *col_units[] = { "", "", "", "" };
   char *col_descs[] = { "", "", "", "" };

   unsigned short s_values[4] = { 1, 2, 3, 4 };
   unsigned long l_values[4] = { 1, 2, 3, 4 };
   short signed_s_values[4] = { 1, 2, 3, 4 };

   char *filename = argv[1];

   (void) argc;

   ds = dmDatasetCreate( filename );
   checkDMError();

   /*
    * Short image
    */

   image = dmDatasetCreateImage( ds, "USHORT_IMG", dmUSHORT, axes, naxes );
   checkDMError();

   data = dmImageGetDataDescriptor( image );
   checkDMError();

   dmImageDataSetSubArray_us( data, ll, ur, s_values );
   checkDMError();

   dmBlockClose( image );

   /*
    * Long image
    */

   image = dmDatasetCreateImage( ds, "ULONG_IMG", dmULONG, axes, naxes );
   checkDMError();

   data = dmImageGetDataDescriptor( image );
   checkDMError();

   dmImageDataSetSubArray_ul( data, ll, ur, l_values );
   checkDMError();

   dmBlockClose( image );

   /*
    * Float-scaled image
    */

   image = dmDatasetCreateImage( ds, "FLOAT_SCALED_IMG", dmSHORT, axes,
     naxes );
   checkDMError();

   data = dmImageGetDataDescriptor( image );
   checkDMError();

   dmImageDataSetSubArray_s( data, ll, ur, signed_s_values );
   checkDMError();

   dmKeyWrite_d( image, "BZERO", 2.0, "", "" );
   dmKeyWrite_d( image, "BSCALE", 3.0, "", "" );
   checkDMError();

   dmBlockClose( image );

   /*
    * Integer-scaled image
    */

   image = dmDatasetCreateImage( ds, "INT_SCALED_IMG", dmSHORT, axes, naxes );
   checkDMError();

   data = dmImageGetDataDescriptor( image );
   checkDMError();

   dmImageDataSetSubArray_s( data, ll, ur, signed_s_values );
   checkDMError();

   dmKeyWrite_l( image, "BZERO", 2, "", "" );
   dmKeyWrite_l( image, "BSCALE", 3, "", "" );
   checkDMError();

   dmBlockClose( image );

   /*
    * Table with unsigned and scaled columns
    */

   table = dmDatasetCreateTable( ds, "TABLE" );
   checkDMError();

   cols = dmTableCreateColumns( table, col_names, col_types, col_lens,
     col_units, col_descs, 4 );
   checkDMError();

   dmSetScalars_us( cols[0], s_values, 1, 4 );
   dmSetScalars_ul( cols[1], l_values, 1, 4 );
   dmSetScalars_s( cols[2], signed_s_values, 1, 4 );
   dmSetScalars_s( cols[3], signed_s_values, 1, 4 );
   checkDMError();

   dmKeyWrite_d( table, "TZERO3", 2.0, "", "" );
   dmKeyWrite_d( table, "TSCAL3", 3.0, "", "" );
   dmKeyWrite_l( table, "TZERO4", 2, "", "" );
   dmKeyWrite_l( table, "TSCAL4", 3, "", "" );
   checkDMError();

   dmBlockClose( table );

   dmDatasetClose( ds );

   return 0;
}

void checkDMError( void ) {
   char msg[MAX_LEN];

   if ( dmGetError() != dmSUCCESS ) {
      dmGetErrorMessage(msg,MAX_LEN);
      fprintf(stderr,"DM ERROR: %s\n",msg);
      exit( EXIT_FAILURE );
   }
}

