/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdio.h>
#include <stdlib.h>
#include "ascdm.h"


int main(int argc, char **argv)
{
   dmBlock* table;
   long nrows = 12;
   long i;

   dmDescriptor* scalar_dd;
   double scalar_data[4] = { 1000.1, 2000.2, 3000.3, 4000.4 };

   dmDescriptor* vector_dd;
   char* cpt_names[] = { "VCPT1", "VCPT2" };
   long vector_data[8] = { 10, 100, 20, 200, 30, 300, 40, 400 };

   dmDescriptor* array_dd;
   long adim = 4;
   long array_dim[4] = { 1, 2, 3, 4 };
   short array_data[4][4] = {
     { 11, 99, 99, 99 }, { 22, 23, 99, 99 },
     { 33, 34, 35, 99 }, { 44, 45, 46, 47 }
   };

   struct row_struct {
     double scalar;
     short array[4];
     long vector[2];
   } row_data;
   void* row_ptr = NULL;
   double* scalar_ptr = NULL;
   long* vector_ptr = NULL;
   short* array_ptr = NULL;

   char* table_name = argv[1];

   (void) argc;

   if ( dmFileExists( table_name ) )
      dmDatasetDestroy( table_name );

   table = dmTableCreate( table_name );
    
   scalar_dd = dmColumnCreate( table, "SCALAR", dmDOUBLE, 0, "x",
       "Scalar column" );
   array_dd = dmColumnCreateVarArray( table, "VARARRAY", dmSHORT, 0, "z",
       "Variable-length array column", adim );
   vector_dd = dmColumnCreateVector( table, "VECTOR", dmLONG, 0, "y",
       "Vector column", cpt_names, 2 );

   for ( i = 0; i < nrows; i++ )
   {
    dmSetScalar_d( scalar_dd, scalar_data[i%4] );
    dmSetArray_s( array_dd, array_data[i%4], array_dim[i%4] );
    dmSetVector_l( vector_dd, vector_data + 2*(i%4), 2 );
    dmTablePutRow( table, NULL );
   }

   dmTableSetRow( table, 7 );
   row_data.scalar = 777.77;
   row_data.array[0] = 71;
   row_data.array[1] = 72;
   row_data.array[2] = 73;
   row_data.array[3] = 74;
   row_data.vector[0] = 70000;
   row_data.vector[1] = 7000000;
   dmTablePutRow( table, &row_data );

   dmTableClose( table );


   table = dmTableOpen( table_name );
   row_ptr = dmTableAllocRow( table );

   dmTableSetRow( table, 6 );
   array_dd = dmTableOpenColumn( table, "VARARRAY" );
   dmGetArray_s( array_dd, array_data[0], 4 );
   printf( "\nIn row 6, VARARRAY contains:  %d %d %d %d\n", array_data[0][0],
       array_data[0][1], array_data[0][2],array_data[0][3] );
   printf( "dmGetCurrentArraySize() returns:  %ld\n",
       dmGetCurrentArraySize( array_dd ) );

   dmTableSetRow( table, 11 );
   dmTableGetRow( table, row_ptr );
   /* dmTableGetColPtr() is deprecated, but it allows us to exercise itself
    * plus dmColumnGetRowPointer() and dmColumnGetRowOffset() */
   scalar_ptr = (double*)dmTableGetColPtr( table, row_ptr, 1 );
   array_ptr  = (short*) dmTableGetColPtr( table, row_ptr, 2 );
   vector_ptr = (long*)  dmTableGetColPtr( table, row_ptr, 3 );
   printf( "\nRow 11 contains:  %g  [ %d, %d, %d, %d ]  ( %ld, %ld )\n\n",
       *scalar_ptr,
       *(array_ptr+0), *(array_ptr+1), *(array_ptr+2), *(array_ptr+3), 
       *(vector_ptr+0), *(vector_ptr+1) );

   free( row_ptr );
   dmTableClose( table );

   return 0;
}

