/*                                                                
**  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdio.h>
#include <stdlib.h>
#include "ascdm.h"

#define MAXLEN 1000
#define NROWS 10


int main(int argc, char **argv)
{
   dmBlock* table;
   dmDescriptor* eqpos;
   dmDescriptor* ra;
   dmDescriptor* dec;

   double ref_eqpos_dat[2*NROWS];
   double eqpos_dat[2*NROWS];
   double ra_dat[NROWS];
   double dec_dat[NROWS];

   char file_spec[MAXLEN];
   char* table_name = argv[1];

   long ii;

   (void) argc;

   /*
    * Opening virtual cols with no 'cols' filter
    */

   printf("\nNo 'cols' filter\n\n");

   table = dmTableOpen(table_name);
   eqpos = dmTableOpenColumn(table, "EQPOS");
   ra = dmTableOpenColumn(table, "RA");
   dec = dmTableOpenColumn(table, "DEC");

   /* Extract virtual data as vector, and as individal columns */
   dmGetVectors_d(eqpos, ref_eqpos_dat, 2L, 1L,  NROWS);
   dmGetScalars_d(ra, ra_dat, 1, NROWS);
   dmGetScalars_d(dec, dec_dat, 1, NROWS);

   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii] != ra_dat[ii])
	 printf("Mismatch in RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii], ra_dat[ii]);

      if (ref_eqpos_dat[2*ii+1] != dec_dat[ii])
	 printf("Mismatch in DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii+1], dec_dat[ii]);
   }

   dmBlockClose(table);

   /*
    * Opening virtual cols with 'cols' filter
    */

   printf("\n[cols EQPOS]\n\n");

   sprintf(file_spec, "%s[cols %s]", table_name, "EQPOS");
   table = dmTableOpen(file_spec);
   eqpos = dmTableOpenColumn(table, "EQPOS");
   ra = dmTableOpenColumn(table, "RA");
   dec = dmTableOpenColumn(table, "DEC");
   dmGetVectors_d(eqpos, eqpos_dat, 2L, 1L, NROWS);
   dmGetScalars_d(ra, ra_dat, 1, NROWS);
   dmGetScalars_d(dec, dec_dat, 1, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii] != eqpos_dat[2*ii])
	 printf("Mismatch in EQPOS/RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii], eqpos_dat[2*ii]);

      if (ref_eqpos_dat[2*ii+1] != eqpos_dat[2*ii+1])
	 printf("Mismatch in EQPOS/DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii+1], eqpos_dat[2*ii+1]);
   }
   for (ii = 0; ii < NROWS; ii++) {
      if (eqpos_dat[2*ii] != ra_dat[ii])
	 printf("Mismatch in RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   eqpos_dat[2*ii], ra_dat[ii]);

      if (eqpos_dat[2*ii+1] != dec_dat[ii])
	 printf("Mismatch in DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   eqpos_dat[2*ii+1], dec_dat[ii]);
   }
   dmBlockClose(table);

   printf("\n[cols RA]\n\n");

   sprintf(file_spec, "%s[cols %s]", table_name, "RA");
   table = dmTableOpen(file_spec);
   ra = dmTableOpenColumn(table, "RA");
   dmGetScalars_d(ra, ra_dat, 1, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii] != ra_dat[ii])
	 printf("Mismatch in RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii], ra_dat[ii]);
   }
   dmBlockClose(table);

   printf("\n[cols DEC]\n\n");

   sprintf(file_spec, "%s[cols %s]", table_name, "DEC");
   table = dmTableOpen(file_spec);
   dec = dmTableOpenColumn(table, "DEC");
   dmGetScalars_d(dec, dec_dat, 1, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii+1] != dec_dat[ii])
	 printf("Mismatch in DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii+1], dec_dat[ii]);
   }
   dmBlockClose(table);

   /*
    * Opening virtual cols with range filter
    */

   printf("\n[ra>233,dec<24]\n\n");

   sprintf(file_spec, "%s[%s]", table_name, "ra>233,dec<24");
   table = dmTableOpen(file_spec);
   eqpos = dmTableOpenColumn(table, "EQPOS");
   ra = dmTableOpenColumn(table, "RA");
   dec = dmTableOpenColumn(table, "DEC");
   dmGetVectors_d(eqpos, eqpos_dat, 2L, 1L, NROWS);
   dmGetScalars_d(ra, ra_dat, 1, NROWS);
   dmGetScalars_d(dec, dec_dat, 1, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii] != eqpos_dat[2*ii])
	 printf("Mismatch in EQPOS/RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii], eqpos_dat[2*ii]);

      if (ref_eqpos_dat[2*ii+1] != eqpos_dat[2*ii+1])
	 printf("Mismatch in EQPOS/DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii+1], eqpos_dat[2*ii+1]);
   }
   for (ii = 0; ii < NROWS; ii++) {
      if (eqpos_dat[2*ii] != ra_dat[ii])
	 printf("Mismatch in RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   eqpos_dat[2*ii], ra_dat[ii]);

      if (eqpos_dat[2*ii+1] != dec_dat[ii])
	 printf("Mismatch in DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   eqpos_dat[2*ii+1], dec_dat[ii]);
   }
   dmBlockClose(table);

   printf("\n[ra>233]\n\n");

   sprintf(file_spec, "%s[%s]", table_name, "ra>233");
   table = dmTableOpen(file_spec);
   ra = dmTableOpenColumn(table, "RA");
   dmGetScalars_d(ra, ra_dat, 1L, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii] != ra_dat[ii])
	 printf("Mismatch in RA,  row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii], ra_dat[ii]);
   }
   dmBlockClose(table);

   printf("\n[dec<24]\n\n");

   sprintf(file_spec, "%s[%s]", table_name, "dec<24");
   table = dmTableOpen(file_spec);
   dec = dmTableOpenColumn(table, "DEC");
   dmGetScalars_d(dec, dec_dat, 1L, NROWS);
   for (ii = 0; ii < NROWS; ii++) {
      if (ref_eqpos_dat[2*ii+1] != dec_dat[ii])
	 printf("Mismatch in DEC, row %2ld: %20.10f  %20.10f\n", ii+1,
	   ref_eqpos_dat[2*ii+1], dec_dat[ii]);
   }
   dmBlockClose(table);

   return 0;
}

