#! /bin/sh

TESTID=dm-smoke001
export TESTID

INFILE='acisf04425_000N003_evt1.fits'
OUTFILE="${INFILE}"
OUTLIST="${OUTFILE}"
TOLFILE="dm-smoke.tol"

#
# Synopsis: Straight copy
TESTCMD='dmtcopy infile=$TESTDIR/$INFILE outfile=$OUTDIR/$OUTFILE clob+'

# Setup CIAO environment
#   (replace with variable via configure)
#
ASCDS_QUIET=1; export ASCDS_QUIET
#ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE
. setup.sh
. ${ASCDS_INSTALL}/bin/ciao_setup.sh


# check for tool existence before running
TESTBIN="$ASCDS_INSTALL/test/bin/datamodel"
TOOLCOPY="$TESTBIN/dmtcopy"

if test ! -s $TOOLCOPY
then
  echo ""
  echo "FAIL: One or more datamodel tools is missing. Cannot run $TESTID"
  exit 1
fi
 
#
# Setup test env
#
PATH=$TESTBIN:$PATH; export PATH
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
SAVEDIR=$TESTDIR; export SAVEDIR

INDIR=$TESTDIR

#
# --!!  check $ASCDS_WORK_PATH exist, writeable, has enough space
#
# Setup local parameter location
PFILES="$OUTDIR/param;$ASCDS_INSTALL/param"; export PFILES

if test -d $OUTDIR
then
  \rm -rf $OUTDIR
fi

mkdir -p $OUTDIR/param

#
# Run command
#
printf "Running test $TESTID"
printf " ."


(eval "$TESTCMD") # > /dev/null 2>&1
if test $? != 0
then
  echo "Problems running command for test: $TESTID"
  exit 1
fi

#
# Run checks on the output files.
#
state=0
ii=1
for f in $OUTLIST
do
  dmtcopy infile=$OUTDIR/${f} outfile=$OUTDIR/f${ii}.dtf'[opt kernel=text/dtf]'
  dmtcopy infile=$SAVEDIR/${f} outfile=$OUTDIR/f${ii}_sav.dtf'[opt kernel=text/dtf]'
  cat $OUTDIR/f${ii}.dtf | egrep -v "CHECKSUM|DATASUM|HISTORY|HISTNUM" > $OUTDIR/tmp.txt ; mv $OUTDIR/tmp.txt $OUTDIR/f${ii}.dtf
  cat $OUTDIR/f${ii}_sav.dtf | egrep -v "CHECKSUM|DATASUM|HISTORY|HISTNUM" > $OUTDIR/tmp.txt ; mv $OUTDIR/tmp.txt $OUTDIR/f${ii}_sav.dtf
  diff "$OUTDIR/f${ii}.dtf" "$OUTDIR/f${ii}_sav.dtf" 2>&1 >> $OUTDIR/diff.log
  if test $? != 0
  then
    state=1
  fi
  ii=${ii}+1
done
printf "."

#
# Exit condition
#
printf ". "
if test $state = 0
then
  echo "PASS"

  \rm -rf $OUTDIR
  xx=`\ls $ASCDS_WORK_PATH/smoke.${LOGNAME}`
  if test "x$xx" = "x"
  then
    rmdir $ASCDS_WORK_PATH/smoke.${LOGNAME}
  fi
else
  echo "FAIL"
fi

exit $state
