/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmelt.c                                                     */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmulib.h"

#define N_ETYPES 3
static long dmu_etype_offset( dmElementType etype );

typedef struct ElementTypeData_s
{
  char* names[N_ETYPES];
  dmElementType data[N_ETYPES];
  long dim[N_ETYPES];
} ElementTypeData;

static ElementTypeData etype_data =
{ 
 { "V", "R", "I" }, { dmVALUE, dmRANGE, dmINTERVAL }, { 1, 2, 3 }
};


/***************************************************************/
/*                                                             */
/* Function:   dmu_parse_etype_code                            */
/*                                                             */
/* Description:                                                */
/*     Return etype data baseed on code string                 */
/*                                                             */
/*  Input parameters:                                          */
/*           char *code  - code string                         */
/*                                                             */
/*  Output parameters:                                         */
/*            none                                             */
/*                                                             */
/*  Returns:                                                   */
/*            dmElementType  etype_data.data[]                 */
/*                                                             */
/***************************************************************/
dmElementType dmu_parse_etype_code( char* code )
{
  long entry = 0;
  entry = ar_search_c( code, etype_data.names, N_ETYPES );
  return etype_data.data[entry-1];
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_etype_name                                  */
/*                                                             */
/* Description:                                                */
/*     Copy etype name                                         */
/*                                                             */
/*  Input parameters:                                          */
/*           dmElementType etype  - type code                  */
/*           long          maxlen - length of name             */
/*                                                             */
/*  Output parameters:                                         */
/*           char          *name - data name                   */
/*                                                             */
/*  Returns:                                                   */
/*            void                                             */
/*                                                             */
/***************************************************************/
void dmu_etype_name( dmElementType etype, char* name, long maxlen )
{
  long i = 0;

  i = dmu_etype_offset( etype );
  if ( i >= 0 )
    cs_copy_siz( etype_data.names[i], name, maxlen );
  else
    cs_copy( " ", name );
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_etype_dim                                   */
/*                                                             */
/* Description:                                                */
/*     Return etype data dim                                   */
/*                                                             */
/*  Input parameters:                                          */
/*           dmElementType etype  - type code                  */
/*                                                             */
/*  Output parameters:                                         */
/*           none                                              */
/*                                                             */
/*  Returns:                                                   */
/*           long  etype_data.dim                              */
/*                                                             */
/***************************************************************/
long dmu_etype_dim( dmElementType etype )
{
  long i = 0;

  i = dmu_etype_offset( etype );
  if ( i >= 0 )
    return etype_data.dim[i];
  return 0;
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_etype_offset                                */
/*                                                             */
/* Description:                                                */
/*     Return index of input etype code. Return code of -1     */
/*     idicates no match                                       */
/*                                                             */
/*  Input parameters:                                          */
/*           dmElementType etype  - type code                  */
/*                                                             */
/*  Output parameters:                                         */
/*          none                                               */
/*                                                             */
/*  Returns:                                                   */
/*          statice long i or -1                               */
/*                                                             */
/***************************************************************/
static long dmu_etype_offset( dmElementType etype )
{
 long i= 0;

  for ( i = 0; i < N_ETYPES; i++ )
  {
    if ( etype_data.data[i] == etype )
     return i;
  }
  return -1;
}


