/*                                                                
**  Copyright (C) 2002-2003,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dminit.c                                                    */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#define DM_DEFINE_GLOBALS
#include "dmulib.h"

/* 
 * Right now the attempt to use the DS error library has failed;
 * I couldn't find a way to isolate the calls and the #defines within the DM.
 * So I'm continuing to use the NUTILS error library.
 */

#define CXC 0
#if CXC
#include "dserror_funcs.h"
static dsErrList* ds_error_list = NULL;
static dsErrCode dmerrcode = dsNOERR;
#endif



/***************************************************************/
/*                                                             */
/* Function:   dmu_err_init                                    */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   none                                      */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmu_err_init( void )
{
#if CXC
  dsErrInstance* ds_error_instance = NULL;
  dsErrGroup group = dsPTGRPERR;
  dm_errmask = 0;

  if ( !dsISLIBINIT )  /* Really want accessor function */
    dsErrInitLib( group, "CXCDMLIB" );

  dsErrCreateInstance( &ds_error_instance );
  dsErrCreateList( &ds_error_list );
#else
  dm_errmask = ut_error_class_register( NULL, "CXCDM" );

  if ( !dm_error )
    dm_error = ut_error_state();
#endif
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_err_status                                  */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   none                                      */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmu_err_status( void )
{
#if CXC
  dsErrCloseLib();
#else
  ut_error_disp( dm_error, NULL );
  ut_error_clear( dm_error );
#endif
}


/***************************************************************/
/*                                                             */
/* Function:   dmuSetVerbose                                   */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   long level                                */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmuSetVerbose( long level )
{
  dmverbose = level;
  ut_dbg_set( level );
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_err_set                                     */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   long  code - error code                   */
/*                   char *text - error text                   */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            dmError  ErrorCode                               */
/*                                                             */
/***************************************************************/
dmErrCode dmu_err_set( long code, char* text  )
{
  if ( !dm_error )
    dm_error = ut_error_state();

#if CXC
  dsErrAdd( ds_error_list, (dsErrCode)code, Individual, Custom, text );
  dmerrcode = code; 
#else
  ut_error_set( dm_error, text, dm_errmask | code, 0 );
#endif

  return (dm_errmask | code);
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_err_set_code                                */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   long  code - error code                   */
/*                   char *text - error text                   */
/*                   long  fcode -                             */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            dmErrCode  ErrorCode                             */
/*                                                             */
/***************************************************************/
dmErrCode dmu_err_set_code( long code, char* text, long fcode )
{
  if ( !dm_error )
    dm_error = ut_error_state();

#if CXC
  dsErrAdd( ds_error_list, (dsErrCode)code, Individual, Custom, text );
  dmerrcode = code;
#else
  ut_error_set( dm_error, text, dm_errmask | code, fcode );
#endif

  return (dm_errmask | code);
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_err_get                                     */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   none                                      */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            dmErrCode ErrCode                                 */
/*                                                             */
/***************************************************************/
dmErrCode dmu_err_get( void )
{
  dmErrCode code = 0;

#if CXC
  code = dmerrcode;
#else
  if ( dm_error )
    code = ut_error_code( *dm_error );
#endif

  if ( code == 0 )
    code = dmSUCCESS;
  else
    code &= ~dm_errmask;

 return code;
}

/***************************************************************/
/*                                                             */
/* Function:   dmu_err_clear                                   */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   none                                      */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmu_err_clear( void )
{
#if CXC
  dsErrRemoveAll( ds_error_list );
#else

  if ( dm_error )
    ut_error_clear( dm_error );

  ut_error_clear( NULL );
#endif
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_err_get_message                             */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   char  *buf - text message buffer          */
/*                   long   maxlen - length of buffer          */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmu_err_get_message( char* buf, long maxlen )
{
#if CXC
  dsErrInstance ds_error;
  dsErrMsg ds_errmsg;

  if ( ds_error_list )
  {
    dsErrLook( ds_error_list, &ds_error );
    ds_errmsg = dsErrGetInstMsg( &ds_error );
    cs_copy_siz( (char*)ds_errmsg, buf, maxlen );
  }
  else
  {
    cs_copy( " ", buf );
  }

#else
  ut_error_message( dm_error, buf, maxlen );
#endif
}

/***************************************************************/
/*                                                             */
/* Function:   dmu_err_disp_select_msgs                        */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                   long  lErrorCode                            */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*            nothing                                          */
/*                                                             */
/***************************************************************/
void dmu_err_disp_select_msgs(long  lErrorCode )
{
  long  lErrorNumber = 0;

  lErrorNumber = lErrorCode | dm_errmask;

  ut_error_disp_select_msgs( lErrorNumber );

}
