/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmtype.c                                                    */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmulib.h"


/***************************************************************/
/*                                                             */
/* Function:   dmu_block_type_name                             */
/*                                                             */
/* Description:                                                */
/*     Maps block type to a string                             */               
/*                                                             */
/*  Input parameters:                                          */
/*       const dmBlockType  type - block type                  */
/*                                                             */
/*  Output parameters:                                         */
/*             char         *name                              */
/*                                                             */
/*  Returns:                                                   */
/*            void                                             */
/*                                                             */
/***************************************************************/
void dmu_block_type_name( const dmBlockType type, char* name )
{
  char *names[] = { "UNKNOWN", "TABLE", "IMAGE" };
  strcpy( name, names[(int)type] ); 
}


/***************************************************************/
/*                                                             */
/* Function:   dmu_bt_tform_to_datatype                        */
/*                                                             */
/* Description:                                                */
/*     Map TFORM code to data type                             */
/*     The FITS TFORM codes are used in some other data        */
/*     formats also                                            */               
/*                                                             */
/*  Input parameters:                                          */
/*       const char    c - format code                         */
/*                                                             */
/*  Output parameters:                                         */
/*             none                                            */
/*                                                             */
/*  Returns:                                                   */
/*            dmDataType type                                  */
/*                                                             */
/***************************************************************/
dmDataType dmu_bt_tform_to_datatype( const char c )
{ 
 switch( c )
 {
  case 'X':  return DT_BIT;
  case 'B':  return DT_UINT1;
  case 'I':  return DT_INT2;
  case 'J':  return DT_INT4;
  case 'E':  return DT_REAL4;
  case 'D':  return DT_REAL8;
  case 'G':  return DT_REAL8;
  case 'L':  return DT_LOGICAL;
  case 'A':  return DT_CHAR; 
  case 'U':  return DT_UINT2; 
  case 'V':  return DT_UINT4;
  default:   return DT_UNK;  
 }
  
}

