/*                                                                
**  Copyright (C) 2002-2004,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmuflib.c                                                   */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmulib.h"


/***************************************************************/
/*                                                             */
/* Function:   dmu_extract_filename                            */
/*                                                             */
/* Description:                                                */
/*     Copy a string and zero out from the first [ char, thus  */
/*     thus removing a dataset name from vspec                 */
/*                                                             */
/*  Input parameters:                                          */
/*          const char   *vspec                                */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*            char *dsname                                     */
/*                                                             */
/***************************************************************/
char* dmu_extract_filename( const char* vspec )
{
  char *ptr = NULL;
  char *dsname = NULL;

  if ( !vspec )
    return  NULL;
  
  dsname = cs_dup( vspec );
  ptr = strchr( dsname, '[' );

  if ( ptr )
    *ptr = '\0';

  return dsname;
}
   

/***************************************************************/
/*                                                             */
/* Function:   dmu_extract_filename_extension                  */
/*                                                             */
/* Description:                                                */
/*      Return trailing extension from bare filename: not to   */
/*      be deallocated                                         */
/*                                                             */
/*  Input parameters:                                          */
/*          const char   *filenamep                            */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*            char *filename_extension                         */
/*                                                             */
/***************************************************************/
char* dmu_extract_filename_extension( const char* filenamep )
{
  char *filename = (char*)filenamep;
  long n = 0;
  long i = 0;

  if ( !filename )
    return  NULL;

  n = strlen( filename );

  for ( i = n; i > 0 && filename[i] != '/'; i-- )
  {
    if ( filename[i] == '.' ) 
      return( filename+i+1 );
  } 
  return NULL;
}
