/*                                                                
**  Copyright (C) 1998-1999,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#define CFAsccsID "%W% %E% %U%"

/*-------------------------------------------------------------------------

   FILE NAME:		%M%

   DEVELOPMENT: 	AXAF Science Center DataModel
 			mnoble@cfa.harvard.edu (Initial version)

   DESCRIPTION:		Common code for ASC DataModel dmtest sample program.

---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ascdm.h"
#include "dmexamples.h"

int ListBlocks(dmDataset *ds,long dumpKeys,long dumpCoords)
{
	dmBlock		*blk = NULL;
	long		num,curr=0;

	num = dmDatasetGetNoBlocks(ds);
	printf("\nDATASET CONTAINS %ld BLOCKS", dmDatasetGetNoBlocks(ds));
	printf("\nNOW LOOPING ON CONTAINED BLOCKS ...\n");

	if (num > 0)
	   while ( curr<num )
	   {
		curr++;
		blk = dmDatasetMoveToBlock( ds, curr );
		if ( blk ) 
                {
	  	 printf("\nOPENED BLOCK %ld ",curr);
		 ListInfo(blk,curr,dumpKeys,dumpCoords);
		 dmBlockClose(blk);
                } else {
		 curr = num;
		}
			
	   }

	return dmSUCCESS;
}
		
int
ListInfo(dmBlock *blk,long blkNo,long dumpKeys,long dumpCoords)
{
	char		tmpStr[dmMAXSTR];
	char 		blkName[dmMAXSTR];
	dmDescriptor	*axis=NULL;
	dmDescriptor    *coord;
	dmDescriptor    *axisGrp;
	long		i,j,k;
	dmDataset       *ds=dmBlockGetDataset(blk);
	long		*axisLens=NULL;


		/* Get the block name in two ways, and compare */
		dmBlockGetName(blk,tmpStr,dmMAXSTR);
		dmDatasetGetBlockName(ds,blkNo,blkName,dmMAXSTR);

		if (strcmp(tmpStr,blkName))
		{
		   printf("\n*** WARNING: BLOCK NAMES DO NOT MATCH");
		   printf("\ndmBlockGetName:        %s",tmpStr);
		   printf("\ndmDatasetGetBlockName: %s\n",blkName);
		}

		printf("\tNAME: %s\tTYPE: ",tmpStr);
		switch(dmBlockGetType(blk))
		{
			case dmIMAGE:
			  	printf("IMAGE");
				strcpy(tmpStr,"IMAGE DATA");
				break;
			case dmTABLE:
			  	printf("TABLE");
				strcpy(tmpStr,"COLUMN");
				break;
			default:
			  	printf("UNKNOWN BLOCK TYPE");
				break;
		}

		printf("\nNUMBER OF ROWS: %ld", dmTableGetNoRows(blk));

		/* Loop on contained descriptors */
		for (j=1; j<=dmTableGetNoCols(blk); j++)
		{
		  	/* Note that this returns the image data  */
			/* descriptor if blk is actually an image */
			axis = dmTableOpenColumnNo(blk,j);
			if (!axis)
	  			ERROR("Couldn't open axisumn!");

			printf("\n\t%s DESCRIPTOR %ld ", tmpStr, j);

			/* Check axis dimensionality and array size */
			if ((k = dmGetArrayDim(axis)) > 0)
		  		k = dmGetArrayDimensions(axis,&axisLens);
				
			printf("\tARRAY DIMENSIONALITY IS %ld", k);
			if (axisLens )
			{
			    for (i=0; i<k; i++)
				printf("\n\t\tAXIS %ld LENGTH: %ld",
				       			i,axisLens[i]);
				  	
			    free(axisLens);
			    axisLens=NULL;
			}

		        if (dumpCoords)
			{
			    long numCoords = dmDescriptorGetNoCoords(axis);
			    long numGrps   = dmArrayGetNoAxisGroups(axis);

			    printf("\n\t   CONTAINS %ld COORDINATE"
				      " TRANSFORM DESCRIPTOR(S)",numCoords);
			    for (i=0; i < numCoords; i++)
			    {
			        if ((coord = dmDescriptorGetCoordNo(axis,i+1)))
			           PrintTransform(coord);
			    }
			    printf("\n\t   CONTAINS %ld AXIS GROUP(S), WHICH",
				   				numGrps);
			    for (i=0; i < numGrps; i++)
			    {
			       axisGrp = dmArrayGetAxisGroup(axis,i+1);
			       numCoords = dmDescriptorGetNoCoords(axisGrp);
			       printf("\n\t\tCONTAINS %ld COORDINATE"
				      " TRANSFORM DESCRIPTOR(S)",numCoords);
			       for (i=0; i < numCoords; i++)
			       {
			          coord   = dmDescriptorGetCoordNo(axisGrp,i+1);
			          PrintTransform(coord);
			       }
			    }
			}
		}

		/* Loop on keys */
		if (dumpKeys)
		{
		   long nkeys=0;
		   dmDescriptor **keys = dmBlockGetKeyList(blk,&nkeys);
		   printf("\nBLOCK CONTAINS %ld HEADER KEYS\n",nkeys);

		   for (i=0; i<nkeys; i++)
		   {
		     dmGetName(keys[i],tmpStr,dmMAXSTR);
		     printf("\tHEADER KEY %ld\tNAME : %s\n",i+1,tmpStr);
		   }

		   free(keys);
		}
 return 0;
}

int
PrintTransform(dmDescriptor *coord)
{
  	char tmpStr[dmMAXSTR];
	long i,dim;
	void *crpix;
	double *crval;
	double *cdelt;
	dmBlock *pBl;

	dmGetName(coord,tmpStr,dmMAXSTR);
	printf("\n\t\t\tTRANSFORM NAME: %s",tmpStr);

	if (dmCoordGetTransformType(coord,tmpStr,&i,dmMAXSTR) == dmSUCCESS)
	   printf("\n\t\t\tTRANSFORM TYPE: %s",tmpStr);
	else
	   ERROR("Could not get coord transform type!");

	dim = (long)dmGetElementDim(dmCoordGetParent(coord));
  	printf("\n\t\t\tTRANSFORM DIMENSION : %ld",dim);
	crval = (double*)malloc(dim*sizeof(double));
	cdelt = (double*)malloc(dim*sizeof(double));

	dmCoordGetTransformType(coord,tmpStr,&i,dmMAXSTR);
  	printf("\n\t\t\tDATA TYPE: ");

	switch(dmGetDataType(coord))
	{
	case dmSHORT:
	  	printf("SHORT");
		crpix = (short*)malloc(dim*sizeof(short));
		dmCoordGetTransform_s(coord,(short*)crpix,crval,cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %hd",i,((short*)crpix)[i]);
		break;
	case dmLONG:
	  	printf("LONG");
		crpix = (long*)malloc(dim*sizeof(long));
		dmCoordGetTransform_l(coord,(long*)crpix,crval,cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %ld",i,((long*)crpix)[i]);
		break;
	case dmFLOAT:
	  	printf("FLOAT");
		crpix = (float*)malloc(dim*sizeof(float));
		dmCoordGetTransform_f(coord,(float*)crpix,crval,cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %f",i,((float*)crpix)[i]);
		break;
	case dmDOUBLE:
	  	printf("DOUBLE");
		crpix = (double*)malloc(dim*sizeof(double));
		dmCoordGetTransform_d(coord,(double*)crpix,crval,cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %f",i,((double*)crpix)[i]);
		break;
	case dmBYTE:
	  	printf("UNSIGNED BYTE");
		crpix = (unsigned char*)malloc(dim*sizeof(unsigned char));
		dmCoordGetTransform_ub(coord,(unsigned char*)crpix,crval,
				      				cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %u",i,
			   			((unsigned char*)crpix)[i]);
		break;
	case dmUSHORT:
	  	printf("UNSIGNED SHORT");
		crpix = (unsigned short*)malloc(dim*sizeof(unsigned short));
		dmCoordGetTransform_us(coord,(unsigned short*)crpix,crval,
				      				cdelt,dim);
		for(i=0; i<dim; i++)
		   printf("\n\t\t\tCRPIX[%ld] is %u",i,
			  			((unsigned short*)crpix)[i]);
		break;
	case dmULONG:
	  	printf("UNSIGNED LONG");
		crpix = (unsigned long*)malloc(dim*sizeof(unsigned long));
		dmCoordGetTransform_ul(coord,(unsigned long*)crpix,crval,
				      				cdelt,dim);
		for (i=0; i<dim; i++)
		    printf("\n\t\t\tCRPIX[%ld] is %lu",i,
			   			((unsigned long*)crpix)[i]);
		break;
	default:
	  	printf("UNKNOWN");
	}

	for (i=0; i<dim; i++)
	    printf("\n\t\t\tCRVAL[%ld] is %f",i,crval[i]);
	for (i=0; i<dim; i++)
	    printf("\n\t\t\tCDELT[%ld] is %f",i,cdelt[i]);

	if (!(pBl = dmDescriptorGetBlock(coord)))
	   ERROR("Couldn't get parent block for coord descriptor!");
	dmBlockGetName(pBl,tmpStr,dmMAXSTR);
	printf("\n\t\t\tNAME OF DESCRIPTOR'S PARENT BLOCK IS: %s",tmpStr);

	free(crval); free(crpix); free(cdelt);
 return 0;
}
