/*                                                                
**  Copyright (C) 1998-1999,2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#define CFAsccsID "%W% %E% %U%"
/*-------------------------------------------------------------------------

   FILE NAME:		%M%

   DEVELOPMENT: 	AXAF Science Center DataModel
 			mnoble@cfa.harvard.edu (Initial version)

   DESCRIPTION:		General purpose image dumper, based in part upon
   			test code provided by Joan Flanagan (joan@cfa).
			Needs updates to work with > 2D images.

---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ascdm.h"
#include "dmexamples.h"

long		vs[2],ve[2];
static int  next_line(dmDescriptor *,dmDataType,int,int,float *line);
static void printhelp(char **argv);
/*static void dumpcoords(dmDescriptor *imgData);*/

/*-------------------------------------------------------------------------*/

int main(int argc, char **argv)
{
  dmBlock	*image;
  dmDescriptor	*data;
  dmDataType 	type;
  int 		naxes;
  long 		*axlens;
  float		*line;
  int 		ii;
  int 		printit;
  int 		row		= 0;
  char 		*vspec		= NULL;
  int 		silent		= 0;
  int 		dumpData	= 1;
  int 		dumpCoords	= 0;
  int 		entire		= 0;
  int		gotlx,gotly,gotux,gotuy;

  if (argc < 2)
     printhelp(argv);

  vs[0] = vs[1] = ve[0] = ve[1] = 0;
  gotlx = gotly = gotux = gotuy = 0;

  /* Look For Command-Line Switches */
  for (ii=1; ii<argc; ii++)
  {
      if (argv[ii][0] != '-' && !vspec)
	 vspec = argv[ii];
      else
      {
	/* should we print image row data? */
  	if (!strncmp(argv[ii],"-s",2))
	   silent=1;
	else if (!strncmp(argv[ii],"-n",2))
	   dumpData=0;
	else if (!strncmp(argv[ii],"-c",2))
	   dumpCoords=1;
	else if (!strncmp(argv[ii],"-e",2))
	   entire=1;
	else if (!strncmp(argv[ii],"-lx=",4))  /* lower x of img section */
	{
	     gotlx=1;
	     vs[0] = atol(argv[ii]+4);
	     if (vs[0] < 1)
	        ERROR("Can't have image pixel number < 1");
	}
	else if (!strncmp(argv[ii],"-ly=",4))  /* lower y of img section */
	{
	     gotly=1;
	     vs[1] = atol(argv[ii]+4);
	     if (vs[1] < 1)
	        ERROR("Can't have image pixel number < 1");
	}
	else if (!strncmp(argv[ii],"-ux=",4))  /* upper x of img section */
	{
	     gotux=1;
	     ve[0] = atol(argv[ii]+4);
	     if (ve[0] < 1)
	        ERROR("Can't have image pixel number < 1");
	}
	else if (!strncmp(argv[ii],"-uy=",4))  /* upper y of img section */
	{
	     gotuy=1;
	     ve[1] = atol(argv[ii]+4);
	     if (ve[1] < 1)
	        ERROR("Can't have image pixel number < 1");
	}
	else if (!strncmp(argv[ii],"-h",2))
	  printhelp(argv);
      }
  }


  image = dmImageOpen(vspec);
  if (!image)
      ERROR("Couldn't open dataset/block");

  data = dmImageGetDataDescriptor(image);
  if (!data)
      ERROR("Couldn't get image data descriptor");

  type = dmGetDataType(data);
  naxes = dmGetArrayDimensions(data,&axlens);
  if (!axlens)
      ERROR("Couldn't get image dimensions");

  ListInfo(image,1,0,dumpCoords);
  printf("\n");


  /* are we dumping an image subsection? */
  /* if (gotlx && gotly && gotux && gotuy) */
  if (vs[0] && vs[1] && ve[0] && ve[1])
  {
      axlens[0] = ve[0] - vs[0] + 1;		/* image subsection */
      axlens[1] = ve[1] - vs[1] + 1;
  }
  else
  {
     vs[0] = vs[1] = 1;;			/* entire image */
     ve[0] = axlens[0]; 
     if (naxes > 1) 
        ve[1] = axlens[1];
     else
        ve[1] = 1;
  }

  /* For now this code only works for images binned from */
  /* tables so the datatype is assumed to be short.	 */
  if (entire)
  {
    short *pixels = (short*)malloc(axlens[0]*axlens[1]*sizeof(short));
    dmImageDataGetSubArray_s(data,vs,ve,pixels);

    if (dumpData)
    {
       /* dump the image section to stdout */
       printf("\nDump of entire image section (%ld,%ld) to (%ld,%ld) :\n",
	   			vs[0],vs[1],ve[0],ve[1]);
       for (ii=0; ii<axlens[0]*axlens[1]; ii++)
   	   printf("%6.2f", (float)pixels[ii]);

       printf("\n");
       exit(EXIT_SUCCESS);
    }
    else
    {
       /* create a copy of the image section, on disk */
       char dsname[1024];
       dmDescriptor *imData;
       dmBlock *tmpImage;

       strcpy(dsname,getenv("LOGNAME"));
       strcat(dsname,".img[image1]");
       system("\\rm -rf ${LOGNAME}.img");
       printf("\nCreating image: %s\n",dsname);
       tmpImage = dmImageCreate(dsname,type,axlens,2);
       if (tmpImage)
       {
	   vs[0] = vs[1] = 1; ve[0] = axlens[0]; ve[1] = axlens[1];
           if ((imData = dmImageGetDataDescriptor(tmpImage)))
    	      dmImageDataSetSubArray_s(imData,vs,ve,pixels);
	   else
	      ERROR("Couldn't get image descriptor to write image section!");
       }
       else
	  ERROR("Couldn't get image block handle to write image section!");
    }
  }
  else
  {
    line = (float*)malloc(axlens[0]*sizeof(float));
    if (!line)
       ERROR("Out of memory");
  }

  if (!dumpData)
     exit(EXIT_SUCCESS);

  printf("\nPrinting non-zero pixel array sections ...\n\n");

  for (row=vs[1]; row <= ve[1]; row++)
  {
      if (next_line(data,type,axlens[0],row,line) != 0)
	return 1;
      
      /* Print pixel section (only if not all zeroes) */
      for (ii = 0; ii < axlens[0]; ++ii)
	  if ((printit = line[ii]))
	  {
	     printf("ROW #%d contains non-zero data ... \n",row);
	     break;
	  }

      /* print data if not running in silent mode */
      if (!silent)
         if (printit)
         {
            for (ii = 0; ii < axlens[0]; ++ii)
	        printf("%6.2f", line[ii]);
            printf("\n");
         }
  }

  printf("\n");
  free(line);
  dmImageClose(image);
  return 0;
}

int next_line(dmDescriptor *data,dmDataType type,int xdim,int row,float *line)
{
  long	lower_left[2];
  long	upper_right[2];
  long	*lbuf;
  short	*sbuf;
  int 	ii;
 
  /* Set up the image section boundaries */
  lower_left[0]  = vs[0];
  upper_right[0] = ve[0];
  lower_left[1]  = upper_right[1] = row;
  
  switch (type)
  {
    case dmSHORT:

      if (!(sbuf = (short*)malloc(xdim * sizeof(short))))
         ERROR("Out of memory");

      if (dmImageDataGetSubArray_s(data,lower_left,upper_right,sbuf) != 1) 
	  ERROR("Error reading image data");
      
      /* convert to float */
      for (ii = 0; ii < xdim; ++ii)
	line[ii] = (float)sbuf[ii];
      
      free(sbuf);

      break;

    case dmLONG:

      if (!(lbuf = (long*)malloc(xdim * sizeof(long))))
         ERROR("Out of memory");

      if (dmImageDataGetSubArray_l(data,lower_left,upper_right,lbuf) != 1)
	  ERROR("Error reading image data");
      
      /* convert to float */
      for (ii = 0; ii < xdim; ++ii)
	line[ii] = (float)lbuf[ii];
      
      free(lbuf);
      break;

    case dmFLOAT:

      if (dmImageDataGetSubArray_f(data,lower_left,upper_right,line) != 1)
	  ERROR("Error reading image data");
      break;

    default:

      ERROR("Unrecognized image data type");

    }

  return 0;
}

void printhelp(char **argv)
{
  	printf("\n%s: AXAF Science Center DataModel image dumper utility",
	       							argv[0]);
        printf("\nUsage: %s virtual_image_specification [-sh]\n",argv[0]);
	printf("\nSupported command-line options are:");
	printf("\n	-n  do NOT open image, just print image dims & coords");
	printf("\n	-s  silent (ie, don't print pixel data)");
	printf("\n	-h  print this help message");
	printf("\n");
	exit(1);
}

#if 0
/* Unused currently */
void dumpcoords(dmDescriptor *imgData)
{
	long 		numcoords;
	dmDescriptor    *coord;
	dmDescriptor    *axisGrp;
	long 		numGrps = dmArrayGetNoAxisGroups(imgData);
	printf("\n\t\tCONTAINS %ld AXIS GROUP(S)",numGrps);
	axisGrp = dmArrayGetAxisGroup(imgData,1);
	numcoords = dmDescriptorGetNoCoords(axisGrp);
	if (numcoords > 0)
	{
		printf("\n\t   CONTAINS %ld COORDINATE"
                     		" TRANSFORM DESCRIPTOR(S)",numcoords);
		coord   = dmDescriptorGetCoord(axisGrp);
               PrintTransform(coord);
	}
	printf("\n");
}
#endif
