/*                                                                
**  Copyright (C) 1997-2000,2003-2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#define CFAsccsID "%W% %E% %U%"


/*-------------------------------------------------------------------------

   FILE NAME:		dmtest.c

   DEVELOPMENT: 	AXAF Science Center DataModel
 			mnoble@cfa.harvard.edu (Initial version)

   DESCRIPTION:		ASC DataModel sample/test program.

---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "ascdm.h"
#include "dmexamples.h"
#ifndef _FITSIO_H
#include "fitsio.h"
#endif
/*-------------------------------------------------------------------------*/

static  void 	PrintHelp(void);
	void	check_for_errors(void);
char xxbuf[1024];
/*-------------------------------------------------------------------------*/

int main(int argc, char **argv)
{
	char		tmpStr[dmMAXSTR];
	dmDataset	*ds;
	dmBlock		*tab;
	dmBlock 	*copyBlock;
	dmBlock 	*copyBlock2;
	dmDescriptor	**cols=NULL;
	long		i,j, ii;
	long   		lvals[10];
	unsigned char   bvals[60];
	short		svals[10];
	char		strVal[STRCOLSIZE+1];
	long		row;
	char 		tabname[LMAX+1]="STDEVT";
	char		newtable[LMAX+1]="TESTTABLE";
	char		*newimage="TSTIMAGE";
	char		filtstr[LMAX+1]="[STDEVT][X>12000]";
	char 		testds[LMAX+1]=FULLDATASET;
	char		creationKernel[LMAX+1]="FITS";
	long		whichTests=1;
	long		dumpKeys=0;
	long		dumpCoords=0;
	unsigned short 	tlmin_ushort, tlmin_ushort_orig;
	unsigned short 	tlmax_ushort, tlmax_ushort_orig;
	unsigned char 	tlmin_bit, tlmin_bit_orig;
	unsigned char 	tlmax_bit, tlmax_bit_orig;
	unsigned char 	tlmin_byte, tlmin_byte_orig;
	unsigned char 	tlmax_byte, tlmax_byte_orig;

	/* Look For Command-Line Switches */
	for (i=1; i<argc; i++)
	{
	  	if (!strncmp(argv[i],"-d=",3))
		  strcpy(testds,argv[i]+3);
		else if (!strncmp(argv[i],"-t=",3))
		  strcpy(tabname,argv[i]+3);
		else if (!strncmp(argv[i],"-k=",3))
		{
		  if (toupper(*(argv[i]+3))=='Q')
		     strcpy(creationKernel,"IRAF");
		}
		else if (!strncmp(argv[i],"-c=",3))
		  whichTests=strtol(argv[i]+3,NULL,10);
		else if (!strncmp(argv[i],"-C",2))
		  dumpCoords=1;
		else if (!strncmp(argv[i],"-f=",3))
		  /* ignore quotes */
		  strcpy(filtstr,argv[i]+3);
		else if (!strncmp(argv[i],"-v",3))
                  dmSetVerbose(3);
		else if (!strncmp(argv[i],"-h",3))
		  PrintHelp();
		else if (!strncmp(argv[i],"-i",3))
		  newimage="";
		else if (!strncmp(argv[i],"-p",3))
		  dumpKeys=1;
		else 
		{
		  fprintf(stderr,"\nERROR: Illegal option");
		  PrintHelp();
		}
	}

  	printf("\n******** Beginning ASC DataModel Test Program ********\n\n");

	/*************************************************************/
	/*********** TEST 0:  GENERIC - ALWAYS PERFORMED *************/
	/*************************************************************/

	/* Nominal Kernel get/set Testing */
	dmKernelGetCopy(tmpStr,256);
	printf("CURRENT COPY KERNEL IS %s\n",tmpStr);
	dmKernelSetCopy("foobie");
	dmKernelGetCopy(tmpStr,256);
	printf("AFTER RESETTING TO ILLEGAL VALUE, COPY KERNEL IS %s\n",tmpStr);
	dmKernelSetCopy("irafqpoe");
	dmKernelGetCopy(tmpStr,256);
	printf("AFTER RESETTING TO LEGAL VALUE, COPY KERNEL IS %s\n",tmpStr);
	dmKernelSetCreate(creationKernel);
	dmKernelGetCreate(tmpStr,256);
	printf("CREATE KERNEL IS %s\n",tmpStr);


	/*************************************************************/
	/********* TEST 1:  GENERIC INFO ON EXISTING TABLE ***********/
	/*************************************************************/

	if (whichTests==1 || whichTests==99)
	{
		ds = dmDatasetOpen(testds);
		if (!ds)
			ERROR("Couldn't open dataset!");

		printf("\nOPENED %s DATASET SUCCESFULLY,",testds);
	 	ListBlocks(ds,dumpKeys,dumpCoords);

#ifdef LOCAL_QPOE_TEST
		{
		  	/* This test will only work properly at SAO, and */
		  	/* needs the following specific QPOE test file:  */
		  	/* /data/tropical/anto/QPOE/n4051_GO96a/ds.qp    */

		  	struct {double time;
			  	short  x;
				short  y;
				float  ra;
				float  dec;
				short  dx;
				short  dy;
				} row;

			/* Re-open first block and skip around in it */
			tab = dmDatasetMoveToBlock(ds,1);

			for (i=0; i<10; i++)
			       dmTableGetRow(tab,&row);

			dmTableSetRow(tab,200000);
			for (i=0; i<10; i++)
			       dmTableGetRow(tab,&row);

			dmTableSetRow(tab,500000);
			for (i=0; i<10; i++)
			       dmTableGetRow(tab,&row);

			dmTableSetRow(tab,900000);
			for (i=0; i<10; i++)
			       dmTableGetRow(tab,&row);

			dmTableSetRow(tab,997990);
			for (i=0; i<20; i++)
			       dmTableGetRow(tab,&row);
		}
#endif

		dmDatasetClose(ds);
	}

	/*************************************************************/
	/****** TEST 2: CREATE NEW DS & TABLE FOR KEYWORD TESTS ******/
	/*************************************************************/
	if (whichTests==2 || whichTests==99)
	{
	  	dmDescriptor	**dmKeys = (dmDescriptor **)calloc(8,
							sizeof(dmDescriptor*));
		short  		sVal;
		double 		dVal;
		long            j;
		short           sarinvals[5] = {3, 5, 9, 15, 23};
		short           sarnewvals[3] = {6, 6, 6};
		double          darinvals[5] = {111.1,222.2,333.3,444.4,555.5};
		short           saroutvals[5] = {0, 0, 0, 0, 0};
		double          daroutvals[5] = {0, 0, 0, 0, 0,};
		dmBool		lVal;
		char   		strVal[LMAX+1];
		long		numKeys;
		dmDescriptor	**keyArray;
		dmDescriptor	*pDesc;
		char            strComm[COMMENTSIZE];

		dmKernelSetCreate(creationKernel);

		sprintf(tmpStr,"\\rm -rf %s",DATASET);
		if (system(tmpStr))
	  		ERROR("Couldn't remove test dataset!");

	  	sprintf(newtable,"%s[KEYTABLE]",DATASET);

		tab = dmTableCreate(newtable);
		if (!tab)
	  		ERROR("Couldn't create new dataset/table!");

		/* How many keywords are there? */
		printf("\nNEWLY-CREATED TABLE, CONTAINS %ld HEADER KEYS\n",
							dmBlockGetNoKeys(tab));
	
		/* Write some keywords */
		dmKeys[0]=dmKeyWrite_s(tab,SHORTKEY,SHORTVAL,"NO UNIT",NULL);
		dmKeys[1]=dmKeyWrite_c(tab,TEXTKEY,TEXTVAL,"NO UNIT",NULL);
		dmKeys[2]=dmKeyWrite_d(tab,DOUBLEKEY,DOUBLEVAL,"NO UNIT",NULL);
		dmKeys[3]=dmKeyWrite_q(tab,LOGICALKEY,LOGVAL,"NO UNIT",NULL);

		/* Get a list of the descriptors */
		numKeys=-1;
		keyArray = dmBlockGetKeyList(tab,&numKeys);
		printf("AFTER ADDING 4 KEYS, TABLE CONTAINS %ld HEADER KEYS\n",
								     numKeys);

		/* Check the key values, using scalar get calls */
		sVal = dmGetScalar_s(dmKeys[0]);
		if (sVal==SHORTVAL)
		 printf("\nSHORT KEYWORD SCALAR GET1 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("SHORT KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %hd\tACTUAL VAL: %hd",SHORTVAL,sVal);
		}

		/* Read short key again, casting to double */
		dVal = dmGetScalar_d(dmKeys[0]);
		if ((short)dVal==SHORTVAL)
		 printf("SHORT KEYWORD GET WITH CASTING AS"
			" DOUBLE SUCCEEDED!\n");
		else
		{
		  printf("SHORT KEYWORD CASTING READ FAILED!\n");
		  printf("EXPECTED VAL: %hd\tACTUAL VAL: %hd\n",SHORTVAL,
			 (short)dVal);
		}

		dVal = dmGetScalar_d(dmKeys[2]);
		if (dVal==DOUBLEVAL)
		  printf("DOUBLE KEYWORD SCALAR GET1 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("DOUBLE KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %f\tACTUAL VAL: %f",DOUBLEVAL,dVal);
		}

		dmGetScalar_c(dmKeys[1],strVal,LMAX);
		if (!strcmp(strVal,TEXTVAL))
		  printf("STRING KEYWORD SCALAR GET1 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("STRING KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %s\tACTUAL VAL: %s",TEXTVAL,strVal);
		}

		/* Write some values using scalar write calls */
		dmSetScalar_s(dmKeys[0],111);
		dmSetScalar_d(dmKeys[2],111.111);
		dmSetScalar_c(dmKeys[1],"FOOPIE");

		/* Read em back */
		dVal = sVal = 0; strVal[0]=0;

		sVal = dmGetScalar_s(dmKeys[0]);
		if (sVal==111)
		 printf("\nSHORT KEYWORD SCALAR GET2 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("SHORT KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %hd\tACTUAL VAL: %hd",SHORTVAL,sVal);
		}

		dVal = dmGetScalar_d(dmKeys[2]);
		if (dVal==111.111)
		  printf("DOUBLE KEYWORD SCALAR GET2 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("DOUBLE KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %f\tACTUAL VAL: %f",DOUBLEVAL,dVal);
		}

		dmGetScalar_c(dmKeys[1],strVal,LMAX);
		if (!strcmp(strVal,"FOOPIE"))
		  printf("STRING KEYWORD SCALAR GET2 MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("STRING KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %s\tACTUAL VAL: %s",TEXTVAL,strVal);
		}

		/* Write more values using scalar write calls */
		dmSetScalar_s(dmKeys[0],SHORTVAL);
		dmSetScalar_d(dmKeys[2],DOUBLEVAL);
		dmSetScalar_c(dmKeys[1],TEXTVAL);

		/* Free the array (but not the individual descriptors) */
		if (keyArray)
		  free(keyArray);

		/* Read them back, using the keyword routines */
		dmKeys[0] = dmKeyRead_s(tab,SHORTKEY,&sVal);
		if (dmKeys[0] && sVal==SHORTVAL)
		  printf("\nSHORT KEYWORD READ MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("SHORT KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %hd\tACTUAL VAL: %hd",SHORTVAL,sVal);
		}

		dmKeys[1] = dmKeyRead_c(tab,TEXTKEY,strVal,LMAX);
		if (dmKeys[1] && !strcmp(strVal,TEXTVAL))
		  printf("STRING KEYWORD READ MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("STRING KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %s\tACTUAL VAL: %s",TEXTVAL,strVal);
		}

		dmKeys[2] = dmKeyRead_d(tab,DOUBLEKEY,&dVal);
		if (dmKeys[2] && dVal==DOUBLEVAL)
		  printf("DOUBLE KEYWORD READ MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("DOUBLE KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %f\tACTUAL VAL: %f",DOUBLEVAL,dVal);
		}

		dmKeys[3] = dmKeyRead_q(tab,LOGICALKEY,&lVal);
		if (dmKeys[3] && lVal==LOGVAL)
		  printf("LOGICAL KEYWORD READ MATCHED VALUE WRITTEN!\n");
		else
		{
		  printf("LOGICAL KEYWORD READ FAILED!\n");
		  printf("EXPECTED VAL: %d\tACTUAL VAL: %d",(int)LOGVAL,
			 					(int)lVal);
		}

		/* Determine the key length using the string col length call */
		printf("\nSTRING LENGTH OF KEY DESCRIPTOR 2 IS %ld\n",
		       				dmGetStrLen(dmKeys[1]));

		/* Delete a key */

		dmDescriptorDelete(dmKeys[2]);

		/* Now, how many keywords are there? */
		printf("\nAFTER DELETING KEY 2, TABLE CONTAINS %ld KEYS\n",
							dmBlockGetNoKeys(tab));

		/* Add two more keys */
		dmKeys[2]=dmKeyWrite_d(tab,"NEWDBLKY",DOUBLEVAL,"NO UNIT",NULL);
		dmKeys[4]=dmKeyWrite_d(tab,DOUBLEKEY,DOUBLEVAL,"NO UNIT",NULL);

		printf("AFTER ADDING 2 KEYS, TABLE CONTAINS %ld HEADER KEYS\n",
							dmBlockGetNoKeys(tab));

		/* Write some comment and history keywords */
		dmBlockWriteComment(tab,"COMMENT",COMMENT1);
		dmBlockWriteComment(tab,"COMMENT",COMMENT2);
		dmBlockWriteComment(tab,"COMMENT",COMMENT3);
		dmBlockWriteComment(tab,"COMMENT",COMMENT4);
		dmBlockWriteComment(tab,"HISTORY",HISTORY1);
		dmBlockWriteComment(tab,"HISTORY",HISTORY2);
		dmBlockWriteComment(tab,"HISTORY",HISTORY3);

		/* Create some columns and keys with the same names */
		cols = (dmDescriptor **)calloc(4,sizeof(dmDescriptor*));
		if(!cols)
	  		ERROR("Couldn't allocate mem for descriptor array!");

		cols[0] = dmColumnCreate(tab,"FOO1",dmSHORT,0,"","");
		cols[1] = dmColumnCreate(tab,"FOO2",dmSHORT,0,"","");
		cols[2] = dmKeyWrite_d(tab,"FOO1",DOUBLEVAL,"NO UNIT",NULL);
		cols[3] = dmKeyWrite_d(tab,"FOOFOO",DOUBLEVAL,"NO UNIT",NULL);

		/* Do various keyword descriptor checks */
		if ( !dmKeys[4] )
   		  printf( "\n DMKEYS[4] IS NULL - ERROR \n" );

		dmSetDesc(dmKeys[4], "Double Key comment.");
		printf("\nDOUBLE KEY COMMENT FIELD UPDATED.");
		if(dmGetDesc(dmKeys[4], strComm, COMMENTSIZE))
		{
		    if(!strcmp(strComm, "Double Key comment."))
			printf("\nCOMMENT MATCHED WRITTEN VALUE.");
		    else
			printf("\nCOMMENT DID NOT MATCH WRITTEN VALUE!!!");
		}
		dmSetUnit(dmKeys[4], "NEW UNIT");
		printf("\nDOUBLE KEY UNIT FIELD UPDATED.");
		if(dmGetUnit(dmKeys[4], strVal, STRCOLSIZE))
		{
		    printf("\nREAD DOUBLE UNIT ");
		    if(strVal && !strcmp(strVal, "NEW UNIT"))
		    	printf("MATCHES UPDATED VALUE.\n");
		    else
		    	printf("DOES NOT MATCH WRITTEN VALUE!\n");
		}
		else
		    printf("\n*** ERROR: COULD NOT READ UPDATED UNIT.\n");

		dmSetName(dmKeys[4], "NEWNAME");
		printf("\nDOUBLEKEY NAME UPDATED TO NEWNAME.");
		if(dmGetName(dmKeys[4], strVal, STRCOLSIZE))
		{
		    printf("\nUPDATED DOUBLE NAME ");
		    if(strVal && !(strcmp(strVal, "NEWNAME")))
			printf("\nMATCHES UPDATED VALUE.\n");
		    else
			printf("\nDOES NOT MATCH UPDATED VALUE!\n");
		}
		else
		    printf("\n*** ERROR: COULD NOT READ UPDATED NAME.");

		printf("\nFOO2 column has col # %ld (s/b 2)",
		       				dmColumnGetNo(cols[1]));
		printf("\nFOO1 column has col # %ld (s/b 1)",
		       				dmColumnGetNo(cols[0]));
		printf("\nFOO1 key has col # %ld (s/b 0)",
		       				dmColumnGetNo(cols[2]));
		printf("\nFOOFOO key has col # %ld (s/b 0)",
		       				dmColumnGetNo(cols[3]));


		printf("\n\nARRAYED KEYWORD DESCRIPTOR TESTS:");
		/* Write and read arrayed keyword descriptor. */
		/* With a short name */

		printf("\n WRITING AND READING DOUBLE TYPE ARRAY KEY");

		dmKeys[6] = dmKeyWriteArray_d(tab, "DBLRAY", darinvals, 
					      "Unit", "Comment here.", 5);
		if ( !dmKeys[6] )
	  	  printf( "\n ERROR: DMKEYS 6 NOT WRITTEN\n" );

		dmGetArray_d(dmKeys[6], daroutvals, 5);
		
		for(j=0; j<5; j++)
		{
		    if(darinvals[j] == daroutvals[j])
			printf("\n KEY INDEX %ld MATCHED VALUE WRITTEN", j);
		    else
		      printf("\n KEY INDEX %ld DOES NOT MATCH WRITTEN VALUE %f %f\n"
			     ,j, darinvals[j], daroutvals[j]);
		}

		/* Write and read arrayed keyword descriptor. */
		/* With a long name */

		printf("\n\n WRITING AND READING SHORT TYPE ARRAY KEY");
		dmKeys[7] = dmKeyWriteArray_s(tab, "SHRTARAY", sarinvals, 
					      "Unit", "Comment here.", 5);

		dmGetArray_s(dmKeys[7], saroutvals, 5);
		
		for(j=0; j<5; j++)
		{
		    if(sarinvals[j] == saroutvals[j])
			printf("\n KEY INDEX %ld MATCHED VALUE WRITTEN", j);
		    else
		      printf("\n KEY INDEX %ld DOES NOT MATCH WRITTEN VALUE!!!"
			     ,j);
		}


		/* Delete arrayed keyword descriptor */

		dmDescriptorDelete(dmKeys[6]);
	        printf("\n\n DOUBLE TYPE ARRAYED KEYWORD DESCRIPTOR");
		printf("\n  HAS BEEN DELETED");

		dmSetName(dmKeys[7], "NEWARRAY");

		printf("\n SHORT TYPE ARRAY KEY HAS BEEN RENAMED");
		
		dmSetArray_s(dmKeys[7], sarnewvals, (long)3);
		printf("\n SHORT TYPE ARRAYED KEYWORD"
		       " VALUES HAVE BEEN UPDATED");
		dmKeyRead_s(tab, "NEWARRAY", &sVal);
		if(sVal != (short)6)
		    printf("\n UPDATED SHORT ARRAYED KEYWORD VALUE"
			   " DOES NOT MATCH VALUE WRITTEN!!!");
		else
		    printf("\n FIRST UPDATED SHORT ARRAYED KEYWORD VALUE"
			   " MATCHED VALUE WRITTEN");

		/* Open key/col by searching on name.  First call should   */
		/* should return a key descriptor, second a col descriptor */
		pDesc = dmKeyOpen(tab,"FOO1");
		if (pDesc)
		  printf("\n\nFOO1 KEY FOUND CORRECTLY BY dmKeyOpen");
		else
		  printf("\n\n*** ERROR: FOO1 KEY NOT FOUND BY dmKeyOpen");

		pDesc = dmKeyOpen(tab,"FOO2");
		if (pDesc)
		  printf("\nFOO2 COLUMN FOUND CORRECTLY BY dmKeyOpen");
		else
		  printf("\n*** ERROR: FOO2 COLUMN NOT FOUND BY dmKeyOpen");
		
		printf("\n\nNOW, CLOSING THEN REOPENING TABLE ...\n");
		/* Close table and dataset at same time */
		dmTableClose(tab);
		free(dmKeys);

		/* Re-open table and redo last dmKeyOpen test */
		tab = dmTableOpen(newtable);

		/* Open key/col by searching on name.  First call should   */
		/* should return a key descriptor, second a col descriptor */
		pDesc = dmKeyOpen(tab,"FOO1");
		if (pDesc)
		  printf("\nFOO1 KEY FOUND CORRECTLY BY dmKeyOpen");
		else
		  printf("\n*** ERROR: FOO1 KEY NOT FOUND BY dmKeyOpen");

		pDesc = dmKeyOpen(tab,"FOO2");
		if (pDesc)
		  printf("\nFOO2 COLUMN FOUND CORRECTLY BY dmKeyOpen");
		else
		  printf("\n*** ERROR: FOO2 COLUMN NOT FOUND BY dmKeyOpen");

		/* Close table for good */
		dmTableClose(tab);

                /* Give writing permition to group for table */
                sprintf(tmpStr,"chmod u+w,g+w %s",DATASET);
                system(tmpStr);

		free(cols);
	}

	/*************************************************************/
	/***** FOR OTHER TESTS, CREATE TEMP COPY OF DATASET     ******/
	/*************************************************************/

	if ((whichTests>2 && whichTests<5) || (whichTests>7 && whichTests<10))
	{
		sprintf(tmpStr,"\\rm -rf %s ; cp -r %s %s ; chmod -R u+w,g+w %s",
					DATASET,testds,DATASET,DATASET);
		printf("\nCREATING COPY OF %s AS %s\n",testds,DATASET);
		if (system(tmpStr))
	  		ERROR("Couldn't duplicate test dataset!");
	}

	/*************************************************************/
	/********* TEST 3:  CREATE AND POPULATE NEW TABLE ************/
	/*************************************************************/

	if (whichTests==3 || whichTests==99)
	{

		dmDescriptor *pDesc_checksum;
	  	char errStr[dmMAXSTR+1];
	  	char **strArray=NULL;
                char* strArrData[3] = { "AAA", "BBBB", "1234" };
                char** TextArrVals = NULL;
                char** strArrVal;
                long bitlength;
                long ival;
		char* ncval = NULL;
                long ret;
		char tmpKernel[80];

		TextArrVals = (char**)malloc( 3 * sizeof( char* ));
                for ( ival = 0; ival < 3; ival++ )
                {
                 TextArrVals[ival] = (char*)malloc( 6 * sizeof( char ) );
                }
                strArrVal = strArrData;
		ds = dmDatasetOpenUpdate(DATASET);
               
		if (!ds)
			ERROR("Couldn't open dataset!");
		dmDatasetGetKernel( ds, tmpKernel, 80 );

		/* Test the setting of the internal buffer size   */
		/* WARNING: DO NOT SET SIZE SMALLER THAN SMALLEST */
		/*	    # OF ROWS READ IN ANY SINGLE CALL.    */
		i = 180;
		dmSetInternals("buffersize",&i);

		tab = dmBlockCreate(ds,newtable,dmTABLE);
		if (!tab)
	  		ERROR("Couldn't create new table!");

   	        printf( "Created %s\n", newtable );
		/* Add Some Columns */
		cols = (dmDescriptor **)calloc(12,sizeof(dmDescriptor*));

		if(!cols)
	  		ERROR("Couldn't allocate memory for new column array!");

		cols[0] = dmColumnCreate(tab,COL1,dmSHORT,0,"","");
		cols[1] = dmColumnCreate(tab,COL2,dmSHORT,0,"","");
		cols[2] = dmColumnCreate(tab,COL3,dmBOOL,0,"","");
		cols[3] = dmColumnCreate(tab,COL6,dmTEXT,STRCOLSIZE,"","");
		cols[4] = dmColumnCreate(tab,COL4,dmUSHORT,0,"","");
		cols[5] = dmColumnCreateArray(tab,COL5,dmLONG,0,"","",10);
                cols[6] = dmColumnCreate(tab,COL7,dmBIT,0,"","");
                cols[7] = dmColumnCreateArray(tab,COL8,dmBIT,0,"","",10);
                cols[8] = dmColumnCreate(tab,COL9,dmBIT,13,"coded","13-bit flag");
                cols[9] = dmColumnCreate(tab,COL10,dmBYTE,0,"","");
                cols[10] = dmColumnCreate(tab,COL11,dmULONG,0,"",""); 

 	 	 cols[11] = dmColumnCreateArray(tab,"StrArrayCol",dmTEXT,5,"","",3);
		if (!(cols[0] || cols[1] || cols[2] || cols[3] || cols[4] ||
		      	  cols[5] || cols[6] || cols[7] || cols[8] ||
                          cols[9] || cols[10] ))
		   ERROR("Couldn't allocate mem for new table columns!");

		dmDescriptorSetNull_c( cols[3], "Null String" );

		/* Write 200 Rows */
		printf("WRITING 200 ROWS ...\n");
		row=0;
		do
		{
		  	row++;

			memset(strVal,0,STRCOLSIZE+1);
			sprintf(strVal,"STRCOL:  ROW # %ld",row);

	  		if (row%2)
				dmSetScalar_q(cols[2],dmTRUE);
			else
				dmSetScalar_q(cols[2],dmFALSE);

			dmSetScalar_s(cols[0],(short)row);
	  		dmSetScalar_s(cols[1],(short)row);
	  		dmSetScalar_c(cols[3],strVal);

                        dmSetArray_c( cols[11], strArrVal, 3 );
			dmTablePutRow(tab,NULL);
		}	while(row < 200);

		/* Use malloc to get string array mem, instead of just	*/
		/* declaring the array above.  This is a more rigorous	*/
		/* test of the DM buffered I/O calls.			*/

		strArray = (char**)malloc(10*sizeof(char*));
		for (i=0; i<10; i++)
			strArray[i] = (char*)calloc(STRCOLSIZE+1,sizeof(char));

		/* Read Back The Middle 100 Rows */
		printf("READING BACK MIDDLE 100 OF ROWS JUST WRITTEN ...\n");
		if (dmTableSetRow(tab,49) != 49)
		   ERROR("Couldn't go to row #49");
		*strVal = '\0';
		printf("\t%s\t%s\t%s\t%s\n",COL1,COL2,COL3,COL6);
		while ((row=dmTableNextRow(tab)) < 151 && row != dmNOMOREROWS)
		{
		  	printf("ROW %ld",row);

	  		/* Every 10th row, do multi-row string col read */
	  		if (!((row-1)%10))
			   dmGetScalars_c(cols[3],strArray,STRCOLSIZE,row,10);

			printf("\t%d\t%d\t%d\t%s\n",dmGetScalar_s(cols[0]),
						    dmGetScalar_s(cols[1]),
			       			    dmGetScalar_q(cols[2]),
						    strArray[(row-1)%10]);
		}

		/* Write 200 More Rows */
		printf("\nWRITING 200 MORE ROWS ...\n");
		dmTableSetRow(tab,dmENDOFTABLE);
		row=200;
		do
		{
		  	row++;

			memset(strVal,0,STRCOLSIZE+1);
			sprintf(strVal,"STRCOL:  ROW # %ld",row);

	  		if (row%2)
				dmSetScalar_q(cols[2],dmTRUE);
			else
				dmSetScalar_q(cols[2],dmFALSE);

			dmSetScalar_s(cols[0],(short)row);
	  		dmSetScalar_s(cols[1],(short)row);
	  		dmSetScalar_c(cols[3],strVal);
			ret = dmTablePutRow(tab,NULL);
                        if ( ret < 0 )
			{
			 printf( "DM WRITE ERROR ROW=%ld\n", row );
                         exit(1);
			}
		}	while(row < 400);

		/* Write 200 more rows (to all columns)*/
		dmTableSetRow(tab,dmENDOFTABLE);
		row=400;
		do
		{
		  	row++;
		  	for (i=0; i<10; i++)
			{
			  	svals[i] = row + i;
				lvals[i] = row;
				bvals[i] = i;
				bvals[10 + i] = 101;
				bvals[20 + i] = i;
				bvals[30 + i] = 101;
				bvals[40 + i] = i;
				bvals[50 + i] = 101;
			}

			memset(strVal,0,STRCOLSIZE+1);
			sprintf(strVal,"STRCOL: ROW # %ld",row);

	  		if (!((row-1)%10))
			{
			    dmSetScalars_s(cols[0],svals,row,10);
			    dmSetScalars_s(cols[1],svals,row,10);
                            dmSetScalars_bit(cols[8], &bvals[40], row, 10);
			}
	  		if (row%2)
				dmSetScalar_q(cols[2],dmTRUE);
			else
				dmSetScalar_q(cols[2],dmFALSE);

	  		dmSetScalar_c(cols[3],strVal);
			dmSetScalar_us(cols[4],(unsigned short)row);
			dmSetArray_l(cols[5],lvals,10);
	  		dmSetScalar_bit(cols[6],&bvals[7]);
			dmSetArray_bit(cols[7],&bvals[20],10);

                        dmSetArray_c( cols[11], strArrVal, 3 );
			ret = dmTablePutRow(tab,NULL);
                        if ( ret < 0 )
			{
			 printf( "DM WRITE ERROR\n" );
                         exit(1);
			}

		}	while(row < 600);

		/* Flush the buffer. */
		/* And test dmBlockOpen */
                dmBlockClose(tab);

                tab = dmBlockOpen(NULL,"");
                tab = dmBlockOpen(ds,"");
                tab = dmBlockOpen(ds,NULL);
                tab = dmBlockOpen(ds,newtable);
		pDesc_checksum = dmKeyWrite_c(tab,"CHECKSUM","","",""); 
                dmBlockClose(tab);
                tab = dmBlockOpen(ds,newtable);
                cols = dmTableOpenColumnList(tab,&i);

		dmDescriptorGetNull_c( cols[3], &ncval );
		if ( ncval )
		{
                 printf( "NCVAL = %s\n", ncval );
		 free( ncval );
		}
		bitlength = dmDescriptorGetLength( cols[6] );
                printf( "BIT LENGTH COL7 = %ld\n", bitlength );
		bitlength = dmDescriptorGetLength( cols[7] );
                printf( "BIT LENGTH COL8 = %ld\n", bitlength );
		bitlength = dmDescriptorGetLength( cols[8] );
                printf( "BIT LENGTH COL9 = %ld\n", bitlength );
		/* Read a short as a long */
		if (dmTableSetRow(tab,10) != 10)
		   ERROR("Couldn't go to row #10");
		for ( i = 0; i < 3; i++ )
                  TextArrVals[i][0] = '\0';


	        dmGetArray_c( cols[11], TextArrVals, 3 );
		printf( " TEXT ARRAY = %s, %s, %s\n", TextArrVals[0], TextArrVals[1], TextArrVals[2] );

		if (dmTableSetRow(tab,550) != 550)
		   ERROR("Couldn't go to row #550");
		lvals[0] = dmGetScalar_l(cols[4]);
                /* Get one scalar value from row 550 */
                for (i=0; i<10; i++)
                {
                    bvals[i] =  i;
                    bvals[i + 10] = 101;
                }
		dmGetScalar_bit(cols[6], &bvals[17]);
                  if (bvals[7] != bvals[17])
                     printf ("bvals[7] != bvals[17] %d %d\n", bvals[7], bvals[17]);
                /* Get an array of 10 values from row 550 */
                for (i=0; i<10; i++)
                {
                    bvals[20 + i] =  i;
                    bvals[30 + i ] = 101;
                }
		dmGetArray_bit(cols[7],&bvals[30],10);
                for (ii = 20; ii<30; ii++)
                  if (bvals[ii] != bvals[ii +10])
                     printf ("bvals[%ld] != bvals[%ld]\n",ii, ii+10);
                   
		/* Get bit values  from rows 551-560 */
                for (i=0; i<10; i++)
                {
                    bvals[40 + i] =  i;
                    bvals[50 + i] = 101;
                }
                dmGetScalars_bit(cols[8], &bvals[50],551, 10);
                for (ii = 40; ii<50; ii++)
                  if (bvals[ii] != bvals[ii + 10])
                     printf ("bvals[%ld] != bvals[%ld] %d %d\n",ii, ii+10, bvals[ii], bvals[ii+10]);



               /* Test dmDescriptorGetRange_xxx and */
                /* dmDescriptorSetRange_xxx */
                dmDescriptorGetRange_us(cols[4],
                                        &tlmin_ushort_orig,
                                        &tlmax_ushort_orig);

                dmDescriptorSetRange_us(cols[4],
                                        tlmin_ushort_orig,
                                        tlmax_ushort_orig);

                dmDescriptorGetRange_bit(cols[8],
                                         &tlmin_bit_orig,
                                         &tlmax_bit_orig);

                dmDescriptorSetRange_bit(cols[8],
                                         tlmin_bit_orig,
                                         tlmax_bit_orig);

                dmDescriptorGetRange_ub(cols[9],
                                        &tlmin_byte_orig,
                                        &tlmax_byte_orig);

                dmDescriptorSetRange_ub(cols[9],
                                        tlmin_byte_orig,
                                        tlmax_byte_orig);


                /* Flush the buffer. */
                /* And test dmBlockOpen */
                dmBlockClose(tab);
                tab = dmBlockOpen(NULL,"");
                tab = dmBlockOpen(ds,"");
                tab = dmBlockOpen(ds,NULL);
                tab = dmBlockOpen(ds,newtable);
                cols = dmTableOpenColumnList(tab,&i);

                /* Test whether dmDescriptorSetRange_xxx values equal  */
                /* dmDescriptorSetRange_xxx values.  */
                dmDescriptorGetRange_us(cols[4],
                                        &tlmin_ushort,
                                        &tlmax_ushort);
                if ( tlmin_ushort != tlmin_ushort_orig )
                    printf ("tlmin_ushort != tlmin_ushort_orig\n");
                if ( tlmax_ushort != tlmax_ushort_orig )
                    printf ("tlmax_ushort != tlmax_ushort_orig\n");

                dmDescriptorGetRange_bit(cols[8],
                                         &tlmin_bit,
                                         &tlmax_bit);
                if ( tlmin_bit != tlmin_bit_orig )
                    printf ("tlmin_bit != tlmin_bit_orig\n");
                if ( tlmax_bit != tlmax_bit_orig )
                    printf ("tlmax_bit != tlmax_bit_orig\n");

                dmDescriptorGetRange_ub(cols[9],
                                        &tlmin_byte,
                                        &tlmax_byte);
                if (tlmin_byte != tlmin_byte_orig)
                    printf ("tlmin_byte != tlmin_byte_orig\n");
                if ( tlmax_byte != tlmax_byte_orig )
                    printf ("tlmax_byte != tlmax_byte_orig\n");



		/* Delete a column (this will fail till ETOOLS is fixed) */
		printf("\nATTEMPTING TO DELETE A COLUMN (should fail)");
		dmDescriptorDelete(cols[4]);
		errStr[0] = 0;
		dmGetErrorMessage(errStr,dmMAXSTR);
		printf("\nRESULTING ERROR MESSAGE: %s\n",errStr);
		/* Determine length of string column */
		i = dmGetStrLen(cols[3]);
		printf("\nATTEMPTING TO DETERMINE STRING COLUMN LENGTH");
		if (i == STRCOLSIZE)
		   printf("\nREPORTED STRING COL LEN MATCHES REQUESTED SIZE");
		else
		{
		   printf("\nERROR: REPORTED STRING COL LENGTH MISMATCH!!!");
		   printf("\nREQUESTED LEN: %d, REPORTED LEN %ld",STRCOLSIZE,i);
		}

		/* Copy the table structure to an unlike kernel dataset */
		{
		   char **availableKers=NULL;
		   char currKer[dmMAXSTR] = "FITS";
		   long i=0,numKers=0;
		   dmDataset *tmpDs = NULL;

		   /* Close original table/dataset to flush contents to disk */
		   dmTableClose(tab);

		   /* Reopen */
		   sprintf(tmpStr,"%s[%s]",DATASET,newtable);
		   tab = dmTableOpenUpdate(tmpStr);

		   /* Peruse the list of available kernels */
		   dmDatasetGetKernel(dmBlockGetDataset(tab),currKer,dmMAXSTR);
		   dmKernelGetList(&availableKers,&numKers);

		   if (availableKers)
		      while (i<numKers && !strcmp(availableKers[i],currKer))
		            i++;
		   else
		      ERROR("Retrieving available kernel list");
/* Screw that...*/
		i=0;			

		   dmKernelSetCreate(availableKers[i]);

		   sprintf(tmpStr,"\\rm -rf %s-%s",DATASET,availableKers[i]);
		   system(tmpStr);
		   sprintf(tmpStr,"%s-%s",DATASET,availableKers[i]);
		   tmpDs = dmDatasetCreate(tmpStr);
		   printf("\nCOPYING TABLE TO %s ...\n",tmpStr);
		   copyBlock = dmBlockCreateCopy(tmpDs,"newblock",tab,dmTRUE);
		   if (copyBlock)
		   {
		      printf("\nTABLE SUCCESFULLY dmBlockCreateCopy'd ...\n");
		      dmTableClose(copyBlock);
                      /* Give writing permission to group */
                      sprintf(tmpStr,"chmod -R u+w,g+w %s-%s",DATASET,
			      				availableKers[i]);
		      system(tmpStr);
		   }
		   else
		      ERROR("Making dmBlockCreateCopy of the table");

		   for (i=0; i<numKers; i++)
		     free(availableKers[i]);
		   free(availableKers);
		}

		printf("\nMAKING ANOTHER COPY IN ORIGINAL DATASET ...\n");
		copyBlock = dmBlockCreateCopy(dmBlockGetDataset(tab),
							"newerblock",tab,dmTRUE);
		if (copyBlock)
		{
		      printf("\nTABLE SUCCESFULLY dmBlockCreateCopy'd ...\n");
		      dmBlockClose(copyBlock);
		}
		else
		      ERROR("Making dmBlockCreateCopy of the table");
		dmTableClose(tab);
 
		for (i=0; i<10; i++)
		    free(strArray[i]);
		free(strArray);
		free(cols);

                for ( ival = 0; ival < 3; ival++ )
                  free( TextArrVals[ival] );
                free( TextArrVals );

	}

	/*************************************************************/
	/********* TEST 4:    ENTIRE-ROW BASED I/O        ************/
	/*************************************************************/

	if (whichTests==4 || whichTests==99)
	{
		struct  {short  	xval;
		  	 short 		yval;
		  	 double 	dval; 

#ifdef QPOE_BYTECOL_WORKS
			 unsigned char	bval;
#endif
			 float  	fval;
			 char   	strval[STRCOLSIZE+1];
			 dmBool		boolval;
			 long   	lArray[7];
			} theRow;

		ds = dmDatasetOpenUpdate(DATASET);
		if (!ds)
			ERROR("Couldn't open dataset!");

		tab = dmBlockCreate(ds,newtable,dmTABLE);
		if (!tab)
	  		ERROR("Couldn't create new table!");

		/* Add Some Columns */
		cols = (dmDescriptor **)calloc(8,sizeof(dmDescriptor*));
		if(!cols)
	  		ERROR("Couldn't allocate memory for new column array!");

		cols[0] = dmColumnCreate(tab,COL1,dmSHORT,0,"","");
		cols[1] = dmColumnCreate(tab,COL2,dmSHORT,0,"","");
		cols[2] = dmColumnCreate(tab,COL3,dmDOUBLE,0,"","");
#ifdef QPOE_BYTECOL_WORKS
		cols[3] = dmColumnCreate(tab,COL4,dmBYTE,0,"","");
#endif
		cols[4] = dmColumnCreate(tab,COL5,dmFLOAT,0,"","");
		cols[5] = dmColumnCreate(tab,COL6,dmTEXT,STRCOLSIZE,"","");
		cols[6] = dmColumnCreate(tab,COL7,dmBOOL,0,"","");
		cols[7] = dmColumnCreateArray(tab,COL8,dmLONG,0,"","",7);

		if (!(cols[0] && cols[1] && cols[2] &&
#ifdef QPOE_BYTECOL_WORKS
		      cols[3]  &&
#endif
		      cols[4] && cols[5] && cols[6] && cols[7]))
		   ERROR("Couldn't allocate mem for new table columns!");
printf( "SIZEOF ROW STRUCT = %d\n", sizeof( theRow ));
printf( "OFFSETS = %d %d %d %d %d %d %d\n",
(char*)&theRow.xval - (char*)&theRow,
(char*)&theRow.yval - (char*)&theRow,
(char*)&theRow.dval - (char*)&theRow,
(char*)&theRow.fval - (char*)&theRow,
(char*)&(theRow.strval[0]) - (char*)&theRow,
(char*)&theRow.boolval - (char*)&theRow,
(char*)&theRow.lArray - (char*)&theRow
);
		/* Write Arbitrary # Rows */
		row=0;
		do
		{
		  	row++;
#ifdef QPOE_BYTECOL_WORKS
			theRow.bval=99;
#endif
			if (row%2)
			  theRow.boolval=dmTRUE;
			else
			  theRow.boolval=dmFALSE;

			theRow.dval = theRow.fval =
			theRow.xval = theRow.yval = row;

		  	for (i=0; i<7; i++)
			  	theRow.lArray[i] = row + i;

			/* Completely initialize the string column */
			sprintf(theRow.strval,"STRCOL: ROW # %ld",row);
			j = strlen(theRow.strval);
			memset(theRow.strval+j,0,STRCOLSIZE-j+1);

			/* Finally, write the row */
			dmTablePutRow(tab,&theRow);
		}	while (row < 2167);

		/* Read Them Back, entire row at a time */
		printf("READING BACK THE ROWS, ENTIRE ROW AT A TIME ...\n");
		row = dmTableSetRow(tab,1);
			memset(theRow.strval,0,STRCOLSIZE+1);
		theRow.dval = theRow.fval = theRow.xval = theRow.yval = 0;
		sprintf(theRow.strval,"@@@@@@@@@@@@@@@@@@@@@@@@@@@@");

#ifdef QPOE_BYTECOL_WORKS
		printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",COL1,COL2,COL3,COL4,
		       					COL7,COL5,COL6,COL8);
#else
		printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\n",COL1,COL2,COL3,
		       					COL7,COL5,COL6,COL8);
#endif

		while(dmTableGetRow(tab,&theRow) != dmNOMOREROWS)
		{
#ifdef QPOE_BYTECOL_WORKS
			printf("%d\t%d\t%6.2f\t%hd\t%hd\t%f\t%s\n\t(%4ld",
			       theRow.xval, theRow.yval,theRow.dval,
			       theRow.bval, theRow.boolval, theRow.fval,
			       theRow.strval, theRow.lArray[0]);
#else
			printf("%d\t%d\t%6.2f\t%hd\t%f\t%s\n\t(%4ld",
			       theRow.xval, theRow.yval,theRow.dval,
			       theRow.boolval, theRow.fval,
			       theRow.strval, theRow.lArray[0]);
#endif
			for (i=1; i<7; i++)
			  	printf(",%4ld",theRow.lArray[i]);
			printf(")\n");
			memset(theRow.strval,0,STRCOLSIZE+1);


		};

		dmBlockClose(tab);
		dmDatasetClose(ds);
		free(cols);
	}

	/*************************************************************/
	/******* TEST 5: ROW-BASED I/O ON SUBSET OF TABLE COLS *******/
	/*************************************************************/

	if (whichTests==5 || whichTests==99)
	{
		struct  {short  	x;
		  	 short		y;
			 double		time;
			} rowStruct;


		ds = dmDatasetOpen(testds);
		if (!ds)
			ERROR("Couldn't open dataset!");

		/* Standard table selection (no type casting) */
		tab = dmTableOpenSelect(ds,tabname,"x,y,time",NULL);
		if (!tab)
		   ERROR("Couldn't open table OR find the selected columns!");

		printf("\nROW #\tx\ty\ttime\n-----\t-\t-\t----\n");

		while ((row=dmTableGetRow(tab,&rowStruct)) <= 200)
			printf("%ld\t%hd\t%hd\t%f\n",row,rowStruct.x,
						rowStruct.y,rowStruct.time);

		dmBlockClose(tab);


		dmDatasetClose(ds);
	}

	/*************************************************************/
	/********* TEST 6:    IMAGE CREATION        	  ************/
	/*************************************************************/

	if (whichTests==6 || whichTests==99)
	{
		dmBlock 	*image;
		char		vspec[81];
		long		axes[4]={10,15,2,2};
		long		naxes=4;
		char		*imgDS;

		/* If user is using default test DS, remove it first */
		if (!strcmp(testds,FULLDATASET))
		{
		   sprintf(tmpStr,"\\rm -rf %s",DATASET);
		   if (system(tmpStr))
	  		ERROR("Couldn't remove test dataset!");
		   imgDS=DATASET;
		}
		else
		   imgDS=testds;

		if (newimage && newimage[0])
		  sprintf(vspec,"%s[%s]",imgDS,newimage);
		else
		  sprintf(vspec,"%s",imgDS);

		/* Create an image */
		if ((image = dmImageCreate(vspec,dmDOUBLE,axes,naxes)))
		{
		dmDescriptor	*imDesc=NULL;
		dmDescriptor    *pDesc_checksum;
		long		lowerLeft[4]={1,1,1,1};
		long		upperRight[4]={10,15,1,1};
		double		row[10];
		double		imData[150];
                char* ccn[2] = { "TAX1", "TAX2" };
                dmDescriptor* ccdd;

		printf("\nImage sucessfully created ...");
		 /* Get its data descriptor */
		if (!(imDesc = dmImageGetDataDescriptor(image)))
		    ERROR("Couldn't retrieve image data descriptor!");

                /* Add a 2D axis group */
		ccdd = dmArrayCreateAxisGroup( imDesc, "TESTG", dmDOUBLE, "pixel",
                    ccn, 2 );
		/* Fabricate the image data */
		for (i=0; i<150; i++)
		      	imData[i] = i;

		 /* Write it to the image */
		dmImageDataSetSubArray_d(imDesc,lowerLeft,upperRight,imData);

		printf("\nImage data sucessfully written ...");

		lowerLeft[0]=1;
		upperRight[0]=10;
		/* Read it back, one image row at a time */
		for (i=1; i<16; i++)
		{
		   	lowerLeft[1]= upperRight[1] = i;
		    	dmImageDataGetSubArray_d(imDesc,lowerLeft,
						       upperRight,row);
		}

		printf("\nImage data sucessfully read ...\n");

		/* Make another copy of the image */
		copyBlock = dmBlockCreateCopy(dmBlockGetDataset(image),
					      "newblock",image,dmTRUE);

		if (copyBlock)
		{
		   printf("\nIMAGE SUCCESSFULLY dmBlockCreateCopy'd ...\n");
		   dmBlockClose(copyBlock);
		}
		else
		   ERROR("Making dmBlockCreateCopy of the image");

		copyBlock2 = dmBlockCreateCopy(dmBlockGetDataset(image),
					      "newblock2",image,dmTRUE);

		if (copyBlock2)
		{
		   pDesc_checksum =dmKeyWrite_c(copyBlock2,"CHECKSUM","","",""); 
		   dmBlockClose(copyBlock2);
		}
		else
		   ERROR("Making dmBlockCreateCopy of the image to copyBlock2");

		/* Close image and dataset */
		dmImageClose(image);

		}
		else
		   ERROR("Couldn't create image!");
	}

	/*************************************************************/
	/********* TEST 7:    FILTERED TABLE I/O       	  ************/
	/*************************************************************/

	if (whichTests==7 || whichTests==99)
	{
		struct  {short  	x;
		  	 short		y;
			 short		pha;
			 short		pi;
			 double		time;
			 short		detx;
			 short		dety;
			 short		status;
			} rowStruct;

		dmDescriptor *xCol;
		dmDescriptor *yCol;
		dmDescriptor *phaCol;

		printf("\nTHE FILTER APPLIED IS : %s\n",filtstr);
		printf("\nNOTE THAT THIS TEST MAY FILTER ON COLS THAT MAY");
		printf("\nNOT BE PRESENT IN THE TABLE YOU'VE SPECIFIED.\n");
	  	strcat(testds,filtstr);

		tab = dmTableOpen(testds);

		dmDatasetGetName(dmBlockGetDataset(tab),tmpStr,dmMAXSTR);
		printf("\nTHE OPENED DATASET NAME IS %s\n",tmpStr);

		if (!tab)
		   ERROR("Couldn't open table/block OR bad filter expression!");

		printf("\nROW #\tX\tY\tPHA\n-----\t-\t-\t---\n");

		/* We could use row-based I/O here, but do not  */
		/* so as to exercise a different section of the */
		/* table traversal code.  So, instead we open   */
		/* each column individually for reading.	*/

		xCol = dmTableOpenColumn(tab,"X");
		yCol = dmTableOpenColumn(tab,"Y");
		phaCol = dmTableOpenColumn(tab,"PHA");

		i = 0;
		do
		{
		      rowStruct.x   = dmGetScalar_s(xCol);
		      rowStruct.y   = dmGetScalar_s(yCol);
		      rowStruct.pha = dmGetScalar_s(phaCol);
		      i++;
		      printf("%ld\t%hd\t%hd\t%hd\n",i,rowStruct.x,
			     			rowStruct.y,rowStruct.pha);
		}
		while (dmTableNextRow(tab) != dmNOMOREROWS);

		printf("\nRESCANNING TABLE ...\n\n");
		i = 0;
		dmTableSetRow(tab,1);
		do
		{
		      rowStruct.x = dmGetScalar_s(xCol);
		      rowStruct.y = dmGetScalar_s(yCol);
		      rowStruct.pha = dmGetScalar_s(phaCol);
		      i++;
		      printf("%ld\t%hd\t%hd\t%hd\n",i,rowStruct.x,
			     			rowStruct.y,rowStruct.pha);
		}
		while (dmTableNextRow(tab) != dmNOMOREROWS);

		dmTableClose(tab);
	}

	/*************************************************************/
	/********* TEST 8:    GENERAL IMAGE I/O        	  ************/
	/*************************************************************/

	if (whichTests==8 || whichTests==99)
	{
		dmBlock 	*pImage;
		dmDescriptor	*imDesc=NULL;
		short		imData[512];
		long		lowerLeft[2]={1,1};
		long		upperRight[2]={512,1};

		ds = dmDatasetOpen(testds);
		if (!ds)
			ERROR("Couldn't open dataset!");

		pImage=dmDatasetNextBlock(ds);
		if (!pImage  || dmBlockGetType(pImage) != dmIMAGE)
		  	ERROR("First block is not a valid image!");

		if (!(imDesc = dmImageGetDataDescriptor(pImage)))
			    ERROR("Couldn't retrieve image data descriptor!");

		for (i=1; i<=512; i++)
		{
		    lowerLeft[1]= upperRight[1] = i;
		    dmImageDataGetSubArray_s(imDesc,lowerLeft,upperRight,
					     			     imData);
		}

		dmImageClose(pImage);
	}

	/*************************************************************/
	/********* TEST 9:    BLOCK DELETION         	  ************/
	/*************************************************************/

	if (whichTests==9 || whichTests==99)
	{
	   /* List the blocks in the dataset */
	   ds = dmDatasetOpenUpdate(DATASET);
	   if (!ds)
		ERROR("Couldn't open dataset!");

	   j = dmDatasetGetNoBlocks(ds);
	   if (j > 0)
	   {
	      /* Print the current dataset contents */
	      ListBlocks(ds,0,0);

	      /* If we have more than 1 block, delete the "middle" block */
	      if (j>1)
	      {
		 i = j/2+1;
		 if (i>j) i = j;	/* establish upper bound */
	      }
	      else
		 i = j;

	      /* Go to the first block and delete it! */
	      tab = dmDatasetMoveToBlock(ds,i);
	      if (tab)
	      {
	         dmBlockGetName(tab,tmpStr,dmMAXSTR);
	         printf("\n\nDELETING BLOCK #%ld, NAME: %s\n\n",i,tmpStr);
	         dmBlockDelete(tab);

	         if (dmGetError() == dmSUCCESS)
		    printf("\nThe block was SUCCESSFULLY deleted ...\n");
	         else
		    ERROR("\nThe block was NOT SUCCESSFULLY deleted ...\n");

	         /* List dataset contents again */
	         ListBlocks(ds,0,0);
	      }
	      else
		 ERROR("\nCould not OPEN block for deletion ...\n");
	   }
	   else
	   {
	     printf("\nDataset is empty ... nothing to delete!");
	   }

	   dmDatasetClose(ds);
	}

	/*************************************************************/
	/********* TEST 10: IRAF FILE/DATASET EQUIVALENCE ************/
	/*************************************************************/

	if (whichTests==10 || whichTests==99)
	{
		dmKernelSetCreate("IRAF");
		strcpy(testds,"x9xtst.qp");
		system("\\rm -rf x9xtst.qp");
		if ((tab = dmTableCreate(testds)))
		{
	           ListBlocks(dmBlockGetDataset(tab),0,0);
	  	   dmTableClose(tab);
		   system("\\rm -rf x9xtst.qp");
		}
		else
		   ERROR("\nCould not CREATE single-file IRAF dataset...\n");
	}

        /*************************************************************/
        /******************* TEST11: SUBSPACE TEST *******************/
        /*************************************************************/
         
        if (whichTests==11 || whichTests==99)
        {
              int             i=0, j=0, k=0, m=0, n=0;
              int             rnumber1, rnumber2, tnumber, cnumber;
              short           sMins1[2];
              short           sMaxs1[2];
              long            lMins[2] = {6, 48};
              long            lMaxs[2] = {12, 56};
              double          dMins1[6] = {1.1, 102.2, 203.3, 304.4, 405.5, 506.6};
              double          dMaxs1[6] = {11.1, 112.2, 213.3, 314.4, 415.5, 516.6};

              double          intMins1[2] = {22.2, 144.4};
              double          intMaxs1[2] = {55.5, 188.8};
              double          intMins2[2] = {33.3};
              double          intMaxs2[2] = {177.7};
              short**         sFilMins;
              short**         sFilMaxs;
              double**        dFilMins;
              double**        dFilMaxs;
              long            lsFilSize = 0;
              long            ldFilSize = 0;
              long            lFilSize = 0;
              char*           pstrValTabCols[2];
              char            strDSSCol1[] = "TSTART";
              char            strDSSCol2[] = "TSTOP";
              char*           colName[4] = {"SUBDATA", "X", "DOUBLEDATA", "Y"};
              dmDescriptor    **ppDSSDescs = (dmDescriptor **)calloc(7,
                                                     sizeof(dmDescriptor*));

              dmKernelSetCreate(creationKernel);

              sprintf(tmpStr,"\\rm -rf %s",DATASET);
              if (system(tmpStr))
                 ERROR("Couldn't remove test dataset!");

              sprintf(newtable,"%s[KEYTABLE]",DATASET);
              tab = dmTableCreate(newtable);
              if (!tab)
                 ERROR("Couldn't create new dataset/table!");


              dmBlockCreateSubspaceCpt(tab);
              dmBlockCreateSubspaceCpt(tab);

              sFilMins = (short**)calloc(2, sizeof(short*));
              sFilMaxs = (short**)calloc(2, sizeof(short*));
              dFilMins = (double**)calloc(3, sizeof(double*));
              dFilMaxs = (double**)calloc(3, sizeof(double*));
              for (i=0; i < 2; i++)
              {
                  sMins1[0]=i+1;
                  sMins1[1]=i+102;
                  sMaxs1[0]=i+11;
                  sMaxs1[1]=i+112;
                  dmBlockSetSubspaceCpt(tab, (long)(i+1));
                  ppDSSDescs[i]=dmSubspaceColCreate_s(tab,
                                            colName[i], "",
                                            sMins1, sMaxs1, (long)2);
                  ppDSSDescs[i+2] = dmSubspaceColCreate_d(tab,
                                            colName[i+2], "seconds",
                                            (double*)(&dMins1[j]),
                                            (double*)(&dMaxs1[j]), (long)3);

                  dmSubspaceColGet_s(ppDSSDescs[i], sFilMins, sFilMaxs,
                                     &lsFilSize);

                  if((*sFilMins)[0] == sMins1[0] &&
                     (*sFilMins)[1] == sMins1[1] &&
                     (*sFilMaxs)[0] == sMaxs1[0] &&
                     (*sFilMaxs)[1] == sMaxs1[1] &&
                     lsFilSize == 2)
                       k = k+1;

                  dmSubspaceColGet_d(ppDSSDescs[i+2], dFilMins, dFilMaxs,
                                     &ldFilSize);

                  if((*dFilMins)[0] == dMins1[j] &&
                     (*dFilMins)[1] == dMins1[j+1] &&
                     (*dFilMins)[2] == dMins1[j+2] &&
                     (*dFilMaxs)[0] == dMaxs1[j] &&
                     (*dFilMaxs)[1] == dMaxs1[j+1] &&
                     (*dFilMaxs)[2] == dMaxs1[j+2] &&
                     ldFilSize == 3)
                  {
                     m = m+1;
                     j = j+3;
                  }

                  if (ppDSSDescs[i] == dmSubspaceColOpen(tab, colName[i]))
                      n = n+1;
              }
     
              if (k == 2)
                 printf("\n dmSubspaceColCreate_s and "
                        "dmSubspaceColGet_s are working");
              else
                 printf("\n dmSubspaceColCreate_s or "
                        "dmSubspaceColGet_s is not working");

              if (m == 2)
                 printf("\n dmSubspaceColCreate_d and "
                        "dmSubspaceColGet_d are working");
              else
                 printf("\n dmSubspaceColCreate_d or "
                        "dmSubspaceColGet_d is not working");

              if (n == 2)
                 printf("\n dmSubspaceColOpen is working");
              else
                 printf("\n dmSubspaceColOpen is not working");

        dmBlockSetSubspaceCpt(tab, (long)3);
        pstrValTabCols[0] = strDSSCol1;
        pstrValTabCols[1] = strDSSCol2;
        ppDSSDescs[4] = dmSubspaceColCreateTable_l(tab, "TABNAME",
                      "TABDSS",
                      pstrValTabCols, "Unit", lMins, lMaxs, (long)2);

        dmSubspaceColSet_s(ppDSSDescs[0], sMins1, sMaxs1, (long)2);
        dmSubspaceColGet_s(ppDSSDescs[0], sFilMins, sFilMaxs,
                           &lFilSize);
        if((*sFilMins)[0] == sMins1[0] &&
           (*sFilMins)[1] == sMins1[1] &&
           (*sFilMaxs)[0] == sMaxs1[0] &&
           (*sFilMaxs)[1] == sMaxs1[1] &&
           lFilSize == 2)
            printf("\n dmSubspaceColSet_s is working");
         else
            printf("\n dmSubspaceColSet_s is not working");

        dmSubspaceColSet_d(ppDSSDescs[3], intMins1, intMaxs1, (long)2);
        dmSubspaceColUpdate_d(ppDSSDescs[3], intMins2, intMaxs2, (long)1);
        dmSubspaceColGet_d(ppDSSDescs[3], dFilMins, dFilMaxs,
                           &lFilSize);
        if((*dFilMins)[0] == intMins2[0] &&
           (*dFilMins)[1] == intMins1[1] &&
           (*dFilMaxs)[0] == intMaxs1[0] &&
           (*dFilMaxs)[1] == intMaxs2[0] &&
           lFilSize == 2)
            printf("\n dmSubspaceColUpdate_d is working");
        else
            printf("\n dmSubspaceColUpdate_d is not working");

        free(sFilMins);
        free(sFilMaxs);
        free(dFilMins);
        free(dFilMaxs);

        dmTableClose(tab);

        tab = dmTableOpen(newtable);
        tnumber = dmBlockGetNoSubspaceCpts(tab);
        if (tnumber == 3)
           printf("\n dmBlockGetNoSubspaceCpts is working");
        else
           printf("\n dmBlockGetNoSubspaceCpts is not working");

        rnumber1 = dmBlockGetCurrSubspaceCpt(tab);

        dmBlockSetSubspaceCpt(tab, (long)tnumber);
        rnumber2 = dmBlockGetCurrSubspaceCpt(tab);
        if ((rnumber1 == 1) && (rnumber2 == tnumber))
           printf("\n dmBlockSetSubspaceCpt and "
                  "dmBlockGetCurrSubspaceCpts are working");
        else
           printf("\n dmBlockSetSubspaceCpt or "
                  "dmBlockGetCurrSubspaceCpts is not working");

        cnumber = dmBlockGetNoSubspaceCols(tab);
        if (cnumber == 5)
           printf("\n dmBlockGetNoSubspaceCols is working");
        else
           printf("\n dmBlockGetNoSubspaceCols is not working");

        dmTableClose(tab);
   	}

        /*************************************************************/
        /**************** TEST 12: LARGE ARRAY COLUMNS ***************/
        /*************************************************************/
        if (whichTests==12 || whichTests==99)
        {
#		define 	SSZ 15000
#		define	DSZ 10000
#		define	LSZ 5000

		double dvals[DSZ];
		long   lvals[LSZ];
		short  svals[SSZ];
		double dvals2[DSZ];
		long   lvals2[LSZ];
		short  svals2[SSZ];
		short  err=0;

		if (!strcmp(creationKernel,"IRAF"))
		   ERROR("Test case 12 does not work yet with IRAF kernel!");

		dmKernelSetCreate(creationKernel);

		sprintf(tmpStr,"\\rm -rf %s",DATASET);
		if (system(tmpStr))
	  		ERROR("Couldn't remove test dataset!");

	  	sprintf(newtable,"%s[KEYTABLE]",DATASET);
		tab = dmTableCreate(newtable);
		if (!tab)
	  		ERROR("Couldn't create new dataset/table!");

		/* Add Some Columns */
		cols = (dmDescriptor **)calloc(3,sizeof(dmDescriptor*));

		if(!cols)
	  		ERROR("Couldn't allocate memory for new column array!");

		cols[0] = dmColumnCreateArray(tab,COL2,dmDOUBLE,0,"","",DSZ);
		cols[1] = dmColumnCreateArray(tab,COL1,dmSHORT,0,"","",SSZ);
		cols[2] = dmColumnCreateArray(tab,COL4,dmLONG,0,"","",LSZ);

		if (!(cols[0] || cols[1] || cols[2]))
		   ERROR("Couldn't allocate mem for new table columns!");

		for (i=0; i<DSZ; i++)
		{
		    dvals[i] = i+1;
		    dvals2[i] = 0;
		}
		for (i=0; i<SSZ; i++)
		{
		    svals[i] = i+1;
		    svals2[i] = 0;
		}
		for (i=0; i<LSZ; i++)
		{
		    lvals[i] = i+1;
		    lvals2[i] = 0;
		}

		/* write two rows */
		dmSetArray_s(cols[1],svals,SSZ);
		dmSetArray_d(cols[0],dvals,DSZ);
		dmSetArray_l(cols[2],lvals,LSZ);
		dmTablePutRow(tab,NULL);
		dmSetArray_s(cols[1],svals,SSZ);
		dmSetArray_d(cols[0],dvals,DSZ);
		dmSetArray_l(cols[2],lvals,LSZ);
		dmTablePutRow(tab,NULL);

		/* read the first row back */
		dmTableSetRow(tab,1);
		dmGetArray_s(cols[1],svals2,SSZ);
		dmGetArray_d(cols[0],dvals2,DSZ);
		dmGetArray_l(cols[2],lvals2,LSZ);

		/* compare the data arrays */
		for (i=0; i<DSZ; i++)
		    if (dvals[i] != dvals2[i])
		    {
		       printf("\nERROR: dvals[%ld] value s/b %f, but is %f",
			      		i,dvals[i],dvals2[i]);
		       err=1;
		    }
		for (i=0; i<SSZ; i++)
		    if (svals[i] != svals2[i])
		    {
		       printf("\nERROR: svals[%ld] value s/b %hd, but is %hd",
			      		i,svals[i],svals2[i]);
		       err=1;
		    }
		for (i=0; i<LSZ; i++)
		    if (lvals[i] != lvals2[i])
		    {
		       printf("\nERROR: lvals[%ld] value s/b %ld, but is %ld",
			      		i,lvals[i],lvals2[i]);
		       err=1;
		    }

		if (!err)
		   printf("\nLarge array column test was SUCCESSFUL!");
		
		/* go home */
		dmTableClose(tab);
	}

        /*************************************************************/
        /**************** TEST13: COORDINATE CALC AND INVERT TESTS ***/
        /*************************************************************/
         
        if (whichTests==13 || whichTests==99)
        {
		#define MAX_LEN 1024
		dmDataset* dataset;
		char dsname[MAX_LEN];
		char transform[MAX_LEN];
		dmBlock *block;
		char bname[MAX_LEN];
		dmBlockType blocktype;
		dmDescriptor* imageData;
		dmDescriptor* group1;
		dmDescriptor* coord;
		long* axes;
		long naxes,dim_group1,dim;
		long nparams1;
		int rresult;
		void *crpix;
		double *crval,*cdelt;
		long i;
		double dequinox,drotation;
		long lresult;
		long arraydim;
		long *axislengths=NULL;
		double x[5];
		double y[5];
		double result[2];
		int results;
		int creturn,ireturn;
		long nparams[10];	
		char ttype[1024];

		strcpy(dsname,"image.fits\0");	

		if (dmDatasetAccess(dsname,"R")==dmFALSE)
		{
			printf("Cannot Access File\n");
			exit(1);
		}
		dataset=dmDatasetOpen(dsname);	
		if (dataset == NULL)
		{
			printf("dataset is NULL\n");
			exit(2);
		}	
		strcpy(bname,"\0");
		block=dmDatasetNextBlock(dataset);
		if (block == NULL)
		{
			printf("block is NULL\n");
			exit(3);
		}
		blocktype=dmBlockGetType(block);
		if(blocktype==dmIMAGE) printf("blocktype=dmIMAGE\n");
		if(blocktype==dmTABLE) 
		{
			printf("blocktype=dmTABLE\n");
			exit(4);
		}
		if(blocktype==dmUNKNOWNBLOCK) 
		{
			printf("blocktype=dmUNKNOWNBLOCK\n");
			exit(4);
		}
		rresult=dmBlockGetName(block,bname,MAX_LEN);
		printf("rresult=%d block name=%s\n",rresult,bname);
		rresult=dmBlockGetNo(block);
		printf("number of block=%d\n",rresult);
		imageData=dmImageGetDataDescriptor(block);
		naxes=dmGetArrayDimensions(imageData,&axes);
		printf("naxes=%ld \n",naxes);
		group1=dmArrayGetAxisGroup(imageData,1);
		coord=dmDescriptorGetCoord(group1);
		dim_group1=dmGetElementDim(group1);
		printf("dim_group1=%ld\n",dim_group1);
		rresult=dmDescriptorGetNoCoords(imageData);
		if (rresult == dmFAILURE)check_for_errors();
		printf("no coords imageData=%d\n",rresult);
		rresult=dmCoordGetTransformType(coord,transform,&nparams1,
			MAX_LEN);
		if (rresult == dmFAILURE)check_for_errors();
		printf("coord:rresult=%d transform type=%s nparams1=%ld\n",
			rresult,transform,nparams1);
		dim=(long)dmGetElementDim(dmCoordGetParent(coord));
		printf("TRANSFORM DIMENSION: %ld\n",dim);
        	crval = (double*)malloc(dim*sizeof(double));
        	cdelt = (double*)malloc(dim*sizeof(double));
        	rresult=dmCoordGetTransformType(coord,transform,&nparams1,
			MAX_LEN);
		if (rresult == dmFAILURE)check_for_errors();
        	printf("\n\t\tDATA TYPE: ");

        	switch(dmGetDataType(coord))
        	{
        		case dmSHORT:
                		printf("SHORT");
                		crpix = (short*)malloc(dim*sizeof(short));
				rresult=dmCoordGetTransform_s(coord,
					(short*)crpix,crval,cdelt,dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %d",i,
						((short*)crpix)[i]);
                		break;

        		case dmLONG:
                		printf("LONG");
                		crpix = (long*)malloc(dim*sizeof(long));
				rresult= dmCoordGetTransform_l(coord,
					(long*)crpix,crval,cdelt,dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %ld",i,
						((long*)crpix)[i]);
                		break;
        		case dmFLOAT:
                		printf("FLOAT");
                		crpix = (float*)malloc(dim*sizeof(float));
				rresult= dmCoordGetTransform_f(coord,
					(float*)crpix,crval,cdelt,dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %f",i,
						((float*)crpix)[i]);
                		break;
        		case dmDOUBLE:
                		printf("DOUBLE");
                		crpix = (double*)malloc(dim*sizeof(double));
				rresult= dmCoordGetTransform_d(coord,
					(double*)crpix,crval,cdelt,dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %f",i,
						((double*)crpix)[i]);
                		break;
        		case dmBYTE:
                		printf("UNSIGNED BYTE");
                		crpix = (unsigned char*)malloc(
					dim*sizeof(unsigned char));
				rresult=dmCoordGetTransform_ub(coord,
					(unsigned char*)crpix,crval,cdelt,
					dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %u",i,
						((unsigned char*)crpix)[i]);
                		break;
        		case dmUSHORT:
                		printf("UNSIGNED SHORT");
                		crpix = (unsigned short*)malloc(
					dim*sizeof(unsigned short));
				rresult=dmCoordGetTransform_us(coord,
					(unsigned short*)crpix,crval,cdelt,
					dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for(i=0; i<dim; i++)
                   			printf("\n\t\tCRPIX[%ld] is %u",i,
						((unsigned short*)crpix)[i]);
                		break;
        		case dmULONG:
                		printf("UNSIGNED LONG");
                		crpix = (unsigned long*)malloc(
					dim*sizeof(unsigned long));
				rresult= dmCoordGetTransform_ul(coord,
					(unsigned long*)crpix,crval,cdelt,
					dim);
				if (rresult == dmFAILURE)check_for_errors();
                		for (i=0; i<dim; i++)
                    			printf("\n\t\tCRPIX[%ld] is %lu",i,
					((unsigned long*)crpix)[i]);
                		break;
        		default:
                		printf("UNKNOWN");
        	}

        	for (i=0; i<dim; i++)
	    		printf("\n\t\tCRVAL[%ld] is %f",i,crval[i]);
        	for (i=0; i<dim; i++)
            		printf("\n\t\tCRDELT[%ld] is %f",i,cdelt[i]);

        	dmBlockGetName(block,bname,MAX_LEN);
        	printf("\n\t\tNAME OF DESCRIPTOR'S PARENT BLOCK IS: %s\n",
			bname);

		dmKeyRead_d(block,"EQUINOX",&dequinox);
		printf("EQUINOX=%f\n",dequinox);
		dmKeyRead_d(block,"CROTA2",&drotation);
		printf("CROTA2=%f\n",drotation);
		arraydim=dmGetArrayDim(imageData);
		printf("arraydim=%ld\n",arraydim);
		lresult=dmGetArrayDimensions(imageData,&axislengths);
		printf("lresult=%ld\n",lresult);
		if (arraydim>0)
		{
			for (i=0;i<arraydim;i++) 
				printf("axis length [%ld] = %ld\n",i,
					axislengths[i]);
		}
		printf("\n");

		x[0]=1;x[3]=1;y[0]=1;y[1]=1;
		x[1]=axislengths[0];x[2]=axislengths[0];
		y[2]=axislengths[1];y[3]=axislengths[1];
		x[4]=axislengths[0]/2;y[4]=axislengths[1]/2;
		for (i=0;i<5;i++)
		{
			results=dmCoordGetTransformType(coord,ttype,
				nparams,1024);
			{
				short value[2],resultout[2]; 
				value[0]=(short)x[i];
				value[1]=(short)y[i];
				creturn=dmCoordCalc_s(coord,value,result);
				printf("%ld %s dmSHORT  %d %d %f %f %d ",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_s(coord,result,
					resultout);
				printf("     %d %d %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
				long value[2],resultout[2]; 
				value[0]=(long)x[i];
				value[1]=(long)y[i];
				creturn=dmCoordCalc_l(coord,value,result);
				printf("%ld %s dmLONG   %ld %ld %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_l(coord,result,
					resultout);
				printf("     %ld %ld %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
		 		float value[2],resultout[2]; 
				value[0]=(float)x[i];
				value[1]=(float)y[i];
				creturn=dmCoordCalc_f(coord,value,result);
				printf("%ld %s dmFLOAT  %f %f %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_f(coord,result,
					resultout);
				printf("     %f %f %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
		 		double value[2],resultout[2]; 
				value[0]=x[i];
				value[1]=y[i];
				creturn=dmCoordCalc_d(coord,value,result);
				printf("%ld %s dmDOUBLE %f %f %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_d(coord,result,
					resultout);
				printf("     %f %f %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
		 		unsigned char value[2],resultout[2]; 
				value[0]=(unsigned char)x[i];
				value[1]=(unsigned char)y[i];
				creturn=dmCoordCalc_ub(coord,value,result);
				printf("%ld %s dmBYTE   %u %u %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_ub(coord,result,
					resultout);
				printf("     %u %u %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
		 		unsigned short value[2],resultout[2]; 
				value[0]=(unsigned short)x[i];
				value[1]=(unsigned short)y[i];
				creturn=dmCoordCalc_us(coord,value,result);
				printf("%ld %s dmUSHORT %u %u %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_us(coord,result,
					resultout);
				printf("     %u %u %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
			{
		 		unsigned long value[2],resultout[2]; 
				value[0]=(unsigned long)x[i];
				value[1]=(unsigned long)y[i];
				creturn=dmCoordCalc_ul(coord,value,result);
				printf("%ld %s dmULONG  %lu %lu %f %f %d",
					i,ttype,value[0],value[1],
					result[0],result[1],creturn);
				ireturn=dmCoordInvert_ul(coord,result,
				resultout);
				printf("     %lu %lu %d\n",
					resultout[0],resultout[1],ireturn);
				if(!creturn)
				{
					printf("creturn:");
					check_for_errors();
				}
				if(!ireturn)
				{
					printf("ireturn:");
					check_for_errors();
				}
			}
		}

	}


        /*************************************************************/
        /**************** TEST14: TANP TESTS ***/
        /*************************************************************/
         
        if (whichTests==14 || whichTests==99)
        {
         dmBlock* table;
         dmDescriptor* dd;
         dmDescriptor* time_dd;
         dmDescriptor* tanp_dd;
         double crpix[2] = { 4096.0, 4096.0 };
         double crval[3] = { 0.01, 0.02 };
         double detpix[2] = { 4100.0, 3840.0 };
         double skypos[2];
         double nominal_asp[3] = { 60.0, -70.0, 0.0 };
         double asp[3] = { 60.01, -70.04, 135.0 };
         double cdelt[2] = { -1.377E-4, 1.377E-4 };
         char* ddcpts[] = { "DETX", "DETY" };
         char* msc_cpts[]  = { "PHI", "THETA" };
	 long nrows = 4;
         double time[4] = { 100.0, 105.0, 110.0, 115.0 };
         long detpos[8] = {  4096, 4096, 
                               4000, 4200,
                               1000, 4096,
                               7000, 4096 };
         long detvals[2];
         double tvals[2];
         long maxlen = 80;
         char tanp_name[80];
         char ddname[80];
         char disp[20];
         double tlmind, tlmaxd, tnulld, tbind;
         long tlminl, tlmaxl, tnulll, tbinl;
         dmDescriptor* array_dd;
         long adim = 5;
         long array_dim[4] = { 1, 5, 0, 2 };
         double retarray[5];
         double array_data[4][5] = {
           { 11., 99., 99., 99., 99. }, { 55., 56., 57., 58., 59. },
           { 99., 99., 99., 99., 99. }, { 22., 23., 99., 99., 99. }
         };

         printf( "TANP TEST\n" );
	 unlink( "tanp.fits" );
         table = dmTableCreate( "tanp.fits[EVENTS]" );
          
         if ( table ) 
          printf( "CREATED FILE TANP.FITS\n" );
         else
          printf( "Table Create failed\n" );

         time_dd = dmColumnCreate( table, "TIME", dmDOUBLE, 0, "s", "Mission Time" );
	 dmSetDisp( time_dd, "F20.4" );
         dd = dmColumnCreateVector( table, "DET", dmLONG, 0, "pixel", "Detector Pos",
                ddcpts, 2 );
         if ( dd ) printf( "CREATED VECTOR DET\n" );
         tanp_dd = dmCoordCreate_d( dd, "MSC", "deg", msc_cpts, 2, "TAN-P", 
                crpix, crval, cdelt, NULL );
         if ( tanp_dd ) printf( "CREATED TAN-P COORD MSC\n" );
         dmDescriptorSetBin_d( time_dd, 10.0 );
         dmDescriptorSetNull_d( time_dd, -1.0 );
	 dmDescriptorSetRange_l( dd, 1, 8192 );
         dmDescriptorSetNull_l( dd, -999 );
         dmDescriptorSetBin_l( dd, 32 );



         dmTanApplyAspect( detpix, nominal_asp, asp, crpix, cdelt, skypos );
         printf( "DET = %10.3f %10.3f SKY = %10.3f %10.3f\n", detpix[0], detpix[1], skypos[0], skypos[1] );
         array_dd = dmColumnCreateVarArray( table, "VAR", dmDOUBLE, 0, "s", "Variable array", adim );
         adim = 0;
         for ( i = 0; i < nrows; i++ )
         {
          dmSetScalar_d( time_dd, time[i] );
          dmSetVector_l( dd, detpos+2*i, 2 );
          dmSetArray_d( array_dd, array_data[i], array_dim[i] );
          dmTablePutRow( table, NULL );
         }
         printf( "WROTE ROWS\n" );
         dmTableClose( table );
         printf( "CLOSED TABLE\n" );
         table = dmTableOpen( "tanp.fits[EVENTS]" );
         if ( table ) printf( "OPENED TABLE\n" );
         dd = dmTableOpenColumn( table, "TIME" );
         if ( dd )
         {
           dmGetName( dd, ddname, maxlen );
           dmGetDisp( dd, disp, 20 );
           printf( "OPENED COLUMN %s DISP=%s\n", ddname, disp );
         }
         if ( dmDescriptorGetRange_d( dd, &tlmind, &tlmaxd ) )
          printf( "%s RANGE = %f:%f\n", ddname, tlmind, tlmaxd );
         else
          printf( "%s RANGE: None\n", ddname );

         if ( dmDescriptorGetNull_d( dd, &tnulld ) )
          printf( "%s NULL = %f\n", ddname, tnulld );
         else
          printf( "%s NULL: None\n", ddname );

         if ( dmDescriptorGetBin_d( dd, &tbind ) )
          printf( "%s BIN = %f\n", ddname, tbind );
         else
          printf( "%s BIN: None\n", ddname );


	 array_dd = dmTableOpenColumn( table, "VAR" );
         if ( array_dd )
         {
           dmGetName( array_dd, ddname, maxlen );
           adim = dmGetArraySize( array_dd );
           printf( "OPENED COLUMN %s ARRAY SIZE = %ld\n", ddname, adim );
         }

         dd = dmTableOpenColumn( table, "DET" );
         if ( dd ) {
           dmGetName( dd, ddname, maxlen );
           printf( "OPENED COLUMN %s\n", ddname );
         }


         if ( dmDescriptorGetRange_l( dd, &tlminl, &tlmaxl ) )
          printf( "%s RANGE = %ld:%ld\n", ddname, tlminl, tlmaxl );
         else
          printf( "%s RANGE: None\n", ddname );

         if ( dmDescriptorGetNull_l( dd, &tnulll ) )
          printf( "%s NULL = %ld\n", ddname, tnulll );
         else
          printf( "%s NULL: None\n", ddname );

         if ( dmDescriptorGetBin_l( dd, &tbinl ) )
          printf( "%s BIN = %ld\n", ddname, tbinl );
         else
          printf( "%s BIN: None\n", ddname );


         tanp_dd = dmDescriptorGetCoord( dd );
         if ( tanp_dd ) {
          dmGetName( dd, tanp_name, maxlen );
          printf( "OPENED COORD %s\n", tanp_name );
         }
         nrows = dmTableGetNoRows( table );
         printf( "TABLE HAS %ld ROWS\n", nrows );
         for ( i = 0; i < nrows; i++ )
         {
          double vl;
          dmGetVector_l( dd, detvals, 2 );
          printf( "ROW %6ld DET = %6ld %6ld \n", i, detvals[0], detvals[1] );
          dmCoordCalc_l( tanp_dd, detvals, tvals );
          vl = ( tvals[1] ) * 60.0;
          printf( "ROW %6ld CCMSC = %20.5f %20.5f %20.5f\n", i, tvals[0], tvals[1], vl );         
          dmGetVector_d( tanp_dd, tvals, 2 );
          printf( "ROW %6ld GVMSC = %20.5f %20.5f\n", i, tvals[0], tvals[1] );         
          dmGetArray_d( array_dd, retarray, 5 );
          printf( "ROW %6ld ARRAY = %10.5f %10.5f %10.5f %10.5f %10.5f\n",
              i, retarray[0], retarray[1], retarray[2], retarray[3], retarray[4] );
          dmTableNextRow( table );
         }
         printf( "TANP TEST COMPLETE\n" );
        }


        if (whichTests==15 )
        {
	 dmBlock* image;
         dmDescriptor* imagedata;
	 dmDescriptor* dd;
	 dmDescriptor* world;
	 char name[80];
	 char wname[80];
         long dim;
	 long wdim;
	 double inpix[2] = { 100.0, 100.0 };
	 double outpix[2];
	 image = dmImageOpen( "aa.fits" );
         imagedata = dmImageGetDataDescriptor( image );
         dd = dmArrayGetAxisGroup( imagedata, 1 );
	 if ( dd ) world = dmDescriptorGetCoord( dd );
         if ( dd && world ) {
	  dim = dmGetElementDim( dd );
          dmGetName( dd, name, 80 );
	  wdim = dmGetElementDim(world );
          dmGetName( world, wname, 80 );
          printf( "AXIS %s %ld WORLD %s %ld\n", name, dim, wname, wdim );
	  if ( dmCoordCalc_d( dd, inpix, outpix ) )
 	   printf( "INPIX %f %f OUTPIX %f %f\n", inpix[0], inpix[1], outpix[0], outpix[1] );
          else
	   printf( "CC ERROR\n" );
         }
	 dmImageClose( image );
	}

        if (whichTests==16 )
        {

         dmBlock* block;
         dmDescriptor* cols[4];
         long np =4;
         long phas[4] = { 20, 10, 0, 2 };
         long row;
         long offset=0;
         long n;
         double vals[50];
         double* dptr = NULL;
         long nvals[] = { 5, 2, 10, 3, 1, 6, 10, 4, 2, 2 };
         block = dmTableCreate( "trmf.fits[RMF]" ); 
         if ( !block )
           printf( "Failed to open output file trmf.fits\n" );
         else
         {
         cols[0] = dmColumnCreate( block, "CHAN", dmLONG, 0, "chan", "Chan" );
         dmKernelSetOption( "VARARRAY" );

         for ( offset = 0; offset < 50; offset++ ) vals[offset] = offset+1.0;

         
         cols[1] = dmColumnCreateArray( block, "RSP", dmDOUBLE, 0, " ", " ", 10 );
         cols[2] = dmColumnCreateArray( block, "PHAS", dmLONG, 0, " ", " ", 4 );
         offset = 0;  /* Reset offset */
         for ( row = 1; row <= 2; row++ )
         {
           dmSetScalar_l( cols[0], row );
           n = nvals[row-1];
           dptr = &vals[offset];
           dmSetArray_d( cols[1], dptr, n );
           offset += n;
           phas[2] = row;   /* Label PHAs array */
           dmSetArray_l( cols[2], phas, np );
           dmTablePutRow( block, NULL );
         }
         dmTableClose( block );
         }
        }	

/* Test 17 and 16 should be the same. But 16 doesn't work: why? */

        if (whichTests==17 )
        {

         int status;
         fitsfile* fptr;
         long phas[4] = { 20, 10, 0, 2 };
         long row;
         long offset=0;
         long n;
         double vals[50];
         double* dptr = NULL;
         char* ttype[3] = {"CHAN", "RSP", "PHAS"};
         char* tform[3] = {"1J", "1PD(10)", "4J"};
         char* tunit[3] = {"km", "", ""};
         long nrows=0;
         long nvals[] = { 5, 2, 10, 3, 1, 6, 10, 4, 2, 2 };
         for ( offset = 0; offset < 50; offset++ ) vals[offset] = offset+1.0;

         fits_create_file( &fptr, "frmf.fits", &status );
         fits_create_tbl( fptr, BINARY_TBL, nrows, 3, ttype, tform, tunit, "RMF", &status );
         offset = 0;  /* Reset offset */
         for ( row = 1; row <= 10; row++ )
         {
           n = nvals[row-1];
           dptr = &vals[offset];
           fits_write_col_lng( fptr, 1, row, 1, 1, &row, &status );         
           fits_write_col_dbl( fptr, 2, row, 1, n, dptr, &status );
           phas[2] = row;   /* Label PHAs array */
           fits_write_col_lng( fptr, 3, row, 1, 4, phas, &status );
           offset += n;
         }
         fits_close_file( fptr, &status );
        }




        printf("\n");
	/* Get and print the DM version */
	dmGetVersion(tmpStr,256);
	fflush(NULL);
	fprintf(stderr,"\n\nCURRENT DATAMODEL VERSION: %s\n",tmpStr);

	return dmSUCCESS;
}

void check_for_errors(void)
{
	#define MAX_LEN 1024
	long errornumber;
	char errMessage[MAX_LEN];

	errornumber=dmGetError();
	dmGetErrorMessage(errMessage,MAX_LEN);
	printf("error:%ld errMessage:%s\n",errornumber,errMessage);
	exit(0);
}
void PrintHelp()
{
  	printf("\ndmtest: AXAF Science Center DataModel Test Program");
	printf("\nSupported command-line options are:");
	printf("\n	-c=##		run the test case specified by ##");
	printf("\n\t\tValid test cases are:");
	printf("\n\t\t1: list general info for all blocks in given DS");
	printf("\n\t\t2: create new DS and run header keyword tests");
	printf("\n\t\t3: create/populate new table in given DS");
	printf("\n\t\t4: like (3), but uses row-based I/O on full table");
	printf("\n\t\t5: use row-based I/O on subset of existing table");
	printf("\n\t\t   (also tests casting of all cols to one datatype)");
	printf("\n\t\t6: tests general image creation");
	printf("\n\t\t7: restricted filtered table I/O test");
	printf("\n\t\t   (table must contain columns as specified)");
	printf("\n\t\t8: restricted image I/O test");
	printf("\n\t\t9: block deletion test");
        printf("\n\t\t11: subspace test");
        printf("\n\t\t12: very large array columns");
        printf("\n\t\t13: coordinate calc and coordinate invert");
	printf("\n	-d=name		use the specified dataset (DS)");
	printf("\n	-f=\"filtexpr\"\tquoted filter expr for test case 7");
	printf("\n\t\tHas form: filtexpr=\"[block specifier][filters]\"");
	printf("\n	-h		print this help message");
        printf("\n      -i              specify blank image name for case 6");
	printf("\n	-k=ker		set DM kernel for DS creation");
	printf("\n			(specify either 'q' or 'f'");
	printf("\n	-p		dump keys during test case 1");
	printf("\n	-r		print col ranges during test case 1");
	printf("\n	-C		dump coords during test case 1");
	printf("\n	-t=name		use the specified table");
	printf("\n");
	exit(0);
}
