/*                                                                
**  Copyright (C) 1999-2000,2003-2004,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmdssaux.c                                                  */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmlibP.h"

dmDescriptor* dmp_subspace_col_create_table( dmBlock *block, char* tabname, char* name, dmDataType type,
			   char** colNames, char* unit, void* value1, 
			   void* value2, long nvalues );



/********************************************************************/
/*                                                                  */
/* Function:   dmSubspaceColCreate_*                                */
/*                                                                  */
/* Description:                                                     */
/*      Create subspace column given input datatype.                */
/*                                                                  */
/*                                                                  */
/*  Input parameters:                                               */
/*             dmBlock      *block                                  */
/*             char         *name                                   */
/*             char         *unit                                   */
/*            <datatype>    *value1                                 */
/*            <datatype>    *value2                                 */
/*             long          nvalues                                */
/*                                                                  */
/*  Output parameters:                                              */
/*              none                                                */
/*                                                                  */
/*  Returns:                                                        */
/*           dmDescriptor *data_descriptor                          */
/*                                                                  */
/********************************************************************/
dmDescriptor*  dmSubspaceColCreate_s(dmBlock *block, char* name, char* unit,
		                     short* value1, short* value2, long nvalues)
{
  dmDataType type     = DT_INT2;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreate_l(dmBlock *block, char* name, char* unit,
	  	                     long* value1, long* value2, long nvalues)
{
  dmDataType type     = DT_INT4;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreate_us(dmBlock *block, char* name, char* unit,
		 unsigned short* value1, unsigned short* value2, long nvalues)
{
  dmDataType type     = DT_UINT2;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreate_ul(dmBlock *block, char* name, char* unit,
		 unsigned long* value1, unsigned long* value2, long nvalues)
{
  dmDataType type     = DT_UINT4;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreate_f(dmBlock *block, char* name, char* unit,
		                     float* value1, float* value2, long nvalues)
{
  dmDataType type     = DT_REAL4;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreate_d(dmBlock *block, char* name, char* unit,
		                     double* value1, double* value2, long nvalues)
{
  dmDataType type     = DT_REAL8;
  char      *tabname  = NULL;
  char     **colNames = NULL;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


/********************************************************************/
/*                                                                  */
/* Function:   dmSubspaceColCreateTable_*                           */
/*                                                                  */
/* Description:                                                     */
/*      Create a subspace column given input datatype, with values  */
/*      stored in separate tables (value tables).  Table has name   */
/*      tabname and two columns named colNames[0] and colNames[1].  */
/*                                                                  */
/*      If the table name is null, create a normal subspace column. */
/*                                                                  */
/*  Input parameters:                                               */
/*             dmBlock      *block                                  */
/*             char         *tabname                                */
/*             char         *name                                   */
/*             char        **colNames                               */
/*             char         *unit                                   */
/*            <datatype>    *value1                                 */
/*            <datatype>    *value2                                 */
/*             long          nvalues                                */
/*                                                                  */
/*  Output parameters:                                              */
/*              none                                                */
/*                                                                  */
/*  Returns:                                                        */
/*           dmDescriptor *data_descriptor                          */
/*                                                                  */
/********************************************************************/
dmDescriptor*  dmSubspaceColCreateTable_s(dmBlock *block, char* tabname, char* name,
			                  char** colNames, char* unit, short* value1, 
			                  short* value2, long nvalues)
{
  dmDataType type = DT_INT2;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}

dmDescriptor*  dmSubspaceColCreateTable_l(dmBlock *block, char* tabname, char* name,
	  		                  char** colNames, char* unit, long* value1, 
			                  long* value2, long nvalues)
{
  dmDataType type = DT_INT4;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreateTable_us(dmBlock *block, char* tabname, char* name,
			                   char** colNames, char* unit, unsigned short* value1, 
			                   unsigned short* value2, long nvalues)
{
  dmDataType type = DT_UINT2;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}

dmDescriptor*  dmSubspaceColCreateTable_ul(dmBlock *block, char* tabname, char* name,
			                   char** colNames, char* unit, unsigned long* value1, 
			                   unsigned long* value2, long nvalues)
{
  dmDataType type = DT_UINT4;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor* dmSubspaceColCreateTable_f(dmBlock *block, char* tabname, char* name,
			                 char** colNames, char* unit, float* value1, 
			                 float* value2, long nvalues)
{
  dmDataType type = DT_REAL4;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}


dmDescriptor*  dmSubspaceColCreateTable_d( dmBlock *block, char* tabname, char* name,
			                   char** colNames, char* unit, double* value1, 
			                   double* value2, long nvalues)
{
  dmDataType type = DT_REAL8;
  return dmp_subspace_col_create_table( block, tabname, name, type, colNames, unit, value1, value2, nvalues );
}

/********************************************************************/
/*                                                                  */
/* Function:   dmp_subspace_col_create_table                        */
/*                                                                  */
/* Description:                                                     */
/*      Create a subspace column given input datatype, with values  */
/*      stored in separate tables (value tables).  Table has name   */
/*      tabname and two columns named colNames[0] and colNames[1].  */
/*                                                                  */
/*      If the table name is null, create a normal subspace column. */
/*                                                                  */
/*  Input parameters:                                               */
/*             dmBlock      *block                                  */
/*             char         *tabname                                */
/*             char         *name                                   */
/*             dmDataType    type                                   */
/*             char        **colNames                               */
/*             char         *unit                                   */
/*             void         *value1                                 */
/*             void         *value2                                 */
/*             long          nvalues                                */
/*                                                                  */
/*  Output parameters:                                              */
/*              none                                                */
/*                                                                  */
/*  Returns:                                                        */
/*           dmDescriptor *data_descriptor                          */
/*                                                                  */
/********************************************************************/
dmDescriptor* dmp_subspace_col_create_table( dmBlock *block, char* tabname, 
                           char* name, dmDataType type,
			   char** colNames, char* unit, void* value1, 
			   void* value2, long nvalues )
{
  dmDescriptor *dd     = NULL;
  regRegion    *region = NULL;

  long vdim   = 1;
  long valdim = 1;
  int  array  = 0;

  if ( !block ) 
    return NULL;

  dd = dmi_subspace_col_create( block, block->subspace, name, colNames, vdim, type, unit, NULL, UT_TRUE );
  dmi_subspace_col_set_values( dd, tabname, value1, value2, nvalues, type, valdim, array, region );
  return dd;
}


/*--------------------------------------------------------------------------
  dmSubspaceColSet routines.


 --------------------------------------------------------------------------*/
/********************************************************************/
/*                                                                  */
/* Function:   dmi_subspace_col_update_null                         */
/*                                                                  */
/* Description:                                                     */
/*    Set the values for the given descriptor in the current        */
/*    component.  Previous values are overwritten.                  */
/*                                                                  */
/*  Input parameters:                                               */
/*             dmDescriptor *dd                                     */
/*             char         *name                                   */
/*             long          cptNo                                  */
/*             dmDataType    type                                   */
/*             char        **colNames                               */
/*             void         *value1                                 */
/*             void         *value2                                 */
/*             long          nvalues                                */
/*             dmBool        update                                 */
/*                                                                  */
/*  Output parameters:                                              */
/*              none                                                */
/*                                                                  */
/*  Returns:                                                        */
/*           int flag                                               */
/*                                                                  */
/********************************************************************/
int  dmi_subspace_col_update_null(dmDescriptor* dd, char* name, long cptNo,
                                  dmDataType type, void* value1,
				  void* value2, long nvalues, dmBool update )
{
  char *nullname = "NULL";
  char *tname    = name;

  if ( nvalues == 0 && !tname )
    tname = nullname;

  /* cptNo is vector cpt */
  return dmi_subspace_col_update_cpt( dd, tname, cptNo, type, value1, value2, nvalues, 0, update ); 
}


