/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*
 *  Interface to raw numbered blocks
 */
#include <unistd.h>
#include "dmilib.h"
#include "ftkernel.h"
#include "ftkernelP.h"

dmErrCode ftBlockDelete( dmDataset* ds, const long blockNo )
{
 
 ftDataset* ftds;
 fitsfile* fptr;
 int ftstatus = FITS_KERNEL_OK;
 int fthdutype;
 long ftBlockNo;           /* FITS HDU number */
 long newBlockNo;
 long i;
 ftBlock* ftblock;
 dmBlock* block;
 int decrement = UT_TRUE;
 long nb;

 if ( !dmi_dataset_mode( ds ) )  /* WRITE TO FILE */
  return dmFAILURE;
 

 ftds = (ftDataset*)dmi_dataset_get_kds( ds );
 fptr = ftds->fptr;

 ftBlockNo = blockNo;   /* In current implementation, 
                              should be same as blockNo (dm level) */
 if ( ftBlockNo == 1 )
 {
  decrement = ftp_delete_primary_hdu( ds ); 
 } else {
  ftds->current = ftBlockNo;
  dmi_dataset_set_current( ds, NULL );
  fits_movabs_hdu( fptr, ftds->current, &fthdutype, &ftstatus );  /* CFITSIO */
  if ( ftstatus == FITS_KERNEL_OK )
   fits_delete_hdu( fptr, NULL, &ftstatus );			/* CFITSIO */
  if ( ftstatus == FITS_KERNEL_OK )
   fits_flush_file( fptr, &ftstatus );				/* CFITSIO */

  ftp_ds_status( ds, ftstatus, "[ftBlockDelete]" );
 }

 /* Update the remaining block numbers in both the block and ds structures */

/*  update block numbers */
 for ( i = ftBlockNo; i < ftds->n; i++ )
  ftds->hdutype[i-1] = ftds->hdutype[i];  
 ftds->n--;

 nb = dmi_dataset_get_nv( ds );
 for ( i = 1; i <= nb; i++ )
 {
  block = dmi_dataset_get_block( ds, i );
  if ( block )
  {
   ftblock  = (ftBlock*)dmi_block_get_kb( block );
   if ( ftblock )
   {
    newBlockNo = ftblock->hdu - 1;
    ftblock->hdu = newBlockNo;
   } else {
    printf( "NO FTBLOCK FOR %ld %s\n", i, dmi_block_get_name( block )); 
   }
  }
 }
 if ( decrement )
  return dmi_block_delete( ds, blockNo );
 else  /* Success, but number of blocks remained the same */
  return dmFAILURE;
}

