/*                                                                
**  Copyright (C) 2002-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   ftkernel.h                                                  */
/*                                                               */
/*   fits kernel function prototypes                             */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmilib.h"
#include "dmlib.h"
#include "dmkernel.h"

void      ftInit( void );
void      ftKeyDelete( dmDescriptor* dd );
dmErrCode ftBlockFlush( dmBlock* block );
void      ftBlockUpdateFilter( dmBlock* block, dmSubspace* rts );
void      ftImageSetAxes( dmBlock* block, const dmDataType type, const long* axlen, const long naxes );
void      ftColCreate( dmBlock* block, dmDescriptor* dd, void* kdd, char* hint );
dmErrCode ftColRead( dmDescriptor* dd, long* llpix, long* urpix, long* dpix, long row1, long row2, void* data );
dmErrCode ftColWrite( dmDescriptor* vector, long* llpix, long* urpix, long* dpix, long row1, long row2, void* data );
dmBool    ftDatasetAccess( const char* filename, const char* mode );
void      ftDescriptorCopy( dmDescriptor* srcdd, dmDescriptor* destdd );
void      ftDescriptorFree( dmDescriptor* dd );
dmDataset* ftDatasetOpen(const char* filename, const char* hint, const char* mode );
dmErrCode ftSetHint( dmBlock* block, const char* name, const char* value );
void      ftDatasetPrint( const dmDataset* ds );
void      ftDatasetClose( dmDataset* ds );
void      ftDatasetDelete( dmDataset* ds );

dmDataset*  ftDatasetCreate( const char* filename, const char* hint );
dmBlock* ftBlockOpen( dmDataset* ds, long BlockNo, dmBlockType type, char* hint );
dmBlock*   ftBlockCreate( dmDataset* ds, long no, const char* name, dmBlockType type, void* kb, char* hint );
dmErrCode ftBlockClose( dmBlock* block );
dmErrCode ftBlockDelete( dmDataset* ds, const long blockNo );
void ftGetKernelKey( dmBlock* block, long no, char* buf, long maxlen );
long ftGetNoKernelKeys( dmBlock* block );
dmBool      ftWriteComment( dmBlock* block, const long keyno, const char* tag, const char* comment );
dmBool      ftReadComment( dmBlock* block, const long keyno, char** tag, char** comment );
dmDescriptor*    ftKeyWrite( dmBlock* block, const long keyno, const char* name, const dmDataType type, const void* value, long nvals,              const char* unit, const char* comment );
dmErrCode ftColSetRange( dmDescriptor* ddp );
dmErrCode ftColSetNull( dmDescriptor* dd );
dmErrCode ftColSetBin( dmDescriptor* dd );
void ftSubspaceDescriptorWrite( dmDescriptor* dd, long cptno, char* value, char* tabname );
void ftSubspaceTableWrite( dmDescriptor* dd, char* tabname, void* val1, void* val2, long nvals );
void ftCoordWrite( dmDescriptor* dd );
void ftBlockSetPref( dmBlock* block, char** prefcols, const long npref );
long ftBufferSize( dmBlock* block );
void ftColFlush( dmDescriptor* dd );
