/*                                                                
**  Copyright (C) 1997-2001,2003-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   ascdm.h                                                     */
/*                                                               */
/*   This file contains prototypes for ASC DataModel API         */
/*   functions.                                                  */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#ifndef ASCDM_H
#define ASCDM_H

#ifdef __cplusplus
extern "C" {
#endif


#include "dmerror.h"
#include "cxcregion.h"


/*-------------------------------------------------*/
/*			dmDataset		   */
/*-------------------------------------------------*/
dmBool			dmDatasetAccess(const char*, const char*);
dmDataset*		dmDatasetCreate(const char*);
dmDataset* 		dmDatasetOpen(const char*);
dmDataset* 		dmDatasetOpenUpdate(const char*);
dmErrCode		dmDatasetClose(dmDataset*);

dmErrCode		dmDatasetGetName(const dmDataset*,char *name,long maxL);
dmErrCode               dmDatasetRename( const char *oldname, const char *newname );
dmErrCode		dmDatasetPrint(const dmDataset*, char*);
dmErrCode		dmDatasetGetBlockName(const dmDataset*,const long blNo,char *name, const long maxlen);
dmBlockType 		dmDatasetGetBlockType(const dmDataset*, const long);
long 			dmDatasetGetNoBlocks(const dmDataset*);
long 			dmDatasetGetCurrentBlockNo(const dmDataset*);
dmBlock* 		dmDatasetNextBlock(dmDataset*);
dmBlock* 		dmDatasetMoveToBlock(dmDataset*, long);
dmBlock* 		dmDatasetAdvanceBlocks(dmDataset*, long);
dmErrCode		dmDatasetDelete(dmDataset* ds);
dmBlock*                dmBlockOpenMode( char* vspec, dmBool update );
dmBool                  dmDatasetWriteable( dmDataset *dataset);

/*-------------------------------------------------*/
/*			dmBlock			   */
/*-------------------------------------------------*/

dmBlock *
dmBlockCreateGenCopy(dmDataset *destDataset, char *destBlockName,
		  dmBlock *srcBlock, char* hist, dmBool copyData, dmBool copySubspace );
dmBlock*		dmBlockCreate(dmDataset *pDs,char *pBlName, dmBlockType blType);
dmErrCode		dmBlockDelete(dmBlock *pBl);
dmErrCode		dmBlockClose(dmBlock *pBl);
dmBlock*		dmBlockOpen(dmDataset *pDs,const char *pBlName);
dmBlockType		dmBlockGetType(const dmBlock *pBl);
dmDataset*		dmBlockGetDataset(const dmBlock *pBl);
long			dmBlockGetNo(const dmBlock *pBl);

dmDescriptor**   	dmBlockGetPref( const dmBlock* block, dmDescriptor** weight, long* npref );
dmErrCode             	dmBlockSetPref( dmBlock* block, char* buffer );
char*            	dmBlockGetTypeStr( const dmBlock* block );

dmErrCode		dmBlockGetName(const dmBlock *pBl, char *blName, long maxlen);
dmErrCode       	dmBlockReadComment(const dmBlock *pBl, char **tag, char **cmnt);
dmErrCode		dmBlockWriteComment(dmBlock *pBl, const char *tag, const char *cmnt);
dmBlock *       	dmBlockCreateCopy(dmDataset *destDataset, char *destBlockName,
				  dmBlock *srcBlock, dmBool copyData);
dmErrCode               dmBlockCopy( dmBlock* srcBlock, dmBlock* destBlock, char* option );
void                    dmBlockSetBufferMaxSize( dmBlock* block, long size );

/*-------------------------------------------------*/
/*		   table dmBlock		   */
/*-------------------------------------------------*/

dmBlock* 		dmTableCreate(char* vspec);

dmBlock* 		dmTableOpen(char* vspec);
dmBlock* 		dmTableOpenUpdate(char* vspec);
dmErrCode			dmTableClose(dmBlock* table);
dmBlock*		dmDatasetCreateTable(dmDataset *ds,const char *blName);

dmBlock* 		dmTableOpenSelect(dmDataset *pDs, char *tabname, char *cols,
				  		    dmDataType *castToType);
long	 		dmTableGetRowNo(dmBlock* pTable);
long	 		dmTableGetNoRows(dmBlock* pTable);
long                    dmTableNextRow(dmBlock* pTable);
long                    dmTableSetRow(dmBlock* pTable, long row);
long	 		dmTablePutRow(dmBlock* pTable, void *rowStruct);
long 			dmTableGetRow(dmBlock* pTable, void *rowStruct);
void **         	dmTableGetRows(dmBlock *tab, long *nrows);

void*			dmTableAllocRow(dmBlock *tab);

dmDescriptor*		dmTableOpenColumn(const dmBlock* pTable, const char* colname);
dmDescriptor*		dmTableOpenColumnNo(const dmBlock* pTable, const long colno);
dmDescriptor**		dmTableOpenColumnList(const dmBlock* pTable, long* numcols);
long			dmTableGetNoCols(const dmBlock* pTable);

/*-------------------------------------------------*/
/*		   image dmBlock		   */
/*-------------------------------------------------*/

dmBlock* 		dmImageCreate(char* vspec,dmDataType,long *axes,long naxes);

dmBlock*	 	dmImageOpen(char* vspec);
dmBlock*	 	dmImageOpenUpdate(char* vspec);
dmErrCode	 	dmImageClose(dmBlock* table);
dmBlock*		dmDatasetCreateImage(dmDataset *ds,const char *blName,
				     const dmDataType dType, long *axLens,const long naxes);

dmDescriptor*		dmImageGetDataDescriptor(dmBlock *pImage);
dmDescriptor* 		dmArrayCreateAxisGroup(dmDescriptor *pImage,char* name,
				dmDataType iType,char *unit, char** cptNames,
				long dim);

dmDescriptor*		dmArrayGetAxisGroup(const dmDescriptor *pImage,const long axisGrpNo);
long			dmArrayGetNoAxisGroups(const dmDescriptor *pImData);

/*------------------------------*/
/*     get image subsection 	*/
/*------------------------------*/

dmErrCode		dmImageDataGetSubArray_s(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                short* values);
dmErrCode		dmImageDataGetSubArray_l(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                long* values);
dmErrCode		dmImageDataGetSubArray_f(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                float* values);
dmErrCode		dmImageDataGetSubArray_d(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                double* values);
dmErrCode		dmImageDataGetSubArray_ub(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned char* values);
dmErrCode		dmImageDataGetSubArray_us(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned short* values);
dmErrCode		dmImageDataGetSubArray_ul(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned long* values);
/*------------------------------*/
/*     set image subsection 	*/
/*------------------------------*/

dmErrCode		dmImageDataSetSubArray_s(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                short* values);
dmErrCode		dmImageDataSetSubArray_l(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                long* values);
dmErrCode		dmImageDataSetSubArray_f(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                float* values);
dmErrCode		dmImageDataSetSubArray_d(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                double* values);
dmErrCode		dmImageDataSetSubArray_ub(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned char* values);
dmErrCode		dmImageDataSetSubArray_us(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned short* values);
dmErrCode		dmImageDataSetSubArray_ul(dmDescriptor* data, long* pixelLl, long* pixelUr,
                                                unsigned long* values);

/*------------------------------*/
/*     get image pixel value 	*/
/*------------------------------*/

short			dmImageDataGetPixel_s(dmDescriptor *imdata, long *pixelNo, long dim);
long			dmImageDataGetPixel_l(dmDescriptor *imdata, long *pixelNo, long dim);
float 			dmImageDataGetPixel_f(dmDescriptor *imdata, long *pixelNo, long dim);
double 			dmImageDataGetPixel_d(dmDescriptor *imdata, long *pixelNo, long dim);
unsigned char 		dmImageDataGetPixel_ub(dmDescriptor *imdata, long *pixelNo, long dim);
unsigned short		dmImageDataGetPixel_us(dmDescriptor *imdata, long *pixelNo, long dim);
unsigned long 		dmImageDataGetPixel_ul(dmDescriptor *imdata, long *pixelNo, long dim);

/*------------------------------*/
/*     set image pixel value 	*/
/*------------------------------*/

dmErrCode		dmImageDataSetPixel_s(dmDescriptor *imdata, long *pixelNo, short val);
dmErrCode		dmImageDataSetPixel_l(dmDescriptor *imdata, long *pixelNo, long val);
dmErrCode		dmImageDataSetPixel_f(dmDescriptor *imdata, long *pixelNo, float val);
dmErrCode		dmImageDataSetPixel_d(dmDescriptor *imdata, long *pixelNo, double val);
dmErrCode		dmImageDataSetPixel_ub(dmDescriptor *imdata, long *pixelNo, unsigned char val);
dmErrCode		dmImageDataSetPixel_us(dmDescriptor *imdata, long *pixelNo, unsigned short val);
dmErrCode		dmImageDataSetPixel_ul(dmDescriptor *imdata, long *pixelNo, unsigned long val);

/*-------------------------------------------------*/
/*		     dmDescriptor		   */
/*-------------------------------------------------*/

/*--------------------------------------*/
/*      descriptor property reading	*/
/*--------------------------------------*/
char* dmGetDataTypeName( const dmDescriptor* dd );
dmDescriptorType 	dmDescriptorGetType(const dmDescriptor *pDesc);
dmBlock*	 	dmDescriptorGetBlock(const dmDescriptor *pDesc);
long			dmDescriptorGetLength(const dmDescriptor *pDesc);
dmErrCode		dmGetName(const dmDescriptor *dd, char *descName, long maxlen);
dmDataType		dmGetDataType(const dmDescriptor *dd);
dmElementType		dmGetElementType(const dmDescriptor *dd);
dmErrCode	        dmGetDisp(const dmDescriptor *dd, char *desc, long maxlen);
long			dmGetElementDim(const dmDescriptor *dd);
dmErrCode		dmGetDesc(const dmDescriptor *dd, char *descrip, long maxlen);
dmErrCode		dmGetUnit(const dmDescriptor *dd, char *unit, long maxlen);
long			dmGetArrayDim(const dmDescriptor *dd);
long			dmGetArraySize(const dmDescriptor *dd);
long			dmGetCurrentArraySize(const dmDescriptor* dd);
long			dmGetArrayDimensions(const dmDescriptor *dd, long **axisLengths);
dmErrCode		dmGetCptName(const dmDescriptor *dd,const long cptNo,char *name,long maxlen);
dmDescriptor*   	dmGetCpt(const dmDescriptor *dd, long cptNo);
double                  dmDescriptorGetArea( const dmDescriptor* dd, double* field_area );
dmDescriptor*   	dmDescriptorGetCoord(const dmDescriptor* dd);
dmDescriptor*   	dmDescriptorGetCoordNo(const dmDescriptor* dd, const long num);
long             	dmDescriptorGetNoCoords(const dmDescriptor* dd);
dmErrCode            	dmDescriptorGetRange_s(dmDescriptor* pDesc, short* tlmin, 
				       short* tlmax);
dmErrCode            	dmDescriptorGetRange_l(dmDescriptor* pDesc, long* tlmin, 
				       long* tlmax);
dmErrCode            	dmDescriptorGetRange_f(dmDescriptor* pDesc, float* tlmin, 
				       float* tlmax);
dmErrCode            	dmDescriptorGetRange_d(dmDescriptor* pDesc, double* tlmin, 
				       double* tlmax);

dmErrCode            	dmDescriptorGetRange_ub(dmDescriptor* pDesc, 
                                        unsigned char* tlmin, 
                                        unsigned char* tlmax);
dmErrCode            	dmDescriptorGetRange_us(dmDescriptor* pDesc, 
                                        unsigned short* tlmin,
                                        unsigned short* tlmax);
dmErrCode            	dmDescriptorGetRange_bit(dmDescriptor* pDesc, 
                                         unsigned char* tlmin,
                                         unsigned char* tlmax);
dmErrCode            	dmDescriptorGetRange_ul(dmDescriptor* pDesc, 
                                        unsigned long* tlmin,
                                        unsigned long* tlmax);
dmBool			dmDescriptorIsVarLength(const dmDescriptor* dd);



/*--------------------------------------*/
/*      descriptor property writing	*/
/*--------------------------------------*/

dmErrCode		dmSetName(dmDescriptor *dd,char* name);
dmErrCode		dmSetDesc(dmDescriptor *dd,char* description);
dmErrCode             	dmSetDisp(dmDescriptor *dd,char* cmnt);

dmErrCode		dmSetUnit(dmDescriptor *dd,char* unit);
dmErrCode		dmSetCptName(dmDescriptor *dd,long cptNo,char* cptName);
dmErrCode            	dmDescriptorSetRange_s(dmDescriptor* pDesc, short tlmin, 
				       short tlmax);
dmErrCode            	dmDescriptorSetRange_l(dmDescriptor* pDesc, long tlmin, 
				       long tlmax);
dmErrCode            	dmDescriptorSetRange_f(dmDescriptor* pDesc, float tlmin, 
				       float tlmax);
dmErrCode            	dmDescriptorSetRange_d(dmDescriptor* pDesc, double tlmin, 
				       double tlmax);

dmErrCode            	dmDescriptorSetRange_us(dmDescriptor* pDesc, 
                                        unsigned short tlmin,
                                        unsigned short tlmax);
dmErrCode            	dmDescriptorSetRange_ub(dmDescriptor* pDesc, 
                                        unsigned char tlmin,
                                        unsigned char tlmax);

dmErrCode            	dmDescriptorSetRange_bit(dmDescriptor* pDesc, 
                                         unsigned char tlmin,
                                         unsigned char tlmax);

dmErrCode            	dmDescriptorSetRange_ul(dmDescriptor* pDesc, 
                                        unsigned long tlmin,
                                        unsigned long tlmax);

dmErrCode		dmDescriptorSetNull_c( dmDescriptor* pDesc, char* value );
dmErrCode		dmDescriptorGetNull_c( const dmDescriptor* pDesc, char** value );
dmErrCode		dmDescriptorSetNull_s( dmDescriptor* pDesc, short value );
dmErrCode		dmDescriptorSetNull_l( dmDescriptor* pDesc, long value );
dmErrCode		dmDescriptorSetNull_f( dmDescriptor* pDesc, float value );
dmErrCode		dmDescriptorSetNull_d( dmDescriptor* pDesc, double value );
dmErrCode		dmDescriptorSetNull_ub( dmDescriptor* pDesc, unsigned char value );
dmErrCode		dmDescriptorSetNull_us( dmDescriptor* pDesc, unsigned short value );
dmErrCode		dmDescriptorSetNull_ul( dmDescriptor* pDesc, unsigned long value );
dmErrCode		dmDescriptorGetNull_s( const dmDescriptor* pDesc, short* value );
dmErrCode		dmDescriptorGetNull_l( const dmDescriptor* pDesc, long* value );
dmErrCode		dmDescriptorGetNull_f( const dmDescriptor* pDesc, float* value );
dmErrCode		dmDescriptorGetNull_d( const dmDescriptor* pDesc, double* value );
dmErrCode		dmDescriptorGetNull_ub( const dmDescriptor* pDesc, unsigned char* value );
dmErrCode		dmDescriptorGetNull_us( const dmDescriptor* pDesc, unsigned short* value );
dmErrCode		dmDescriptorGetNull_ul( const dmDescriptor* pDesc, unsigned long* value );
dmErrCode		dmDescriptorSetBin_s( dmDescriptor* pDesc, short value );
dmErrCode		dmDescriptorSetBin_l( dmDescriptor* pDesc, long value );
dmErrCode		dmDescriptorSetBin_f( dmDescriptor* pDesc, float value );
dmErrCode		dmDescriptorSetBin_d( dmDescriptor* pDesc, double value );
dmErrCode		dmDescriptorSetBin_ub( dmDescriptor* pDesc, unsigned char value );
dmErrCode		dmDescriptorSetBin_us( dmDescriptor* pDesc, unsigned short value );
dmErrCode		dmDescriptorSetBin_ul( dmDescriptor* pDesc, unsigned long value );
dmErrCode		dmDescriptorGetBin_s( const dmDescriptor* pDesc, short* value );
dmErrCode		dmDescriptorGetBin_l( const dmDescriptor* pDesc, long* value );
dmErrCode		dmDescriptorGetBin_f( const dmDescriptor* pDesc, float* value );
dmErrCode		dmDescriptorGetBin_d( const dmDescriptor* pDesc, double* value );
dmErrCode		dmDescriptorGetBin_ub( const dmDescriptor* pDesc, unsigned char* value );
dmErrCode		dmDescriptorGetBin_us( const dmDescriptor* pDesc, unsigned short* value );
dmErrCode		dmDescriptorGetBin_ul( const dmDescriptor* pDesc, unsigned long* value );

/*------------------------------*/
/*         miscellaneous	*/
/*------------------------------*/

dmErrCode		dmDescriptorDelete(dmDescriptor *pDesc);
dmBool           	dmDescriptorCompare(dmDescriptor* pDesc1, dmDescriptor* pDesc2,
				     long mask);

/*--------------------------------------*/
/*         single-row scalar reads	*/
/*--------------------------------------*/

short			dmGetScalar_s(dmDescriptor* pDesc);
long			dmGetScalar_l(dmDescriptor* pDesc);
float			dmGetScalar_f(dmDescriptor* pDesc);
double			dmGetScalar_d(dmDescriptor* pDesc);
void			dmGetScalar_c(dmDescriptor* pDesc,  char *val, long maxlen);
/* Deprecated */
#define dmGetScalar_br dmGetScalar_c

dmErrCode		dmGetScalar_q(dmDescriptor* pDesc);
unsigned char		dmGetScalar_ub(dmDescriptor* pDesc);
dmErrCode            	dmGetScalar_bit(dmDescriptor* pDesc, void * pBitValues);
unsigned short		dmGetScalar_us(dmDescriptor* pDesc);
unsigned long		dmGetScalar_ul(dmDescriptor* pDesc);

/*--------------------------------------*/
/*         single-row scalar writes 	*/
/*--------------------------------------*/

dmErrCode		dmSetScalar_s(dmDescriptor* pDesc, short);
dmErrCode		dmSetScalar_l(dmDescriptor* pDesc, long);
dmErrCode	   	dmSetScalar_f(dmDescriptor* pDesc, float);
dmErrCode	    	dmSetScalar_d(dmDescriptor* pDesc, double);
dmErrCode	    	dmSetScalar_c(dmDescriptor* pDesc, char*);
/* Deprectated */
#define dmSetScalar_br dmSetScalar_c

dmErrCode	    	dmSetScalar_q(dmDescriptor* pDesc,  dmBool);
dmErrCode	    	dmSetScalar_ub(dmDescriptor* pDesc, unsigned char);
dmErrCode            	dmSetScalar_bit(dmDescriptor* pDesc, void *);
dmErrCode    		dmSetScalar_us(dmDescriptor* pDesc, unsigned short);
dmErrCode    		dmSetScalar_ul(dmDescriptor* pDesc, unsigned long);

/*--------------------------------------*/
/*         multi-row scalar reads	*/
/*--------------------------------------*/

long 			dmGetScalars_s(dmDescriptor*, short*, 	   long row, long nrows);
long 			dmGetScalars_l(dmDescriptor*, long* ,	   long row, long nrows);
long 			dmGetScalars_f(dmDescriptor*, float*,	   long row, long nrows);
long 			dmGetScalars_d(dmDescriptor*, double*,	   long row, long nrows);
long 			dmGetScalars_c(dmDescriptor*, char **,long maxlen, long row, long nrows);
/* Deprecated */
#define dmGetScalars_br dmGetScalars_c

long 			dmGetScalars_q(dmDescriptor*, dmBool* ,	   long row, long nrows);
long 			dmGetScalars_ub(dmDescriptor*,unsigned char*, long row, long nrows);
long 			dmGetScalars_bit(dmDescriptor*,void*, long row, long nrows);
long 			dmGetScalars_us(dmDescriptor*,unsigned short*,long row, long nrows);
long 			dmGetScalars_ul(dmDescriptor*,unsigned long*, long row, long nrows);

/*--------------------------------------*/
/*         multi-row scalar writes	*/
/*--------------------------------------*/

long 			dmSetScalars_s(dmDescriptor*, short*, 	   long row, long nrows);
long 			dmSetScalars_l(dmDescriptor*, long* ,	   long row, long nrows);
long 			dmSetScalars_f(dmDescriptor*, float*,	   long row, long nrows);
long 			dmSetScalars_d(dmDescriptor*, double*,	   long row, long nrows);
long 			dmSetScalars_c(dmDescriptor*, char **,	   long row, long nrows);
/* Deprecated */
#define dmSetScalars_br dmSetScalars_c

long 			dmSetScalars_q(dmDescriptor*, dmBool* ,	   long row, long nrows);
long 			dmSetScalars_ub(dmDescriptor*,unsigned char*, long row, long nrows);
long 			dmSetScalars_bit(dmDescriptor*,void*, long row, long nrows);
long 			dmSetScalars_us(dmDescriptor*,unsigned short*,long row, long nrows);
long 			dmSetScalars_ul(dmDescriptor*,unsigned long*, long row, long nrows);

/*--------------------------------------*/
/*         single-row array reads	*/
/*--------------------------------------*/

dmErrCode		dmGetArray_s(dmDescriptor*, short*,	long npixels);
dmErrCode		dmGetArray_l(dmDescriptor*, long* ,	long npixels);
dmErrCode		dmGetArray_f(dmDescriptor*, float*,	long npixels);
dmErrCode		dmGetArray_d(dmDescriptor*, double*,long npixels);
dmErrCode		dmGetArray_c(dmDescriptor*, char **,long npixels);
/* Deprecated */
#define dmGetArray_br dmGetArray_c

dmErrCode		dmGetArray_q(dmDescriptor*, dmBool*,	long npixels);
dmErrCode		dmGetArray_ub(dmDescriptor*,unsigned char*, long npixels);
dmErrCode		dmGetArray_bit(dmDescriptor*,void*, long npixels);
dmErrCode		dmGetArray_us(dmDescriptor*,unsigned short*,long npixels);
dmErrCode		dmGetArray_ul(dmDescriptor*,unsigned long*, long npixels);

/*--------------------------------------*/
/*         single-row array writes 	*/
/*--------------------------------------*/

dmErrCode		dmSetArray_s(dmDescriptor*, short*,	long npixels);
dmErrCode		dmSetArray_l(dmDescriptor*, long* ,	long npixels);
dmErrCode		dmSetArray_f(dmDescriptor*, float*,	long npixels);
dmErrCode		dmSetArray_d(dmDescriptor*, double*,long npixels);
dmErrCode		dmSetArray_c(dmDescriptor*, char **,long npixels);
/* Deprecated */
#define dmSetArray_br dmSetArray_c

dmErrCode		dmSetArray_q(dmDescriptor*, dmBool* ,	long npixels);
dmErrCode		dmSetArray_ub(dmDescriptor*,unsigned char*, long npixels);
dmErrCode		dmSetArray_bit(dmDescriptor*,void*, long npixels);
dmErrCode		dmSetArray_us(dmDescriptor*,unsigned short*,long npixels);
dmErrCode		dmSetArray_ul(dmDescriptor*,unsigned long*, long npixels);

/*--------------------------------------*/
/*         single-row vector reads 	*/
/*--------------------------------------*/

dmErrCode		dmGetVector_s(dmDescriptor* pDesc,short* outvals, long dim);
dmErrCode		dmGetVector_l(dmDescriptor* pDesc,long* outvals, long dim);
dmErrCode		dmGetVector_f(dmDescriptor* pDesc, float* outvals, long dim);
dmErrCode		dmGetVector_d(dmDescriptor* pDesc, double* outvals, long dim);
dmErrCode		dmGetVector_q(dmDescriptor* pDesc, dmBool* outvals, long dim);
dmErrCode		dmGetVector_c(dmDescriptor* pDesc, char** outvals, long dim);
/* Deprecated */
#define dmGetVector_br dmGetVector_c

dmErrCode		dmGetVector_ub(dmDescriptor* pDesc,unsigned char* outvals, long dim);
dmErrCode		dmGetVector_us(dmDescriptor* pDesc, unsigned short* outvals, long dim);
dmErrCode		dmGetVector_ul(dmDescriptor* pDesc, unsigned long* outvals, long dim);


/*--------------------------------------*/
/*         single-row vector writes 	*/
/*--------------------------------------*/

dmErrCode		dmSetVector_s(dmDescriptor*, short*, long npixels);
dmErrCode		dmSetVector_l(dmDescriptor*, long* , long npixels);
dmErrCode		dmSetVector_f(dmDescriptor*, float*, long npixels);
dmErrCode		dmSetVector_d(dmDescriptor*, double*,long npixels);
dmErrCode		dmSetVector_c(dmDescriptor*, char **,long npixels);
/* Deprectated */
#define dmSetVector_br dmSetVector_c

dmErrCode		dmSetVector_q(dmDescriptor*, dmBool* , long npixels);
dmErrCode		dmSetVector_ub(dmDescriptor*,unsigned char*, long npixels);
dmErrCode		dmSetVector_us(dmDescriptor*,unsigned short*,long npixels);
dmErrCode		dmSetVector_ul(dmDescriptor*,unsigned long*, long npixels);

/*--------------------------------------*/
/*         multi-row vector reads 	*/
/*--------------------------------------*/

dmErrCode		dmGetVectors_s(dmDescriptor* pDesc, short* outvals, long dim, long row, long nrows);
dmErrCode		dmGetVectors_l(dmDescriptor* pDesc, long* outvals, long dim, long row, long nrows);
dmErrCode		dmGetVectors_f(dmDescriptor* pDesc, float* outvals, long dim, long row, long nrows);
dmErrCode		dmGetVectors_d(dmDescriptor* pDesc, double* outvals, long dim, long row,  long nrows);
dmErrCode		dmGetVectors_q(dmDescriptor* pDesc, dmBool* outVals, long dim, long row, 
				       long nrows);
dmErrCode		dmGetVectors_c(dmDescriptor* pDesc, char** outvals, long dim, long row, 
				       long nrows);
/* Deprecated */
#define dmGetVectors_br dmGetVectors_c

dmErrCode		dmGetVectors_ub(dmDescriptor* pDesc, unsigned char* outvals, long dim, long row, 
					long nrows);
dmErrCode		dmGetVectors_us(dmDescriptor* pDesc, unsigned short* outvals, long dim, long row, 
					long nrows);
dmErrCode		dmGetVectors_ul(dmDescriptor* pDesc, unsigned long* outvals, long dim, long row, 
					long nrows);


/*--------------------------------------*/
/*         multi-row vector writes 	*/
/*--------------------------------------*/

dmErrCode		dmSetVectors_s(dmDescriptor*, short*, long dim, long row, long nrows);
dmErrCode	        dmSetVectors_l(dmDescriptor*, long* , long dim,long row, long nrows);
dmErrCode		dmSetVectors_f(dmDescriptor*, float*, long dim,long row, long nrows);
dmErrCode		dmSetVectors_d(dmDescriptor*, double*,long dim,long row, long nrows);
dmErrCode		dmSetVectors_c(dmDescriptor*, char **,long dim,long row, long nrows);
/* Deprecated */
#define dmSetVectors_br dmSetVectors_c

dmErrCode		dmSetVectors_q(dmDescriptor*, dmBool* , long dim,long row, long nrows);
dmErrCode		dmSetVectors_ub(dmDescriptor*,unsigned char*, long dim,long row, long nrows);
dmErrCode		dmSetVectors_us(dmDescriptor*,unsigned short*,long dim,long row, long nrows);
dmErrCode		dmSetVectors_ul(dmDescriptor*,unsigned long*, long dim,long row, long nrows);


dmErrCode    dmSetVectorArray_s(dmDescriptor* pDesc, short* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_l(dmDescriptor* pDesc, long* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_f(dmDescriptor* pDesc, float* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_d(dmDescriptor* pDesc, double* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_q(dmDescriptor* pDesc, dmBool* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_c(dmDescriptor* pDesc, char** invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_ub(dmDescriptor* pDesc, unsigned char* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_us(dmDescriptor* pDesc, unsigned short* invals, long dim, long nvals);
dmErrCode    dmSetVectorArray_ul(dmDescriptor* pDesc, unsigned long* invals, long dim, long nvals);
dmErrCode    dmSetVectorArrays_s(dmDescriptor* pDesc, short* invals, long dim, long row, long nrows);
dmErrCode    dmSetVectorArrays_l(dmDescriptor* pDesc, long* invals, long dim, long row, long nrows);
dmErrCode    dmSetVectorArrays_f(dmDescriptor* pDesc, float* invals, long dim, long row, long nrows);
dmErrCode    dmSetVectorArrays_d(dmDescriptor* pDesc, double* invals, long dim, long row, long nrows);
dmErrCode    dmSetVectorArrays_q(dmDescriptor* pDesc, dmBool* invals, long dim, long row, long nrows);
dmErrCode    dmSetVectorArrays_c(dmDescriptor* pDesc, char** invals, long dim,  long row, long nrows);
dmErrCode    dmSetVectorArrays_ub(dmDescriptor* pDesc, unsigned char* invals, long dim, 
		    long row, long nrows);
dmErrCode   dmSetVectorArrays_us(dmDescriptor* pDesc, unsigned short* invals, long dim,  long row, 
		long nrows);
dmErrCode   dmSetVectorArrays_ul(dmDescriptor* pDesc, unsigned long* invals, long dim, long row, 
		long nrows);
dmErrCode   dmGetVectorArray_s(dmDescriptor* pDesc, short* outvals, long dim, long nvals);
dmErrCode   dmGetVectorArray_l(dmDescriptor* pDesc, long* outvals, long dim, long nvals);
dmErrCode   dmGetVectorArray_f(dmDescriptor* pDesc, float* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_d(dmDescriptor* pDesc, double* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_q(dmDescriptor* pDesc, dmBool* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_c(dmDescriptor* pDesc, char** outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_ub(dmDescriptor* pDesc, unsigned char* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_us(dmDescriptor* pDesc, unsigned short* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArray_ul(dmDescriptor* pDesc, unsigned long* outvals, long dim, long nvals );
dmErrCode   dmGetVectorArrays_s(dmDescriptor* pDesc, short* outvals, long dim, long nvals , long row, long nrows);
dmErrCode   dmGetVectorArrays_l(dmDescriptor* pDesc, long* outvals, long dim, long nvals , long row, long nrows);
dmErrCode   dmGetVectorArrays_f(dmDescriptor* pDesc, float* outvals, long dim, long nvals , long row, long nrows);
dmErrCode   dmGetVectorArrays_d(dmDescriptor* pDesc, double* outvals, long dim, long nvals , long row, long nrows);
dmErrCode   dmGetVectorArrays_q(dmDescriptor* pDesc, dmBool* outvals, long dim, long nvals , long row, long nrows);
dmErrCode   dmGetVectorArrays_c(dmDescriptor* pDesc, char** outvals, long dim, long nvals, long row, long nrows);
dmErrCode   dmGetVectorArrays_ub(dmDescriptor* pDesc, unsigned char* outvals, long dim, long nvals, long row, long nrows);
dmErrCode   dmGetVectorArrays_us(dmDescriptor* pDesc, unsigned short* outvals, long dim, long nvals, long row, long nrows);
dmErrCode   dmGetVectorArrays_ul(dmDescriptor* pDesc, unsigned long* outvals, long dim, long nvals, long row, long nrows);


/*-------------------------------------------------*/
/*		   column dmDescriptor		   */
/*-------------------------------------------------*/

dmDescriptor*		dmColumnCreate(dmBlock *table, char *name, dmDataType,
			       		long desclen, char *unit, char *desc);

dmDescriptor**		dmTableCreateColumns(dmBlock *table, char **names,
				     dmDataType* types, long *strLens,
				     char **units, char **descs, long numCols);

dmDescriptor*		dmColumnCreateVector(dmBlock* table, char* name, dmDataType,
				     long strLen, char* unit, char* desc, 
				     char** cptNames, long dim);

dmDescriptor*		dmColumnCreateArray(dmBlock *table, char *name, dmDataType,
				    long desclen, char *unit, char *desc,
				    long arraySize);

dmDescriptor*		dmColumnCreateVarArray(dmBlock *table, char *name, dmDataType,
				    long strLen, char *unit, char *desc,
				    long arraySize);


dmDescriptor*   	dmColumnCreateNDArray(dmBlock *table, char *name, dmDataType dType,
		      long strLens, char *unit, char *desc, long* axlen,
		      long naxes);


dmDescriptor*		dmColumnCreateGeneric(dmBlock *table, char *name, dmDataType,
				    long strLen, char *unit, char *desc,
				    dmElementType elemType, char **cptNames,
				    long arrayDim,long *axesLens,long numAxes);




/*-------------------------------------------------*/
/*	 	header key dmDescriptor		   */
/*-------------------------------------------------*/

dmDescriptor*		dmBlockAdvanceKeys(dmBlock *block, long numkeys);
dmDescriptor*		dmBlockGetKey( dmBlock* pBl, long keyNo);
dmDescriptor*		dmBlockMoveToKey(dmBlock *block, char *name);
dmDescriptor *  	dmBlockMoveToKeyNo(dmBlock *block, long keynum);
long 			dmBlockGetNoKeys(const dmBlock* pBlock);
dmDescriptor**		dmBlockGetKeyList(const dmBlock* pBl, long *numKeys);
dmDescriptor*		dmKeyOpen( dmBlock* pBl, char *keyName);


/*--------------------------------------*/
/* 	     scalar key reads 		*/
/*--------------------------------------*/

dmDescriptor* 		dmKeyRead_s( dmBlock *pBlock, const char *name, short *val);
dmDescriptor* 		dmKeyRead_l( dmBlock *pBlock, const char *name, long *val);
dmDescriptor* 		dmKeyRead_f( dmBlock *pBlock, const char *name, float *val);
dmDescriptor* 		dmKeyRead_d( dmBlock *pBlock, const char *name, double *val);
dmDescriptor* 		dmKeyRead_q( dmBlock *pBlock, const char *name, dmBool *val);
dmDescriptor* 		dmKeyRead_c( dmBlock *pBlock, const char *name, char *val,long maxlen);
/* Deprecated */
#define dmKeyRead_br dmKeyRead_c

dmDescriptor* 		dmKeyRead_ub( dmBlock *pBlock, const char *name, unsigned char *val);
dmDescriptor* 		dmKeyRead_us( dmBlock *pBlock, const char *name, unsigned short *val);
dmDescriptor* 		dmKeyRead_ul( dmBlock *pBlock, const char *name, unsigned long *val);

/*--------------------------------------*/
/* 	     scalar key writes 		*/
/*--------------------------------------*/

dmDescriptor* 		dmKeyWrite_s(dmBlock *pBlock, const char *name, const short val, 
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_l(dmBlock *pBlock, const char *name, const long val, 
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_f(dmBlock *pBlock, const char *name, const float val, 
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_d(dmBlock *pBlock, const char *name, const double val, 
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_q(dmBlock *pBlock, const char *name, const dmBool val,
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_c(dmBlock *pBlock, const char *name, const char *val, 
			   			const char *unit, const char *desc);
/* Deprecated */
#define dmKeyWrite_br dmKeyWrite_c

dmDescriptor* 		dmKeyWrite_ub(dmBlock *pBlock, const char *name, const unsigned char val,
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_us(dmBlock *pBlock, const char *name, const unsigned short val,
			   			const char *unit, const char *desc);
dmDescriptor* 		dmKeyWrite_ul(dmBlock *pBlock, const char *name, const unsigned long val,
			   			const char *unit, const char *desc);

/*-------------------------------------------------*/
/*                 Arrayed key writes              */
/*-------------------------------------------------*/

dmDescriptor* 		dmKeyWriteArray_s(dmBlock *pBlock, const char *name, const short* val, 
				const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_l(dmBlock *pBlock, const char *name, const long* val, 
				const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_f(dmBlock *pBlock, const char *name, const float* val, 
				const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_d(dmBlock *pBlock, const char *name, const double* val, 
				const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_q(dmBlock *pBlock, const char *name, const dmBool* val,
				const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_c(dmBlock *pBlock, const char *name, const char* *val, 
				const char *unit, const char *desc, const long size);
/* Deprecated */
#define dmKeyWriteArray_br dmKeyWriteArray_c

dmDescriptor* 		dmKeyWriteArray_ub(dmBlock *pBlock, const char *name, 
				 const unsigned char* val,
				 const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_us(dmBlock *pBlock, const char *name, 
				 const unsigned short* val,
				 const char *unit, const char *desc, const long size);
dmDescriptor* 		dmKeyWriteArray_ul(dmBlock *pBlock, const char *name, 
				 const unsigned long* val, const char *unit, const char *desc, 
				 const long size);

/*-------------------------------------------------*/
/*		   	kernel			   */
/*-------------------------------------------------*/

char*			dmKernelSetCreate(const char *kernelID);
dmErrCode			dmKernelGetCreate(char *kernelStr,long maxlen);
char*			dmKernelSetCopy(const char *kernelID);
dmErrCode			dmKernelGetCopy(char *kernelStr,long maxlen);
dmErrCode			dmKernelGetList(char ***kernelIDs,long *numKers);
dmErrCode			dmDatasetGetKernel(const dmDataset *pDs,char *kernelName,long maxlen);
dmErrCode  dmBlockGetKernelKey( dmBlock* block, long keyno, char* buf, long maxlen );
long dmBlockGetNoKernelKeys( dmBlock* block );
/*-------------------------------------------------*/
/*	         Subspace Functions    		   */
/*-------------------------------------------------*/
dmDescriptor**  	dmBlockGetSubspace( dmBlock* block, long* ncols, long* ncpts );
dmErrCode            	dmBlockPrintSubspace(dmBlock* pBlock );

dmErrCode            	dmBlockSetSubspaceCpt(dmBlock *pBlock, long cptNo);
long            	dmBlockGetNoSubspaceCpts(dmBlock *pBlock);
long            	dmBlockGetCurrSubspaceCpt(dmBlock  *pBlock);
dmErrCode			dmBlockCreateSubspaceCpt(dmBlock *pBlock);

dmDescriptor*   	dmSubspaceColCreate_s(dmBlock *pBl, char* name, char* unit,
				 short* value1, short* value2, long nvalues);
dmDescriptor*   	dmSubspaceColCreate_l(dmBlock *pBl, char* name, char* unit,
				 long* value1, long* value2, long nvalues);
dmDescriptor*   	dmSubspaceColCreate_f(dmBlock *pBl, char* name, char* unit,
				 float* value1, float* value2, long nvalues);
dmDescriptor*   	dmSubspaceColCreate_d(dmBlock *pBl, char* name, char* unit,
				 double* value1, double* value2, 
				 long nvalues);
dmDescriptor*   	dmSubspaceColCreateTable_s(dmBlock *pBlock, char* tabname, 
			   char* name, char** colNames, char* unit, 
			    short* value1, short* value2, long nvalues);
dmDescriptor*   	dmSubspaceColCreateTable_l(dmBlock *pBlock, char* tabname, 
			   char* name, char** colNames, char* unit, long* value1, long* value2, 
			    long nvalues);
dmDescriptor*   	dmSubspaceColCreateTable_f(dmBlock *pBlock, char* tabname, 
			   char* name, char** colNames, char* unit, float* value1, float* value2, 
			    long nvalues);
dmDescriptor*   	dmSubspaceColCreateTable_d(dmBlock *pBlock, char* tabname, 
			   char* name, char** colNames, char* unit, double* value1, 
			    double* value2, long nvalues);

dmErrCode           	 	dmSubspaceColSetTableName(dmDescriptor* dmDSS, char* name);
dmErrCode            	dmSubspaceColGet_s(dmDescriptor* filter, short** value1, 
				   short** value2, long* nvalues);
dmErrCode            	dmSubspaceColGet_l(dmDescriptor* filter, long** value1, 
				   long** value2, long* nvalues);
dmErrCode            	dmSubspaceColGet_f(dmDescriptor* filter, float** value1, 
				   float** value2, long* nvalues);
dmErrCode            	dmSubspaceColGet_d(dmDescriptor* filter, double** value1, 
				   double** value2, long* nvalues);
dmErrCode            	dmSubspaceColSet_s(dmDescriptor* filter,
				   short* value1, short* value2, long nvalues);
dmErrCode            	dmSubspaceColSet_l(dmDescriptor* filter,
				   long* value1, long* value2, long nvalues);
dmErrCode            	dmSubspaceColSet_f(dmDescriptor* filter, 
				   float* value1, float* value2, long nvalues);
dmErrCode            	dmSubspaceColSet_d(dmDescriptor* filter, 
				   double* value1, double* value2, 
				   long nvalues);
dmErrCode            	dmSubspaceColUpdate_s(dmDescriptor* filter, short* value1,
				   short* value2, long nvalues);
dmErrCode            	dmSubspaceColUpdate_l(dmDescriptor* filter, long* value1,
				   long* value2, long nvalues);
dmErrCode            	dmSubspaceColUpdate_f(dmDescriptor* filter, float* value1,
				   float* value2, long nvalues);
dmErrCode            	dmSubspaceColUpdate_d(dmDescriptor* filter, double* value1,
				   double* value2, long nvalues);
long            	dmSubspaceColGetTableName(dmDescriptor *dd, 
                                          char *name, long maxlen);

dmDescriptor* 		dmSubspaceColRead_s(dmBlock* pBlock, char* name, short** value1, 
		    short** value2, long* nvalues);
dmDescriptor* 		dmSubspaceColRead_l(dmBlock* pBlock, char* name, long** value1, 
		    long** value2, long* nvalues);
dmDescriptor* 		dmSubspaceColRead_f(dmBlock* pBlock, char* name, float** value1, 
		    float** value2, long* nvalues);
dmDescriptor* 		dmSubspaceColRead_d(dmBlock* pBlock, char* name, 
				  double** value1, double** value2, 
				  long* nvalues);
dmErrCode          		dmBlockIntersectSubspace(dmBlock* block1, dmBlock* block2,
				       dmBlock* block0);
dmErrCode          		dmBlockMergeSubspace(dmBlock* block1, dmBlock* block2, 
				   dmBlock* block0);

dmDescriptor*
dmSubspaceColCreateRegion(dmBlock *pBlock, char* name,
			   char** colNames, dmDataType type, char* unit, regRegion* region );
dmDescriptor* dmSubspaceColCreateRegionFromString( dmBlock* block, char* name,
 char** cptNames, dmDataType type, char* unit, char* region_string );
int dmSubspaceColSetRegion( dmDescriptor* dd, regRegion* region );

dmDescriptor*   	dmSubspaceColOpen(dmBlock* pBlock, char* name);
dmDescriptor* 		dmBlockGetSubspaceColNo(dmBlock* pBlock, long colNum);
long           		dmBlockGetNoSubspaceCols(dmBlock* pBlock);

/*-------------------------------------------------*/
/*		   Coordinate Systems		   */
/*-------------------------------------------------*/

dmDescriptor* 		dmCoordCreate_s(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      short* crpix, double* crval, double* cdelt,
			      double* params);
dmDescriptor* 		dmCoordCreate_l(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      long* crpix, double* crval, double* cdelt,
			      double* params);
dmDescriptor* 		dmCoordCreate_f(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      float* crpix, double* crval, double* cdelt,
			      double* params);
dmDescriptor* 		dmCoordCreate_d(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      double* crpix, double* crval, double* cdelt,
			      double* params);
dmDescriptor* 		dmCoordCreate_ub(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      unsigned char* crpix, double* crval, 
			      double* cdelt, double* params);
dmDescriptor* 		dmCoordCreate_us(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      unsigned short* crpix, double* crval, 
			      double* cdelt, double* params);
dmDescriptor* 		dmCoordCreate_ul(dmDescriptor* dd, char* name, char* unit,
			      char** cptNames, long dim, char* transform,
			      unsigned long* crpix, double* crval, 
			      double* cdelt, double* params);

dmDescriptor* 		dmCoordCreateLinear_s(dmDescriptor* desc, char* name, 
				     char* unit, short crpix, double crval, 
				     double cdelt);

dmDescriptor* 		dmCoordCreateLinear_l(dmDescriptor* desc, char* name, 
				     char* unit, long crpix, double crval, 
				     double cdelt);

dmDescriptor* 		dmCoordCreateLinear_f(dmDescriptor* desc, char* name, 
				     char* unit, float crpix, double crval, 
				     double cdelt);

dmDescriptor* 		dmCoordCreateLinear_d(dmDescriptor* desc, char* name, 
				     char* unit, double crpix, double crval, 
				     double cdelt);

dmDescriptor* 		dmCoordCreateLinear_ub(dmDescriptor* desc, char* name, 
				     char* unit, unsigned char crpix, 
				     double crval, double cdelt);

dmDescriptor* 		dmCoordCreateLinear_us(dmDescriptor* desc, char* name, 
				     char* unit, unsigned short crpix, 
				     double crval, double cdelt);

dmDescriptor* 		dmCoordCreateLinear_ul(dmDescriptor* desc, char* name, 
				     char* unit, unsigned long crpix, 
				     double crval, double cdelt);

dmErrCode          	dmCoordCalc_s(dmDescriptor* dd, short* value, double* results);

dmErrCode          	dmCoordCalc_l(dmDescriptor* dd, long* value, double* results);

dmErrCode          	dmCoordCalc_f(dmDescriptor* dd, float* value, double* results);

dmErrCode          	dmCoordCalc_d(dmDescriptor* dd, double* value, double* results);

dmErrCode          	dmCoordCalc_ub(dmDescriptor* dd, unsigned char* value,
                                     double* results);

dmErrCode          	dmCoordCalc_us(dmDescriptor* dd, unsigned short* value,
                                     double* results);

dmErrCode          	dmCoordCalc_ul(dmDescriptor* dd, unsigned long* value,
                                     double* results);

dmErrCode          	dmCoordInvert_s(dmDescriptor* dd, double* value, short* results);

dmErrCode          	dmCoordInvert_l(dmDescriptor* dd, double* value, long* results);

dmErrCode          	dmCoordInvert_f(dmDescriptor* dd, double* value, float* results);

dmErrCode          	dmCoordInvert_d(dmDescriptor* dd, double* value, double* results);

dmErrCode          	dmCoordInvert_ub(dmDescriptor* dd, double* value,
                                     unsigned char* results);

dmErrCode          	dmCoordInvert_us(dmDescriptor* dd, double* value,
                                     unsigned short* results);

dmErrCode          	dmCoordInvert_ul(dmDescriptor* dd, double* value,
                                     unsigned long* results);

dmErrCode          	dmCoordSetTransform_s(dmDescriptor* dd, short* crpix, 
				     double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordSetTransform_l(dmDescriptor* dd, long* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordSetTransform_f(dmDescriptor* dd, float* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordSetTransform_d(dmDescriptor* dd, double* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordSetTransform_ub(dmDescriptor* dd, unsigned char* crpix, 
				     double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordSetTransform_us(dmDescriptor* dd, unsigned short* crpix, 
				     double* crval, double* cdelt, long dim);

dmErrCode 	        dmCoordSetTransform_ul(dmDescriptor* dd, unsigned long* crpix, 
				     double* crval, double* cdelt, long dim);


dmErrCode          	dmCoordGetTransform_s(dmDescriptor* dd, short* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode 	        dmCoordGetTransform_l(dmDescriptor* dd, long* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordGetTransform_f(dmDescriptor* dd, float* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordGetTransform_d(dmDescriptor* dd, double* crpix, 
				    double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordGetTransform_ub(dmDescriptor* dd, unsigned char* crpix, 
				     double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordGetTransform_us(dmDescriptor* dd, unsigned short* crpix, 
				     double* crval, double* cdelt, long dim);

dmErrCode          	dmCoordGetTransform_ul(dmDescriptor* dd, unsigned long* crpix, 
				     double* crval, double* cdelt, long dim);

dmDescriptor* 		dmCoordGetParent(dmDescriptor* dd);

double* 		dmCoordGetParams(dmDescriptor* dd, long* nparams);

dmErrCode    	      	dmCoordGetTransformType(dmDescriptor* dd, char* ttype, 
					   long* nparams, long maxlen);

long          		dmCoordGetAxisGroupNo(dmDescriptor* dd);


/*-------------------------------------------------*/
/*			misc			   */
/*-------------------------------------------------*/

dmErrCode		dmGetVersion(char *str,long maxlen);
dmErrCode		dmSetInternals(char *param,void *value);


void 			dmInit(const int a, const void *b);
void 			dmInitFunc(int a, void *b);
void 			dmResetErrorState( void );
void 			dmSetError( dmErrCode errCode, char* errStr );
void 			dmGetErrorMessage( char* str, long maxlen );
dmErrCode		dmGetError( void );
        
dmErrCode		dmKernelSetOption( char* option );
dmErrCode		dmDatasetDestroy( const char* dsname );
int			dmFileExists( char* filename );
void			dmFree( void* ptr );

regRegion*              dmTableReadRegion( dmBlock* block );
dmBlock*                dmTableWriteRegion( dmDataset* ds, char* name, dmDescriptor* dd, regRegion* region );
dmErrCode               dmSubspaceColGetRegionString( dmDescriptor* dd, char* region_string, long maxlen );
regRegion*              dmSubspaceColGetRegion( dmDescriptor* dd );
void                    dmSetVerbose( const int verbose );
dmErrCode               dmBlockCreateEmptySubspaceCpt(dmBlock* pBlock);
int                     dmSubspaceColSet_us(dmDescriptor* filter, unsigned short* value1,
				   unsigned short* value2, long nvalues);
int                     dmSubspaceColSet_ul(dmDescriptor* filter, unsigned long* value1,
				   unsigned long* value2, long nvalues);
int                     dmSubspaceColGet_us(dmDescriptor* filter, unsigned short** value1, 
				   unsigned short** value2, long* nvalues);
int                     dmSubspaceColGet_ul(dmDescriptor* filter, unsigned long** value1, 
				   unsigned long** value2, long* nvalues);
dmDescriptor*           dmSubspaceColRead_ul(dmBlock* pBlock, char* name, unsigned long** value1, 
					     unsigned long** value2, long* nvalues);
dmDescriptor*           dmSubspaceColRead_us(dmBlock* pBlock, char* name, unsigned short** value1, 
					     unsigned short** value2, long* nvalues);
dmDescriptor*           dmSubspaceColCreateTable_us(dmBlock *pBlock, char* tabname, char* name,
			   char** colNames, char* unit, unsigned short* value1, 
			   unsigned short* value2, long nvalues);
dmDescriptor*           dmSubspaceColCreateTable_ul(dmBlock *pBlock, char* tabname, char* name,
			   char** colNames, char* unit, unsigned long* value1, 
			   unsigned long* value2, long nvalues);

dmDescriptor*           dmSubspaceColCreate_us(dmBlock *pBlock, char* name, char* unit,
					       unsigned short* value1, unsigned short* value2, long nvalues);
dmDescriptor*           dmSubspaceColCreate_ul(dmBlock *pBlock, char* name, char* unit,
					       unsigned long* value1, unsigned long* value2, long nvalues);

#define			dmDatasetTableCreate	dmTableCreate
#define			dmDatasetTableOpen	dmTableOpen
#define			dmDatasetTableClose	dmTableClose
#define			dmGetStrLen dmDescriptorGetLength

void dmTanApplyAspect( double* det, double* nom, double* asol, double* crpix, double* cdelt, double* sky );
void dmTanPixToWorld( double* pix, double* asp, double* crpix, double* cdelt, double* world );
void dmTanWorldToPix( double* world, double* asp, double* crpix, double* cdelt, double* pix );

int dmBlockCopyWCS(dmBlock* srcBlock, dmBlock* destBlock);
int dmBlockCopyData( dmBlock* srcBlock, dmBlock* destBlock );

void dmMapDataTypeName( dmDataType dataType, char* buf );
void dmCoordSetParam( dmDescriptor* dd, char* name, double rot );

dmDescriptor* dmBlockCopyCol( dmBlock* destBlock, dmDescriptor* dd );
#define dmPCopyCol dmBlockCopyCol
int dmCopyGeneric( dmDescriptor* src, dmDescriptor* dest );
dmDescriptor** dmTableOpenCopyColumnList( dmBlock* destblock, dmDescriptor** src, long n );
int dmTableCopyColumnListData( dmDescriptor** src, dmDescriptor** dest, long n );
long dmBinningParse( dmBlock* block, char* name, char* spec, int compact, double** mins, double** maxes, long* n, int* partial );
void dmBinningTypeName( long type, char* ttypename, long maxlen );
void dmBinningDisp( dmBlock* block, char* name, char* spec, long minr,long maxr );
long dmTableGetRowSize(dmBlock *bl);
regRegion* dmRegParse( char* buf );
int
dmDescriptorGetRangeCpt_d(dmDescriptor* pDesc, const long vcpt, double* tlmin, double* tlmax);
long dmColumnGetRowOffset( dmDescriptor* dd );
void* dmColumnGetRowPointer(dmDescriptor* dd, void *row );
long dmDescriptorGetNo( const dmDescriptor* dd );

#define dmKeyGetNo dmDescriptorGetNo

int dmRegConvertWorldRegion( dmDescriptor* vcoord, regRegion* region );
/* Deprecated */
void*			dmTableGetColPtr(dmBlock *tab, void *row, long colno);
short			dmTableGetColOffset(dmBlock *tab, long colno);


/* Used in a hack in dmlist somewhere */
void dmi_subspace_col_region_area( dmDescriptor* dd, regRegion* region, double* rrarea, double* rfarea, int verbose );
void dmpRegionArea( dmDescriptor* dd, regRegion* region, double* rrarea, double* rfarea, int verbose );


int dmBlockListKeys( dmBlock* block );
char* dmStrDup( char* a );
void dmErrList( void );
dmDescriptor* dmColumnCreateGen(dmBlock *block, char *name, dmDataType type, 
		      long length, char *unit, char *desc, 
		      dmElementType etype, char **cptNames,
		      long vdim, long* axes, long naxes, dmBool varlen, char* hint );
void dmBlockFlush( dmBlock* block );

/* Under test */
void dmFilterFree( dmBuffer* buffer );
dmBuffer* dmFilterCreate( dmBlock* block, char* spec );
dmBool dmFilterRow( dmBuffer* buffer, void* data );

void dmRegConvertInit( dmDescriptor* dd );
void dmRegConvert( double* world, double* pix );

/* Row Filtering */
dmRowFilter* dmTableAllocRowFilter( dmBlock* block, const char* vspec );
int dmTableFilterRow( dmBlock* block, dmRowFilter* filter, void* row );
void dmRowFilterFree( dmRowFilter* filter );

/* Deprecated */
void			dmFreeDescriptor(dmDescriptor* pDesc);
void			dmFreeDescriptorArray(dmDescriptor **pDescArray,
							long numDescriptors);
long dmColumnGetNo( dmDescriptor* dd );
#ifdef __cplusplus
}
#endif

#endif

