/*                                                                
**  Copyright (C) 2002-2003,2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmconfig.h                                                  */
/*                                                               */
/*  The file Makevars.in determines basic system properties using*/
/*  autoconf 2.52 or later and the configure.ac script in this   */
/*  directory.                                                   */
/*  Configure it to make the system-specific file 'Makevars'.    */
/*  1) Create configure.ac                                       */
/*  2) Run autoconf to create ./configure                        */
/*      >  autoconf                                              */
/*     Autoconf requires the presence of the INIT file.          */
/*  3) Run ./configure                                           */
/*      >  ./configure                                           */
/*     This uses config.guess, config.sub and install-sh         */
/*     It generates config.log and config.status (which can both */
/*     be ignored) and it generates Makevars, which this file is */
/*     the template for.                                         */
/*  (c) Jonathan McDowell 2001                                   */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "../../../include/config.h"


/*
 *  System architecture
 */
/* Pending CXC configure symbols...*/

#if defined( __sparc) | defined( sparc )
#define UT_ARCH    "solaris"
#define UT_SYSTEM  "solaris"
#define UT_UNAME   "SunOS-5.8"
#define UT_BIGEND  1
#endif

#if defined(LINUX_X86) && !defined(__APPLE__)
#define UT_ARCH    "linux"
#define UT_SYSTEM  "i686-pc-linux-gnu"
#define UT_UNAME   "Linux-2.4.18-3"
#if __BYTE_ORDER == __BIG_ENDIAN
#define UT_BIGEND  1
#else
#define UT_BIGEND  0
#endif
#endif

#ifdef OSF1
#define UT_ARCH    "OSF1"
#define UT_SYSTEM  "alpha"
#define UT_UNAME    "alpha"
#define UT_BIGEND  0
#endif

#ifdef __APPLE__
#define UT_ARCH   "Apple"
#define UT_SYSTEM "Apple"
#define UT_UNAME  "Apple"
#if defined( __LITTLE_ENDIAN__ )
#define UT_BIGEND  0
#else
#define UT_BIGEND  1
#endif
#endif 

/* Numerical architecture */

#define UT_INT32T    unsigned int
#define UT_SZ_SHORT  SIZEOF_SHORT
#define UT_SZ_LONG   SIZEOF_LONG
#define UT_SZ_DOUBLE SIZEOF_DOUBLE
#define UT_SZ_FLOAT  SIZEOF_FLOAT
#define UT_SZ_INT    SIZEOF_INT
#define UT_SZ_CHAR   SIZEOF_UNSIGNED_CHAR
#define UT_SZ_P      SIZEOF_CHAR_P

