/*                                                                
**  Copyright (C) 2002-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmerror.h                                                   */
/*                                                               */
/*   This file contains global definitions that are accessable   */
/*   by both internal and external code.                         */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/


#ifndef DMDEF_H
#define DMDEF_H

/* Version info */
#define ASCDM_MAJOR_VERSION  2
#define ASCDM_MINOR_VERSION  10
#define ASCDM_VERSION       "2.10"



/* Map UTLIB to DM */

#define CXC 0

#define dmBool    int
/* Internal code only */

#include "dmcopyright.h"

#ifdef DMLIB_INTERNAL
#include "dmnutils.h"
#include <string.h>
#include <ctype.h>

#endif


#ifndef DMNUTILS_H
/* This duplicates the DT_ datatypes used internal to the DM.
   But we don't want to expose the DM internal dmnutils.h,
   and we want back compat with existing DM code. So this is
  loaded only if DMNUTILS_H is not already loaded.
 */

typedef enum { DT_VOID, DT_INT2, DT_PTR, DT_INT4, DT_REAL4, DT_REAL8, DT_CHAR, 
               DT_UINT1, 
               DT_LOGICAL, DT_UNK, DT_UINT2, DT_UINT4, DT_BIT } dmDataType;

#else

#define dmDataType DataType

#endif

/* Map public dmDataType to internal types */

#define dmUNKNOWNTYPE DT_UNK
#define dmSHORT       DT_INT2
#define dmLONG        DT_INT4
#define dmFLOAT       DT_REAL4
#define dmDOUBLE      DT_REAL8
#define dmTEXT        DT_CHAR
#define dmBYTE        DT_UINT1
#define dmBOOL        DT_LOGICAL
#define dmUSHORT      DT_UINT2
#define dmULONG       DT_UINT4
#define dmBIT         DT_BIT


typedef struct dmDataset_s    dmDataset;
typedef struct dmDescriptor_s dmDescriptor;
typedef struct dmBlock_s      dmBlock;
typedef struct dmSubspace_s   dmSubspace;  /* Newly promoted to allow user filtering */
typedef struct dmBuffer_s     dmBuffer;
typedef struct dmSubspace_s   dmRowFilter;
typedef struct dmRTSCache_s   dmRTSCache;


#define dmErrCode int
#define dmBool    int
/* Max number of components to a vector */
#define dmMAXDIM 10
#define dmMAXLEN 1024
#define dmTRUE  1
#define dmFALSE 0
#define dmSUCCESS 1
#define dmFAILURE 0

#define dmBADDIM -504
#define dmBADROW -501
#define dmBADBUFFER -560
#define dmENDOFTABLE -503



/* OPERAND: needed to make RESULT */
/* FILT: needed to calculate filter */
/* RESULT: calculated value after exec */
#define DM_FLAG_VISIBLE    0x01
#define DM_FLAG_OPERAND    0x02
#define DM_FLAG_FILT       0x04
#define DM_FLAG_RAW        0x08
#define DM_FLAG_RESULT     0x10
#define DM_FLAG_ROW        0x20
#define DM_FLAG_UNBUF      0x40
#define DM_FLAG_IMG        0x80
#define DM_FLAG_KEY        0x100
#define DM_FLAG_COORD      0x200
#define DM_FLAG_SUBSPACE   0x400
#define DM_FLAG_VECTOR     0x800
#define DM_FLAG_CPT       0x1000
#define DM_FLAG_COPY      0x2000
#define DM_FLAG_AXIS      0x4000
#define DM_FLAG_PHYS      0x8000
#define DM_FLAG_BIN     0x10000
#define DM_FLAG_BINNED  0x20000
#define DM_FLAG_LOGICAL 0x40000
#define DM_FLAG_TEMP    0x80000
#define DM_FLAG_FILT_OPERAND  0x100000
/* AUX_COLUMN identifies hidden columns that exist in the buffer (but
 * not in the block) to hold auxilary information for visible columns
 * (e.g. a variable-length array column has an AUX_COLUMN that contains
 * the length of each row).  They are copied with the buffer but not
 * flushed with the block. */
#define DM_FLAG_AUX_COLUMN    0x200000

#define dmMAXSTR 1024

#define DM_READ_WRITE 0
#define DM_READ_ONLY  1
#define DM_CREATE     2

#define DMF_NOPERM   -1
#define DMF_NOFILE    0
#define DMF_READONLY  1
#define DMF_READWRITE 2

/*
 *  Region file column ids.
 */
#define REGNCOLS 5
typedef enum { REG_POS_COL, REG_SHAPE_COL, REG_R_COL, REG_ROT_COL, REG_CPT_COL } dmRegColOffset;


/* Element types */

typedef enum 
{
 dmVALUE,
 dmINTERVAL,
 dmRANGE
} dmElementType;

typedef enum
{
 dmUNKBLOCK,
 dmTABLE,
 dmIMAGE
} dmBlockType;

#define dmUNKNOWNBLOCK dmUNKBLOCK

typedef enum
{
 dmBADDESC,
 dmKEY,
 dmSUBSPACE,
 dmCOORD,
 dmUNK1,   /* Obsolete */
 dmIMAGEDATA,   /* Obsolete - should replace with dmCOLUMN when
                  all CIAO programs checked to see if they explicitly use this */
 dmCOLUMN,
 dmUNK2 /* Obsolete */
} dmDescriptorType;


  
/*
 *  The possible binning types
 */
#define DM_BINS_NONE -1
#define DM_BINS_N    1
#define DM_BINS_LOG  2
#define DM_BINS_COMPACT 4
#define DM_BINS_FILE 8


#define dmNOMOREROWS    -504
/*
 *  Fatal errors
 */
/* Errors */
#define DMU_WARN           0
#define DM_ERR_NO_FILE      100
#define DM_ERR_NULL      101
#define DM_ERR_KERNEL    102
#define DM_ERR_MEM        103
#define DM_ERR_SYNC      104
#define DM_ERR_CLOBBER   105
#define DM_ERR_INDEX     106
#define DM_ERR_SYS_CALL  107
#define DM_ERR_INTERNAL  108
#define DM_ERR_BAD_NAME     109
#define DM_ERR_RANGE  110
#define DM_ERR_NO_MFORM       111
#define DM_ERR_BAD_METYP_DIM  112
#define DM_ERR_OPEN_FAIL      113
#define DM_ERR_GENERIC   114
#define DM_ERR_SUBSPACE  115
#define DM_ERR_BAD_TYPE  116
#define DM_ERR_NOT_IMP   117
#define DM_ERR_BAD_DIM       118
#define DM_ERR_BAD_DESC_TYPE 119
#define DM_ERR_COORD 120
#define DM_ERR_VECTOR 121

/*
 *  DM level fatal errors
 */
#define DM_ERR_NO_KERNEL     200
#define DM_ERR_NO_PERM       201
#define DM_ERR_PARSE         202

/*
 *  Warnings
 */
#define DM_WARN_NAME_CLASH     301
#define DM_WARN_PARSE_OPTION   302
#define DM_WARN_DATA_TRUNCATED 303
#define DM_WARN_NULL_REGION    304


/* Bit masks, initially implemented for field masking in dmDescriptorCompare.
 * To add a new comparison field, add a new macro and update dmDESCALL
 * accordingly (2^n - 1 for n fields) */

#define dmDESCNAME           0x01
#define dmDESCVALUE          0x02
#define dmDESCUNIT           0x04
#define dmDESCKIND           0x08    /* Kind of descriptor (key,col, etc)  */
#define dmDESCTYPE           0x10    /* Datatype of descriptor             */
#define dmDESCELMTYPE        0x20    /* Element type                       */
#define dmDESCDIM            0x40    /* Dimension                          */
#define dmDESCLENS           0x80    /* Axis lengths                       */
#define dmDESCELMDIM         0x100   /* Element dimensionality             */
#define dmDESCCPTNAMES       0x200   /* Component names                    */
#define dmDESCCMNT           0x400   /* Descriptor comment                 */
#define dmDESCALL            0x7FF   /* Compare all fields relvant to kind */


/* Back compat - deprecated */
#define dmBLOCKREF    400
#define dmGENERROR    DM_ERR_GENERIC
#define dmIRAFKERNEL  "IRAF"
#define dmFITSKERNEL  "FITS"
#define dmHISTORY     "HISTORY"
#define dmCOMMENT     "COMMENT"

#define dmDEFAULTBUFFERSIZE 	2048
#define dmTABLEBUFFERSIZE       "buffersize"

#define dmBINFAIL    2022 
#define dmNOMEM	     DM_ERR_MEM

#endif  /* DMDEF_H */


