/*                                                                
**  Copyright (C) 2002-2012  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmilib.h                                                    */
/*                                                               */
/*   Function prototypes and definitions                         */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

/* DMI routines */
#ifndef DMILIB_H
#define DMILIB_H

#include "dmulib.h"
#include "dmkernel.h"
#include "cxcregion.h"
#define DM_FILT_EPSILON 1.0E-6
#define DM_MAX_MEG_DEFAULT 50
#define DM_REGION_BIN_DEFAULT 0.25
#define DM_REGION_NMAX_DEFAULT 100000

#ifndef M_PI
/* pi -- needed for degree/radian conversions */
#define M_PI  3.14159265358979323846
#endif


typedef struct dmRangeList_s dmRangeList;
typedef struct dmFilter_s dmFilter;
typedef struct dmSubspaceData_s dmSubspaceData;
typedef struct dmIndexBlock_s dmIndexBlock;

typedef struct dmParseList_s dmParseList;
typedef struct dmCoordTransform_s dmCoordTransform;
typedef struct dmExecuteFunc_s *dmExecuteFunc;
typedef struct dmExecute_s *dmExecute;
typedef struct dmBinSpec_s dmBinSpec;


/* Dataset access methods */
char*       dmi_dataset_get_name( const dmDataset* ds );
long        dmi_dataset_get_n( const dmDataset* ds );
dmBlock*    dmi_dataset_get_block( const dmDataset* ds, const long no );
dmkDataset* dmi_dataset_get_kds( const dmDataset* ds );
long        dmi_dataset_get_current( const dmDataset* ds );
int         dmi_dataset_mode( dmDataset* ds );

/* Dataset modify methods */
void        dmi_dataset_set_current( dmDataset* ds, dmBlock* block );

/* Block access methods */

int            dmi_block_writeable( dmBlock* block );


char*          dmi_block_get_name( const dmBlock* block );
dmBlockType    dmi_block_get_raw_type( const dmBlock* block );
dmkBlock*      dmi_block_get_kb( const dmBlock* block );
dmDataset*     dmi_block_get_ds( const dmBlock* block );
long           dmi_block_get_nrows( dmBlock* block );
long           dmi_block_get_ncols( dmBlock* block );
dmDescriptor*  dmi_block_get_col( const dmBlock* block, const long colNo );

/* Special help for regions: see REG_*_COL symbols in dmdefs.h */
long           dmi_region_file_cols( dmBlock* block, dmDescriptor*** colsp, long** dimsp );

/* Block access methods for hints */

int            dmi_block_is_unscaled( dmBlock* block );

/* Block modify methods */

int            dmi_block_delete( dmDataset* ds, const long no );
void           dmi_block_set_open( dmBlock* block, const dmBool open );
void           dmi_block_set_name( dmBlock* block, const char* name );
void           dmi_block_set_type( dmBlock* block, const dmBlockType type );
void           dmi_block_set_kb( dmBlock* block, const dmkBlock* kb );
void           dmi_block_set_nrows( dmBlock* block, long nrows );
void           dmi_block_set_raw_cols( dmBlock* block, dmDescriptor** rawcols, long nrcols );
void           dmi_block_set_pref( dmBlock* block, char** pref, const long npref, const int flag );
void           dmi_block_close_descriptors( dmBlock* block, void (*DescriptorFree)( dmDescriptor* ));
void           dmi_block_make_default_subspace( dmBlock* block, const long maxcpts );
void           dmi_block_set_subspace_cpt( dmBlock* block, long cptno );
dmErrCode      dmi_block_set_hint( dmBlock* block, const char* name, const char* value );
void           dmi_block_set_update( dmBlock* block );
void           dmi_block_clear_update( dmBlock* block );
void           dmi_block_edit_subspace ( dmBlock * block, char *comstr);

/* Descriptor create/destroy */

dmDescriptor*  dmi_descriptor_alloc( dmBlock* block, const dmDescriptorType descType, const long no, const char* name, const dmDataType type, const char* unit, const char* desc, const long dim, const dmElementType etype, const long naxes, const long* axes, const long flag );
void           dmi_make_unscaled_groups( dmDescriptor** vectors, long nvectors );
dmDescriptor*  dmi_make_vector( dmBlock* block, char* name, dmDescriptor** ddlist, long dim );
dmDescriptor*  dmi_make_element_vector( dmBlock* block, char* name, dmDescriptor** ddlist, long dim, dmElementType etype, char* desc );

/* Descriptor access methods */
dmBlock*       dmi_descriptor_get_block( const dmDescriptor* dd );
char*          dmi_descriptor_get_name( const dmDescriptor* dd );
char*          dmi_descriptor_get_qualname( const dmDescriptor* dd, char* buf );
long           dmi_descriptor_get_no( dmDescriptor* dd );
long           dmi_descriptor_get_vector_dim( const dmDescriptor* dd );
long           dmi_descriptor_get_element_dim( const dmDescriptor* dd );
long           dmi_descriptor_get_array_dim( const dmDescriptor* dd );
long           dmi_descriptor_get_array_dimensions( const dmDescriptor* dd, long** axisLens );
long           dmi_descriptor_get_array_size( const dmDescriptor* dd );
long           dmi_descriptor_get_length( const dmDescriptor* dd );
long           dmi_descriptor_get_nvals( dmDescriptor* dd );
long           dmi_descriptor_get_bytes( dmDescriptor* dd );
char*          dmi_descriptor_get_desc( const dmDescriptor* dd );
char*          dmi_descriptor_get_disp( const dmDescriptor* dd );
char*          dmi_descriptor_get_unit( const dmDescriptor* dd );
dmDataType     dmi_descriptor_get_datatype( const dmDescriptor* dd );
dmElementType  dmi_descriptor_get_element_type( const dmDescriptor* dd );
void*          dmi_descriptor_get_kdd( const dmDescriptor* dd );
long           dmi_descriptor_get_flag( dmDescriptor* dd );
char*          dmi_descriptor_get_cpt_name( const dmDescriptor* dd, const long cptNo );
void           dmi_descriptor_get_name_with_cpts( const dmDescriptor* dd, char* fullname, const long maxlen );
long           dmi_descriptor_get_no_coords( const dmDescriptor* dd );
long           dmi_descriptor_get_no_groups( const dmDescriptor* dd );
void*          dmi_descriptor_get_aux( const dmDescriptor* dd);

dmDescriptor* dmi_descriptor_get_cpt( const dmDescriptor* dd, long no );
dmDescriptor* dmi_descriptor_get_parent( const dmDescriptor* dd );
dmDescriptor* dmi_descriptor_get_scaled( const dmDescriptor* dd );
dmDescriptor* dmi_descriptor_get_unscaled( const dmDescriptor* dd );
dmDescriptor* dmi_descriptor_get_coord_no( const dmDescriptor* dd, const long no );
dmDescriptor* dmi_descriptor_get_group_no( const dmDescriptor* dd, long no );
dmBool        dmi_descriptor_is_varlen( const dmDescriptor* dd );


/* Descriptor modify methods */

void dmi_descriptor_set_name( dmDescriptor* dd, char* name );
void dmi_descriptor_set_no( dmDescriptor* dd, const long no );
void dmi_descriptor_set_kdd( dmDescriptor* dd, const void* kernel_dd );
void dmi_descriptor_set_aux( dmDescriptor* dd, const void* aux);
void dmi_descriptor_set_unit( dmDescriptor* dd, const char* unit );
void dmi_descriptor_set_disp( dmDescriptor* dd, const char* disp );
void dmi_descriptor_set_length( dmDescriptor* dd, const long length );
int  dmi_descriptor_set_null( dmDescriptor* dd, dmDataType type, void* value );
void dmi_descriptor_set_range( dmDescriptor* dd, dmDataType type, void* tlmin, void* tlmax );
int  dmi_descriptor_set_cpt_range( const dmDescriptor* ddp, const long vcpt, const dmDataType type, void* tlmin, void* tlmax );
int  dmi_descriptor_set_bin( dmDescriptor* dd, dmDataType type, void* value );
void dmi_descriptor_add_coord( dmDescriptor* dd, dmDescriptor* ddc );
void dmi_descriptor_remove_coord( dmDescriptor* dd, long coord_no );
void dmi_descriptor_set_element_names( dmDescriptor* dd, dmElementType etype, char** enames, long dim );
void dmi_descriptor_set_varlen( dmDescriptor* dd, dmBool varlen );

void dmi_descriptor_clear_update( dmDescriptor* dd );
void dmi_descriptor_set_update( dmDescriptor* dd );


/* Col descriptor routines */

void dmi_col_print( const dmDescriptor* col, char* buf, const long maxlen );
void dmi_col_print2( const dmDescriptor* dd );

/* Coord descriptor create */

dmDescriptor* dmi_create_linear_wcs( dmBlock* block, dmDescriptor* col, char* name, dmDataType type, double val, double pix, double delt );

/* Coord descriptor access */

dmDescriptor* dmi_coord_get_parent( const dmDescriptor* dd );
long          dmi_coord_get_transform_no( dmDescriptor* dd );
long          dmi_coord_get_scalar_col_no( dmDescriptor* dd );
long          dmi_descriptor_get_group_axis_no( const dmDescriptor* dd );
double        dmi_coord_get_equinox( dmCoordTransform* t );
double        dmi_coord_get_latpole( dmCoordTransform* t );
double        dmi_coord_get_longpole( dmCoordTransform* t );
double        dmi_coord_get_theta( dmCoordTransform* t );
void          dmi_coord_get_poles( dmCoordTransform* t, double* poles );
dmDescriptor* dmi_coord_get_image( dmDescriptor* vector );
char*         dmi_coord_get_radesys( dmCoordTransform* t );

/* Coord descriptor modify */

int  dmi_coord_set_val( dmDescriptor* dd, const double* val, const long dim );
int  dmi_coord_set_pix( dmDescriptor* dd, const double* pix, const long dim );
int  dmi_coord_set_delt( dmDescriptor* dd, const double* delt, const long dim );
void dmi_coord_set_theta( dmDescriptor* dd, double theta );
void dmi_coord_set_longpole( dmDescriptor* dd, double value );
void dmi_coord_set_latpole( dmDescriptor* dd, double value );
void dmi_coord_set_equinox( dmDescriptor* dd, double value );
void dmi_coord_convert_lw_to_pw( dmDescriptor* dd, const dmDescriptor* phys );
void dmi_coord_convert_pw_to_lw( dmDescriptor* dd, const dmDescriptor* dd_lp, double* crpix, double* crval, double* cdelt, long dim );
void dmi_coord_set_transform_type( dmDescriptor* dd, char* type );
void dmi_coord_set_radesys( dmDescriptor* dd, char* radesys );


/* Subspace descriptor create */
dmDescriptor*  dmi_subspace_col_init( dmBlock* block, const char* total_name, dmDescriptor* col_dd, dmDataType type );
dmDescriptor*  dmi_column_create_subtable( dmBlock* block, dmDescriptor* parent );

/* Subspace descriptor methods */
int            dmi_filter_get_simple_range( dmDescriptor* rts_dd, long vcpt, double* vals );
int            dmi_parse_filter_string( char* dsval, dmDescriptor* dd, void** value1, void** value2, long* nvalsp, regRegion** regionp, int* pincnull );

void dmi_parse_subspace_line( dmBlock *block, dmSubspace *pSubspace, char *line);
void dmi_subspace_col_reset( dmBlock *block, dmSubspace *pSubspace, char *columnName, long min,
                             long max);



/* Key modify */

void           dmi_key_delete( dmDescriptor* dd );

/* Parsing and composing for kernel conventions */

void dmi_column_make_raw_name( char* name, long no, char* rawname );
int  dmi_wcs_make_ctype( char* name, char* ttype, long subaxis, char* ctype, int wcslib );

/* --------------------------------------------------------------- *
 * *************************************************************** *
 *  The routines below this line are not called by the kernels.
 *
 * --------------------------------------------------------------- 
 */

char* dmi_block_type_name( const dmBlockType blockType );

/* Dataset functions */

dmDataset*     dmi_dataset_alloc( const char* dsname, dmkDataset* kds );
dmErrCode      dmi_dataset_close( dmDataset* ds );
dmBlock*       dmi_block_alloc( dmDataset* ds, const long BlockNo, int virtual );
long           dmi_block_search( const dmDataset* ds, const char* name );
void           dmi_dataset_set_mode( dmDataset* ds, const int mode );
void           dmi_dataset_reset_block_no( dmDataset* ds, dmBlock* block, const long no );
void           dmi_dataset_set_block( dmDataset* ds, dmBlock* block, const long no );
char*        dmi_dataset_select_block_name( dmDataset* ds, char* name );

/* Block functions */

/* Block access methods for DMP */

void           dmi_block_set_keyno( dmBlock* block, long no );
dmKernel       dmi_block_kernel( dmBlock* block );
void           dmi_block_init( dmBlock* block );
void           dmi_block_close( dmBlock* block );
dmDescriptor*  dmi_column_create(dmBlock *block, char *name, dmDataType type, long length, char *unit, char *desc, dmElementType etype, char **cptNames, long vdim, long* axes, long naxes, dmBool varlen );
void           dmi_block_free( dmBlock* block );
dmBlockType    dmi_block_get_type( const dmBlock* block );
long        dmi_block_get_maxmeg( dmBlock* block );
int         dmi_block_is_null_primary( dmBlock* block );
long        dmi_block_get_npref( const dmBlock* block );
char*       dmi_block_get_pref( const dmBlock* block, const long no );
int         dmi_block_get_tstart( dmBlock* block, double* tstart, double* tstop );
dmBool      dmi_block_check_update( dmBlock* block );
dmDescriptor*  dmi_key_open( dmBlock* block, const char* name );
void           dmi_column_add_to_list( dmBlock* block, dmDescriptor* dd );
void           dmi_key_print( dmDescriptor* dd, char* pbuf );
void         dmi_block_print( dmBlock* block );

dmDescriptor** dmi_block_get_pref_list( const dmBlock* block, dmDescriptor** weightp, long* nprefp );
dmDescriptor*  dmi_block_get_raw_col( const dmBlock* block, const long colNo );
void           dmi_block_set_current_row( dmBlock* block, long row );
long           dmi_block_get_current_row( dmBlock* block );
long           dmi_block_reset_current_row( dmBlock* block );
void           dmi_block_reset_rows( dmBlock* block );
void           dmi_block_set_maxmeg( dmBlock* block, long maxmeg );
int            dmi_block_pref_default( const dmBlock* block );
void           dmi_block_update_gti_ccd_key( dmBlock* pBlock, char* tabname, long cpt );
void           dmi_list_vectors( dmBlock* block, dmBool filtered );
long           dmi_block_get_nkeys( const dmBlock* block );
long           dmi_block_get_no( const dmBlock* block );
void           dmi_block_set_buffer_maxsize( dmBlock* block, long size );

dmDescriptor*  dmi_block_search_identifier( const dmBlock* block, const char* name );
dmDescriptor*  dmi_block_search_public_identifier( const dmBlock* block, const char* name );
dmDescriptor*  dmi_block_search_col_identifier( dmBlock* block, const char* name );
dmDescriptor*  dmi_block_search_raw_col_identifier( dmBlock* block, const char* name );
void dmi_block_check_tstart( dmBlock* block ); 

/* Block Subspace */

dmSubspace*    dmi_block_get_subspace( const dmBlock* block );

int            dmi_block_merge_subspace(dmBlock* block0, dmBlock* block1);
int            dmi_block_apply_subspace(dmBlock *srcBlock, dmBlock *destBlock, dmBool copyDefault );
int            dmi_block_copy_subspace(dmBlock *srcBlock, dmBlock *destBlock);
void           dmi_subspace_free( dmSubspace* subspace, void (*kernel_free)( dmDescriptor* ) );

dmSubspace*    dmi_subspace_alloc( long ncols, long ncpts );
void           dmi_subspace_free_cache( dmSubspace* subspace );
void           dmi_subspace_add_cpt( dmSubspace* subspace );
void           dmi_subspace_add_col( dmSubspace* subspace, dmDescriptor* dd );
long           dmi_subspace_get_ncpts( dmSubspace* subspace );
long           dmi_subspace_get_ncols( dmSubspace* subspace );
dmDescriptor*  dmi_subspace_search( dmBlock* block, dmSubspace* rts, char* name, long* ty, int q );
void           dmi_subspace_combine_tabnames( dmSubspace* subspace, long CptNo, long lCptNo );
dmDescriptor*  dmi_subspace_col_no(      const dmSubspace* subspace, const long no );
int            dmi_subspace_remove_col_cpt( dmSubspace* subspace, long i, long CptNo  );
int            dmi_subspace_collapse( dmBlock* block, dmSubspace* subspace );
int            dmi_subspace_collapse_cpt(dmBlock* pBlock, dmSubspace* subspace, long CptNo, long CompareCptNo );
int            dmi_subspace_remove_cpt( dmSubspace* subspace, long CptNo );
int            dmi_subspace_apply( dmSubspace* srcSubspace, dmSubspace* destSubspace, dmBlock* destBlock, dmBool copyDefault, dmBool requireTable );
void           dmi_subspace_print_cpt( dmSubspace* subspace, long cptNo );
int            dmi_subspace_print( dmSubspace* subspace );
int            dmi_subspace_create_cpt(dmSubspace* subspace, int empty );
dmSubspace*    dmi_subspace_invert( dmSubspace* rts );
void           dmi_subspace_invert_intersect( dmSubspace* irts, dmFilter*** fdir, long ncols, long ncpts );
int            dmi_subspace_is_null( dmSubspace* subspace );

/* Block hints */
void           dmi_region_set_bin( dmBlock* block, double bin );
void           dmi_region_set_nmax( dmBlock* block, long nmax );
long           dmi_region_get_nmax( const dmBlock* block );
double         dmi_region_get_bin( const dmBlock* block );
void           dmi_block_set_unscaled( dmBlock* block, int q );
int            dmi_block_filter_is_compact( dmBlock* block );
void           dmi_block_set_filter_compact( dmBlock* block, int q );
int            dmi_block_get_subspace_update( dmBlock* block );
void           dmi_block_set_subspace_update( dmBlock* block, int q );
void           dmi_block_set_image_null( dmBlock* block, const char* value );
dmDataType     dmi_block_get_image_bin_type( dmBlock *block );
void           dmi_block_set_image_bin_type( dmBlock *block, dmDataType t );

/* Descriptor functions */

dmDescriptor*  dmi_descriptor_search( const dmDescriptor* dd, const char* name );
dmDescriptor*  dmi_descriptor_list_search_descriptor( const dmDescriptor** list, const long n, const char* name  );
dmDescriptor*  dmi_descriptor_list_search_vector( const dmDescriptor** list, const long n, const char* name );
dmDescriptor*  dmi_descriptor_list_search( const dmDescriptor** list, const long n, const char* name );
long           dmi_descriptor_match( dmDescriptor* dd, dmDescriptor** list, long n );

void           dmi_vector_make_cpts( dmDescriptor* dd, long trueDim );
void           dmi_descriptor_set_cpt( dmDescriptor* vector, long cptno, dmDescriptor* ddc );
long           dmi_descriptor_get_group_axis( dmDescriptor* dd, long groupno, long vcpt );
void           dmi_dd_print_flag( const dmDescriptor* dd, char* buf );
void           dmi_descriptor_free( dmDescriptor* dd );
dmDescriptorType dmi_descriptor_get_type( const dmDescriptor* dd );
int            dmi_descriptor_get_cpt_range( const dmDescriptor* ddp, const long vcpt, const dmDataType type, void* tlmin, void* tlmax );
dmDescriptor*  dmi_descriptor_copy( dmBlock* block, dmDescriptor* parent, dmDescriptor* dd );
void           dmi_descriptor_copy_kdd( dmDescriptor* dd1, dmDescriptor* dd2 );
void           dmi_descriptor_set_coord( dmDescriptor* w, const long no, dmDescriptor* p );
long           dmi_descriptor_get_index( const dmDescriptor* dd, long i );
dmDescriptor*  dmi_descriptor_alloc_vec( long dim, long vdim, dmElementType etype );
void           dmi_descriptor_set_datatype( dmDescriptor* dd, const dmDataType type);
void           dmi_descriptor_set_axes( dmDescriptor* col, const long* axlen, const long naxes );
dmBool         dmi_descriptor_get_range( const dmDescriptor* dd, void* tlmin, void* tlmax );
dmBool         dmi_descriptor_get_null( const dmDescriptor* dd, void* null );
dmBool         dmi_descriptor_get_nullval( const dmDescriptor* dd, void* null );
dmBool dmi_descriptor_has_nullval( const dmDescriptor* dd );
dmBool         dmi_descriptor_get_bin( const dmDescriptor* dd, void* bin );
int dmi_descriptor_get_cast_null( const dmDescriptor* dd, const dmDataType type, void* value );
int  dmi_descriptor_get_cast_bin( const dmDescriptor* pDesc, const dmDataType type, void* value );
void           dmi_descriptor_kernel_free( dmDescriptor* dd, void (*kernel_free)( dmDescriptor* ));
void           dmi_descriptor_copy_range( dmDescriptor* dd, dmDescriptor* new_dd );
int            dmi_descriptor_check( const dmDescriptor* dd );
dmBool         dmi_descriptor_check_update( dmDescriptor* dd );
void           dmi_compose_total_name( dmDescriptor* dd, char* buf );
void           dmi_descriptor_convert_type( dmDescriptor* dd, dmDataType type );

dmDescriptor* dmi_descriptor_get_physical_axis( const dmDescriptor* dd, const long axnum_target );
dmDescriptor* dmi_descriptor_get_logical_axis( const dmDescriptor* dd, const long axnum_target );
void           dmi_descriptor_set_axis( dmDescriptor* dd, long axis, dmDescriptor* axis_dd );
void           dmi_descriptor_set_groups( dmDescriptor* col, dmDescriptor** vectors, long ng );
long           dmi_group_get_axis_no( dmDescriptor* dd, long groupno );
void           dmi_descriptor_get_group_group_no( const dmDescriptor* ddp, dmDescriptor** parentp, long* groupnop, long* vcptp );
long           dmi_descriptor_get_groupno_of_axis( dmDescriptor* dd, long t_axisno, long* vcpt );
#if 0 /* ryan */
dmDescriptor*  dmi_descriptor_search_group( dmDescriptor* dd, char* name, long* cptNo, long* code );
#endif
dmDescriptor*  dmi_image_col_create( dmBlock* block, dmDataType type, long* axes, long naxes );
void           dmi_map_filtered_image_subset( dmDescriptor* image, long* llpix,  long* v_llpix );
void           dmi_image_filter_regions( dmDescriptor* image, dmDescriptor* raw_image, long* v_llpix, long* v_urpix, long* dpix, void* values, dmSubspace* rts );
dmFilter**     dmi_map_filters_to_image_groups( dmDescriptor* raw_image, dmSubspace* rts, int compact );
void           dmi_descriptor_axis_bounds( dmDescriptor* imagedata, long** ll, long** ur, long* size );

dmDescriptor*  dmi_key_read( dmBlock* block, const char* name, const dmDataType type, void* value );
dmDescriptor*  dmi_key_alloc( dmBlock* block, const long keyno, const char* name,                           const dmDataType type, const void* value, long nvals, const char* unit, const char* desc );

dmDescriptor*  dmi_col_alloc( dmBlock* block, const long no, const char* name, const dmDataType type, const char* unit, const char* desc, const long vdim, const dmElementType etype, const long naxes, const long* axes );
void           dmi_delete_col( dmDescriptor* dd );
dmDescriptor*  dmi_column_get_ss( dmBlock* pBl, dmDescriptor* col, long* cpt );
void           dmi_column_extremes(dmBlock *pBl, dmDescriptor* col, double* vals );


/* Subspace descriptor functions */

double         dmi_descriptor_get_area( const dmDescriptor* dd, double* field_area );
void           dmi_subspace_col_extremes( dmSubspace* subspace, dmDescriptor* dd, long cptno, dmDataType type, double* vals );
dmDescriptor*  dmi_descriptor_make_subspace( dmDescriptor* col_dd, dmSubspace* subspace );
void           dmi_subspace_col_set_values( dmDescriptor* dd, char* tabname, void* values1, void* values2, long nvalues, DataType type, long dim, int array, regRegion* region );
dmDescriptor*  dmi_subspace_col_create( dmBlock* block, dmSubspace* subspacep, char* name, char** cptnames, long dim, DataType type, char* unitp, dmDescriptor* col_dd, dmBool default_filter );
dmSubspaceData* dmi_ss_get_data( dmDescriptor* dd );
void           dmi_subspace_col_check_tabnames( dmDescriptor* dd, long ncpts );
dmDescriptor*  dmi_subspace_col_open( const dmSubspace* subspace, const char* name );
int            dmi_subspace_col_update_null(dmDescriptor* dd, char* name, long cptNo, dmDataType type, void* value1,		   void* value2, long nvalues, dmBool update );
int            dmi_subspace_col_update_cpt(dmDescriptor* dd, char* name, long cptNo, dmDataType type, void* value1, void* value2, long nvalues, int incnull, dmBool update );
int            dmi_subspace_col_init_cpt(dmDescriptor* dd, char* name, long cptNo, dmDataType type, void* value1, void* value2, long nvalues );
int            dmi_subspace_col_init_cpt_incnull( dmDescriptor* dd, char* name, long cptNo, dmDataType type, int incnull );
dmDescriptor*  dmi_subspace_col_cpt_parent( dmDescriptor* sub_dd );
dmBool         dmi_subspace_col_is_gti( dmDescriptor* pDesc, dmFilter* filter );
dmBool           dmi_subspace_col_map_range( dmBlock* raw, dmDescriptor* raw_dd, dmSubspace* rts, dmFilter* filter, long ty );
void           dmi_subspace_invert_entry_vcpt( dmDescriptor* dd, long vcpt, dmRangeList* range, dmRangeList* new_range );
void           dmi_subspace_col_invert_entry( dmDescriptor* dd, dmFilter*** fdir, dmSubspace *rts, long col, long cpt );
void           dmi_subspace_col_init_filters( dmSubspace* subspace, dmDescriptor* dd, dmFilter* filter );

dmFilter*      dmi_subspace_col_reset_filter( dmDescriptor* dd, long cpt );
void           dmi_image_subspace_set_bounds( dmDescriptor* ss_imdata, long axno, double xmin, double xmax );
void           dmi_subspace_col_make_null( dmDescriptor* dd );
dmFilter*      dmi_subspace_col_get_filter( dmDescriptor* dd, long cpt );
int            dmi_subspace_col_set_filter( dmDescriptor* dd, long cpt, dmFilter* f );
int            dmi_subspace_col_update_from_filter( dmDescriptor* indd, dmFilter* filter );

void           dmi_subspace_col_update_region( dmDescriptor* dmDSS, const char* tname, regRegion* region, dmBool update );
void           dmi_subspace_col_init_region( dmDescriptor* dmDSS, const char* tname, regRegion* region );
void           dmi_filter_update_region( dmDescriptor* dmDSS, const char* name, dmFilter* filter, regRegion* region, dmBool update );
regRegion*     dmi_subspace_col_get_region_pointer( dmDescriptor* dd, dmSubspace* subspace );
dmDescriptor*  dmi_subspace_col_create_region(dmBlock *block, char* name, char** colNames, dmDataType type, char* unit, regRegion* region );
regRegion*     dmi_subspace_col_alt_get_region_pointer( dmDescriptor* dd, dmSubspace* subspace );
void           dmi_subspace_col_region_area( dmDescriptor* dd, regRegion* region, double* rrarea, double* rfarea, int verbose );
int            dmi_subspace_col_get_values(dmDescriptor* filter_dd, long cptno, dmDataType type, void** value1, void** value2, long* nvalues );

/* Coord descriptor functions */

long           dmi_coord_get_group_no(dmDescriptor* dd);
int            dmi_coord_is_default_axis( dmDescriptor* dd );
int            dmi_coord_calc_d(  dmDescriptor* dd, double* value, double* result );
int            dmi_coord_invert_d( dmDescriptor* dd, double* value, double* result );
int            dmi_coord_transform_check_is_unit( dmDescriptor* dd, long dim );
dmCoordTransform* dmi_coord_transform_alloc( dmDescriptor* dd, char* transform, double* val, double* pix, double* delt, long dim );
long           dmi_coord_get_scalar_coord_no( dmDescriptor* dd );
char*          dmi_coord_get_transform_type( const dmDescriptor* dd );
int            dmi_coord_is_group( dmDescriptor* dd );
dmDescriptor*  dmi_coord_init(dmBlock* block, char* name, char* unit, char** cptNames, long dim, char* ttype, void* pix, void* val, void* delt, dmDataType dtype, long groupno, double* params);
void           dmi_coord_transform_set_scaled( dmCoordTransform *transform, long flag );

dmDescriptor*  dmi_coord_create(dmDescriptor* dd, char* name, char* unit, char** cptNames, long dim, char* ttype, void* pix, void* val, void* delt, dmDataType dtype, double* params);
long           dmi_coord_get_params( dmDescriptor* dd, double* params );
void           dmi_coord_make_transform_vector( dmDescriptor* dd );
void           dmi_coord_detach_cpts( dmDescriptor* dd );
dmDescriptor*  dmi_create_unit_wcs( dmBlock* block, dmDescriptor* col, char* name );
void           dmi_coord_get_transform_d( dmDescriptor* dd, double* crpix, double* crval, double* cdelt, long dim );
int            dmi_coord_transform_is_unit( dmDescriptor* dd );
dmDescriptor*  dmi_coord_make_descriptor(dmBlock* block, char* name, dmDataType dtype, char* unit, char** cptNames, long dim);
int            dmi_coord_get_transform_cast(dmDescriptor* dd, void* crpix, dmDataType dType, double* crval,double* cdelt,long dim);    
int            dmi_coord_set_transform(dmDescriptor* dd, void* crpix, dmDataType dType,double* crval,double* cdelt,long dim);    
int            dmi_coord_get_range( const dmDescriptor* group, const long vcpt, const dmDataType type, double* tlmin, double* tlmax );

int            dmi_coord_calc( const dmDescriptor* dd, void* value, dmDataType type1, dmDataType type2, void* result );
int            dmi_coord_invert( const dmDescriptor* dd, void* value, dmDataType type1, dmDataType type2, void* result );
int            dmi_coord_convert_world_region( dmDescriptor* vcoord, regRegion* region );
int            dmi_coord_transform_apply(  dmDescriptor* dd, double* invals, double* outvals, long dim );

/* CoordTransform routines */

int            dmi_coord_mink_coat(dmCoordTransform* wcsvals, const dmDescriptor* dd);
dmCoordTransform* dmi_coord_transform_copy( dmDescriptor* dd, dmCoordTransform* old, long dim );
dmCoordTransform* dmi_coord_transform_make(dmDescriptor* dd, char* ttype, double* pix, void* val, void* delt, long dim, double* params);
void           dmi_coord_transform_free( dmCoordTransform* transform );
int            dmi_coord_transform_image_bounds( double* bounds,  dmCoordTransform* wcsvals, long cptno, int invert );
void           dmi_coord_region_convert_init( dmDescriptor* dd );
void           dmi_coord_region_convert( double* world, double* pix );
void           dmi_coord_convert_region_phys( double* logic, double* pix );

/* Buffer functions */
long dmi_buffer_eval_size( dmBlock* block, long insize );


void dmi_buffer_list( dmBlock* block, dmBuffer* buf );
long dmi_buffer_offset( dmBuffer* buffer, long field_offset, long row_offset );
void dmi_buffer_print_data( dmBlock* block );
void dmi_buffer_print_row( dmBuffer* buffer, long rrowno, char* buf );
void dmi_buffer_print_vb( dmBlock* block, long vr1, long vr2  );
void           dmi_block_add_col( dmBlock* block, dmDescriptor* dd, long flag );
dmBuffer*      dmi_buffer_alloc( void );
long           dmi_buffer_get_col_offset( dmBuffer* buffer, dmDescriptor* dd );
long           dmi_buffer_get_cpt_offset( dmBuffer* buffer, dmDescriptor* dd );
void           dmi_buffer_print( dmBlock* block );
void           dmi_buffer_free( dmBuffer* buf );
void dmi_bin_map_offsets( dmBinSpec* binspec, dmDescriptor* parent_image, dmDescriptor* image );
long* dmi_calc_image_weights( long* axes, long naxes );
void dmi_copy_row_to_buffer( dmBlock* block, void* row, long rowno );
void dmi_copy_buffer_to_row( dmBlock* block, void* row, long rowno );
long dmi_calc_image_buffer_size( long* taxes, long naxes, long pnaxes, dmBinSpec* binspec, dmDataType type, long default_size, int force_buffered, int* is_buffered );
void dmi_binspec_free_cache( dmBinSpec* binspec );
dmDescriptor* dmi_buffer_get_sub_dd( dmSubspace* rts, dmDescriptor* dd );
void dmi_buffer_row_filter_extremes( dmDescriptor* sub_dd, long ncpts, long* minrow, long* maxrow );
void dmi_virtual_buffer_reset( dmBlock* block, dmBuffer* buffer );
void dmi_buffer_set_row_vals( long* values, long raw_row1, long raw_row2 );
long dmi_filter_rows( dmBuffer* buffer, void* data );
long dmi_buffer_index_flag( long flag );
void dmi_buffer_add_col( dmBuffer* buffer, dmDescriptor* dd, long flag );
void dmi_buffer_add_varlen_length_col( dmBlock* block, dmDescriptor* varlen_dd );
dmDescriptor* dmi_buffer_get_varlen_length_col( dmBlock* block, dmDescriptor* varlen_dd );
void dmi_buffer_write_varlen_length( dmBlock* block, dmDescriptor* dd, dmDescriptor* length_dd, void* dd_ptr, long length );
long dmi_buffer_read_varlen_length( dmBlock* block, dmDescriptor* dd, dmDescriptor* length_dd, void* dd_ptr );

dmSubspace* dmi_block_update_runtime_subspace( dmBlock* block, dmSubspace* rts );
char* dmi_buffer_index_flag_name( long flag );
void dmi_image_rebin_segment( dmDescriptor* image, void* image_buffer, long image_buffer_size, long* toffset, long* llpix, long* urpix, double* tstep,  long onaxes, void* data );
void dmi_buffer_calc_image( dmBlock* block, dmDescriptor* image, long nvrows_total );
int dmi_buffer_copy( dmBuffer* buffer1, dmBuffer* buffer2, void* data1, void* data2, long nvrows_total );
int dmi_buffer_copy_to_vdata( dmBuffer*  buffer, long nvrows_total );
long dmi_buffer_resync( dmBlock* block, dmBuffer* buffer, long nvrows );

dmExecute     dmi_execute_stack_alloc( long nmax );
void          dmi_buffer_execute_stack( dmBuffer* buffer, dmExecute execute, long nvrows, int filtered );
void          dmi_buffer_execute_eval( dmBuffer* buffer, long func, long* args, long nargs, long nvrows, int filtered );
void          dmi_buffer_add_execute_func( dmExecute* executep, long id, long* args, long nargs );
void          dmi_execute_stack_add( dmExecute execute, dmExecuteFunc func );
void          dmi_execute_stack_free( dmExecute execute );
void          dmi_execute_func_free( dmExecuteFunc func );
dmExecuteFunc dmi_execute_func_alloc( long id, long* args, long nargs );
void dmi_buffer_add_execute_coord( dmBuffer* buffer, dmExecute* execute, long id );

/* Filters */
dmFilter*      dmi_filter_alloc( long dim, dmDataType type );
long           dmi_filter_get_dim( dmFilter* filter );
dmDataType     dmi_filter_datatype( dmFilter* filter );
void           dmi_filter_get_tabname( dmFilter* filter, char* name, long maxlen );
void           dmi_filter_copy_tabname( dmFilter* cv, dmFilter* sv );
int            dmi_filter_set_tabname( dmFilter* vals, const char* name );
void           dmi_filter_set_region_string( dmFilter* filter, char* regstring );
void           dmi_filter_print_ranges( char* name, dmFilter* vals );
void           dmi_filter_print_region( char* name, dmFilter* filter );
void           dmi_filter_print( char* name, dmFilter* vals );
void           dmi_filter_setup( dmDescriptor* dd, dmFilter* filter, long vcpt, dmDataType vtype, void* value1, void* value2, long nvalues, char* tabname, regRegion* region );
int            dmi_filter_compare( dmFilter* filter1, dmFilter* filter2 );
int            dmi_filter_compare_for_combine( dmFilter* filter1, dmFilter* filter2, double* fieldx, double* fieldy, int* can_combine );
dmBool         dmi_filter_compare_ranges( dmFilter* filter1, dmFilter* filter2 );
void           dmi_filter_combine( dmFilter* filter1, dmFilter* filter2 );
int            dmi_filter_extract( dmFilter* filter, long cptno, dmDataType type, void** value1, void** value2, long* nvalues, int* pincnull );
void           dmi_filter_free( dmFilter* f );
void           dmi_filter_clear( dmFilter* f );
void           dmi_filter_zero( dmFilter* f );
dmFilter*      dmi_subspace_col_get_current_filter( dmDescriptor* dd );
void           dmi_filter_copy( dmFilter* p, dmFilter* f );
dmFilter*      dmi_filter_init( dmFilter* fstart, long dim, dmDataType type );
int            dmi_filter_is_implicit( dmFilter* filter );
dmBool         dmi_filter_range_is_default( dmDescriptor* colDesc, dmFilter* filter, long vcpt );
dmBool         dmi_filter_range_is_tstart( dmDescriptor* colDesc, dmFilter* filter, long vcpt );
dmBool         dmi_filter_region_is_default( dmFilter* filter );
void           dmi_filter_set_null( dmFilter* filter );
int            dmi_filter_is_null( dmFilter* filter );
void           dmi_filter_copy_cpt( dmFilter* p, dmFilter* f, long cpt );
dmFilter*      dmi_filter_intersect( dmFilter* ofilter, dmFilter* filter, int intersect, long cpt, long vdim );
void           dmi_filter_intersect_update( dmFilter* nfilter, dmFilter* filter, dmDataType type, int intersect, long cpt, int apply_empty );
void           dmi_filter_get_region_bounds( dmFilter* filter, double* boundx, double* boundy );
char*          dmi_filter_rename_tab( char* old_name, dmSubspaceData* ssdata, long ncpts, long cptno );
dmBool         dmi_filter_is_default( dmDescriptor* colDesc, dmFilter* filter );
dmDataType     dmi_filter_type( dmDataType type );
void           dmi_filter_print2( char* name, dmFilter* filter );
void           dmi_filter_extremes( dmFilter* filter, long cptno, dmDataType type, double* vals );
dmRangeList*   dmi_filter_get_range( dmFilter* filter, long cptno );
void           dmi_filter_add_range( dmFilter* filter, long cpt, void* specvals, int exclude );
void           dmi_filter_include_null( dmFilter* filter, long cpt );
void           dmi_filter_exclude_null( dmFilter* filter, long cpt );
void           dmi_filter_set_incnull( dmFilter* filter, long cpt, int incnull );
#if 0
void           dmi_filter_add_range_c( dmFilter* filter, long cpt, char** specvals, int exclude );
#endif
void           dmi_filter_get_cpt_bounds( dmFilter* filter, long vcpt, double* fieldx );
void           dmi_filter_copy_element( dmFilter* input_filter, dmFilter* output_filter, long output_vector_cpt );
int            dmi_filter_update( dmFilter* filter, long cptNo, char* name, char* ddname, dmDataType type, void* value1, void* value2, long nvalues, int incnull, dmBool update , int* remove, int* dirty );
int            dmi_filter_parse_typed_rangelist( char* name, dmDataType type, long mincpt, long maxcpt, char** pptr, dmFilter* filter, char* op, int loop, int *pincnull );

dmFilter**     dmi_filter_alloc_array( dmFilter** array, long dim, long start, long n );
void           dmi_filter_free_array( dmFilter*** fdir, long ncols, long ncpts );
void           dmi_filterlist_free( dmFilter** flist, long n );

regRegion*     dmi_filter_get_region( dmFilter* filter );
dmBool         dmi_filter_compare_region( dmFilter* src, dmFilter* cmp );
regRegion*     dmi_filter_alt_get_region( dmFilter* filter );
void           dmi_filter_update_region_with_field( dmFilter* filter, const char* name, regRegion* region, regRegion* l, double* fieldx, double* fieldy, dmBool update );

int            dmi_region_inside_bounds( regRegion* region, double* x, double* y );
int            dmi_region_is_rectangle( regRegion* region, double* x, double* y );

/* Rangelist routines */
dmRangeList*   dmi_rangelist_alloc_array( dmRangeList* array, long n );
void dmi_rangelist_init( dmRangeList* range );
void    dmi_rangelist_free( dmRangeList* r, int ischar );
int     dmi_rangelist_is_empty( dmRangeList* range );
int     dmi_rangelist_is_default( dmRangeList* range, dmDescriptor* colDesc, dmDataType type );
void    dmi_rangelist_compose_string( dmRangeList* range, dmDataType type, char** filter_buf );
void    dmi_rangelist_extremes( dmRangeList* dssVal, dmDataType type, double* vals );
void    dmi_rangelist_adjust( dmRangeList* vals, double* extremes );
void    dmi_rangelist_print2( char* name, dmRangeList* range, dmDataType type );

int     dmi_rangelist_compare( dmRangeList* srcVals, dmRangeList* cmpVals, dmDataType type );
void    dmi_rangelist_combine( dmRangeList* v, dmRangeList* u, dmDataType type );
long    dmi_rangelist_get_size( dmRangeList* range );
void    dmi_rangelist_print( dmRangeList* range, dmDataType type );
void    dmi_rangelist_set( dmRangeList* vals, dmDataType type, void* value1, void* value2,   long nvalues );
void    dmi_rangelist_add( dmRangeList* range, dmDataType type, void* vmin, void* vmax );
void    dmi_rangelist_set_incnull( dmRangeList* range, int incnull );
void dmi_rangelist_include_null( dmRangeList* range );
void dmi_rangelist_exclude_null( dmRangeList* range );
void dmi_rangelist_ignore_null( dmRangeList* range );
int  dmi_rangelist_get_incnull( dmRangeList* range );
void dmi_rangelist_add_rangelist( dmRangeList* orange, dmDataType type, dmRangeList* irange );
void    dmi_rangelist_pointers( dmRangeList* vals, void** value1, void** value2, long* n );
void    dmi_rangelist_range_pointers( dmRangeList* range, dmDataType type, void** value1, void** value2, long n );
void    dmi_range_pair_copy( void* mins, void* maxes, long n, dmDataType type, void** value1, void** value2 );
void    dmi_rangelist_extract( dmRangeList* vals, dmDataType type, void** value1, void** value2, long* length, int* pincnull );
int     dmi_rangelist_intersect( dmRangeList* oranges, dmDataType type, void* mins, void* maxes, long n, int incnull, int apply_empty );
int     dmi_rangelist_intersect_mode( dmRangeList* ranges, dmRangeList* oranges, dmDataType type, int intersect, int apply_empty );
int     dmi_rangelist_to_rectangle( dmRangeList* range1, dmRangeList* range2, char* buf );
void    dmi_rangelist_reduce( dmRangeList* range, dmDataType type );
void           dmi_rangelist_copy( dmRangeList* a, dmRangeList* b, dmDataType type );
void           dmi_rangelist_adjust_bounds( dmBlock* raw, dmRangeList* range, dmDescriptor* dd, long vcpt );
void    dmi_rangelist_invert_range( dmRangeList* orange, dmDataType type, void* vmin, void* vmax, void* tmin, void* tmax );
void dmi_rangelist_intersect_self( dmRangeList* range, dmDataType type );
void dmi_rangelist_invert( dmRangeList* range, dmDataType type, void* tmin, void* tmax );
void dmi_rangelist_invert_copy( dmRangeList* range, dmRangeList* new_range, dmDataType type, void* tmin, void* tmax );

/* Parsing */
char*         dmi_parse_filter_assign( char* itemp, long itemno, long* axisnop, char* name, char* op, char* ptr, char* tptr );
int           dmi_parse_special_syntax( char* name, char* op, long itemno, long* axisnop );
dmBool        dmi_subspace_col_apply_filter( dmBlock* block, dmSubspace* rts, dmDescriptor* sub_parent, long vcpt, long ty, int intersect, dmFilter* filter );
dmDescriptor* dmi_subspace_search_and_init( dmBlock* block, dmSubspace* rts, char* name, long* vcptp, long* typ, dmFilter** filterp );

char*         dmi_region_check_bounds_syntax( char* ptr, int* extent, int* is_reg );
regRegion*    dmi_region_bound( regRegion* region, double* fieldx, double* fieldy );
regRegion*    dmi_region_resolve( char* ptr, regRegion* (*read_file)(char*), double* fieldx, double* fieldy );
void          dmi_filter_check_regions( dmDescriptor* rts_dd, long ncpts );

void dmi_block_setup_runtime_filter( dmBlock* block, dmBuffer* buffer, dmSubspace* rts );
void          dmi_parse_list_free( dmParseList* list );
dmParseList*  dmi_parse_split( const char* buf );
int           dmi_parse_token_type( char** ptr, int* compact );
dmParseList*  dmi_block_parse_syntax( dmDataset* ds, const char* vspec, dmBlockType blockType );
long          dmi_parse_bin_spec( char* spec, double* specvals, long retval, int is_integer );
int dmi_binning_is_one( dmDescriptor* dd, char* spec, char* name, char* filter );
dmDescriptor* dmi_parse_bin_token( dmBlock* block, char* item, long nitems, char** specp );
char* dmi_check_binning_special_case( dmBlock* block, const char* bin_spec, int* specialp );
void         dmi_block_open_set_default_select(  dmBlock* block, dmParseList* list );
void         dmi_block_open_set_binning( dmBlock* block, dmParseList* list, dmBlockType btype );
void dmi_block_apply_select( dmBlock* block, char* select );
void dmi_map_image_filter_bounds( dmDescriptor* raw_image, dmFilter** filter_list, long* image_mins, long* image_maxes, long naxes, int compact, int reset );
long dmi_axis_steps( double* vals, long type );
void dmi_get_binning_defaults( dmBlock* block, dmDescriptor* col, long retval, double* vvals );
void dmi_adjust_binning_edges( char* name, long vcpt, double* vals, int is_integer );
long dmi_preparse_bin_spec( char* inspec, int compact, char** spec );
long dmi_eval_bin_spec( dmBlock* pBl, dmDescriptor* col, char* inspec, double** vvals, long* gdim, int compact, int is_integer );
dmDescriptor* dmi_parse_bin_filter( dmBlock* block, const char* bin_spec );
int dmi_coord_list_copy( dmDescriptor* col_dd, dmDescriptor* dest );

dmBool dmi_descriptor_compare( dmDescriptor* dd1, dmDescriptor* dd2, long mask );
long dmi_array_check_range( dmDescriptor* dd, long* llpix, long* urpix );
int dmi_block_update_time_range( dmBlock* block, const char* name, double value );


void dmi_binspec_free( dmBinSpec* binspec );
dmBinSpec* dmi_block_get_binspec( dmBlock* block );
void dmi_image_descriptor_clear( dmBlock* block, void (*DescriptorFree)( dmDescriptor* ));
dmDataType dmi_parse_type( const char* value );

void dmi_alloc_to_list( void*** listp, long* n, long* nmax, void* p, long no );
void dmi_ddlist_free( dmDescriptor** list, long n );
void dmi_block_alloc_row_dd( dmBlock* block );


long dmp_buffer_read_offset( dmBlock* block, long col_offset, long vrowno, long nvrows, void* values, long item_offset, long item_nbytes, long vrow_out_offset, long* llpix, long* urpix, long* dpix );

/* Index block routines */




void dmi_dataset_block_free( dmDataset* ds, const long no );
long dmi_dataset_block_alloc( dmDataset* ds, const long no );
void dmi_dataset_set_block_type( dmDataset* ds, long no, const dmBlockType type );
dmBlockType dmi_dataset_get_block_type( const dmDataset* ds, const long no );
dmIndexBlock* dmi_ds_index( const dmDataset* ds, const long no );
void dmi_dataset_set_block_name( dmDataset* ds, const long no, const char* name );
char* dmi_dataset_get_block_name( const dmDataset* ds, const long no );
long dmi_dataset_get_nv( const dmDataset* ds );
void dmi_descriptor_set_desc( dmDescriptor* dd, const char* desc );

dmDescriptor* dmi_block_get_key( dmBlock* block, const long no );
void dmi_descriptor_dump_axes( dmDescriptor* image );
int dmi_descriptor_check_groups( dmDescriptor* dd );
void dmi_key_free_aux( dmDescriptor *dd );
void dmi_key_alloc_aux( dmDescriptor *dd, const void *value, const long nvals );
dmDescriptor* dmi_block_get_scalar_col( dmBlock* block, long scalar_no );
dmDescriptorType dmi_dd_effective_type( dmDescriptor* dd );
int dmi_descriptor_silent_groups( dmDescriptor* dd );
void dmi_buffer_status( dmBlock* block );


/* Pixel-Mask routines */

regRegion* dmMaskParse(char *regstring);
void dmi_mask_set_masktag_first(char *spec);
int dmi_mask_isMatching_ddAndCoords(regShape *shape, int coord, char *name);


#endif





