/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmiparser.h                                                 */
/*                                                               */
/*   This file contains Struct definitions and function          */
/*   prototypes.                                                 */
/*                                                               */
/*                                                               */
/*****************************************************************/

/* Maximum number of [] sections allowed in a DM specification */
#define DMP_MAXENTRY   20

#define dmFBAD_PARSE -2
#define dmVECPARSE    3
#define MAX_NAME_LENGTH 80
#define MAX_BUF_LENGTH 1024
            
/* Structures for now, for handling binning - may go away */
typedef struct {
 dmDescriptor* col;
 char* spec;
 double** vals;
} BinnedCol;
             
typedef struct {
 BinnedCol* data;
 long n;
 int compact;  /* Do we bin tight around the filter? */
} BinnedTableSpec;

typedef struct {
 char** clauses;
 long n;     
} dmClauseList;

typedef struct
{
 long op;
 long arg[2];
 char* ptr;
} dmParseStackEntry; 
 
typedef struct {
 dmParseStackEntry* entry;        /* Algebra eval stack */
 long ns;
 long nsmax;
 long* id;
 long* expect;
 long group_no;
 long next;
 long clause_depth;
 int init;
} dmParseStack;
 
typedef struct {
 dmParseStack* stack;
 dmSubspace* rts;
} dmParseState;
 

void            dmi_clauselist_free( dmClauseList *c );
int             dmi_parse_clause_stack( char* buf, dmParseStack* stack );
dmClauseList*   dmi_parse_stack_entry_eval( dmParseStack* stack, long* estack, dmBool* eval, long* depth, long id );
dmClauseList*   dmi_parse_stack_eval( dmParseStack* stack );
int             dmi_parse_clause_line( char* buf, dmParseStack* stack );

void            dmi_parse_stack_format_name( dmParseStack* stack, long arg1, char* narg1 );
void            dmi_parse_state_disp( dmParseStack* stack );
long            dmi_parse_stack_arg( dmParseStack* stack, long id );
void            dmi_parse_external_ascii_filter( char* filename, dmParseStack* state );
dmParseState*   dmi_parse_state_init( long n );
dmParseStack*   dmi_parse_stack_init( long n );

dmParseStackEntry*  dmi_parse_stack_entry( dmParseStack* stack, long no );
void            dmi_parse_stack_free( dmParseStack* Stack );
void            dmi_parse_stack_set( dmParseStack* stack, long id, long op, long arg1, long arg2 );
long            dmi_parse_stack_push( dmParseStack* stack, long op, long arg1, long arg2 );
long            dmi_parse_stack_arg_push( dmParseStack* stack, long op, char* ptr );
int             dmi_parse_test_clause_open( char* ptr );
int             dmi_parse_operator( char* ptr );
int             dmi_parse_clause_operator( char* ptr );

int dmi_parse_rangelist( dmDescriptor* dd, long vcpt, char** pptr, dmFilter* filter, char* op, int loop );


/* DMP layer */

int dmp_parse_ext_filter( dmBlock* RawBlock, char* buf, dmParseState* state );
void dmp_apply_external_ascii_filter( dmBlock* RawBlock, char* filename, dmParseState* state );
void dmp_parse_filter_line( dmBlock* RawBlock, char* buf, dmParseState* state );
