/*                                                                
**  Copyright (C) 2002-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmnutils.h                                                  */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#ifndef DMNUTILS_H
#define DMNUTILS_H

#define JCMLIB "JCMLIB UTILS"
#define JCMLIBVER "CV2.0"

/* V0.1 1985 Nov    Initial version */
/* V1.1 1997 Feb 19 Made consistent use of long int */
/* V1.2 1997 Mar  9 Moved all prototypes into jcmproto.h */
/* V1.3 1997 Mar 10 Fixed SIO and TFR bugs, unitlib bug */
/* V1.4 1997 May  2 Added new allocate routines, vsub_d */
/* V1.5 1997 Nov 27 Cleaned up, added const declarations, moved stuff back to jcmlib.h */
/* V2.0 2001 Jun  2 Rewrite */
/* V2.1 2002 Jul  1 Revision for DM use */

#define UT_ERR_START 1000
#include "dmconfig.h"
#include "dmlimits.h"
#include <stdlib.h>
#include <stdio.h>
/* 
 * JCMLIB global symbols
 */

#undef TRUE
#undef FALSE
#define TRUE  1
#define FALSE 0

#define UT_TRUE  1
#define UT_FALSE 0

#define UT_NULL_CHAR '\0'

#define CARD_SIZE 80
#define FN_SIZE  512
#define TEXT_SIZE 1024
#define WORD_SIZE   31
#define UT_LIST_SIZE 4096

enum TMODE_OPTIONS { TMODE_WORD, TMODE_STRING, TMODE_LINE };
#define UT_INTERSECT_ERR 0
#define UT_INTERSECT_NULL -1
#define UT_INTERSECT_OK 1

/*
 *  File symbols
 */
#define FIO_DEFAULT_IN  5
#define FIO_DEFAULT_OUT 6
#define FIO_DEFAULT_ERR 7

#define FIO_NOPERM   -1
#define FIO_NOFILE    0
#define FIO_READONLY  1
#define FIO_READWRITE 2
#define FIO_FILE_STD  1
#define FIO_FILE_DIR  2
#define FIO_FILE_CDEV 3
#define FIO_FILE_BDEV 4
#define FIO_FILE_FIFO 5
#define FIO_FILE_SOCKET 6

#define UT_FILE_CLOSED 0
#define UT_FILE_OPEN   1
#define UT_FILE_EOF   -1
#define UT_FILE_ERR    2

#define FIO_CLEAR     0
#define FIO_CHECK     1
#define FIO_INPUT  0
#define FIO_OUTPUT 1
#define FIO_ERR    2

#define UT_CHAR_UMASK 255
#define UT_CHAR_MASK  127
#define UT_CHAR_ERR  -1
#define UT_CHAR_NULL  0
#define UT_CHAR_CTRLA 1
#define UT_CHAR_CTRLB 2
#define UT_CHAR_CTRLC 3
#define UT_CHAR_CTRLD 4
#define UT_CHAR_CTRLE 5
#define UT_CHAR_CTRLF 6
#define UT_CHAR_BEEP  7
#define UT_CHAR_BS    8
#define UT_CHAR_HT    9
#define UT_CHAR_LF   10
#define UT_CHAR_VT   11
#define UT_CHAR_FF   12
#define UT_CHAR_CR   13
#define UT_CHAR_SO   14
#define UT_CHAR_ESC  27
#define UT_CHAR_SPACE 32


#define UT_SZ_B 1
#define UT_SZ_R UT_SZ_FLOAT
#define UT_SZ_I UT_SZ_LONG
#define UT_SZ_C UT_SZ_CHAR
#define UT_SZ_S UT_SZ_SHORT
#define UT_SZ_D UT_SZ_DOUBLE
#define UT_SZ_Q UT_SZ_INT

/* Max no of digits in an long in read_val_i */
#define UT_MAX_DIGITS 10
#define UT_SXG 60
#define UT_CIRCLE 360.0
#define UT_DEG_RAD  ( 2 * M_PI / UT_CIRCLE )

#define FIO_POP_CLOSE  1
#define FIO_POP_ONLY  -1

/* Global zero */
extern const double VZERO_D[3];

/* Special chars */




/* Special matrix macros */
#define ar_lmat22_d(m,i,j)   (m[2*(i-1)+(j-1)])
#define ar_lmat33_d(m,i,j)   (m[3*(i-1)+(j-1)])
#define ar_lmatnn_d(m,n,i,j)   (m[n*(i-1)+(j-1)])



/*
 *   1,1  0
 *   1,2  1
 *   1,3  2
 *   2,1  3
 *   2,2  4
 *   2,3  5
 *   3,1  6
 *   3,2  7
 *   3,3  8
 */

/* JCM special types */

/*
 *
 *  JCMLIB types
 *  In general I have avoided the use of special types, which tend to
 *  make a general purpose library less reusable.
 *  For example, 3-vector arguments are just double* and you must read the
 *  documentation to know that you need a double[3].

 *  The 'logical' type is a typedef for int and is used for 
 *  boolean values.

 *
 *  There are also some special string types:TextWord for short strings,
 *  TextCard for 80 byte ASCII cards, TextBuf for long strings, and Filename
 *  for pathnames. These are never used in the external prototypes,
 *  but you may want to use FSIZE to check if your pathnames will exceed
 *  JCMLIB's capacity.
 *
 *  Note that all string tests in the library ignore trailing blanks;
 *  and a string of blanks is considered the same as a null string.
 *  This improves compatibility with Fortran programs.
 */



typedef int logical;    /* C standard specifies result of logical expr is int */

#define MAKE_CHARS_UNSIGNED 0
#if MAKE_CHARS_UNSIGNED == 1
#define character unsigned char
typedef unsigned char* string;
#else
#define character char
/* typedef char* string; */
#endif

/* These are always explicitly unsigned */
#define uchar unsigned char
typedef unsigned char* ustring;

/* New convention: types are capitalized, sizes are prefixed */
typedef char TextCard[CARD_SIZE+1];
typedef char Filename[FN_SIZE+1];
typedef char TextWord[WORD_SIZE+1];
typedef char TextBuf[TEXT_SIZE+1];

typedef struct { float x;   float y; }   complex;
typedef struct { double x; double y; } dcomplex;

/* The UnitDomain object */
typedef struct UnitDomain_s     *UnitDomain;
/* The ErrorClasses object */
typedef struct ErrorClasses_s  *ErrorClasses;
typedef struct ErrorData_s     *Error;
typedef struct ErrorGlobal_s   *ErrorGlobal;
/* The File IO objects */
typedef struct FioFileList_s   *FioFileList;
typedef struct FioFileObject_s *FioFile;
typedef struct FioStackData_s  *FioStack;
typedef struct FioPager_s      *FioPager;

typedef struct Tpio_s          *Tpio;
#if 0
typedef void* Tpio;  /* Dummy out TPIO system for this version */
#endif
/* Generic type support */
typedef enum { DT_VOID, DT_INT2, DT_PTR, DT_INT4, DT_REAL4, DT_REAL8, DT_CHAR, 
               DT_UINT1, 
               DT_LOGICAL, DT_UNK, DT_UINT2, DT_UINT4, DT_BIT } DataType;
typedef double GenericType;


/* Parser support */

typedef enum p_ctype { PTT_Null, PTT_Control, PTT_Special, PTT_Space, 
       PTT_LPar, PTT_RPar,
       PTT_Op, PTT_Sign, PTT_Comma, PTT_Point, PTT_Rel, PTT_SLPar, PTT_SRPar,
       PTT_Underbar, PTT_Digit, PTT_UC_Let, PTT_LC_Let,
       PTT_Letter, PTT_Letters, PTT_Alphanumeric, 
       PTT_long, PTT_float, PTT_float_E, PTT_Double, PTT_Logical,
       PTT_Word, PTT_Number, PTT_Symbol } ParseTokenType;

/* Dynamically expanding vector of names.
   The names array of pointers is reallocated in large chunks for efficiency */
typedef struct NameList_s {
 char** data;
 long n;
 long nmax;
 long size;
} *NameList;

typedef struct PathList_s {
 NameList local;
 NameList global;
} *PathList;

typedef struct AliasList_s {
 char** name;
 char*** cpts;
 long* dim;
 long n;
 long nmax;
} *AliasList;

typedef struct FileDirectory_s *FileDirectory;


typedef struct MixArray_s {
 void* data;
 DataType type;
 DataType* types;
 long size;
 long n;
 long nmax;
} *MixArray;

typedef struct FileBuffer_s {
 char* data;
 long size;         /* Bytes in buffer */
 long top;          /* Bytes filled */
 long chunk_size;   /* Bytes to I/O at once */
 long* line;          /* Array of pointers to start of each line */
/* These are long offsets so we can realloc the data without recalc */
 long nmax;     /* Size of pointer array */
 long n;            /* Number of lines found */
 long pos;      /* Current line */
} *FileBuffer;


/*
 *   JCMLIB routine prototypes
 *  --  GENERAL: --
 *   1. Initialization
 *   2. System related routines
 *   3. Bit related functions
 *  --  STRINGS: --
 *   4. Converting between strings and numbers.
 *   5. Basic string manipulation
 *   6. Substrings
 *   7. String cleanup routines
 *   8. F90 style string functions
 *   9. Miscellaneous string functions
 *  10. Token parsing
 *  --  NUMBERS: --
 *  11. Sorting
 *  12. Arithmetic and Math
 *  13. Sexagesimal functions
 *  14. Special functions
 *  15. Complex number routines
 *  16. Extreme value handling
 *  17. Random numbers.
 *  18. Circular and trig functions
 *  19. 3-vector routines
 *  --   ARRAYS:  ---
 *  20. Array memory management
 *  21. Basic array operations
 *  22. F90 style array operations
 *  23. Logical arrays
 *  --   INPUT AND OUTPUT --
 *  24. General file I/O (FIO)
 *  25. Byte streams (SIO)
 *  26. Raw char streams (CIO)
 *  27. Text files (FIO)
 *  28. Message streams (IOFS)
 *  29. Command line interface (TPIO)
 *  30. Escape commands (ESC)
 *  31. Debug routines (DEBUG)
 *  32. Error handling (ERR)
 *  --   MISCELLANEOUS --
 *  33. Units
 *  34. Another string parser
 *  35. Fortran strings
 */

#define spharc_r utm_spharc_r
#define spharc_d utm_spharc_d
#define dmcos utm_cos
#define dmsin utm_sin
#define rcosd utm_rcosd
#define rsind utm_rsind
#define dcosd utm_dcosd
#define dsind utm_dsind
#define dacosd utm_dacosd
#define dasind utm_dasind
#define racosd utm_racosd
#define rasind utm_rasind
#define iplus utm_iplus
#define rotate2_d utm_rotate2_d
#define argd_d utm_argd_d
#define argd_r utm_argd_r
#define radin_d utm_radin_d
#define radin_r utm_radin_r
#define radout_d utm_radout_d
#define radout_r utm_radout_r
#define circ_d utm_circ_d
#define circ_r utm_circ_r
#define circ_ew_r utm_circ_ew_r
#define circ_ew_d utm_circ_ew_d
#define circ_rad_r utm_circ_rad_r
#define circ_rad_d utm_circ_rad_d
#define circ_ew_rad_r utm_circ_ew_rad_r
#define circ_ew_rad_d utm_circ_ew_rad_d
#define nint_d utm_nint_d
#define nint_r utm_nint_r
#define rpow utm_rpow
#define rlog10 utm_rlog10
#define rabs utm_rabs
#define rsqrt utm_rsqrt
#define lint_r utm_lint_r
#define lint_d utm_lint_d
#define order_r utm_order_r
#define order_d utm_order_d
#define round_r utm_round_r
#define round_d utm_round_d
#define frac_r utm_frac_r
#define frac_d utm_frac_d
#define logbase_i utm_logbase_i
#define logbase_r utm_logbase_r
#define logbase_d utm_logbase_d
#define rmod_i utm_rmod_i
#define rmod_s utm_rmod_s
#define rmod_d utm_rmod_d
#define rmod_r utm_rmod_r
#define divisible utm_divisible
#define min_i utm_min_i
#define max_i utm_max_i
#define modulo_i utm_modulo_i
#define modulo_s utm_modulo_s
#define modulo_r utm_modulo_r
#define modulo_d utm_modulo_d
#define floor_r utm_floor_r
#define floor_d utm_floor_d
#define ceiling_r utm_ceiling_r
#define ceiling_d utm_ceiling_d
#define min_d utm_min_d
#define max_d utm_max_d

#define hexagam utm_hexagam
#define acych utm_acych
#define acyc  utm_acyc
#define E1_func utm_E1_func
#define dplanckn utm_dplanckn
#define ipow_r utm_ipow_r
#define ipow_d utm_ipow_d
#define dgauss utm_dgauss
#define tpow_r utm_tpow_r
#define tpow_d utm_tpow_d
#define acosh utm_acosh
#define asinh utm_asinh
#define atanh utm_atanh
#define esinh utm_esinh
#define eexp utm_eexp

#define clip_d utm_clip_d
#define clip_r utm_clip_r
#define overlap_d utm_overlap_d
#define overlap_r utm_overlap_r
#define rlap_check_d utm_rlap_check_d
#define rlap_check_r utm_rlap_check_r
#define poly_d utm_poly_d
#define poly_r utm_poly_r
#define bose_einstein utm_bose_einstein
#define ecosh utm_ecosh
void ut_init_jcmlib( void ); 
void ut_free_jcmlib( void ); 
void ut_ver_jcmlib( char* ver, const long maxlen ); 
logical ut_ver_init( const char* prog, const char* ver ); 
void ut_ver_msg( void ); 
void ut_id_jcmlib( char* label, const char* ver ); 
void sys_filepath( const char* dir, const char* name, const char* ext, char* buf, long siz ); 
void sys_pathstrip( const char* path, char* dir, char* name, const long maxlen ); 
void sys_os( char*  ); 
void sys_getenv( const char* name, char* value ); 
void sys_delete( const char* name ); 
void sys( const char* cmd ); 
void sys_type( const char* filename ); 
void sys_date_and_time( char* date, char* ctime, char* zone ); 
void sys_date( char* buf, const long size ); 
void sys_time( char* buf, const long size ); 
void sys_today( char* buf, const long size ); 
void sys_wait( const long t ); 
double sys_stamp( void ); 
void bytes_split_i( const long value, long* bytes ); 
void bytes_split_r( const float value, long* bytes ); 
void bytes_split_d( const double value, long* bytes ); 
void bytes_split_s( const short value, long* bytes ); 
void bytes_split_c( const char* value, long* bytes, const long n_bytes ); 
long bytes_combine_i( const long* bytes ); 
short bytes_combine_s( const long* bytes ); 
float bytes_combine_r( const long* bytes ); 
double bytes_combine_d( const long* bytes ); 
void bytes_combine_c( const long* bytes, char* value, const long n_bytes ); 
void byte_set_s( short* number, const long value, const long byte ); 
void byte_set_i( long* number, const long value, const long byte ); 
void byte_set_r( float* number, const long value, const long byte ); 
void byte_set_d( double* number, const long value, const long byte ); 
void byte_set_c( char* data, const long value, const long byte ); 
long byte_get_d( const double value, const long byte ); 
long byte_get_s( const short value, const long byte ); 
long byte_get_r( const float value, const long byte ); 
long byte_get_i( const long value, const long byte ); 
long byte_get_c( const char* value, long byte ); 
unsigned short bit_single_s( const long bit_no ); 
unsigned long bit_single_i( const long bit_no ); 
void bit_set_i( long* value, const long bit_no ); 
void bit_set_s( short* value, const long bit_no ); 
void bit_clr_i(  long* value, const long bit_no ); 
void bit_clr_s(  short* value, const long bit_no ); 
short bit_shft_s( const short value, const long bits ); 
long bit_shft_i( const long value, const long bits ); 
logical bit_test_i( const long value, const long bit ); 
logical bit_test_s( const short value, const long bit ); 
long bit_unpack_i( const long value, const long start_bit, const long n_bits ); 
long bit_unpack_s( const short value, const long start_bit, const long n_bits ); 
short bit_ar_set_s( const long* bits ); 
long bit_ar_set_i( const long* bits ); 
float bit_ar_set_r( const long* bits ); 
double bit_ar_set_d( const long* bits ); 
long bit_ar_get_r( const float value, long* bits ); 
long bit_ar_get_d( const double value, long* bits ); 
long bit_ar_get_s( const short value, long* bits ); 
long bit_ar_get_i( const long value, long* bits ); 
long bits_unsigned( const short value ); 
long bits_val( const long* bits, const long n ); 
void bit_write( const long* bits, const long nbits, const char mode, char* buf ); 
double cs_read_binary_d( const char* buf, const char mode ); 
float cs_read_binary_r( const char* buf, const char mode ); 
long cs_read_binary_i( const char* buf, const char mode ); 
short cs_read_binary_s( const char* buf, const char mode ); 
void cs_write_binary_d( const double value, char* buf, const char mode ); 
void cs_write_binary_r( const float value, char* buf, const char mode ); 
void cs_write_binary_s( const short value, char* buf, const char mode ); 
void cs_write_binary_i( const long value, char* buf, const char mode ); 
void ieee_get_i( const char* buf, long* data, const long n ); 
void ieee_get_s( const char* buf, short* data, const long n ); 
void ieee_get_d( const char* buf, double* data, const long n ); 
void ieee_get_r( const char* buf, float* data, const long n ); 
void ieee_put_r( const float* data, char* buf, const long n ); 
void ieee_put_d( const double* data, char* buf, const long n ); 
void ieee_put_s( const short* data, char* buf, const long n ); 
void ieee_put_i( const long* data, char* buf, const long n ); 
void ut_dbg_set( const long level ); 
logical ut_dbg_level( const long i );	 
long ut_dbg_get( void ); 
void ut_msg_d( const long level, const char* buf );  
void ut_dbg_msg_tag( const long level, char* buf ); 
void ut_error_init( char* name ); 
void ut_error( char* buf, long code ); 
void ut_error_status(void); 
void ut_error_set( Error* old_error, char* text, long code, long auxcode ); 
long ut_error_code( Error error ); 
void ut_error_disp( Error* error, ErrorClasses error_context );
void ut_error_disp_select_msgs( long lErrorCode );  
void ut_error_clear( Error* error );  
void ut_error_class_decode( ErrorClasses list, long code, char* text ); 
long ut_error_class_register( ErrorClasses list, char* name ); 
ErrorClasses ut_error_class_init( void ); 
ErrorClasses ut_error_class(void); 
Error* ut_error_state(void); 
void ut_null_init( void ); 
float ut_nan_r( void );  
double ut_nan_d( void );  
float ut_null_r( void );  
double ut_null_d( void );  
short ut_null_s( void );  
long ut_null_i( void ); 
double ut_infinity_d( const long i );  
float ut_infinity_r( const long i );  
short ut_infinity_s( const long i );  
long ut_infinity_i( const long i ); 
void ut_null_set( char typecode, char* opt, char* pattern ); 
void ut_null_set_ieee( void );  
void ut_null_set_safe( void ); 
logical ut_qnull_i( const long i ); 
logical ut_qnull_s( const short i ); 
logical ut_qnan_r( const float x ); 
logical ut_qnan_d( const double x ); 
logical ut_qnull_d( const double x ); 
logical ut_qnull_r( const float x ); 
logical ut_qinf_i( const long i );  
logical ut_qinf_s( const short i );  
logical ut_qinf_r( const float x );  
logical ut_qinf_d( const double x ); 
double ut_rand( void ); 
double ut_rand_eval(unsigned int* seed ); 
logical ut_eflag_syntax( char* opt, char* param, char* value ); 
logical ut_eflag_parse( char** args, long nargs, logical local_flag_set(char*, char*, logical)); 
logical ut_eflag_set( char* param, char* value, char** paramlist,  char** valuelist, long nparams, logical* flags, logical eswitch );  
void ut_eflag_list( char** paramlist, logical* flags, char** valuelist, long nparams, char* cmt, long tab );  
char* cs_dup( const char* buf ); 
char* cs_dup_siz( const char* buf, const long siz ); 
void cs_copy_siz( const char* source, char* dest, long siz );	 
void cs_copy( const char* source, char* dest ); 
long cs_ends( const char* buf ); 
long cs_begs( const char* buf ); 
void cs_upper( char* buf );	 
void cs_lower( char* buf );	 
void cs_mixcase( char* buf ); 
logical cs_is_blank( const char* buf );					 
logical cs_eq( const char* buf1, const char* buf2 );			 
logical cs_eq_siz( const char* buf1, const char* buf2, const long siz ); 
logical cs_uceq_siz( const char* buf1, const char* buf2, const long siz ); 
logical cs_uceq( const char* buf1, const char* buf2 ); 
long cs_match_minimum( const char* buf, const char** array, const long n ); 
void cs_zero( char* line );	 
void cs_blank( char* line, long maxlen ); 
void cs_null( char* line, const long siz );	 
logical cs_nullstr( const char* source ); 
logical cs_alpha_char( const char c ); 
logical cs_digit_char( const char c ); 
logical cs_numeric_char( const char c ); 
void cs_subcmd_split( const char* opt, char* cmd, char* sub, const char sep ); 
void cs_strip_digits( const char* name, char* pattern, const long maxlen );
char* cs_conc( const char* buf1, const char* buf2, char* buf ); 
char* cs_conc1( const char* buf1, const char* buf2, char* buf ); 
logical cs_gt( const char* x, const char* y ); 
logical cs_lt( const char* x, const char* y ); 
long cs_list_add( char** list, long* list_size, const long max_size, const char* item ); 
char* cs_get_ss( const char* source, char* dest, const long pos1, const long pos2 );	 
void cs_put_ss( const char* source, char* dest, const long pos1, const long pos2 );		 
logical cs_eq_ss( const char* buf, const long pos1, const long pos2, const char* comp ); 
logical cs_ss( const char* buf, const char* sub ); 
long cs_index( const char* buf, const char* sub ); 
long cs_eindex( const char* buf, const long pos, const char* sub ); 
long cs_index_char( const char* buf, const char c ); 
logical cs_ss_char( const char* buf, const char sub ); 
logical cs_split_string( const char* buf, const char sep, char* buf1, char* buf2 );  
char* cs_upto( const char* source, const char targ, char* dest ); 
void cs_denull( char* buf, const long siz ); 
void cs_detab( char* buf ); 
void cs_decmt( char* buf, const char esc, const char cmt ); 
void cs_remove_nl( char* line );	 
void cs_ubar( char* source ); 
void cs_deubar( char* source ); 
void cs_compress_spaces( const char* source, char* dest );	 
void cs_strpak( const char* source, char* dest ); 
void cs_string_replace( char* cbuf, const char* old, const char* new ); 
void cs_append_delim( char* buf, char* delim, char* text );
char* cs_sparen( const char* source, char* dest ); 
char* cs_paren( const char* source, char* dest ); 	
void cs_gparen( const char* source, const char lpar, const char rpar, char* dest ); 
void cs_deparen( char** buf, char* dest ); 
void cs_parse_assignment( char** pptr, char* name, long maxlen, char* op );
void cs_remove_quotes_and_trail( const char* val, char* dest, const long maxlen );
long cs_count_char(const char* buf, const char c ); 
void cs_fs_format_parse( const char*  Fortran_Format, char* C_Format, char* DataType, int* Field_Length ); 
long cs_read_i( const char* buf ); 
double cs_read_d( const char* buf ); 
float cs_read_r( const char* buf ); 
short cs_read_s( const char* buf ); 
logical cs_read_q( const char* buf ); 
long cs_read_digit( const char c ); 
long cs_read_hex_digit( const char c ); 
long cs_read_i1( const char* buf ); 
long cs_read_val_int( char** buf, long* digits ); 
double cs_read_val_frac( char** buf, long* digits ); 
long cs_read_val_sign( char** buf ); 
void cs_read_val_d( char** buf, long* sign_val, long* int_val, double* frac_val ); 
void cs_read_val_de( char** buf, long* sign_val, long* int_val, double* frac_val ); 
void cs_read_val_ee( char** buf, long* sign_val, long* int_val, double* frac_val, long* exp_val ); 
char* cs_write_i( const long val, char* buf, const long siz ); 
char* cs_write_d( const double val, char* buf, const long siz ); 
char* cs_write_r( const  float val, char* buf, const long siz ); 
char* cs_write_s( const short val, char* buf, const long siz ); 
char cs_write_q( const logical q ); 
char* cs_writef_i( const long val, const char* fmt, char* buf, const long siz ); 
char* cs_writef_r( const float val, const char* fmt, char* buf, const long siz ); 
char* cs_writef_s( const short val, const char* fmt, char* buf, const long siz ); 
char* cs_writef_d( const double val, const char* fmt, char* buf, const long siz ); 
char* cs_vwritef_d( const double* p, const char* fmt, char* buf, const long siz ); 
char* cs_vwritef_r( const float* p, const char* fmt, char* buf, const long siz ); 
char* cs_write_opt_s( const char* prompt, const short val, char* buf, const long siz ); 
char* cs_write_opt_i( const char* prompt, const long val, char* buf, const long siz ); 
char* cs_write_opt_r( const char* prompt, const float val, char* buf, const long siz ); 
char* cs_write_opt_d( const char* prompt, const double val, char* buf, const long siz ); 
char* cs_write_opt_c( const char* prompt, const char* val, char* buf, const long siz ); 
char* cs_write_e_r( const float x, const float off, char* buf, const long siz ); 
char* cs_write_e_d( const double x, const double off, char* buf, const  long siz ); 
char* cs_write_tp_i( const long i, char* buf, const long siz ); 
char* cs_write_rj_i( const long i, char* buf, const long siz ); 
void cs_numberclean( char* tbuf, long sig, char* buf, long maxlen ); 
char cs_logic( const logical q ); 
logical cs_nancode( const char* buf ); 
void cs_greek_letter( const long n, char* buf ); 
long cs_greek_no( const char* buf ); 
long cs_roman_parse( const char* buf ); 
void cs_roman_numeral( const long n, char* buf ); 
char* cs_advptr( char** buf ); 
void cs_get_c( char** buf, char* token, const long siz ); 				
void cs_get_cs( char** buf, char* token, const long siz ); 
void cs_get_cl( char** buf, char* token, const long siz ); 			
double cs_get_d( char** buf ); 
float cs_get_r( char** buf ); 
long cs_get_i( char** buf ); 
short cs_get_s( char** buf ); 
logical cs_get_q( char** buf ); 
double cs_getd_d( char** buf, const double xdefault ); 
float cs_getd_r( char** buf, const float xdefault ); 
short cs_getd_s( char** buf, const short xdefault ); 
long cs_getd_i( char** buf, const long xdefault ); 
void cs_put_c( const char* token, char** buf, long* siz ); 
void cs_put_cs( const char* token, char** buf, long* siz ); 	
void cs_put_cn( const char* token, char** buf, long* siz ); 
void cs_put_d( const double token, char** buf, long* siz ); 
void cs_put_r( const float token, char** buf, long* siz ); 
void cs_put_s( const short token, char** buf, long* siz ); 
void cs_put_i( const long token, char** buf, long* siz ); 
long cs_list_parse( char* buf, char** list, const long maxitems ); 
char** cs_parse_paren_list( const char* buf, long* n ); 
char** cs_parse_parcom_list( const char* buffer, long* np, logical mode );
char** cs_parse_comma_list( const char* buffer, long* np );
char cs_copy_token_esc( char** buf, const char* seps, const char* pars, const char esc, char* dest, const long gsiz );
char*  cs_parse_paren_token( char** pptr, char lsep, char rsep, char* result, long maxlen );
char*  cs_parse_paren_token_validate( char** pptr, char lsep, char rsep, char* result, long maxlen, logical *status );
char*  cs_parse_term_token( char** pptr, char rsep );
void cs_append( char** pptr, char* sep, char* buf );

void cs_put_token( const char* buf, char** dest, const char sep, long* siz ); 
void cs_get_token( char** buf, char* token, const long mode, const long siz ); 
void cs_advance( char** buf ); 
long cs_fs_ends( char* buf, long size ); 
void cs_fs_zero( char* line, long size ); 
void cs_copy_cf( char* cname, char* fname,  long fsiz );  
void cs_copy_fc( char* fname, char* cname, long fsiz, long maxlen ); 
char* cs_fs_dup( char* fstring, long maxlen ); 
void cs_fs_free( char* cstring, char* fstring, long fsiz ); 
char** cs_fs_alloc_array( char* array, long fsiz, long n );  
void cs_fs_free_array( char** fc_StringArray, char* value, long fsiz, long n ); 
void cs_fs_copy_item( char* cstring, char* fstring, long maxlen, long i );  
void cs_fs_put_item( char* fstring, long maxlen, long i, char* cstring ); 
long cs_fs_match( const char* buf, const char* array, const long n, const long size ); 
UnitDomain cs_unit_init( const char* filename ); 
void cs_unit_free( UnitDomain u ); 
void cs_unit_texmode( UnitDomain u, const logical q ); 
void cs_unit_mult( UnitDomain unit_domain, const char* u1, const char* u2, char* u, const long siz ); 
void cs_unit_div( UnitDomain u, const char* u1, const char* u2, char* udiv, const long siz ); 
void cs_unit_exp( UnitDomain u, const char* unit1, const double power, char* unit2, const long siz ); 
void cs_unit_resolve( UnitDomain unit_domain, const char* u1, char* u2, const long siz ); 
void cs_unit_dparse( UnitDomain unit_domain, const char* u1, char* u2, const long siz ); 
void si_write_r( float x, char* cy );  
void si_write_d( double x, char* cy ); 
float si_read_r( char* cy );  
double si_read_d( char* cy ); 
long si_exp( char* pfx, logical* ok ); 
void si_getpfx( long exponent, char* pfx, long* remainder ); 
void cs_char_name( char c, char* name ); 
void cs_parser( char* buf, long* itemcode, long* itempos, long* nitems, char* cmode ); 
void cs_parse_disp( char* buf, long* itemcode, long* itempos, long nitems );  
void cs_parse_tdisp( char* buf, long* itemcode, long* itempos, long nitems ); 
void cs_parse_read( long i, char* buf, long* itempos, char* token, long* siz ); 
void cs_parse_fmt( char* buf, long* itemcode, long* itempos, long nitems, char* mbuf ); 
long utm_rmod_d( const double x, double* y, const double r ); 
long utm_rmod_i( const long x, long* y, const long r ); 
long utm_rmod_s( const short value, short* remainder, const short divisor ); 
long utm_rmod_r( const float value, float* remainder, const float divisor ); 
long utm_logbase_i( const long x, const long n ); 
double utm_logbase_d( const double x, const long n ); 
float utm_logbase_r( const float x, const long n ); 
void utm_frac_r( const float value, long* int_val, float* fraction );  
void utm_frac_d( const double value, long* int_val, double* fraction ); 
long utm_min_i( const long i, const long j ); 
long utm_max_i( const long i, const long j ); 
double utm_min_d( double x, double y ); 
double utm_max_d( double x, double y ); 
double utm_range_d( double x, double xmin, double xmax );
logical utm_divisible( const long a, const long b ); 
long utm_ceiling_d( const double x );  
long utm_ceiling_r( const float x ); 
long utm_floor_d( const double x );  
long utm_floor_r( const float x ); 
long utm_nint_d( const double x );  
long utm_nint_r( const float x ); 
long utm_modulo_i( const long x, const long base );  
short utm_modulo_s( const short x, const short base );  
float utm_modulo_r( const float x, const float base );  
double utm_modulo_d( const double x, const double base ); 
double utm_round_d( const double x, const long f );  
float utm_round_r( const float x, const long f ); 
void utm_order_r( float* x, float* y);  
void utm_order_d( double* x, double* y ); 
logical utm_clip_r( const float x, const float y, const float x1, const float x2, const float yy1, const float yy2 );  
logical utm_clip_d( const double x, const double y, const double x1, const double x2, const double yy1, const double yy2 );  
logical cx_clip_r( const complex z, const complex z1, const complex z2 );  
logical cx_clip_d( const dcomplex z, const dcomplex z1, const dcomplex z2 ); 
double utm_rlap_check_d( const double x, const double x1, const double x2, logical* left, logical* right );  
float utm_rlap_check_r( const float x, const float x1, const float x2, logical* left, logical* right ); 
logical utm_overlap_r( const float xx1, const float xx2, const float yy1, const float yy2,    float* u1, float* u2 );  
logical utm_overlap_d( const double xx1, const double xx2, const double yy1, const double yy2,    double* u1, double* u2 ); 
double utm_lint_d( const double x1, const double x2, const double yy1, const double yy2, const double x0 );  
float utm_lint_r( const float x1, const float x2, const float yy1, const float yy2, const float x0 ); 
float utm_rabs( const float x ); 
float utm_rlog10( const float x ); 
float utm_rsqrt( const float x ); 
float utm_rpow( const float x, const float n ); 
float utm_ipow_r( const float x, const long n );  
double utm_ipow_d( const double x, const long n ); 
float utm_tpow_r( const long n );  
double utm_tpow_d( const long n ); 
double ar_linterp_d( const double* x, const double* y, const long n, const double x0 );  
float ar_linterp_r( const float* x, const float* y, const long n, const float x0 ); 
void ar_movag_d( const double* yy0, double* y, const long n, const long k );  
void ar_movag_r( const float* yy0, float* y, const long n, const long k ); 
float utm_poly_r( const float x, const float* coeffs, const long degree );  
double utm_poly_d( const double x, const double* coeffs, const long degree ); 
void sx_val_to_sxa_i( const long v, long* sign, long* deg, long* min, long* sec );  
void sx_val_to_sxa_r( const float v, long* sign, long* deg, long* min, long* sec, float* frac );  
void sx_val_to_sxa_d( const double v, long* sign, long* deg, long* min, long* sec, double* frac ); 
long sx_sxa_to_val_i( const long sign, const long deg, const long min, const long sec );  
float sx_sxa_to_val_r( const long sign, const long deg, const long min, const long sec, const float frac );  
double sx_sxa_to_val_d( const long sign, const long deg, const long min, const long sec, const  double frac ); 
void sx_wval_to_sxa_r( const float r, const long w, logical* sign, long* d, long* m, long* s, float* f );  
void sx_wval_to_sxa_d( const double r, const long w, logical* sign, long* d, long* m, long* s, double* f ); 
float sx_wsxa_to_val_r( const logical sign, const long d, const long m, const long s, const float f, const long w );  
double sx_wsxa_to_val_d( const logical sign, const long d, const long m, const long s, const double f, const long w ); 
void sx_psx_to_sxa_i( const long r, logical* sg, long* d, long* m, long* s );  
void sx_psx_to_sxa_d( const double r, logical* sg, long* d, long* m, long* s, double* f );  
void sx_psx_to_sxa_r( const float r, logical* sg, long* d, long* m, long* s, float* f ); 
double sx_sxa_to_psx_d( const logical sg, const long d, const long m, const long s, const double f );  
float sx_sxa_to_psx_r( const logical sg, const long d, const long m, const long s, const float f ); 
void cs_sex_fmt( const logical sg, const long d, const long m, const long s, const double fs, char* buf, const long siz ); 
char* cs_sexwrite_hr_d( const double x, char* buf, const long siz );  
char* cs_sexwrite_d( const double x, char* buf, const long siz ); 
double utm_eexp( const double x ); 
double utm_esinh( const double x );  
double utm_ecosh( const double x ); 
double utm_acosh( const double x );  
double utm_asinh( const double x );  
double utm_atanh( const double x ); 
double utm_dgauss( const double x, const double x0, const double sigma ); 
double utm_bose_einstein( const double x ); 
double utm_dplanckn( const double x, const long n ); 
double utm_hexagam( const double x );  
double utm_acych( const double x );  
double utm_acyc( const double x );  
double utm_E1_func( const double x ); 
complex cx_set_r( const float x, const float y );  
dcomplex cx_set_d( const double x, const double y ); 
float cx_float_r( const complex z );  
double cx_float_d( const dcomplex z );  
float cx_imag_r( const complex z );  
double cx_imag_d( const dcomplex z ); 
complex cx_conj_r( const complex z1 );  
dcomplex cx_conj_d( const dcomplex z1 ); 
complex cx_mult_r( const complex z1, const complex z2 );  
dcomplex cx_mult_d( const dcomplex z1, const dcomplex z2 ); 
complex cx_add_r( const complex z1, const complex z2 );  
dcomplex cx_add_d( const dcomplex z1, const dcomplex z2 ); 
void cx_inc_r( const complex z1, complex* z2 );  
void cx_inc_d( const dcomplex z1, dcomplex* z2 ); 
dcomplex cx_prod_d( const dcomplex z1, const dcomplex z2 );  
complex cx_prod_r( const complex z1, const complex z2 ); 
logical cx_gt_r( const complex z1, const complex z2 );  
logical cx_ge_r( const complex z1, const complex z2 );  
logical cx_lt_r( const complex z1, const complex z2 );  
logical cx_le_r( const complex z1, const complex z2 );  
logical cx_gt_d( const dcomplex z1, const dcomplex z2 );  
logical cx_ge_d( const dcomplex z1, const dcomplex z2 );  
logical cx_lt_d( const dcomplex z1, const dcomplex z2 );  
logical cx_le_d( const dcomplex z1, const dcomplex z2 ); 
float utm_radin_r( const float x ) ;  
double utm_radin_d( const double x ) ; 
float utm_radout_r( const float x );  
double utm_radout_d( const double x ); 
float utm_circ_r( const float x );  
double utm_circ_d( const double x ); 
float utm_circ_ew_r( const float x );  
double utm_circ_ew_d( const double x ); 
float utm_circ_rad_r( const float x );  
double utm_circ_rad_d( const double x ); 
float utm_circ_ew_rad_r( const float x );  
double utm_circ_ew_rad_d( const double x ); 
double utm_argd_d( const double x, const double y );  
float utm_argd_r( const float x, const float y );  
float cx_argd_r( const complex z );  
double cx_argd_d( const dcomplex dz ); 
double utm_cos( const double x );
double utm_sin( const double x );
float utm_rsind( const float x );  
double utm_dsind( const double x );  
float utm_rcosd( const float x );  
double utm_dcosd( const double x ); 
float utm_rasind( const float x );  
double utm_dasind( const double x );  
float utm_racosd( const float x );  
double utm_dacosd( const double x ); 
void utm_rotate2_d( const double x, const double y, const double roll, double* xr1, double* yr1 ); 
long utm_iplus( const logical q ); 
float utm_spharc_r( const float a1, const float b1, const float a2, const float b2 );  
double utm_spharc_d( const double a1, const double b1, const double a2, const double b2 ); 
void ar_zero_i( long* a, const long siz );  
void ar_zero_r( float* a, const long siz );  
void ar_zero_s( short* a, const long siz );  
void ar_zero_d( double* a, const long siz );  
void ar_zero_c( char** x, const long n );  
void ar_zero_dz( dcomplex* z, const long n );  
void ar_zero_z( complex* z, const long n ); 
void ar_copy_d( const double* a, double* b, const long siz );  
void ar_copy_i( const long* a, long* b, const long siz );  
void ar_copy_s( const short* a, short* b, const long siz );  
void ar_copy_r( const float* a, float* b, const long siz );  
void ar_copy_q( const logical* a, logical* b, const long siz );  
void ar_copy_c( const char** x, char** y, const long n );  
void ar_copy_z( const complex* z1, complex* z2, const long size );  
void ar_copy_dz( const dcomplex* z1, dcomplex* z2, const long size ); 
double ar_dot_d( const double* x, const double* y, const long siz );  
float ar_dot_r( const float* x, const float* y, const long n );  
short ar_dot_s( const short* x, const short* y, const long n );  
long ar_dot_i( const long* x, const long* y, const long siz );  
dcomplex ar_dot_dz( const dcomplex* x, const dcomplex* y, const long size );  
complex ar_dot_z( const complex* x, const complex* y, const long size ); 
void ar_bounds_r( const float* x, const long n, float* xmin, float* xmax );  
void ar_bounds_s( const short* x, const long n, short* xmin, short* xmax );  
void ar_bounds_i( const long* x, const long n, long* xmin, long* xmax );  
void ar_bounds_d( const double* x, const long n, double* xmin, double* xmax ); 
void ar_stats_s( const short* x, const long n, float* mean, float* var );  
void ar_stats_r( const float* x, const long n, float* mean, float* var );  
void ar_stats_i( const long* x, const long n , double* mean, double* var );  
void ar_stats_d( const double* x, const long n , double* mean, double* var ); 
void ar_fill_c( char** x, const long n, const char* x0 );  
void ar_fill_i( long* x, const long n, const long x0 );  
void ar_fill_s( short* x, const long n, const short x0 );  
void ar_fill_d( double* x, const long n, const double x0 );  
void ar_fill_r( float* x, const long n, const float x0 );  
void ar_fill_z( complex* z, const long n, const complex z0 );  
void ar_fill_dz( dcomplex* z, const long n, const dcomplex z0 ); 
void ar_cmult_s( short* x, const long n, const short x0 );  
void ar_cmult_i( long* x, const long n, const long x0 );  
void ar_cmult_r( float* x, const long n, const float x0 );  
void ar_cmult_d( double* x, const long n, const double x0 );  
void ar_cmult_z( complex* z, const long n, const complex z0 );  
void ar_cmult_dz( dcomplex* z, const long n, const dcomplex z0 ); 
void ar_cadd_s( short* x, const long n, const short x0 );  
void ar_cadd_i( long* x, const long n, const long x0 );  
void ar_cadd_r( float* x, const long n, const float x0 );  
void ar_cadd_d( double* x, const long n, const double x0 );  
void ar_cadd_z( complex* z, const long n, const complex z0 );  
void ar_cadd_dz( dcomplex* z, const long n, const dcomplex z0 ); 
void ar_add_d( const double* x, double* y, const long n );  
void ar_add_s( const short* x, short* y, const long n );  
void ar_add_r( const float* x, float* y, const long n );  
void ar_add_i( const long* x, long* y, const long n );  
void ar_add_z( const complex* z1, complex* z2, const long n );  
void ar_add_dz( const dcomplex* z1, dcomplex* z2, const long n ); 
void ar_sub_s( const short* x, short* y, const long n );  
void ar_sub_i( const long* x, long* y, const long n );  
void ar_sub_r( const float* x, float* y, const long n );  
void ar_sub_d( const double* x, double* y, const long n );  
void ar_sub_dz( const  dcomplex* z1, dcomplex* z2, const long n );  
void ar_sub_z( const complex* z1, complex* z2, const long n ); 
void ar_wadd_s( const short* x1, const short* x2, const short w1, const short w2,short* y, const long n );  
void ar_wadd_i( const long* x1, const long* x2, const long w1, const long w2,long* y, const long n );  
void ar_wadd_r( const float* x1, const float* x2, const float w1, const float w2,float* y, const long n );  
void ar_wadd_d( const double* x1, const double* x2, const double w1, const double w2,double* y, const long n );  
void ar_wadd_z( const complex* z1, const complex* z2, const complex w1, const complex w2,  complex* z, const long n );  
void ar_wadd_dz( const dcomplex* z1, const dcomplex* z2, const dcomplex w1, const dcomplex w2,  dcomplex* z, const long n ); 
void ar_bin_s( short* x, const long n, const short x0, const short dx );  
void ar_bin_r( float* x, const long n, const float x0, const float dx );  
void ar_bin_i( long* x, const long n, const long x0, const long dx );  
void ar_bin_d( double* x, const long n, const double x0, const double dx );  
void ar_bin_dz( dcomplex* z, const long n, const dcomplex z0, const dcomplex dz );  
void ar_bin_z( complex* z, const long n, const complex z0, const complex dz ); 
long ar_cbisect_d( const double* x, const long n, const double x0 );  
long ar_cbisect_r( const float* x, const long n, const float x0 );  
long ar_cbisect_s( const short* x, const long n, const short x0 );  
long ar_cbisect_i( const long* x, const long n, const long x0 ); 
long ar_bisect_d( const double* x, const long n, const double x0 );  
long ar_bisect_r( const float* x, const long n, const float x0 );  
long ar_bisect_s( const short* x, const long n, const short x0 );  
long ar_bisect_i( const long* x, const long n, const long x0 ); 
void ar_copy_si( const   short* x,  long* y,  const long n);  
void ar_copy_is( const  long* x, short* y, const long n);  
void ar_copy_rd( const   float* x,  double* y,  const long n);  
void ar_copy_dr( const double* x, float* y, const long n);  
void ar_copy_dzz( const dcomplex* x, complex* y, const long n);  
void ar_copy_zdz( const complex* x, dcomplex* y, const long n); 
long ar_match_r(   const float value,  const float* list,  const long n );  
long ar_match_i(   const long value,  const long* list,  const long n );  
long ar_match_d(   const double value,  const double* list,  const long n );  
long ar_match_s(   const short value,  const short* list,  const long n );  
long ar_match_c( const char* buf, char** array, const long n ); 
long ar_match_sorted_c( const char* buf, char** array, const long n );
long ar_cmatch_r(   const float value,  const float* list,  const long n );  
long ar_cmatch_i(   const long value,  const long* list,  const long n );  
long ar_cmatch_d(   const double value,  const double* list,  const long n );  
long ar_cmatch_s(   const short value,  const short* list,  const long n );  
long ar_cmatch_c( const char* buf, char** array, const long n ); 
void ar_csort_i( const long* x, const long n, long* jsort );  
void ar_csort_d( const double* x, const long n, long* jsort ); 
void ar_csort_r( const float* x, const long n, long* jsort ); 
void ar_csort_s( const short* x, const long n, long* jsort );  
void ar_csort_c( char** x, const long n, long* jsort ); 
void ar_creindex_i( const long* jsort, const long n, long* isort ); 
void ar_cresort_s( const short* x, const long n, short* y, long* jsort ); 
void ar_cresort_d( const double* x, const long n, double* y, long* jsort ); 
void ar_cresort_r( const float* x, const long n, float* y, long* jsort ); 
void ar_cresort_i( const long* x, const long n, long* y, long* jsort ); 
void ar_sort_r( const float* x, const long n, long* jsort ); 
void ar_sort_s( const short* x, const long n, long* jsort );  
void ar_sort_d( const double* x, const long n, long* jsort ); 
void ar_sort_i( const long* x, const long n, long* jsort );  
void ar_sort_c( char** x, const long n, long* jsort );  
void ar_reindex_i( const long* x, const long n, long* y ); 
void ar_resort_i( const long* x, long n, long* y, long* jsort );  
void ar_resort_s( const short* x, const long n, short* y, long* jsort ); 
void ar_resort_d( const double* x, const long n, double* y, long* jsort ); 
void ar_resort_r( const float* x, const long n, float* y, long* jsort ); 
void ar_poleval_r( const float* x, float* y, const long n, const float* coeffs, const long degree );  
void ar_poleval_d( const double* x, double* y, const long n, const double* coeffs, const long degree ); 
void ar_lsfit_d( const double* x, const double* y, const long n, double* a, double* b );  
void ar_lsfit_r( const float* x, const float* y, const long n, float* a, float* b ); 
float ar_trap_r( const float* x, const float* y, const long n );  
double ar_trap_d( const double* x, const double* y, const long n ); 
float ar_trap_log_r( const float* x, const float* y, const long n );  
double ar_trap_log_d( const double* x, const double* y, const long n ); 
float ar_trap_dx_r( const float* y, const long n, const float dx );  
double ar_trap_dx_d( const double* y, const long n, const double dx ); 
double** ar_valloc_d( const long n ); 
void ar_vfree_d( double** v ); 
float ar_vdot_r( const float* a, const float* b );  
double ar_vdot_d( const double* a, const double* b ); 
float ar_vnorm_r( const float* a );  
double ar_vnorm_d( const double* a ); 
void ar_vcross_r( const float* a, const float* b, float* c );  
void ar_vcross_d( const double* a, const double* b, double* c ); 
float ar_vmaxmod_r( const float* r );  
double ar_vmaxmod_d( const double* r ); 
float ar_vunit_r( float* r );  
double ar_vunit_d( double* r ); 
void ar_vcmult_r( float* p, const float c );  
void ar_vdelta_d( const double* p, const double delta, const double* dp, double* r );
void ar_vcmult_d( double* p, const double c ); 
void ar_vadd_r( const float* p, const float* q, float* r );  
void ar_vadd_d( const double* p, const double* q, double* r ); 
void ar_vsub_r( const float* p, const float* q, float* r );  
void ar_vsub_d( const double* p, const double* q, double* r ); 


void ar_vzero_r( float* p );  
void ar_vzero_d( double* p ); 
logical ar_vnull_r( const float* v );  
logical ar_vnull_d( const double* v ); 
void ar_vcopy_r( const float* p, float* r );  
void ar_vcopy_d( const double* p, double* r ); 
double ar_vcopy_unit_d( double* r, double* unit ); 
void ar_vcdiv_d( double* p, const double c ); 
void ar_lmat_prex_d( double* aa, const double* y, double* x );  
void ar_lmat_postx_d( double* aa, const double* y, double* x ); 
void ar_mat_prex_r( float** aa, const float* y, float* x );  
void ar_mat_prex_d( double** aa, const double* y, double* x ); 
void ar_mat_postx_r( float** aa, const float* y, float* x );  
void ar_mat_postx_d( double** aa, const double* y, double* x ); 
void ar_mat_mult_r( float** aa, float** bb, float** cc );  
void ar_mat_mult_d( double** aa, double** bb, double** cc ); 

void ar_lmatnn_prex( double* v1, double* r, double* v2, long n );
void ar_lmat_mult_d( double* a, double* b, double* c );

double utm_quat_unit( double* p );
double utm_quat_dot( double* p, double* q );
void utm_quat_mult( double* p, double* q, double* r );
void utm_quat_rot( double theta, double* n, double* p );
void utm_quat_from_matrix( double* rotate, double* q );
void utm_quat_to_matrix( double* q, double* rotate );

void ar_euler_rotate_alt( double* euler, double* rotate );
void ar_euler_calc_gen( double* r, long* axes, double* euler );
void ar_euler_rotate_gen( double* euler, long* axes, double* rotate );
void ar_euler_rotate( double* euler, double* rotate );
void ar_euler_calc( double* rotate, double* euler );
void ar_transform_backward( const double* in, double* rotate, double* translate, double* out );
void ar_transform_forward( const double* in, double* rotate, double* translate, double* out );

void ar_vec_r( const float r, const float theta, const float phi, float* p );  
void ar_vec_d( const double r, const double theta, const double phi, double* p ); 
void ar_polar_r( const float* p, float* r, float* theta, float* phi );  
void ar_polar_d( const double* p, double* r, double* theta, double* phi ); 
float ar_polarll_r( const float* p, float* llong, float* lat );  
double ar_polarll_d( const double* p, double* llong, double* lat ); 
void ar_unitll_d( const double llong, const double lat, double* p );  
void ar_unitll_r( const float llong, const float lat, float* p ); 
void ar_vec6_r( const float r, const float theta, const float phi, const float rrdot, const float tdot, const float phidot, float* p );  
void ar_vec6_d( const double r, const double theta, const double phi, const double rrdot, const double tdot, const double phidot, double* p ); 
void ar_polar6_r( const float* p, float* r, float* theta, float* phi, float* rrdot, float* tdot, float* phidot );  
void ar_polar6_d( const double* p, double* r, double* theta, double* phi, double* rrdot, double* tdot, double* phidot ); 
void ut_free( void* p ); 
char** ar_alloc_c( const long size, const long n ); 
void ar_free_c( char** word ); 
void ar_free_cp( char** word, const long n ); 
short** ar_alloc_s( const long dim, const long n );  
long** ar_alloc_i( const long dim, const long n );  
float** ar_alloc_r( const long dim, const long n );  
double** ar_alloc_d( const long dim, const long n ); 
void ar_free_s( short** p );  
void ar_free_i( long** p );  
void ar_free_r( float** p );  
void ar_free_d( double** p ); 
float** ar_alloc_matrix_r(   const long xmin,  const long xmax,  const long ymin,  const long ymax);  
double** ar_alloc_matrix_d(   const long xmin,  const long xmax,  const long ymin,  const long ymax);  
long** ar_alloc_matrix_i(   const long xmin,  const long xmax,  const long ymin,  const long ymax);  
short** ar_alloc_matrix_s(   const long xmin,  const long xmax,  const long ymin,  const long ymax);  
complex** ar_alloc_matrix_z(   const long xmin,  const long xmax,  const long ymin,  const long ymax);  
dcomplex** ar_alloc_matrix_dz(   const long xmin,  const long xmax,  const long ymin,  const long ymax); 
void ar_free_matrix_s( short** matrix, const long xmin, const long ymin);  
void ar_free_matrix_i( long** matrix, const long xmin, const long ymin);  
void ar_free_matrix_r( float** matrix, const long xmin, const long ymin);  
void ar_free_matrix_d( double** matrix, const long xmin, const long ymin);  
void ar_free_matrix_z( complex** matrix, const long xmin, const long ymin);  
void ar_free_matrix_dz( dcomplex** matrix, const long xmin, const long ymin); 
void ar_transpose_s(   const short** x,  short** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax);  
void ar_transpose_i(   const long** x,  long** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax);  
void ar_transpose_r(   const float** x,  float** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax);  
void ar_transpose_d(   const double** x,  double** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax);  
void ar_transpose_q(   const logical** x,  logical** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax);  
void ar_transpose_dz(   const dcomplex** x,  dcomplex** y,  const long xmin,  const long xmax, const long ymin,  const long ymax);  
void ar_transpose_z(   const complex** x,  complex** y,  const long xmin,  const long xmax,  const long ymin,  const long ymax); 
void ar_minmax_d(  const double* x,	const  long  n,	 double* xmin,	 double* xmax,	 long*  imin, long*  imax	);  
void ar_minmax_r(  const float* x,	const  long  n,	 float* xmin,	 float* xmax,	 long*  imin, long*  imax	);  
void ar_minmax_i(  const long* x,	const  long  n,	 long* xmin,	 long* xmax,	 long*  imin, long*  imax	);  
void ar_minmax_s(  const short* x,	const  long  n,	 short* xmin,	 short* xmax,	 long*  imin, long*  imax	); 
long ar_product_i(  const long* a,	 const long n 	 );  
short ar_product_s(  const short* a,	const  long n 	 );  
float ar_product_r(  const float* a,	 const long n 	 );  
double ar_product_d(  const double* a,const 	 long n 	 );  
dcomplex ar_product_dz(  const dcomplex* a,const 	 long n );  
complex ar_product_z(  const complex* a,	 const long n 	 ); 
long ar_sum_i(  const long* a,	 const long n 	 );  
short ar_sum_s(  const short* a, const long n  );  
float ar_sum_r(  const float* a,	const  long n 	 );  
double ar_sum_d( const double* a,	const  long n 	 );  
dcomplex ar_sum_dz(  const dcomplex* a,	const  long n 	 );  
complex ar_sum_z(  const complex* a,	const  long n 	 ); 
void ar_merge_s(  const short* true, const short* false, short* merge, const logical* mask, const long size);  
void ar_merge_i(  const long* true, const long* false, long* merge, const logical* mask, const long size);  
void ar_merge_d(  const double* true, const double* false, double* merge, const logical* mask, const long size);  
void ar_merge_r(  const float* true, const float* false, float* merge, const logical* mask, const long size);  
void ar_merge_c(  const char** true, const char** false, char** merge, const logical* mask, const long size);  
void ar_merge_cp(  char** true, char** false, char** merge, const logical* mask, const long size);  
void ar_merge_z( const complex* z1, const complex* z2, complex* z, const logical* mask, const long size );  
void ar_merge_dz( const dcomplex* z1, const dcomplex* z2, dcomplex* z, const logical* mask, const long size ); 
long ar_pack_s( const short* x,	 const logical* mask,	 short* y,	 const long n	);  
long ar_pack_i( const long* x,	const 	 logical* mask, long* y, const long n);  
long ar_pack_r( const float* x,	 const logical* mask,	 float* y,	 const long n	);  
long ar_pack_d( const double* x,	 const logical* mask,	 double* y,	 const long n	);  
long ar_pack_c( const char** x,	 const logical* mask,	 char** y,	 const long n	);  
long ar_pack_cp( char** x,	 const logical* mask,	 char** y,	 const long n	);  
long ar_pack_dz( const dcomplex* z1, const logical* mask, dcomplex* z2, const long n );  
long ar_pack_z( const complex* z1, const logical* mask, complex* z2, const long n ); 
long ar_unpack_s( const short* x,	 const logical* mask,	 short* y,	 const short  x0, 	 const long n	);  
long ar_unpack_i( const long* x,	 const logical* mask,	 long* y,	 const long  x0, 	 const long n	);  
long ar_unpack_r( const float* x,	 const logical* mask,	 float* y,	 const float  x0, 	 const long n	);  
long ar_unpack_d( const double* x,	 const logical* mask,	 double* y,	 const double  x0, 	 const long n	);  
long ar_unpack_cp( char** x,	 const logical* mask,	 char** y,	 char*  x0, 	 const long n	);  
long ar_unpack_c( const char** x,	 const logical* mask,	 char** y,	 const char*  x0, 	 const long n	 );  
long ar_unpack_dz( const dcomplex* z1, const logical* mask, dcomplex* z2, const dcomplex z0, const long n );  
long ar_unpack_z( const complex* z1, const logical* mask, complex* z2,  const complex z0, const long n ); 
void ar_cshift_s(  const short* x, short* y, const long n, const long shift);  
void ar_cshift_i(  const long* x, long* y, const long n, const long shift);  
void ar_cshift_r(  const float* x, float* y, const long n, const long shift);  
void ar_cshift_d(  const double* x, double* y, const long n, const long shift);  
void ar_cshift_cp(  char** x, char** y, const long n, const long shift);  
void ar_cshift_c(  const char** x, char** y, const long n, const long shift);  
void ar_cshift_dz(  const dcomplex* x, dcomplex* y, const long n, const long shift);  
void ar_cshift_z(  const complex* x, complex* y, const long n, const long shift); 
void ar_eoshift_s(  const short* x, short* y, const long n, const long shift, const short fill);  
void ar_eoshift_i(  const long* x, long* y, const long n, const long shift, const long fill);  
void ar_eoshift_r(  const float* x, float* y, const long n, const long shift, const float fill);  
void ar_eoshift_d(  const double* x, double* y, const long n, const long shift, const double fill);  
void ar_eoshift_cp( char** x, char** y, const long n, const long shift, char* fill);  
void ar_eoshift_c(  const char** x, char** y, const long n, const long shift, const char* fill);  
void ar_eoshift_dz(  const dcomplex* x, dcomplex* y, const long n, const long shift, const dcomplex fill);  
void ar_eoshift_z(  const complex* x, complex* y, const long n, const long shift, const complex fill); 
void ar_not_q( logical* mask, const long n ); 
void ar_true_q( logical* mask, const long n );  
void ar_false_q( logical* mask, const long n ); 
logical ar_all_q( const logical* mask, const long n ); 
logical ar_any_q( const logical* mask, const long n ); 
long ar_count_q( const logical* mask, const long n ); 
void ar_and_q( const logical* x, logical* y, const long n );  
void ar_or_q( const logical* x, logical* y, const long n ); 
FioFile fio_open_file( const char* filename, char* mode, long rec_bytes );  
logical fio_inquire( const char* name ); 
long fio_access( const char* name ); 
long fio_stat_type( const char* name );
void fio_file_get_name( FioFile file, char* name, const long maxlen ); 
void fio_file_get_type( FioFile file, char* type, const long maxlen );	 
long fio_file_status( FioFile file, long mode ); 
FioFile fio_file_ptr( long FID ); 
logical fio_file_close( FioFile file ); 
long fio_file_get_lineno( FioFile file ); 
FILE* fio_file_get_fptr(FioFile file); 
logical fio_file_test_eof(FioFile file); 
void fio_file_clear_eof( FioFile file ); 
logical fio_file_mode_eof( FioFile file, long mode ); 
logical fio_any_eof( long mode ); 
void fio_file_set_status ( FioFile file, logical ios ) ; 
long fio_file_get_cfd( FioFile file ); 
long fio_file_n( void );	 
void fio_list( void ); 
void fio_file_rewind( FioFile file ); 
long fio_file_get_lineno( FioFile file ); 
long fio_file_get_pos( FioFile file ); 
long fio_file_fid( FioFile file ); 
logical fio_file_interactive( FioFile file ); 
void fio_set_retry( logical q ); 
logical fio_get_retry( void ); 
long fio_tty_in( void );  
long fio_tty_out( void );  
long fio_tty_err( void ); 
void fio_file_flush( FioFile file ); 
long fio_file_get_recl( FioFile file ); 
void fio_file_set_recl( FioFile file, long recl ); 
FioFileList fio_get_list(void); 
void fio_path_set( char* pathlist ); 
void fio_path_list( void ); 

char* fio_tmpname( void );



FioFile fio_open_sr( const char* filename );  
FioFile fio_open_sw( const char* filename );  
FioFile fio_open_srw( const char* filename ); 
logical fio_seek( FioFile file, long pos ); 
logical fio_write_q( FioFile file, logical* buf, long n, long* nw );  
logical fio_write_c( FioFile file, char* buf, long n, long* nw );  
logical fio_write_d( FioFile file, double* buf, long n, long* nw );  
logical fio_write_r( FioFile file, float* buf, long n, long* nw );  
logical fio_write_i( FioFile file, long* buf, long n, long* nw );  
logical fio_write_s( FioFile file, short* buf, long n, long* nw ); 
logical fio_read_c( FioFile file, char* buf, long n, long* nw );  
logical fio_read_d( FioFile file, double* buf, long n, long* nw );  
logical fio_read_r( FioFile file, float* buf, long n, long* nw );  
logical fio_read_q( FioFile file, logical* buf, long n, long* nw );  
logical fio_read_i( FioFile file, long* buf, long n, long* nw );  
logical fio_read_s( FioFile file, short* buf, long n, long* nw ); 
logical fio_read_sio( FioFile file, void* buf, long n, long* nw, size_t siz );  
logical fio_write_sio( FioFile file, void* buf, long n, long* nw, size_t siz ); 
FioFile fio_open_cr( const char* filename );  
FioFile fio_open_cw( const char* filename );  
FioFile fio_open_crw( const char* filename ); 
logical fio_write_bytes( FioFile file, char* buf, long n, long* m ); 
logical fio_read_bytes( FioFile file, char* buf, long n, long* m ); 
logical fio_file_tty( FioFile file ); 
long fio_read_byte( FioFile file ); 
FioFile fio_open_raw( void ); 
long fio_read_raw( FioFile file ); 
FioFile fio_open_ar( const char* filename ); 
FioFile fio_open_aw( const char* filename ); 
FioFile fio_open_aa( const char* filename ); 
logical fio_read_line( FioFile file, char* line, long lmax );  
logical fio_tty_read( char* buf, long siz ); 
logical fio_write_line( FioFile file, const char* buf );  
logical fio_tty_write( const char* buf ); 
logical fio_write_text( FioFile file, char* buf );  
logical fio_tty_writel( char* buf ); 
void fio_pager_set( FioFileList list, logical q ); 
void fio_pager_set_length( FioFileList list, long page_length ); 
void fio_pager_print_state( FioFileList list ); 
void fio_process_file( char* filename, void (*process_line)( char* ) );  
void fio_process_file1( char* filename, void (*process_line)( void*, char* ), void* ptr ); 
void fio_stack_reinit( void ); 
void fio_stack_in( FioFile file ); 
void fio_stack_out( FioFile file ); 
void fio_stack_set_err( FioFile file ); 
void fio_msg( char* buf ); 
void fio_tmsg( char* buf ); 
void fio_msgl( char* buf ); 
void fio_tmsgl( char* buf ); 
void fio_std_msg( char* buf, logical newline ); 
void fio_set_msg( void (*msgfunc)(char*, logical )); 
void fio_msg_endline( void ); 
void fio_dashline( long n ); 
void ut_err_msg( const char* buf );  
logical fio_stack_read_input( char* buf, long siz );	 		  
logical fio_tty_input( char* buf, long siz );	 
void fio_stack_rewind_input( void ); 
logical fio_stack_interactive( long  fiomode ); 
FioFile fio_stack_file( long mode ); 
logical fio_input_test_eof( void ); 
void fio_basic_print_state( void ); 
FioStack fio_get_stack( void ); 
void fio_stack_opush( FioFile file );  
void fio_stack_ipush( FioFile file ); 
void fio_stack_ipop( long mode );  
void fio_stack_opop( long mode ); 
void fio_stack_print_state( void ); 
logical fio_stack_eof( long mode );  
logical fio_stack_ok( long mode );  
long fio_stack_status( long mode ); 
void fio_stack_set_detab( logical q ); 
logical fio_stack_read( char* buf, char* prompt, long siz ); 
long fio_stack_get_level( void ); 
FioFile fio_stack_input_entry( long level ); 
long fio_list_n(FioFileList list); 
FioFileList fio_list_init(long in, long out, long err) ;
void fio_list_free(FioFileList list); 
void fio_list_print(FioFileList list); 
FioFile fio_list_open_file( FioFileList list, const char* filename, char* mode, long rec_bytes );  
FioFile fio_list_get_file(FioFileList list, long FID); 
logical fio_list_get_retry(FioFileList list); 
void fio_list_set_retry(FioFileList list, logical q); 
long fio_list_tty_in(FioFileList list); 
long fio_list_tty_err(FioFileList list); 
long fio_list_tty_out(FioFileList list); 
logical fio_list_any_eof(FioFileList list, long mode); 
void fio_list_set_path(FioFileList list, char* pathlist); 
void fio_list_list_path(FioFileList list); 
logical fio_list_inquire_path( FioFileList f, char* name, char* pathname, long siz ); 
logical fio_pager_tty_read( FioFileList list, char* buf, long siz ); 
logical fio_pager_tty_write( FioFileList list, const char* buf,logical newline ); 
FioStack fio_iostack_init(void ); 
void fio_iostack_free( FioStack stack ); 
void fio_iostack_opush( FioStack stack, FioFileList list, FioFile file );  
void fio_iostack_ipush( FioStack stack, Tpio tpio, FioFileList list, FioFile file ); 
void fio_iostack_ipop( FioStack stack, Tpio tpio, FioFileList list, long mode );  
void fio_iostack_opop( FioStack stack, FioFileList list, long mode ); 
FioFile fio_iostack_file( FioStack stack, long fiomode ); 
long fio_iostack_get_level( FioStack stack, long fiomode ); 
void fio_iostack_set( FioStack stack, FioFileList list, FioFile file, long fiomode ); 
void fio_iostack_rewind_input( FioStack stack ); 
FioFile fio_iostack_input_level( FioStack stack, long level ); 
void fio_iostack_set_detab(FioStack stack, logical q); 
logical fio_iostack_read( FioStack stack, char* buf, char* prompt, long siz ); 
void fio_iostack_print_state( FioStack stack ); 
void fio_iostack_msg( FioStack iofs, FioFileList list, char* buf, logical newline, logical tty ); 
void fio_iostack_set_msg( FioStack iofs, void (*msgfunc)(char*, logical )); 
void fio_iostack_dashline( FioStack iofs, long n ) ;
void tpio_tp_init( Tpio tpio ); 
Tpio tpio_get( void ); 
void tpio_c( char* prompt, char* token, long siz );  
void tpio_cs( char* prompt, char* token, long siz );  
void tpio_g_cs( char* prompt, char** pptr, char* token, long siz );
void tpio_g_c( char* prompt, char** pptr, char* token, long siz );

void tpio_token( char* prompt, char** ptr, long mode, char* token, long siz );
void tpio_cl( char* prompt, char* token, long siz ); 
long tpio_i( char* prompt );  
short tpio_s( char* prompt );  
float tpio_r( char* prompt );  
double tpio_d( char* prompt ); 
long tpio_d_i( char* prompt, long xdefault );  
double tpio_d_d( char* prompt, double xdefault );  
short tpio_d_s( char* prompt, short xdefault );  
float tpio_d_r( char* prompt, float xdefault ); 
void tpio_in( char* filename ); 
void tpio_cc( char* prompt, char* buf, long siz ); 
void tpio_dbg( void ); 
void tpio_print_state( void );  
void tpio_disp( long i ); 
void tpio_tp_dbg( Tpio tpio ); 
void tpio_tp_disp( Tpio tpio, FioStack stack, FioFileList list, long i ); 
void tpio_tp_print_state( Tpio tpio, FioStack iofs, FioFileList list );  
void tpio_setbuf( char** arr, long n ); 
void tpio_cmdargs( char** args, long nargs, logical* interact, logical* opts ); 
void tpio_tp_in( Tpio tpio, FioFileList list, FioStack iofs, char* filename ); 
void tpio_tp_out( FioFileList list, FioStack iofs, char* filename,logical append, logical syserr ); 
void tpio_tp_cc(Tpio tpio, FioFileList list, FioStack iofs, char* pr, char* buf, long siz ); 
void esc_init( void ); 
void esc_status( void );	 
char esc_get( const char* name ); 
char esc_get_default( const char* name ); 
void esc_reset( const char* name, const char val ); 

ParseTokenType char_code( char c );
ParseTokenType cs_parse_namecode( char* name );

logical fio_posix_access( const char* name, const char* mode );

NameList cs_namelist_alloc( long n, long size );
void cs_namelist_free( NameList list );
void cs_namelist_add( NameList list, const char* name );
long cs_namelist_size( NameList list );
char* cs_namelist_entry( NameList list, long i );
long cs_namelist_match( NameList list, const char* name );
long cs_namelist_search( NameList list, const char* name );
logical cs_namelist_set( NameList list, const long entry, const char* name );
long cs_namelist_entry_size( NameList list );
long ar_search_c( const char* buf, char** array, const long n );

logical fio_pathlist_search( PathList pathlist, char* name, char* mode, char* result );
logical fio_pathlist_search_namelist( NameList list, char* name, char* mode, char* result );
PathList fio_pathlist_expand( char* path );
void fio_pathlist_free( PathList plist );

void ar_mix_print_generic( void* ptr, DataType type, char* buf, long size );
int ar_mix_set_elt( MixArray array, const long offset, const DataType type, const void* value );
int ar_mix_get_elt( const MixArray array, const long offset, DataType* type, void* value );
void ar_mix_copy_value( const void* ptr, const DataType type, void* value, logical copy, long size );

void ar_mix_delete_elt( MixArray array, const long offset );
void ar_mix_copy_elt( const MixArray src, const long srcoffset, MixArray dest, const long destoffset );
void ar_mix_move_elt( MixArray src, const long srcoffset, MixArray dest, const long destoffset );
void* ar_mix_ptr( const MixArray array, const long offset );
void ar_mix_numeric_cast( const void* in, const DataType intype, const DataType outtype, void* out );
void ar_mix_cast_array( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals );
void ar_mix_numeric_castArray( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals );
MixArray ar_mix_alloc_type( const DataType type, const long size, const long n );
MixArray ar_mix_alloc( const long n );
int ar_mix_extend( MixArray array, const long n );
void ar_mix_free( MixArray array );
void ar_mix_cast_elt( MixArray array, const long offset, const DataType type, void* value );
int ar_mix_cast_elt_cp( MixArray array, const long offset, char* value );
char* ar_mix_print_elt( const MixArray array, const long offset, char* buf, long size );

/* Unsupported?*/
void ar_mix_numeric_cast_array( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals );

long ar_alias_search( const AliasList array, const char* cptname );
void ar_alias_print( const AliasList array );
void ar_alias_free( AliasList array );
int ar_alias_extend( AliasList array, const long n );
AliasList ar_alias_alloc( const long n );
long ar_alias_set_name( AliasList array, char* name );
long ar_alias_get_dim( AliasList array, long offset );
void ar_alias_get_name( AliasList array, long offset, char* name, long maxlen );
void ar_alias_get_cpt( AliasList array, long offset, long cpt,  char* name, long maxlen );
void ar_alias_set_cpts( AliasList array, long entry, char** cpts, long dim );
double cs_read_numeric_val(  char** pptr );

DataType* ar_mix_array_types( MixArray array );
long ar_mix_type_size( const DataType type );
long ar_mix_array_size( const MixArray array );
void ar_mix_type_name( const DataType type, char* buf, long maxlen );

void* ut_realloc( void* data, long const old, long const new, const long size );
void* ar_mix_elt( const MixArray array, const long offset );
long ar_mix_n( MixArray array );
long ar_mix_nmax( MixArray array );
void ar_mix_type_min( const DataType type, void* value );
void ar_mix_type_max( const DataType type, void *value );
void ar_mix_type_zero( const DataType type, void* value );
void cs_compose_paren_list( char** items, const long n, char* buf, const long maxlen );

void cs_writef_zeros( double x, long ni, long nf, char* xbuf, long maxlen );

FileBuffer fio_file_buffer_alloc( long size, long chunk_size, long maxlines );
long fio_file_buffer_lines( FileBuffer buffer );
void fio_file_buffer_free( FileBuffer buffer );
void fio_file_buffer_clear( FileBuffer buffer );
void fio_file_buffer_status( FileBuffer buffer );
void fio_file_buffer_find_lines( FileBuffer buffer );
logical fio_file_buffer_read_line( FileBuffer buffer, char* line, long maxlen );
logical fio_file_buffer_seek( FileBuffer buffer, long n );
void fio_file_buffer_write_line( FileBuffer buffer, char* line );
logical fio_file_buffer_read( char* filename, FileBuffer buffer );

void fio_file_buffer_write( FileBuffer buffer, char* filename );
void fio_file_buffer_write_partial_line( FileBuffer buffer, char* line );
void fio_file_buffer_write_general_line( FileBuffer buffer, char* line, logical newline );
void fio_file_buffer_output( FileBuffer buffer );
void fio_file_buffer_msg( char* buf, logical newline );
long fio_file_buffer_pos( FileBuffer buffer );
logical fio_file_buffer_move( FileBuffer buffer, long n);
logical fio_file_buffer_skip_blanks( FileBuffer buffer );
logical fio_file_buffer_print_line( FileBuffer buffer );
void fio_file_buffer_msg( char* buf, logical newline );
void fio_file_buffer_output( FileBuffer buffer );

void cs_reverse( char* buf );
void cs_trim( char* buf );

long fio_dir_next_normal_file( FileDirectory dir, char* name, long maxlen );
FileDirectory fio_dir_open( const char* idirname );
void fio_dir_close( FileDirectory dir );
long fio_dir_next_filepath( FileDirectory dir, char* fn, long maxlen );
logical fio_read_line_alloc( FioFile file, char** line, long* lmax );
logical fio_get_line_alloc( FioFile file, char** bufp, long* sizp, logical dynamic );

logical ar_mix_is_max( const DataType type, void* value );
logical ar_mix_is_min( const DataType type, void* value );
char** ar_alias_get_cpt_list( AliasList array, long offset );
void* ar_gen_alloc( const DataType type, const long n );
logical ar_gen_get_cast_elt( const void* array, const long offset, const long n, const DataType array_type, const DataType type, void* value );
logical ar_gen_get_elt( const void* array, const long offset, const DataType type, void* value );
logical ar_gen_set_elt( void* array, const long offset, const DataType type, void* value );
void* ar_gen_ptr( const void* data, const DataType type, const long size, const long offset );
logical ar_mix_is_numeric( const DataType type );
long ar_mix_intersect( void* mins1, void* maxes1, long n1, void* mins2, void* maxes2, long n2,
                 DataType type, void** minsp, void** maxesp, long* np );
logical cs_strcmp( char* a, char* b );
void ar_mix_numeric_cast_elt( const void* in, const DataType intype, const DataType outtype, void* out, long inoffset, long outoffset );

char* cs_writef_clean_d( const double val, const char* fmt, const logical clean, const long sig, char* buf, const long siz );
char* cs_dup_cat( char* a, char* b, char* sep );
void ut_error_message( Error* errorp, char* buf, long maxlen );
int ar_mix_eq( const DataType type, void* value1, void* value2 );
char** cs_dup_array( char** data, long n );
char** cs_parse_parsep_list( const char* buffer, char* pars, char sep, long* np, logical mode );
long ut_sys_align( long total, long size, long old_size );
char** cs_parse_sequential_list( const char* buffer, char* quote, char sep, long* np );
#endif

/* Pixel coordinate functions in pixel.c */
long pixelno_from_pixel( long* Pixel, long* axes, long naxes );
void pixel_from_pixelno( long PixelNo, long* axes, long naxes, long* Pixel );

