/*                                                                
**  Copyright (C) 2002-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*
 *  kertable.h
 *
 *  This file contains the public interface to the ASCDM2 kernels
 *
 */


#ifndef KERTABLE_H
#define KERTABLE_H

#include "dmulib.h"
#include "dmkernel.h"

struct KernelStruct {
 char*  name;
 void (*Init)( void );
 dmBool (*DatasetAccess)( const char* name, const char* mode );
 dmDataset* (*DatasetOpen)( const char* name, const char* hint, const char* mode );
 dmDataset* (*DatasetCreate)( const char* name, const char* hint );
 void   (*DatasetClose)( dmDataset* ds );
 void   (*DatasetDelete)( dmDataset* ds );
 void   (*DatasetPrint)( const dmDataset* ds );

 dmBlock* (*BlockCreate)( dmDataset* ds, long no, const char* name, dmBlockType type, void* kb, char* hint );
 dmBlock* (*BlockOpen)( dmDataset* ds, const long blockNo, dmBlockType type, char* hint );
 dmErrCode (*BlockClose)( dmBlock* block );
 dmErrCode (*BlockDelete)( dmDataset* ds, const long blockNo );
 dmErrCode (*BlockFlush)( dmBlock* block );
 void (*ImageSetAxes)( dmBlock* block, const dmDataType type, const long* axlen, const long naxes );
 void (*BlockSetPref)( dmBlock* block, char** prefcols, const long npref );
 void (*BlockUpdateFilter)( dmBlock* block, dmSubspace* subspace );
 dmDescriptor* (*KeyWrite)( dmBlock* block, long pos, const char* name, const dmDataType type, const void* value, long nvals,
                const char* unit, const char* comment );
 void (*KeyDelete)( dmDescriptor* dd );
 dmBool  (*ReadComment)( dmBlock* block, const long keyno, char** tag, char** comment);
 dmBool  (*WriteComment)( dmBlock* block, const long keyno, const char* tag, const char* comment );
 void (*ColCreate)( dmBlock* block, dmDescriptor* dd, void* kdd, char* hint );
 dmErrCode (*ColSetRange)( dmDescriptor* dd ); 
 dmErrCode (*ColSetNull)( dmDescriptor* dd );
 dmErrCode (*ColSetBin)( dmDescriptor* dd );
 dmErrCode (*ColRead)( dmDescriptor* dd, long* llpix, long* urpix, long* dpix, long row1, long row2, void* value );
 dmErrCode (*ColWrite)( dmDescriptor* dd, long* llpix, long* urpix, long* dpix, long row1, long row2, void* value );
 dmErrCode (*ReadRows)( dmBlock* block, dmBuffer* buf, long row1, long row2 );
 dmErrCode (*WriteRows)( dmBlock* block, dmBuffer* buf, long row1, long row2 );
 void (*ColFlush)( dmDescriptor* dd );
 void (*DescriptorCopy)( dmDescriptor* srcdd, dmDescriptor* destdd );
 void (*DescriptorFree)( dmDescriptor* dd );
 void (*SubspaceDescriptorWrite)( dmDescriptor* dd, long cptno, char* value, char* tabname );
 void (*SubspaceTableWrite)( dmDescriptor* dd, char* tabname, void* val1, void* val2, long nvals );
 void (*CoordWrite)( dmDescriptor* dd );
 long (*BufferSize)( dmBlock* block );
 dmErrCode (*SetHint)( dmBlock* block, const char* name, const char* value );
 void (*GetKernelKey)( dmBlock* block, long no, char* buf, long maxlen );
 long (*GetNoKernelKeys)( dmBlock* block );

};


/*
 *  Private kernel functions
 */
long dmkGetNoKernels( void );
dmKernel dmkKernelNo( const long kerId );
void dmkSetKernel( const long kerId );
dmKernel dmkKernel( void );
void dmkGetKernelName( const long kerId, char* name, const long maxlen );
dmDataset* dmk_dataset_open( const char* dsname, const char* hint, const char* mode );
dmErrCode dmkKernelSetOption( dmBlock* block, const char* option );
long dmkGetTypedKernel( const char* kerType );
long dmkGetNamedKernel( const char* name );
void dmkSetTypedKernel( const long kerId, const char* kerType );
int dmkKernelSet( const char* buf, const char* kerType );
dmKernel dmk_block_kernel( dmBlock* block );
#endif
