/*                                                                
**  Copyright (C) 2002-2004,2007,2013  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/* ************************************************************* */
/*                                                               */
/*   dmlibP.h                                                    */
/*                                                               */
/*   Function prototypes and LivetimeData struct definition      */
/*                                                               */
/* ************************************************************* */

#ifndef DMLIBP_H
#define DMLIBP_H

/* Trigger loading of nutils.h */
#ifndef DMLIB_INTERNAL
#define DMLIB_INTERNAL
#endif

#include "kertable.h"
#include "dmi.h"
#include "dmlib.h"
#include "ascdm.h"
#define DM_BUF_BADROW -1

typedef struct {
 double* livetime;
 double* ontime;
 long* id;
 long n;
} LivetimeData;

/* ========================================================================== */
/* Dataset level prototypes */

dmDataset*  dmp_dataset_open_mode( const char* ddsname, const char* hints, const char* mode );


/* ========================================================================== */
/* Block level prototypes */

int           dmp_block_close( dmBlock* block );
dmDescriptor* dmp_block_copy_image( dmBlock* srcBlock, dmBlock* destBlock );
void          dmp_block_flush( dmBlock* block, dmBool flush_tables );
int           dmp_block_flush_coords( dmBlock* block );
int           dmp_block_flush_subspace( dmBlock* pBlock, dmBool flush_tables );
dmBlock*      dmp_block_open( dmDataset* ds, long blockno, dmBlockType type );
dmBlock*      dmp_block_open_mode( dmDataset* ds, const char* vspec, long no, dmBlockType btype, void* kb, const char* mode );
dmBlock*      dmp_block_open_raw( dmDataset* ds, long no, char* name, dmBlockType type, void* kb, char* hint, const char* mode );
dmSubspace*   dmp_block_parse_filter( dmBlock* RawBlock, char* buf );
void          dmp_block_process_range_filter( dmBlock* FilteredBlock, char* spec );

void          dmp_block_update_special( dmBlock* block );

dmDescriptor* dmp_image_create_default_col( dmBlock* block );

/* ========================================================================== */
/* Descriptor */

dmBool        dmp_descriptor_compare( dmDescriptor* dd1, dmDescriptor* dd2, long mask );
void          dmp_descriptor_free( dmDescriptor* dd );
int           dmp_descriptor_set_bin( dmDescriptor* pDesc, const dmDataType type, void* value );
int           dmp_descriptor_set_null( dmDescriptor* pDesc, const dmDataType type, void* value );
int           dmp_descriptor_set_range( dmDescriptor* dd, const dmDataType type, void* tlmin, void* tlmax );
int           dmp_descriptor_reset_range( dmDescriptor* dd, const dmDataType type, void* tlmin, void* tlmax );

dmDescriptor* dmp_key_write( dmBlock* block, const char* name, const dmDataType type, const void* value, 
	                     long nvals, const char* unit, const char* desc, const dmBool special );

dmDescriptor* dmp_subspace_col_init( dmBlock* block, const char* total_name, dmDescriptor* col_dd );
int           dmp_subspace_col_write_keys( dmBlock* pBlock, dmDescriptor* pDesc, long count, char* strUsedValue, char* TabName );
dmBool        dmp_subspace_col_write_values( dmBlock* block, dmDescriptor* pDesc, long ncpts, dmBool flush_tables );

void          dmp_image_set_axes( dmBlock* block, dmDataType type, long* axlen, long naxes );
int           dmp_image_get_subarray(dmDescriptor* imdata, long* pixelLl, long* pixelUr, dmDataType type, void* values);
int           dmp_image_set_subarray(dmDescriptor* imdata, long* pixelLl, long* pixelUr, dmDataType type, void* values);

/* ========================================================================== */
/* Region */

regRegion* dmp_region_read_file( char* regfile );


/* ========================================================================== */
/* General */

void dmk_init( void );
void dmp_init_func( const long a, const void* b );
int  dmk_kernel_eq( dmDataset* ds1, dmDataset* ds2 );

void dmp_block_copy_buffer( dmBlock* block1, dmBlock* block2 );
long dmp_buffer_adjust( dmBlock* block, long rowno, long nrows, int create );
void dmp_buffer_flush( dmBlock* block );
void dmp_buffer_free( dmBuffer* buf );
long dmp_buffer_get_data( dmBlock* block, dmDescriptor* dd, long rowno, long nrows, long item1, 
                          long item2,  long* ll, long* ur, long* dp, void* data );
long dmp_buffer_get_cast_data( dmDescriptor* dd, void* val, dmDataType type, long row, long nrows, 
                          long start, long npixels, long* llpix, long* urpix, long* dpix );
long dmp_buffer_set_cast_data( dmDescriptor* dd, void* val, dmDataType type,  long vrowno, 
                          long nvrows, long start, long nitems );

void dmp_buffer_rebin_image( dmBlock* block, dmDescriptor* parent_image, dmDescriptor* image, void* data );

void dmp_subspace_free( dmSubspace* subspace );


/* ========================================================================== */
/* May be public someday */
int           dmCopyCoord( dmDescriptor* col_dd, dmDescriptor* parent, long coordNo );
dmErrCode     dmGetVectorArray( dmDescriptor* dd, dmDataType type, void* vals, long dim, long start, long nvals);
long          dmGetVectorArrays( dmDescriptor* dd, dmDataType type, void* vals, long dim, long row, long nrows, long start, long nvals);
long          dmSetVectorArrays( dmDescriptor* dd, dmDataType type, void* vals, long dim, long row, long nrows, long start, long nvals);
dmErrCode     dmSetVectorArray( dmDescriptor* dd, dmDataType type, void* vals, long dim, long start, long nvals);
dmDescriptor* dmp_key_read( dmBlock* block, const char* name, const dmDataType type, void* value );
long          dmp_buffer_read_offset( dmBlock* block, long col_offset, long vrowno, long nvrows, void* values, long item_offset, 
                 long item_nbytes, long vrow_out_offset, long* llpix, long* urpix, long* dpix );
void          dmp_copy_buffer_to_row( dmBlock* block, void* row, long rowno );
int           dmp_block_close_mode( dmBlock* block, dmBool flush_gtis );
void          dmp_flush_cols( dmBlock* block );
void          dmp_block_flush_kernel( dmBlock* block );
char*         dmKernelGetOption( void );
#endif /* DMLIBP_H */



