/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm.c
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:  
 *   Provides a C-extension for Python to the DM functions
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */

#include "cxcdm.h"

/* Initialization function for _cxcdm module */
PyMODINIT_FUNC init_cxcdm(void)
{
  PyObject* m = NULL;

  if (PyType_Ready(&dmDataset_Type) < 0)
    goto fail;

  if (PyType_Ready(&dmBlock_Type) < 0)
    goto fail;

  if (PyType_Ready(&dmDescriptor_Type) < 0)
    goto fail;

  if (PyType_Ready(&dmBlockType_Type) < 0)
    goto fail;

  if (PyType_Ready(&dmDescriptorType_Type) < 0)
    goto fail;

  if (PyType_Ready(&dmElementType_Type) < 0)
    goto fail;

  m = Py_InitModule( (char*)"_cxcdm", cxcdmFuncs );
  if ( m == NULL )
    goto fail;

  import_array();  /* Must be present for NumPy. */

  /* Import the region module   */
  if ( import_region() < 0 )
    goto fail;

  Py_INCREF(&dmDataset_Type);
  Py_INCREF(&dmBlock_Type);
  Py_INCREF(&dmDescriptor_Type);
  Py_INCREF(&dmBlockType_Type);
  Py_INCREF(&dmDescriptorType_Type);
  Py_INCREF(&dmElementType_Type);

  PyModule_AddObject(m, "dmDataset",    (PyObject *)&dmDataset_Type);
  PyModule_AddObject(m, "dmBlock",      (PyObject *)&dmBlock_Type);
  PyModule_AddObject(m, "dmDescriptor", (PyObject *)&dmDescriptor_Type);
  PyModule_AddObject(m, "dmBlockType", (PyObject *)&dmBlockType_Type);
  PyModule_AddObject(m, "dmDescriptorType", (PyObject *)&dmDescriptorType_Type);
  PyModule_AddObject(m, "dmElementType", (PyObject *)&dmElementType_Type);

  /* MCD TEMP:  example for adding an integer constant */
  /* PyModule_AddIntConstant(m, "dmUNKBLOCK", (long)dmUNKBLOCK); */
  PyModule_AddIntConstant(m, "dmNOMOREROWS", (long)dmNOMOREROWS); 
  PyModule_AddIntConstant(m, "dmENDOFTABLE", (long)dmENDOFTABLE); 

  /* descriptor mask constants from dmerror.h */
  PyModule_AddIntConstant(m, "dmDESCNAME", (unsigned long)dmDESCNAME);
  PyModule_AddIntConstant(m, "dmDESCVALUE", (unsigned long)dmDESCVALUE);
  PyModule_AddIntConstant(m, "dmDESCUNIT", (unsigned long)dmDESCUNIT);
  PyModule_AddIntConstant(m, "dmDESCKIND", (unsigned long)dmDESCKIND);
  PyModule_AddIntConstant(m, "dmDESCTYPE", (unsigned long)dmDESCTYPE);
  PyModule_AddIntConstant(m, "dmDESCELMTYPE", (unsigned long)dmDESCELMTYPE);
  PyModule_AddIntConstant(m, "dmDESCDIM", (unsigned long)dmDESCDIM);
  PyModule_AddIntConstant(m, "dmDESCLENS", (unsigned long)dmDESCLENS);
  PyModule_AddIntConstant(m, "dmDESCELMDIM", (unsigned long)dmDESCELMDIM);
  PyModule_AddIntConstant(m, "dmDESCCPTNAMES", (unsigned long)dmDESCCPTNAMES);
  PyModule_AddIntConstant(m, "dmDESCCMNT", (unsigned long)dmDESCCMNT);
  PyModule_AddIntConstant(m, "dmDESCALL", (unsigned long)dmDESCALL);

  

  return;

 fail:
  Py_XDECREF(m);
  return;
}
