/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm.h
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:
 *    header info for cxcdm Python Module
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */
#include "Python.h"

#include "ascdm.h"
#include <numpy/noprefix.h>

char errmsg[dmMAXSTR+1];

#include "cxcdm_mm.h"
#include "cxcdm_dataset.h"
#include "cxcdm_block.h"
#include "cxcdm_descriptor.h"

CMMLL* cxcdm_memory_master = NULL;
long   cxcdm_verbose_level = 0;

/* ----------------------- */
#include "cxcdm_utils.c"
#include "cxcdm_mm.c"
#include "cxcdm_dataset.c"
#include "cxcdm_block.c"
#include "cxcdm_descriptor.c"
#include "cxcdm_misc.c"

void init_cxcdm(void);

/* Set up the methods table  */
static PyMethodDef cxcdmFuncs[] =
{
  {"dmDatasetAccess",    (PyCFunction)_dmDatasetAccess, METH_VARARGS, 
      "Example:  dmDatasetAccess( fname, mode=<R|RW> )"},
  {"dmDatasetClose",    (PyCFunction)_dmDatasetClose, METH_VARARGS, 
      "Example:  dmDatasetClose( ds )"},
  {"dmDatasetDelete",    (PyCFunction)_dmDatasetDelete, METH_VARARGS, 
      "Example:  dmDatasetDelete( ds )"},
  {"dmDatasetDestroy",   (PyCFunction)_dmDatasetDestroy, METH_VARARGS,
      "Example:  dmDatasetDestroy( fname )"},
  {"dmDatasetCreate",   (PyCFunction)_dmDatasetCreate, METH_VARARGS, 
      "Example:  ds = dmDatasetCreate( fname )"},
  {"dmDatasetCreateImage", (PyCFunction)_dmDatasetCreateImage, METH_VARARGS, 
      "Example:  ds = dmDatasetCreateImage( ds, name, dtype, axes )"},
  {"dmDatasetGetBlock",(PyCFunction)_dmDatasetGetBlock, METH_VARARGS, 
      "Example:  block = dmDatasetGetBlock( ds, blkno | blkname )"},
  {"dmDatasetGetBlockName",(PyCFunction)_dmDatasetGetBlockName, METH_VARARGS, 
      "Example:  name = dmDatasetGetBlockName( ds, blkno )"},
  {"dmDatasetGetBlockType",(PyCFunction)_dmDatasetGetBlockType, METH_VARARGS, 
      "Example:  blocktype = dmDatasetGetBlockType( ds, blkno )"},
  {"dmDatasetGetCurrentBlockNo", (PyCFunction)_dmDatasetGetCurrentBlockNo, METH_VARARGS, 
      "Example:  blkno = dmDatasetGetCurrentBlockNo( ds )"},
  {"dmDatasetGetName",     (PyCFunction)_dmDatasetGetName, METH_VARARGS, 
      "Example:  name = dmDatasetGetName( ds )"},
  {"dmDatasetGetNoBlocks", (PyCFunction)_dmDatasetGetNoBlocks, METH_VARARGS, 
      "Example:  nblocks = dmDatasetGetNoBlocks( ds )"},
  {"dmDatasetMoveToBlock", (PyCFunction)_dmDatasetMoveToBlock, METH_VARARGS, 
      "Example:  block = dmDatasetMoveToBlock( ds, blkno )"},
  {"dmDatasetOpen",    (PyCFunction)_dmDatasetOpen,  METH_VARARGS | METH_KEYWORDS, 
      "Example:  ds = dmDatasetOpen( fname [, update=<True|False>] )"},
  {"dmDatasetWriteable", (PyCFunction)_dmDatasetWriteable,  METH_VARARGS, 
      "Example:  res = dmDatasetWriteable( ds )"},

  {"dmBlockFlush",    (PyCFunction)_dmBlockFlush, METH_VARARGS, 
      "Example:  dmBlockFlush( block )"},
  {"dmBlockClose",    (PyCFunction)_dmBlockClose,    METH_VARARGS, 
      "Example:  dmBlockClose( block )"},
  {"dmBlockCopy",    (PyCFunction)_dmBlockCopy,    METH_VARARGS, 
      "Example:  dmBlockCopy( srcblock, destblock, option=<HEADER|SUBSPACE>)"},
  {"dmBlockCopyCol",    (PyCFunction)_dmBlockCopyCol,    METH_VARARGS, 
      "Example:  dmBlockCopyCol( block, desc )"},
  {"dmBlockCopyData",    (PyCFunction)_dmBlockCopyData,    METH_VARARGS, 
      "Example:  dmBlockCopyData( srcblock, destblock )"},
  {"dmBlockCopyWCS",    (PyCFunction)_dmBlockCopyWCS,    METH_VARARGS, 
      "Example:  dmBlockCopyWCS( srcblock, destblock )"},
  {"dmBlockCreate", (PyCFunction)_dmBlockCreate,    METH_VARARGS | METH_KEYWORDS, 
      "Example:  block = dmBlockCreate( ds, name, type=<dmTABLE|dmIMAGE>]  )"},
  {"dmBlockCreateGenCopy", (PyCFunction)_dmBlockCreateGenCopy,    METH_VARARGS | METH_KEYWORDS, 
      "Example:  block = dmBlockCreateGenCopy( ds, name, inblock[, histstr=<str>, copydata=<True|FALSE>, copysubspace=<True|False>]  )"},
  {"dmBlockCreateSubspaceCpt",    (PyCFunction)_dmBlockCreateSubspaceCpt,    METH_VARARGS, 
      "Example:  dmBlockCreateSubspaceCpt( block )"},
  {"dmBlockCreateEmptySubspaceCpt",    (PyCFunction)_dmBlockCreateEmptySubspaceCpt,    METH_VARARGS, 
      "Example:  dmBlockCreateEmptySubspaceCpt( block )"},
  {"dmBlockIntersectSubspace",    (PyCFunction)_dmBlockIntersectSubspace,    METH_VARARGS, 
      "Example:  dmBlockIntersectSubspace( inblock1, inblock2, outblock )"},
  {"dmBlockMergeSubspace",    (PyCFunction)_dmBlockMergeSubspace,    METH_VARARGS, 
      "Example:  dmBlockMergeSubspace( inblock1, inblock2, outblock )"},
  {"dmBlockDelete",    (PyCFunction)_dmBlockDelete,    METH_VARARGS, 
      "Example:  dmBlockDelete( block )"},
  {"dmBlockGetCurrSubspaceCpt", (PyCFunction)_dmBlockGetCurrSubspaceCpt,    METH_VARARGS, 
      "Example:  css  = dmBlockGetCurrSubspaceCpt( block )"},
  {"dmBlockGetDataset", (PyCFunction)_dmBlockGetDataset, METH_VARARGS, 
      "Example:  name = dmBlockGetDataset( block )"},
  {"dmBlockGetKey",     (PyCFunction)_dmBlockGetKey,     METH_VARARGS, 
      "Example:  key  = dmBlockGetKey( block, keyno )"},
  {"dmBlockGetKeyList", (PyCFunction)_dmBlockGetKeyList, METH_VARARGS, 
      "Example:  keys = dmBlockGetKeyList( block )"},
  {"dmBlockMoveToKey",  (PyCFunction)_dmBlockMoveToKey,  METH_VARARGS, 
      "Example:  key  = dmBlockMoveToKey( block, <str name>|<int colno> )"},
  {"dmBlockAdvanceKeys",  (PyCFunction)_dmBlockAdvanceKeys,  METH_VARARGS, 
      "Example:  key  = dmBlockAdvanceKeys( block, step )"},
  {"dmBlockGetName",    (PyCFunction)_dmBlockGetName,    METH_VARARGS, 
      "Example:  name = dmBlockGetName( block )"},
  {"dmBlockGetNo",      (PyCFunction)_dmBlockGetNo,      METH_VARARGS, 
      "Example:  blkno = dmBlockGetNo( block )"},
  {"dmBlockGetNoKeys",      (PyCFunction)_dmBlockGetNoKeys,      METH_VARARGS, 
      "Example:  nokeys = dmBlockGetNoKeys( block )"},
  {"dmBlockGetNoSubspaceCpts", (PyCFunction)_dmBlockGetNoSubspaceCpts,    METH_VARARGS, 
      "Example:  nss  = dmBlockGetNoSubspaceCpts( block )"},
  {"dmBlockGetSubspace", (PyCFunction)_dmBlockGetSubspace,    METH_VARARGS, 
      "Example:  (ncpts, ncols, descs) = dmBlockGetSubspace( block )"},
  {"dmBlockGetSubspaceCol", (PyCFunction)_dmBlockGetSubspaceCol,    METH_VARARGS, 
      "Example:  desc = dmBlockGetSubspaceCol( block, <str name>|<int colno> )"},
  {"dmBlockListKeys",      (PyCFunction)_dmBlockListKeys,      METH_VARARGS, 
      "Example:  keystring = dmBlockListKeys( block )"},
  {"dmBlockSetSubspaceCpt", (PyCFunction)_dmBlockSetSubspaceCpt,    METH_VARARGS,
      "Example:  dmBlockSetSubspaceCpt( block, cptno )"},
  {"dmBlockGetNoSubspaceCols", (PyCFunction)_dmBlockGetNoSubspaceCols,    METH_VARARGS, 
      "Example:  nsc  = dmBlockGetNoSubspaceCols( block )"},
  {"dmBlockGetType",    (PyCFunction)_dmBlockGetType,    METH_VARARGS, 
      "Example:  blocktype = dmBlockGetType( block )"},
  {"dmBlockReadComment",    (PyCFunction)_dmBlockReadComment,    METH_VARARGS, 
      "Example:  (tag, comment) = dmBlockReadComment( block )"},
  {"dmBlockWriteComment",    (PyCFunction)_dmBlockWriteComment,   METH_VARARGS, 
      "Example:  dmBlockWriteComment( block, 'COMMENT'|'HISTORY', str value )"},
  {"dmBlockSetBufferMaxSize",    (PyCFunction)_dmBlockSetBufferMaxSize, METH_VARARGS, 
      "Example:  dmBlockSetBufferMaxSize( block, size )"},
  {"dmCopyGeneric",      (PyCFunction)_dmCopyGeneric,      METH_VARARGS, 
      "Example:  dmCopyGeneric( srcdesc, destdesc )"},
  {"dmTableClose",      (PyCFunction)_dmTableClose,      METH_VARARGS, 
      "Example:  dmTableClose( block )"},
  {"dmTableGetNoCols",  (PyCFunction)_dmTableGetNoCols,  METH_VARARGS, 
      "Example:  ncols = dmTableGetNoCols( block )"},
  {"dmTableGetNoRows",  (PyCFunction)_dmTableGetNoRows,  METH_VARARGS, 
      "Example:  nrows = dmTableGetNoRows( block )"},
  {"dmTableGetRowNo",  (PyCFunction)_dmTableGetRowNo,  METH_VARARGS, 
      "Example:  rownum = dmTableGetRowNo( block )"},
  {"dmTableNextRow",  (PyCFunction)_dmTableNextRow,  METH_VARARGS, 
      "Example:  rownum = dmTableNextRow( block )"},
  {"dmTableOpenColumn", (PyCFunction)_dmTableOpenColumn, METH_VARARGS, 
      "Example:  col   = dmTableOpenColumn( block, <str name>|<int colno> )"},
  {"dmTableOpenColumnList", (PyCFunction)_dmTableOpenColumnList, METH_VARARGS, 
      "Example:  cols  = dmTableOpenColumnList( block )"},
  {"dmTableSetRow",  (PyCFunction)_dmTableSetRow,  METH_VARARGS, 
      "Example:  rownum = dmTableSetRow( block, rownum )"},

  {"dmImageGetDataDescriptor", (PyCFunction)_dmImageGetDataDescriptor, METH_VARARGS,
      "Example:  new_dd = dmImageGetDataDescriptor( block )"},

  {"dmGetArrayDim", (PyCFunction)_dmGetArrayDim, METH_VARARGS,
      "Example:  ndim = dmGetArrayDim( dd )"},
  {"dmGetArrayDimensions", (PyCFunction)_dmGetArrayDimensions, METH_VARARGS,
      "Example:  ndarray = dmGetArrayDimensions( dd )"},
  {"dmGetArraySize", (PyCFunction)_dmGetArraySize, METH_VARARGS,
      "Example:  size = dmGetArraySize( dd )"},
  {"dmGetCurrentArraySize", (PyCFunction)_dmGetCurrentArraySize, METH_VARARGS,
      "Example:  size = dmGetCurrentArraySize( dd )"},
  {"dmDescriptorGetArea", (PyCFunction)_dmDescriptorGetArea, METH_VARARGS,
      "Example:  (area, field_area) = dmDescriptorGetArea( dd )"},
  {"dmDescriptorGetBin", (PyCFunction)_dmDescriptorGetBin, METH_VARARGS,
      "Example:  binsize = dmDescriptorGetBin( dd )"},
  {"dmDescriptorGetBlock", (PyCFunction)_dmDescriptorGetBlock, METH_VARARGS,
      "Example:  block = dmDescriptorGetBlock( dd )"},
  {"dmDescriptorDelete", (PyCFunction)_dmDescriptorDelete, METH_VARARGS,
      "Example:  dmDescriptorDelete( dd )"},
  {"dmDescriptorCompare", (PyCFunction)_dmDescriptorCompare, METH_VARARGS,
      "Example:  dmDescriptorCompare( dd, dd2, mask )"},

  {"dmDescriptorGetCoord", (PyCFunction)_dmDescriptorGetCoord, METH_VARARGS | METH_KEYWORDS,
      "Example:  new_dd = dmDescriptorGetCoord( dd [, coordno] )"},
  {"dmDescriptorGetLength",(PyCFunction)_dmDescriptorGetLength,  METH_VARARGS, 
      "Example:  len  = dmDescriptorGetLength( dd )"},
  {"dmGetElementType",(PyCFunction)_dmGetElementType,  METH_VARARGS,
      "Example:  dmElementType  = dmGetElementType( dd )"},
  {"dmDescriptorGetType",(PyCFunction)_dmDescriptorGetType,  METH_VARARGS,
      "Example:  dmDescriptorType  = dmDescriptorGetType( dd )"},
  {"dmDescriptorGetNo", (PyCFunction)_dmDescriptorGetNo,  METH_VARARGS, 
      "Example:  no = dmDescriptorGetNo( dd )"},
  {"dmDescriptorGetNoCoords", (PyCFunction)_dmDescriptorGetNoCoords,  METH_VARARGS, 
      "Example:  ncoords = dmDescriptorGetNoCoords( dd )"},
  {"dmDescriptorGetRange", (PyCFunction)_dmDescriptorGetRange, METH_VARARGS, 
      "Example:  (rangemin, rangemax) = dmDescriptorGetRange( dd )"},
  {"dmDescriptorGetNull", (PyCFunction)_dmDescriptorGetNull, METH_VARARGS,
      "Example:  tnull = dmDescriptorGetNull( dd )"},
  {"dmDescriptorIsVarLength", (PyCFunction)_dmDescriptorIsVarLength, METH_VARARGS,
      "Example:  isvar = dmDescriptorIsVarLength( dd )"},
  {"dmGetCpt",      (PyCFunction)_dmDescriptorGetCpt,      METH_VARARGS, 
      "Example:  desc = dmGetCpt( dd, cptno )"},
  {"dmGetCptName",      (PyCFunction)_dmGetCptName,      METH_VARARGS, 
      "Example:  cptname = dmGetCptName( dd, cptno )"},
  {"dmGetDataType", (PyCFunction)_dmDescriptorGetDataType,  METH_VARARGS, 
      "Example:  type = dmGetDataType( dd )"},
  {"dmGetDesc",     (PyCFunction)_dmDescriptorGetDesc,      METH_VARARGS, 
      "Example:  desc = dmGetDesc( dd )"},
  {"dmGetDisp",     (PyCFunction)_dmGetDisp,      METH_VARARGS, 
      "Example:  disp = dmGetDisp( dd )"},
  {"dmGetElementDim",(PyCFunction)_dmDescriptorGetElementDim,  METH_VARARGS, 
      "Example:  vdim = dmGetElementDim( dd )"},
  {"dmGetName",     (PyCFunction)_dmDescriptorGetName,      METH_VARARGS, 
      "Example:  name = dmGetName( dd )"},
  {"dmGetUnit",     (PyCFunction)_dmDescriptorGetUnit,      METH_VARARGS, 
      "Example:  unit = dmGetUnit( dd )"},
  {"dmGetData",     (PyCFunction)_dmDescriptorGetData,      METH_VARARGS | METH_KEYWORDS,
      "Example:  data = dmGetData( dd[, rowno, nrows] )"},
  {"dmImageClose",      (PyCFunction)_dmImageClose,      METH_VARARGS,
      "Example:  dmImageClose( block )"},
  {"dmImageGetData",(PyCFunction)_dmDescriptorGetImageData, METH_VARARGS | METH_KEYWORDS,
      "Example:  data = dmImageGetData( dd[, pixelLL, pixelUR] )"},
  {"dmImageSetData",(PyCFunction)_dmDescriptorSetImageData, METH_VARARGS | METH_KEYWORDS,
      "Example:  dmImageSetData( dd, values[, pixelLL] )"},
  {"dmSetCptName",     (PyCFunction)_dmSetCptName,      METH_VARARGS | METH_KEYWORDS,
      "Example:  dmSetCptName( dd, cptno, name )"},
  {"dmSetData",     (PyCFunction)_dmDescriptorSetData,      METH_VARARGS | METH_KEYWORDS,
      "Example:  dmSetData( dd, values[, rowno] )"},
  {"dmSetDesc",     (PyCFunction)_dmDescriptorSetDesc,      METH_VARARGS, 
      "Example:  dmSetDesc( dd, description )"},
  {"dmSetDisp",     (PyCFunction)_dmSetDisp,      METH_VARARGS, 
      "Example:  dmSetDisp( dd, dispFormat )"},
  {"dmSetName",     (PyCFunction)_dmDescriptorSetName,      METH_VARARGS, 
      "Example:  name = dmSetName( dd )"},
  {"dmSetUnit",     (PyCFunction)_dmDescriptorSetUnit,      METH_VARARGS, 
      "Example:  unit = dmSetUnit( dd )"},
  {"dmDescriptorSetBin", (PyCFunction)_dmDescriptorSetBin,      METH_VARARGS, 
      "Example:  dmDescriptorSetBin( dd, binval )"},
  {"dmDescriptorSetNull", (PyCFunction)_dmDescriptorSetNull,      METH_VARARGS, 
      "Example:  dmDescriptorSetNull( dd, nullval )"},
  {"dmDescriptorSetRange",  (PyCFunction)_dmDescriptorSetRange,  METH_VARARGS,
      "Example:  dmSetRange( dd, tmin, tmax )"},

  {"dmKeyOpen",       (PyCFunction)_dmKeyOpen,       METH_VARARGS | METH_KEYWORDS, 
      "Example:  key = dmKeyOpen( block, \"MISSION\" )"},
  {"dmKeyRead",       (PyCFunction)_dmKeyRead,       METH_VARARGS | METH_KEYWORDS, 
      "Example:  (key, value) = dmKeyRead( block, \"MISSION\" [, dtype] )"},
  {"dmKeyWrite",      (PyCFunction)_dmKeyWrite,      METH_VARARGS | METH_KEYWORDS, 
      "Example:  key = dmKeyWrite( block, \"EQUINOX\", 2000.0 [, unit=<str>, desc=<str>] )"},


  {"dmColumnCreate",  (PyCFunction)_dmColumnCreate,  METH_VARARGS | METH_KEYWORDS, 
      "Example:  col = dmColumnCreate( block, \"SLOT\", dtype[, itemsize=<long>, unit=<str>, desc=<str>] ) "},

  {"dmSubspaceColCreate",  (PyCFunction)_dmSubspaceColCreate,  METH_VARARGS | METH_KEYWORDS, 
      "Example:  sscol = dmSubspaceColCreate( block, <str tabname|None>, <str name>, <str colnames|None>, <str units>, <mins>, <maxes> ) "},
  {"dmSubspaceColCreateRegion",  (PyCFunction)_dmSubspaceColCreateRegion,  METH_VARARGS | METH_KEYWORDS, 
      "Example:  sscol = dmSubspaceColCreateRegion( block, <str name>, <str colnames>, dtype,  <str unit>, <regRegion|str region> ) "},
  {"dmSubspaceColOpen",  (PyCFunction)_dmSubspaceColOpen,  METH_VARARGS, 
      "Example:  sscol = dmSubspaceColOpen( block, <str name> ) "},
  {"dmSubspaceColGet",  (PyCFunction)_dmSubspaceColGet,  METH_VARARGS,
      "Example:  (mins,maxes) = dmSubspaceColGet( dd ) "},
  {"dmSubspaceColGetTableName",  (PyCFunction)_dmSubspaceColGetTableName,  METH_VARARGS,
      "Example:  name = dmSubspaceColGetTableName( dd ) "},
  {"dmSubspaceColGetRegion",  (PyCFunction)_dmSubspaceColGetRegion,  METH_VARARGS,
      "Example:  reg = dmSubspaceColGetRegion( dd ) "},
  {"dmSubspaceColGetRegionString",  (PyCFunction)_dmSubspaceColGetRegionString,  METH_VARARGS,
      "Example:  reg_str = dmSubspaceColGetRegionStr( dd ) "},
  {"dmSubspaceColRead",  (PyCFunction)_dmSubspaceColRead,  METH_VARARGS, 
      "Example:  (sscol,mins,maxes) = dmSubspaceColRead( block, <str name> ) "},
  {"dmSubspaceColSet",  (PyCFunction)_dmSubspaceColSet,  METH_VARARGS,
      "Example:  dmSubspaceColSet( dd, mins, maxes ) "},
  {"dmSubspaceColSetRegion",  (PyCFunction)_dmSubspaceColSetRegion,  METH_VARARGS,
      "Example:  dmSubspaceColSetRegion( dd, reg ) "},
  {"dmSubspaceColSetTableName",  (PyCFunction)_dmSubspaceColSetTableName,  METH_VARARGS,
      "Example:  dmSubspaceColSetTableName( dd, name ) "},
  {"dmSubspaceColUpdate",  (PyCFunction)_dmSubspaceColUpdate,  METH_VARARGS,
      "Example:  dmSubspaceColUpdate( dd, mins, maxes ) "},
  {"dmArrayCreateAxisGroup", (PyCFunction)_dmArrayCreateAxisGroup, METH_VARARGS,
      "Example:  new_dd = dmArrayCreateAxisGroup( dd, name, dtype, unit, cptnames )"},
  {"dmArrayGetAxisGroup", (PyCFunction)_dmArrayGetAxisGroup, METH_VARARGS,
      "Example:  new_dd = dmArrayGetAxisGroup( dd,  axisgroupno )"},
  {"dmArrayGetNoAxisGroups", (PyCFunction)_dmArrayGetNoAxisGroups, METH_VARARGS,
      "Example:  ngroups = dmArrayGetNoAxisGroups( dd )"},
  {"dmCoordGetAxisGroupNo", (PyCFunction)_dmCoordGetAxisGroupNo,  METH_VARARGS,
      "Example:  grpno = dmCoordGetAxisGroupNo( dd )"},
  {"dmCoordGetParams", (PyCFunction)_dmCoordGetParams,  METH_VARARGS,
      "Example:  params = dmCoordGetParams( dd )"},
  {"dmCoordGetTransform",  (PyCFunction)_dmCoordGetTransform,  METH_VARARGS,
      "Example:  (crpix,crval,cdelt) = dmCoordGetTransform( dd )"},
  {"dmCoordGetTransformType",  (PyCFunction)_dmCoordGetTransformType,  METH_VARARGS,
      "Example:  (ttype, nparam) = dmCoordGetTransformType( dd )"},
  {"dmIsEmpty",  (PyCFunction)_dmIsEmpty,  METH_VARARGS, 
      "Example:  res = dmIsEmpty( block )"},
  {"dmCoordSetParam",  (PyCFunction)_dmCoordSetParam,  METH_VARARGS,
      "Example:  dmCoordSetTransform( dd, name, value )\n             name = ['LONGPOLE','LATPOLE','ROT','EQUINOX']"},
  {"dmCoordSetTransform",  (PyCFunction)_dmCoordSetTransform,  METH_VARARGS,
      "Example:  dmCoordSetTransform( dd, crpix, crval, cdelt )"},

  {"_dmpBlockOpenMode", (PyCFunction)_dmpBlockOpenMode, METH_VARARGS, ""},
  {"_dmCoordCalc",      (PyCFunction)_dmCoordCalc,      METH_VARARGS, ""},
  {"_dmCoordInvert",    (PyCFunction)_dmCoordInvert,    METH_VARARGS, ""},
  {"dmCoordCreate",    (PyCFunction)_dmCoordCreate,    METH_VARARGS|METH_KEYWORDS, 
      "Example: dmCoordCreate ( dd, name, unit [, cptnames, transform, crpix, crval, cdelt, params])"},
  {"dmGetErrorMessage",     (PyCFunction)_dmGetErrorMessage,     METH_VARARGS, 
       "Example:  str = dmGetErrorMessage()"},
  {"dmGetVersion",     (PyCFunction)_dmGetVersion,     METH_VARARGS, 
       "Example:  str = dmGetVersion()"},
  {"dmResetErrorState",     (PyCFunction)_dmResetErrorState,     METH_VARARGS, 
       "Example:  dmResetErrorState()"},
  {"_dmSetVerbose",     (PyCFunction)_dmSetVerbose,     METH_VARARGS, ""},
  {"dmTanPixToWorld", (PyCFunction)_dmTanPixToWorld, METH_VARARGS|METH_KEYWORDS,
       "Example:  world = dmTanPixToWorld( pix, aspect, crpix, cdelt)"},
  {"dmTanWorldToPix", (PyCFunction)_dmTanWorldToPix, METH_VARARGS|METH_KEYWORDS,
       "Example:  pix = dmTanWorldToPix( world, aspect, crpix, cdelt)"},
  {"dmTableReadRegion",  (PyCFunction)_dmTableReadRegion,  METH_VARARGS,
      "Example:  reg = dmTableReadRegion( block ) "},
  {"dmTableWriteRegion",  (PyCFunction)_dmTableWriteRegion,  METH_VARARGS,
      "Example:  block = dmTableWriteRegion( ds, name, dd, reg ) "},
  {NULL, NULL, 0, NULL}     /* Sentinel - marks the end of this structure */

};/*end... */
