/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 * FILE NAME:  cxcdm_block.c
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:  
 *   Provides a C-extension for Python to the DM functions 
 *   related to the dmBlock object.
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */
#include "Python.h"

#include <ctype.h>

/* ---------------------------------------------------------------------- */
/*                     *** dmBlockType datatype ***                       */
/* ---------------------------------------------------------------------- */
static PyTypeObject dmBlockType_Type = 
{
  PyObject_HEAD_INIT(NULL)
  0,                                    /*ob_size*/
  (char*)"dmBlockType",                 /*tp_name*/
  sizeof(pyBlockType),                  /*tp_basicsize*/
  0,                                    /*tp_itemsize*/
  (destructor)pyBlockType_dealloc,      /*tp_dealloc*/
  0,                                    /*tp_print*/
  0,                                    /*tp_getattr*/
  0,                                    /*tp_setattr*/
  0,                                    /*tp_compare*/
  pyBlockType_repr,                     /*tp_repr*/
  0,                                    /*tp_as_number*/
  0,                                    /*tp_as_sequence*/
  0,                                    /*tp_as_mapping*/
  0,                                    /*tp_hash */
  0,                                    /*tp_call*/
  0,                                    /*tp_str*/
  0,                                    /*tp_getattro*/
  0,                                    /*tp_setattro*/
  0,                                    /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,                   /*tp_flags*/
  "dmBlockType object",                 /* tp_doc */
  0,		                        /* tp_traverse */
  0,		                        /* tp_clear */
  pyBlockType_rcmp,                     /* tp_richcompare */
  0,		                        /* tp_weaklistoffset */
  0,		                        /* tp_iter */
  0,		                        /* tp_iternext */
  0,                                    /* tp_methods */
  0,                                    /* tp_members */
  0,                                    /* tp_getset */
  0,                                    /* tp_base */
  0,                                    /* tp_dict */
  0,                                    /* tp_descr_get */
  0,                                    /* tp_descr_set */
  0,                                    /* tp_dictoffset */
  pyBlockType_init,                     /* tp_init */
  0,                                    /* tp_alloc */
  pyBlockType_new,                      /* tp_new */

};

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlockType_new                                              */
/* ---------------------------------------------------------------------- */
static PyObject* pyBlockType_new( pyBlockType* self, PyObject *args )
{

  self = (pyBlockType*) pyBlockType_alloc();

  return (PyObject*)self;
}

static PyObject* pyBlockType_alloc( void )
{
  pyBlockType* res = NULL;

  res = PyObject_New( pyBlockType, &dmBlockType_Type);
  res->typenum = dmUNKBLOCK;

  return (PyObject*)res;
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlockType_init                                             */
/* ---------------------------------------------------------------------- */
static int pyBlockType_init( pyBlockType* self, PyObject *args )
{
  PyObject* obj1;
  long intype;
  int retval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O",  &obj1)){ return -1;}

  if ( PyObject_TypeCheck( obj1, &dmBlockType_Type) )
  {
    intype = ((pyBlockType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    intype = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmBlockType|PyInt_Type)" );
      return -1;
  }

  retval =  pyBlockType_set( self, intype );

  return retval;
}

static int pyBlockType_set( pyBlockType* self, long blocktype )
{
  int retval = 0;

  switch ( blocktype ){
  case dmUNKBLOCK:
    self->typenum = dmUNKBLOCK;
    break;
  case dmTABLE:
    self->typenum = dmTABLE;
    break;
  case dmIMAGE:
    self->typenum = dmIMAGE;
    break;
  default:
      PyErr_SetString(PyExc_ValueError, "bad value; must be dmUNKBLOCK, dmTABLE, or dmIMAGE" );
      retval = -1 ;
      break;
  }
  return( retval );
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlockType_dealloc                                          */
/* ---------------------------------------------------------------------- */
static void pyBlockType_dealloc( pyBlockType* self )
{
  if ( cxcdm_verbose_level > 4 )
    printf("  VERBOSE: pyBlockType_dealloc() called\n");

  /* free the python object */
  PyObject_Del(self);
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlockType_repr                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyBlockType_repr( pyBlockType* self )
{
  char buf[15];
  if ( self->typenum == dmTABLE )
    sprintf(buf,"dmTABLE");
  else if ( self->typenum == dmIMAGE )
    sprintf(buf,"dmIMAGE");
  else
    sprintf(buf,"dmUNKBLOCK");

  return (PyObject*)PyString_FromString(buf);
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlockType_rcmp                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyBlockType_rcmp( PyObject* obj1, PyObject* obj2, int op )
{
  PyObject*  ret = NULL;
  Bool       res = FALSE;

  long a = -1;
  long b = -1;

  if ( PyObject_TypeCheck( obj1, &dmBlockType_Type) )
  {
    a = ((pyBlockType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    a = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmBlockType|PyInt_Type)" );
      return NULL;
  }

  if ( PyObject_TypeCheck( obj2, &dmBlockType_Type) )
  {
    b = ((pyBlockType*)obj2)->typenum;
  }
  else if ( PyInt_Check( obj2 ) )
  {
    b = PyInt_AsLong( obj2 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmBlockType|PyInt_Type)" );
      return NULL;
  }

  switch ( op ){
  case Py_LT:
    res = ( a < b );
    break;
  case Py_LE:
    res = ( a <= b );
    break;
  case Py_EQ:
    res = ( a == b );
    break;
  case Py_NE:
    res = ( a != b );
    break;
  case Py_GT:
    res = ( a > b );
    break;
  case Py_GE:
    res = ( a >= b );
    break;
  default:
    res = FALSE;
    break;
  }

  if ( res == TRUE )
    Py_RETURN_TRUE;
  else
    Py_RETURN_FALSE;
}

/* ****************************************************************************** */
/* ****************************************************************************** */

/* ---------------------------------------------------------------------- */
/*                       *** dmBlock datatype ***                         */
/* ---------------------------------------------------------------------- */
static PyTypeObject dmBlock_Type = 
{
  PyObject_HEAD_INIT(NULL)
  0,                                    /*ob_size*/
  (char*)"dmBlock",                     /*tp_name*/
  sizeof(pyBlock),                      /*tp_basicsize*/
  0,                                    /*tp_itemsize*/
  (destructor)pyBlock_dealloc,          /*tp_dealloc*/
  0,                                    /*tp_print*/
  0,                                    /*tp_getattr*/
  0,                                    /*tp_setattr*/
  0,                                    /*tp_compare*/
  0,                                    /*tp_repr*/
  0,                                    /*tp_as_number*/
  0,                                    /*tp_as_sequence*/
  0,                                    /*tp_as_mapping*/
  0,                                    /*tp_hash */
  0,                                    /*tp_call*/
  0,                                    /*tp_str*/
  0,                                    /*tp_getattro*/
  0,                                    /*tp_setattro*/
  0,                                    /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,                   /*tp_flags*/
  "dmBlock object",                     /* tp_doc */
  0,		                        /* tp_traverse */
  0,		                        /* tp_clear */
  0,		                        /* tp_richcompare */
  0,		                        /* tp_weaklistoffset */
  0,		                        /* tp_iter */
  0,		                        /* tp_iternext */
  block_methods,                        /* tp_methods */
  0,                                    /* tp_members */
  0,                                    /* tp_getset */
  0,                                    /* tp_base */
  0,                                    /* tp_dict */
  0,                                    /* tp_descr_get */
  0,                                    /* tp_descr_set */
  0,                                    /* tp_dictoffset */
  0,                                    /* tp_init */
  0,                                    /* tp_alloc */
  pyBlock_new,                          /* tp_new */

};

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlock_new                                                  */
/* ---------------------------------------------------------------------- */
static PyObject* pyBlock_new()
{
  pyBlock* self = PyObject_New( pyBlock, &dmBlock_Type);
  self->block = NULL;

  return (PyObject*)self;
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyBlock_dealloc                                              */
/* ---------------------------------------------------------------------- */
static void pyBlock_dealloc( pyBlock* self )
{
  CMMObject* node = NULL;
  dmBlock*   block = self->block;

  if ( cxcdm_verbose_level > 1 )
    printf("  VERBOSE: pyBlock_dealloc() called\n");

  /* ****   CXCDM MEMORY SEGMENT - BEGIN **** */
  node = cxcdm_memory_find( block );
  if (node)
  {
    cxcdm_memory_release_holder( node, (PyObject*)self );
  }
  if ( cxcdm_verbose_level > 0 )
    cxcdm_memory_print();
  /* ****   CXCDM MEMORY SEGMENT - END   **** */

  /* free the python object */
  PyObject_Del(self);
}


/* ---------------------------------------------------------------------- */
/*                      *** dmBlock functions ***                         */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockClose                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockClose(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockClose";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockClose", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmBlockClose( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }

    /* Remove this object from memory management */
    cxcdm_memory_delete( ((pyBlock*)obj1)->block );
  }

  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCreateGenCopy                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCreateGenCopy(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmBlockCreateGenCopy";

  PyObject* obj1   = NULL;
  PyObject* obj3   = NULL;
  PyObject* obj5   = NULL;
  PyObject* obj6   = NULL;
  PyObject* result  = NULL;

  char* name = NULL;
  char* hist = NULL;
  dmBool copydata = dmFALSE;
  dmBool copysub  = dmFALSE;

  dmBlock* _block = NULL;

  static char *kwlist[] =
    {"ds", "name", "block", "histstr", "copydata", "copysubspace", NULL};

  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!sO!|sO!O!:dmBlockCreateGenCopy", kwlist, 
				   &dmDataset_Type, &obj1, 
				   &name,
				   &dmBlock_Type, &obj3,
				   &hist,
				   &PyBool_Type, &obj5,
				   &PyBool_Type, &obj6 )){ return NULL;}

  /* Check for empty input structures */
  if ( ((pyDataset*)obj1)->ds == NULL )
  {
    sprintf( errmsg, DMDS_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else  if ( ((pyBlock*)obj3)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Process copydata flag for boolean value */
  if ( obj5 &&  PyObject_IsTrue(obj5) ){ copydata = dmTRUE; }

  /* Process copysubspace flag for boolean value */
  if ( obj6 &&  PyObject_IsTrue(obj6) ){ copysub = dmTRUE; }

  _block = dmBlockCreateGenCopy( ((pyDataset*)obj1)->ds,
				 name,
				 ((pyBlock*)obj3)->block,
				 hist,
				 copydata, copysub );
  if ( _block == NULL )
  {
    sprintf(errmsg, DMBLOCK_OPEN_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_IOError, errmsg );
    return NULL;
  }

  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCreateSubspaceCpt                                     */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCreateSubspaceCpt(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCreateSubspaceCpt";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockCreateSubspaceCpt", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmBlockCreateSubspaceCpt( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCreateEmptySubspaceCpt                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCreateEmptySubspaceCpt(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCreateEmptySubspaceCpt";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockCreateEmptySubspaceCpt", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmBlockCreateEmptySubspaceCpt( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockDelete                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockDelete(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockDelete";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockDelete", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmBlockDelete( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }

    /* Remove this object from memory management */
    cxcdm_memory_delete( ((pyBlock*)obj1)->block );
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetDataset                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetDataset(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetDataset";

  PyObject*  obj1   = NULL;
  PyObject* result = NULL;

  dmDataset* _ds = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetDataset", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    _ds = dmBlockGetDataset( ((pyBlock*)obj1)->block );
    if ( _ds == NULL )
    {
      sprintf(errmsg, DMBLOCK_FUNC_ERR_MSG ,funcName);
      PyErr_SetString(PyExc_RuntimeError, errmsg );
      return NULL;
    }
  }

  result = pyDataset_new();
  ((pyDataset*)result)->ds = _ds;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetName                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetName(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetName";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  dmErrCode status;

  char blkname[dmMAXSTR+1];
  long maxlen = dmMAXSTR;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetName", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    status = dmBlockGetName( ((pyBlock*)obj1)->block, blkname, maxlen);
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( blkname );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetNo                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetNo(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetNo";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long _blkno;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetNo", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _blkno = dmBlockGetNo( ((pyBlock*)obj1)->block );
    result = Py_BuildValue("l", _blkno );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetNoKeys                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetNoKeys(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetNoKeys";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long _nokeys;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetNoKeys", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _nokeys = dmBlockGetNoKeys( ((pyBlock*)obj1)->block );
    result = Py_BuildValue("l", _nokeys );
  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockSetSubspaceCpt                                        */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockSetSubspaceCpt(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockSetSubspaceCpt";

  PyObject* obj1   = NULL;
  long      cptno  = 0;

  dmErrCode status;

  /* Parse and verify input arguments */
  
  if (!PyArg_ParseTuple(args, "O!l:dmBlockSetSubspaceCpt", &dmBlock_Type, &obj1, &cptno)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    status = dmBlockSetSubspaceCpt( ((pyBlock*)obj1)->block, cptno  );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetNoSubspaceCpts                                     */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetNoSubspaceCpts(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetNoSubspaceCpts";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long lval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetNoSubspaceCpts", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    lval = dmBlockGetNoSubspaceCpts( ((pyBlock*)obj1)->block );
  }

  result = PyLong_FromLong( lval );
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetNoSubspaceCols                                     */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetNoSubspaceCols(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetNoSubspaceCols";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long lval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetNoSubspaceCols", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    lval = dmBlockGetNoSubspaceCols( ((pyBlock*)obj1)->block );
  }

  result = PyLong_FromLong( lval );
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetType                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetType(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetType";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  dmBlockType _res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetType", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _res = dmBlockGetType( ((pyBlock*)obj1)->block );

    result = pyBlockType_alloc();
    if ( pyBlockType_set( (pyBlockType*)result, (long)_res ) < 0 ) 
    {
      /* invalid type.. clear result and return error */
      Py_DECREF(result);
      return NULL;
    }
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmpBlockOpenMode                                             */
/*    This binding serves as the foundation for the various block open    */
/*    functions.  It is essentially a wrapper for dmp_block_open_mode().  */
/* ---------------------------------------------------------------------- */
static PyObject* _dmpBlockOpenMode(PyObject *self, PyObject *args )
{
  char* funcName = "_dmpBlockOpenMode";

  PyObject* result;

  char*      name   = NULL;
  dmBlock*   _block = NULL;
  dmDataset* _ds    = NULL;
  dmBlockType type  = dmUNKNOWNBLOCK;

  long     blkno  = 0;
  char*    mode;

  PyObject* obj1  = NULL;
  PyObject* obj2  = NULL;

  long errcd;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "s|OO!ss:_dmpBlockOpenMode", &name,
				   &obj1, 
			           &dmBlockType_Type, &obj2,
			           &mode,
			           &funcName)){ return NULL;}


  /* Process input dataset argument */
  if ( PyObject_TypeCheck( obj1, &dmDataset_Type) )
  {
    /* extract dataset ptr if provided */
    _ds = ((pyDataset*)obj1)->ds;

    if ( _ds == NULL )
    {
      sprintf(errmsg, DMDS_EMPTY_PTR_MSG ,funcName);
      PyErr_SetString(PyExc_TypeError, errmsg );
      return NULL;
    }
  }
  else if ( obj1 != Py_None )
  {
    sprintf(errmsg, DMBLOCK_INVALID_ARGTYP_MSG ,funcName, 2L, "dmDataset or None");
    PyErr_SetString(PyExc_TypeError, errmsg );
    return NULL;
  }

  /* Process input block type argument */
  type = ((pyBlockType*)obj2)->typenum;

  /* Process mode argument */
  if ( (strcmp(mode, "RW") != 0 ) &&
       (strcmp(mode, "R" ) != 0 ) &&
       (strcmp(mode, "W" ) != 0 )   )
  {
    sprintf(errmsg, DMBLOCK_INVALID_ARGVAL_MSG ,funcName);
    PyErr_SetString(PyExc_ValueError, errmsg );
    return NULL;
  }

  _block = dmp_block_open_mode( _ds, name, blkno, type, NULL, mode );
  if ( _block == NULL )
  {
    errcd  = dmGetError();
    if ( errcd == DM_ERR_NO_FILE )
    {
      sprintf(errmsg, DMBLOCK_NOFILE_ERR_MSG ,funcName, name);
      dmResetErrorState();
    }
    else
    {
      sprintf(errmsg, DMBLOCK_OPEN_ERR_MSG ,funcName, name);
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );
    }
    PyErr_SetString(PyExc_IOError, errmsg );
    return NULL;
  }

  dmBlockType btype = dmBlockGetType( _block );
  
  if ( btype == dmIMAGE )
  {
    /* Verify image is viable ( shields users from issues resulting in bad images ) */
    dmDescriptor* _dd = dmImageGetDataDescriptor( _block );
    if ( dmGetArraySize( _dd ) <  0 )
    {
      dmBlockClose( _block );

      char tmpstr[1024];
      sprintf(tmpstr, "failed to open image, resulting image size < 0. \'%s\'", name );
      sprintf(errmsg, DMBLOCK_GENERIC_MSG, funcName, tmpstr);
      PyErr_SetString(PyExc_IOError, errmsg );
      return NULL;
    }
  }

  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return (PyObject*)result;
}


/* ---------------------------------------------------------------------- */
/*                      *** dmTable functions ***                         */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableClose                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableClose(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableClose";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableClose", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    /* This function also closes dataset.. clean up from that level */
    dmDataset* _ds = dmBlockGetDataset( ((pyBlock*)obj1)->block );

    status = dmTableClose( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }

    /* Remove this object from memory management */
    cxcdm_memory_delete( (void*)_ds );
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableGetNoCols                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableGetNoCols(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableGetNoCols";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long _ncols = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableNoCols", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _ncols = dmTableGetNoCols( ((pyBlock*)obj1)->block );
    result = Py_BuildValue("l", _ncols );
  }

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableGetNoRows                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableGetNoRows(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableGetNoRows";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long _nrows = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableGetNoRows", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _nrows = dmTableGetNoRows( ((pyBlock*)obj1)->block );
    result = Py_BuildValue("l", _nrows );
  }

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableGetRowNo                                              */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableGetRowNo(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableGetRowNo";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long _rownum = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableGetRowNo", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _rownum = dmTableGetRowNo( ((pyBlock*)obj1)->block );
    result = Py_BuildValue("l", _rownum );
  }

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableNextRow                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableNextRow(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableNextRow";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long _rownum = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableNextRow", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _rownum = dmTableNextRow( ((pyBlock*)obj1)->block );
    if ( _rownum == dmNOMOREROWS )
    {
      Py_RETURN_NONE;
    }
    else if ( _rownum == dmBADROW )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
    }
    else
    {
      result = Py_BuildValue("l", _rownum );
    }
  }

  return( result );
}


/* ---------------------------------------------------------------------- */
/*                      *** dmImage functions ***                         */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmImageClose                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmImageClose(PyObject *self, PyObject *args )
{
  char* funcName = "dmImageClose";

  PyObject* obj1   = NULL;

  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmImageClose", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    /* This function also closes dataset.. clean up from that level */
    dmDataset* _ds = dmBlockGetDataset( ((pyBlock*)obj1)->block );

    status = dmImageClose( ((pyBlock*)obj1)->block );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }

    /* Remove this object from memory management */
    cxcdm_memory_delete( (void*)_ds );
  }
  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDatasetCreateImage                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDatasetCreateImage(PyObject *self, PyObject *args )
{

  char* funcName = "dmDatasetCreateImage";

  PyObject*  obj1    = NULL;
  PyObject*  obj4    = NULL;
  PyObject*  result  = NULL;

  char*      name   = NULL;
  dmDataType dtype  = dmUNKNOWNTYPE;;
  long*      axes   = NULL;
  long       naxes  = 0;
  dmDataset* _ds    = NULL;
  dmBlock*  _block  = NULL;

  PyArray_Descr* descr = NULL;
  PyArrayObject* tmparr = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "OsO&O!:dmDatasetCreateImage", 
			&obj1,
			&name,
			&PyArray_DescrConverter2, &descr,
			&PyArray_Type, &obj4)){ return NULL;}


  /* Process input dataset argument */
  if ( PyObject_TypeCheck( obj1, &dmDataset_Type) )
  {
    /* extract dataset ptr if provided */
    _ds = ((pyDataset*)obj1)->ds;
    
    /* extract dataset ptr if provided */
    if ( _ds == NULL )
    {
      sprintf(errmsg, DMDS_EMPTY_PTR_MSG ,funcName);
      PyErr_SetString(PyExc_TypeError, errmsg );
      return NULL;
    }
  }
  else if ( obj1 != Py_None )
  {
    sprintf(errmsg, DMBLOCK_INVALID_ARGTYP_MSG ,funcName, 2L, "dmDataset or None");
    PyErr_SetString(PyExc_TypeError, errmsg );
    return NULL;
  }

  /* Process dtype argument */
  if ( descr != NULL )
  {
    /* validate input dtype */
    if ( !( PyDataType_ISNUMBER(descr)) )
    {
      /* problem */
      sprintf(errmsg, DMBLOCK_BAD_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    /* convert to DM datatype */
    dtype = cxc_get_dm_type( descr->type_num );
    if ( dtype == dmUNKNOWNTYPE )
    {
      /* problem */
      sprintf(errmsg, DMBLOCK_UNK_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }
  }

  naxes = (long)PyArray_SIZE((PyObject*)obj4);
  /* Make sure the arrays are of correct type */
  if (PyArray_CanCastSafely( ((PyArrayObject*)obj4)->descr->type_num, NPY_LONG))
  {
    long ii;
    long *tmp_axes;
    tmparr = (PyArrayObject *)PyArray_Cast( (PyArrayObject*)obj4, NPY_LONG);
    tmp_axes   = (long*) PyArray_DATA(tmparr);

    /* reverse the axes so that they are fortran style */
    axes = (long *)malloc (naxes * sizeof(long));
    for (ii=naxes; ii--;)
      axes[ii] = tmp_axes[naxes-ii-1];

  }
  else
  {
    sprintf(errmsg, DMBLOCK_UNK_DTYPE_MSG ,funcName, ((PyArrayObject*)obj4)->descr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  _block = dmDatasetCreateImage( _ds, name, dtype, axes, naxes );
  if ( _block == NULL )
  {
    sprintf(errmsg, DMBLOCK_OPEN_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_IOError, errmsg );
    goto fail;
  }

  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  free (axes);

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;

 fail:
  if ( axes )
  {
    free (axes); 
    Py_XDECREF( tmparr );
  }
  return NULL;

}



/* ---------------------------------------------------------------------- */
/* Binding: _dmDatasetGetBlock                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDatasetGetBlock(PyObject *self, PyObject *args )
{
  char* funcName = "dmDatasetGetBlock";

  PyObject* obj1   = NULL;
  PyObject* obj2   = NULL;
  PyObject* result = NULL;

  long ii;
  long blockNo     = -1;
  char* blockName  = NULL;
  long nvblocks; 

  dmDataset*  _ds  = NULL;
  dmBlock* _block  = NULL;
  dmBlock* tmp     = NULL;


  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O:dmDatasetGetBlock", &dmDataset_Type, &obj1, &obj2 )){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDataset*)obj1)->ds == NULL )
  {
    sprintf( errmsg, DMDS_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  
  /* Validate second input, and execute */
  if ( PyInt_Check( obj2 ) )
    blockNo =  PyInt_AsLong( obj2 );
  else if ( PyString_Check( obj2 ) )
  { 
    blockName = PyString_AsString( obj2 );
  }
  else
  {
    /* problem */
    sprintf(errmsg, DMBLOCK_INVALID_ARGTYP_MSG, funcName, 2L, (char*)"int or string");
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }

  _ds = ((pyDataset*)obj1)->ds;
  nvblocks = dmi_dataset_get_nv( _ds );

  if (blockName)
  {
    /* Find which block has the input name */
    blockNo = dmi_block_search( _ds, blockName );
    if (blockNo < 1 )
    {
      sprintf(errmsg, DMBLOCK_UNK_NAME_MSG, funcName, blockName);
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
  }

  /* check for ptr to already open block */
  for ( ii = 1 ; ii <= nvblocks; ii++ )
  {
    tmp = dmi_dataset_get_block( _ds, ii );
    if ( tmp && (dmBlockGetNo( tmp ) == blockNo))
    {
      _block = tmp;
      dmi_dataset_set_current( _ds, _block );
      break;
    }
  }
  if ( _block == NULL )
  {
    /* no ptr to already open block.. attempt to open. */
    _block = dmDatasetMoveToBlock( _ds, blockNo );
  }
  if ( _block == NULL )
  {
    if (blockName)
       sprintf(errmsg, DMBLOCK_MOVE_STR_ERR_MSG ,funcName, blockName);
    else
       sprintf(errmsg, DMBLOCK_MOVE_ERR_MSG ,funcName, blockNo);
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }
      
  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;

 fail:
  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDatasetMoveToBlock                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDatasetMoveToBlock(PyObject *self, PyObject *args )
{
  char* funcName = "dmDatasetMoveToBlock";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long blockNo;
  dmBlock* _block  = NULL;


  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmDatasetMoveToBlock", &dmDataset_Type, &obj1, &blockNo )){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDataset*)obj1)->ds == NULL )
  {
    sprintf( errmsg, DMDS_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _block = dmDatasetMoveToBlock( ((pyDataset*)obj1)->ds, blockNo );
    if ( _block == NULL )
    {
      sprintf(errmsg, DMBLOCK_MOVE_ERR_MSG ,funcName, blockNo);
      PyErr_SetString(PyExc_IOError, errmsg );
      return NULL;
    }
      
    result = pyBlock_new();
    ((pyBlock*)result)->block = _block;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockReadComment                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockReadComment(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockReadComment";

  PyObject*     obj1 = NULL;
  PyObject*  result  = NULL;
  char *tag;
  char *cmt;

  if (!PyArg_ParseTuple(args, "O!:dmBlockReadComment", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }

  if (dmBlockReadComment(((pyBlock*)obj1)->block, &tag, &cmt) == dmFALSE)
  {
     result = Py_BuildValue("ss",NULL,NULL);
  }
  else
  {
     result = Py_BuildValue("OO", PyString_FromString(tag), PyString_FromString(cmt));
     free(tag);
     free(cmt);
  }
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockWriteComment                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockWriteComment(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockWriteComment";

  PyObject*     obj1 = NULL;
  char tmptag[dmMAXSTR];
  char *tag;
  char *cmt;
  int done = 0;

  if (!PyArg_ParseTuple(args, "O!ss:dmBlockWriteComment", &dmBlock_Type, &obj1, 
                                   &tag,
                                   &cmt)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL; 
  }

  /* verify that we have a COMMENT or HISTORY tag */
  tmptag[0] = '\0';
  if (tag && (strlen(tag) < dmMAXSTR))
  {
    int long ii, jj = 0; 
    for (ii = 0; ii < strlen(tag) && !done; ii++)
      if (isalpha(tag[ii]))
        tmptag[jj++] = toupper(tag[ii]);
      else done = 1;  
    if (!done)  tmptag[jj] = '\0';
  }
  if ((strcmp(tmptag, dmHISTORY) != 0) && (strcmp(tmptag, dmCOMMENT) != 0))
  {
    sprintf(errmsg, DMBLOCK_INVALID_ARGTYP_MSG ,funcName, 2L, "HISTORY or COMMENT");
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL; 
  }

  if (dmBlockWriteComment(((pyBlock*)obj1)->block, tag, cmt) != dmSUCCESS) 
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_IOError, errmsg);
    return NULL;
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockFlush                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockFlush(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockFlush";

  PyObject* obj1   = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockFlush", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  dmBlockFlush( ((pyBlock*)obj1)->block );
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockReadComment                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockListKeys(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockListKeys";

  PyObject*     obj1 = NULL;
  PyObject*  result  = NULL;
  char *tag, *cmt, *buf = NULL; 
  long buflen = 10000; 
  char kbuf[dmMAXSTR+1];
  long ii, nkeys, ln, ndx = 0; 
  dmDescriptor*  kdesc = NULL;

  if (!PyArg_ParseTuple(args, "O!:dmBlockListKeys", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }

  /* allocate the text buffer- we'll track and increase size if needed */ 
  if ((buf = (char*) malloc((buflen + 2)* sizeof(char))) == NULL)
  {
    sprintf(errmsg, "%s() problem allocating memory for %s.", funcName, "header list");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    return NULL;
  }
  *buf = '\0';

  /* loop through keys */
  nkeys = dmBlockGetNoKeys( ((pyBlock*)obj1)->block );
  for (ii = 1; ii <= nkeys; ii++)
  {
    if ((kdesc = dmBlockMoveToKeyNo(((pyBlock*)obj1)->block, ii)) != NULL)
    {
      /* append current key to return string */ 
      dmi_key_print( kdesc, kbuf );
      ln = strlen(kbuf);
      if (ndx + ln > buflen)  /* need to allocate more memory */
      {
        char* tmpbuf = (char*) malloc((2*buflen + 2)* sizeof(char));
        if (tmpbuf == NULL)
        {
          sprintf(errmsg, "%s() problem allocating memory for %s.", funcName, "header list");
          PyErr_SetString(PyExc_MemoryError, errmsg );
          return NULL;
        }
        strcpy(tmpbuf, buf);
        free(buf);
        buf = tmpbuf;
        buflen *= 2;
      } 

      if (ndx + ln < buflen) /* copy key data to buffer */ 
      {
        strcpy(&((char*)buf)[ndx], kbuf);
        ndx += ln;
        ((char*)buf)[ndx++] = '\n'; /* add newline after each key */ 
        ((char*)buf)[ndx] = '\0';  
      } 

      /* append comment/history associated w/ current key to return string */ 
      while( dmBlockReadComment( ((pyBlock*)obj1)->block, &tag, &cmt ) )
      {
        if (cmt)
        {
          /* format like dmBlockListKeys */
          sprintf(kbuf, "    %-20s %s\n", tag, cmt);
          ln = strlen(kbuf);

          if (ndx + ln > buflen)  /* need to allocate more memory */
          {
            char* tmpbuf = (char*) malloc((2*buflen + 2)* sizeof(char));
            if (tmpbuf == NULL)
            {
              sprintf(errmsg, "%s() problem allocating memory for %s.", funcName, "header list");
              PyErr_SetString(PyExc_MemoryError, errmsg );
              return NULL;
            }
            strcpy(tmpbuf, buf);
            free(buf);
            buf = tmpbuf;
            buflen *= 2;
          }
          if (ndx + ln < buflen)  /* copy comment/data to buffer */
          {
            strcpy(&((char*)buf)[ndx], kbuf);
            ndx += ln;
          }
        }
        free( tag );
        free( cmt );
      }
    }  
  } 
  result = Py_BuildValue("O", PyString_FromString(buf));

  free (buf);
  buf = NULL; 
  
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockSetBufferMaxSize                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockSetBufferMaxSize(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockSetBufferMaxSize";
  PyObject* obj1   = NULL;
  long bufSize;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmBlockSetBufferMaxSize", &dmBlock_Type, &obj1, &bufSize )){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  
  dmBlockSetBufferMaxSize( ((pyBlock*)obj1)->block, bufSize );
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableSetRow                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableSetRow(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableSetRow";
  PyObject* obj1   = NULL;
  PyObject* result = NULL;
  long rowNo, retNo;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmTableSetRow", &dmBlock_Type, &obj1, &rowNo )){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  
  retNo = dmTableSetRow( ((pyBlock*)obj1)->block, rowNo );
  result = PyLong_FromLong( retNo );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCopy                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCopy(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCopy";
  PyObject *obj1 = NULL, *obj2 = NULL;
  char* opt;
  char opt_u[16]; 


  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!s:dmBlockCopy", &dmBlock_Type, &obj1, &dmBlock_Type, &obj2, &opt)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL || ((pyBlock*)obj2)->block == NULL)
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  /* Check option is either HEADER|SUBSPACE */
  if (opt)
  {
    int rr, pos = strlen(opt); 
    if (pos > 15) pos = 15;
    opt_u[pos] = '\0';
    /* convert option to uppercase for validation */
    while(pos--)
      opt_u[pos] = toupper(opt[pos]);
    if (strcmp(opt_u, "HEADER") && strcmp(opt_u, "SUBSPACE")) 
    {
      sprintf(errmsg, DMBLOCK_INVALID_ARGTYP_MSG, funcName, 3, "HEADER|SUBSPACE");
      PyErr_SetString(PyExc_ValueError, errmsg);
      return NULL;
    } 
  }

  dmBlockCopy( ((pyBlock*)obj1)->block, ((pyBlock*)obj2)->block, opt_u);
  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCopyCol                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCopyCol(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCopyCol";
  PyObject *obj1 = NULL, *obj2 = NULL, *result = NULL;
  dmDescriptor* desc = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!:dmBlockCopyCol", &dmBlock_Type, &obj1, &dmDescriptor_Type, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj2)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  if ((desc = dmBlockCopyCol( ((pyBlock*)obj1)->block, ((pyDescriptor*)obj2)->dd)) == NULL)
  {
    PyErr_SetString(PyExc_RuntimeError, "Copy of descriptor to block failed" );
    return NULL;
  }
  else
  {
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = desc;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCopyData                                              */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCopyData(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCopyData";
  PyObject *obj1 = NULL, *obj2 = NULL, *result = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!:dmBlockCopyData", &dmBlock_Type, &obj1, &dmBlock_Type, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL || ((pyBlock*)obj2)->block == NULL)
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dmBlockCopyData(((pyBlock*)obj1)->block, ((pyBlock*)obj2)->block);
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCopyWCS                                              */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCopyWCS(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockCopyWCS";
  PyObject *obj1 = NULL, *obj2 = NULL, *result = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!:dmBlockCopyWCS", &dmBlock_Type, &obj1, &dmBlock_Type, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL || ((pyBlock*)obj2)->block == NULL)
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dmBlockCopyWCS(((pyBlock*)obj1)->block, ((pyBlock*)obj2)->block);
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockCreate                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockCreate(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmBlockCreate";

  PyObject *obj1 = NULL, *obj3   = NULL;
  char *name; 
  PyObject* result  = NULL;

  dmBlock* _block = NULL;

  static char *kwlist[] =
    {"ds", "name", "type", NULL};

  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!sO!:dmBlockCreate", kwlist, 
				   &dmDataset_Type, &obj1, 
				   &name,
				   &dmBlockType_Type, &obj3)){ return NULL;}

  /* Check for empty input structures */
  if ( ((pyDataset*)obj1)->ds == NULL )
  {
    sprintf( errmsg, DMDS_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  _block = dmBlockCreate( ((pyDataset*)obj1)->ds, name, ((pyBlockType*)obj3)->typenum);

  if ( _block == NULL )
  {
    sprintf(errmsg, DMBLOCK_OPEN_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_IOError, errmsg );
    return NULL;
  }

  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetCurrSubspaceCpt                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetCurrSubspaceCpt(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetCurrSubspaceCpt";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  long lval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetCurrSubspaceCpt", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    lval = dmBlockGetCurrSubspaceCpt( ((pyBlock*)obj1)->block );
  }

  result = PyLong_FromLong( lval );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableReadRegion(PyObject *self, PyObject *args )           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableReadRegion(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableReadRegion";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;
  regRegion* reg   = NULL;

  long lval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmTableReadRegion", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    if ((reg = dmTableReadRegion( ((pyBlock*)obj1)->block )) != NULL )
    {
      result = pyRegion_New( reg );
    }
  }
  if ( result == NULL )
  {
    Py_RETURN_NONE;
  }
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmTableWriteRegion                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableWriteRegion(PyObject *self, PyObject *args )
{
  char *funcName  = "dmTableWriteRegion", *name = NULL;
  PyObject *obj1  = NULL, *obj2 = NULL, *obj3 = NULL, *result = NULL;
  dmDescriptor* _dd = NULL;
  regRegion* _reg = NULL;
  dmBlock* _block = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!sO!O!:dmTableWriteRegion", &dmDataset_Type, &obj1, &name, &dmDescriptor_Type, &
obj2, &Region_Type, &obj3))
    {return NULL;}

  _dd  = ((pyDescriptor*)obj2)->dd;

  /* Check underlying dmDataset pointer is available */
  if ( ((pyDataset*)obj1)->ds == NULL )
  {
    sprintf( errmsg, "%s() operation on empty dmDataset pointer.", funcName );
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    return NULL;
  }
  /* Check underlying Region pointer is available */
  if ( (_reg = ((pyRegion*)obj3)->reg) == NULL )
  {
    sprintf( errmsg, "%s() operation on empty Region pointer.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  /* check name */
  if ((name && strlen(name) == 0) || (name == NULL))
  {
    sprintf( errmsg, "%s() Output region block name must be provided.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Set region to descriptor */
  if ((_block = dmTableWriteRegion(((pyDataset*)obj1)->ds, name, _dd, _reg)) == NULL)
  {
    sprintf(errmsg, DMBLOCK_OPEN_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_IOError, errmsg );
    return NULL;
  }

  result = pyBlock_new();
  ((pyBlock*)result)->block = _block;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockMergeSubspace                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockMergeSubspace(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockMergeSubspace";

  PyObject* obj1   = NULL;
  PyObject* obj2   = NULL;
  PyObject* obj3   = NULL;
  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!O!:dmBlockMergeSubspace", &dmBlock_Type, &obj1, &dmBlock_Type, &obj2,  &dmBlock_Type, &obj3)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL || ((pyBlock*)obj1)->block == NULL ||
       ((pyBlock*)obj1)->block == NULL)
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    if (dmBlockMergeSubspace( ((pyBlock*)obj1)->block, ((pyBlock*)obj2)->block,
        ((pyBlock*)obj3)->block ) != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockIntersectSubspace                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockIntersectSubspace(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockIntersectSubspace";

  PyObject* obj1   = NULL;
  PyObject* obj2   = NULL;
  PyObject* obj3   = NULL;
  dmErrCode status = dmFAILURE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!O!:dmBlockIntersectSubspace", &dmBlock_Type, &obj1, &dmBlock_Type, &obj2,  &dmBlock_Type, &obj3)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL || ((pyBlock*)obj1)->block == NULL ||
       ((pyBlock*)obj1)->block == NULL)
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    if (dmBlockIntersectSubspace(((pyBlock*)obj1)->block,((pyBlock*)obj2)->block,
        ((pyBlock*)obj3)->block ) != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_IOError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}


