/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm_block.h
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:
 *    definitions related to dmBlock wrappers.
 *    for the Python 'cxcdm' module
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */

#ifndef CXCDM_BLOCK_H
#define CXCDM_BLOCK_H


/* ****************************************************************************** */

typedef struct {
  PyObject_HEAD
  dmBlockType typenum;
} pyBlockType;

staticforward PyTypeObject dmBlockType_Type;

static PyObject*  pyBlockType_new( pyBlockType* self, PyObject *args);
static PyObject*  pyBlockType_alloc(void);
static int        pyBlockType_init( pyBlockType* self, PyObject *args);
static int        pyBlockType_set( pyBlockType* self, long blocktype );
static void       pyBlockType_dealloc( pyBlockType* );
static PyObject*  pyBlockType_repr( pyBlockType* );
static PyObject*  pyBlockType_rcmp( PyObject* obj1, PyObject* obj2, int op );

/* ****************************************************************************** */

typedef struct {
  PyObject_HEAD
  dmBlock* block;
} pyBlock;

staticforward PyTypeObject dmBlock_Type;

static PyObject*  pyBlock_new( void );
static void       pyBlock_dealloc( pyBlock* );

static PyObject*  _dmBlockClose(PyObject *self, PyObject *args);
static PyObject*  _dmBlockCreate(PyObject *self, PyObject *args, PyObject *kwds);
static PyObject*  _dmBlockCreateGenCopy(PyObject *self, PyObject *args, PyObject *kwds);
static PyObject*  _dmBlockCreateSubspaceCpt(PyObject *self, PyObject *args);
static PyObject*  _dmBlockCreateEmptySubspaceCpt(PyObject *self, PyObject *args);
static PyObject*  _dmBlockIntersectSubspace(PyObject *self, PyObject *args);
static PyObject*  _dmBlockMergeSubspace(PyObject *self, PyObject *args);
static PyObject*  _dmBlockDelete(PyObject *self, PyObject *args);
static PyObject*  _dmBlockFlush(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetCurrSubspaceCpt(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetDataset(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetName(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetNo(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetNoKeys(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetNoSubspaceCpts(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetNoSubspaceCols(PyObject *self, PyObject *args);
static PyObject*  _dmBlockGetType(PyObject *self, PyObject *args);
static PyObject*  _dmBlockListKeys(PyObject *self, PyObject *args);
static PyObject*  _dmpBlockOpenMode(PyObject *self, PyObject *args);
static PyObject*  _dmBlockSetSubspaceCpt(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetMoveToBlock(PyObject *self, PyObject *args);

static PyObject*  _dmTableClose(PyObject *self, PyObject *args);
static PyObject*  _dmTableGetNoCols(PyObject *self, PyObject *args);
static PyObject*  _dmTableGetNoRows(PyObject *self, PyObject *args);
static PyObject*  _dmTableGetRowNo(PyObject *self, PyObject *args);
static PyObject*  _dmTableNextRow(PyObject *self, PyObject *args);
static PyObject*  _dmTableSetRow(PyObject *self, PyObject *args);
static PyObject*  _dmTableReadRegion(PyObject *self, PyObject *args);
static PyObject*  _dmTableWriteRegion(PyObject *self, PyObject *args);

static PyObject*  _dmBlockReadComment(PyObject *self, PyObject *args);
static PyObject*  _dmBlockWriteComment(PyObject *self, PyObject *args);

static PyObject* _dmImageClose(PyObject *self, PyObject *args);
static PyObject* _dmDatasetCreateImage(PyObject *self, PyObject *args );
static PyObject* _dmBlockSetBufferMaxSize(PyObject *self, PyObject *args );

static PyObject*  _dmBlockCopy(PyObject *self, PyObject *args);
static PyObject*  _dmBlockCopyCol(PyObject *self, PyObject *args);
static PyObject*  _dmBlockCopyData(PyObject *self, PyObject *args);
static PyObject*  _dmBlockCopyWCS(PyObject *self, PyObject *args);

/* DM internal functions */
extern dmBlock* dmp_block_open_mode( dmDataset* ds, const char* name, const long blkno, 
				     dmBlockType btype, void* kb, const char* mode );
extern long     dmi_block_search( dmDataset*, char* name );
extern long     dmi_dataset_get_nv( dmDataset* );
extern dmBlock* dmi_dataset_get_block(  dmDataset*, long );
extern void	dmi_dataset_set_current( dmDataset*, dmBlock* );
extern void     dmi_key_print( dmDescriptor* dd, char* pbuf );

/* Error Messages */
#define DMBLOCK_GENERIC_MSG         "%s() %s"
#define DMBLOCK_FUNC_ERR_MSG        "%s() failed."
#define DMBLOCK_NOFILE_ERR_MSG      "%s() file does not exist. \'%s\'"
#define DMBLOCK_OPEN_ERR_MSG        "%s() failed to open file. \'%s\'"
#define DMBLOCK_MOVE_ERR_MSG        "%s() failed to open block %ld."
#define DMBLOCK_MOVE_STR_ERR_MSG    "%s() failed to open block \'%s\'."
#define DMBLOCK_EMPTY_PTR_MSG       "%s() operation on empty dmBlock pointer."
#define DMBLOCK_INVALID_ARGTYP_MSG  "%s() argument %ld must be %s"
#define DMBLOCK_INVALID_ARGVAL_MSG  "%s() invalid argument value."
#define DMBLOCK_BAD_DTYPE_MSG       "%s() unrecognized datatype (\'%s\'), please input numerical type."
#define DMBLOCK_UNK_DTYPE_MSG       "%s() unsupported datatype (\'%s\')"
#define DMBLOCK_UNK_NAME_MSG        "%s() failed to find block \'%s'."

/* dmBlock method structure array */
static PyMethodDef block_methods[] =
{
  {NULL, NULL, 0, NULL}     /* Sentinel - marks the end of this structure */
};


#endif
