/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm_dataset.h
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:
 *    definitions related to dmDataset wrappers.
 *    for the Python 'cxcdm' module
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */

#ifndef CXCDM_DATASET_H
#define CXCDM_DATASET_H

typedef struct {
  PyObject_HEAD
  dmDataset* ds;
} pyDataset;

staticforward PyTypeObject dmDataset_Type;

static PyObject*  pyDataset_new( void );
static void       pyDataset_dealloc( pyDataset* );

static PyObject*  _dmDatasetAccess(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetClose(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetDelete(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetDestroy(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetCreate(PyObject *self, PyObject *args);
static PyObject*  _dmDatasetGetBlockName(PyObject *self, PyObject *args );
static PyObject*  _dmDatasetGetBlockType(PyObject *self, PyObject *args );
static PyObject*  _dmDatasetGetCurrentBlockNo(PyObject *self, PyObject *args );
static PyObject*  _dmDatasetGetName(PyObject *self, PyObject *args );
static PyObject*  _dmDatasetGetNoBlocks(PyObject *self, PyObject *args );
static PyObject*  _dmDatasetOpen(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject*  _dmDatasetWriteable(PyObject *self, PyObject *args );

/* Error Messages */
#define DMDS_GENERIC_MSG      "%s() %s"
#define DMDS_FUNC_ERR_MSG     "%s() failed."
#define DMDS_EMPTY_PTR_MSG    "%s() operation on empty dmDataset pointer."
#define DMDS_OPEN_ERR_MSG     "%s() could not open file. \'%s\'"
#define DMDS_CREATE_ERR_MSG   "%s() could not create file. \'%s\'"

static PyMethodDef dataset_methods[] =
{
  {NULL, NULL, 0, NULL}     /* Sentinel - marks the end of this structure */
};/*end... */

#endif
