/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 * FILE NAME:  cxcdm_descriptor.c
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:  
 *   Provides a C-extension for Python to the DM functions 
 *   related to the dmDescriptor object.
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */
#include "Python.h"

/* ---------------------------------------------------------------------- */
/*                     *** dmDescriptor datatype ***                      */
/* ---------------------------------------------------------------------- */
static PyTypeObject dmDescriptor_Type = 
{
  PyObject_HEAD_INIT(NULL)
  0,                                    /*ob_size*/
  (char*)"dmDescriptor",                /*tp_name*/
  sizeof(pyDescriptor),                 /*tp_basicsize*/
  0,                                    /*tp_itemsize*/
  (destructor)pyDescriptor_dealloc,     /*tp_dealloc*/
  0,                                    /*tp_print*/
  0,                                    /*tp_getattr*/
  0,                                    /*tp_setattr*/
  0,                                    /*tp_compare*/
  0,                                    /*tp_repr*/
  0,                                    /*tp_as_number*/
  0,                                    /*tp_as_sequence*/
  0,                                    /*tp_as_mapping*/
  0,                                    /*tp_hash */
  0,                                    /*tp_call*/
  0,                                    /*tp_str*/
  0,                                    /*tp_getattro*/
  0,                                    /*tp_setattro*/
  0,                                    /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,                   /*tp_flags*/
  "dmDescriptor object",                /* tp_doc */
  0,                                    /* tp_traverse */
  0,                                    /* tp_clear */
  0,                                    /* tp_richcompare */
  0,                                    /* tp_weaklistoffset */
  0,                                    /* tp_iter */
  0,                                    /* tp_iternext */
  descriptor_methods,                   /* tp_methods */
  0,                                    /* tp_members */
  0,                                    /* tp_getset */
  0,                                    /* tp_base */
  0,                                    /* tp_dict */
  0,                                    /* tp_descr_get */
  0,                                    /* tp_descr_set */
  0,                                    /* tp_dictoffset */
  0,                                    /* tp_init */
  0,                                    /* tp_alloc */
  pyDescriptor_new,                     /* tp_new */

};

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptor_new                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyDescriptor_new()
{
  pyDescriptor* self = PyObject_New( pyDescriptor, &dmDescriptor_Type);
  self->dd = NULL;

  return (PyObject*)self;
}


char* descriptor_type_as_string(dmDescriptorType desctype)
{
  static char mybuf[15];
  switch (desctype){
  case dmBADDESC:
    sprintf(mybuf,"dmBADDESC");
    break;
  case dmKEY:
    sprintf(mybuf,"dmKEY");
    break;
  case dmSUBSPACE:
    sprintf(mybuf,"dmSUBSPACE");
    break;
  case dmCOORD:
    sprintf(mybuf,"dmCOORD");
    break;
  case dmIMAGEDATA:              // Fallthrough intentional
  case dmCOLUMN:
    sprintf(mybuf,"dmCOLUMN");
    break;
  default:
    sprintf(mybuf,"unknown type");
  }
  return mybuf; 
}


/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptor_dealloc                                         */
/* ---------------------------------------------------------------------- */
static void pyDescriptor_dealloc( pyDescriptor* self )
{
  CMMObject* node  = NULL;
  dmDescriptor* dd = self->dd;

  if ( cxcdm_verbose_level > 1 )
  printf("  VERBOSE: pyDescriptor_dealloc() called\n");

  /* ****   CXCDM MEMORY SEGMENT - BEGIN **** */
  node = cxcdm_memory_find( dd );
  if (node)
  {
    cxcdm_memory_release_holder( node, (PyObject*)self );
  }
  if ( cxcdm_verbose_level > 0 )
    cxcdm_memory_print();
  /* ****   CXCDM MEMORY SEGMENT - END   **** */


  PyObject_Del(self);
}


/* ---------------------------------------------------------------------- */
/*                     *** dmDescriptorType datatype ***                       */
/* ---------------------------------------------------------------------- */
static PyTypeObject dmDescriptorType_Type =
{
  PyObject_HEAD_INIT(NULL)
  0,                                    /*ob_size*/
  (char*)"dmDescriptorType",            /*tp_name*/
  sizeof(pyDescriptorType),             /*tp_basicsize*/
  0,                                    /*tp_itemsize*/
  (destructor)pyDescriptorType_dealloc, /*tp_dealloc*/
  0,                                    /*tp_print*/
  0,                                    /*tp_getattr*/
  0,                                    /*tp_setattr*/
  0,                                    /*tp_compare*/
  pyDescriptorType_repr,                /*tp_repr*/
  0,                                    /*tp_as_number*/
  0,                                    /*tp_as_sequence*/
  0,                                    /*tp_as_mapping*/
  0,                                    /*tp_hash */
  0,                                    /*tp_call*/
  0,                                    /*tp_str*/
  0,                                    /*tp_getattro*/
  0,                                    /*tp_setattro*/
  0,                                    /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,                   /*tp_flags*/
  "dmDescriptorType object",            /* tp_doc */
  0,                                    /* tp_traverse */
  0,                                    /* tp_clear */
  pyDescriptorType_rcmp,                /* tp_richcompare */
  0,                                    /* tp_weaklistoffset */
  0,                                    /* tp_iter */
  0,                                    /* tp_iternext */
  0,                                    /* tp_methods */
  0,                                    /* tp_members */
  0,                                    /* tp_getset */
  0,                                    /* tp_base */
  0,                                    /* tp_dict */
  0,                                    /* tp_descr_get */
  0,                                    /* tp_descr_set */
  0,                                    /* tp_dictoffset */
  pyDescriptorType_init,                /* tp_init */
  0,                                    /* tp_alloc */
  pyDescriptorType_new,                 /* tp_new */

};

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptorType_new                                              */
/* ---------------------------------------------------------------------- */
static PyObject* pyDescriptorType_new( pyDescriptorType* self, PyObject *args )
{

  self = (pyDescriptorType*) pyDescriptorType_alloc();

  return (PyObject*)self;
}

static PyObject* pyDescriptorType_alloc( void )
{
  pyDescriptorType* res = NULL;

  res = PyObject_New( pyDescriptorType, &dmDescriptorType_Type);
  res->typenum = dmBADDESC;

  return (PyObject*)res;
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptorType_init                                        */
/* ---------------------------------------------------------------------- */
static int pyDescriptorType_init( pyDescriptorType* self, PyObject *args )
{
  PyObject* obj1;
  long intype;
  int retval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O",  &obj1)){ return -1;}

  if ( PyObject_TypeCheck( obj1, &dmDescriptorType_Type) )
  {
    intype = ((pyDescriptorType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    intype = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmDescriptorType|PyInt_Type)" );
      return -1;
  }

  retval =  pyDescriptorType_set( self, intype );

  return retval;
}

static int pyDescriptorType_set( pyDescriptorType* self, long desctype )
{
  int retval = 0;

  switch ( desctype ){
  case dmBADDESC:                // Fallthrough intended
  case dmKEY:                    // Fallthrough intended
  case dmSUBSPACE:               // Fallthrough intended
  case dmCOORD:                  // Fallthrough intended
  case dmCOLUMN:
    self->typenum = (dmDescriptorType)desctype;
    break;
  case dmIMAGEDATA:              // account for deprecation
    self->typenum = dmCOLUMN;
    break;
  default:
      PyErr_SetString(PyExc_ValueError, "bad value; must be dmBADDESC, dmKEY, dmSUBSPACE, dmCOORD, dmCOLUMN" );
      retval = -1 ;
      break;
  }
  return( retval );
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptorType_dealloc                                          */
/* ---------------------------------------------------------------------- */
static void pyDescriptorType_dealloc( pyDescriptorType* self )
{
  if ( cxcdm_verbose_level > 4 )
    printf("  VERBOSE: pyDescriptorType_dealloc() called\n");

  /* free the python object */
  PyObject_Del(self);
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptorType_repr                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyDescriptorType_repr( pyDescriptorType* self )
{
  char buf[15];
  switch ( self->typenum ){
  case dmBADDESC:
    sprintf(buf,"dmBADDESC");
    break;
  case dmKEY:
    sprintf(buf,"dmKEY");
    break;
  case dmSUBSPACE:
    sprintf(buf,"dmSUBSPACE");
    break;
  case dmCOORD:
    sprintf(buf,"dmCOORD");
    break;
  case dmIMAGEDATA:              // Fallthrough intentional
  case dmCOLUMN:
    sprintf(buf,"dmCOLUMN");
    break;
  default:
    sprintf(buf,"UNKNOWN");
    break;
  }

  return (PyObject*)PyString_FromString(buf);
}


/* ---------------------------------------------------------------------- */
/* TypeFunc: pyDescriptorType_rcmp                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyDescriptorType_rcmp( PyObject* obj1, PyObject* obj2, int op )
{
  Bool       res = FALSE;

  long a = -1;
  long b = -1;

  if ( PyObject_TypeCheck( obj1, &dmDescriptorType_Type) )
  {
    a = ((pyDescriptorType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    a = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmDescriptorType|PyInt_Type)" );
      return NULL;
  }

  if ( PyObject_TypeCheck( obj2, &dmDescriptorType_Type) )
  {
    b = ((pyDescriptorType*)obj2)->typenum;
  }
  else if ( PyInt_Check( obj2 ) )
  {
    b = PyInt_AsLong( obj2 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmDescriptorType|PyInt_Type)" );
      return NULL;
  }

  switch ( op ){
  case Py_LT:
    res = ( a < b );
    break;
  case Py_LE:
    res = ( a <= b );
    break;
  case Py_EQ:
    res = ( a == b );
    break;
  case Py_NE:
    res = ( a != b );
    break;
  case Py_GT:
    res = ( a > b );
    break;
  case Py_GE:
    res = ( a >= b );
    break;
  default:
    res = FALSE;
    break;
  }

  if ( res == TRUE )
    Py_RETURN_TRUE;
  else
    Py_RETURN_FALSE;
}

/* ---------------------------------------------------------------------- */
/*                     *** dmElementType datatype ***                       */
/* ---------------------------------------------------------------------- */
static PyTypeObject dmElementType_Type =
{
  PyObject_HEAD_INIT(NULL)
  0,                                    /*ob_size*/
  (char*)"dmElementType",                 /*tp_name*/
  sizeof(pyDescriptorType),                  /*tp_basicsize*/
  0,                                    /*tp_itemsize*/
  (destructor)pyElementType_dealloc,      /*tp_dealloc*/
  0,                                    /*tp_print*/
  0,                                    /*tp_getattr*/
  0,                                    /*tp_setattr*/
  0,                                    /*tp_compare*/
  pyElementType_repr,                     /*tp_repr*/
  0,                                    /*tp_as_number*/
  0,                                    /*tp_as_sequence*/
  0,                                    /*tp_as_mapping*/
  0,                                    /*tp_hash */
  0,                                    /*tp_call*/
  0,                                    /*tp_str*/
  0,                                    /*tp_getattro*/
  0,                                    /*tp_setattro*/
  0,                                    /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,                   /*tp_flags*/
  "dmElementType object",                 /* tp_doc */
  0,                                    /* tp_traverse */
  0,                                    /* tp_clear */
  pyElementType_rcmp,                     /* tp_richcompare */
  0,                                    /* tp_weaklistoffset */
  0,                                    /* tp_iter */
  0,                                    /* tp_iternext */
  0,                                    /* tp_methods */
  0,                                    /* tp_members */
  0,                                    /* tp_getset */
  0,                                    /* tp_base */
  0,                                    /* tp_dict */
  0,                                    /* tp_descr_get */
  0,                                    /* tp_descr_set */
  0,                                    /* tp_dictoffset */
  pyElementType_init,                     /* tp_init */
  0,                                    /* tp_alloc */
  pyElementType_new,                      /* tp_new */

};


/* ---------------------------------------------------------------------- */
/* TypeFunc: pyElementType_new                                              */
/* ---------------------------------------------------------------------- */
static PyObject* pyElementType_new( pyElementType* self, PyObject *args )
{

  self = (pyElementType*) pyElementType_alloc();

  return (PyObject*)self;
}

static PyObject* pyElementType_alloc( void )
{
  pyElementType* res = NULL;

  res = PyObject_New( pyElementType, &dmElementType_Type);
  res->typenum = dmVALUE;

  return (PyObject*)res;
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyElementType_init                                             */
/* ---------------------------------------------------------------------- */
static int pyElementType_init( pyElementType* self, PyObject *args )
{
  PyObject* obj1;
  long intype;
  int retval = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O",  &obj1)){ return -1;}

  if ( PyObject_TypeCheck( obj1, &dmElementType_Type) )
  {
    intype = ((pyElementType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    intype = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmElementType|PyInt_Type)" );
      return -1;
  }

  retval =  pyElementType_set( self, intype );

  return retval;
}

static int pyElementType_set( pyElementType* self, long elemtype )
{
  int retval = 0;

  switch ( elemtype ){
  case dmUNKBLOCK:
    self->typenum = dmVALUE;
    break;
  case dmTABLE:
    self->typenum = dmINTERVAL;
    break;
  case dmIMAGE:
    self->typenum = dmRANGE;
    break;
  default:
      PyErr_SetString(PyExc_ValueError, "bad value; must be dmVALUE, dmINTERVAL, or dmRANGE" );
      retval = -1 ;
      break;
  }
  return( retval );
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyElementType_dealloc                                          */
/* ---------------------------------------------------------------------- */
static void pyElementType_dealloc( pyElementType* self )
{
  if ( cxcdm_verbose_level > 4 )
    printf("  VERBOSE: pyElementType_dealloc() called\n");

  /* free the python object */
  PyObject_Del(self);
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyElementType_repr                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyElementType_repr( pyElementType* self )
{
  char buf[15];
  if ( self->typenum == dmVALUE )
    sprintf(buf,"dmVALUE");
  else if ( self->typenum == dmINTERVAL )
    sprintf(buf,"dmINTERVAL");
  else
    sprintf(buf,"dmRANGE");

  return (PyObject*)PyString_FromString(buf);
}

/* ---------------------------------------------------------------------- */
/* TypeFunc: pyElementType_rcmp                                             */
/* ---------------------------------------------------------------------- */
static PyObject* pyElementType_rcmp( PyObject* obj1, PyObject* obj2, int op )
{
  Bool       res = FALSE;

  long a = -1;
  long b = -1;

  if ( PyObject_TypeCheck( obj1, &dmElementType_Type) )
  {
    a = ((pyElementType*)obj1)->typenum;
  }
  else if ( PyInt_Check( obj1 ) )
  {
    a = PyInt_AsLong( obj1 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmElementType|PyInt_Type)" );
      return NULL;
  }

  if ( PyObject_TypeCheck( obj2, &dmElementType_Type) )
  {
    b = ((pyElementType*)obj2)->typenum;
  }
  else if ( PyInt_Check( obj2 ) )
  {
    b = PyInt_AsLong( obj2 );
  }
  else
  {
      PyErr_SetString(PyExc_TypeError, "argument is not a supported type (dmElementType|PyInt_Type)" );
      return NULL;
  }

  switch ( op ){
  case Py_LT:
    res = ( a < b );
    break;
  case Py_LE:
    res = ( a <= b );
    break;
  case Py_EQ:
    res = ( a == b );
    break;
  case Py_NE:
    res = ( a != b );
    break;
  case Py_GT:
    res = ( a > b );
    break;
  case Py_GE:
    res = ( a >= b );
    break;
  default:
    res = FALSE;
    break;
  }

  if ( res == TRUE )
    Py_RETURN_TRUE;
  else
    Py_RETURN_FALSE;
}

/* ****************************************************************************** */
/* ****************************************************************************** */


/* ---------------------------------------------------------------------- */
/*                  *** dmDescriptor functions ***                        */
/*                  ***       General          ***                        */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetArea                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetArea( PyObject *self, PyObject *args ) 
{
  char* funcName = "dmDescriptorGetArea";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  double area;
  double field_area;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetArea", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    area = dmDescriptorGetArea( ((pyDescriptor*)obj1)->dd, &field_area );

    /* convert results to Python objects. */
    result = Py_BuildValue("dd", area, field_area);
  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetBin                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetBin(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetBin";

  PyObject*      obj1   = NULL;

  int            ptype;
  PyArrayObject* res1 = NULL;

  dmDescriptor*  _dd    = NULL;
  dmDataType     dtype  = dmUNKNOWNTYPE;
  dmErrCode      status = dmFAILURE;

  intp  dims[1];
  void* _binval = NULL;


  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetBin", &dmDescriptor_Type, &obj1))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( _dd );
  ptype = cxc_get_py_type( dtype );


  /* create memory to hold output value */
  dims[0] = 1;
  res1 = (PyArrayObject*) PyArray_ZEROS(0, dims, ptype, 0 );
  if (res1 == NULL)
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "descriptor bin size");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  _binval = PyArray_DATA(res1);

  status = dmi_descriptor_get_cast_bin( _dd, dtype, _binval );
  if (status == dmFAILURE)
  {
    /* This function returns dmFAILURE if no null is set on the descriptor  */
    /* There is no error message set for this condition, so we can detect.  */
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if ( strlen( errmsg ) > 1 )
    {
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      goto fail;
    }
  }

  /* convert results to Python objects. */
  return Py_BuildValue("O", PyArray_Return(res1));

 fail:

  if (res1)
    PyObject_Del ( res1 );

  return NULL;

}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetBlock                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetBlock( PyObject *self, PyObject *args ) 
{
  char* funcName = "dmDescriptorGetBlock";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  dmBlock*   _block = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetBlock", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _block = dmDescriptorGetBlock( ((pyDescriptor*)obj1)->dd );
    if ( _block == NULL )
    {
      sprintf(errmsg, DMDESC_GENERIC_MSG ,funcName, "unable to obtain block from input descriptor.");
      PyErr_SetString(PyExc_RuntimeError, errmsg );
    }
    else
    {
      result = pyBlock_new();
      ((pyBlock*)result)->block = _block;

      /* Register this object with memory management */
      cxcdm_memory_register( result );
    }
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorDelete                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorDelete( PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorDelete";

  PyObject* obj1    = NULL;

  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorDelete", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {

    status = dmDescriptorDelete( ((pyDescriptor*)obj1)->dd );
    if (status == dmFAILURE)
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      if ( strlen( errmsg ) > 1 )
      {
        PyErr_SetString(PyExc_RuntimeError, errmsg);
        return NULL;
      }
    }
    else
       cxcdm_memory_delete ( ((pyDescriptor*)obj1)->dd );
  }
  Py_RETURN_NONE;
}



/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetDataType                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetDataType(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetDataType";

  PyObject* obj1    = NULL;

  dmDataType _dtype = dmUNKNOWNTYPE;
  int        ptype;

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetDataType", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    _dtype = dmGetDataType( ((pyDescriptor*)obj1)->dd );
    ptype = cxc_get_py_type( _dtype );
    result = PyArray_TypeObjectFromType( ptype );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetDesc                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetDesc(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetDesc";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char _cstr[dmMAXSTR+1];

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetDesc", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmGetDesc( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( _cstr );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetElementDim                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetElementDim(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetElementDim";

  PyObject* obj1    = NULL;

  long lval;

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetElementDim", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    lval = dmGetElementDim( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( lval );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetLength                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetLength(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetLength";

  PyObject* obj1    = NULL;

  long lval;

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetLength", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    lval = dmDescriptorGetLength( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( lval );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetElementType                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetElementType(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetElementType";

  PyObject* obj1    = NULL;

  dmElementType elem_type;

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetElementType", &dmDescriptor_Type, &obj1)){ return NULL;}


  /* Check underlying dmElement pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    elem_type = dmGetElementType( ((pyDescriptor*)obj1)->dd );
    result = pyElementType_alloc();
    if ( pyElementType_set( (pyElementType*)result, (long)elem_type ) < 0 )
    {
      /* invalid type.. clear result and return error */
      Py_DECREF(result);
      return NULL;
    }

  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetType                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetType(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetType";

  PyObject* obj1    = NULL;

  dmDescriptorType desc_type;

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetType", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    desc_type = dmDescriptorGetType( ((pyDescriptor*)obj1)->dd );
    result = pyDescriptorType_alloc();
    if ( pyDescriptorType_set( (pyDescriptorType*)result, (long)desc_type ) < 0 )
    {
      /* invalid type.. clear result and return error */
      Py_DECREF(result);
      return NULL;
    }

  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetName                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetName(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetName";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char _cstr[dmMAXSTR+1];

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetName", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmGetName( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( _cstr );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetNo                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetNo(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetNo";

  PyObject* obj1 = NULL;
  PyObject* result  = NULL;

  long lval;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetNo", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    lval = dmDescriptorGetNo( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( lval );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetNoCoords                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetNoCoords(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetNoCoords";

  PyObject* obj1 = NULL;
  PyObject* result  = NULL;

  long lval;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetNoCoords", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    lval = dmDescriptorGetNoCoords( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( lval );

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetRange                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetRange(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetRange";

  PyObject*      obj1   = NULL;

  int            ptype;
  PyArrayObject* res1 = NULL;
  PyArrayObject* res2 = NULL;

  dmDescriptor*  _dd    = NULL;
  dmDataType     dtype  = dmUNKNOWNTYPE;
  dmErrCode      status = dmFAILURE;

  intp  dims[1];
  void* rangemin = NULL;
  void* rangemax = NULL;


  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetRange", &dmDescriptor_Type, &obj1))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( _dd );
  ptype = cxc_get_py_type( dtype );


  /* create memory to hold output range values */
  dims[0] = 1;
  res1 = (PyArrayObject*) PyArray_ZEROS(0, dims, ptype, 0 );
  res2 = (PyArrayObject*) PyArray_ZEROS(0, dims, ptype, 0 );
  if ((res1 == NULL) || (res2 == NULL))
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "ranges");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }
  
  rangemin = PyArray_DATA(res1);
  rangemax = PyArray_DATA(res2);

  status = dmi_descriptor_get_cpt_range( _dd, 1L, dtype, rangemin, rangemax );
  if (status == dmFAILURE)
  {
    /* This function returns dmFAILURE if no range is set on the descriptor */
    /* and returns the open range for min/max.. which is appropriate.       */
    /* There is no error message set for this condition, so we can detect.  */
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if ( strlen( errmsg ) > 1 )
    {
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      goto fail;
    }
  }

  /* convert results to Python objects. */
  return Py_BuildValue("OO", PyArray_Return(res1), PyArray_Return(res2));

 fail:

  if ( res1 )
    PyObject_Del ( res1 );
  if ( res2 )
    PyObject_Del ( res2 );

  return NULL;

}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetNull                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetNull(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorGetNull";

  PyObject*      obj1   = NULL;

  int            ptype;
  PyArrayObject* res1 = NULL;

  dmDescriptor*  _dd    = NULL;
  dmDataType     dtype  = dmUNKNOWNTYPE;
  dmErrCode      status = dmFAILURE;

  intp  dims[1];
  void* tnull = NULL;


  if (!PyArg_ParseTuple(args, "O!:dmDescriptorGetNull", &dmDescriptor_Type, &obj1))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( _dd );
  ptype = cxc_get_py_type( dtype );


  /* create memory to hold output null value */
  dims[0] = 1;
  res1 = (PyArrayObject*) PyArray_ZEROS(0, dims, ptype, 0 );
  if (res1 == NULL)
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "ranges");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  tnull = PyArray_DATA(res1);

  status = dmi_descriptor_get_cast_null( _dd, dtype, tnull );
  if (status == dmFAILURE)
  {
    /* This function returns dmFAILURE if no null is set on the descriptor  */
    /* There is no error message set for this condition, so we can detect.  */
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if ( strlen( errmsg ) > 1 )
    {
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      goto fail;
    }
    else
    {
      /* No null set on descriptor.. return None object */
      PyObject_Del ( res1 );
      Py_RETURN_NONE;
    }
  }

  /* convert results to Python objects. */
  return Py_BuildValue("O", PyArray_Return(res1));

 fail:
  if (res1){ PyObject_Del ( res1 ); }
  return NULL;

}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetUnit                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetUnit(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetUnit";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char _cstr[dmMAXSTR+1];

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetUnit", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmGetUnit( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( _cstr );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorIsVarLength                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorIsVarLength( PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorIsVarLength";

  PyObject* obj1   = NULL;
  PyObject* result = NULL;

  dmBool _res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmDescriptorIsVarLength", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _res = dmDescriptorIsVarLength( ((pyDescriptor*)obj1)->dd );
    result = PyBool_FromLong( (long)_res );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetBin                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetBin(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorSetBin";

  PyObject* obj1    = NULL;
  PyObject* obj2    = NULL;

  dmDataType dtype  = dmUNKNOWNTYPE;
  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O&:dmDescriptorSetBin", &dmDescriptor_Type, &obj1, &PyArray_Converter, &obj2 ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* get data type */ 
  void* value = PyArray_DATA( obj2 );
  dtype = cxc_get_dm_type(PyArray_TYPE( obj2 ));

  switch ( dtype)
  {
    case dmSHORT:
    case dmLONG:
    case dmFLOAT:
    case dmDOUBLE:
    case dmBYTE:
    case dmUSHORT:
    case dmULONG:
      status = dmp_descriptor_set_bin(((pyDescriptor*)obj1)->dd, dtype, (void*)value);
    break;
    default:
      sprintf( errmsg, "%s() argument 2 is not a supported type.", funcName );
      PyErr_SetString(PyExc_TypeError, errmsg );
      goto fail;
    break;
  }

  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  /* Decrement results of PyArray_Converter */
  Py_DECREF(obj2);
  Py_RETURN_NONE;

 fail:
  Py_DECREF(obj2);
  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetDesc                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetDesc(PyObject *self, PyObject *args )
{
  char* funcName = "dmSetDesc";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char* cstr;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!s:dmSetDesc", &dmDescriptor_Type, &obj1, &cstr ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSetDesc( ((pyDescriptor*)obj1)->dd, cstr );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetName                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetName(PyObject *self, PyObject *args )
{
  char* funcName = "dmSetName";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char* cstr;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!s:dmSetName", &dmDescriptor_Type, &obj1, &cstr ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSetName( ((pyDescriptor*)obj1)->dd, cstr );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetUnit                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetUnit(PyObject *self, PyObject *args )
{
  char* funcName = "dmSetUnit";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char* cstr;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!s:dmSetUnit", &dmDescriptor_Type, &obj1, &cstr ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSetUnit( ((pyDescriptor*)obj1)->dd, cstr );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetRange                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetRange(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorSetRange";

  PyObject* obj1    = NULL;
  PyObject* obj2    = NULL;
  PyObject* obj3    = NULL;

  dmErrCode status;

  void* tmin = NULL;
  void* tmax = NULL;

  dmDataType     dtype  = dmUNKNOWNTYPE;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O&O&:dmDescriptorSetRange", 
                         &dmDescriptor_Type, &obj1,
                         &PyArray_Converter, &obj2,
                         &PyArray_Converter, &obj3  ))
    { return NULL;}



  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  /* verify the second and third arguments are numbers */
  if ( !obj2 || !( PyArray_ISNUMBER(obj2)))
  {
    /* problem */
    sprintf( errmsg, "%s() argument 2 type must be a number.", funcName );
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }

  /* verify the second and third arguments are numbers */
  if ( !obj3 || !( PyArray_ISNUMBER(obj3)))
  {
    /* problem */
    sprintf( errmsg, "%s() argument 3 type must be a number.", funcName );
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }

  /* check that the two types are the same */
  dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );
  if (dtype != cxc_get_dm_type( PyArray_TYPE( obj3 ) ))
  {
     /* problem */
    sprintf( errmsg, "%s() arguments 2 and 3 must be of the same type.", funcName);

    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }

  /* check that the two types are the same */
  if (dtype == dmTEXT)
  {
     /* problem */
    sprintf( errmsg, DMDESC_UNK_DTYPE_MSG, funcName, "dmTEXT"  );
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }


  tmin = PyArray_DATA(obj2);
  tmax = PyArray_DATA(obj3);

  status = dmp_descriptor_set_range( ((pyDescriptor*)obj1)->dd, dtype, tmin, tmax);
  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }
  /* Decrement results of PyArray_Converter */
  Py_DECREF(obj2);
  Py_DECREF(obj3);

  Py_RETURN_NONE;

 fail:
  if ( obj2 ){ Py_DECREF(obj2);}
  if ( obj3 ){ Py_DECREF(obj3);}
  return NULL;

}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetArrayDim                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetArrayDim(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetArrayDim";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetArrayDim", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    res = dmGetArrayDim( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( res );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetArrayDimensions                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetArrayDimensions(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetArrayDimensions";

  PyObject* obj1    = NULL;
  

  PyArrayObject* result = NULL;
  intp dims[1];
  long *dimarr;
  long ndim;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetArrayDimensions", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    /* values allocated within DM */
    ndim = dmGetArrayDimensions( ((pyDescriptor*)obj1)->dd, &dimarr);

    dims[0] = ndim;
    result = (PyArrayObject*) PyArray_ZEROS(1, dims, NPY_LONG, 0 );
    if (ndim > 0)
    {
      long* dptr = (long*)PyArray_DATA(result);
      int ii;
      /* reverse the ptr so that it's c style */
      for (ii=ndim; ii--;)
         dptr[ndim-ii-1] = dimarr[ii];
      free( dimarr );
    }
  }
  if ( result == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    if ( dimarr )
      free (dimarr);
    return NULL;
  }

  return (PyObject *)result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetArraySize                                               */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetArraySize(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetArraySize";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetArraySize", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    res = dmGetArraySize( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( res );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetCurrentArraySize                                        */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetCurrentArraySize(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetCurrentArraySize";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetCurrentArraySize", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    res = dmGetCurrentArraySize( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( res );
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetCpt                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetCpt( PyObject *self, PyObject *args ) 
{
  char* funcName = "dmDescriptorGetCpt";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  dmDescriptor* _dd = NULL;
  long cptno        = -1;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmDescriptorGetCpt", &dmDescriptor_Type, &obj1, &cptno)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _dd = dmGetCpt( ((pyDescriptor*)obj1)->dd, cptno );
    if ( _dd == NULL )
    {
      sprintf(errmsg, DMDESC_GENERIC_MSG ,funcName, "unable to obtain requested component from descriptor.");
      PyErr_SetString(PyExc_RuntimeError, errmsg );
    }
    else
    {
      result = pyDescriptor_new();
      ((pyDescriptor*)result)->dd = _dd;

      /* Register this object with memory management */
      cxcdm_memory_register( result );
    }
  }

  return (PyObject *)result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorGetCoord                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetCoord(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmDescriptorGetCoord";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  dmDescriptor* _dd = NULL;
  long coordno = -1;

  static char *kwlist[] = {"dd", "coordno", NULL};


  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|l:dmDescriptorGetCoord", kwlist, &dmDescriptor_Type, &obj1, &coordno)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    if (coordno != -1)
       _dd = dmDescriptorGetCoordNo( ((pyDescriptor*)obj1)->dd, coordno );
    else
       _dd = dmDescriptorGetCoord( ((pyDescriptor*)obj1)->dd);

    if ( _dd == NULL )
    {
      sprintf(errmsg, DMDESC_GENERIC_MSG ,funcName, "coordinate not found on descriptor.");
      PyErr_SetString(PyExc_RuntimeError, errmsg );
      return NULL;
    }
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = _dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return( result );
}


/* ---------------------------------------------------------------------- */
/*                  *** dmDescriptor functions ***                        */
/*                  ***       Keyword          ***                        */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetKey                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetKey(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetKey";

  PyObject* obj1 = NULL;
  long     keyno = -1;

  dmDescriptor*  _key = NULL;
  PyObject*    result = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmBlockGetKey", &dmBlock_Type, &obj1, &keyno)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Get Key from DM layer */
  _key = dmBlockGetKey( ((pyBlock*)obj1)->block, keyno );
  if ( _key == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if ( strlen(errmsg) < 2 )
    {
      sprintf( errmsg, DMKEY_OPEN2_ERR_MSG, funcName, keyno );
    }
    PyErr_SetString(PyExc_RuntimeError, errmsg);
  }
  else
  {
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _key;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetKeyList                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetKeyList(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetKeyList";

  PyObject* obj1   = NULL;
  PyObject* key    = NULL;
  PyObject* result = NULL;

  dmDescriptor** klist = NULL;
  dmDescriptor*   _key = NULL;
 
  long ii;
  long nkeys;
  Py_ssize_t len;
  Py_ssize_t ndx;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetKeyList", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Get KeyList from DM layer */
  klist = dmBlockGetKeyList( ((pyBlock*)obj1)->block, &nkeys );
  if ( klist == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
  }
  else
  {
    /* Create a Python list to hold keys */
    len = (Py_ssize_t)nkeys;
    result = PyList_New(len);
    if ( result == NULL )
    {
      sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "Key List");
      PyErr_SetString(PyExc_MemoryError, errmsg );
    }
    else
    {
      /* Transfer DM keys to Python list of Python keys. */
      for ( ii = 0; ii < nkeys; ii++ )
      {
        _key = klist[ii];
        key  = pyDescriptor_new();
        ((pyDescriptor*)key)->dd = _key;

        /* Register this object with memory management */
        cxcdm_memory_register( key );
     
        ndx = (Py_ssize_t)ii;
        PyList_SET_ITEM( result, ndx, key );
      }
    }
    /* DM allocated the list ptrs.. no longer needed */
    free(klist);
  }
      
  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockMoveToKey                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockMoveToKey(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockMoveToKey";

  PyObject* obj1 = NULL;
  PyObject* obj2 = NULL;

  long  keyno   = -1;
  char* keyname = NULL;

  dmDescriptor*  _key = NULL;
  PyObject*    result = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O:dmBlockMoveToKey", &dmBlock_Type, &obj1, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Validate second input, and execute */
  if ( PyInt_Check( obj2 ) )
  {
    keyno =  PyInt_AsLong( obj2 );
    if ( keyno > 0 )
    {
      _key  =  dmBlockMoveToKeyNo( ((pyBlock*)obj1)->block, keyno );
    }
    else
    {
      sprintf(errmsg, DMDESC_INVALID_ARGVAL_MSG ,funcName, 2L );
      PyErr_SetString(PyExc_ValueError, errmsg );
      return NULL;
    }
  }
  else if ( PyString_Check( obj2 ) )
  {
    keyname = PyString_AsString( obj2 );
    _key = dmBlockMoveToKey( ((pyBlock*)obj1)->block, keyname );
  }
  else
  {
    /* problem */
    sprintf(errmsg, DMDESC_INVALID_ARGTYP_MSG, funcName, 2L, (char*)"int or string", obj2->ob_type->tp_name);
    PyErr_SetString(PyExc_TypeError, errmsg );
    return NULL;
  }

  if ( _key == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if ( strlen(errmsg) < 2 )
    {
      if ( keyname == NULL )
	sprintf( errmsg, DMKEY_OPEN2_ERR_MSG, funcName, keyno );
      else
	sprintf( errmsg, DMKEY_OPEN_ERR_MSG, funcName, keyname );
    }
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _key;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockAdvanceKeys                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockAdvanceKeys(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockAdvanceKeys";

  PyObject* obj1 = NULL;
  long     step = -1;

  dmDescriptor*  _key = NULL;
  PyObject*    result = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmBlockAdvanceKeys", &dmBlock_Type, &obj1, &step)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Get Key from DM layer */
  _key = dmBlockAdvanceKeys( ((pyBlock*)obj1)->block, step );
  if ( _key == NULL )
  {
    sprintf( errmsg, "%s() can not move %1ld keys from the current position.", funcName, step );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _key;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }
  return result;
}

/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmKeyOpen                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmKeyOpen(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmKeyOpen";

  PyObject*     obj1  = NULL;
  char*         name  = NULL;

  dmDescriptor* _key = NULL;
  PyObject*   result = NULL;

  static char *kwlist[] =
    {"block", "name", NULL};

  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!s:dmKeyOpen", kwlist, &dmBlock_Type, &obj1, &name)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
  }
  else
  {
    _key = dmKeyOpen( ((pyBlock*)obj1)->block, name );
    if ( _key == NULL )
    {
      sprintf(errmsg, DMKEY_OPEN_ERR_MSG ,funcName, name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      return NULL;
    }

    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _key;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmKeyRead                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmKeyRead(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmKeyRead";

  static char *kwlist[] = {"block", "name", "dtype", NULL};

  PyObject*      obj1  = NULL;
  PyArray_Descr* descr = NULL;
  char*          name  = NULL;

  dmDescriptor*   _key = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  int            ptype;
  PyArray_Dims   dims = {NULL, 0};
  long           bdim = 0;
  int         itemsize;

  long  ii;
  long  nrowsgot = 0;

  void*  data  = NULL;
  char*  buf   = NULL;
  char** cdata = NULL;
  dmBool *bdata=NULL;

  PyObject*      res1 = NULL;
  PyArrayObject* res2 = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!s|O&:dmKeyRead",
                       kwlist,
                       &dmBlock_Type, &obj1,
                       &name,
                       &PyArray_DescrConverter2, &descr)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  /* Get the keyword descriptor */
  _key = dmKeyOpen( ((pyBlock*)obj1)->block, name );
  if ( _key == NULL )
  { 
    sprintf(errmsg, DMKEY_OPEN_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }


  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( _key );
  ptype = cxc_get_py_type( dtype );
  cxc_get_dimarr( _key, &dims, NULL );
  itemsize = dmDescriptorGetLength( _key );
 
  /* Process dtype argument */
  if ( descr != NULL )
  {
    /* validate input dtype */
    if ( !( PyDataType_ISNUMBER(descr) || 
         PyDataType_ISSTRING(descr) ) )
    {
      /* problem */
      sprintf(errmsg, DMDESC_BAD_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }
  

    /* convert to DM datatype */
    int        ptype_new = descr->type_num;
    dmDataType dtype_new = cxc_get_dm_type( ptype_new );
    if ( (dtype_new == dmTEXT) && (dtype != dmTEXT ) )
    {
      /* converting non-string to string.. */
      /* set itemsize so memory will be allocated to store value */
      /* itemsize = dmMAXLEN - 1; */

      /* NOTE: ar_mix_array_cast() replaces the memory sent in with new      */
      /* memory containing the converted value (without freeing)             */
      /* using cs_dup().. so converting to string will not work so long      */
      /* as we create the Python array first.                                */

      /* problem */
      sprintf(errmsg, DMDESC_BAD_CONV_MSG ,funcName, (char*)"non-string", descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    ptype = ptype_new;
    dtype = dtype_new;
  }
  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  /* hhe @ 09/18/2012
   *  as DM Document (R2.1) states:
   *  for the case of a vector or array of key, return the value of 
   *  the first component of the the first element.
   *
   * Note that if the named 'key' is image block name, this routine will segV
   *
   *  Below is to check the the data dimention and set it to 1
   */
  if(dims.ptr != NULL && dims.len >= 1){
    if ( cxcdm_verbose_level > 1 )
      printf("  VERBOSE: %s, set vector/array the first element of the first component to 1.\nThe original ",funcName);
    for (ii = 0; ii < dims.len; ii++ ){
      if ( cxcdm_verbose_level > 1 )
	printf("  VERBOSE: %s, dim=%i sz=%i\n",funcName, (int)dims.len, (int)dims.ptr[ii]);
      dims.ptr[ii] =1; /* pick the first element */
    }
    dims.len=0; /* pick the first component */
  }
 
  
  /* MCD NOTE: Much of the rest of this code is repeated in dmDescriptorGetData     */
  /*           Could get consolidated if more usage of the array handling is needed */
  
  

  if ( dtype == dmTEXT )
  {
    itemsize++;  /* add one for terminator */
 
    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, itemsize, 0 , NULL );
    buf = PyArray_DATA(res2);

    long cdim = 1;
    for (ii = 0; ii < dims.len; ii++ )
      cdim *= dims.ptr[ii];

    /* Create string array pointer for DM layer.. The data pointer in the array is just the underlying buffer */
    cdata = (char**)calloc( cdim, sizeof(char*) );
    cdata[0] = buf;
    for ( ii=1; ii < cdim; ii++){
      cdata[ii] = cdata[ii-1] + itemsize;
    }

    data = (void*)cdata;
  }
  else if ( dtype == dmBOOL )
  {
    bdim = 1;
    for (ii = 0; ii < dims.len; ii++ )
      bdim *= dims.ptr[ii];

    /* Need separate memory for DM since it stores bool as different size */
    bdata  = (dmBool*)calloc( bdim, sizeof(dmBool) );

    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, 0, 0, NULL );
    data = (void*)bdata;
  }
  else
  {
    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, itemsize, 0, NULL );
    data = PyArray_DATA(res2);
  }
  if ( res2 == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  /* Load value(s) for descriptor.. converting to requested datatype           */
  
    
  nrowsgot = dmGetVectorArrays( _key, dtype, data, 1L,   1,   1  , 1,  1 );
   
  if ( nrowsgot < 0 )
  {
    sprintf(errmsg, DMDESC_VALUES_ERR_MSG ,funcName, (char*)"getting");
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );    

    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }
  else if ( nrowsgot == 0 )
  {
    sprintf(errmsg, DMDESC_NOROWS_ERR_MSG ,funcName);
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }

  if ( dtype == dmBOOL )
  {
    /* Transfer boolean value(s) to Python memory */
    npy_bool *bbuf = (npy_bool *)(res2->data);
    dmBool *_bdata = (dmBool *)data;
    for (ii=bdim; ii--;)
       *(bbuf++) = (npy_bool)(*(_bdata++));
  }

  if ( cdata )
    free(cdata);
  else if ( bdata )
    free(bdata);

  if ( dims.ptr )
    PyDimMem_FREE( dims.ptr );

  /* Create Python object for output descriptor */
  res1 = pyDescriptor_new();
  ((pyDescriptor*)res1)->dd = _key;

  /* Register this object with memory management */
  cxcdm_memory_register( res1 );

  
  return Py_BuildValue("ON", res1, PyArray_Return( res2 )); 

 fail:
  if ( dims.ptr )
    PyDimMem_FREE( dims.ptr );
  if ( cdata )
    free(cdata);
  else if ( bdata )
    free(bdata);

  return NULL;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmKeyWrite                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmKeyWrite(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmKeyWrite";

  static char *kwlist[] = {"block", "name", "value", "unit", "desc", NULL};

  PyObject* obj1  = NULL;

  PyArrayObject* obj2 = NULL;
  PyArrayObject* tmp   = NULL;

  char*          name  = NULL;
  char*          unit  = NULL;
  char*          desc  = NULL;
  long           nvals = 1;
  void*          data  = NULL;
  dmBool*        bdata = NULL;

  dmDescriptor*    _key = NULL;
  dmDataType      dtype = dmUNKNOWNTYPE;
  PyObject*       result = NULL;

  long  ii;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!sO&|ss:dmKeyWrite",
                       kwlist,
                       &dmBlock_Type, &obj1,
                       &name,
                       &PyArray_Converter, &obj2,
                       &unit,
                       &desc )){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  if ( PyArray_NDIM( obj2 ) > 1 )
  {
    sprintf( errmsg, "%s() value must be scalar or 1D array.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  else
  {
    nvals = PyArray_SIZE( obj2 );
    dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );
    if ( dtype == dmUNKNOWNTYPE )
    {
      /* problem */
      sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, obj2->descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }
    else if ( dtype == dmTEXT )
    {
      /* Text type.. python stores only enough space for string (no terminator).. */
      data = cxc_strings_to_carray( obj2 );
    }
    else if ( dtype == dmBOOL )
    {
      /* DM booleans are not 1Byte.. provide mapping */
      bdata  = (dmBool*)calloc( nvals, sizeof(dmBool) ); 
      dmBool *buf = bdata;

      npy_bool *bbuf = (npy_bool *)(PyArray_DATA( obj2 ));
      for (ii=nvals; ii--;)
	*(buf++) = (dmBool)(*(bbuf++));

      data = (void*)bdata;
    }
    else if ( (PyArray_TYPE(obj2) == NPY_INT )&&(dtype == dmLONG) )
    {
      /* For 64bit compatibility.  DM layer expects 8byte values for  */
      /* long type where here, we are getting 4byte integers          */
      /* If this condition is true on 32bit, it should cast to the    */
      /* the same 4byte size.                                         */
      tmp = (PyArrayObject*) PyArray_Cast(obj2, NPY_LONG);
      data = PyArray_DATA( tmp );
    }
    else
    {
      data = PyArray_DATA( obj2 );
    }
  }

  _key = dmp_key_write( ((pyBlock*)obj1)->block, name, dtype, 
                    data, nvals, unit, desc, dmTRUE );
  if ( _key == NULL )
  { 
    sprintf(errmsg, DMKEY_WRITE_ERR_MSG ,funcName, name);
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = _key;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  if ( bdata ){ free (bdata); }
  if ( tmp ){ Py_DECREF(tmp);}

  Py_DECREF(obj2);
  if ( dtype == dmTEXT )
  {
    free( ((char**)data)[0] );
    free( (char**)data );
  }

  return result;

 fail:
  if ( bdata ){ free (bdata); }
  if ( tmp ){ Py_DECREF(tmp);}

  Py_DECREF(obj2);
  return NULL;
}

/* ---------------------------------------------------------------------- */
/*                  *** dmDescriptor functions ***                        */
/*                  ***        Column          ***                        */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* Binding: _dmColumnCreate                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmColumnCreate(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmColumnCreate";

  PyObject*     obj1  = NULL; /* for block      */
  PyObject*     obj2  = NULL; /* for cptnames   */
  PyObject*     obj3  = NULL; /* for axes/shape */
  char*         name  = NULL;
  char*         unit  = NULL;
  char*  description  = NULL;
  long      itemsize  = 0;
  PyArray_Descr* typdescr = NULL;

  dmDataType    dtype;
  dmElementType etype = dmVALUE;

  char **cptNames = NULL;
  long  vdim = 0;
  long* axes = NULL;
  long naxes = 0;  

  dmDescriptor* _col  = NULL;
  PyObject*   tmparr  = NULL;
  PyObject*   result  = NULL;

  static char *kwlist[] =
    {"block", "name", "dtype", "itemsize", "unit", "desc", "cptnames", "shape",  NULL};


  /* Parse and verify input arguments */
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!sO&|lssO&O&:dmColumnCreate", kwlist, 
                          &dmBlock_Type, &obj1,
                                               &name,
                  &PyArray_DescrConverter2, &typdescr,
                                         &itemsize,
                                         &unit,
                                         &description,
                     &PyArray_Converter, &obj2,
                     &PyArray_Converter, &obj3    ))
  { return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dtype = cxc_get_dm_type( typdescr->type_num );
  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, typdescr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  if (( dtype == dmTEXT ) && (itemsize == 0 ))
      {
	/* problem */
	sprintf(errmsg, DMDESC_BAD_STRLEN_MSG ,funcName);
	PyErr_SetString(PyExc_ValueError, errmsg );
	goto fail;
      }

  if ( obj2 != NULL )
  {
    /* This object contains the component names for a vector column */
    /* It MUST be a 1D string array */
    if ( (PyArray_TYPE( obj2 ) != NPY_STRING )||(PyArray_NDIM( obj2 ) > 1 ))
    {
      sprintf( errmsg, "%s() cptnames must be 1D string array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      cptNames = (char**)cxc_strings_to_carray( (PyArrayObject*)obj2 );
      if ( PyArray_NDIM( obj2 ) == 0 ){ vdim = 1; }
      else{ vdim = (long)PyArray_DIM( obj2 , 0);  }
    }
  }

  if ( obj3 != NULL )
  {
    /* This contains the shape of the column..               */
    /* should be a 1D array of longs..                       */
    /* How careful do we need to be about what type obj3 is? */
    if ( PyArray_NDIM( obj3 ) > 1 )
    {
      sprintf( errmsg, "%s() shape must be scalar or 1D array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      long ii;
      long *tmp_axes;
      tmparr = PyArray_Cast( (PyArrayObject*)obj3, NPY_LONG );
      tmp_axes   = PyArray_DATA( tmparr );

      if ( PyArray_NDIM( tmparr ) == 0 ){ naxes = 1; }
      else{ naxes = (long)PyArray_DIM( tmparr , 0);  }

      /* reverse the axes so that they are fortran style */
      axes = (long *)malloc (naxes * sizeof(long));
      for (ii=naxes; ii--;)
        axes[ii] = tmp_axes[naxes-ii-1];
    }
  }

  _col = dmColumnCreateGeneric( ((pyBlock*)obj1)->block, name, dtype, 
                    itemsize, unit, description, etype, 
                    cptNames, vdim, axes, naxes );

  if ( tmparr )
  {
    Py_DECREF( tmparr );
  }

  if ( cptNames )
  {
    free(cptNames[0]);
    free(cptNames);
  }

  if (axes)
    free (axes);

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = _col;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return result;

 fail:
  if (axes)
  {
    free (axes);
    Py_XDECREF( tmparr );
  }
  return NULL;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmTableOpenColumn                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableOpenColumn(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableOpenColumn";

  PyObject* obj1 = NULL;
  PyObject* obj2 = NULL;

  long  colno;
  char* colname;
  
  dmDescriptor* _col   = NULL;
  PyObject*     result = NULL;

  if (!PyArg_ParseTuple(args, "O!O:dmTableOpenColumn", &dmBlock_Type, &obj1, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Validate second input, and execute */
  if ( PyInt_Check( obj2 ) )
  {
    colno =  PyInt_AsLong( obj2 );
    if ( colno > 0 )
    {
      _col  =  dmTableOpenColumnNo( ((pyBlock*)obj1)->block, colno );
    }
    else
    {
      sprintf(errmsg, DMDESC_INVALID_ARGVAL_MSG ,funcName, 2L );
      PyErr_SetString(PyExc_ValueError, errmsg );
      return NULL;
    }
  }
  else if ( PyString_Check( obj2 ) )
  {
    colname = PyString_AsString( obj2 );
    _col = dmTableOpenColumn( ((pyBlock*)obj1)->block, colname );
  }
  else
  {
    /* problem */
    sprintf(errmsg, DMDESC_INVALID_ARGTYP_MSG, funcName, 2L, (char*)"int or string", obj2->ob_type->tp_name);
    PyErr_SetString(PyExc_TypeError, errmsg );
    return NULL;
  }

  if ( _col == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg );
  }
  else
  {
    /* All is well */
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _col;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

   return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmTableOpenColumnList                                        */
/* ---------------------------------------------------------------------- */
static PyObject* _dmTableOpenColumnList(PyObject *self, PyObject *args )
{
  char* funcName = "dmTableOpenColumnList";

  PyObject* obj1   = NULL;
  PyObject* col    = NULL;
  PyObject* result = NULL;

  dmDescriptor** collist = NULL;
  dmDescriptor*  _col    = NULL;

  long  ii;
  long  ncols;
  Py_ssize_t len;
  Py_ssize_t ndx;
  
  if (!PyArg_ParseTuple(args, "O!:dmTableOpenColumnList", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  collist = dmTableOpenColumnList( ((pyBlock*)obj1)->block, &ncols );
  if ( collist == NULL )
  {
    /* problem */
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_TypeError, errmsg );
  }
  else
  {
    /* Create a Python list to hold column dds */
    len = (Py_ssize_t)ncols;
    result = PyList_New(len);
    if ( result == NULL )
    {
      sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "Column List");
      PyErr_SetString(PyExc_MemoryError, errmsg );
    }
    else
    {
      /* Transfer DM columns to Python list of Python cols. */
      for ( ii = 0; ii < ncols; ii++ )
      {
     _col = collist[ii];
     col  = pyDescriptor_new();
     ((pyDescriptor*)col)->dd = _col;
     
     /* Register this object with memory management */
     cxcdm_memory_register( col );

     ndx = (Py_ssize_t)ii;
     PyList_SET_ITEM( result, ndx, col );
      }
    }
    /* DM allocated the list ptrs.. no longer needed */
    free(collist);
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/*                  *** dmDescriptor functions ***                        */
/*                  ***       Subspace         ***                        */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColOpen                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColOpen(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColOpen";

  PyObject* obj1    = NULL;
  char*     name    = NULL;

  dmDescriptor* _col = NULL;

  PyObject* result  = NULL;

  if (!PyArg_ParseTuple(args, "O!s:dmSubspaceColOpen", &dmBlock_Type, &obj1, &name)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  
  _col = dmSubspaceColOpen( ((pyBlock*)obj1)->block, name );
  if ( _col == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg );
  }
  else
  {
    /* All is well */
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _col;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColGet                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColGet(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColGet";

  PyObject* obj1 = NULL;
  

  PyArrayObject* res_min = NULL;
  PyArrayObject* res_max = NULL;

  dmDescriptor*  dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  int            ptype;
  npy_intp       dimarr[1];

  char*          dptr = NULL;
  void*          mins = NULL;
  void*          maxes = NULL;
  long           nvals = 0;
  long           size;

  if (!PyArg_ParseTuple(args, "O!:dmSubspaceColGet", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( dd );
  ptype = cxc_get_py_type( dtype );

  int dmerror = dmi_subspace_col_get_values(dd, 1, dtype, &mins, &maxes, &nvals);
  if (dmerror == dmFAILURE)
  {
    sprintf(errmsg, DMDESC_VALUES_ERR_MSG ,funcName, (char*)"getting");
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );

    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }

  /* Create arrays owned by python to hold values from DM */
  dimarr[0] = nvals;
  res_min = (PyArrayObject*) PyArray_ZEROS(1, dimarr, ptype, 0 );
  res_max = (PyArrayObject*) PyArray_ZEROS(1, dimarr, ptype, 0 ); 

  if ( (res_min == NULL)||(res_max == NULL) )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  /* copy */
  size = (long)PyArray_NBYTES(res_min);

  dptr = (char*)PyArray_DATA(res_min);
  memcpy( dptr, (char*)mins, size );

  dptr = (char*)PyArray_DATA(res_max);
  memcpy( dptr, (char*)maxes, size );

  /* Free DM allocated memory */
  if (mins){ free(mins); }
  if (maxes){ free(maxes); }

  return Py_BuildValue("OO", PyArray_Return( res_min ), PyArray_Return( res_max ));

 fail:
  if (res_min)
    PyObject_Del (res_min);
  else if (mins)
    free (mins);
  if (res_max)
    PyObject_Del (res_max);
  else if (maxes)
    free (maxes);

  return NULL;

}


/* ---------------------------------------------------------------------- */
/*                      *** dmImage functions ***                         */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/* Binding: _dmImageGetDataDescriptor                                     */
/* ---------------------------------------------------------------------- */
static PyObject* _dmImageGetDataDescriptor(PyObject *self, PyObject *args )
{
  char* funcName = "dmImageGetDataDescriptor";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  dmDescriptor* _dd;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmImageGetDataDescriptor", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    _dd = dmImageGetDataDescriptor( ((pyBlock*)obj1)->block );
    if ( _dd == NULL )
    {
      sprintf(errmsg, DMBLOCK_FUNC_ERR_MSG ,funcName);
      PyErr_SetString(PyExc_RuntimeError, errmsg );
      return NULL;
    }
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = _dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmArrayGetAxisGroup                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmArrayCreateAxisGroup(PyObject *self, PyObject *args )
{
  char* funcName = "dmArrayCreateAxisGroup";

  PyObject* obj1    = NULL;
  PyObject* obj2    = NULL;
  PyObject* result  = NULL;

  char*     name    = NULL;
  char*     unit    = NULL;
  dmDataType dtype  = dmUNKNOWNTYPE;;
  dmDescriptor* result_dd;

  long      dim     = 0;
  char**    cptnames = NULL;

  PyArray_Descr* descr = NULL;
  PyArrayObject* tmparr = NULL;


  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!sO&sO&:dmArrayCreateAxisGroup", 
                                &dmDescriptor_Type, &obj1, 
                                &name,
                                &PyArray_DescrConverter2, &descr,
                                &unit,
                                &PyArray_Converter, &obj2)){ return NULL;}


  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  
  /* Process dtype argument */
  if ( descr != NULL )
  {
    /* validate input dtype */
    if ( !( PyDataType_ISNUMBER(descr)) )
    {
      /* problem */
      sprintf(errmsg, DMDESC_BAD_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    /* convert to DM datatype */
    dtype = cxc_get_dm_type( descr->type_num );
    if ( dtype == dmUNKNOWNTYPE )
    {
      /* problem */
      sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }
  }


  if ( obj2 != NULL )
  {
    /* This object contains the component names for a vector column */
    /* It MUST be a 1D string array */
    if ( (PyArray_TYPE( obj2 ) != NPY_STRING )||(PyArray_NDIM( obj2 ) > 1 ))
    {
      sprintf( errmsg, "%s() cptnames must be 1D string array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      dim = (long)PyArray_SIZE((PyObject*)obj2);
      cptnames = (char**)cxc_strings_to_carray( (PyArrayObject*)obj2 );
    }
  }

  result_dd = dmArrayCreateAxisGroup( ((pyDescriptor*)obj1)->dd, name, dtype, unit, cptnames, dim );
  if ( result_dd == NULL )
  {

    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = result_dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  /* Decrement results of PyArray_Converter */
  if ( obj2 ){ Py_DECREF(obj2);}

  return( result );

  fail:
  /* Decrement results of PyArray_Converter */
  if ( obj2 ){ Py_DECREF(obj2);}

   if ( cptnames ){ Py_XDECREF( tmparr );}
   return NULL;

}



/* ---------------------------------------------------------------------- */
/* Binding: _dmArrayGetAxisGroup                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmArrayGetAxisGroup(PyObject *self, PyObject *args )
{
  char* funcName = "dmArrayGetAxisGroup";

  long axisgroupno;
  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  dmDescriptor* _dd;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmArrayGetAxisGroup", &dmDescriptor_Type, &obj1, &axisgroupno)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    _dd = dmArrayGetAxisGroup( ((pyDescriptor*)obj1)->dd, axisgroupno );
    if ( _dd == NULL )
    {
      sprintf(errmsg, DMDESC_GENERIC_MSG ,funcName, "axis group not found on descriptor.");
      PyErr_SetString(PyExc_RuntimeError, errmsg );
      return NULL;
    }
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = _dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmArrayGetNoAxisGroups                                       */
/* ---------------------------------------------------------------------- */
static PyObject* _dmArrayGetNoAxisGroups(PyObject *self, PyObject *args )
{
  char* funcName = "dmArrayGetNoAxisGroups";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;

  long res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmArrayGetNoAxisGroups", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    res = dmArrayGetNoAxisGroups( ((pyDescriptor*)obj1)->dd );
  }

  result = PyLong_FromLong( res );
  return result;

}

/* ---------------------------------------------------------------------- */
/* Binding:  _dmDescriptorGetData                                                   */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetData(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmGetData";

  PyObject*      obj1  = NULL;
  long           rowno = -1;
  long           nrows = 1;
  long           nvals = 1;
  long           vdim  = 1;

  dmDescriptor*  dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  int            ptype;
  PyArray_Dims   dims = {NULL, 0};
  int            itemsize;
  dmDescriptorType ddtype;

  long  ii;
  long  nrowsgot = 0;

  void* data   = NULL;
  char*  buf   = NULL;
  char** cdata = NULL;
  dmBool* bdata= NULL;
  long  bdim = 0;
  PyArrayObject* res2 = NULL;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"dd", "rowno", "nrows", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|ll:dmGetData",
                                   kwlist,
                                   &dmDescriptor_Type, &obj1,
                                   &rowno,
                                   &nrows)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  
  dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( dd );
  ptype = cxc_get_py_type( dtype );
  cxc_get_dimarr( dd, &dims, &nrows );
  itemsize = dmDescriptorGetLength( dd );
  vdim = dmGetElementDim( dd );
  nvals = dmGetArraySize( dd );

  /* Process dtype argument */
  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, "dmUNKNOWNTYPE");
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  if ( rowno < 1 )
  {
    dmBlock *block = dmDescriptorGetBlock( dd );
    rowno = dmTableGetRowNo( block );
  }

  /* MCD NOTE: Much of the rest of this code is repeated in dmKeyRead               */
  /*           Could get consolidated if more usage of the array handling is needed */
  cdata = NULL;
  bdata = NULL;
 
  if ( dtype == dmTEXT )
  {
    /* Protect against empty string */
    if ( itemsize == 0 ){ itemsize++; }

    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, itemsize, 0, NULL );

    long cdim = 1;
    for (ii = 0; ii < dims.len; ii++ )
      cdim *= dims.ptr[ii];

    /* Create string array pointer for DM layer.. The data pointer in the array is just the underlying buffer */
    /* add one for terminator */

    buf = (char*)calloc( cdim * (itemsize + 1), sizeof(char));
    cdata = (char**)calloc( cdim, sizeof(char*) );

    cdata[0] = buf;
    for ( ii=1; ii < cdim; ii++){
      cdata[ii] = cdata[ii-1] + itemsize + 1;
    }

    data = (void*)cdata;
  }
  else if ( dtype == dmBOOL )
  {
    bdim = 1;
    for (ii = 0; ii < dims.len; ii++ )
      bdim *= dims.ptr[ii];

    /* Need separate memory for DM since it stores bool as different size */
    bdata  = (dmBool*)calloc( bdim, sizeof(dmBool) );
 
    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, 0, 0, NULL );
    data = (void*)bdata;

  }
  else
  {
    /* create array to hold value(s) */
    res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, dims.len, dims.ptr, ptype, NULL, NULL, itemsize, 0, NULL );
    data = PyArray_DATA(res2);
  }
  if ( res2 == NULL )
  {
    if (itemsize == 0)
      goto fail;

    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  /* Load value(s) for descriptor.. converting to requested datatype             */
  /*                                             vdim  row,   nrows, 1, nvals ); */
  nrowsgot = dmGetVectorArrays( dd, dtype, data, vdim, rowno, nrows, 1, nvals );

  if ( nrowsgot < 0 )
  {
    sprintf(errmsg, DMDESC_VALUES_ERR_MSG ,funcName, (char*)"getting");
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );    

    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }
  else if ( nrowsgot == 0 )
  {
    sprintf(errmsg, DMDESC_NOROWS_ERR_MSG ,funcName);
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }


  if ( dtype == dmTEXT )
  {
    long cdim = 1;
    for (ii = 0; ii < dims.len; ii++ ) {
      cdim *= dims.ptr[ii];
    }

    buf = PyArray_DATA(res2);
    char** tmpdata = (char**)calloc(cdim, sizeof(char*));

    tmpdata[0] = buf;
    strncpy(tmpdata[0], cdata[0], itemsize);

    for ( ii=1; ii < cdim; ii++){
      tmpdata[ii] = tmpdata[ii-1] + itemsize;
      strncpy(tmpdata[ii], cdata[ii], itemsize);
    }

  }
  if ( dtype == dmBOOL )
  {
    npy_bool *bbuf = (npy_bool *)(res2->data);
    dmBool *_bdata = (dmBool *)data;
    for (ii=bdim; ii--;)
      *(bbuf++) = (npy_bool)(*(_bdata++));
  }


  if ( dims.ptr )
    PyDimMem_FREE( dims.ptr );
 
 
  if ( cdata )
    free(cdata);
  else if ( bdata )
    free(bdata);

  return PyArray_Return( res2 );

 fail:
  if ( dims.ptr )
    PyDimMem_FREE( dims.ptr );
  if ( cdata )
    free(cdata);
  else if ( bdata )
    free(bdata);

  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding:  _dmDescriptorSetData                                         */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetData(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmSetData";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* tmp   = NULL;
  npy_intp*  dims  = NULL;
  long           rowno = -1;
  long           nrows = 1;
  long           nvals = 1;
  long           ndim = 1;
  long           vdim = 1; 
  long           dd_dim = 0;

  long           value_dim = 1;
  long           value_nvals = 0;
  long           ntotal = 0;

  dmDescriptor*   dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;

  long  ii;
  long  nret = 0;

  void* data   = NULL;
  char** cdata = NULL;
  dmBool *bdata = NULL;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"dd", "values", "rowno", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|l:dmSetData",
                                   kwlist,
                                   &dmDescriptor_Type, &obj1,
                                   &PyArray_Converter, &obj2,
                                   &rowno)){ return NULL;}

  
  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;
  ndim = dmGetArrayDim( dd );
  vdim = dmGetElementDim( dd );
  nvals = dmGetArraySize( dd );

  value_dim = PyArray_NDIM( obj2 );
  dims = PyArray_DIMS( obj2 );
  value_nvals = PyArray_SIZE( obj2 );
  dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );

  /* if the row number was set use the current row number */
  if ( rowno < 1 )
  {
    dmBlock *block = dmDescriptorGetBlock( dd );
    rowno = dmTableGetRowNo( block );
  }


  /* Make sure the dimensions match the descriptor */
  if ( vdim > 1 )
  {
    // add 1 dim for the vector
    dd_dim++;

    // This code assumes that all components will have the same shape
    // Therefore it is only necessary to check the first component.

    dmDescriptor *cpt = dmGetCpt(dd, 0);
    ndim = dmGetArrayDim( cpt );

    dd_dim += (int)ndim;
  }
  else if (ndim > 0)
    dd_dim += (int)ndim;

  if ( dd_dim + 1 == value_dim )
     nrows = dims[0];
  else if ( dd_dim == value_dim )
     nrows = 1;
  else
  {
    sprintf( errmsg, DMDESC_INVALID_DIM_MSG, funcName, (int)value_dim, (int)dd_dim );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }


  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, obj2->descr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }
  else if ( dtype == dmTEXT )
  {
    /* Text type.. python stores only enough space for string (no terminator).. */
    data = cxc_strings_to_carray( obj2 );
  }
  else if ( dtype == dmBOOL )
  {
    bdata  = (dmBool*)calloc( value_nvals, sizeof(dmBool) ); 
    dmBool *buf = bdata;

    npy_bool *bbuf = (npy_bool *)(PyArray_DATA( obj2 ));
    for (ii=value_nvals; ii--;)
       *(buf++) = (dmBool)(*(bbuf++));

    // create array to hold value(s)
    data = (void*)bdata;
  }
  else if ( (PyArray_TYPE(obj2) == NPY_INT )&&(dtype == dmLONG) )
  {
    /* For 64bit compatibility.  DM layer expects 8byte values for  */
    /* long type where here, we are getting 4byte integers          */
    /* If this condition is true on 32bit, it should cast to the    */
    /* the same 4byte size.                                         */
    tmp = (PyArrayObject*) PyArray_Cast(obj2, NPY_LONG);
    data = PyArray_DATA( tmp );
  }
  else
  {
    data = PyArray_DATA( obj2 );
  }


  if (dd_dim > 0)
  {
    ntotal = nrows * nvals * vdim;

    /* make sure there are the expected number of values per row 
         -- make sure number of values passed in is less than 
            the possible number of values for the given number of
            rows 
         -- for vectors make sure there are the same amount of data
            for each vector
    */
    if ( ( value_nvals > (unsigned long)ntotal ) ||
        ( ( value_nvals % (nrows*vdim)) != 0 ) )

    {
      /* problem */
      sprintf( errmsg, DMDESC_INVALID_VALUE_MSG, funcName, (int)value_nvals );
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    /* adjust nvals if there are less than the max number of values for the row */
    if ( value_nvals < (unsigned long)ntotal )
      nvals = value_nvals / (nrows*(dd_dim));

  }

  /* Load value(s) for descriptor.. converting to requested datatype */
  /*                                          vdim  row,   nrows, 1, nvals ); */

  nret  = dmSetVectorArrays( dd, dtype, data, vdim, rowno, nrows, 1, nvals );

  if ( nret < 0 )
  {
    sprintf(errmsg, DMDESC_VALUES_ERR_MSG ,funcName, (char*)"setting");
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );    

    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }
  else if ( nret == 0 )
  {
    sprintf(errmsg, DMDESC_NOROWS_ERR_MSG ,funcName);
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }

  if ( bdata ){ free (bdata); }
  if ( cdata ){ free(cdata); }
  if ( tmp ){ Py_DECREF(tmp);}

  /* Decrement results of PyArray_Converter */
  if ( obj2 ){ Py_DECREF(obj2);}


  Py_RETURN_NONE;

 fail:
  if ( cdata ){ free(cdata); }
  else if ( bdata ){ free(bdata); }
  if ( tmp ){ Py_DECREF(tmp);}

  /* Decrement results of PyArray_Converter */
  if ( obj2 ){ Py_DECREF(obj2);}

  return NULL;
}


/* ---------------------------------------------------------------------- */
/* Binding:  _dmDescriptorGetImageData                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorGetImageData(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmGetImageData";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;
  
  dmDescriptor*   dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  int            ptype;
  long           ndim;
  npy_intp*      dimarr = NULL;
  long*          pixelLL = NULL;
  long*          pixelUR = NULL;
  long           ndim_ll = 0;

  long  ii;
  void* data   = NULL;


  PyArrayObject* res2 = NULL;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"dd", "pixelLL", "pixelUR", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O&O&:dmGetImageData",
                                   kwlist,
                                   &dmDescriptor_Type, &obj1,
                                   &PyArray_Converter, &obj2,
                                   &PyArray_Converter, &obj3)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = dmGetDataType( dd );
  ptype = cxc_get_py_type( dtype );


  /* Process dtype argument */
  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, "dmUNKNOWNTYPE");
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  /* Check for null image (no values) */
  long nvals = 0;
  nvals = dmGetArraySize( dd );
  if ( nvals == 0 )
    Py_RETURN_NONE;

  ndim = dmGetArrayDim( dd );
  dimarr  = (npy_intp*)calloc( ndim, sizeof(npy_intp) );
  pixelLL = (long*)calloc( ndim, sizeof(long) );
  pixelUR = (long*)calloc( ndim, sizeof(long) );
  ndim = dmGetArrayDimensions( dd, (void *)(&pixelUR));

  for ( ii=ndim; ii--; )
    pixelLL[ii] = 1;

  if ( obj2 != NULL )
  {
    void *tmp_pixels = PyArray_DATA( obj2 );
    int array_type = PyArray_TYPE( obj2 );
    ndim_ll = PyArray_SIZE( obj2 );

    if (ndim_ll > ndim)
    {
      /* problem */
      sprintf(errmsg, DMDESC_INVALID_DIM_MSG,funcName, (int)ndim_ll, (int)ndim);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    switch (array_type){
    case NPY_LONG:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = ((long *)tmp_pixels)[ii];
      break;
    case NPY_ULONG:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = (long)(((unsigned long *)tmp_pixels)[ii]);
      break;
    case NPY_INT:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = (long)(((int *)tmp_pixels)[ii]);
      break;
    case NPY_UINT:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = (long)(((unsigned int *)tmp_pixels)[ii]);
      break;
    case NPY_SHORT:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = (long)(((short *)tmp_pixels)[ii]);
      break;
    case NPY_USHORT:
      for ( ii=ndim_ll; ii--;)
        pixelLL[ii] = (long)(((unsigned short *)tmp_pixels)[ii]);
      break;
    default:
      /* problem */
      sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, "dmUNKNOWNTYPE");
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
      break;
    }
  }

  if ( obj3 != NULL )
  {
    void *tmp_pixels = PyArray_DATA( obj3 );
    int array_type = PyArray_TYPE( obj3 );
    long ndim_ur = PyArray_SIZE( obj3 );

    if (ndim_ur > ndim)
    {
      /* problem */
      sprintf(errmsg, DMDESC_INVALID_DIM_MSG,funcName, (int)ndim_ur, (int)ndim);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    switch (array_type){
    case NPY_LONG:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = ((long *)tmp_pixels)[ii];
      break;
    case NPY_ULONG:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = (long)(((unsigned long *)tmp_pixels)[ii]);
      break;
    case NPY_INT:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = (long)(((int *)tmp_pixels)[ii]);
      break;
    case NPY_UINT:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = (long)(((unsigned int *)tmp_pixels)[ii]);
      break;
    case NPY_SHORT:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = (long)(((short *)tmp_pixels)[ii]);
      break;
    case NPY_USHORT:
      for ( ii=ndim_ur; ii--;)
        pixelUR[ii] = (long)(((unsigned short *)tmp_pixels)[ii]);
      break;
    default:
      /* problem */
      sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, "dmUNKNOWNTYPE");
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
      break;
    }
  }
  /* if the lower left was set but not the upper right use the lower left  */
  /* this allows for single pixel */
  else if (obj2)
  {
    for ( ii=ndim_ll; ii--;)
      pixelUR[ii] = pixelLL[ii];  
  }


  for ( ii=0; ii<ndim; ii++)
    dimarr[ndim-ii-1] = abs (pixelUR[ii]-pixelLL[ii]+1);


  /* create array to hold value(s) */
  res2 = (PyArrayObject*) PyArray_New(&PyArray_Type, ndim, dimarr, ptype, NULL, NULL, 0, 0, NULL );
  data = PyArray_DATA(res2);

  if ( res2 == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }


  /* Load value(s) for descriptor.. converting to requested datatype */
  int dmerror = dmp_image_get_subarray( dd, pixelLL, pixelUR, dtype, data );


  if (dmerror == dmFAILURE)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString (PyExc_MemoryError,errmsg);
    goto fail;
  }

  if ( pixelLL ){ free(pixelLL); }
  if ( pixelUR ){ free(pixelUR); }
  if ( dimarr ) { free (dimarr); }
  if ( obj2 )   { Py_DECREF(obj2);}
  if ( obj3 )   { Py_DECREF(obj3);}

  return PyArray_Return( res2 );

 fail:
  if ( pixelLL ){ free(pixelLL); }
  if ( pixelUR ){ free(pixelUR); }
  if ( dimarr ) { free (dimarr); }
  if ( obj2 )   { Py_DECREF(obj2);}
  if ( obj3 )   { Py_DECREF(obj3);}

  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding:  _dmDescriptorGetImageData                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetImageData(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmSetImageData";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;
  PyArrayObject* objll = NULL;
  PyArrayObject* tmp   = NULL;

  dmDescriptor*   dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  long           ndim;
  long*          pixelLL = NULL;
  long*          pixelUR = NULL;

  long  ii;
  void* data   = NULL;
  long  data_ndim = 0;
  npy_intp* data_dims = NULL;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"dd", "values", "pixelLL", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|O&:dmSetImageData",
                                   kwlist,
                                   &dmDescriptor_Type, &obj1,
                                   &PyArray_Converter, &obj2,
                                   &PyArray_Converter, &obj3)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  /*  dtype = dmGetDataType( dd ); */
  dtype = cxc_get_dm_type(PyArray_TYPE( obj2 ));

  /* Process dtype argument */
  if ( dtype == dmUNKNOWNTYPE )
  {
    /* problem */
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, "dmUNKNOWNTYPE");
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  ndim = dmGetArrayDim( dd );
  pixelLL = (long*)calloc( ndim, sizeof(long) );
  pixelUR = (long*)calloc( ndim, sizeof(long) );
  ndim = dmGetArrayDimensions( dd, (void *)(&pixelUR));

  for ( ii=ndim; ii--; )
    pixelLL[ii] = 1;


  data_ndim = PyArray_NDIM( obj2 );
  data_dims = PyArray_DIMS( obj2 );

  /* NOTE: we don't expect image data of the other volitile types */
  if ( (PyArray_TYPE(obj2) == NPY_INT )&&(dtype == dmLONG) )
  {
    /* For 64bit compatibility.  DM layer expects 8byte values for  */
    /* long type where here, we are getting 4byte integers          */
    /* If this condition is true on 32bit, it should cast to the    */
    /* the same 4byte size.                                         */
    tmp = (PyArrayObject*) PyArray_Cast(obj2, NPY_LONG);
    data = PyArray_DATA( tmp );
  }
  else
  {
    data = PyArray_DATA( obj2 );
  }

  if ( obj3 != NULL )
  {
    /* ensure argument is long type */
    objll = (PyArrayObject*)PyArray_Cast(obj3, NPY_LONG);

    void *tmp_pixels = PyArray_DATA( objll );
    long ndim_ll = PyArray_SIZE( obj3 );

    if (ndim_ll > ndim)
    {
      /* problem */
      sprintf(errmsg, DMDESC_INVALID_DIM_MSG,funcName, (int)ndim_ll, (int)ndim);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    /* Set lower left pixel boundary. */
    for ( ii = ndim_ll; ii--;)
      pixelLL[ii] = ((long *)tmp_pixels)[ii];

  }

  if (data_ndim > ndim)
  {
    /* problem */
    sprintf(errmsg, DMDESC_INVALID_DIM_MSG,funcName, (int)data_ndim, (int)ndim);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  for ( ii=ndim; ii--;)
  {
    if (ii < data_ndim)
       pixelUR[ii] = (pixelLL[ii]+data_dims[data_ndim-ii-1]-1);
    else
       pixelUR[ii] = pixelLL[ii];
  }


  /* Load value(s) for descriptor.. converting to requested datatype */
  int dmerror = dmp_image_set_subarray( dd, pixelLL, pixelUR, dtype, data );


  if (dmerror == dmFAILURE)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString (PyExc_MemoryError,errmsg);
    goto fail;
  }

  if ( pixelLL ){ free(pixelLL); }
  if ( pixelUR ){ free(pixelUR); }
  if ( obj2 )   { Py_DECREF(obj2);}
  if ( obj3 )   { Py_DECREF(obj3);}
  if ( objll )   { Py_DECREF(objll);}
  if ( tmp )   { Py_DECREF(tmp);}

  Py_RETURN_NONE;

 fail:
  if ( pixelLL ){ free(pixelLL); }
  if ( pixelUR ){ free(pixelUR); }
  if ( obj2 )   { Py_DECREF(obj2);}
  if ( obj3 )   { Py_DECREF(obj3);}
  if ( objll )   { Py_DECREF(objll);}
  if ( tmp )   { Py_DECREF(tmp);}

  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordCalc                                                  */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordCalc(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordCalc";

  PyObject*      obj1   = NULL;
  PyArrayObject* obj2   = NULL;

  int            ptype;
  PyArray_Dims   dims   = {NULL, 0};
  PyArrayObject* result = NULL;

  dmDescriptor*  _dd    = NULL;
  dmDataType     dtype  = dmUNKNOWNTYPE;
  dmErrCode      status;

  void*          ivalues = NULL;
  void*          ovalues = NULL;


  if (!PyArg_ParseTuple(args, "O!O&:dmCoordCalc", &dmDescriptor_Type, &obj1,
                                                  &PyArray_Converter, &obj2))
                                                  { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = cxc_get_dm_type( PyArray_TYPE(obj2) );
  ptype = cxc_get_py_type( dmDOUBLE );
  dims.ptr = PyArray_DIMS( obj2 );
  dims.len = PyArray_NDIM( obj2 );

  /* create array to hold output value(s) */
  result = (PyArrayObject*) PyArray_ZEROS(dims.len, dims.ptr, ptype, 0 ); 
  if ( result == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  ivalues = PyArray_DATA ( obj2 );
  ovalues = PyArray_DATA(result);

  status = dmi_coord_calc( _dd, ivalues, dtype, dmDOUBLE, ovalues );
  if (status == dmFAILURE)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  /* decrement result of PyArray_Converter */
  Py_DECREF( obj2 );

  return PyArray_Return( result );

 fail:

  /* must decrement result of PyArray_Converter */
  Py_DECREF( obj2 ); 

  if ( result )
    PyObject_Del ( result );

  return NULL;

}


/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordInvert                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordInvert(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordInvert";

  PyObject*      obj1   = NULL;
  PyArrayObject* obj2   = NULL;

  int            ptype;
  PyArray_Dims   dims = {NULL, 0};
  PyArrayObject* result = NULL;

  dmDescriptor*  _dd   = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  dmErrCode      status;

  void*          ivalues = NULL;
  void*          ovalues = NULL;


  if (!PyArg_ParseTuple(args, "O!O&:dmCoordInvert", &dmDescriptor_Type, &obj1,
                                                  &PyArray_Converter, &obj2))
                                                  { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  dtype = cxc_get_dm_type( PyArray_TYPE (obj2) );
  ptype = cxc_get_py_type( dmDOUBLE );
  dims.ptr = PyArray_DIMS( obj2 );
  dims.len = PyArray_NDIM( obj2 );

  /* create array to hold output value(s) */
  result = (PyArrayObject*) PyArray_ZEROS(dims.len, dims.ptr, ptype, 0 ); 
  if ( result == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  ivalues = PyArray_DATA ( obj2 );
  ovalues = PyArray_DATA(result);

  status = dmi_coord_invert( _dd, ivalues, dtype, dmDOUBLE, ovalues );
  if (status == dmFAILURE)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  Py_DECREF(obj2);

  return PyArray_Return( result );

 fail:
  Py_DECREF(obj2);

  if ( result )
    PyObject_Del ( result );

  return NULL;

}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordCreate                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordCreate(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmCoordCreate";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;
  PyArrayObject* obj4  = NULL;
  PyArrayObject* obj5  = NULL;
  PyArrayObject* obj6  = NULL;

  PyArrayObject* obj_crpix = NULL;
  PyArrayObject* obj_crval = NULL;
  PyArrayObject* obj_cdelt = NULL;
  PyArrayObject* obj_params = NULL;


  PyObject*      result = NULL;

  dmDescriptor*   dd = NULL;
  dmDescriptor*   coord_dd = NULL;

  char        *name;
  char        *unit;
  char        *transform = NULL;
  char        **cptNames = NULL;
  char        *default_transform = "LINEAR\0";

  double      *crpix = NULL;
  double      *crval = NULL;
  double      *cdelt = NULL;
  double      *params = NULL;

  long        vdim = 0;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"dd", "name", "unit", "cptnames", "transform", "crpix", "crval", "cdelt", "params", NULL};


  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!ss|O&sO&O&O&O&:dmCoordCreate",
                                   kwlist,
                                   &dmDescriptor_Type, &obj1,
                                   &name,
                                   &unit,
                                   &PyArray_Converter, &obj2,
                                   &transform,
                                   &PyArray_Converter, &obj3,
                                   &PyArray_Converter, &obj4,
                                   &PyArray_Converter, &obj5,
                                   &PyArray_Converter, &obj6)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;

  if ( obj2 != NULL )
  {
    /* This object contains the component names for a vector column */
    /* It MUST be a 1D string array */
    if ( (PyArray_TYPE( obj2 ) != NPY_STRING )||(PyArray_NDIM( obj2 ) > 1 ))
    {
      sprintf( errmsg, "%s() cptnames must be 1D string array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      cptNames = (char**)cxc_strings_to_carray( (PyArrayObject*)obj2 );
      if ( PyArray_NDIM( obj2 ) == 0 ){ vdim = 1; }
      else{ vdim = (long)PyArray_DIM( obj2 , 0);  }
    }
  }

  /* if any of the transform arguments exist and no cptnames
     exists then they should have a dimension temporarily 
     set it to -1 */
  if ((obj3 || obj4 || obj5) && (vdim == 0))
    vdim = -1;

  /* ensure that all transform arguments exist if the dimension
     is set */
  if ((vdim != 0) && (!obj3 || !obj4 || !obj5))
  {
    sprintf( errmsg, "%s() expected values for crpix, crvals and cdelt if any of them exists or if cptnames exists.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  

  if ( obj3 != NULL )
  {
    /* if the size of the array hasn't been set grab it now */
    if (vdim == -1)
       vdim = PyArray_SIZE( obj3 );

    /* This contains the crpix of the column..               */
    /* should be a 1D array of doubles..                       */
    if ( PyArray_NDIM( obj3 ) > 1 )
    {
      sprintf( errmsg, "%s() crpix must be scalar or 1D array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else if (PyArray_SIZE( obj3 ) != vdim)
    {
      sprintf( errmsg, "%s() crpix is expected to be of length %1ld.", funcName, vdim );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      obj_crpix = (PyArrayObject*)PyArray_Cast (obj3, NPY_DOUBLE);
      crpix = PyArray_DATA (obj_crpix);
    }
  }

  if ( obj4 != NULL )
  {
    /* if the size of the array hasn't been set grab it now */
    if (vdim == -1)
       vdim = PyArray_SIZE( obj4 );

    /* This contains the crval                                 */
    /* should be a 1D array of doubles..                       */
    if ( PyArray_NDIM( obj4 ) > 1 )
    {
      sprintf( errmsg, "%s() crval must be scalar or 1D array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else if (PyArray_SIZE( obj4 ) != vdim)
    {
      sprintf( errmsg, "%s() crval is expected to be of length %1ld.", funcName, vdim );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      obj_crval = (PyArrayObject*)PyArray_Cast (obj4, NPY_DOUBLE);
      crval = PyArray_DATA (obj_crval);
    }
  }

  if ( obj5 != NULL )
  {
    /* if the size of the array hasn't been set grab it now */
    if (vdim == -1)
       vdim = PyArray_SIZE( obj5 );

    /* This contains the cdelt                                 */
    /* should be a 1D array of doubles..                       */
    if ( PyArray_NDIM( obj5 ) > 1 )
    {
      sprintf( errmsg, "%s() cdelt must be scalar or 1D array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else if (PyArray_SIZE( obj5 ) != vdim)
    {
      sprintf( errmsg, "%s() cdelt is expected to be of length %ld.", funcName, vdim );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      obj_cdelt = (PyArrayObject*)PyArray_Cast (obj5, NPY_DOUBLE);
      cdelt = PyArray_DATA (obj_cdelt);
    }
  }

  if ( obj6 != NULL )
  {
    /* This contains the param                                 */
    /* should be a 1D array of doubles..                       */
    if ( PyArray_NDIM( obj6 ) > 1 )
    {
      sprintf( errmsg, "%s() param must be a 1D array.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      obj_params = (PyArrayObject*)PyArray_Cast (obj6, NPY_DOUBLE);
      params = PyArray_DATA (obj_params);
    }
  }

  if ((vdim > 1) && !cptNames)
  {
    sprintf( errmsg, "%s() for dimensions greater than 1, the cptnames must be supplied.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  

  if (!transform)
    transform = default_transform; 

  coord_dd = dmi_coord_create(dd, name, unit,
                             cptNames, vdim, transform,
                             (void *)crpix, (void *)crval, (void *)cdelt,
                             dmDOUBLE, params);

  if (!coord_dd)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  if (obj_crpix){ Py_DECREF ( obj_crpix ); }
  if (obj_cdelt){ Py_DECREF ( obj_cdelt ); }
  if (obj_crval){ Py_DECREF ( obj_crval ); }
  if (obj_params){ Py_DECREF ( obj_params );}

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = coord_dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  if ( obj2 ){ Py_DECREF( obj2 ); }
  if ( obj3 ){ Py_DECREF( obj3 ); }
  if ( obj4 ){ Py_DECREF( obj4 ); }
  if ( obj5 ){ Py_DECREF( obj5 ); }
  if ( obj6 ){ Py_DECREF( obj6 ); }

  return result;

 fail:
  if ( obj2 ){ Py_DECREF( obj2 ); }
  if ( obj3 ){ Py_DECREF( obj3 ); }
  if ( obj4 ){ Py_DECREF( obj4 ); }
  if ( obj5 ){ Py_DECREF( obj5 ); }
  if ( obj6 ){ Py_DECREF( obj6 ); }
  if (obj_crpix){ Py_DECREF ( obj_crpix ); }
  if (obj_cdelt){ Py_DECREF ( obj_cdelt ); }
  if (obj_crval){ Py_DECREF ( obj_crval ); }
  if (obj_params){ Py_DECREF ( obj_params ); }
  
  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordGetAxisGroupNo                                        */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordGetAxisGroupNo(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordGetAxisGroupNo";
  PyObject* obj1    = NULL;
  PyObject* result  = NULL;
  long _grpno = 0;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmCoordGetAxisGroupNo", &dmDescriptor_Type, &obj1)) { return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  _grpno = dmCoordGetAxisGroupNo( ((pyDescriptor*)obj1)->dd );
  result = Py_BuildValue("l", _grpno );

  return( result );
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordGetParams                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordGetParams(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordGetParams";

  PyObject* obj1 = NULL;
  PyArrayObject* result = NULL;

  dmDescriptor* _dd = NULL;
  double*    params = NULL;
  double*    values = NULL;
  long       nparam = 0;
  long       ii;
  npy_intp   dimarr[1];


  if (!PyArg_ParseTuple(args, "O!:dmCoordGetParams", &dmDescriptor_Type, &obj1)) { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  /* Pull out DM descriptor pointer */
  _dd = ((pyDescriptor*)obj1)->dd;

  params = dmCoordGetParams( _dd, &nparam );
  dimarr[0] = (npy_intp)nparam;

  /* create array to hold value(s) */
  result = (PyArrayObject*) PyArray_New(&PyArray_Type, 1, dimarr, NPY_DOUBLE, NULL, NULL, 0, 0, NULL );
  if ( result == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "params");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }
  values = PyArray_DATA(result);

  /* Transfer populated values.. DM returns params array that is 4 values long.. */
  for ( ii = 0; ii < nparam; ii++ )
    values[ii] = params[ii];

  if (params)
    free(params);

  return PyArray_Return( result );

 fail:
  if (params)
    free(params);

  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordGetTransform                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordGetTransform(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordGetTransform";

  PyObject* obj1    = NULL;

  PyArrayObject* crpix = NULL;
  PyArrayObject* crval = NULL;
  PyArrayObject* cdelt = NULL;


  dmDescriptor*  dd = NULL;
  int            ptype;
  npy_intp       dimarr[1];

  void*          crpix_values = NULL;
  void*          crval_values = NULL;
  void*          cdelt_values = NULL;

  if (!PyArg_ParseTuple(args, "O!:dmCoordGetTransform", &dmDescriptor_Type, &obj1)) { return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = ((pyDescriptor*)obj1)->dd;

  /* Initialize variables describing value(s) */
  ptype = cxc_get_py_type( dmDOUBLE );
  
  dimarr[0] = dmGetElementDim( dd );

  /* create array to hold value(s) */
  crpix = (PyArrayObject*) PyArray_New(&PyArray_Type, 1, dimarr, ptype, NULL, NULL, 0, 0, NULL );
  if ( crpix == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "crpix");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }
  crpix_values = PyArray_DATA(crpix);

  crval = (PyArrayObject*) PyArray_New(&PyArray_Type, 1, dimarr, ptype, NULL, NULL, 0, 0, NULL );
  if ( crval == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "crval");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }
  crval_values = PyArray_DATA(crval);

  cdelt = (PyArrayObject*) PyArray_New(&PyArray_Type, 1, dimarr, ptype, NULL, NULL, 0, 0, NULL );
  if ( cdelt == NULL )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "cdelt");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }
  cdelt_values = PyArray_DATA(cdelt);


  int dmerror = dmCoordGetTransform_d( dd, (double *)crpix_values, (double *)crval_values, (double *)cdelt_values, (long)dimarr[0]);

  if (dmerror == dmFAILURE)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString (PyExc_MemoryError,errmsg);
    goto fail;
  }

  return Py_BuildValue("OOO", PyArray_Return( crpix ), PyArray_Return( crval ), PyArray_Return( cdelt ));

 fail:

  if ( crpix )
    PyObject_Del ( crpix );
  if ( crval )
    PyObject_Del ( crval );
  if ( cdelt )
    PyObject_Del ( cdelt );

  return NULL;

}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordGetTransformType                                      */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordGetTransformType(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordGetTransformType";

  PyObject*     obj1 = NULL;
  dmDescriptor*  _dd = NULL;

  char ttype[dmMAXSTR];
  long nparams;
  dmErrCode status;

  if (!PyArg_ParseTuple(args, "O!:dmCoordGetTransformType", &dmDescriptor_Type, &obj1)) { return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  _dd = ((pyDescriptor*)obj1)->dd;

  status = dmCoordGetTransformType( _dd, ttype, &nparams, dmMAXSTR );
  if ( status == dmFAILURE) 
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString (PyExc_MemoryError,errmsg);
    return NULL;
  }

  return Py_BuildValue("Ol", PyString_FromString(ttype), nparams);
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordSetParam                                              */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordSetParam(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordSetParam";

  PyObject* obj1 = NULL;

  dmDescriptor* _dd = NULL;
  dmDescriptorType ddtype; 

  char*  pname;
  double dval;


  if (!PyArg_ParseTuple(args, "O!sd:dmCoordSetParam", &dmDescriptor_Type, &obj1, &pname, &dval ) )
  { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  /* Pull out DM descriptor pointer */
  _dd = ((pyDescriptor*)obj1)->dd;

  /* Verify that the descriptor is a coordinate */
  if ((ddtype = dmDescriptorGetType( _dd )) != dmCOORD)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmCOORD", 
             descriptor_type_as_string(ddtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  else
  {
    /* Call C function.. no possibility for failure */
    dmCoordSetParam( _dd, pname, dval );
  }

  Py_RETURN_NONE;

  fail:
  return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCoordSetTransform                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCoordSetTransform(PyObject *self, PyObject *args )
{
  char* funcName = "dmCoordSetTransform";

  PyObject* obj1    = NULL;
  PyObject* obj2    = NULL;
  PyObject* obj3    = NULL;
  PyObject* obj4    = NULL;
  PyArrayObject* dobj3 = NULL; 
  PyArrayObject* dobj4 = NULL; 

  void* crpix   = NULL;
  double* crval = NULL;
  double* cdelt = NULL;

  long           dim2, dim3, dim4;
  dmDataType     dtype = dmUNKNOWNTYPE;
  dmErrCode      status = dmFAILURE;

  if (!PyArg_ParseTuple(args, "O!O&O&O&:dmCoordSetTransform",
                        &dmDescriptor_Type, &obj1,
                        &PyArray_Converter, &obj2,    /* crpix */ 
                        &PyArray_Converter, &obj3,    /* crval */
                        &PyArray_Converter, &obj4))   /* cdelt */
  { return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  if (obj2 == NULL || obj3 == NULL || obj4 == NULL)  
  {
    sprintf( errmsg, "%s() the crpix, crval, and cdelt parameters must be arrays.", funcName);
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }

  /* only support numeric coord settings */
  dmDataType typ2 = cxc_get_dm_type( PyArray_TYPE( obj2 )),
             typ3 = cxc_get_dm_type( PyArray_TYPE( obj3 )),
             typ4 = cxc_get_dm_type( PyArray_TYPE( obj4 ));

  if (typ2 == dmTEXT)
    sprintf( errmsg, "%s() argument 2 (crpix) must be a numeric scalar or array, not a string type.", funcName);
  else if (typ2 == dmBOOL)
    sprintf( errmsg, "%s() argument 2 (crpix) must be a numeric scalar or array, not a boolean type .", funcName);
  else if (typ2 == dmUNKNOWNTYPE)
    sprintf( errmsg, "%s() argument 2 (crpix) must be a numeric scalar or array.", funcName);
  else if (typ3 == dmTEXT)
    sprintf( errmsg, "%s() argument 3 (crval) must be a numeric scalar or array, not a string type.", funcName);
  else if (typ3 == dmBOOL)
    sprintf( errmsg, "%s() argument 3 (crval) must be a numeric scalar or array, not a boolean type.", funcName);
  else if (typ3 == dmUNKNOWNTYPE)
    sprintf( errmsg, "%s() argument 3 (crval) must be a numeric scalar or array.", funcName);
  else if (typ4 == dmTEXT)
    sprintf( errmsg, "%s() argument 4 (cdelt) must be a numeric scalar or array, not a string type.", funcName);
  else if (typ4 == dmBOOL)
    sprintf( errmsg, "%s() argument 4 (cdelt) must be a numeric scalar or array, not a boolean type.", funcName);
  else if (typ4 == dmUNKNOWNTYPE)
    sprintf( errmsg, "%s() argument 4 (cdelt) must be a numeric scalar or array.", funcName);
  else 
    status = dmSUCCESS;

  if ( status != dmSUCCESS )
  {
    PyErr_SetString(PyExc_TypeError, errmsg );
    goto fail;
  }   

  dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );
  crpix = PyArray_DATA( obj2 );
  dim2 = (long)PyArray_SIZE((PyObject*)obj2);
  dobj3 = (PyArrayObject*)PyArray_Cast( (PyArrayObject*)obj3, NPY_DOUBLE ); 
  crval = PyArray_DATA( dobj3 );
  dim3 = (long)PyArray_SIZE((PyObject*)dobj3);
  dobj4 = (PyArrayObject*)PyArray_Cast( (PyArrayObject*)obj4, NPY_DOUBLE );
  cdelt = PyArray_DATA( dobj4 );
  dim4 = (long)PyArray_SIZE((PyObject*)dobj4);
  
  if ((dim2 != dim3) || (dim3 != dim4))
  {
    sprintf( errmsg, "%s() crpix, crval, and cdelt arguments must be the same length.", funcName);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }
  if (dim2 < 1)
  {
    sprintf( errmsg, "%s() crpix, crval, and cdelt arguments must contain at least 1 value.", funcName);
    PyErr_SetString(PyExc_ValueError, errmsg );
    goto fail;
  }

  status = dmi_coord_set_transform(((pyDescriptor*)obj1)->dd, (void*)crpix, dtype, crval, cdelt, dim2);

  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    goto fail;
  }

  Py_DECREF(obj2);
  Py_DECREF(obj3);
  Py_DECREF(obj4);

  Py_RETURN_NONE;

  fail:
   Py_DECREF(obj2);
   Py_DECREF(obj3);
   Py_DECREF(obj4);
   return NULL;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmGetDisp                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetDisp(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetDisp";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;
  dmErrCode status;
  char _cstr[dmMAXSTR+1];

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmGetDisp", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmGetDisp( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( _cstr );
  }
  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSetDisp                                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSetDisp(PyObject *self, PyObject *args )
{
  char* funcName = "dmSetDisp";

  PyObject* obj1    = NULL;

  dmErrCode status;
  char* cstr;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!s:dmSetDisp", &dmDescriptor_Type, &obj1, &cstr ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSetDisp( ((pyDescriptor*)obj1)->dd, cstr );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
  }
  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorSetNull                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorSetNull(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorSetNull";

  PyObject* obj1    = NULL;
  PyObject* obj2    = NULL;

  dmDataType dtype  = dmUNKNOWNTYPE;
  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O&:dmDescriptorSetNull", &dmDescriptor_Type, &obj1, &PyArray_Converter, &obj2 ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* get data type */ 
  void* value = PyArray_DATA( obj2 );
  dtype = cxc_get_dm_type(PyArray_TYPE( obj2 ));

  switch ( dtype )
  {
    case dmTEXT:
    case dmSHORT:
    case dmLONG:
    case dmFLOAT:
    case dmDOUBLE:
    case dmBYTE:
    case dmUSHORT:
    case dmULONG:
      status = dmp_descriptor_set_null(((pyDescriptor*)obj1)->dd, dtype, (void*)value);
    break;
    default:
      sprintf( errmsg, "%s() argument 2 is not a supported type.", funcName );
      PyErr_SetString(PyExc_TypeError, errmsg );
      goto fail;
    break;
  }

  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    return NULL;
  }

  Py_DECREF(obj2);
  Py_RETURN_NONE;

 fail:
  Py_DECREF(obj2);
  return NULL;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSetCptName                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSetCptName(PyObject *self, PyObject *args )
{
  char* funcName = "dmSetCptName";

  PyObject* obj1    = NULL;
  long dim, cptno;
  char* cstr;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!ls:dmSetCptName", &dmDescriptor_Type, &obj1, &cptno, &cstr ))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dim = dmGetElementDim(((pyDescriptor*)obj1)->dd);

  if (cptno > dim)
  {
    sprintf( errmsg, "%s() unable to set component number %1ld of descriptor with dimensionality of %1ld.", funcName, cptno, dim );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else if (cptno < 1)
  {
    sprintf( errmsg, "%s() the component number value %1ld must be at least 1.", funcName, cptno);
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dmSetCptName(((pyDescriptor*)obj1)->dd, cptno, cstr );

  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmGetCptName                                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmGetCptName(PyObject *self, PyObject *args )
{
  char* funcName = "dmGetCptName";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;
  dmErrCode status;
  char _cstr[dmMAXSTR+1];
  long dim, cptno;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!l:dmGetCptName", &dmDescriptor_Type, &obj1, &cptno)){ return NULL; }

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dim = dmGetElementDim(((pyDescriptor*)obj1)->dd);

  if (cptno > dim)
  {
    sprintf( errmsg, "%s() unable to get component number %1ld of descriptor with dimensionality of %1ld.", funcName, cptno, dim );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else if (cptno < 1)
  {
    sprintf( errmsg, "%s() the component number value %1ld must be at least 1.", funcName, cptno);
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  status = dmGetCptName( ((pyDescriptor*)obj1)->dd, cptno, _cstr, dmMAXSTR );
  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    return NULL;
  }
  result = PyString_FromString( _cstr );

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetSubspace                                           */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetSubspace(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetSubspace";

  PyObject* obj1   = NULL;
  PyObject* col    = NULL;
  PyObject* result = NULL;

  dmDescriptor** collist = NULL;
  dmDescriptor*  _col    = NULL;

  long  ii, ncols, ncpts;
  Py_ssize_t len, ndx;
  
  if (!PyArg_ParseTuple(args, "O!:dmBlockGetSubspace", &dmBlock_Type, &obj1)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  collist = dmBlockGetSubspace( ((pyBlock*)obj1)->block, &ncols, &ncpts );
  if ( collist == NULL )
  {
    /* problem */
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_TypeError, errmsg );
  }
  else
  {
    /* Create a Python list to hold column dds */
    len = (Py_ssize_t)ncols;
    result = PyList_New(len);
    if ( result == NULL )
    {
      sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "Column List");
      PyErr_SetString(PyExc_MemoryError, errmsg );
    }
    else
    {
      /* Transfer DM columns to Python list of Python cols. */
      for ( ii = 0; ii < ncols; ii++ )
      {
        _col = collist[ii];
        col  = pyDescriptor_new();
        ((pyDescriptor*)col)->dd = _col;
     
        /* Register this object with memory management */
        cxcdm_memory_register( col );

        ndx = (Py_ssize_t)ii;
        PyList_SET_ITEM( result, ndx, col );
      }
    }
    /* DM allocated the list ptrs.. no longer needed */
    free(collist);
  }

  return Py_BuildValue("llO", ncpts, ncols, result);
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmBlockGetSubspaceCol                                        */
/* ---------------------------------------------------------------------- */
static PyObject* _dmBlockGetSubspaceCol(PyObject *self, PyObject *args )
{
  char* funcName = "dmBlockGetSubspaceCol";

  PyObject* obj1 = NULL;
  PyObject* obj2 = NULL;

  long  colno;
  char* colname;
  
  dmDescriptor* _col   = NULL;
  PyObject*     result = NULL;

  if (!PyArg_ParseTuple(args, "O!O:dmBlockGetSubspaceCol", &dmBlock_Type, &obj1, &obj2)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Validate second input, and execute */
  if ( PyInt_Check( obj2 ) )
  {
    colno =  PyInt_AsLong( obj2 );
    if ( colno > 0 )
    {
      _col  =  dmBlockGetSubspaceColNo( ((pyBlock*)obj1)->block, colno );
    }
    else
    {
      sprintf(errmsg, DMDESC_INVALID_ARGVAL_MSG ,funcName, 2L );
      PyErr_SetString(PyExc_ValueError, errmsg );
      return NULL;
    }
  }
  else if ( PyString_Check( obj2 ) )
  {
    colname = PyString_AsString( obj2 );
    _col = dmSubspaceColOpen( ((pyBlock*)obj1)->block, colname );
  }
  else
  {
    /* problem */
    sprintf(errmsg, DMDESC_INVALID_ARGTYP_MSG, funcName, 2L, (char*)"int or string", obj2->ob_type->tp_name);
    PyErr_SetString(PyExc_TypeError, errmsg );
    return NULL;
  }

  if ( _col == NULL )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    if(strlen(errmsg)==0 || errmsg[0]==' '){
      char emsg[120];
      if(PyInt_Check( obj2 ))
        sprintf(emsg,"unable to open column No'%li' from input block.", colno);
      else
        sprintf(emsg,"unable to open column '%s' from input block.", colname);
      sprintf(errmsg, DMDESC_GENERIC_MSG ,funcName, emsg);
    }
    PyErr_SetString(PyExc_RuntimeError, errmsg );
  }
  else
  {
    /* All is well */
    result = pyDescriptor_new();
    ((pyDescriptor*)result)->dd = _col;

    /* Register this object with memory management */
    cxcdm_memory_register( result );
  }

   return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColGetTableName                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColGetTableName(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColGetTableName";

  PyObject* obj1    = NULL;
  dmDescriptorType dtype; 
  dmErrCode status;
  char _cstr[dmMAXSTR+1];

  PyObject* result  = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmSubspaceColGetTableName", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((dtype = dmDescriptorGetType(((pyDescriptor*)obj1)->dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(dtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSubspaceColGetTableName( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }
    result = PyString_FromString( _cstr );
  }

  return result;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColGetRegion                                       */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColGetRegion(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColGetRegion";

  PyObject* obj1    = NULL;
  PyObject* result  = NULL;
  dmDescriptorType dtype; 

  regRegion*   _reg = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmSubspaceColGetRegion", &dmDescriptor_Type, &obj1)){return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((dtype = dmDescriptorGetType(((pyDescriptor*)obj1)->dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(dtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    _reg = dmSubspaceColGetRegion( ((pyDescriptor*)obj1)->dd );
    if ( _reg != NULL )
    {
      result = pyRegion_New( _reg );
    }
  }
  if ( result == NULL )
  {
    Py_RETURN_NONE;
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColGetRegionString                                 */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColGetRegionString(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColGetRegionString";
  char _cstr[dmMAXSTR+1];
  PyObject* obj1    = NULL;
  PyObject* result  = NULL;
  dmDescriptorType dtype; 

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!:dmSubspaceColGetRegionString", &dmDescriptor_Type, &obj1)){ return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((dtype = dmDescriptorGetType(((pyDescriptor*)obj1)->dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(dtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    /* don't care about return status- dm returns 'DEFAULT' when failures */
    dmSubspaceColGetRegionString( ((pyDescriptor*)obj1)->dd, _cstr, dmMAXSTR );
    result = PyString_FromString( _cstr );
  }

  return result;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColSet                                             */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColSet(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColSet";

  PyObject* obj1 = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;

  void* mins = NULL;
  void* maxs = NULL;

  long nvals = 0;
  dmDescriptorType ddt; 
  dmDescriptor*    _dd = NULL; 
  dmDataType       dtype = dmUNKNOWNTYPE;
  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O&O&:dmSubspaceColSet", &dmDescriptor_Type, &obj1, 
			                                 &PyArray_Converter, &obj2,
			                                 &PyArray_Converter, &obj3))
    {return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  _dd = ((pyDescriptor*)obj1)->dd;
  if ( _dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((ddt = dmDescriptorGetType(_dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(ddt) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  nvals = PyArray_SIZE( obj2 );
  if ( nvals  != PyArray_SIZE( obj3 ) )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "input arrays are not the same length." );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );
  /* Supported types = dmSHORT, dmUSHORT, dmLONG, dmULONG, dmFLOAT, dmDOUBLE */
  /* NOT: dmTEXT, dmBYTE, dmBOOL, dmBIT */
  if ( (dtype == dmUNKNOWNTYPE ) || (dtype == dmTEXT) || (dtype == dmBYTE) ||
       (dtype == dmBOOL) || (dtype == dmBIT) )
  {
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, obj2->descr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    return NULL;
  }
  if ( dtype  != cxc_get_dm_type(PyArray_TYPE( obj3 )) )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "input arrays are not the same data type." );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  mins = PyArray_DATA( obj2 );
  maxs = PyArray_DATA( obj3 );

  status = dmi_subspace_col_update_null( _dd, NULL, 1L, dtype, mins, maxs, nvals, dmFALSE);
  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    return NULL;
  }
  
  Py_DECREF(obj2);
  Py_DECREF(obj3);
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColSetRegion                                       */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColSetRegion(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColSetRegion";

  PyObject*  obj1 = NULL;
  PyObject*  obj2 = NULL;

  dmDescriptorType dtype; 
  dmDescriptor* _dd = NULL;
  regRegion*   _reg = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!:dmSubspaceColSetRegion", &dmDescriptor_Type, &obj1, &Region_Type, &obj2))
    {return NULL;}

  _dd  = ((pyDescriptor*)obj1)->dd;
  _reg = ((pyRegion*)obj2)->reg;

  /* Check underlying dmDescriptor pointer is available */
  if ( _dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  /* Check underlying Region pointer is available */
  if ( _reg == NULL )
  {
    sprintf( errmsg, "%s() operation on empty Region pointer.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  /* Check that input descriptor is a subspace descriptor */
  dtype = dmDescriptorGetType(_dd);
  if ( dtype != dmSUBSPACE )
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(dtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Set region to descriptor */
  dmSubspaceColSetRegion( _dd, _reg  );

  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColSetTableName                                    */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColSetTableName(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColSetTableName";

  PyObject* obj1 = NULL;
  char* cstr = NULL;

  dmDescriptorType dtype; 
  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!s:dmSubspaceColSetTableName", &dmDescriptor_Type, &obj1, &cstr))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((dtype = dmDescriptorGetType(((pyDescriptor*)obj1)->dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(dtype) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else if ( strcmp(cstr,"") == 0 )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "table name can not be blank (\"\")");
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    status = dmSubspaceColSetTableName( ((pyDescriptor*)obj1)->dd, cstr );
    if ( status != dmSUCCESS )
    {
      cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
      PyErr_SetString(PyExc_RuntimeError, errmsg);
      return NULL;
    }

  }
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmDescriptorCompare                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmDescriptorCompare(PyObject *self, PyObject *args )
{
  char* funcName = "dmDescriptorCompare";

  PyObject* obj1 = NULL;
  PyObject* obj2 = NULL;
  PyObject* result = NULL;
  unsigned long mask; 
  dmBool res;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!k:dmDescriptorCompare", &dmDescriptor_Type, &obj1, &dmDescriptor_Type, &obj2, &mask))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ( ((pyDescriptor*)obj2)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  else if ( mask == 0 )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "Expected comparison mask field to be non-zero");
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  else
  {
    res = dmDescriptorCompare( ((pyDescriptor*)obj1)->dd, ((pyDescriptor*)obj2)->dd, mask );
    result = PyBool_FromLong( (long)res );

    return result;
  }
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmCopyGeneric                                                */
/* ---------------------------------------------------------------------- */
static PyObject* _dmCopyGeneric(PyObject *self, PyObject *args )
{
  char* funcName = "dmCopyGeneric";
  PyObject *obj1 = NULL, *obj2 = NULL;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O!:dmCopyGeneric", &dmDescriptor_Type, &obj1, &dmDescriptor_Type, &obj2))
    { return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  if ( ((pyDescriptor*)obj1)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ( ((pyDescriptor*)obj2)->dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dmCopyGeneric(((pyDescriptor*)obj1)->dd, ((pyDescriptor*)obj2)->dd);
  Py_RETURN_NONE;
}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColUpdate                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColUpdate(PyObject *self, PyObject *args )
{
  char* funcName = "dmSubspaceColUpdate";

  PyObject* obj1 = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;

  void* mins = NULL;
  void* maxs = NULL;

  long nvals = 0;
  dmDescriptorType ddt; 
  dmDescriptor*    _dd = NULL; 
  dmDataType       dtype = dmUNKNOWNTYPE;
  dmErrCode status;

  /* Parse and verify input arguments */
  if (!PyArg_ParseTuple(args, "O!O&O&:dmSubspaceColUpdate", &dmDescriptor_Type, &obj1, 
			                                 &PyArray_Converter, &obj2,
			                                 &PyArray_Converter, &obj3))
    {return NULL;}

  /* Check underlying dmDescriptor pointer is available */
  _dd = ((pyDescriptor*)obj1)->dd;
  if ( _dd == NULL )
  {
    sprintf( errmsg, DMDESC_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }
  if ((ddt = dmDescriptorGetType(_dd)) != dmSUBSPACE)
  {
    sprintf( errmsg, DMDESC_BAD_DESCTYPE_MSG, funcName, "dmSUBSPACE", 
             descriptor_type_as_string(ddt) );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  nvals = PyArray_SIZE( obj2 );
  if ( nvals  != PyArray_SIZE( obj3 ) )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "input arrays are not the same length." );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  dtype = cxc_get_dm_type( PyArray_TYPE( obj2 ) );
  /* Supported types = dmSHORT, dmUSHORT, dmLONG, dmULONG, dmFLOAT, dmDOUBLE */
  /* NOT: dmTEXT, dmBYTE, dmBOOL, dmBIT */
  if ( (dtype == dmUNKNOWNTYPE ) || (dtype == dmTEXT) || (dtype == dmBYTE) ||
       (dtype == dmBOOL) || (dtype == dmBIT) )
  {
    sprintf(errmsg, DMDESC_UNK_DTYPE_MSG ,funcName, obj2->descr->typeobj->tp_name);
    PyErr_SetString(PyExc_ValueError, errmsg );
    return NULL;
  }
  if ( dtype  != cxc_get_dm_type(PyArray_TYPE( obj3 )) )
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "input arrays are not the same data type." );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  mins = PyArray_DATA( obj2 );
  maxs = PyArray_DATA( obj3 );

  status = dmi_subspace_col_update_null( _dd, NULL, 1L, dtype, mins, maxs, nvals, dmTRUE);
  if ( status != dmSUCCESS )
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg);
    return NULL;
  }
  
  Py_DECREF(obj2);
  Py_DECREF(obj3);
  Py_RETURN_NONE;
}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColRead                                            */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColRead(PyObject *self, PyObject *args )
{
  char *funcName = "dmSubspaceColRead", *name = NULL;

  PyObject *obj1 = NULL, *result = NULL;

  PyArrayObject* res_min = NULL;
  PyArrayObject* res_max = NULL;

  dmDescriptor*  dd = NULL;
  dmDataType     dtype = dmUNKNOWNTYPE;
  int            ptype;
  npy_intp       dimarr[1];

  char*          dptr = NULL;
  void*          mins = NULL;
  void*          maxes = NULL;
  long           nvals = 0;
  long           size;

  if (!PyArg_ParseTuple(args, "O!s:dmSubspaceColRead", &dmBlock_Type, &obj1, &name)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    return NULL;
  }

  /* Initialize variables describing value(s) */
  if ((dd = dmSubspaceColOpen(((pyBlock*)obj1)->block, name)) == NULL)
  {
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 0 );
    PyErr_SetString(PyExc_RuntimeError, errmsg );
    return NULL; 
  }

  dtype = dmGetDataType( dd );
  ptype = cxc_get_py_type( dtype );

  int dmerror = dmi_subspace_col_get_values(dd, 1, dtype, &mins, &maxes, &nvals);
  if (dmerror == dmFAILURE)
  {
    sprintf(errmsg, DMDESC_VALUES_ERR_MSG ,funcName, (char*)"getting");
    cxc_get_dm_errmsg( errmsg, dmMAXSTR, 1 );

    PyErr_SetString(PyExc_RuntimeError, errmsg );
    goto fail;
  }

  /* Create arrays owned by python to hold values from DM */
  dimarr[0] = nvals;
  res_min = (PyArrayObject*) PyArray_ZEROS(1, dimarr, ptype, 0 );
  res_max = (PyArrayObject*) PyArray_ZEROS(1, dimarr, ptype, 0 ); 

  if ( (res_min == NULL)||(res_max == NULL) )
  {
    sprintf(errmsg, DMDESC_ALLOC_ERR_MSG ,funcName, "value(s)");
    PyErr_SetString(PyExc_MemoryError, errmsg );
    goto fail;
  }

  /* copy */
  size = (long)PyArray_NBYTES(res_min);

  dptr = (char*)PyArray_DATA(res_min);
  memcpy( dptr, (char*)mins, size );

  dptr = (char*)PyArray_DATA(res_max);
  memcpy( dptr, (char*)maxes, size );

  /* Free DM allocated memory */
  if (mins){ free(mins); }
  if (maxes){ free(maxes); }

  /* Register dmDescriptor object with memory management */
  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = dd;
  cxcdm_memory_register( result );

  return Py_BuildValue("OOO", result, PyArray_Return( res_min ), PyArray_Return( res_max ));

 fail:
  if (res_min)
    PyObject_Del (res_min);
  else if (mins)
    free (mins);
  if (res_max)
    PyObject_Del (res_max);
  else if (maxes)
    free (maxes);

  return NULL;

}

/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColCreate                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColCreate(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmSubspaceColCreate";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyArrayObject* obj3  = NULL;
  PyArrayObject* obj4  = NULL;
  PyObject*      result = NULL;

  dmDataType     dtype = dmUNKNOWNTYPE;
  dmDescriptor*  dd = NULL;
  dmBlock*       block = NULL; 
  char          *tabname = NULL; 
  char          *name = NULL;
  char          *unit = NULL;
  char         **colNames = NULL;
  void          *mins = NULL;
  void          *maxes = NULL;
  long           nvals = 0;
  long           vdim = 1;
  regRegion     *region = NULL;
  long           valdim = 1;
  int            array  = 0;

  /* Parse and verify input arguments */
  static char *kwlist[] = {"block", "tabname", "name", "colnames", "unit", "mins", "maxes", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!zsO&sO&O&:dmSubspaceColCreate",
                                   kwlist,
                                   &dmBlock_Type, &obj1,
                                   &tabname,
                                   &name,
                                   &PyArray_Converter, &obj2,
                                   &unit,
                                   &PyArray_Converter, &obj3,
                                   &PyArray_Converter, &obj4)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  block = ((pyBlock*)obj1)->block;

  if ( (obj2 != NULL) && (PyArray_SIZE( obj2 ) > 1) )
  {
    if ( (PyArray_TYPE( obj2 ) != NPY_STRING )||(PyArray_NDIM( obj2 ) > 1 )||(PyArray_SIZE( obj2 ) != 2))
    {
      sprintf( errmsg, "%s() colnames must be a 1D string array containing two elements.", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      colNames = (char**)cxc_strings_to_carray( (PyArrayObject*)obj2 );
    }
  }

  if ( obj3 == NULL )
  {
    sprintf( errmsg, DMDESC_INVALID_NULL_MSG, funcName, "mins array" );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;       
  }
  else if ( PyArray_NDIM( obj3 ) > 1 )
  {
    sprintf( errmsg, "%s() mins must be scalar or 1D array.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  else if ( obj4 == NULL )
  {
    sprintf( errmsg, DMDESC_INVALID_NULL_MSG, funcName, "maxes array" );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  else if ( PyArray_NDIM( obj4 ) > 1 )
  {
    sprintf( errmsg, "%s() maxes must be scalar or 1D array.", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }
  nvals = PyArray_SIZE( obj3 );
  if ( PyArray_SIZE( obj4 ) != nvals)
  {
    sprintf( errmsg, DMDESC_GENERIC_MSG, funcName, "mins and maxes arrays must contain the same number of entries" );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  } 

  mins  = PyArray_DATA( obj3 );
  maxes = PyArray_DATA( obj4 );

  dtype = cxc_get_dm_type(PyArray_TYPE( obj3 ));
 
  switch ( dtype)
  {
    case dmSHORT:
    case dmLONG:
    case dmFLOAT:
    case dmDOUBLE:
    case dmUSHORT:
    case dmULONG:


    if ( tabname && colNames )
    {
     /* Create subspace Table with scalar columns */
      vdim = 1;
    }
    else if ( !tabname && colNames )
    {
     /* Create vector subspace column (for region) */
      vdim = 2;
    }

    /* Bug 13683 - need to go down another layer in order to create  */
    /*  subspace column for Vector on which to hang a Region.        */

    dd = dmi_subspace_col_create( block, NULL, name, colNames, vdim, dtype, unit, NULL, 1 );
    dmi_subspace_col_set_values( dd, tabname, mins, maxes, nvals, dtype, valdim, array, region );

    break;
    default:
      sprintf( errmsg, "%s() mins or maxes argument is not a supported type.", funcName );
      PyErr_SetString(PyExc_TypeError, errmsg );
      goto fail;
    break;
  }

  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  if (obj2)
  {
    PyObject_Del( obj2 );
    if (colNames)
    {
      free( ((char**)colNames)[0] );
      free( (char**)colNames );
      colNames = NULL;
    }
  }

  return( result );

 fail:

  if (obj2)
  {
    PyObject_Del( obj2 );
    if (colNames)
    {
      free( ((char**)colNames)[0] );
      free( (char**)colNames );
      colNames = NULL; 
    }
  }
  if (obj3)
    PyObject_Del( obj3 );
  if (obj4)
    PyObject_Del( obj4 );
  
  return NULL;

}


/* ---------------------------------------------------------------------- */
/* Binding: _dmSubspaceColCreate                                          */
/* ---------------------------------------------------------------------- */
static PyObject* _dmSubspaceColCreateRegion(PyObject *self, PyObject *args, PyObject *kwds )
{
  char* funcName = "dmSubspaceColCreateRegion";

  PyObject*      obj1  = NULL;
  PyArrayObject* obj2  = NULL;
  PyObject*      obj3  = NULL;
  PyObject*      result = NULL;
  PyArray_Descr* descr = NULL;

  dmDataType     dtype = dmUNKNOWNTYPE;
  dmDescriptor*  dd = NULL;
  dmBlock*       block = NULL; 

  char          *unit = NULL; 
  char          *name = NULL;
  char         **colNames = NULL;
  regRegion*     reg = NULL; 

  /* Parse and verify input arguments */
  static char *kwlist[] = {"block", "name", "colnames", "dtype", "unit", "region", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!sO&O&sO:dmSubspaceColCreateRegion",
                                   kwlist,
                                   &dmBlock_Type, &obj1,
                                   &name,
                                   &PyArray_Converter, &obj2,
                                   &PyArray_DescrConverter2, &descr,
                                   &unit,
                                   &obj3)){ return NULL;}

  /* Check underlying dmBlock pointer is available */
  if ( ((pyBlock*)obj1)->block == NULL )
  {
    sprintf( errmsg, DMBLOCK_EMPTY_PTR_MSG, funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  block = ((pyBlock*)obj1)->block;

  /* Process dtype argument */
  if ( descr != NULL )
  {
    /* validate input dtype */
    if ( !( PyDataType_ISNUMBER(descr)) )
    {
      /* problem */
      sprintf(errmsg, DMBLOCK_BAD_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }

    /* convert to DM datatype */
    dtype = cxc_get_dm_type( descr->type_num );
    if ( dtype == dmUNKNOWNTYPE )
    {
      /* problem */
      sprintf(errmsg, DMBLOCK_UNK_DTYPE_MSG ,funcName, descr->typeobj->tp_name);
      PyErr_SetString(PyExc_ValueError, errmsg );
      goto fail;
    }
  }

  if (obj2 != NULL)
  {
    /* This object contains the column names for a vector column */
    /* It MUST be a 1D string array */
    if ( (PyArray_TYPE( obj2 ) != NPY_STRING )||(PyArray_NDIM( obj2 ) > 1 ))
    {
      sprintf( errmsg, "%s() colnames must be a 1D string array", funcName );
      PyErr_SetString(PyExc_ValueError, errmsg);
      goto fail;
    }
    else
    {
      colNames = (char**)cxc_strings_to_carray( (PyArrayObject*)obj2 );
    }
  }

  if( PyObject_TypeCheck( obj3, &Region_Type) )
  {
    /* Object is a region struct */
    reg = ((pyRegion*)obj3)->reg;
  }
  else if ( PyString_Check( obj3))
  {
    /* Object is a string- convert object and make regRegion */
    char *regstr = PyString_AsString(obj3);  
    reg = regParse(regstr); 
  }                                   
  else 
  {
    sprintf( errmsg, "%s() region must be either a regRegion struct or region string", funcName );
    PyErr_SetString(PyExc_ValueError, errmsg);
    goto fail;
  }

  dd = dmSubspaceColCreateRegion( block, name, colNames, dtype, unit, reg);
 
  result = pyDescriptor_new();
  ((pyDescriptor*)result)->dd = dd;

  /* Register this object with memory management */
  cxcdm_memory_register( result );

  if (obj2)
  {
    PyObject_Del( obj2 );
    if (colNames)
    {
      free( ((char**)colNames)[0] );
      free( (char**)colNames );
      colNames = NULL;
    }
  }

  return( result );

 fail:

  if (obj2)
  {
    PyObject_Del( obj2 );
    if (colNames)
    {
      free( ((char**)colNames)[0] );
      free( (char**)colNames );
      colNames = NULL; 
    }
  }
  if (obj3)
    PyObject_Del( obj3 );
  
  return NULL;

}



