/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm_descriptor.h
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:
 *    definitions related to dmDescriptor wrappers.
 *    for the Python 'cxcdm' module
 *
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */

#ifndef CXCDM_DESCRIPTOR_H
#define CXCDM_DESCRIPTOR_H

#include "cxcregion.h"
#include <pyregion/pyregion.h>

typedef struct {
  PyObject_HEAD
  dmDescriptor* dd;
} pyDescriptor;

staticforward PyTypeObject dmDescriptor_Type;
static PyObject*  pyDescriptor_new( void );
static void       pyDescriptor_dealloc( pyDescriptor* );


typedef struct {
  PyObject_HEAD
  dmDescriptorType typenum;
} pyDescriptorType;

staticforward PyTypeObject dmDescriptorType_Type;

static PyObject*  pyDescriptorType_new( pyDescriptorType* self, PyObject *args);
static PyObject*  pyDescriptorType_alloc(void);
static int        pyDescriptorType_init( pyDescriptorType* self, PyObject *args);
static int        pyDescriptorType_set( pyDescriptorType* self, long blocktype );
static void       pyDescriptorType_dealloc( pyDescriptorType* );
static PyObject*  pyDescriptorType_repr( pyDescriptorType* );
static PyObject*  pyDescriptorType_rcmp( PyObject* obj1, PyObject* obj2, int op );

typedef struct {
  PyObject_HEAD
  dmElementType typenum;
} pyElementType;

staticforward PyTypeObject dmElementType_Type;
static PyObject*  pyElementType_new( pyElementType* self, PyObject *args);
static PyObject*  pyElementType_alloc(void);
static int        pyElementType_init( pyElementType* self, PyObject *args);
static int        pyElementType_set( pyElementType* self, long blocktype );
static void       pyElementType_dealloc( pyElementType* );
static PyObject*  pyElementType_repr( pyElementType* );
static PyObject*  pyElementType_rcmp( PyObject* obj1, PyObject* obj2, int op );

/* ****************************************************************************** */

static PyObject* _dmGetArrayDim(PyObject *self, PyObject *args );
static PyObject* _dmGetArrayDimensions(PyObject *self, PyObject *args );
static PyObject* _dmGetArraySize(PyObject *self, PyObject *args );
static PyObject* _dmGetCurrentArraySize(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetArea(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetBin(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetBlock(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetCpt(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetCoord(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmDescriptorGetDataType(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetDesc(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetElementDim(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetLength(PyObject *self, PyObject *args );
static PyObject* _dmGetElementType(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetType(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetName(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetNo(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetNoCoords(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetRange(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetNull(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetUnit(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorIsVarLength(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetBin(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetDesc(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetName(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetNull(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetUnit(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorSetRange(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorDelete( PyObject *self, PyObject *args );
static PyObject* _dmDescriptorCompare( PyObject *self, PyObject *args );

static PyObject* _dmBlockGetKeyList(PyObject *self, PyObject *args);
static PyObject* _dmBlockGetSubspace(PyObject *self, PyObject *args);
static PyObject* _dmBlockGetSubspaceCol(PyObject *self, PyObject *args);
static PyObject* _dmBlockAdvanceKeys(PyObject *self, PyObject *args);
static PyObject* _dmKeyOpen(PyObject *self, PyObject *args, PyObject *kwds);
static PyObject* _dmKeyRead(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmKeyWrite(PyObject *self, PyObject *args, PyObject *kwds );

static PyObject* _dmColumnCreate(PyObject *self, PyObject *args, PyObject *kwds);
static PyObject* _dmTableOpenColumn(PyObject *self, PyObject *args );
static PyObject* _dmTableOpenColumnList(PyObject *self, PyObject *args );

static PyObject* _dmSubspaceColCreate(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmSubspaceColCreateRegion(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmSubspaceColOpen(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColGet(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColGetRegion(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColGetRegionString(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColGetTableName(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColRead(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColSet(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColSetRegion(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColSetTableName(PyObject *self, PyObject *args );
static PyObject* _dmSubspaceColUpdate(PyObject *self, PyObject *args );

static PyObject* _dmImageGetDataDescriptor(PyObject *self, PyObject *args );
static PyObject* _dmArrayCreateAxisGroup(PyObject *self, PyObject *args );
static PyObject* _dmArrayGetAxisGroup(PyObject *self, PyObject *args );
static PyObject* _dmArrayGetNoAxisGroups(PyObject *self, PyObject *args );
static PyObject* _dmDescriptorGetData(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmDescriptorSetData(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmDescriptorGetImageData(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmDescriptorSetImageData(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmCoordCalc(PyObject *self, PyObject *args );
static PyObject* _dmCoordCreate(PyObject *self, PyObject *args, PyObject *kwds );
static PyObject* _dmCoordGetAxisGroupNo(PyObject *self, PyObject *args );
static PyObject* _dmCoordGetParams(PyObject *self, PyObject *args );
static PyObject* _dmCoordGetTransform(PyObject *self, PyObject *args );
static PyObject* _dmCoordGetTransformType(PyObject *self, PyObject *args );
static PyObject* _dmCoordInvert(PyObject *self, PyObject *args );
static PyObject* _dmCoordSetParam(PyObject *self, PyObject *args );
static PyObject* _dmCoordSetTransform(PyObject *self, PyObject *args );

static PyObject* _dmGetCptName(PyObject *self, PyObject *args );
static PyObject* _dmGetDisp(PyObject *self, PyObject *args );
static PyObject* _dmSetDisp(PyObject *self, PyObject *args );
static PyObject* _dmSetCptName(PyObject *self, PyObject *args );


/* DM internal functions */
extern long dmGetVectorArrays( dmDescriptor* dd, dmDataType type, void* vals, long dim, long row, long nrows, long start, long nvals);
extern long dmSetVectorArrays( dmDescriptor* dd, dmDataType type, void* vals, long dim, long row, long nrows, long start, long nvals);
extern dmDescriptor*  dmp_key_write( dmBlock* block, char* name, dmDataType dtype, void* data, long nvals, char* unit, char* desc, dmBool special );
extern dmDescriptor*  dmi_coord_create(dmDescriptor* dd, char* name, char* unit, char** cptNames, long dim, char* ttype, void* pix, void* val, void* delt, dmDataType dtype, double* params);
extern dmErrCode dmp_image_get_subarray(dmDescriptor* imdata, long* pixelLl, long* pixelUr, dmDataType type, void* values);
extern dmErrCode dmp_image_set_subarray(dmDescriptor* imdata, long* pixelLl, long* pixelUr, dmDataType type, void* values);
extern dmErrCode dmi_descriptor_get_cpt_range( const dmDescriptor* dd, const long cptno, const dmDataType dtype, void* rangemin, void* rangemax );
extern int dmp_descriptor_set_range( dmDescriptor* dd, const dmDataType type, void* tlmin, void* tlmax );
extern int dmi_subspace_col_get_values(dmDescriptor* filter_dd, long cptno, dmDataType type, void** value1, void** value2, long* nvalues );
extern int dmi_coord_calc( const dmDescriptor* dd, void* value, dmDataType type1, dmDataType type2, void* result );
extern int dmi_coord_invert( const dmDescriptor* dd, void* value, dmDataType type1, dmDataType type2, void* result );
extern int dmi_coord_set_transform(dmDescriptor* dd, void* crpix, dmDataType dType,double* crval,double* cdelt,long dim);
extern int dmp_descriptor_set_bin(dmDescriptor* dd, const dmDataType type, void* value);
extern int dmp_descriptor_set_null(dmDescriptor* dd, const dmDataType type, void* value);
extern int dmi_descriptor_get_cast_null( const dmDescriptor* dd, const dmDataType type, void* value );
extern int dmi_descriptor_get_cast_bin( const dmDescriptor* dd, const dmDataType type, void* value );
extern int dmi_subspace_col_update_null(dmDescriptor* dd, char* name, long cptNo, dmDataType type, void* value1, void* value2, long nvalues, dmBool update );
extern dmDescriptor* dmp_subspace_col_create_table( dmBlock* block, char* tabname, char* name, dmDataType type, char** colNames, char* unit, void* value1, void* value2, long nvalues );
extern dmDescriptor* dmi_subspace_col_create( dmBlock* block, dmSubspace* subspacep, char* name, char** cptnames, long dim, dmDataType type, char* unitp, dmDescriptor* col_dd, dmBool default_filter );
extern void dmi_subspace_col_set_values( dmDescriptor* dd, char* tabname, void* value1, void* value2, long nvalues, dmDataType type, long dim, int array, regRegion* region );

/* Error Messages */
#define DMDESC_GENERIC_MSG        "%s() %s"
#define DMDESC_EMPTY_PTR_MSG      "%s() operation on empty dmDescriptor pointer."
#define DMDESC_INVALID_NULL_MSG   "%s() argument %s can not be NULL"
#define DMDESC_INVALID_ARGTYP_MSG "%s() argument %ld must be %s, not %s"
#define DMDESC_INVALID_ARGVAL_MSG "%s() argument %ld has invalid value"
#define DMDESC_BAD_CONV_MSG    "%s() unsupported datatype conversion (\'%s\' -> \'%s\')"
#define DMDESC_BAD_DTYPE_MSG   "%s() unrecognized datatype (\'%s\'), please input numerical, bool or string type."
#define DMDESC_BAD_STRLEN_MSG  "%s() string length (itemsize) must be > 0"
#define DMDESC_UNK_DTYPE_MSG   "%s() unsupported datatype (\'%s\')"
#define DMDESC_ALLOC_ERR_MSG   "%s() problem allocating memory for %s."
#define DMDESC_VALUES_ERR_MSG  "%s() problem %s values for descriptor."
#define DMDESC_NOROWS_ERR_MSG  "%s() no rows returned for descriptor."
#define DMKEY_OPEN_ERR_MSG     "%s() could not find key. \'%s\'"
#define DMKEY_OPEN2_ERR_MSG    "%s() could not find key #%-ld"
#define DMKEY_WRITE_ERR_MSG    "%s() could not write key. \'%s\'"
#define DMDESC_INVALID_DIM_MSG "%s() input dimensions, %d, does not match the descriptor dimensions, %d."
#define DMDESC_INVALID_VALUE_MSG "%s() invalid number of values, %d."
#define DMDESC_BAD_DESCTYPE_MSG "%s() descriptor type must be %s, not %s"

static PyMethodDef descriptor_methods[] =
{
  {NULL, NULL, 0, NULL}     /* Sentinel - marks the end of this structure */
};/*end... */

#endif
