/*                                                                
**  Copyright (C) 2010  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* *****************************************************************
 *
 * FILE NAME:  cxcdm_mm.h
 *
 * DEVELOPMENT: DataModel
 *
 * DESCRIPTION:
 *
 * REVISION HISTORY:
 *
 ***************************************************************** */

#ifndef CXCDM_MEM_H
#define CXCDM_MEM_H


/* ****************************************************************************** */
typedef struct cmmll{
  struct cmmobj* cobj;             /* local  object containing DM C memory ptr      */
  PyObject*      pobj;             /* Python object containing DM C memory ptr      */
  struct cmmll*  next;
} CMMLL;


typedef struct cmmobj{
  void*  cptr;                   /* pointer to DM C memory struct (e.g. dmBlock*) */

  PyTypeObject* type;            /* what kind of object is this (e.g. dmBlock)    */
  long   refcnt;                 /* number of open dependencies..                 */

  struct cmmll* parent;          /* pointer to struct for parent of cptr          */
  struct cmmll* children;        /* list of children linked to this cptr          */
  struct cmmll*  holders;        /* list of python objects using this cptr        */

} CMMObject;
/* ****************************************************************************** */

CMMObject* cxcdm_memory_new_cmmobj( void );
CMMLL*     cxcdm_memory_new_cmmll( void );


void   cxcdm_memory_add( CMMObject* );
void   cxcdm_memory_decref( CMMObject* );
void   cxcdm_memory_delete( void* cptr );
CMMObject* cxcdm_memory_find( void* cptr );
void   cxcdm_memory_incref( CMMObject* );
void   cxcdm_memory_print( void );
void   cxcdm_memory_register( PyObject* );
void   cxcdm_memory_remove( CMMObject* node );
void   cxcdm_memory_remove_tree( CMMObject* node );
int    cxcdm_memory_unlink( CMMObject* );

void   cxcdm_memory_add_child(  CMMObject*, CMMObject* );
void   cxcdm_memory_add_holder( CMMObject*, PyObject* );
void   cxcdm_memory_add_parent( CMMObject* );
void   cxcdm_memory_empty_holders( CMMObject* );
void   cxcdm_memory_release_holders( CMMObject* );
void   cxcdm_memory_release_holder( CMMObject* , PyObject* );
void   cxcdm_memory_release_child( CMMObject* , CMMObject* );

#endif
