/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

/* Bit functions; note bits are numbered 1 to n */

/*
 *  Give me a number with a single bit set in the specified position.
 */
unsigned short bit_single_s( const long bit_no )
{
 unsigned short value = 1;
 return( value << bit_no );
}

unsigned long int bit_single_i( const long bit_no )
{
 unsigned long int value = 1;
 return( value << bit_no );
}

/*
 * Set bit in value 
 */
void bit_set_i( long* value, const long bit_no )
{
 *value |= bit_single_i( bit_no );
}


void bit_set_s( short* value, const long bit_no )
{
 *value |= bit_single_i( bit_no );
}

void bit_clr_i(  long* value, const long bit_no )
{
 *value &= ~bit_single_i( bit_no );
}

void bit_clr_s(  short* value, const long bit_no )
{
 *value &= ~ bit_single_s( bit_no );
}


short bit_shft_s( short value, const long bits_to_shift )
{
 if ( bits_to_shift >= 0 ) {
  return( value << bits_to_shift );
 } else { 
  return( value >> -bits_to_shift );
 }
}

long bit_shft_i( long value, const long bits_to_shift )
{
 if ( bits_to_shift >= 0 ) {
  return( value << bits_to_shift );
 } else { 
  return( value >> -bits_to_shift );
 }
}

logical bit_test_i( long value, const long bit_no )
{
 long tmp;
 tmp =  bit_single_i( bit_no ) & value ;
 return( tmp != 0 );
}

logical bit_test_s( short value, const long bit_no )
{
 short tmp;
 logical result;
 tmp =  bit_single_s( bit_no ) & value ;
 result = ( tmp != 0 );
 return( result );
}

long bit_unpack_i( const long value, const long start_bit, const long n_bits )
{
 long result;
 unsigned long int mask;

 mask = ~( ~0 << n_bits );  /* n ones at the right */
 result = mask & ( value >> start_bit );
 return( result );
}

long bit_unpack_s( const short value, const long start_bit, const long n_bits )
{
 unsigned short result;
 unsigned short mask;
 unsigned long int iresult;

 mask = ~( ~0 << n_bits );  /* n ones at the right */
 result = mask & ( value >> start_bit );
 iresult = (unsigned long int) result;
 return( (long)iresult );
}

