/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "utlib.h"
#include "fiolist.h"

/*
 *  These routines use the fcntl.h file access mechanism
 */

FioFile fio_open_c( FioFileList list, char* filename, char* jmode, char* mode )
{
 FioFile file;
 long fd;
 long imode;
  if ( cs_eq( mode, "cw" ) ) { 
   fd = creat( filename, 0644 );
  } else {
   imode = cs_eq(mode, "crw" ) ? O_RDWR : O_RDONLY;
   fd = open( filename, imode );
  }
  file = fio_cfile_alloc( list, filename, jmode, fd );
  return( file );
}

/*
 *  Routines to read and write bytes from a CIO file.
 */

logical fio_read_bytes( FioFile file, char* buf, long n, long* m )
{
 long fd;
 fd = fio_file_get_cfd( file );
 *m= read( fd, buf, n );
 return( *m == n );
}

logical fio_write_bytes( FioFile file, char* buf, long n, long* m )
{
 long fd;
 fd = fio_file_get_cfd( file );
 *m= write( fd, buf, n );
 return( *m == n );
}

/*
 *  Wrap POSIX isatty
 */
logical fio_file_tty( FioFile file )
{
 long fd;
 if ( file ) {
  fd = fio_file_get_cfd( file );
  return( isatty( fd ) );
 } else {
  return( isatty( 0 ) );
 }
}

/* Read a single byte. Usually used in raw mode */

long fio_read_byte( FioFile file )
{
 logical status;
 char buf;
 long fd;
 long byte;

 fd = fio_file_get_cfd( file );
 status = ( read( fd, &buf, 1 ) < 1 );
 if ( status ) {
  byte  = (long) ( buf & UT_CHAR_MASK );
 } else {
  byte = -1;
 }
 return( byte );
}

/*
 *  Open a raw mode file. Used for graphics cursors.
 */

FioFile fio_list_open_raw( FioFileList list )
{
 logical start;
 FioFile file;
 long fd;
 start = list->stdin_is_tty;
 if ( start  < 0  ) 
  list->stdin_is_tty = fio_file_tty( NULL ) ? 1 : 0;

 if ( list->stdin_is_tty > 0 ) {
  file = fio_open_crw( "/dev/tty" );
  if ( file ) {
   fio_file_set_type( file, "CIO RAW" );
   fd = fio_file_get_cfd( file );
   if ( start ) cio_setup_raw( list, fd );
   cio_set_raw( list, fd );
  }
  return( file );
 } else {
  fio_tmsg( "Failed to open raw mode: stdin is not tty" );
  return( NULL );
 }
}

/*
 * Read a byte from a raw file. Used to get a cursor value.
 */
long fio_read_raw( FioFile file )
{

 long c;

 if ( !file ) {
  c = UT_CHAR_ERR;
 } else {
  c = fio_read_byte( file );
  if ( c < 0 ) { 
   fio_file_close( file ); 
  } else if ( c == UT_CHAR_CTRLC ) {
   fio_tmsg( "CTRL/C seen but ignored" );
  }
 }
 return( c );
}


