/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "fiolist.h"

void cio_setup_raw( FioFileList f,  long fd )
{
 f->termios_ncc= ccio_get( f, fd, f->termios_cc_save, f->termios_flags_save );
 cs_copy_siz( f->termios_cc_save, f->termios_cc_raw, UT_TERMIOS_TSIZE );
 ar_copy_i( f->termios_flags_save, f->termios_flags_raw, 4 );
 ccio_set_raw( f->termios_cc_raw, f->termios_flags_raw );
}


void cio_set_raw( FioFileList list, long fd )
{
 ccio_set( list, fd, list->termios_cc_raw, list->termios_flags_raw );
}

void cio_clear_raw( FioFileList list, long fd )
{
 ccio_set( list, fd, list->termios_cc_save, list->termios_flags_save );
}


long ccio_get( FioFileList f, long fd, char* ptr, long* fptr )
{
 long i;

 (void)tcgetattr( fd, &f->termios_arg );

/* Save initial values */

 for ( i = 0; i < NCCS; i++ ) {
  *ptr = f->termios_arg.c_cc[i];
  ptr++;
 }
 *ptr = f->termios_arg.c_cc[VMIN]; ptr++;
 *ptr = f->termios_arg.c_cc[VTIME]; ptr++;
 *fptr = (long) f->termios_arg.c_iflag; fptr++;
 *fptr = (long) f->termios_arg.c_oflag; fptr++;
 *fptr = (long) f->termios_arg.c_cflag; fptr++;
 *fptr = (long) f->termios_arg.c_lflag; fptr++;
 return( NCCS );
}


void ccio_set( FioFileList f, long fd, char* ptr, long* fptr )
{
 long i;

 for ( i = 0; i < NCCS; i++ ) {
  f->termios_arg.c_cc[i] = *ptr++;
 }
 f->termios_arg.c_iflag = (unsigned int)*fptr++;
 f->termios_arg.c_oflag = (unsigned int)*fptr++;
 f->termios_arg.c_cflag = (unsigned int)*fptr++;
 f->termios_arg.c_lflag = (unsigned int)*fptr++;

 (void)tcsetattr(fd,TCSADRAIN,&f->termios_arg);
}


void ccio_set_raw( char* ptr, long* fptr )
{
/* Set raw mode values */
 fptr[0] = ~( INLCR | ICRNL );
 fptr[3] = ~( ICANON | ECHO );
 ptr[VMIN] = 1;
 ptr[VTIME] = 0;
 ptr[VSUSP] = -1;
}

