/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"


float utm_radin_r( const float x ) 
{
 return( x * ut_constants.r.deg_rad );
}

float utm_radout_r( const float x )
{
 return( x / ut_constants.r.deg_rad );
}

double utm_radin_d( const double x ) 
{
 return( x * ut_constants.d.deg_rad );
}

double utm_radout_d( const double x )
{
 return( x / ut_constants.d.deg_rad );
}



double utm_circ_d( const double x )
{
 return( modulo_d( x, ut_constants.d.circle ) );
}


float utm_circ_r( const float x )
{
 return( modulo_r( x, ut_constants.r.circle ) );
}

float utm_circ_ew_r( const float x )
{
 float y;
 y = modulo_r( x, ut_constants.r.circle );
 if ( y >= ut_constants.r.circle/2 ) y -= ut_constants.r.circle;
 return( y );
}

double utm_circ_ew_d( const double x )
{
 double y;
 y = modulo_d( x, ut_constants.d.circle );
 if ( y >= ut_constants.d.circle/2 ) y -= ut_constants.d.circle;
 return( y );
}


double utm_circ_rad_d( const double x )
{
 return( modulo_d( x, ut_constants.d.twopi ) );
}


float utm_circ_rad_r( const float x )
{
 return( modulo_r( x, ut_constants.r.twopi )  );
}

float utm_circ_ew_rad_r( const float x )
{
 float y;
 y = modulo_r( x, ut_constants.r.twopi );
 if ( y >= M_PI ) y -= ut_constants.r.twopi;
 return( y );
}

double utm_circ_ew_rad_d( const double x )
{
 double y;
 y = modulo_d( x, ut_constants.d.twopi );
 if ( y >= M_PI ) y -= ut_constants.d.twopi;
 return( y );
}



float utm_argd_r( const float x, const float y )
{
 float a;
 if ( x == 0.0 && y == 0.0 ) {
  a = 0.0;
 } else {
  a = atan2( y, x ) / ut_constants.r.deg_rad;
  a = circ_r( a );
 }
 return( a );
}


double utm_argd_d( const double x, const double y )
{
 double a;
 if ( x == 0.0 && y == 0.0 ) {
  a = 0.0;
 } else {
  a = atan2( y, x ) / ut_constants.d.deg_rad;
  a = circ_d( a );
 }
 return( a );
}


