/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "fio.h"

long fio_file_status( FioFile file, long mode )
{
  long status = 0;
  if ( file )
  {
   status = file->status;
   if ( mode == FIO_CLEAR ) file->status = UT_FILE_OPEN;
  }
  return( status );
}



void fio_set_status_pos( FioFile file, logical ios, long pos ) 
{
 if ( !file ) return;
 fio_file_set_status( file, ios );
 if ( ios ) {
  file->line++;
  fio_file_set_pos( file, pos );
 }
}

void fio_file_set_status( FioFile file, logical status )
{
 if ( file )
  file->status = status ? UT_FILE_OPEN : UT_FILE_EOF;
}


void fio_delta_pos( FioFile file, long pos )
{
 if ( file ) 
  file->pos += pos;
}

void fio_file_rewind( FioFile file )
{
 if ( !fio_file_interactive( file ) )
  rewind( file->fptr );
 fio_line_reset( file );
}

/*----------------------------------------------------------------------*/
/*
 *
 * Set the values in a JCM file structure.
 *
 */
/*
 * This routine uses the fio_file structure definition.
 * Uses:
 *  csproto.h: cs_copy
 * <stdio.h>: FILE* 
 */

long fio_checkin( FioFile file, const char* Name, char* Type, long Status, FILE* Fptr, long stream )
{
  cs_copy( Name, file->name );
  cs_copy( Type, file->type );
  file->status = Status;
  file->fptr = Fptr;
  file->stream = stream;
  fio_line_reset( file );
  return file->fid;
}





void fio_line_reset( FioFile file )
{
 file->pos = 0;
 file->line= 0;
}
/*----------------------------------------------------------------------*/
/*
 * fio_file_alloc
 *
 * Allocate and set a new JCM file given its properties, but only if it is
 * a STDIO file.
 * Uses:
 *   <stdio.h>:  printf
 *   copen:      fio_alloc, fio_file_set
 */

FioFile fio_file_alloc( FioFileList list, const char* Name, char* Type, long Status, FILE* Fptr )
{

/*
 * This routine uses the fio_file structure definition.
 */
  FioFile file;

  if ( Fptr == NULL ) {
   file = NULL;
  } else {
   file = fio_list_alloc_file( list );
   fio_checkin( file, Name, Type, Status, Fptr, fileno( Fptr ) );
  }
  return( file );
}

FioFile fio_cfile_alloc( FioFileList list, char* Name, char* Type, long fd )
{

/*
 * This routine uses the fio_file structure definition.
 */

  FioFile file;

  if ( fd < 0 ) {
   file = NULL;
  } else {
   file = fio_list_alloc_file( list );
   fio_checkin( file, Name, Type, UT_FILE_OPEN, (FILE *)NULL, fd );
  }
  return( file );
}


/*----------------------------------------------------------------------*/
/*
 *  fio_file_clear
 *
 */

void fio_file_clear( FioFile file )
{
/*
 * This routine uses the fio_file structure definition.
 */
 if ( file ) {
  fio_checkin( file, " ", " ", UT_FILE_CLOSED, NULL, 0 );   
 }
}

long fio_file_get_cfd( FioFile file )
{
 return( file->stream );
}


