/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

/*----------------------------------------------------------------------*/
/*
 * fio_open_list
 *
 * List all open files
 * Uses:
 *  <stdio.h>: sprintf
 *  csproto.h: cs_copy
 *  fileproto.h: fio_file_name, fio_get_type, fio_file_status fio_file_n
 *  msg
 */

void fio_list( void ) 
{
 fio_list_print( NULL );
}

void fio_list_print( FioFileList f ) 
{
 long n;
 long file_no;
 Filename name;
 TextWord type;
 long status;
 TextBuf buf;
 TextWord eof;
 logical qopen;
 long pos, line, recl;
 long istatus;
 long cfd;

 FioFile file;

 if ( !f ) f = fio_get_list();
 n = fio_list_n( f );
 fio_msg( " " );
 fio_dashline( 40 );
 fio_msg( " JCMLIB File IO System (FIO) Status " );
 fio_dashline( 40 );
 fio_msg( "  File  FD  Line  Cpos  Recl  IOS  EOF  Type        Name " );
 for ( file_no = 1; file_no <= n; file_no++ ) {
  file = fio_list_get_file( f, file_no );
  fio_file_get_name( file, name, FN_SIZE );
  fio_file_get_type( file, type, WORD_SIZE );
  cfd = fio_file_get_cfd( file );
  status = fio_file_status( file, FIO_CHECK );
  qopen = status != UT_FILE_CLOSED;
  if ( qopen ) {
   line = fio_file_get_lineno( file );
   pos  = fio_file_get_pos( file );
   recl = fio_file_get_recl( file );
   if ( status == UT_FILE_EOF ) {
    istatus = -1;
    cs_copy( "EOF", eof );
   } else {
    istatus = 0;
    cs_copy( " ", eof );
   }
   sprintf( buf, "  %-6ld%-4ld%-6ld%-6ld%-6ld%-6ld%-4s%-12s%s",
    (long)file_no, (long)cfd, (long)line, (long)pos, (long)recl, (long)istatus, eof, type, name );
   fio_msg( buf ); 
  }
 }
}




FioFile fio_open_file( const char* filename, char* mode, long rec_bytes )
{
 FioFileList list = fio_get_list();
 return fio_list_open_file( list, filename, mode, rec_bytes );
}

FioFile fio_open_raw( void )
{
 return fio_open_file( NULL, "RAW", 0 ); 
}

FioFile fio_open_ar( const char* filename )
{
 return fio_open_file( filename, "SEQ R", 0 ); 
}

FioFile fio_open_sr( const char* filename )
{ 
 return fio_open_file( filename, "SIO R", 0 );
}

FioFile fio_open_sw( const char* filename )
{
 return fio_open_file( filename, "SIO W", 0 );
}

FioFile fio_open_srw( const char* filename )
{
 return fio_open_file( filename, "SIO RW", 0 );
}

FioFile fio_open_aw( const char* filename )
{
 return fio_open_file( filename, "SEQ W", 0 );
}

FioFile fio_open_aa( const char* filename )
{
 return fio_open_file( filename, "SEQ A", 0 );
}

FioFile fio_open_crw( const char* filename )
{
 return fio_open_file( filename, "CIO RW", 0 );
}

FioFile fio_open_cr( const char* filename )
{
 return fio_open_file( filename, "CIO R", 0 );
}

FioFile fio_open_cw( const char* filename )
{
 return fio_open_file( filename, "CIO W", 0 );
}


