/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*======================================================================*/
/*
 *    cs2
 *
 *    Null terminated char* ("C char*") handling routines
 *
 */
/*======================================================================*/
#include <ctype.h>
#include "utlib.h"

/*----------------------------------------------------------------------*/
/*    cs_strpak
 *    
 * Pack a char*, forcing to uppercase and removing spaces and underbars.
 * Useful for name resolvers.
 * 1996 Sep 30 JCM
 */

void cs_strpak( const char* isource, char* dest )
{
 char* source = (char*)isource;
 while ( *source != '\0' ) {
  while ( *source == ' ' || *source == '_' ) source++;
  if ( *source !='\0' ) 
  {
   *dest = toupper( *source++ );
   dest++;
  }
 }
 *dest = '\0';
}

/*----------------------------------------------------------------------*/
/*
 *+cs_gparen
 *
 */

void cs_gparen( const char* isource, const char lpar, const char rpar, char* dest )
{
 long k;
 char* source = (char*)isource;
 *dest++ = lpar;
 k = cs_ends( source );
 while ( k-- ) { *dest = *source++; dest++; }
 *dest = rpar; dest++;
 *dest   = '\0';
}

/*----------------------------------------------------------------------*/
/*
 *+cs_paren
 *
 */
char* cs_paren( const char* source, char* dest )
{
 cs_gparen( source, '(', ')', dest );
 return( dest );
}

/*----------------------------------------------------------------------*/
/*
 *+cs_sparen
 *
 */
char* cs_sparen( const char* source, char* dest )
{
 cs_gparen( source, '[', ']', dest );
 return( dest );
}


void cs_ubar( char* source )
{
 long n;
 n = cs_ends( source );
 while ( n-- ) {
  if ( *source == ' ' ) *source = '_';
  source++;
 } 
}

void cs_deubar( char* source )
{
 long n;
 n = cs_ends( source );
 while ( n-- ) {
  if ( *source == '_' ) *source = ' ';
  source++;
 } 
}

logical cs_nullstr( const char* source )
{
 return( *source == '\0' );
}



void cs_deparen( char** buf, char* dest )
{
 char* ptr;
 const char lpar = '(';
 const char rpar = ')';
 long b,e;
 long npar;

 npar = 1;
 ptr = *buf;
 b = cs_begs( ptr );
 e = cs_ends( ptr );
 ptr += b - 1;
 if (*ptr == lpar ) ptr++;
 while ( npar > 0 && ptr - *buf + 1 <= e  ) {
  if ( *ptr == lpar ) {
   npar++;
  } else if ( *ptr == rpar ) {
   npar--;
  }
  if ( npar > 0 ) { *dest++ = *ptr++; }
 }
 
 if ( npar > 0 ) { 
  *buf = NULL;
  *dest = '\0';
 } else {
  *buf =++ptr;
  *dest = '\0';
 }  
}


