/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"


/*----------------------------------------------------------------------*/
/*
 *+cs_list_add
 *
 *
 *  Add an entry 'item' to a list of char*s and return its number in the list.
 *  If item is already in the list, return its number, i.
 *  If it is not already in the list, add it to the end of the list,
 *  increment list_size, and return its number i = list_size.
 *  The number in the list is counted from 1, so item = list[i-1].
 */
long cs_list_add( char** list, long* list_size, const long max_size, const char* item )
{
 long item_no;

 item_no = ar_match_c( item, list, *list_size );
 if ( item_no == 0 ) {
  if ( *list_size >= max_size ) {
   fio_tmsg( "Routine list_add: list overflow" );
  } else {
   (*list_size)++;
   item_no = *list_size;
   cs_copy( item, list[item_no-1] );
  }
 }
 return( item_no );
}



