/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*======================================================================*/
/*
 *   cs op
 *
 *   C char* routines to manipulate char*s
 *
 *
 */
/*======================================================================*/
#include "utlib.h"

/*----------------------------------------------------------------------*/
/*
 *   cs_lt
 *
 *   Test for char* A < char* B, up to trailing spaces.
 *   (Note that strcmp cares about trailing spaces and we don't.)
 *   1996 Aug 29 JCM
 *   Args:
 *	buf1		(i)	char* A
 * 	buf2		(i)	char* B
 */

logical cs_lt( const char* ibuf1, const char* ibuf2 )
{
 char* buf1 = (char*)ibuf1;
 char* buf2 = (char*)ibuf2;
 while( *buf1 == *buf2 && *buf1 != '\0' )  {
  buf1++;
  buf2++;
 }
 if ( *buf1 == '\0' ) return( !cs_is_blank( buf2 ) );
 if ( *buf2 == '\0' ) return( UT_FALSE );
 return( *buf1 < *buf2 );
}

/*----------------------------------------------------------------------*/
/*
 *   cs_gt
 *
 *   Test for char* A > char* B, up to trailing spaces.
 *   (Note that strcmp cares about trailing spaces and we don't.)
 *   1996 Aug 29 JCM
 *   Args:
 *	buf1		(i)	char* A
 * 	buf2		(i)	char* B
 */

logical cs_gt( const char* ibuf1, const char* ibuf2 )
{
 char* buf1 = (char*)ibuf1;
 char* buf2 = (char*)ibuf2;
 while( *buf1 == *buf2 && *buf1 != '\0' )  {
  buf1++;
  buf2++;
 }
 if ( *buf1 == '\0' ) return( UT_FALSE );
 if ( *buf2 == '\0' ) return( !cs_is_blank( buf1 ) );
 return( *buf1 > *buf2 );
}


