/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "errclass.h"

/* The error classes allow us to register ranges of longs
   (of size ut_ERR_CLASS) for use as error codes.
 */

/*
 * Construct an ErrorClasses object of initial size ERR_CLASS_INIT_SIZE.
 */

ErrorClasses ut_error_class_init( void )
{
 ErrorClasses list = calloc( 1, sizeof( struct ErrorClasses_s ));
 list->nmax = UT_ERR_CLASS_INIT_SIZE;
 list->names = calloc( list->nmax, UT_SZ_P );
 return list;
}

void ut_error_class_free( ErrorClasses list )
{ 
 ar_free_cp( list->names, list->nmax );
 free( list );
}

/*
 *  Register a new named error class with the list. 
 *  If the name already exists, don't make a new one.
 *  Return the class mask.
 */
long ut_error_class_register( ErrorClasses list, char* name )
{
 long class_no = -1;
 long class_mask;

 if ( !list ) 
  list = ut_error_class_global( NULL );
 if ( name )
  class_no = ar_cmatch_c( name, list->names, list->n );

 if ( class_no < 0 )
 {
  if ( list->n >= list->nmax )
  {
   list->nmax = 2 * list->n;
   list->names = (char**)realloc( list->names, list->nmax * UT_SZ_P );
  }
  class_no = list->n;
  list->names[class_no] = cs_dup( name );
  list->n++;
 }
 class_mask = class_no *  UT_ERR_CLASS;
 return class_mask;
}


/* Convert error code to [class-name errno].
 * Given a list of error classes and an error code,
 * split the error code into class number and class code.
 * Then return a char* with the class name and class code.
 */
void ut_error_class_decode( ErrorClasses list, long code, char* text )
{
 long class_no;
 long class_code;
 char* name = NULL;
 char* dname = "Unknown error class";

 if ( !list )
 {
  cs_copy( "", text );
  return;
 }

 class_no = rmod_i( code, &class_code, UT_ERR_CLASS );

 if ( class_no < list->n )
  name = list->names[class_no];
 if ( !name )
  name = dname;

 sprintf( text, "[%s %3ld]", name, class_code );

}

