/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "uterr.h"


void ut_error_init( char* buf )
{
 ut_error_init_user( NULL, buf );
}

void ut_error_status( void )
{
 Error* error = ut_error_state();
 ErrorClasses list = ut_error_class_global( NULL );
 ut_error_disp( error, list );
 ut_error_clear( error );
}

void ut_error( char* buf, long code )
{
 long value;
 long user_mask = ut_error_user_mask();
 Error* error = ut_error_state();
 value = user_mask | code; 
 ut_error_set( error, buf, value, 0 );
}

long ut_error_user_mask( void )
{
 ErrorGlobal error_global = ut_error_global();
 return error_global->user_mask;
}

void ut_err_util( char* buf, long code )
{
 Error* error = ut_error_state();
 long value;
 value = ut_error_mask( NULL ) | code;
 ut_error_set( error, buf, value, 0 );
 ut_msg_d( UT_VERBOSE, buf );
}

/* Return a pointer to the error state */

Error* ut_error_state( void )
{
 ErrorGlobal error_global = ut_error_global();
 return &error_global->error;
}

void ut_error_init_user( ErrorGlobal error_global, char* buf )
{
 if ( !error_global ) error_global = ut_error_global();
 error_global->user_mask = ut_error_class_register( error_global->error_classes, buf );
}


/*
 *  Handle JCMUTILS error classes
 */

/*
 *    Initialize and register the JCMUTILS error class
 */
ErrorGlobal ut_error_init_state( void )
{
 ErrorGlobal error_global = calloc( 1, sizeof( struct ErrorGlobal_s ));
 error_global->error = NULL;
 error_global->error_classes = ut_error_class_init();
 error_global->ut_mask = ut_error_class_register( error_global->error_classes, "JCMUTILS" );
 error_global->user_mask = 0;
 return error_global;
}

void ut_error_free_state( ErrorGlobal error_global )
{
 ut_error_clear( &error_global->error );
 free( error_global );
}

ErrorClasses ut_error_class( void )
{
 return ut_error_class_global( NULL );
}

ErrorGlobal ut_error_global( void )
{
 return ut_state_error_global( NULL );
}

ErrorClasses ut_error_class_global( ErrorGlobal error_global )
{
 if ( !error_global ) error_global = ut_error_global();
 return error_global->error_classes;
}

long ut_error_mask( ErrorGlobal error_global )
{
 if ( !error_global ) error_global = ut_error_global();
 return error_global->ut_mask;
}


