/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/* Program Control Characters */

#include "utlib.h"
#include "esc_types.h"
#include "jcmstate.h"

/*
 *---------------------------
 * GLOBALS
 *---------------------------
 */

void esc_init( void )
{
 esc_init_state( NULL );
}

/*
 *---------------------------
 * ESC
 * esc::alloc
 * esc::init_state
 * esc::free
 *Get by type
 * esc::special_char
 * esc::get_default_char
 *Get by name
 * esc::get
 * esc::get_default
 *Set by name
 * esc::reset
 *Print
 * esc::status
 *---------------------------
 */

UtilEscData esc_alloc( void )
{
 UtilEscData edata = calloc( 1, sizeof( struct UtilEscapeData_s ));
 edata->data = calloc( ESC_NTYPES, sizeof( UtilEscapes ));
 edata->n = ESC_NTYPES;
 esc_init_state( edata );
 return edata;
}

void esc_init_state( UtilEscData edata )
{ 
 long type;
 UtilEscapes* data;
 char escdefaults[] = "\\(),;$$#$ ";
 char* escnames[] = { "ESC", "LPAR", "RPAR", "WORD", "char*", "SYS", "CONT", "CMT", "MACRO", "ARG" };
 if ( !edata ) edata = ut_state_escapes( NULL );
 for ( type = 0; type < edata->n; type++ ) {
  data = &edata->data[type];
  cs_copy( escnames[type], data->name  );
  data->echar  =   escdefaults[type];
  data->edefault = escdefaults[type];
 }
}

void esc_free( UtilEscData edata )
{
 free( edata->data );
 free( edata );
}

static long esc_id( const char* name )
{
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 
 long type;
 for ( type = 0; type < ESC_NTYPES; type++ ) {
  if ( cs_eq( name, esc_data[type].name ) ) return type;
 }
 sprintf( ut_tmp, "ESC: unknown keyword %s", name );
 ut_err_util( ut_tmp, UT_ERR_ESC );
 return -1;
}


void esc_get_name( const long type, char* name )
{
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 if ( type >= 0 )
  cs_copy( esc_data[type].name, name );
 else 
  cs_copy( " ", name );
}
/*----------------------------------------------------------------------*/
/*
 * esc_get_char
 *
 */

char esc_get_default_char( const long type )
{
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 if ( type >= 0 )
  return( esc_data[type].edefault );
 else 
  return ' ';
}

char esc_special_char( const long type )
{
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 if ( type >= 0 )
  return( esc_data[type].echar );
 else
  return ' ';
}

void esc_reset( const char* name, const char val )
{
 long type;
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 type = esc_id( name );
 if ( type >= 0 )
   esc_data[type].echar = val;
}




char esc_get( const char* name )
{
 long type;
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 type = esc_id( name );
 if ( type >= 0 ) 
  return esc_data[type].echar;
 else
  return ' ';
}

char esc_get_default( const char* name )
{
 long type;
 UtilEscData edata = ut_state_escapes( NULL );
 UtilEscapes* esc_data = edata->data;
 type = esc_id( name );
 if ( type >= 0 ) 
  return esc_data[type].edefault;
 else
  return ' ';
 }


void esc_status( void )
{
 /* Dump the esc struct */
 long type;
 char echar;
 char edefault;
 char name[8];
 char buf[80];
 FioStack iofs = NULL;
 if ( !iofs ) iofs = fio_get_stack();
 fio_iostack_msg( iofs, NULL, "--- JCMLIB Program Control Characters ---", UT_TRUE, UT_FALSE );
 fio_iostack_msg( iofs, NULL, "No  Name   Value  Default Value Default", UT_TRUE, UT_FALSE);
 for ( type = 0; type < ESC_NTYPES; type++ ) {
  echar = esc_special_char( type );
  edefault = esc_get_default_char( type );
  esc_get_name( type, name );
  sprintf( buf, "%2ld %-8s %c     %c  %8d %8d", (long)type, name, echar, edefault,
    echar, edefault );
  fio_iostack_msg( iofs, NULL, buf, UT_TRUE, UT_FALSE );
 }
}
  
 

