/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include <dirent.h>
#include <sys/stat.h>

struct FileDirectory_s {
 DIR* dirptr; 
 char* name;
};

FileDirectory fio_dir_open( const char* idirname )
{
 char* dirname = (char*)idirname;
 FileDirectory dir;
 char* current = "./";   /* Unix current directory */
 dir = calloc( 1, sizeof( struct FileDirectory_s ));
 dir->name = cs_dup( dirname );
 if ( !dirname || !(*dirname ))
  dirname = current;
 dir->dirptr = opendir( dirname );
 return dir;
}

void fio_dir_close( FileDirectory dir )
{
 if ( !dir ) return;
 closedir( dir->dirptr );
 free( dir->name );
 free( dir );
}

long fio_dir_next_filepath( FileDirectory dir, char* fn, long maxlen )
{
  struct dirent* dp;
  Filename name;
  dp = readdir( dir->dirptr );
  if ( dp )
  {
   cs_copy_siz( dp->d_name, name, FN_SIZE );
   sys_filepath( dir->name, name, " ", fn, maxlen );
   return fio_stat_type( fn );
  }
  else
  {
   *fn = 0;
   return FIO_NOFILE;
  }
}

long fio_dir_next_normal_file( FileDirectory dir, char* name, long maxlen )
{
	long type = FIO_NOFILE;
	logical loop;
	loop = UT_TRUE;

	while( loop )
	{
	 type = fio_dir_next_filepath( dir, name, maxlen );
         loop = type != FIO_NOFILE && type != FIO_FILE_STD && type != FIO_FILE_DIR;
	}
	if ( type == FIO_FILE_DIR )
	 strcat( name, "/" );
	return type;
}

