/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "jcmstate.h"
#include <unistd.h>


/*
 * Repeatedly expand all environment variables in a char*.
 * Each environment variable reference begins with a $ and ends with
 * a space, colon or semicolon.
 */

void fio_pathlist_free( PathList plist )
{
 if ( !plist ) return;
 cs_namelist_free( plist->local );
 cs_namelist_free( plist->global );
 free( plist );
}

PathList fio_pathlist_expand( char* path )
{
 PathList plist;
 NameList list;
 char mark;
 Filename word;
 Filename dest;
 long n = 10;  /* Initial default size of list */
 char esc='\\';
 char* ptr = path; 
 plist = calloc( 1, sizeof( struct PathList_s ));
 plist->local = cs_namelist_alloc( n, 0 );
 plist->global = NULL;  
 list = plist->local;
 while( ptr != NULL && *ptr != '\0' )
 {
/* Copy up to next : or ; or space to get name of env var; make note of separator */
   mark = cs_copy_token_esc( &ptr, " :;", "", esc, word, FN_SIZE );

   while( word[0] == '$' ) {
/* Found an environment variable */
/* Recursively expand the variable into the output buffer */
    sys_getenv( word+1, dest );
    cs_copy( dest, word );
   }

   cs_namelist_add( list, word );

/* The semicolon marks the start of the global list */
   if ( mark == ';' )
   {
    plist->global = cs_namelist_alloc( n, 0 );
    list = plist->global;
   }
 }

 return plist;
}


/*
 * Search a path list, testing access for a file.
 * Compatible with SAO parameter interface search path.
 * Repeatedly expands environment variable names in the path list.
 */

logical fio_pathlist_search_namelist( NameList list, char* name, char* mode, char* result )
{
 char* dir; 
 long n;
 long i;
 Filename filename;

 if ( list ) 
  n = cs_namelist_size( list );
 else
  n = 1;

   for ( i = 0; i < n; i++ )
   {
    dir = cs_namelist_entry( list, i );
    sys_filepath( dir, name, " ", filename, FN_SIZE );
    if ( fio_posix_access( filename, mode ) )
    {
     cs_copy( filename, result );
     return( UT_TRUE );
    }
   }   

  return( UT_FALSE );
}

logical fio_pathlist_search( PathList pathlist, char* name, char* mode, char* result )
{

  logical global;
  logical local;
  logical absolute;
  logical q;

  cs_copy( " ", result );
  global = strpbrk(mode, ">") != NULL;  /* skip if > */
  local = strpbrk(mode, "<") != NULL;  /* pick if < */

  cs_copy( " ", result );
  if ( global && local ) return(UT_FALSE);
/* absolute path? */
  absolute = ( name[0] == '.' && name[1] == '/' ) || ( name[0] == '/' );
  if ( !pathlist || absolute )
    q = fio_pathlist_search_namelist( NULL, name, mode, result );

  if ( !q && !global )
    q = fio_pathlist_search_namelist( pathlist->local, name, mode, result );
  
  if ( !q && !local )
    q = fio_pathlist_search_namelist( pathlist->global, name, mode, result );

  return q;
}
