/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

/*----------------------------------------------------------------------*/
/*+af1_parse_fmt:   Parse a Fortran style format; return C format, data type and field length */
void cs_fs_format_parse( const char*  Fortran_Format, char* C_Format, char* DataType, int* Field_Length )
{
 char* ptr;
 int length; /* Field length */
 int dp;  /* Decimal points */
 int dp_pos;
 char type;
 TextCard tmp;

 type = *Fortran_Format;
 ptr =  (char*)(Fortran_Format + 1); /* Remove const */
 if ( cs_is_blank( ptr ) ) {
  length = ( type == 'A' ) ? 0 : 1;   /* Special case of 'A' meaning whole char* */
  dp = 0;
 } else {
  dp_pos = cs_index( ptr, "." );    /* cs_index returns the 1-based position in the char*, or 0 on failure */
  if ( dp_pos == 0 ) {   /* No decimal point */
   dp = 0;
   length = cs_read_i( ptr );   /* read_i is a fancier version of atoi */
  } else {
   cs_get_ss( ptr, tmp, 1, dp_pos-1 );  /* cs_get_ss extracts the subchar* into tmp (if only C did subchar*s...) */
   length = cs_read_i( tmp );   /* Read long up to position of decimal point */
   ptr += dp_pos;
   dp = cs_read_i( ptr );       /* Read long following decimal point */
  }
 }

 switch( type ) {
  case 'I' :                /*  Note that Fortran makes no distinction of %d and %ld */
   if ( dp > 0 ) {
    sprintf( C_Format, "%%%d.%dld ", length, dp );     /* I4.3 -> %4.3ld; */
   } else {
    sprintf( C_Format, "%%%dld", length );       /* I4 -> %4ld */
   }
   break;
  
  case 'A' :
   if ( length == 0 ) {
    sprintf( C_Format, "%%s" );     /* A -> %s, special case */
   } else { 
    sprintf( C_Format, "%%-%d.%ds ", length, length );     /* A4 -> %-4.4s */
   }
   break;
  
  case 'F' :
   sprintf( C_Format, "%%%d.%df ", length, dp );   /* F4.2 -> %4.2f */
   break;

  case 'E' :
   sprintf( C_Format, "%%%d.%dE ", length , dp );   /* E4.2 -> %4.2E */
   break;

  case 'G' :
   sprintf( C_Format, "%%%d.%dG ", length, dp );    /* G4.2 -> %4.2G */
   break;

  case 'Z' :
   if ( dp > 0 ) {
    sprintf( C_Format, "%%%d.%dX ", length, dp );     /* Z4.2 -> %4.2X;    */
   } else {
    sprintf( C_Format, "%%%dX", length );       /* Z4 -> %4X */
   }
   break;

  default:
   sprintf( "Failed to parse format %s", Fortran_Format );
   ut_err_util( ut_tmp, UT_ERR_UNKNOWN_FFMT );
   cs_copy( " ", C_Format );
   break; 
  }
 *DataType = type;
 *Field_Length = length;
}


