/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

/*
 * Safe mem free
 */

void ut_free( void* p )
{
 if ( p ) free( p );
}
/*
 * Allocate array of 3-vectors of length n 
 */

double** ar_valloc_d( const long n )
{
 double* p;
 long c;
 double** v;
 v = (double**)malloc( n * UT_SZ_D );
 p = (double *) malloc( n * 3 * UT_SZ_D );
 for ( c = 0; c < n; c++ ) {
  v[c] = p + 3 * c;
  ar_vzero_d( v[c] );
 }
 return( v );
}

/* Free an array of 3-vectors */

void ar_vfree_d( double** v )
{
 free( v[0] );
 free( v );
}

/* Print a 3-vector */

char* cs_vwritef_d( const double* p, const char* fmt, char* buf, const long siz )
{
 TextCard fbuf;
 TextCard tbuf;
 sprintf( fbuf, "%s %s %s", fmt, fmt, fmt );
 sprintf( tbuf, fbuf, p[0], p[1], p[2] );
 cs_copy_siz( tbuf, buf, siz );
 return( buf );
}

char* cs_vwritef_r( const float* p, const char* fmt, char* buf, const long siz )
{
 TextCard fbuf;
 TextCard tbuf;
 sprintf( fbuf, "%s %s %s", fmt, fmt, fmt );
 sprintf( tbuf, fbuf, p[0], p[1], p[2] );
 cs_copy_siz( tbuf, buf, siz );
 return( buf );
}


/* Test if a 3-vector is null */

logical ar_vnull_d( const double* v )
{
 return( v[0] == 0.0 && v[1] == 0.0 && v[2] == 0.0 );
}

logical ar_vnull_r( const float* v )
{
 return( v[0] == 0.0 && v[1] == 0.0 && v[2] == 0.0 );
}


/* 
 *  Rotate 2-D coordinates through a given angle.
 *  The angle is in degrees.
 */

void rotate2_d( const double x, const double y, const double roll, double* xx1, double* yy1 )
{
 *xx1 = x * dcosd( roll ) - y * dsind( roll );
 *yy1 = x * dsind( roll ) + y * dcosd( roll );
}


