/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "esc_types.h"
#include "fiolist.h"

/*----------------------------------------------------------------------*/
/*
 * esc_continue
 * 
 */

static logical esc_continue( char** ptr, char cont_char, char esc_char )
{
 long j;
 char* buf;
 buf = *ptr;
 j = strlen( buf );
 if( buf[j-1] == cont_char ) {
  if( buf[j-2] == esc_char ) {
  /*  Escaped cont char */
   buf[j-2] = cont_char;
   buf[j-1] = 0;
  } else {
  /* Have a continuation */
   *ptr = &buf[j-1];
   return( UT_TRUE );
  }
 }
 return( UT_FALSE );
}

/*----------------------------------------------------------------------*/
/*
 * iofs_read
 * Uses csproto.h
 * Uses <stdlib.h>: system
 * Reads a line, handling continuations and removal of tabs, and
 * supporting system escapes.
 */


logical fio_iostack_read( FioStack iofs, char* buf, char* prompt, long siz )
{
 logical cont_trap;  /* Continuation lines */
 logical loop;     
 FioFile file;
 FioFileList list = NULL;
 char sys_char, esc_char, cont_char;
 char* cmd;
 logical ios;

 if ( !iofs ) iofs = fio_get_stack();
 sys_char = esc_special_char( SYS_CHAR );
 esc_char = esc_special_char( ESC_CHAR );
 cont_char = esc_special_char( CONT_CHAR );
 fio_iostack_set_input( iofs, NULL, list );
 file = fio_iostack_file( iofs, FIO_INPUT );
 loop = UT_TRUE;
 while ( loop ) {
  if( fio_file_interactive( file ) ) 
  { 
   fio_tmsgl( prompt ); 
   fio_tmsgl( ": "); 
  }
  cont_trap = UT_TRUE;
  cmd = buf;
  while( cont_trap && loop ) {
   loop = fio_read_line( file, cmd, siz );
   ios = loop;
   if ( !loop )
   {
     if ( file && file->name )
      sprintf( ut_tmp, "EOF found on file %s", file->name );
     else
      cs_copy( ut_tmp, "EOF found" );
     ut_msg_d( UT_VERBOSE, ut_tmp );
   }
   if ( loop ) cont_trap = esc_continue( &cmd, esc_char, cont_char );
  } 

  /* end inner loop */
  /* OK we have a line including continuations */
  if ( *buf == sys_char ) {
   system( &buf[1] );
  } else {
   if ( buf[1] == esc_char && buf[2] == sys_char ) {
    cs_copy( &buf[1], buf );
   }
   loop = UT_FALSE;
  } /* end if sys char test */
 } /* end outer loop */

 if ( iofs->detab_flag ) cs_detab( buf );
 return( ios );
}




