/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#include <stdio.h>
#define UT_STATE_DEF 1
#define UT_DEFINE_GLOBALS 1
#include "utlib.h"
#include "jcmstate.h"


/*
 *----------------------------*
 * GLOBALS
 *----------------------------*
 */ 

/* Initialize the library */
void ut_init_jcmlib( void )
{
 ut_state();
}

void ut_free_jcmlib( void )
{
 UtilState state = ut_state();
 ut_state_free( state );
 jcmlib_state = NULL;
}

void ut_dbg_set( long g )
{
 ut_state_dbg_set( NULL, g );
}

long ut_dbg_get( void )
{
 UtilState state = ut_state();
 return state->global_dbg;
}

UtilState ut_state( void )
{
 if ( !jcmlib_state ) 
 {
  jcmlib_state = ut_state_alloc();
  ut_init_state( jcmlib_state );
 }
 return jcmlib_state;
}

/*
 *----------------------------*
 * UtilState object
 *
 *  state::alloc
 *  state::free
 * Get data members
 *  state::ver
 *  state::fio_list
 *  state::dbg_get
 *  state::error_global
 *  state::get_ver_data

 *  state::escapes
 *  state::nulls
 *  state::seed
 * Set data members
 *  state::dbg_set
 *----------------------------*
 */ 

/* Initialize the library state */

UtilState ut_state_alloc( void )
{
 UtilState state = calloc( 1, sizeof( struct UtilState_s ));
 return state; 
}

void ut_state_free( UtilState state )
{
 ut_version_data_free( ut_state_ver_data( state ));
 fio_list_free( ut_state_fio_list( state ) );
 ut_error_free_state( ut_state_error_global( state ));
 tpio_free( ut_state_tpio( state ));
 ut_null_free( ut_state_nulls(state));
 esc_free( state->escapes );
 free( state->ver );
 free( state );
}

/* User routine to return library version */
void ut_ver_jcmlib( char* ver, const long maxlen )
{
 UtilState state = ut_state();
 cs_copy_siz( state->ver, ver, maxlen );
}

/* Internal routine to initialize a state */
void ut_init_state( UtilState state )
{
 if ( !state ) return;
 state->ver = cs_dup( JCMLIBVER );
 ut_state_dbg_set( state, ut_constants.zero ); /* Refer to ut_constants */
 ut_state_ver_data( state );
 ut_state_fio_list( state );
 ut_state_tpio( state );
 ut_state_nulls( state );
 ut_state_error_global( state );
 state->seed = 0;
}

unsigned int* ut_state_seed( UtilState state )
{
 if ( !state ) state = ut_state();
 return &state->seed;
}

FioFileList ut_state_fio_list( UtilState state )
{
 if ( !state ) state = ut_state();
 if ( !state->fio ) state->fio = fio_list_init(0,0,0);

 return state->fio;
}

void ut_state_dbg_set( UtilState state, long g )
{
 if ( !state ) state = ut_state();
 state->global_dbg = g;
}

long ut_state_dbg_get( UtilState state )
{
 if ( !state ) state = ut_state();
 return state->global_dbg;
}


ErrorGlobal ut_state_error_global( UtilState state )
{
 if ( !state ) state = ut_state();
 if ( !state->error_global ) state->error_global = ut_error_init_state();
 return state->error_global;
}

UtilVersionData ut_state_ver_data( UtilState state )
{
 if ( !state ) state = ut_state();
 if ( !state->versions )
 { 
   state->versions = ut_version_data_create();
   ut_ver_init_data( state->versions, JCMLIB, JCMLIBVER );
 }
 return state->versions;
}

Tpio ut_state_tpio( UtilState state )
{
 if ( !state ) state = ut_state();
 if ( !state->tpio ) state->tpio = tpio_data_init();
 return state->tpio;
}




UtilEscData ut_state_escapes( UtilState state )
{
 if (!state) state = ut_state();
 if (!state->escapes ) state->escapes = esc_alloc();
 return state->escapes;
}

UtilNulls ut_state_nulls( UtilState state )
{
 if ( !state ) state = ut_state();
 if ( !state->nulls ) state->nulls = ut_null_alloc();
 return state->nulls;

}
