/*                                                                
**  Copyright (C) 2002-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include <limits.h>
#include <float.h>

void ar_mix_numeric_cast_generic( const void* in, const DataType intype, const DataType outtype, 
				  void* out, long inoffset, long outoffset, const long nvals );
void ar_mix_copy_c( char** tmp );
logical ar_mix_get_cast_elt( const MixArray array, const long offset, DataType ttype, 
			     DataType* typep, void* value );


void* ar_gen_alloc( const DataType type, const long n )
{
 long size = ar_mix_type_size( type );
 return calloc( n, size );
}

void ar_mix_numeric_cast( const void* in, const DataType intype, const DataType outtype, 
			  void* out )
{
  ar_mix_numeric_cast_generic( in, intype, outtype, out, 0, 0, 1 );
}

void ar_mix_numeric_cast_elt( const void* in, const DataType intype, const DataType outtype, 
			      void* out, long inoffset, long outoffset )
{
  ar_mix_numeric_cast_generic( in, intype, outtype, out, inoffset, outoffset, 1);
}

void ar_mix_numeric_cast_array( const void* in, const DataType intype, const DataType outtype, 
				void* out, const long nvals )
{
  ar_mix_numeric_cast_generic(in, intype, outtype, out, 0, 0, nvals);
}

/*
 * added generic casting function to consolidate code
 *
 */
void ar_mix_numeric_cast_generic( const void* in, const DataType intype, const DataType outtype, 
				  void* out, long inoffset, long outoffset, const long nvals )
{
  long ii = 0;
  switch( intype ) {                                  
      case DT_INT2: 
	switch( outtype )
	  {
	  case DT_INT2:   
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((short*)(out))[ii+outoffset] = (short)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  case DT_INT4:  
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((long*)(out))[ii+outoffset] = (long)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  case DT_REAL4:
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((float*)(out))[ii+outoffset] = (float)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  case DT_REAL8: 
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((double*)(out))[ii+outoffset] = (double)((short*)in)[ii+inoffset]; 
	    };
	    break;
	  case DT_UINT2: 
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((unsigned short*)(out))[ii+outoffset] = (unsigned short)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  case DT_UINT4:   
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((unsigned long*)(out))[ii+outoffset] = (unsigned long)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  case DT_UINT1:   case DT_BIT:   
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) {
		if ( ((short*)in)[ii+inoffset] >= DT_UINT1_MAX) 
		  ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
		else if ( ((short*)in)[ii+inoffset] <= DT_UINT1_MIN) 
		  ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
		else 
		  ((uchar*)(out))[ii+outoffset] = (uchar)((short*)in)[ii+inoffset]; 
	      }
	    };
	    break;
	  case DT_LOGICAL:
	    { 
	      for ( ii = 0; ii < nvals; ii++ ) 
		((logical*)(out))[ii+outoffset] = (logical)((short*)in)[ii+inoffset]; 
	    }; 
	    break;
	  default: out = NULL; break;                                    
	  }
	break;    


  case DT_INT4:     
    switch( outtype ) 
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((long*)in)[ii+inoffset] >= DT_INT2_MAX) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MAX;
	    else if ( ((long*)in)[ii+inoffset] <= DT_INT2_MIN) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MIN;
	    else 
	      ((short*)(out))[ii+outoffset] = (short)((long*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((long*)(out))[ii+outoffset] = (long)((long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((double*)(out))[ii+outoffset] = (double)((long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((long*)in)[ii+inoffset] >= DT_UINT2_MAX) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MAX;
	    else if ( ((long*)in)[ii+inoffset] <= DT_UINT2_MIN) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MIN;
	    else 
	      ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((long*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((long*)in)[ii+inoffset] >= DT_UINT1_MAX) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
	    else if ( ((long*)in)[ii+inoffset] <= DT_UINT1_MIN) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
	    else 
	      ((uchar*)(out))[ii+outoffset] = (uchar)((long*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((long*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_REAL4: 
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((float*)in)[ii+inoffset] >= DT_INT2_MAX) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MAX;
	    else if ( ((float*)in)[ii+inoffset] <= DT_INT2_MIN) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MIN;
	    else 
	      ((short*)(out))[ii+outoffset] = (short)((float*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((float*)in)[ii+inoffset] >= DT_INT4_MAX) 
	      ((long*)out)[ii+outoffset] = DT_INT4_MAX;
	    else if ( ((float*)in)[ii+inoffset] <= DT_INT4_MIN) 
	      ((long*)out)[ii+outoffset] = DT_INT4_MIN;
	    else 
	      ((long*)(out))[ii+outoffset] = (long)((float*)in)[ii+inoffset]; 
	  }
	};
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((float*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((double*)(out))[ii+outoffset] = (double)((float*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((float*)in)[ii+inoffset] >= DT_UINT2_MAX) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MAX;
	    else if ( ((float*)in)[ii+inoffset] <= DT_UINT2_MIN) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MIN;
	    else
	      ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((float*)in)[ii+inoffset]; 
	  }
	};
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((float*)in)[ii+inoffset] >= DT_UINT4_MAX) 
	      ((unsigned long*)out)[ii+outoffset] = DT_UINT4_MAX;
	    else if ( ((float*)in)[ii+inoffset] <= DT_UINT4_MIN) 
	      ((unsigned long*)out)[ii+outoffset] = DT_UINT4_MIN;
	    else 
	      ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((float*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((float*)in)[ii+inoffset] >= DT_UINT1_MAX) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
	    else if ( ((float*)in)[ii+inoffset] <= DT_UINT1_MIN) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
	    else 
	      ((uchar*)(out))[ii+outoffset] = (uchar)((float*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((float*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_REAL8:    
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_INT2_MAX) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_INT2_MIN) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MIN;
	    else 
	      ((short*)out)[ii+outoffset] = (short)((double*)in)[ii+inoffset];
	  }
	};
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_INT4_MAX) 
	      ((long*)out)[ii+outoffset] = DT_INT4_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_INT4_MIN) 
	      ((long*)out)[ii+outoffset] = DT_INT4_MIN;
	    else 
	      ((long*)(out))[ii+outoffset] = (long)((double*)in)[ii+inoffset]; 
	  }
	};
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_REAL4_MAX) 
	      ((float*)out)[ii+outoffset] = DT_REAL4_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_REAL4_MIN) 
	      ((float*)out)[ii+outoffset] = DT_REAL4_MIN;
	    else 
	      ((float*)(out))[ii+outoffset] = (float)((double*)in)[ii+inoffset]; 
	  }
	};
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_REAL8_MAX) 
	      ((double*)out)[ii+outoffset] = DT_REAL8_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_REAL8_MIN) 
	      ((double*)out)[ii+outoffset] = DT_REAL8_MIN;
	    else 
	      ((double*)(out))[ii+outoffset] = (double)((double*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_UINT2_MAX) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_UINT2_MIN) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MIN;
	    else 
	      ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((double*)in)[ii+inoffset]; 
	  } 
	};
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_UINT4_MAX) 
	      ((unsigned long*)out)[ii+outoffset] = DT_UINT4_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_UINT4_MIN) 
	      ((unsigned long*)out)[ii+outoffset] = DT_UINT4_MIN;
	    else 
	      ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((double*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((double*)in)[ii+inoffset] >= DT_UINT1_MAX) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
	    else if ( ((double*)in)[ii+inoffset] <= DT_UINT1_MIN) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
	    else 
	      ((uchar*)(out))[ii+outoffset] = (uchar)((double*)in)[ii+inoffset]; 
	  }
	};
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((double*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_UINT2:    
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((short*)(out))[ii+outoffset] = (short)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((long*)(out))[ii+outoffset] = (long)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((double*)(out))[ii+outoffset] = (double)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((unsigned short*)in)[ii+inoffset] >= DT_UINT1_MAX) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
	    else if ( ((unsigned short*)in)[ii+inoffset] <= DT_UINT1_MIN) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
	    else 
	      ((uchar*)(out))[ii+outoffset] = (uchar)((unsigned short*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((unsigned short*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_UINT4:    
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((unsigned long*)in)[ii+inoffset] >= DT_INT2_MAX) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MAX;
	    else if ( ((unsigned long*)in)[ii+inoffset] <= DT_INT2_MIN) 
	      ((short*)out)[ii+outoffset] = DT_INT2_MIN;
	    else 
	      ((short*)(out))[ii+outoffset] = (short)((unsigned long*)in)[ii+inoffset]; 
	  } 
	}; 
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((long*)(out))[ii+outoffset] = (long)((unsigned long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((unsigned long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ for ( ii = 0; ii < nvals; ii++ ) 
	  ((double*)(out))[ii+outoffset] = (double)((unsigned long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((unsigned long*)in)[ii+inoffset] >= DT_UINT2_MAX) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MAX;
	    else if ( ((unsigned long*)in)[ii+inoffset] <= DT_UINT2_MIN) 
	      ((unsigned short*)out)[ii+outoffset] = DT_UINT2_MIN;
	    else 
	      ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((unsigned long*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((unsigned long*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) {
	    if ( ((unsigned long*)in)[ii+inoffset] >= DT_UINT1_MAX) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MAX;
	    else if ( ((unsigned long*)in)[ii+inoffset] <= DT_UINT1_MIN) 
	      ((uchar*)out)[ii+outoffset] = DT_UINT1_MIN;
	    else 
	      ((uchar*)(out))[ii+outoffset] = (uchar)((unsigned long*)in)[ii+inoffset]; 
	  }
	}; 
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((unsigned long*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_UINT1: case DT_BIT:   
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((short*)(out))[ii+outoffset] = (short)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((long*)(out))[ii+outoffset] = (long)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((double*)(out))[ii+outoffset] = (double)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((uchar*)(out))[ii+outoffset] = (uchar)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((uchar*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  case DT_LOGICAL:  
    switch( outtype )
      {
      case DT_INT2:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((short*)(out))[ii+outoffset] = (short)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_INT4:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((long*)(out))[ii+outoffset] = (long)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL4:
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((float*)(out))[ii+outoffset] = (float)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_REAL8: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((double*)(out))[ii+outoffset] = (double)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT2: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned short*)(out))[ii+outoffset] = (unsigned short)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT4:   
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((unsigned long*)(out))[ii+outoffset] = (unsigned long)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_UINT1: case DT_BIT:  
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((uchar*)(out))[ii+outoffset] = (uchar)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      case DT_LOGICAL: 
	{ 
	  for ( ii = 0; ii < nvals; ii++ ) 
	    ((logical*)(out))[ii+outoffset] = (logical)((logical*)in)[ii+inoffset]; 
	}; 
	break;
      default: out = NULL; break;                                    
      }
    break;
  default: out = NULL; break;                                         
  } 
}


/*
 *  Cast from one type to another, with allocation copy in case of text
 */
void ar_mix_cast_array( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals )
{
 double dval;
 double* dptr = &dval;
 char* rem;
 TextCard buf;
 long i;
 if ( intype == DT_CHAR )
 {
  if ( outtype == DT_CHAR )
  {
   for ( i = 0; i < nvals; i++ )
    ((char**)out)[i] = cs_dup( ((char**)in)[i] );
  }
  else
  {
   for ( i = 0; i < nvals; i++ )
   {
    dval = strtod( ((char**)in)[i], &rem );
    ar_mix_numeric_cast_elt( dptr, DT_REAL8, outtype, out, 0, i );
   }
  }
 } 
 else if ( outtype == DT_CHAR )
 {
  for ( i = 0; i < nvals; i++ )
  {
   ar_mix_numeric_cast_elt( in, intype, DT_REAL8, dptr, i, 0 );
   sprintf( buf, "%f", dval );
   ((char**)out)[i] = cs_dup( buf );
  }
 }
 else 
  ar_mix_numeric_cast_array( in, intype, outtype, out, nvals );
}


MixArray ar_mix_alloc( const long n )
{
 return ar_mix_alloc_type( DT_UNK, 0, n );
}


MixArray ar_mix_alloc_type( const DataType type, const long tsize, const long n )
{
 MixArray array;
 long size = tsize;
 if ( size == 0 )
  size = ar_mix_type_size( type );
 array = (MixArray)calloc( 1, sizeof( struct MixArray_s ) );
 if ( !array ) return NULL;
 array->type = type;
 array->types= NULL;
 array->data = NULL;
 if ( n > 0 )
 {
  array->data = (void*)calloc( n, size );
  if ( type == DT_UNK )
  {
   array->types= (DataType*)calloc( n, sizeof( DataType ));
  } 
 }
 array->size = size;
 array->n    = 0;
 array->nmax = n;
 return array;
}

long ar_mix_array_size( const MixArray array )
{
 if ( array )
  return array->size;
 else
  return 0;
}

/* Calloc-like realloc */
void* ut_realloc( void* data, long const old, long const new, const long size )
{
 void* ndata;
 if( new <= old )
  ndata = data;
 else
 {
  ndata = realloc( data, new * size );
  memset( (char*)ndata+old*size, 0, (new-old)*size );
 }
 return ndata;
}
/*
 *  Extend a Mixed type array
 */

logical ar_mix_extend( MixArray array, const long n )
{
 long nmax;
 if ( !array ) return UT_FALSE;

 nmax = array->nmax; 
 array->data = (void*)ut_realloc( array->data, nmax, n, array->size );

 if ( !array->data ) return UT_FALSE;

 if ( array->type == DT_UNK )
 {
  array->types= (DataType*)ut_realloc( array->types, nmax, n, sizeof( DataType ) );
  if ( !array->types ) return UT_FALSE;
 }

 array->nmax = n;

 return UT_TRUE;
}

void ar_mix_free( MixArray array )
{
 long offset;
 void* ptr;
 char* str;
 DataType type;

 if ( array ) 
 {
  type = array->type;
  for ( offset = 0; offset < array->n; offset++ )
  {
   if ( array->type == DT_UNK ) 
    type = array->types[offset];

   if ( type == DT_CHAR )
   {
    ptr = ar_mix_ptr( array, offset );
    str = *(char**)ptr;

    if ( str ) free( str );

   }
  }

  free( array->data );

  if ( array->types )
   free( array->types );

  free( array );
 }
}

void ar_mix_copy_c( char** tmp )
{
 *tmp = cs_dup( *tmp );
}


void ar_mix_cast_elt( MixArray array, const long offset, const DataType type, void* value )
{
 DataType mixType;
 ar_mix_get_cast_elt( array, offset, type, &mixType, value );
}
   
int ar_mix_cast_elt_cp( MixArray array, const long offset, char* value )
{
 GenericType tmp;
 DataType mixType;

 if ( !value )
  return UT_FALSE;

 ar_mix_get_elt( array, offset, &mixType, (void*)&tmp );
 if ( mixType == DT_CHAR )
 {
  cs_copy( *(char**)&tmp, value );
  return UT_TRUE;
 }
 else
 {
  *value = '\0';
  return UT_FALSE;
 }
}
   
/*
 *  Print a given element of an array of specified type
 *  leaving the char* in the user-provided buf and returning a pointer
 *  to that buf. Debug routine.
 */

char* ar_mix_print_elt( const MixArray array, const long offset, char* buf, long size )
{
 void* ptr;
 DataType type;

 if ( !array || !buf )
   return NULL;

 ptr = (void*)ar_mix_ptr( array, offset );
 if ( !ptr ) 
  return NULL;


 type = array->type;
 if ( type == DT_UNK )
  type = array->types[offset];

 ar_mix_print_generic( ptr, type, buf, size );

 return buf;
}

void ar_mix_print_generic( void* ptr, DataType type, char* buf, long size )
{
 switch( type )
 {
  case DT_INT2:
   sprintf( buf, "%d", *(short*)ptr );
   break;
  case DT_UINT2:
   sprintf( buf, "%u", *(unsigned short*)ptr );
   break;
  case DT_UINT4: 
   sprintf( buf, "%lu", *(unsigned long*)ptr );
   break;
  case DT_UINT1: case DT_BIT:
   sprintf( buf, "%d", *(uchar*)ptr );
   break;
  case DT_INT4:
   sprintf( buf, "%ld", *(long*)ptr );
   break;
  case DT_REAL4:
   sprintf( buf, "%f",  *(float*)ptr );
   break;
  case DT_REAL8:
#if 0
   sprintf( buf, "%g", *(double*)ptr );
#endif
/* DM1 compat */
   { 
   double x = *(double*)ptr;
   double abs_x = fabs(x);
   if ( abs_x < 1.0E-3 || abs_x > 1.0E9 ) {
    sprintf( buf, "%G", x );
   } else {
    if ( abs_x < 1.0E6 ) {
     sprintf( buf, "%16.8f", x );
    } else {
     sprintf( buf, "%20.8f", x );
    }
   }
   }
   break;
  case DT_LOGICAL:
   sprintf( buf, "%c", *(logical*)ptr ? 'T' : 'F' );
   break;
  case DT_CHAR:
   cs_copy_siz( *(char**)ptr, buf, size );
   break;
  case DT_PTR:
   cs_copy_siz( *(char**)ptr, buf, size );
   break;
  default:
   cs_copy( " ", buf );
   break;
 }
 return;
  
}

/*
 *  Set element number offset+1 of the array given the element's type
 *  In the case of a char* value, *(char**)value must point to
 *  allocated memory. A new copy of the char* is not made.
 *  The routine frees any char* previously stored
 *  in that mixed array element.
 */
logical ar_mix_set_elt( MixArray array, const long offset, const DataType type, const void* value )
{
 void* ptr;
 char* str;
 long size;

 if ( !array || !array->data )
   return UT_FALSE;

 if ( offset < 0 || offset > array->n )
  return UT_FALSE;

 if ( offset == array->n )
 {
  /* Allocate ample extra space if needed */
  if( offset >= array->nmax ) 
      ar_mix_extend( array, 2 * offset );
  array->n++;
 }
 size = array->size;
 if ( array->type == DT_UNK )
 {
  array->types[offset] = type;
  size = ar_mix_type_size( type );
 }
 else if ( array->type != type )
   return UT_FALSE;

 ptr = (void*)ar_mix_ptr( array, offset );
 if ( type == DT_CHAR )
 {
  str = *(char**)ptr;
  if ( str ) free( str ); 
 }
 ar_mix_copy_value( value, type, ptr, UT_FALSE, size );
 return UT_TRUE;
}

void* ar_mix_elt( const MixArray array, const long offset )
{
 void* ptr = NULL;
 DataType type;
 ar_mix_get_elt( array, offset, &type, &ptr );
 if ( type != DT_PTR )
  return NULL;
 else
  return ptr;
}
/*
 *  Get the value of element number offset+1 of the array
 */
logical ar_mix_get_elt( const MixArray array, const long offset, DataType* typep, void* value )
{
 return ar_mix_get_cast_elt( array, offset, DT_UNK, typep, value );
}

logical ar_gen_set_elt( void* array, const long offset, const DataType type, void* value )
{
 long size;
 void* ptr;

 size = ar_mix_type_size( type );
 ptr = ar_gen_ptr( array, type, size, offset );
 if ( !value  || offset < 0 ) 
 {
  memset( ptr, 0, size );
  return UT_FALSE;
 }
 ar_mix_copy_value( value, type, ptr, UT_FALSE, size );
 return UT_TRUE;
}


/* Simple call with no checking */
logical ar_gen_get_elt( const void* array, const long offset, const DataType type, void* value )
{
 return ar_gen_get_cast_elt( array, offset, offset+1, type, type, value );
}

logical ar_gen_get_cast_elt( const void* array, const long offset, const long n, const DataType array_type, const DataType type, void* value )
{
 long size;
 void* ptr;

 if ( !value ) return UT_FALSE;
 size = ar_mix_type_size( type );
 if ( !array  || offset < 0 || offset > n ) 
 {
  memset( value, 0, size );
  return UT_FALSE;
 }
 if ( type == array_type )
 {
  ptr = ar_gen_ptr( array, type, size, offset );
  ar_mix_copy_value( ptr, type, value, UT_FALSE, size );
 } else {
  size = ar_mix_type_size( array_type );
  ptr = ar_gen_ptr( array, array_type, size, offset );
  if ( type == DT_CHAR )
   ar_mix_copy_c( (char**)ptr );
  ar_mix_numeric_cast( (void*)ptr, array_type, type, value );
 }
 return UT_TRUE; 
}

logical ar_mix_get_cast_elt( const MixArray array, const long offset, DataType ttype, DataType* typep, void* value )
{

 void* ptr = NULL;
 long size;
 logical q = UT_TRUE;
 logical in_range = UT_TRUE;
 DataType type;

 if ( !value ) return UT_FALSE;

 if ( !array || !array->data )
   q = UT_FALSE;

 if ( q && (offset < 0 || offset >= array->n ))
   in_range = UT_FALSE;

 if ( !q )
 {
  type = DT_UNK;
  size = UT_SZ_P;
 }
 else if ( array->type == DT_UNK && in_range )
 {
  type = array->types[offset];
  size = ar_mix_type_size( type );
 }
 else 
 {
  type = array->type;
  size = array->size;
 }
 if ( q && in_range )
  ptr = ar_mix_ptr( array, offset );

 if ( type == ttype || ttype == DT_UNK )
 {
  ar_mix_copy_value( ptr, type, value, UT_FALSE, size );
 }
 else
 { 
  if ( type == DT_CHAR )
   ar_mix_copy_c( (char**)ptr );
  ar_mix_numeric_cast( (void*)ptr, type, ttype, value );
 }

 if( typep )
  *typep = type;

 return ( q && in_range );
}


/*
 *  Copy a mixed type value; if copyText is true and type is text, copy the char*, else just the pointer.
 */
void ar_mix_copy_value( const void* ptr, const DataType type, void* value, logical copy, long size )
{

 if ( !ptr && value ) 
 {
  /* Generic set to zero */
  memset( value, 0, size );
  return;
 }

 if ( type == DT_CHAR  )
 {
  if ( copy ) 
  {
   *(char**)value = cs_dup( *(char**)ptr );
  }
  else
  {
   *(char**)value = *(char**)ptr;
  }

 }
 else
 {
   /* !DT_CHAR */
   if ( size == 0 )  
     size = ar_mix_type_size( type );

   if ( ptr ) 
   {
     if ( ptr != value )
       memcpy( value, ptr, size );
   }
   else
   {
     memset( value, 0, size );
   }
 }
}


/*
 *  Return pointer to array of types, if array is mixed.
 */
DataType* ar_mix_array_types( MixArray array )
{
 if ( array )
  return array->types;
 else
  return NULL;
} 
/*
 *  If we change the type of an element to char*,
 *  and then set it, there's a problem if the pointer is
 *  not preset to null since SetMixElt will try and
 *  free it.
 *
 */
void ar_mix_delete_elt( MixArray array, const long offset )
{
 void* ptr;
 char* str;
 DataType type;

 type = array->type;
 if ( type == DT_UNK )
  type = array->types[offset];

 ptr = ar_mix_ptr( array, offset );

 switch( type )
 {
  case DT_CHAR:
   str = *(char**)ptr;
   if ( str ) free( str );
  case DT_PTR: /* Fallthrough intended */
   *(void**)ptr = NULL;
   break;
  case DT_REAL8:
   *(double*)ptr = 0.0;
   break;
  case DT_REAL4:
   *(float*)ptr = 0.0;
   break;
  case DT_INT4:
  case DT_UINT4:
   *(long*)ptr = 0;
   break;
  case DT_INT2:
  case DT_UINT2:
   *(short*)ptr = 0;
   break;
  case DT_BIT:
  case DT_UINT1:
  default:
   *(uchar*)ptr = 0;
   break;
 }
}  


/* 
 *  Copy a mixed array element from one mixed array to another (or to
 *  another location in the same one ).
 */
void ar_mix_copy_elt( const MixArray src, const long srcoffset, MixArray dest, const long destoffset )
{ 
 GenericType tmp;    /* Storage to copy element */
 DataType type;
 ar_mix_get_elt( src, srcoffset, &type, (void*)&tmp );
 if( type == DT_CHAR )
  ar_mix_copy_c( (char**)&tmp );
 ar_mix_set_elt( dest, destoffset, type, (void*)&tmp );
}

/*
 *  Copy a mixed array element from one mixed array to another (or to
 *  another location in the same one ). Zero the pointer in
 *  the source array (as in etgDeleteElt) but don't free any associated
 *  char* memory.
 */
void ar_mix_move_elt( MixArray src, const long srcoffset, MixArray dest, const long destoffset )
{
 void* ptr;
 GenericType tmp;    /* Storage to copy element */
 DataType type;
 ar_mix_get_elt( src, srcoffset, &type, (void*)&tmp );
#if 0
 if( type == DT_CHAR )
  ar_mix_copy_c( (char**)&tmp );
#endif
 ar_mix_set_elt( dest, destoffset, type, (void*)&tmp );
 ptr = ar_mix_ptr( src, srcoffset );
 *(char**)ptr = NULL;
}



/*
 *  Return a pointer to element number offset+1 of the array
 */
void* ar_mix_ptr( const MixArray array, const long offset )
{
 if ( !array || !array->data || offset > array->nmax )
  return NULL;

/* Auto-extend if needed */
 if ( offset == array->nmax )  
  ar_mix_extend( array, 2 * offset );

 return ar_gen_ptr( array->data, array->type, array->size, offset );
}

void* ar_gen_ptr( const void* data, const DataType type, const long size, const long offset )
{
 if ( !data )
  return NULL;

 if ( offset < 0 )
  return NULL;

 switch( type )
 {
  case DT_UNK:
   return &((GenericType*)data)[offset];
  case DT_INT2:
   return &((short*)data)[offset];
  case DT_UINT2:
   return &((unsigned short*)data)[offset] ;
  case DT_UINT4:
   return &((unsigned long*)data)[offset];
  case DT_UINT1: case DT_BIT:
   return &((uchar*)data)[offset];
  case DT_INT4:
   return &((long*)data)[offset];
  case DT_REAL4:
   return &((float*)data)[offset];
  case DT_REAL8:
   return &((double*)data)[offset];
  case DT_LOGICAL:
   return &((logical*)data)[offset];
  case DT_CHAR:
   return &((char**)data)[offset];
  case DT_PTR:
  default:
   return ((char*)data + offset * size);
 }

}
 
   


#if 0
/* Obsolete alternate macros */

/*
 *  Given an array of type T, return element value
 */
#define ar_mix_cast_array_ptr( type, array, offset )    ((type*)(array))[offset]
#define ar_mix_cast_void_ptr( type, value )    *((type*)(value))

#define ar_mix_numeric_explicit_array_cast_macro( in, intype, outtype, out, nvals )  \
   { long i = 0; \
     for ( i = 0; i < nvals; i++ ) \
      ar_mix_cast_array_ptr( outtype, out, i ) = (outtype)ar_mix_cast_array_ptr( intype, in, i ); \
   }

#define ar_mix_numeric_explicit_cast_macro( in, intype, outtype, out ) { \
      ar_mix_cast_void_ptr( outtype, out ) = (outtype)ar_mix_cast_void_ptr( intype, in ); \
   }

#endif


void ar_mix_type_name( const DataType type, char* buf, long maxlen )
{
 char* names[] = {  "Void", "Int2", "Ptr", "Int4", "Real4", "Real8", "Text", "Byte", 
                    "Logical", "UNKNOWN", "UInt2", "UInt4", "Bit" }; 
 long nmax = 13;
 long offset;
 
 offset = (long)type;
 if ( offset < 0 || offset >= nmax )
  offset = 0;
 cs_copy_siz( names[offset], buf, maxlen );
}
/*
 *  The size of DT_CHAR is UT_SZ_P not UT_SZ_C
 *  since we always pass in character strings as pointers to these routines.
 */

long ar_mix_type_size( const DataType type )
{
 switch( type )
 {
  case DT_UNK:   return UT_SZ_D;
  case DT_INT2:  return UT_SZ_S;
  case DT_INT4:  return UT_SZ_I;
  case DT_REAL4: return UT_SZ_R;
  case DT_REAL8: return UT_SZ_D;
  case DT_UINT2: return UT_SZ_S;
  case DT_UINT4: return UT_SZ_I;
  case DT_UINT1: return UT_SZ_B;
  case DT_LOGICAL: return UT_SZ_Q;
  case DT_CHAR:  return UT_SZ_P;
  case DT_PTR: return UT_SZ_P;
  case DT_BIT: return UT_SZ_B;
  default: return 0;
 }
}



logical ar_mix_is_min( const DataType type, void* value )
{
 if ( !value ) return UT_FALSE;
 switch( type )
 {
  case DT_INT2:
   return (*(short*)value == DT_INT2_MIN);
  case DT_UINT2:
   return (*(unsigned short*)value == DT_UINT2_MIN);
  case DT_UINT4:
   return(*(unsigned long*)value == DT_UINT4_MIN);
  case DT_UINT1:  
   return (*(uchar*)value == DT_UINT1_MIN);
  case DT_INT4:
   return( *(long*)value == DT_INT4_MIN);
  case DT_REAL4:
   return(*(float*)value == DT_REAL4_MIN);
  case DT_REAL8:
   return( *(double*)value == DT_REAL8_MIN);
  case DT_LOGICAL:
   return( *(logical*)value == UT_FALSE);
  case DT_CHAR:
   return (*(char**)value== NULL);
  case DT_PTR:
   return( *(char**)value== NULL);
  case DT_BIT:
   return (*(char*)value== 0);
  default:
   return UT_FALSE;
 }

}

/*
 *   Return the minimum legal value for a given data type
 */
void ar_mix_type_min( const DataType type, void* value )
{
 if ( !value ) return;

 switch( type )
 {
  case DT_INT2:
   *(short*)value = DT_INT2_MIN;
   break;
  case DT_UINT2:
   *(unsigned short*)value = DT_UINT2_MIN;
   break;
  case DT_UINT4:
   *(unsigned long*)value = DT_UINT4_MIN;
   break;
  case DT_UINT1:  
   *(uchar*)value = DT_UINT1_MIN;
   break;
  case DT_INT4:
   *(long*)value = DT_INT4_MIN;
   break;
  case DT_REAL4:
   *(float*)value = DT_REAL4_MIN;
   break;
  case DT_REAL8:
   *(double*)value = DT_REAL8_MIN;
   break;
  case DT_LOGICAL:
   *(logical*)value = UT_FALSE;
   break;
  case DT_CHAR:
   *(char**)value= NULL;
   break;
  case DT_PTR:
   *(char**)value= NULL;
   break;
  case DT_BIT:
   *(char*)value= 0;
   break;
  default:
   break;
 }

}

logical ar_mix_is_numeric( const DataType type )
{
 switch( type )
 {
  case DT_INT2:
  case DT_UINT2:
  case DT_UINT4:
  case DT_UINT1:  
  case DT_INT4:
  case DT_REAL4:
  case DT_REAL8:
  case DT_LOGICAL:
   return UT_TRUE;

  case DT_CHAR:
  case DT_PTR:
  case DT_BIT:
  default:
   return UT_FALSE;

 }
}

logical ar_mix_is_max( const DataType type, void* value )
{
 if ( !value ) return UT_FALSE;
 switch( type )
 {
  case DT_INT2:
   return (*(short*)value == DT_INT2_MAX );
  case DT_UINT2:
   return (*(unsigned short*)value == DT_UINT2_MAX );
  case DT_UINT4:
   return(*(unsigned long*)value == DT_UINT4_MAX );
  case DT_UINT1:  
   return (*(uchar*)value == DT_UINT1_MAX );
  case DT_INT4:
   return( *(long*)value == DT_INT4_MAX);
  case DT_REAL4:
   return(*(float*)value == DT_REAL4_MAX);
  case DT_REAL8:
   return( *(double*)value == DT_REAL8_MAX);
  case DT_LOGICAL:
   return( *(logical*)value == UT_TRUE);
  case DT_CHAR:
   return (*(char**)value== NULL);
  case DT_PTR:
   return( *(char**)value== NULL);
  case DT_BIT:
   return (*(unsigned char*)value== 0);
  default:
   return UT_FALSE;
 }

}


int ar_mix_eq( const DataType type, void* value1, void* value2 )
{
 int q;
 switch( type )
 {
  case DT_INT2:
   q = *(short*)value1 == *(short*)value2;
   break;
  case DT_UINT2:
   q = *(unsigned short*)value1 == *(unsigned short*)value2;
   break;
  case DT_UINT4:
   q=  *(unsigned long*)value1 == *(unsigned long*)value2;
   break;
  case DT_UINT1:
   q = *(uchar*)value1 == *(uchar*)value2;
   break;
  case DT_INT4:
   q =    *(long*)value1 == *(long*)value2;
   break;
  case DT_REAL4:
   q = *(float*)value1 ==  *(float*)value2;
   break;

  case DT_REAL8:
   q = *(double*)value1 == *(double*)value2;
   break;
  case DT_LOGICAL:
   q = *(logical*)value1 == *(logical*)value2;
   break;
  case DT_CHAR:
   q = cs_eq( *(char**)value1, *(char**)value2 );
   break;
  case DT_PTR:
   q = ( *(char**)value1 == *(char**)value2 );
   break;
  case DT_BIT:
/* A bit dubious - bit value array size not known. This tests only 1-byte */
   q = *(unsigned char*)value1 == *(unsigned char*)value2;
   break;
  default:
   q = UT_FALSE;
   break;
 }
 return q;

}

/*
 *  Return the maximum legal value for a given data type.
 */
void ar_mix_type_max( const DataType type, void *value )
{
 switch( type )
 {
  case DT_INT2:
   *(short*)value= DT_INT2_MAX;
   break;
  case DT_UINT2:
   *(unsigned short*)value = DT_UINT2_MAX;
   break;
  case DT_UINT4:
   *(unsigned long*)value = DT_UINT4_MAX;
   break;
  case DT_UINT1:
   *(uchar*)value = DT_UINT1_MAX;
   break;
  case DT_INT4:
   *(long*)value = DT_INT4_MAX;
   break;
  case DT_REAL4:
   *(float*)value = DT_REAL4_MAX;
   break;
  case DT_REAL8:
   *(double*)value = DT_REAL8_MAX;
   break;
  case DT_LOGICAL:
   *(logical*)value = UT_TRUE;
   break;
  case DT_CHAR:
   *(char**)value= NULL;
   break;
  case DT_PTR:
   *(char**)value= NULL;
   break;
  case DT_BIT:
   *(unsigned char*)value= (unsigned char)UCHAR_MAX;
   break;
  default:
   break;
 }
   
}
  



/*
 *   Return the zero value
 */
void ar_mix_type_zero( const DataType type, void* value )
{
 if ( !value ) return;

 switch( type )
 {
  case DT_INT2:
   *(short*)value = 0;
   break;
  case DT_UINT2:
   *(unsigned short*)value = 0;
   break;
  case DT_UINT4:
   *(unsigned long*)value = 0;
   break;
  case DT_UINT1:  
   *(uchar*)value = 0;
   break;
  case DT_INT4:
   *(long*)value = 0;
   break;
  case DT_REAL4:
   *(float*)value = 0;
   break;
  case DT_REAL8:
   *(double*)value = 0;
   break;
  case DT_LOGICAL:
   *(logical*)value = UT_FALSE;
   break;
  case DT_CHAR:
   *(char**)value= NULL;
   break;
  case DT_PTR:
   *(char**)value= NULL;
   break;
  case DT_BIT:
   *(char*)value= 0;
   break;
  default:
   break;
 }

}



long ar_mix_n( MixArray array )
{
 if ( array )
  return array->n;
 else
  return 0;
}


long ar_mix_nmax( MixArray array )
{
 if ( array )
  return array->nmax;
 else
  return 0;
}

