/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

NameList cs_namelist_alloc( long n, long size )
{
 NameList list;
 list = calloc( 1, sizeof( struct NameList_s ));
 list->nmax = n; 
 if ( n > 0 )
  list->data = calloc( n, UT_SZ_P );
 list->n = 0;
 list->size = size;
 if ( size && n )
  list->data[0] = calloc( n * ( size + 1 ), UT_SZ_C );
 return list;
}

void cs_namelist_free( NameList list )
{
 long i;
 long nfree = 1;
 if ( !list ) return;
 if ( !list->size )
  nfree = list->n;

 for ( i = 0; i < nfree; i++ )
   free( list->data[i] );

 free( list->data );
 free( list );
}


void cs_namelist_add( NameList list, const char* name )
{
 char* ptr;
 long i;
 if (  list->n >= list->nmax )
 {
  list->nmax = 2 * list->nmax + 10;
  list->data = realloc( list->data, list->nmax * UT_SZ_P );
  if ( list->size ) 
  {
   list->data[0] = realloc( list->data[0], list->nmax * (list->size+1) * UT_SZ_C ); 
   for ( i = 1; i < list->n; i++ )
    list->data[i] = list->data[0] + (i-1) * (list->size+1);
  }
 }
 if ( list->size )
 {
  ptr = list->data[0] + list->n * (list->size+1);
  cs_copy_siz( name, ptr, list->size );
  list->data[list->n] = ptr;
 } else {
  list->data[list->n] = cs_dup( name );
 }
 list->n++;

}

long cs_namelist_size( NameList list )
{
 return list->n;
}

long cs_namelist_entry_size( NameList list )
{
 return list->size;
}


char* cs_namelist_entry( NameList list, long i )
{
 char* ptr = NULL;
 if ( list && i > 0 && i <= list->n )
  ptr =  list->data[i-1];

 return ptr;
}

long cs_namelist_match( NameList list, const char* name )
{
 return ar_match_c( name, list->data, list->n );
}

/* Case insensitive match */
long cs_namelist_search( NameList list, const char* name )
{
 return ar_search_c( name, list->data, list->n );
}


logical cs_namelist_set( NameList list, const long entry, const char* name )
{
 logical q = UT_FALSE;
 if ( !list )
  return q;
 if ( entry <= 0 || entry > list->n + 1 )
  return q;

 q = UT_TRUE; 
 if( entry > list->n )
 {
  cs_namelist_add( list, name );
 } else if ( list->size ) {
  cs_copy_siz( name, list->data[entry-1], list->size );
 } else {
  free( list->data[entry-1] );
  list->data[entry-1] = cs_dup( name );
 }


 return q;

}
