/*                                                                
**  Copyright (C) 2002-2003,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "jcmstate.h"
#include "utnan.h"


#ifdef _POSIX_C_SOURCE
  #if _POSIX_C_SOURCE < 200112L
int isnan( double x );
/* 
 * The Single Unix Specification requires isnan to be defined, but
 * the CXC use of -DPOSIX_C_SOURCE disables it. 
 */

int isnan( double x )
{
 return dmt_dNAN(x);
}
  #endif
#endif

/* 
 * We support NAN-like values even for longs 
 */

/*
 *------------------------------
 * GLOBALS
 *------------------------------
 */

void ut_null_init( void )
{
 ut_null_state_init( NULL );
}

void ut_null_set_ieee( void )
{
 ut_null_state_ieee( NULL );
}

void ut_null_set( char typecode, char* opt, char* pattern )
{
 ut_null_state_set( NULL, typecode, opt, pattern );
}

/*
 * NULLS
 *
 * null:alloc
 * null:state_init
 * null:free
 *Set  
 * null:set_safe
 * null:state_ieee
 * null:state_set
 *Get
 * null:null_<sird>
 * null:nan_<rd>
 * null:infinity_<sird>
 *Test
 * null:qnull_<sird>
 * null:qinf_<sird>
 */

UtilNulls ut_null_alloc( void )
{
 UtilNulls nulls = calloc( 1, sizeof( struct UtilNull_s ));
 ut_null_state_init( nulls );
 return nulls;
}

void ut_null_state_init( UtilNulls nulls )
{
 if ( !nulls ) nulls = ut_state_nulls( NULL );

 ut_null_state_set( nulls, 'S', "NULL", "8000" );
 ut_null_state_set( nulls, 'S', "INF",  "8001" );
 ut_null_state_set( nulls, 'S', "NINF", "8002" );
 ut_null_state_set( nulls, 'I', "NULL", "7FFFFFFF" );
 ut_null_state_set( nulls, 'I', "INF",  "7F800000" );
 ut_null_state_set( nulls, 'I', "NINF", "FF800000" );
 
 nulls->d_ieee_nan = ut_nan_create_d( UT_NAN_D_CXC );
 nulls->d_ieee_inf = ut_nan_create_d( UT_INF_D );
 nulls->d_ieee_ninf= ut_nan_create_d( UT_NINF_D );
 nulls->r_ieee_nan = ut_nan_create_r( UT_NAN_R_CXC);
 nulls->r_ieee_inf = ut_nan_create_r( UT_INF_R);
 nulls->r_ieee_ninf= ut_nan_create_r( UT_NINF_R);
 ut_null_state_ieee( nulls );
}

void ut_null_free( UtilNulls nulls )
{
 free( nulls );
}
/*
 * The old patterns; these work if NaNs cause your code to die
 */
void ut_null_set_safe( void )
{
 ut_null_set( 'R', "NULL", "77FFFFFF" );
 ut_null_set( 'D', "NULL", "77FFFFFFFFFFFFFF" );
 ut_null_set( 'R', "INF",  "7EFFFFFF" );
 ut_null_set( 'R', "NINF", "FEFFFFFF" );
 ut_null_set( 'D', "INF",  "7EFFFFFFFFFFFFFF" );
 ut_null_set( 'D', "NINF", "FEFFFFFFFFFFFFFF" );
}

/*
 *  The IEEE patterns
 */

void ut_null_state_ieee( UtilNulls nulls )
{
 if ( !nulls ) nulls = ut_state_nulls( NULL );
 nulls->d_null = nulls->d_ieee_nan;
 nulls->d_inf = nulls->d_ieee_inf;
 nulls->d_ninf= nulls->d_ieee_ninf;
 nulls->r_inf = nulls->r_ieee_inf;
 nulls->r_ninf= nulls->r_ieee_ninf;
 nulls->r_null= nulls->r_ieee_nan;
}

void ut_null_state_set( UtilNulls nulls, char typecode, char* opt, char* pattern )
{

 if ( !nulls) nulls = ut_state_nulls( NULL );
 if ( cs_uceq( opt, "NULL" ))
 {
  switch( typecode )
  {
    case 'S':
      nulls->s_null  = cs_read_binary_s( pattern, 'Z' );
      break;
    case 'I':
      nulls->i_null  = cs_read_binary_i( pattern, 'Z' );
      break;
    case 'R':
      nulls->r_null = cs_read_binary_r( pattern, 'Z' );
      break;
    case 'D':
      nulls->d_null = cs_read_binary_d( pattern, 'Z' );
      break;
    default:
      break;  
  }
 }


 else if ( cs_uceq( opt, "INF" ))
 {
  switch( typecode )
  {
    case 'S':
      nulls->s_inf  = cs_read_binary_s( pattern, 'Z' );
      break;
    case 'I':
      nulls->i_inf  = cs_read_binary_i( pattern, 'Z' );
      break;
    case 'R':
      nulls->r_inf = cs_read_binary_r( pattern, 'Z' );
      break;
    case 'D':
      nulls->d_inf = cs_read_binary_d( pattern, 'Z' );
      break;
    default:
      break;  
  }
 }


 else if ( cs_uceq( opt, "NINF" ))
 {
  switch( typecode )
  {
    case 'S':
      nulls->s_ninf  = cs_read_binary_s( pattern, 'Z' );
      break;
    case 'I':
      nulls->i_ninf  = cs_read_binary_i( pattern, 'Z' );
      break;
    case 'R':
      nulls->r_ninf = cs_read_binary_r( pattern, 'Z' );
      break;
    case 'D':
      nulls->d_ninf = cs_read_binary_d( pattern, 'Z' );
      break;
    default:
      break;  
  }
 }
 else if ( cs_uceq( opt, "NAN" ))
 {
  switch( typecode )
  {
    case 'S':
      break;
    case 'I':
      break;
    case 'R':
      nulls->r_ieee_nan = cs_read_binary_r( pattern, 'Z' );
      break;
    case 'D':
      nulls->d_ieee_nan = cs_read_binary_d( pattern, 'Z' );
      break;
    default:
      break;  
  }
 }
}



long ut_null_i( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->i_null );
}

short ut_null_s( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->s_null );
}

float ut_null_r( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->r_null );
}

double ut_null_d( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->d_null );
}

double ut_nan_d( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->d_ieee_nan );
}

float ut_nan_r( void )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( nulls->r_ieee_nan );
}


long ut_infinity_i( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );
 return( i >= 0 ? nulls->i_inf : nulls->i_ninf );
}

short ut_infinity_s( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( i >= 0 ? nulls->s_inf : nulls->s_ninf );
}


float ut_infinity_r( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( i >= 0 ? nulls->r_inf : nulls->r_ninf );
}

double ut_infinity_d( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( i >= 0 ? nulls->d_inf : nulls->d_ninf );
}


logical ut_qnull_i( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( i == nulls->i_null );
}

logical ut_qnull_s( const short i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 logical q;
 unsigned short ui = (unsigned short)i;
 unsigned short us = (unsigned short)nulls->s_null;
 q = (us == ui);
 return q;
}

logical ut_qnan_r( const float x )
{
 UT_INT32T* ptr = (UT_INT32T*)&x;
 UT_INT32T  val;
 if ( *ptr == UT_INF_R || *ptr == UT_NINF_R )
  return 0; /* Value is INF */

 val = *ptr & UT_NAN_R_MASK; 
 return( val == UT_NAN_R_MASK );
}

logical ut_qnan_d( const double x )
{
/* Use SUS function for efficiency: isnan is required to be in math.h */
 return (logical)isnan( x ); 
}

logical ut_qnull_d( const double x )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 if ( isnan( nulls->d_null ))
  return isnan( x );
 else
  return ( x == nulls->d_null );
}



logical ut_qnull_r( const float x )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 if ( ut_qnan_r( nulls->r_null ))
  return ut_qnan_r( x );
 else
  return ( x == nulls->r_null );
} 


logical ut_qinf_i( const long i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( i == nulls->i_inf || i == nulls->i_ninf );
}

logical ut_qinf_s( const short i )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 unsigned short ui = (unsigned short)i;
 unsigned short us = (unsigned short)nulls->s_inf;
 unsigned short uns = (unsigned short)nulls->s_ninf;
 return( ui == us || ui == uns );
}

logical ut_qinf_r( const float x )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( x == nulls->r_inf || x == nulls->r_ninf );
}
logical ut_qinf_d( const double x )
{
 UtilNulls nulls = ut_state_nulls( NULL );

 return( x == nulls->d_inf || x == nulls->d_ninf );
}

/* We create this once and store it for later use */
double ut_nan_create_d( UT_INT32T pattern )
{
 double x;
 short offset=UT_BIGEND;
 UT_INT32T* ptr;

 ptr = (UT_INT32T*)&x;
 ptr[offset]   = 0;
 ptr[1-offset] = pattern;
 return x;
}


float ut_nan_create_r( UT_INT32T pattern )
{
 float x;
 UT_INT32T* ptr;
 ptr = (UT_INT32T*)&x;
 *ptr = pattern;
 return x;
}

#if 0
FNANMASK 0x7F80 0x7F00  (VMS)  0x7F800000  val  0x7FC00000 (new)
DNANMASK 0x7FF0  0x7FE0 (VMS)  0x7FF00000  val  0x7FF80000 (new)
DINFMASK 0x7FF0
DNINFMASK 0xFFF0


BIG_END is opposite of BYTE_SWAPPED.
						sys/byteorder.h
               ALPHA UNIX                      BIG_END             NOT
              <nan.h>
                                                 A = 1 B=0 C=0     A=0 B=1 C=3
FNAN(X)
        unsigned int* ip = &x;	               ulong* sp = &x;
        unsigned int ival = *ip;                 sval = sp[0] & 0x7F00

        float if = (float)ival;                  if ( sval == 0x7F00 ) return 1
        if ( if == 0.0 )                         if ( sval == 0      ) return 2     /* Denorm underflow */
         return 0;
        else
         mask = ival & 0x7F800000
        if ( mask == 0x7F800000) return 1;
No      if  (mask == 0       ) return 2;
        else return 0;

          

DNAN(X)
        IsNANorINF(X)                          x=0? 0
					       ulong* sp = &x;

              			       sval = sp[B] & 0x7FF0
                                               if ( sval == 0x7F00 ) return 1
                                               if ( sval == 0      ) return 2
			
DINF(X)
        IsNANorINF(X) &&                       if ( sp[B] == 0x7FF0
        IsINF(X) &&
        IsPosNAN(X)
NINF(X)
        IsNANorINF(X) && 
        IsINF(X) &&
        IsNegNAN(X)
MAKE_DNAN(X)  
             dnan* np = (dnan*)&x;            ulong* up = (ulong*)&x;
             np->nan_parts.exponent = 0x7FF   up[B] = 0x7FF80000 up[A] = 0
             np->nan_parts.qnan_bit = 0x1
MAKE_FNAN(X)
             unsigned int* ip = &x;           up[0] = 0x7FF80000
             *ip = 0x7FC00000

             
#endif
