/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "jcmstate.h"
#include "fiolist.h"

FioPager fio_pager_init( void )
{
 FioPager pager = calloc( 1, sizeof( struct FioPager_s ));
 pager->nlines = 40;
 pager->line = 0;
 pager->quiet = UT_FALSE;
 pager->on = UT_FALSE;
 return pager;
}

void fio_pager_free( FioPager pager )
{
 free( pager );
}

FioPager fio_pager( FioFileList list )
{
 if ( !list )
  list = fio_get_list();
 return list->pager;
}

void fio_pager_print_state( FioFileList list )
{
 FioPager pager = fio_pager( list );
 TextCard buf;
 long last_line;
 TextWord state;

 last_line = pager->line;
 if ( pager->on ) {
  if ( pager->quiet ) {
   cs_copy( "Quiet", state );
  } else {
   cs_copy( "Active", state );
  }
 } else {
  cs_copy( "Off", state );
 }
 sprintf( buf, " Pager status:       %s", state ); fio_msg( buf );
 sprintf( buf, " Pager page size:    %ld", (long)pager->nlines ); fio_msg( buf );
 sprintf( buf, " Pager current line: %ld", (long)last_line );    fio_msg( buf );
 sprintf( buf, " Pager new current line: %ld", (long)pager->line );    fio_msg( buf );
 fio_msg( " " );
}

void fio_pager_set_length( FioFileList list, long page_length )
{
 FioPager pager = fio_pager( list );
 pager->nlines = page_length;
}

void fio_pager_set( FioFileList list, logical q )
{
 FioPager pager = fio_pager( list );
 pager->on = q;
 pager->quiet = UT_FALSE;
}


logical fio_pager_tty_read( FioFileList list, char* buf, long siz )
{
 FioPager pager = fio_pager( list );
 logical ios;
 ios = fio_get_line( NULL, buf, siz );
 pager->line = 0;
 pager->quiet = UT_FALSE;
 return( ios );
}


logical fio_pager_tty_write( FioFileList list, const char* buf, logical newline )
{
 FioPager pager = fio_pager( list );
 logical ios = UT_TRUE;
 if ( ! pager->quiet ) {
  if ( newline )
  {
   ios = fio_put_line( NULL, buf, UT_TRUE, UT_TRUE );
   pager->line = ( pager->line + 1 ) % pager->nlines;
   if ( pager->on && pager->line == 0 ) fio_tty_pager_query( list );
  } else {
   ios = fio_put_line( NULL, buf, UT_FALSE, UT_FALSE );
  }
 }
 return( ios );
}



void fio_tty_pager_query( FioFileList list )
{
 FioPager pager = fio_pager( list );
 char c;
 printf( "--Press Return to Continue--" );
 c = getc( stdin );
 if ( c == 'q' || c == 'Q' ) pager->quiet = UT_TRUE;
}

