/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/* Emulate F90 date and time */
#include <time.h>
#include <unistd.h>
#include "utlib.h"
/* This emulates the F90 routine date_and_time */

void sys_date_and_time( char* c_date, char* c_time, char* c_zone )
{
 struct tm *t;
 time_t now;
 TextWord t_date;
 TextWord t_time;
 TextWord t_zone;

 now = time( (time_t *)NULL );
 t = localtime( &now );
 strftime( t_date, WORD_SIZE, "%Y%m%d", t );
 strftime( t_time, WORD_SIZE, "%H%M%S.000", t );
 strftime( t_zone, WORD_SIZE, "%z", t );
 cs_copy( t_date, c_date );
 cs_copy( t_time, c_time );
 cs_copy( t_zone, c_zone );
}


void sys_today( char* buf, const long size )
{
 struct tm *t;
 time_t now;
 now = time( (time_t *)NULL );
 t = gmtime( &now );
 strftime( buf, size,"%Y %b %d %H:%M:%S", t );
}

void sys_time( char* buf, const long size )
{
 struct tm *t;
 time_t now;
 now = time( (time_t *)NULL );
 t = gmtime( &now );
 strftime( buf, size,"%H:%M:%S", t );
}


void sys_date( char* buf, const long size )
{
 struct tm *t;
 time_t now;
 now = time( (time_t *)NULL );
 t = gmtime( &now );
 strftime( buf, size,"%d/%m/%y", t );
}

void sys_wait( const long t )
{
 sleep( t );
}

void sys_getenv( const char* name, char* value )
{
 cs_copy( getenv( name ), value );
}

void sys_delete( const char* name )
{
 unlink( name );
}

void sys_cmdline( char** args, long nargs, char* cmdline, long csiz )
{
 long i;
 char* ptr;
 long siz;
 ptr = cmdline;
 for ( i = 1; i < nargs; i++ ) {
  siz = cs_ends( args[i] );
  siz = ( siz < csiz ) ? siz : csiz;
  if ( siz > 0 ) cs_copy_siz( args[i], ptr, siz );
  ptr += siz;
  *ptr = ' '; ptr++;
  csiz -= siz+1;
 }
 *ptr = '\0';
}

void sys_type( const char* name )
{
 TextBuf cmd;
 sprintf( cmd, "cat %s", name );
 system( cmd );
}

void sys( const char* cmd )
{
 system( cmd );
}


double sys_stamp( void )
{
 struct tm *t;
 time_t now;
 long j, k;
 double ddate;
 now = time( (time_t *)NULL );
 t = gmtime( &now );
 j = t->tm_sec + 100 * ( t->tm_min + 100 * t->tm_hour );
 k = t->tm_mday + 100 * ( t->tm_mon + 100 * t->tm_year );
 ddate = k + ( (double)j / 1.0E6 );
 return( ddate );
}


void ut_id_jcmlib( char* label, const char* ver )
{ 
 TextCard cdate;
 sys_today( cdate, CARD_SIZE );
 sprintf( label, "%s (%s)", cdate, ver );
}
