/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "esc_types.h"
/* Routines added 1997 Jun 24 */

/*
 * Process each line of an ASCII file with the function 
 * process_line, except for blank lines and lines whose first
 * character is #. This function must have a single char*
 * argument which is the input line it will process.
 * It returns void. Input lines have tabs converted to spaces
 * before they are sent to process_line.
 * The idea of this routine is that many operations involve
 * processing ASCII files on a line-by-line basis, except for
 * some comment lines, so this code fragment is repeated
 * pretty often.
 */
void fio_process_file( char* filename, void (*process_line)( char* ) )
{
 TextBuf line;
 char cmt;
 FioFile file;
 cmt = esc_special_char( CMT_CHAR );
 file =  fio_open_ar( filename );
 if ( !file) {
   sprintf( ut_tmp, "Failed to open process-file %s", filename );
   ut_msg_d( UT_VERBOSE, ut_tmp ); 
   return; 
 }
 while( fio_read_line( file, line, TEXT_SIZE ) ) {
  cs_detab( line );
  if ( !cs_is_blank( line ) && line[0] != cmt ) {
   process_line( line );
  }
 }
 fio_file_close( file );
}

/*
 * A variation on the previous function, with an extra void* argument
 * in both this call and the process_line function.
 * Typical example:
 *   process_file1( myfile, myfunction, mode );
 * ...
 *   void myfunction( char* mode, char* line );
 */
void fio_process_file1( char* filename, void (*process_line)( void*, char* ), void* ptr )
{
 TextBuf line;
 FioFile file;
 char cmt;
 cmt = esc_special_char( CMT_CHAR );
 file =  fio_open_ar( filename );
 if ( !file) { 
   sprintf( ut_tmp, "Failed to open process-file %s", filename );
   ut_msg_d( UT_VERBOSE, ut_tmp ); 
   return;
 }
 while( fio_read_line( file, line, TEXT_SIZE ) ) {
  cs_detab( line );
  if ( !cs_is_blank( line ) && line[0] != cmt ) {
   process_line( ptr, line );
  }
 }
 fio_file_close( file );
}


