/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"

/* Bisection and search routines */

/* Fortran convention */

/*
 *  x0 < x(1)     Return 1
 *  x0 > x(n)     Return n
 *  x0 = x(m)-e   Return m-1
 *  x0 = x(m)     Return m
 *  x0 = x(m)+e   Return m
 *  so  x(m) <= x0 < x(m+1)
 */

long ar_bisect_d( const double* x, const long n, const double x0 )
{
 /* Simple bisection search to find position of x0 in array x */
 long element;
 long offset;
 long ju, jl, jm;
 if ( x0 <= x[0] ) {
  element = 1;
 } else if ( x0 >= x[n-1] ) {
  element = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  element = jl;   /* ie (jl-1) + 1 */
  offset = element - 1;
  /* We know 
   *    x0 in  [x(element),x(element+1)]
   * ie    x[offset] and x[offset+1]
   * We want to return 
   *    x0 in [x(element),x(element+1))
   */
  if( x[offset + 1] == x0 ) element++;
 }
 return( element );
}

long ar_cbisect_d( const double* x, const long n, const double x0 )
{
 return( ar_bisect_d( x, n, x0 ) - 1 );
}

long ar_bisect_r( const float* x, const long n, const float x0 )
{
 long j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

long ar_cbisect_r( const float* x, const long n, const float x0 )
{
 return( ar_bisect_r( x, n, x0 ) - 1 );
}


long ar_bisect_s( const short* x, const long n, const short x0 )
{
 long j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

long ar_cbisect_s( const short* x, const long n, const short x0 )
{
 return( ar_bisect_s( x, n, x0 ) - 1 );
}


long ar_bisect_i( const long* x, const long n, const long x0 )
{
 long j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

long ar_cbisect_i( const long* x, const long n, const long x0 )
{
 return( ar_bisect_i( x, n, x0 ) - 1 );
}

