/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include "utlib.h"
#include "fio.h"
logical fio_write_q( FioFile file, logical* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_Q ) );
}

logical fio_write_d( FioFile file, double* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_D ) );
}

logical fio_write_r( FioFile file, float* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_R ) );
}

logical fio_write_i( FioFile file, long* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_I ) );
}

logical fio_write_s( FioFile file, short* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_S ) );
}

logical fio_write_c( FioFile file, char* buf, long n, long* nw )
{
 return( fio_write_sio( file, (void *)buf, n, nw, UT_SZ_C ) );
}


logical fio_write_sio( FioFile file, void* buf, long n, long* nw, size_t siz )
{
 FILE* fptr;
 long m;
 logical status;

 fptr = fio_file_get_fptr( file );
 m = fwrite( (void *)buf, siz, n, fptr );
 status = (m == n );
 fio_file_set_status( file, status );
 fio_delta_pos( file, m * siz );
 *nw = m;
 return( status );
}


logical fio_read_sio( FioFile file, void* buf, long n, long* nw, size_t siz )
{
 FILE* fptr;
 long m;
 logical status;

 fptr = fio_file_get_fptr( file );
 m = fread( (void *)buf, siz, n, fptr );
 status = ! ( feof( fptr ) || ferror( fptr ) );
 fio_file_set_status( file, status );
 fio_delta_pos( file, m * siz );
 *nw = m;
 return( status );
}

logical fio_read_q( FioFile file, logical* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_Q ) );
}

logical fio_read_d( FioFile file, double* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_D ) );
}

logical fio_read_c( FioFile file, char* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_C ) );
/* bug fix 1997 Mar 10 */
}

logical fio_read_r( FioFile file, float* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_R ) );
}

logical fio_read_i( FioFile file, long* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_I ) );
}

logical fio_read_s( FioFile file, short* buf, long n, long* nw )
{
 return( fio_read_sio( file, (void *)buf, n, nw, UT_SZ_S ) );
}



/*
 *  Currently support SIO files only
 */
logical fio_seek( FioFile file, long pos )
{
 if ( cs_eq_siz( file->type, "SIO", 3 ))
  return fio_seek_s( file, pos );

 return UT_FALSE;
}

logical fio_seek_s( FioFile file, long pos )
{
 FILE* fptr;
 logical status;

 fio_file_clear_eof( file );
 fptr = fio_file_get_fptr( file );
 status = fseek( fptr, pos, SEEK_SET );
 return( !status );
}



