/*                                                                
**  Copyright (C) 2002-2003,2005-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <math.h>


#define UT_NULL_S      0x8000
#define UT_INF_S       0x8001
#define UT_NINF_S      0x8002

#define UT_NULL_I      0x7FFFFFFF
#define UT_INF_I       0x7F800000
#define UT_NINF_I      0xFF800000

#define UT_NAN_R_MASK  0x7F800000
#define UT_NAN_D_MASK  0x7FF00000
#define UT_INF_D_MASK  0x7FF00000
#define UT_NINF_D_MASK 0xFFF00000

#define UT_NAN_R_CXC   0x7FC00000
#define UT_NAN_D_CXC   0x7FF80000

#define UT_NAN_R       0xFFC00000
#define UT_NAN_D       0xFFF80000

#define UT_NAN_R_OLD   0x77FFFFFF
#define UT_INF_R_OLD   0x7EFFFFFF
#define UT_NINF_R_OLD  0xFEFFFFFF

#define UT_NAN_D_OLD   0x77FFFFFFFFFFFFFF
#define UT_INF_D_OLD   0x7EFFFFFFFFFFFFFF
#define UT_NINF_D_OLD  0xFEFFFFFFFFFFFFFF

#define UT_INF_R       0x7F800000
#define UT_NINF_R      0xFF800000

#define UT_INF_D       0x7FF00000
#define UT_NINF_D      0xFFF00000

float ut_nan_create_r( UT_INT32T pattern );
double ut_nan_create_d( UT_INT32T pattern );

/* Code for POSIX (when isnan(x) not available) */

#if defined(__alpha) && defined(__unix__)
#include <nan.h>
/* test for double NaN */
#define dmt_dNAN(X) IsNANorINF(X)
#else

#if defined(__sparc) || defined(sparc)
/* Extract value of double high order word and cast to unsigned long */
/* 32 bit mask to catch double NaN or Inf */

#define DNANCAST(M) ( (unsigned int )(*(unsigned int*)&(M)) )
#else
/************************************************************
  Linux definitions
************************************************************/

#if defined(__APPLE__)
#include <machine/endian.h>
#else
#if defined (__CYGWIN32__)
#define BYTE_ORDER LITTLE_ENDIAN
#else
#include <endian.h>
#endif
#endif

#if BYTE_ORDER == LITTLE_ENDIAN
#define DNANCAST(M) ( (unsigned int )(*(((unsigned int*)&(M))+1)) )
#else
#define DNANCAST(M) ( (unsigned int )(*(((unsigned int*)&(M))  )) )
#endif

#endif

/* Test for double NaN */
#define dmt_dNAN(L) \
     (L == 0.0 ?  0 : \
      ( ( DNANCAST(L) & UT_NAN_D_MASK) == UT_NAN_D_MASK ?  1 : 0))
#endif
