/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */




/* UT STATE */
UtilState ut_state( void );
UtilState ut_state_alloc( void );
void ut_state_free( UtilState state );
void ut_init_state( UtilState state );
UtilEscData ut_state_escapes( UtilState state );
UtilNulls ut_state_nulls( UtilState state );
Tpio ut_state_tpio( UtilState state );
ErrorGlobal ut_state_error_global( UtilState state );
void ut_state_dbg_set( UtilState state, long g );
long ut_state_dbg_get( UtilState state );
FioFileList ut_state_fio_list( UtilState state );
unsigned int* ut_state_seed( UtilState state );

/* UT INIT */
void ut_version_data_free( UtilVersionData verdata );
UtilVersionData ut_version_data_create( void );
UtilVersionData ut_state_ver_data( UtilState state );
UtilVersionData ut_ver_data( void );
logical ut_ver_init_data( UtilVersionData verdata, const char* prog, const char* ver );

/* UT SYSTEM */
void sys_cmdline( char** args, long nargs, char* cmdline, long siz );

/* UT BITS */

void sys_byte_d( const long byte, long* segment, long* ibyte );

/* UT ERROR */
void ut_err_util( char* buf, long code );

void ut_error_free_state( ErrorGlobal error_global );
void ut_error_init_user( ErrorGlobal error_global, char* buf );
long ut_error_mask( ErrorGlobal eg );
ErrorClasses ut_error_class_global( ErrorGlobal error_global );
ErrorGlobal ut_error_init_state( void );
long ut_error_user_mask( void );
ErrorGlobal ut_error_global( void );

void ut_error_class_free( ErrorClasses list );

/* UT NULLS */
void ut_null_state_ieee( UtilNulls nulls );
void ut_null_state_set( UtilNulls nulls, char typecode, char* opt, char* pattern );
void ut_null_free( UtilNulls nulls );
UtilNulls ut_null_alloc( void );
void ut_null_state_init( UtilNulls nulls );

/* CS UNIT */
void unit_si_resolve( long n, char** unit, long* fac );
void unit_compose( char** base_unit, double* exp_unit, long* si_fac, long n_uo, logical texmode, char* unit, long siz );
void unit_compitem( char** ptr, char* item, double exp_unit, long si_fac, long* siz );
void unit_parse_mantissa( char* base_unit, double* exp_unit );
void unit_parse_exp( const char* unit_item, char* base_unit, double* exp_unit );

void unit_tmult( long* n1, char** base1, double* exponent1, long* si_fac1,
                 long* n2, char** base2, double* exponent2, long* si_fac2, long power );

void si_tmult( long fac1, double exponent1, long fac2, double exponent2, long* fac, double* mantissa );
long si_eval_r( float x );
long si_eval_d( double x );
void si_calc_d( double x, double* y, char* pfx, long* exponent, char* mode );
void si_calc_r( float x, float* y, char* pfx, long* exponent, char* mode );
void si_check( char* unit, long* factor, long* pos );


/* AR SORT */
void cheapsort_d( const double* x, const long n, long* jsort );	/* Use heapsort algorithm */
void cheapsort_i( const long* x, const long n, long* jsort );
void cheapsort_c( char** x, const long n, long* jsort );
void cheapsort_r( const float* x, const long n, long* jsort );	/* Use heapsort algorithm */
void cheapsort_s( const short* x, const long n, long* jsort );
void cshellsort_d( const double* x, const long n, long* jsort );	/* Use shellsort algorithm */
void cshellsort_i( const long* x, const long n, long* jsort );
void cshellsort_c( char** x, const long n, long* jsort );
void cshellsort_r( const float* x, const long n, long* jsort );	/* Use shellsort algorithm */
void cshellsort_s( const short* x, const long n, long* jsort );

void heapsort_d( const double* x, const long n, long* jsort );	/* Use heapsort algorithm */
void heapsort_i( const long* x, const long n, long* jsort );
void heapsort_c( char** x, const long n, long* jsort );
void shellsort_d( const double* x, const long n, long* jsort );	/* Use shellsort algorithm */
void shellsort_i( const long* x, const long n, long* jsort );
void shellsort_c( char** x, const long n, long* jsort );
void heapsort_r( const float* x, const long n, long* jsort );	/* Use heapsort algorithm */
void heapsort_s( const short* x, const long n, long* jsort );
void shellsort_r( const float* x, const long n, long* jsort );	/* Use shellsort algorithm */
void shellsort_s( const short* x, const long n, long* jsort );

/* FIO LIB */
void fio_apply_path( const char*, char*, long );
void fio_set_status_pos ( FioFile file, logical ios, long pos ) ;
void fio_set_eof( long file );			/*  Set file EOF condition */
void fio_file_clear( FioFile file );
void fio_file_set_type( FioFile file, char* type );
void fio_file_set_pos( FioFile file, long pos );
void fio_delta_pos( FioFile file, long pos );
void fio_line_reset( FioFile file );
logical fio_inquire_path( char* name, char* pathname, long siz );
long fio_checkin( FioFile file, const char* Name, char* Type, long Status, FILE* Fptr, long stream );
FILE* fio_file_fptr( long );

/* FIO STACK */
void fio_iostack_realloc( FioStack iofs, long n );
FioFile fio_stack_input( void );
FioFile fio_stack_output( void );
FioFile fio_stack_err( void );
void fio_iostack_get_entry( FioStack iofs, char* io_type, long level, char* buf, long* fid, char* status );
void fio_iostack_set_input( FioStack iofs, Tpio tpio, FioFileList list );
void fio_iostack_iclear( FioFileList list, FioStack iofs );
FioFile fio_iostack_input_entry( FioStack iofs, long level );
void fio_iostack_reinit( FioFileList list, FioStack iofs );

void fio_iostack_print_top( FioStack iofs, char* type, char* pfx );
void fio_iostack_print_stack( FioStack iofs, char* type, long top );
void fio_stack_reinit( void );

/* FIO SIO */
logical fio_seek_s( FioFile file, long pos );
/* FIO CIO */
void cio_set_raw( FioFileList list, long fd );
void cio_setup_raw( FioFileList f,  long fd );
void cio_clear_raw( FioFileList f, long fd );
long ccio_get( FioFileList f, long fd, char* ptr, long* fptr );
void ccio_set( FioFileList f, long fd, char* ptr, long* fptr );
void ccio_set_raw( char* ptr, long* fptr );

/* FIO ASR */
logical fio_put_line( FioFile file, const char* buf, logical strip, logical nl );
logical fio_get_line( FioFile file, char* buf, long siz );
void fio_pager_free( FioPager pager );
long fio_terminate_buf( long pos, const char* buf, char* buf2, logical strip, logical nl, long siz );
FioPager fio_pager_init( void );
FioPager fio_pager( FioFileList list );
void fio_tty_pager_query( FioFileList list );

/* FIO LIST */
void fio_list_free_path( FioFileList f );
FioFile fio_list_open_raw( FioFileList list );
long fio_list_alloc_fid( FioFileList f );
FioFile fio_list_alloc_file( FioFileList f );
FioFile fio_list_open_generic( FioFileList list, const char* filename, char* mode, long rec_bytes );
FioFile fio_cfile_alloc( FioFileList list, char* Name, char* Type, long fd );
FioFile fio_open_c( FioFileList list, char* filename, char* jmode, char* mode );
FioFile fio_file_init( FioFileList f, long fid );
FioFile fio_file_alloc( FioFileList list, const char* Name, char* Type, long Status, FILE* Fptr );

/* TPIO */
void tpio_free( Tpio tpio );
logical tpio_getbuf( Tpio tpio, FioStack iofs, char* prompt );
void tpio_cclear( Tpio tpio, long level );
void tpio_tp_token( Tpio tpio, FioFileList list, FioStack iofs, char* prompt, char** pptr, long mode, char* token, long siz );

void loop_echo( Tpio tpio, FioStack iofs );
void loop_end( Tpio tpio, FioStack iofs );
void loop_do( Tpio tpio, FioStack iofs );
void loop_goto( FioStack iofs, long addr );
void tpio_out( char* buf, logical append, logical syserr );					/* Set log file */
void tpio_tp_setbuf( Tpio tpio, FioStack iofs, char** arr, long n );
extern void tpio_tp_cc_start_check( Tpio tpio, char* buf, logical* mode );
extern void tpio_tp_cc_cmd(  Tpio tpio, FioFileList list, FioStack iofs, char* prompt, logical* mode, char* buf, long siz );
extern void tpio_tp_cc_check(  Tpio tpio, FioFileList list, FioStack iofs, char* prompt, logical* eot, logical* found_esc, char* buf, long siz );
extern void tpio_tp_cc_esc( Tpio tpio, FioFileList list, FioStack iofs, char* ecmd, char* prompt, logical* eot,  logical* found_esc, char* buf, long siz );
Tpio tpio_data_init( void );


/* TPIO ESC */

UtilEscData esc_alloc( void );
void esc_free( UtilEscData edata );
void esc_init_state( UtilEscData esc_data );
char esc_special_char( const long type );
char esc_get_default_char( const long type );
void esc_get_name( const long type, char* name );

