/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#include <stdio.h>
#include "utlib.h"
#include "jcmstate.h"


UtilVersionData ut_version_data_create( void )
{
  UtilVersionData verdata;
  verdata = calloc( 1, sizeof( struct UtilVersionData_s ));
  verdata->nver = 0;
  return verdata;
}

void ut_version_data_free( UtilVersionData verdata )
{
 ar_free_c( verdata->program );
 ar_free_c( verdata->version );
 free( verdata );
}

/*
 *   Add program prog, version ver to the version tree.
 *   Warns if same program loaded with different versions
 *   If verbose set, displays state
 */
logical ut_ver_init( const char* prog, const char* ver )
{
 return ut_ver_init_data( NULL, prog, ver );
}

logical ut_ver_init_data( UtilVersionData verdata, const char* prog, const char* ver )
{
 long i;
 TextCard buf;

 if ( !verdata ) verdata = ut_ver_data( );
 if ( verdata->program == NULL ) {
  verdata->program = ar_alloc_c( CARD_SIZE, UT_NVER );
  verdata->version = ar_alloc_c( WORD_SIZE, UT_NVER );
  verdata->nver = 0;
 }
 i = ar_cmatch_c( prog, verdata->program, verdata->nver );
 if ( i >= 0  ) {
  if ( !cs_eq( ver, verdata->version[i] ) ) 
  {
    sprintf( ut_tmp, "JCMLIB WARNING: Package version mismatch for %s ", prog );
    ut_err_util( ut_tmp, UT_ERR_VER_INIT );
  }
  return( UT_TRUE );
 } else {
  if ( verdata->nver < UT_NVER-1 ) verdata->nver++;
  i = verdata->nver - 1;
  cs_copy( prog, verdata->program[i] );
  cs_copy( ver, verdata->version[i] );
  sprintf( buf, "Init %s %s\n", prog, ver );
  ut_msg_d( UT_VERBOSE, buf );
  return( UT_FALSE );
 }
}

/*
 *   Display the version tree to MSG
 */
void ut_ver_msg( void )
{
 TextCard buf;
 long i;
 UtilVersionData verdata = ut_ver_data( );

 for ( i = 0; i < verdata->nver; i++ ) {
  sprintf( buf, "%s %s", verdata->program[i], verdata->version[i] );
  fio_msg( buf );
 }
}


UtilVersionData ut_ver_data( void )
{
 return ut_state_ver_data( NULL );
}

