/*                                                                
**  Copyright (C) 1999-2001,2003-2004,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*****************************************************************/
/*                                                               */
/*   dmtcommon_ui.c                                              */
/*                                                               */
/*   Functions that are common between dmcopy and dmlist         */
/*   used for command line input                                 */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmtlib.h"
#include "dmtcommon.h"
#include "dmtcommon_ui.h"


/***************************************************************/
/*                                                             */
/* Function:   checkVerboseOption                              */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*   Called from functions dmlistCheckOptionString             */
/*                         dmcopyCheckOptionString             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*       char   *option                                        */
/*                                                             */
/*  Output parameters:                                         */
/*       int    *debug                                         */
/*                                                             */
/*  Returns:                                                   */
/*           none                                              */
/*                                                             */
/***************************************************************/
void checkVerboseOption(char *option, int *debug)
{
  char  number[10];
  char *tag = NULL;
  char *comma = NULL;
  char  cBlank = ' ';
  int   paramLen = strlen(option);
  int   ii = 0;
 
  if ( (tag = strstr(option, "VER=")) ||
       (tag = strstr(option, "VERBOSE=")) )
  {
    tag = strstr(option, "=");

    comma = strstr(option, ",");

    if (comma)
      paramLen = (comma-1) - (tag+1);

    strncpy(number, (tag+1), paramLen);

    *debug = atoi(number);
  }
  else
    *debug = 0;

  if ( comma )
    paramLen = comma - option;

   // remove from options string
   for ( ii = 0; ii < paramLen; ii++)
   {
     *(option+ii) = cBlank;
   }
}


/***************************************************************/
/*                                                             */
/* Function:   getDesignatedParameter                          */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*   Called from functions dmGetdmcopyInput()                  */
/*                         dmGetdmlistInput()                  */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*       char   *temp_in                                       */
/*                                                             */
/*  Output parameters:                                         */
/*       dmBool *got_inName                                    */
/*       dmBool *got_outName                                   */
/*       dmBool *got_option                                    */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*           none                                              */
/*                                                             */
/***************************************************************/
void getDesignatedParameter(char *temp_in, dmBool *got_inName,
                            dmBool *got_outName, dmBool * got_option)
{
  char *tag = NULL;

  // designated parameter - which one?

  tag = strstr(temp_in, "INFILE=");

  if ( tag )
  {
    *got_inName = dmTRUE;
    return;
  }

  tag = strstr(temp_in, "OUTFILE=");

  if ( tag )
  {
    *got_outName = dmTRUE;          
    return;
  }

  tag = strstr(temp_in, "OPT=");

  if ( tag )
  {
    *got_option = dmTRUE;          
  }

  // catch all
  tag = strstr(temp_in, "KER");

  if (!tag)
    tag = strstr(temp_in, "VER");

  if (!tag)
    tag = strstr(temp_in, "CLOB");

  if (!tag)
    tag = strstr(temp_in, "ROWS=");

  if (!tag)
    tag = strstr(temp_in, "CELLS=");

  if ( tag )
  {
    *got_option = dmTRUE;          
  }

  return;
}


/***************************************************************/
/*                                                             */
/* Function:  paramsInit                                       */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*   Called from functions dmcopy                              */
/*                         dmlist                              */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*        dmtools_params_s                                     */
/*                                                             */
/*  Output parameters:                                         */
/*          none                                               */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*           none                                              */
/*                                                             */
/***************************************************************/
void paramsInit(dmtools_params *params)
{
  *params->infile  = '\0';
  *params->outfile = '\0';
  *params->option  = '\0';
  *params->kernel  = '\0';
  *params->rows    = '\0';
  *params->cells   = '\0';
  params->clobber = dmFALSE;
  params->verbose = 0;
}
