/*                                                                
**  Copyright (C) 1998-1999,2003,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*****************************************************************/
/*                                                               */
/*   dmcopy.c                                                    */
/*                                                               */
/*   This is the callable version of the main program routine    */
/*   for tool dmcopy.                                            */
/*                                                               */
/*                                                               */
/* FILE NAME:		dmcopy.c                                 */
/*                                                               */
/* DEVELOPMENT: 	AXAF Science Center Tools (DataModel)    */
/*			jcm@cfa.harvard.edu (Initial version)    */
/*                                                               */
/* DESCRIPTION:		The AXAF Science Center dmcopy tool      */
/*                                                               */
/*****************************************************************/


/* trigger loading of dmnutils.h */
#ifndef DMLIB_INTERNAL
#define DMLIB_INTERNAL
#endif

#include "dmtlib.h"
#include "dmtcommon.h"
#include "new_dmcopy.h"
#include "dmtcommon_ui.h"




/***************************************************************/
/*                                                             */
/* Function:   dmcopy                                          */
/*                                                             */
/* Description:                                                */
/*   function for tool dmcopy                                  */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*           dmtools_params  *params                           */
/*                                                             */
/*  Output parameters:                                         */
/*           none                                              */
/*                                                             */
/*  Returns:                                                   */
/*           int return value from n_dmcopy()                  */
/*                                                             */
/***************************************************************/
int dmcopy(dmtools_params *params)
{
  char 	     infile[DC_MAXSTR];
  char 	     outfile[DC_MAXSTR];
  char 	     kerstr[DC_MAXSTR];
  char 	     optstr[DC_MAXSTR];

  int        debug    = 0;   /* external value for debug level */
  int        status   = dmSUCCESS;
  dmBool     bclobber = dmFALSE;

  // set defaults
  optstr[0] = '\0';
  strcpy(kerstr, "default");

  // copy input vavlues
  strncpy(infile, params->infile, DC_MAXSTR);
  strncpy(outfile, params->outfile, DC_MAXSTR);

  if ( !cs_is_blank(params->kernel) )
    strncpy(kerstr, params->kernel, DC_MAXSTR);

  if ( !cs_is_blank(params->option) )
    strncpy(optstr, params->option, DC_MAXSTR);

  debug = params->verbose;

  bclobber = params->clobber;


  if (debug > 0)
  {
    printf("The debug level is %d \n",debug);
  }

  dmaErrInit( "DMCOPY" );

  status = n_dmcopy( infile, outfile, optstr, kerstr, bclobber, debug );

  return status;
}


