/*                                                                
**  Copyright (C) 1998-2000,2007,2008  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */



/*****************************************************************/
/*                                                               */
/*   dmlist.c                                                    */
/*                                                               */
/*   This is the callable version of the main program routine    */
/*   for tool dmlist.                                            */
/*                                                               */
/*                                                               */
/* FILE NAME:		dmlist.c                                 */
/*                                                               */
/* DEVELOPMENT: 	AXAF Science Center Tools (DataModel)    */
/*			jcm@cfa.harvard.edu (Initial version)    */
/*                                                               */
/* DESCRIPTION:		The AXAF Science Center dmlist tool      */
/*                                                               */
/*****************************************************************/

/* trigger loading of dmnutils.h */
#ifndef DMLIB_INTERNAL
#define DMLIB_INTERNAL
#endif

#include "dmtlib.h"
#include "dmtcommon.h"
#include "dmlist.h"



/***************************************************************/
/*                                                             */
/* Function:   dmlist                                          */
/*                                                             */
/* Description:                                                */
/*   function for tool dmlist                                  */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*           dmtools_params  *params                           */
/*                                                             */
/*  Output parameters:                                         */
/*           none                                              */
/*                                                             */
/*  Returns:                                                   */
/*           int return value from n_dmlist()                  */
/*                                                             */
/***************************************************************/
int dmlist(dmtools_params *params)
{
  char input[DC_MAXSTR];     /* Input virtual file */
  char output[DC_MAXSTR];    /* Output log file */
  char rows[DCSIZE];         /* Row specification  min:max */
  char cells[DCSIZE];        /* Cell specification min:max */
  char opt[DC_MAXSTR];       /* Options to dmlist, comma-separated list */

  int  bclobber = 1;
  int  debug    = 0;

  cells[0]  = '\0';
  rows[0]   = '\0';
  output[0] = '\0';
  input[0]  = '\0';
  opt[0]    = '\0';


  /* Get the input name, output name, options and debug parameters */
  strncpy(input, params->infile, DC_MAXSTR);

  if ( !cs_is_blank(params->outfile) )
    strncpy(output, params->outfile, DC_MAXSTR);

  strncpy(opt, params->option, DC_MAXSTR);

  if ( !cs_is_blank(params->rows) )
    strncpy(rows, params->rows, DCSIZE);

  if ( !cs_is_blank(params->cells) )
    strncpy(cells, params->cells, DCSIZE);

  debug = params->verbose;


  if (debug > 1)
  {
    printf("The debug level is %d \n",debug);
    dmSetVerbose( debug );
  }

  dmaErrInit( "DMLIST" );

  return n_dmlist( input, output, opt, rows, cells, bclobber, debug );
}



