/*                                                                
**  Copyright (C) 1998-2000,2007,2008  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*****************************************************************/
/*   t_dmtlist.c                                                 */
/*                                                               */
/*   This is the callable version of the main program routine    */
/*   for tool dmtlist.                                           */
/*                                                               */
/* FILE NAME:		t_dmtlist.c                              */
/*                                                               */
/* DEVELOPMENT: 	AXAF Science Center DataModel Tools      */
/*                                                               */
/* DESCRIPTION:		IRAF task wrapper for ASC dmtlist tool.  */
/*                                                               */
/*****************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "dmtlib.h"
#include "dmtcommon.h"
#include "dmtlist_ui.h"
#include "dmlist_main.h"


/***************************************************************/
/*                                                             */
/* Function:           main                                    */
/*                                                             */
/* Description:                                                */
/*   main function for tool dmtlist                            */
/*                                                             */
/*  Input parameters:                                          */
/*     int    argc                                             */
/*     char **argv                                             */
/*                                                             */
/*  Output parameters:                                         */
/*     none                                                    */
/*                                                             */
/*  Returns:                                                   */
/*    EXIT_SUCCESS  - No Error                                 */
/*    EXIT_FAILURE  - Error                                    */
/*                                                             */
/***************************************************************/
int main( int argc, char** argv )
{
  dmtools_params params;       /* Tool parameters struct       */
  int status = EXIT_SUCCESS;   /* Error status                 */

  /* Note: this tool is does not use the CXC Error library,    */
  /*       Parameter interface or Stacks library.              */

  /* Init error library */
  status = dmaErrInit( argv[0] );
  if ( status != dmSUCCESS )
    return  EXIT_FAILURE;
  
  /* Clear params struct */
  paramsInit(&params);

  /* Load tool parameters                   */
  /*   (returns 1 if input is OK, 0 if bad) */
  status = dmGetdmlistInput(argc, argv, &params);
  if ( !status )
  {
    status = EXIT_FAILURE;
    dmaErrPrint("ERROR: Problem loading parameters.");
  }
  else
  {
    status = EXIT_SUCCESS;
    dmlist(&params);
  }

  exit(status);
}
