/*                                                                
**  Copyright (C) 1999-2001,2003-2004,2006-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* ***************************************************************************** */
/*  FILE NAME: 		@(#)dmlist.h	1.1 98/04/03 13:15:03                    */
/*                                                                               */
/*  DEVELOPMENT: 	AXAF Science Center DataModel Tools                      */
/*                                                                               */
/*  DESCRIPTION:	Header file for dmlist DM tool.                          */
/*                                                                               */
/* ***************************************************************************** */

#ifndef DM_DMLIST_H
#define DM_DMLIST_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>


/*                              */
/* Functions in dmlist.c        */
/*                              */
int dmlist(dmtools_params *params);


/*                              */
/* Functions in dmlist_blocks.c */
/*                              */

/*  dmDatasetListBlocks: List the blocks in a dataset, possibly opened 
 *  as a virtual file. The block opened as virtual is also passed in
 *  (it can be null), so that you get the info for the filtered block not
 *  the raw block.
 *  
 */
int dmlistDatasetListBlocks( 
 dmDataset* ds,                     /* Dataset to list */
 dmBlock* main_block,               /* This block will not be reopened */
 long main_block_no                 /* Number of the `main block' */
);

int  dmlistPrintBlockInfo( dmBlock* block, long blockno, char* buf, long* nrows );
int  dmlistPrintCounts( dmBlock* block );


/*                              */
/* Functions in dmlist_col.c    */
/*                              */

int  dmlistBlockListCols( dmBlock* block ) ;
int  dmlistBlockListRawCols( dmBlock* block );
void dmlistFormatType( dmDescriptor* dd, char* buf, long maxlen );


/*                              */
/* Functions in dmlist_data.c   */
/*                              */

void dmlistGetValueString( dmDescriptor* dd, long verbose,
                           dmDataType type, long mincell, long maxcell,
                           void* null, char* value, long maxlen );

/*  List info about data section for a block.
 *  Lists only filtered rows  minrow to maxrow, and for each
 *  array column lists only cell elements mincell to maxcell.
 */
int dmlistPrintArrayData( dmBlock* block, long nrows, long minrow,
                          long maxrow, long mincell, long maxcell, 
                          dmBool bRawOpt, long verbose );
int dmlistPrintData( dmBlock* block, long nrows, long minrow, long maxrow,
                     long mincell, long maxcell, dmBool bRawOpt, long verbose );


/*                                */
/* Functions in dmlist_format.c   */
/*                                */

long dmuPrint_d( double x, double* null, int vsigned, char* buf, int maxlen );
long Print_d( double x, char* nbuf );
long PrintSigned_d( double x, char* buf, long maxlen );
long PrintArrayItem_c( char** data, long offset, void* null, char* buf );
long PrintArrayItem_d( double* data, long offset, void* null, char* buf );
long PrintArrayItem_l( long* data, long offset, void* null, char* buf );
long PrintArrayItem_ul( unsigned long* data, long offset, void* null, char* buf );
void PrintVector_c( char** data, long dim, long n, int fancy, long spare,
                    char* value, long maxlen );
void PrintVector_d( double* data, long dim, long n, int mode, void* null,
                    char* value, long maxlen );
void PrintSignedVector_d( double* data, long dim, long n, int mode, void* null,
                          char* value, long maxlen );
void PrintVector_l( long* data, long dim, long n, int mode, void* null,
                    char* value, long maxlen );
void PrintVector_ul( unsigned long* data, long dim, long n, int mode,
                     void* null, char* value, long maxlen );
void PrintVector_q( dmBool* data, long dim, long nvals, int fancy,
                    long spare, char *value, long maxlen);

/*                              */
/* Functions in dmlist_funcs.c  */
/*                              */
int n_dmlist( char* input, char* output, char* opt, char* rows,
              char* cells, dmBool clobber, int debug );
dmBlock* dmListBlock( dmBlock* block, int dmverbose, void* vinfo, void* params );



/*                                */
/* Functions in dmlist_header.c   */
/*                                */
int dmlistPrintHeader( dmBlock* block, int verbose, int rk, int rc );
int dmlistPrintRawHeader( dmBlock* block, int v );



/*                              */
/* Functions in dmlist_misc.c   */
/*                              */
int  dmaBlockIsNull( dmBlock* block );
void dmaGetArray_c( dmDescriptor* dd, char** iarr, long mincell, long maxcell );
void dmaGetArray_d( dmDescriptor* dd, double* iarr, long mincell, long maxcell );
void dmaGetArray_l( dmDescriptor* dd, long* iarr, long mincell, long maxcell );
void dmaGetArray_ul( dmDescriptor* dd, unsigned long* iarr, long mincell,
                     long maxcell );
void dmaGetArray_q( dmDescriptor* dd, dmBool* barr, long mincell,
                     long maxcell );
void dmaGetBit( dmDescriptor* dd, long mincell, long maxcell, char* value,
                long maxlen );
int  dmaGetDataTypeName( dmDescriptor* dd, char* name, long maxlen );
void dmaListInfoInit( dmListInfo* info );
void dmaListInfoSet( dmListInfo* info, char* rows, char* cells, char* opt );
void dmaPixelNoToPixel( long pixelNo, long* axes, long naxes, long* pixel);
void dmlistBanner( char* buf );
void dmlistDashline( void ) ;
void dmlistFormatAxes( long* axes, long dim, char* sizebuf, long maxlen );
long dmlistGetColSize( dmDescriptor* dd );
int dmlistGetDescCompositeName( dmDescriptor* dd, char* name, long maxlen );
void dmlistParseRange( char* cells, long* umin, long* umax );
int  dmlistPrintBins( dmBlock* vblock, char* input, long minrow, long maxrow );
void dmlistSkip( void );
void dmlist_print( char* buf );
int  dmTableComponentList( dmBlock* table, dmDescriptor*** list,
                           long** dims, long** colno, long* ncols );


/*                              */
/* Functions in dmlist_range.c  */
/*                              */
int dmlistGetRange( dmDescriptor* dd, char* range );


/*                           */
/* Functions in dmlist_ss.c  */
/*                           */

/*  List info about the subspace. Slightly higher level than*/
/*  dmBlockPrintSubspace in the main library. */

int dmlistBlockPrintSubspace( dmBlock* block );


#endif
