/*                                                                
**  Copyright (C) 1999,2004,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*******************************************************************/
/*  dmtcommon.h                                                    */
/*                                                                 */
/*     Contains defnitions and prototypes for functions in         */
/*     dmtcommon.c                                                 */
/*                                                                 */
/*                                                                 */
/*******************************************************************/

#ifndef DMAUX_COMMON_H
#define DMAUX_COMMON_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


/* Maximum number of array dimensions supported */
#define DMA_MAXDIM  20
#ifndef DC_MAXSTR
#define DC_MAXSTR 1024
#endif

#define DCSIZE 80
#define DMAMAXBITS 400
#define BYTE_BITS 8


typedef struct dmtools_params_s
{
  char   infile[DC_MAXSTR];
  char   outfile[DC_MAXSTR];
  char   option[DC_MAXSTR];
  char   kernel[DC_MAXSTR];
  char   rows[DC_MAXSTR];
  char   cells[DC_MAXSTR];
  dmBool clobber;
  int    verbose; 
} dmtools_params;


int dmaProcessFile(char* inname, char* outname, char* toolname, int mode,
		   int debug, BlockProcess func, void* params,
                   long fileNumber, long numberInFiles);

void dmcopy_clobber( char* outfile, dmBool clobber, int verbose );
void dmKernelSetOptionList( char* opt );

int dmu_option_match( char** optarr, int nopts, char* tststr, long minsize, long* nmatch );

/*
 *  Send dmlist output to a file instead of standard output.
 *  This routine is called by r_dmlist, but if you use the lower
 *  level routines like dmDatasetListBlocks you may need to call it
 *  explicitly.
 */
void dmlist_io_out( FILE* file );

void dmlist_print_std( char* buf );

int   dmaErrInit( char* tool );
int   dmaErrExit( char* buf );
int   dmaErrPrint( char* buf );

FILE* dmaGetNextOutputFile( char* outstack, dmBool clobber, int debug);

dmDataset* dmaGetNextOutputDataset( char* outstack, dmBool clobber, int debug);

#endif
