/*                                                                
**  Copyright (C) 1999-2001,2003,2005-2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* H******************************************************************
 * You may do anything you like with this file except remove this
 * copyright. 
 * 
 * FILE NAME: DMTNAN.h
 *
 * DEVELOPMENT: libdev
 * 
 * DESCRIPTION: The macros defined in this header file can be used
 *              to test whether a number is normal  or NaN or INF.
 *              Also, macros are defined to set NaN values for floats
 *              and doubles.
 * LAST MODIFIED: 
H********************************************************************/

#ifndef DMTNAN_H
#define DMTNAN_H

/* 32 bit floating NaN */
#define FNANVAL 0x7FF80000  /* should be 0x7FC0000 */

/* top 32 bits of double NaN */
#define DNANVAL 0x7FF80000
 
/* 32 bit mask to catch floating NaN or Inf */
#define FNANMASK        0x7F800000

/* 32 bit mask to catch double NaN or Inf */
#define DNANMASK        0x7FF00000

#define DINFMASK    0x7FF0
#define DNINFMASK   0xFFF0


#if defined(__alpha) && defined(__unix__)
/************************************************************
  Alpha / Unix definitions
************************************************************/

#include <nan.h>


/* Extract value of float and cast to unsigned int (32 bits) */
#define FNANCAST(M)     ((unsigned int )(*(unsigned int*)&(M)) )

/* test for floating NaN */
#define dmt_fNAN(L) \
((float)(*((unsigned int*) (&(L)))) == 0.0 ?  0 : \
( ( FNANCAST(L) & FNANMASK) == FNANMASK ?  1 : 0))

/* test for double NaN */
#define dmt_dNAN(X) IsNANorINF(X)

#define dmt_dINF(X) (IsNANorINF(X) && IsINF(X) && IsPosNAN(X))
#define dmt_dNINF(X) (IsNANorINF(X) && IsINF(X) && IsNegNAN(X))


/* Set floating or double NaN values */
#define dmt_MAKE_DMT_FNAN(x) (*(unsigned int *)&(x)=FNANVAL)
#define dmt_MAKE_DNAN(X)  (((dnan *)&(X))->nan_parts.exponent = 0x7ff) ; \
 (((dnan *)&(X))->nan_parts.qnan_bit = 0x1)


#elif defined(__sparc) || defined(sparc)
/************************************************************
    Sparc definitions
************************************************************/

/* Extract value of float and cast to unsigned long */
#define FNANCAST(M) ( (unsigned int )(*(unsigned int*)&(M)) )

/* Manufacture pointer to floating NaN */
#define FNANPTR(x) (*(unsigned int*)&(x))


/* Extract value of double high order word and cast to unsigned long */
#define DNANCAST(M) ( (unsigned int )(*(unsigned int*)&(M)) )

/* Manufacture pointers to high and low words of double NaN */
#define DNANPTR1(x) (*(unsigned int*)&(x))
#define DNANPTR2(x) (*((unsigned int*)&(x)+1))

/* Test for floating NaN */
#define dmt_fNAN(L) \
     (L == 0.0 ?  0 : \
     ( ( FNANCAST(L) & FNANMASK) == FNANMASK ?  1 : 0 ))
#define dmt_dINF(L) \
   ( L == 0.0 ? 0 : \
   ( DNANCAST(L)  == DINFMASK ? 1 : 0 ))

#define dmt_dNINF(L) \
   ( L == 0.0 ? 0 : \
   ( DNANCAST(L)  == DNINFMASK ? 1 : 0 ))

/* Test for double NaN */
#define dmt_dNAN(L) \
     (L == 0.0 ?  0 : \
      ( ( DNANCAST(L) & DNANMASK) == DNANMASK ?  1 : 0))


/* Set floating or double NaN values */
#define dmt_MAKE_DMT_FNAN(x) FNANPTR(x)=((unsigned int) FNANVAL)
#define dmt_MAKE_DNAN(x) DNANPTR1(x)=((unsigned int)DNANVAL); DNANPTR2(x)=0

#elif defined(__linux__) || defined(__APPLE__) || defined(__CYGWIN32__)

/************************************************************
  Linux definitions
************************************************************/

#if defined(__APPLE__)
#include <machine/endian.h>
#else
#if defined(__linux__)
#include <endian.h>
#else
#define BYTE_ORDER LITTLE_ENDIAN
#endif
#endif

/* Extract value of float and cast to unsigned long */
#define FNANCAST(M) ( (unsigned int )(*((unsigned int*)&(M))) )
/* Manufacture pointer to floating NaN */
#define FNANPTR(x) (*(unsigned int*)&(x))

#if BYTE_ORDER == LITTLE_ENDIAN
/* Extract value of double high order word and cast to unsigned int */
#define DNANCAST(M) ( (unsigned int )(*(((unsigned int*)&(M))+1)) )
/* Manufacture pointers to high and low words of double NaN */
#define DNANPTR1(x) (*((unsigned int*)&(x)+1))
#define DNANPTR2(x) (*(unsigned int*)&(x))
#else
/* Extract value of double high order word and cast to unsigned int */
#define DNANCAST(M) ( (unsigned int )(*((unsigned int*)&(M))) )
/* Manufacture pointers to high and low words of double NaN */
#define DNANPTR1(x) (*(unsigned int*)&(x))
#define DNANPTR2(x) (*((unsigned int*)&(x)+1))
#endif


/* Test for floating NaN */
#define dmt_fNAN(L) \
     (L == 0.0 ?  0 : \
     ( ( FNANCAST(L) & FNANMASK) == FNANMASK ?  1 : 0 ))
 
/* Test for double NaN */
#define dmt_dNAN(L) \
     (L == 0.0 ?  0 : \
      ( ( DNANCAST(L) & DNANMASK) == DNANMASK ?  1 : 0))


#define dmt_dINF(L) \
   ( L == 0.0 ? 0 : \
   ( DNANCAST(L)  == DINFMASK ? 1 : 0 ))

#define dmt_dNINF(L) \
   ( L == 0.0 ? 0 : \
   ( DNANCAST(L)  == DNINFMASK ? 1 : 0 ))


/* Set floating or double NaN values */
#define dmt_MAKE_DMT_FNAN(x) FNANPTR(x)=((unsigned int) FNANVAL)
#define dmt_MAKE_DNAN(x) DNANPTR1(x)=((unsigned int)DNANVAL); DNANPTR2(x)=0

#else

/* unknown cpu/OS */
#endif

#endif /* DMTNAN_H */
    
