/*                                                                
**  Copyright (C) 1998,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*************************DSERROR_DEMO.C*******************************/
/* This is a program for demonstrating and checking error routines */

/* Include statements - system libraries*/ 
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>         /* used by pid function */
#include <setjmp.h>            /* for setjump and longjump */
#include <ctype.h>             /* for isalpha and toupper */

/* my routines */
#include <liberr.h>            /* for ASC error routines */



/* Function prototypes */
int print_menu(void);
int err_ret_demo(void);
int err_exit_demo(void);
int err_quit_demo(void);
int err_dump_demo(void);
int err_msg_demo(void);
int trace_demo(void);
void simulate_segmentation_fault(void);
void simulate_floating_point_fault(void);
void setjump_demo(void);
int variable_length_arguments_demo(void);
void dserror_test(char *func_name);
extern void print_instance(dsErrInstance *print_p);



/* declaration of variables */
/* jmp_buf jmpbuf; */        /* Optional */


int main(int argc, char *argv[])
{
  int choice;

  init_error_lib( argv[0]);
  printf("program_name =%s\n", argv[0]); 
  print_menu();
  while  ((choice = getchar()) != EOF) {
    switch(toupper(choice)) {
    case 'A':
      err_ret_demo();
      break;
    case 'B':  
      err_exit_demo();
      break;
    case 'C': 
      err_dump_demo();
      break;
    case 'D':
      err_quit_demo();
      break;
    case 'E':
      err_msg_demo();
      break;
    case 'F':
      trace_demo();
      break; 
    case 'G':
      setjump_demo();
      break;
    case 'H':
      simulate_floating_point_fault();
      break; 
    case 'I':
      simulate_segmentation_fault();       /* simulate crash */ 
      break;
    case 'J':
      variable_length_arguments_demo();
      break;
    case 'T':
      dserror_test(argv[0]);
      break;
    case 'X':
      return 0;
    default:
      break;
    }  
    if (isalpha(choice)) print_menu();
  }
  return 0;
}


int print_menu(void)
{
  printf("\nPlease enter a letter for your selection. The choices are\n");
  printf(" A: demo of err_ret: Print system error message and return\n");
  printf(" B: demo of err_exit: Print system error message and exit\n");
  printf(" C: demo of err_dump: Print system error message and abort\n");
  printf(" D: demo of err_quit: No system  error message and  exit\n");
  printf(" E: demo of err_msg: Print your message and return\n");
  printf(" F: demo of trace_test: traceback through functions and exit\n");
  printf(" G: demo of setjump - signal_handler\n");
  printf(" H: demo of simulate_floating_point_fault() - signal_handler\n");
  printf(" I: demo of simulate_segmentation_fault() - signal_handler\n");
  printf(" J: demo of message with multiple arguments\n"); 
  printf(" T: test of error lists: addition, retrieval, and deletion\n");
  printf(" X: exit\n");
  return 0;  
}
/**************************************************************************/
int err_ret_demo(void) 
{
  FILE *fp;

  printf("***err_ret_demo***\n"); 
  if( (fp = fopen("junk2", "r")) == NULL)
     err_ret("can't open file");
  return 0;
}

/**************************************************************************/

int err_exit_demo(void)
{   
  FILE *fp; 

  printf("***err_exit_demo***\n");
  if( (fp = fopen("/tmp_mnt/proj/", "w")) == NULL)
    err_exit( ENOENT, "fopen  error");
  return 0;
}

/**************************************************************************/
int err_quit_demo(void)
{
  FILE *fp;


  printf("***err_quit_demo***\n");
  if( (fp = fopen("junk2", "r")) == NULL)
    err_quit( ENOENT, "Your message here");
  return 0;
}

/**************************************************************************/
int err_dump_demo(void)
{
  printf("***err_dump_demo***\n");
  err_dump("Your message here");
  return 0;
}

/**************************************************************************/

int err_msg_demo(void)
{
  printf("***err_msg_demo***\n");
  err_msg("Your message here");
  return 0;
}

/**************************************************************************/

/* Function prototypes */
void testfunc1(void);
void testfunc2(void);

int trace_demo(void)
{
  check_in("trace_demo");
  printf("\n***trace_demo***\n");
  testfunc1();
  check_out();
  return 0;
}

/**************************************************************************/

void testfunc1(void)
{
  check_in("testfunc1");
  testfunc2();
  check_out();
  return;
}

/**************************************************************************/

void testfunc2(void)
{
  FILE *fp;
 
  check_in("testfunc2");
  if( (fp = fopen("junk2", "r")) == NULL)
  err_exit( ENOENT, "died in testfunc2 while opening file"); 
  printf("checkpoint1");
  check_out(); 
  return;
}

/**************************************************************************/

void simulate_segmentation_fault( void )
{ 
  float *tmp=NULL;
  tmp[123] = 0.0; /* WRONG */
  return;
}

/**************************************************************************/

void simulate_floating_point_fault( void )
{ 
  float a;
  a=1/0;
  return;
}

/**************************************************************************/

void setjump_demo(void)
{
  int i;                       /* counter for for loop */
  int x, y;
  static char line[80];
  
  printf("This option loops three times\n"); 
  gets(line);                 /* flush buffer */
  for (i=0; i< 3; i++) {
     if (setjmp(dserr_jmpbuf) != 0) {
        printf("error ..... the for loop continues\n");
     }
     else {
        printf("\nPlease enter a number, 0 to cause a crash\n");
        gets(line);
        sscanf(line, "%d", &x); 
        y=1/x;
        printf("%d", y);  
     } 
  }
  return;
}

/**************************************************************************/

int variable_length_arguments_demo(void)
{
  int n = 7; 

  printf("***variable_length_arguments_demo***\n");
  err_msg("The error occured at loop number %d \n", n);
  return 0;
}








