/*                                                                
**  Copyright (C) 1998,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* This is a program for demonstrating and checking error list routines */

/* Include statements - system libraries*/ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* my routines */
#ifndef __liberr_h
#include <liberr.h>            /* for ASC error routines */
#endif

#ifndef __dserror_funcs_priv_h
#include "dserror_funcs_priv.h"
#endif

void dserror_test(char *func_name)
{
  dsErrGroup groups_to_add_t = dsDBGRPERR;
  dsErrList *base_list_p = NULL, *new_list_p = NULL;
  dsErrCode local_errstat_t = dsNOERR;
  dsErrInstance *holder_p = NULL;

  /* Initialize library, and structures that will be needed. */
  groups_to_add_t |= dsPTGRPERR;
  local_errstat_t = dsErrInitLib(groups_to_add_t, func_name);
  if(local_errstat_t == dsNOERR)
    {
      local_errstat_t = dsErrCreateList(&base_list_p);
      local_errstat_t = dsErrCreateList(&new_list_p);
      local_errstat_t = dsErrCreateInstance(&holder_p);
      
      /* Add errors to the list.  Get a population of Warnings and Fatals,
	 , Individuals and Accumulations. */
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsALLOCERR, Individual,
				     Custom, "This is a custom memory allocation error message\n");
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, 
				     Individual, Custom, 
				     "This is an individual event error\n");
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, 
				     Accumulation, Generic, NULL);
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsDATABASEEXISTSERR, 
				     Individual, Generic, "DB");
	}
      
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, 
				     Accumulation, Custom, 
		      "This is an error message that shouldn't get used.\n");
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, 
				     Accumulation, Generic, NULL);
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsALLOCERR, Individual,
				     Generic, NULL);
	}
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, Individual,
				     Custom, 
			       "This is the second individual event error\n");
	}
      
      if(local_errstat_t == dsNOERR)
	{
	  local_errstat_t = dsErrAdd(base_list_p, dsGENERICERR, Individual,
				     Custom, 
			       "This is the third individual event error\n");
	}

      /* Print the list out so we see what it looks like initially */
      printf("Printing the base list from which all operations will draw.\n");
      dsErrPrintList(base_list_p, dsErrTrue);
      
      /* Look for the first Fatal error and print it out */
      printf("Looking for a Fatal error in the base list.\n");
      if(dsErrLookSev(base_list_p, Fatal, holder_p) == dsNOERR)
	dsErrPrintInstance(holder_p, dsErrTrue);
      
      /* Look for the first individual instance of a dsGENERICERR and 
	 print */
      printf("Looking for the first accumulation instance of dsGENERICERR.\n");
      if(dsErrLookCode(base_list_p, dsGENERICERR, Accumulation, holder_p) 
	 == dsNOERR)
	dsErrPrintInstance(holder_p, dsErrTrue);

      /* Look for the first instance of any type of dsGENERICERR, and 
	 print */
      printf("Looking for the first instance of any type of dsGENERICERR.\n");
      if(dsErrLookCode(base_list_p, dsGENERICERR, Both, holder_p) == 
	 dsNOERR)
	dsErrPrintInstance(holder_p, dsErrTrue);

      /* Get the number of occurances of a dsGENERICERR, and print: Test the various
	 functionalities of dsErrGetNumOccur(). */
      printf("\nNumber of occurances of dsGENERICERRs breaks down as follows:\n");
      printf("\tTotal number of occurances: %d\n", 
	     dsErrGetNumOccur(base_list_p, dsGENERICERR, Both));
      printf("\tTotal number of accumulation occurances: %d\n", 
	     dsErrGetNumOccur(base_list_p, dsGENERICERR, Accumulation));
      printf("\tTotal number of individual occurances: %d\n", 
	     dsErrGetNumOccur(base_list_p, dsGENERICERR, Individual));


      /* Get all Fatal errors and put them in a new list */
      printf("\nPutting all Fatal errors into a new list.\n");
      if(dsErrLookAllSev(base_list_p, Fatal, new_list_p) == dsNOERR)
	{
	  printf("Printing the new list.\n");
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Try removing a Warning severity error.  This shouldn't do 
	     anything, as there should only be Fatal errors in the list. */
	  printf("Trying to remove the first Warning error from the list.\n");
	  if(dsErrRemoveSev(new_list_p, Warning) == dsErrTrue)
	    dsErrPrintList(new_list_p, dsErrTrue);
	  else
	    printf("No Warning's found\n");
	  
	  /* Remove a Fatal error from the list and print the list */
	  printf("Trying to remove the first Fatal error from the list.\n");
	  dsErrRemoveSev(new_list_p, Fatal);
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Remove all Fatalities from the list and print the list 
	     This should empty the list. */
	  printf("Removing all Fatal errors from the list.\n");
	  dsErrRemoveAllSev(new_list_p, Fatal);
	  dsErrPrintList(new_list_p, dsErrTrue);
	}
      
      /* Get all individual GENERICERR errors and put them into 
	 a new list */
      printf("\nPutting all individual instances of the dsGENERICERR error into a new list.\n");
      if(dsErrLookAllCode(base_list_p, dsGENERICERR, Individual, 
			  new_list_p) == dsNOERR)
	{
	  printf("Printing the new list.\n");
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Try to remove a GENERICERR error with an accumulation type.
	     This should fail, as there should only be individual errors in 
	     this list */
	  printf("Trying to remove the first dsGENERICERR accumulation error from the list.\n");
	  if(dsErrRemoveCode(new_list_p, dsGENERICERR, Accumulation) == 
	     dsErrTrue)
	    dsErrPrintList(new_list_p, dsErrTrue);  
	  else
	    printf("No dsGENERICERR accumulations found\n");
	  
	  /* remove the first instance of a GENERICERR error with an 
	     individual type, and print the list */
	  printf("Trying to remove the first dsGENERICERR individual error from the list.\n");
	  dsErrRemoveCode(new_list_p, dsGENERICERR, Individual);
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Remove all individual EVENTCOORDERRs from the list and print it.
	     The list should be empty after this. */
	  printf("Removing all dsGENERICERR individual errors from the list.\n");
	  dsErrRemoveAllCode(new_list_p, dsGENERICERR, Individual);
	  dsErrPrintList(new_list_p, dsErrTrue);
	}
      
      /* Get all GENERICERR errors, regardless of type, and put them
	 in a new list */
      printf("\nPutting all instances of the dsGENERICERR error into a new list.\n");
      if(dsErrLookAllCode(base_list_p, dsGENERICERR, Both, new_list_p) == 
	 dsNOERR)
	{
	  printf("Printing the new list.\n");
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Remove the first GENERICERR error of either type */
	  printf("Trying to remove the first dsGENERICERR error of either type from the list.\n");
	  dsErrRemoveCode(new_list_p, dsGENERICERR, Both);
	  dsErrPrintList(new_list_p, dsErrTrue);
	  
	  /* Remove all GENERICERR errors of either type.  This should empty
	     the list */
	  printf("Removing all dsGENERICERR errors of either type from the list.\n");
	  dsErrRemoveAllCode(new_list_p, dsGENERICERR, Both);
	  dsErrPrintList(new_list_p, dsErrTrue);
	}

      /* Test a few odd's and ends functions.*/
      printf("\nTest populating an instance using dsErrSetInstance(), and then\nadding that error using dsErrAddToEnd()\n");
      if(dsErrSetInstance(dsCOLUMNTYPEERR, Individual, Generic, holder_p, "foo", 
			  "not-a-file", "double") == dsNOERR)
	{
	  printf("The user populated error instance:\n");
	  dsErrPrintInstance(holder_p, dsErrTrue);
	  if(dsErrAddToEnd(base_list_p, holder_p) == dsNOERR)
	    {
	      printf("Print the base list after addition of new error\n");
	      dsErrPrintList(base_list_p, dsErrTrue);
	    }
	}
      
      printf("Test removing the Nth error: we will remove the 3rd\n");
      dsErrRemoveN(base_list_p, 3);
      dsErrPrintList(base_list_p, dsErrTrue);
      
      printf("Test looking for the first error in the list\n");
      if(dsErrLook(base_list_p, holder_p) == dsNOERR)
	dsErrPrintInstance(holder_p, dsErrTrue);

      printf("Test the list accessor functions.\n");
      printf("\tThe number of error in the list are %d.\n", 
	     dsErrGetErrorCt(base_list_p));
      printf("\tThe number of fatal errors in the list are %d.\n", 
	     dsErrGetFatalCt(base_list_p));
      printf("\tThe number of warnings in the list are %d.\n", 
	     dsErrGetWarningCt(base_list_p));

      printf("\nTest reseting the error instance.\n");
      dsErrResetInstance(holder_p);
      dsErrPrintInstance(holder_p, dsErrTrue);

      printf("\nTest removing all errors in the list\n");
      dsErrRemoveAll(base_list_p);
      printf("\tPrinting all errors in the list\n");
      dsErrPrintList(base_list_p, dsErrTrue);

      printf("\nTest the custom error facility\n");
      dsErrSetInstance(dsGENERICERR, Individual, Custom, holder_p, "WARNING: This is an %s string, containing a number, %d.\n", "error formatting", 56);
      dsErrPrintInstance(holder_p, dsErrTrue);

      printf("\nTest the error instance accessor functions on the instance just\npopulated using the custom error facility.\n");
      printf("\tThe error code of the instance is: %d\n", dsErrGetInstCode(holder_p));
      printf("\tThe error severity of the instance is: %s\n",
	     ((dsErrGetInstSev(holder_p) == Fatal) ? "Fatal" : "Warning"));
      printf("\tThe number of occurances of the error is: %d\n", dsErrGetInstCt(holder_p));
      printf("\tThe message of the instance is: %s", dsErrGetInstMsg(holder_p));
    }
  dsErrCloseLib();

}
