// 
//  Copyright (C) 2003,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <sys/types.h>
#include <regex.h>
#include "AhelpDirectory.hh"
#include "AhelpDB.hh"
#include "AhelpDoc.hh"
#include "XMLFParser.hh"
#include "LookupTable.hh"
#include "SortedList.hh"
#include "AhelpUtils.hh"

/*
** DESC: Constructor
**
**
** PRE: 
**
**
**
** POST: local dir is used to init DB
**
**
*/

AhelpDB::AhelpDB()
{
  SetPaths("");
};

/*
** DESC: Constructor
**
**
** PRE: inPath points to location ahelp DB dir structure is rooted
**      /doc, /xml dirs are located
**
**
**
** POST: inPath is used to initialize DB
**
**
*/

AhelpDB::AhelpDB(const AhelpStrings& inPath)
{
  SetPaths(inPath);
};




/*
** DESC: Recreates indes files for ahelp database
**
**
** PRE: inPath - location of ciao tree ( where doc/ is found)
**      inPath has to be writable together with old index files
**      Format for main DB table and index tables
**      AhelpDBTable :
**      key|context|contribFlag|xml_file_path|local_html_help_file|internet_url_ahelp|synopsis
**
**      helpDBIndex01Table:
**      key|context|row_offset_in_AhelpDBTable
**
**
**      helpDBIndex1Table:
**      context|row_offset_in_AhelpDBTable
**
**      crossreferenceDBTable
**      keyword|row_offset_in_AhelpDBTable
**
**      seealsoDBTable
**      groupName|row_offset_in_AhelpDBTable
**
** POST: All files with extention xml are scanned in inPath/doc/xml
**       and inPath/contrib/doc/xml. Files are scanned and parsed.
**       Information about key,context,synopsis is extracted and put 
**       into main index file together with xml file location, html
**       files locations and synopsis. All index files are created in 
**       inPath/doc/xml. Corresponding index files  for key, 
**       context, keyword and seealsogroups are created too. 
**       Returns 1 on success 
**               0 otherwise
**
**       
*/

int AhelpDB::Reindex(const AhelpStrings& inPath)
{
  AhelpStrings tmpString,tmpString1;
  XMLFParser  myparser;
  AhelpDoc tmpDoc,tmpDoc0;
  AhelpStrings XMLPath[2];
  AhelpStrings DTDPath;
  AhelpStringsList tmpRow;
  int indx,indx1,indx2;
  AhelpList<int> colList;
  AhelpStrings tmpOffset;
  AhelpStrings tmpHTMLPath;
  AhelpStrings tmpInternetHTMLPath;
  AhelpStrings tmpValue;
  long failCount=0;
  long successCount=0;
  long duplicatCount=0;
  AhelpStrings tmpPath;

  if ( inPath.isEmpty())
    {
      tmpPath=".";
    }
  else
    {
      tmpPath=inPath;
    };

  try 
    {

      // Initialize paths for index tables
      
      tmpString=tmpPath;
      tmpString.append(XMLDIR);
      tmpString.append(helpDBFileName);
      AhelpDBTable helpDBTable(tmpString,1);
      helpDBTable.Clear();
      
      tmpString=tmpPath;
      tmpString.append(XMLDIR);
      tmpString.append(helpDBIndex01FileName);
      AhelpDBTable helpDBIndex01Table(tmpString,1);
      helpDBIndex01Table.Clear();
      
      
      tmpString=tmpPath;
      tmpString.append(XMLDIR);
      tmpString.append(helpDBIndex1FileName);
      AhelpDBTable helpDBIndex1Table(tmpString,1);
      helpDBIndex01Table.Clear();
      
      
      
      tmpString=tmpPath;
      tmpString.append(XMLDIR);
      tmpString.append(crossreferenceDBFileName);
      AhelpDBTable crossreferenceDBTable(tmpString,1);
      crossreferenceDBTable.Clear();
      cout << crossreferenceDBTable<<endl;

      
      tmpString=tmpPath;
      tmpString.append(XMLDIR);
      tmpString.append(seealsoDBFileName);
      AhelpDBTable seealsoDBTable(tmpString,1);
      seealsoDBTable.Clear();


      XMLPath[0]=tmpPath;
      XMLPath[0].append(XMLDIR);
      
      XMLPath[1]=tmpPath;
      XMLPath[1].append(CONTRIBXMLDIR);
      
      
      
      DTDPath=tmpPath;
      DTDPath.append(XMLDIR);
      DTDPath.append(DTDFileName);
      
      AhelpDirectory* tmpDir[2];
      tmpDir[0] = new AhelpDirectory(XMLPath[0].chars(),".*\\.xml$");
      tmpDir[1] = new AhelpDirectory(XMLPath[1].chars(),".*\\.xml$");
      LookupTable tmpLookupTable(tmpDir[0]->GetLength()+tmpDir[1]->GetLength());
      
      //Scan all the xml files in appropriate dirs
      
      for(indx=0;indx<2;indx++)
	{
	  for(indx1=0;indx1<tmpDir[indx]->GetLength();indx1++)
	    {

	      // Parse the xml file
	      
	      tmpString1=XMLPath[indx];
	      tmpString1.append(tmpDir[indx]->GetIndex(indx1));
	      cout <<"Processing file '"<<tmpString1<<"'...";
		  
	      
	      if ( ! myparser.parse(tmpString1,DTDPath,true,tmpDoc0) )
		{

		  failCount++; 
		  tmpDoc0.Format(tmpString);
		  cout << "FAILED"<<endl;
		  cout << tmpString<<endl;
		  cout << "Skipping..."<<endl<<endl;
		  continue;
		}
	      else
		{
		  tmpDoc0.SetBlocks(0,0,0,0,0,0,0,"",0,"",0,0,0,0);
		  tmpDoc0.Format(tmpString);
		  tmpString.clear();
		};

	      for(int currContextIndx=0;currContextIndx<tmpDoc0.ContextCount();currContextIndx++)
		{
		  
		  if ( ! myparser.parse(tmpString1,DTDPath,true,tmpDoc) )
		    {
		      failCount++; 
		      tmpDoc.Format(tmpString);
		      cout << "FAILED"<<endl;
		      cout << tmpString<<endl;
		      cout << "Skipping..."<<endl<<endl;
		    }
		  else
		    {
		      tmpDoc.SetBlocks(0,1,0,1,0,0,0,"",0,"",0,0,0,0);
		      tmpDoc.SetWidth();
		      tmpDoc.Format(tmpString,tmpDoc0.Context(currContextIndx));
		      
		      tmpValue=tmpDoc.Key();
		      tmpValue.append(" ");
		      tmpValue.append(tmpDoc0.Context(currContextIndx));
		      
		      if ( (++(tmpLookupTable[tmpValue]) ) > 1 )
			{
			  cout << "FAILED"<<endl;
			  cout << "Duplicat Subject Found "<<endl;
			  cout << "Key    : "<<tmpDoc.Key()<<endl;
			  cout << "Context: "<<tmpDoc0.Context(currContextIndx)<<endl;
			  cout << "Skipping..."<<endl<<endl;
			  duplicatCount++;
			  continue;
			};
		      
		      // Create intermediate web page if multiple ADDRESS blocks
		      // found in the file
		      
		      if ( CreateWEBPages(tmpDoc.Key(), tmpDoc0.Context(currContextIndx),indx,tmpDoc.AddressList(),tmpHTMLPath,tmpInternetHTMLPath) <=0 )
			{
			  failCount++;
			  cout << "FAILED"<<endl;
			  cout << "Web page creation FAILED"<<endl;
			  cout << "SKipping..."<<endl<<endl;
			  continue;
			};
		      
		      if ( currContextIndx == (tmpDoc0.ContextCount()-1))
			{
			  cout << "OK"<<endl<<endl;	      
			  successCount++;
			};

		      //Create Row for index table on key and context
		      tmpRow.Append(tmpDoc.Key());
		      tmpRow.Append(tmpDoc0.Context(currContextIndx));
		      tmpOffset.append(helpDBTable.GetEndOffset());
		      tmpRow.Append(tmpOffset);
		      
		      if ( helpDBIndex01Table.AddRow(tmpRow) <= 0 )
			{
			  
			};
		      tmpRow.RemoveAll();
		      
		      
		      tmpRow.Append(tmpDoc0.Context(currContextIndx));
		      tmpRow.Append(tmpOffset);
		      helpDBIndex1Table.AddRow(tmpRow);
		      tmpRow.RemoveAll();
		      
		      
		      // Remove SYNOPSIS header and compact actual synopsis
		      
		      tmpString.Substitute("SYNOPSIS","",0);
		      tmpString.remove_leading_white_space(1);
		      tmpString.remove_trailing_white_space(1);
		      tmpString.Substitute("\n"," ",1);
		      tmpString.pack_spaces();
		      tmpRow.Append(tmpDoc.Key());
		      tmpRow.Append(tmpDoc0.Context(currContextIndx));
		      if ( indx )
			{
			  tmpRow.Append("1");
			}
		      else
			{
			  tmpRow.Append("0");
			};
		      tmpRow.Append(tmpDir[indx]->GetIndex(indx1));
		      
		      // Add web page and internet web page info to the row
		      
		      tmpRow.Append(tmpHTMLPath);
		      tmpRow.Append(tmpInternetHTMLPath);
		      tmpRow.Append(tmpString);
		      helpDBTable.AddRow(tmpRow);
		      tmpRow.RemoveAll();
		      
		      // Write down to disk new row in main DB file for ahelp
		      
		      
		      if ( helpDBTable.Commit() <=0 )
			{
			  cout << "ERROR: Commit for helpDBTable FAILED"<<endl;
			  return 0;
			};
		      
		      
		      // Create refkeywords table
		      
		      AhelpStringsList tmpRefkeywordsList  = tmpDoc0.Refkeywords(currContextIndx).Split();
		      tmpRefkeywordsList.Append(tmpDoc.Key());
		      if ( tmpDoc.Key() != tmpDoc0.Context(currContextIndx))
			{
			  tmpRefkeywordsList.Append(tmpDoc0.Context(currContextIndx));
			};
		      
		      for(indx2=0;indx2< tmpRefkeywordsList.GetLength();indx2++)
			{
			  tmpRow.Append(tmpRefkeywordsList.GetIndex(indx2));
			  tmpRow.Append(tmpOffset);
			  crossreferenceDBTable.AddRow(tmpRow);
			  tmpRow.RemoveAll();
			};
		      
		      
		      // Create seealsogroups table
		      
		      AhelpStringsList tmpSeealsoList  = tmpDoc0.Seealsogroups(currContextIndx).Split();
		      
		      for(indx2=0;indx2< tmpSeealsoList.GetLength();indx2++)
			{
			  tmpRow.Append(tmpSeealsoList.GetIndex(indx2));
			  tmpRow.Append(tmpOffset);
			  seealsoDBTable.AddRow(tmpRow);
			  tmpRow.RemoveAll();
			};
		      
		      tmpOffset.clear();
		      
		      
		    };
		};
	    };
	};

      //Print statistics when done
      
      cout <<"Processed Files  : " << failCount+successCount+duplicatCount<<endl;
      cout <<"Succeeded Files  : " << successCount<<endl;
      cout <<"Failed Files     : " << failCount<<endl;
      cout <<"Purged Duplicates: " << duplicatCount<<endl<<endl;
      

      //Write down index table on key and context
      
      cout <<"Creating Key Indexes...";
      helpDBIndex01Table.Sort();
      if  (helpDBIndex01Table.CommitWithMaxRecordWidth() > 0 )
	{
	  cout <<"DONE"<<endl;
	  cout <<"Subject DB Size: "<<helpDBIndex01Table.GetLength()<<endl<<endl;
	}
      else
	{
	  cout <<"FAILED"<<endl;
	  return 0;
	};
      

      //Write down index table on context
      
      cout <<"Creating Context Indexes...";
      helpDBIndex1Table.Sort();
      if  (helpDBIndex1Table.CommitWithMaxRecordWidth() > 0 )
	{
	  cout <<"DONE"<<endl;
	  cout <<"Context DB Size: "<<helpDBIndex1Table.GetLength()<<endl<<endl;
	}
      else
	{
	  cout <<"FAILED"<<endl;
	  return 0;
	};
      
      
      // Write down index table on refkeywords

      cout <<"Creating Crossreference DB...";
      colList.Append(0);
      colList.Append(1);
      crossreferenceDBTable.Sort(colList);
      if ( crossreferenceDBTable.CommitWithMaxRecordWidth() )
	{
	  cout <<"DONE"<<endl;
	  cout <<"Crossreferences DB Size: "<<crossreferenceDBTable.GetLength()<<endl<<endl;
	}
      else
	{
	  cout <<"FAILED"<<endl;
	  return 0;
	};
      

      //Write doen index table on seealsogroups

      cout <<"Creating Seealso  DB...";
      seealsoDBTable.Sort(colList);
      if ( seealsoDBTable.CommitWithMaxRecordWidth() )
	{
	  cout <<"DONE"<<endl;
	  cout <<"Seealso DB Size: "<<seealsoDBTable.GetLength()<<endl<<endl;
	}
      else
	{
	  cout <<"FAILED"<<endl;
	  return 0;
	};
      
    }
  catch(int status)
    {
      return status;
    };



  return 1;

};



/*
** DESC: Recreates db files in dbPath/doc/xml
**
**
** PRE: dbPath has to be writable together with old index files
**
**
** POST: All files with extention xml are scanned in dbPath/doc/xml
**       and dbPath/contrib/doc/xml. Files are scanned and parsed.
**       Information about key,context,synopsis is extracted and put 
**       into main index file together with xml file location, html
**       files locations and synopsis. All index files are created in 
**       dbPath/doc/xml. Corresponding index files  for key, 
**       context, keyword and seealsogroups are created too. 
**       Returns 1 on success 
**               0 otherwise
**
**
**
*/


int AhelpDB::Reindex()
{

  return Reindex(dbPath);

};




/*
** DESC: Finds an xml file(s) for corresponding inKeyword
**
**
** PRE: inKeyword is a key attribute that occurs in some xml files
**      outPathList is empty list of strings
**
**
** POST: Ahelp DB is scanned and paths of files which have key attribute
**       equal to inKeyword are put into outPathList
**       Returns 1 on success 
**               0 otherwise
**
*/


int AhelpDB::FindXML(const AhelpStrings& inKeyword,AhelpStringsList& outPathList)
{

  return FindFile(XML,inKeyword,outPathList);

};



/*
** DESC: Finds an xml file(s) for corresponding inKeyword and inContext
**
**
** PRE: inKeyword is a key attribute that occurs in some xml files
**      inContext is a context attribute that occurs in some xml files
**      outPathList is empty list of strings
**
**
** POST: Ahelp DB is scanned and paths of files which have key attribute
**       equal to inKeyword and context attribute equal to inContext 
**       are put into outPathList
**       Returns 1 on success 
**               0 otherwise
**
*/


int AhelpDB::FindXML(const AhelpStrings& inKey, const AhelpStrings& inContext,AhelpStringsList& outPathList)
{

  return FindFile(XML,inKey,inContext,outPathList);

};



/*
** DESC: For inKeyword, related local or internet html files' paths are 
**       extracted.
**
**
** PRE: inKeyword is key attribute of some xml file in ahelp DB
**      inLocalFlag is 1 for local html file, 0 for internet url address
**      outPathList is empty list
**
**
** POST: Ahelp DB is searched and depending on inLocalFlag either list of
**       local html filepaths or internet urls is extracted relevant to
**       inKeyword.
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::FindHTML(const AhelpStrings& inKeyword,int inLocalFlag,AhelpStringsList& outPathList)
{
  AhelpPathType tmpPType;

  if ( inLocalFlag)
    {
      tmpPType=HTML;
    }
  else
    {
      tmpPType=INTERNET;
    };


  return FindFile(tmpPType,inKeyword,outPathList);

};



/*
** DESC: For inKeyword and inContext, related local or internet html 
**       files' paths are extracted.
**
**
** PRE: inKeyword is key attribute of some xml file in ahelp DB
**      inContext is context attribute of some xml file in ahelp DB
**      inLocalFlag is 1 for local html file, 0 for internet url address
**      outPathList is empty list
**
**
** POST: Ahelp DB is searched and depending on inLocalFlag either list of
**       local html filepaths or internet urls is extracted relevant to
**       inKeyword and inContext.
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::FindHTML(const AhelpStrings& inKey, const AhelpStrings& inContext,int inLocalFlag,AhelpStringsList& outPathList)
{

  AhelpPathType tmpPType;


  if ( inLocalFlag)
    {
      tmpPType=HTML;
    }
  else
    {
      tmpPType=INTERNET;
    };


  return FindFile(tmpPType,inKey,inContext,outPathList);

};


/*
** DESC: Finds subjects that belong to inGroupName seealsogroup
**
**
** PRE: inGroupName is name of a seealsogroup
**      outSeealsoList is empty list of Strings
**
**
** POST: Ahelp DB is searched for inGroupName and pairs key-context are
**       retrieved that belong to inGroupNameseealsogroup results are put
**       into outSeealsoList
**       Returns 1 on success 
**               0 otherwise
**
*/


int AhelpDB::FindSEEALSO(const AhelpStrings& inGroupName,AhelpStringsList& outSeealsoList)
{
  AhelpStrings tmpString;
  long rowIndx;
  SortedList<long> tmpOffsetList;
  AhelpStrings tmpString1;


  try
    {
      
      // Open relevant tables 

      AhelpDBTable helpDBTable(helpDBPath);  
      AhelpDBTable seealsoDBTable(seealsoDBPath);
      
      
      outSeealsoList.RemoveAll();
      

      if ( seealsoDBTable.Find(0,inGroupName,1,tmpOffsetList) <=0 )
	{
	  return 0;
	};
      
      if ( tmpOffsetList.GetLength() > 1 )
	{
	  tmpOffsetList.Sort(TRUE,TRUE);
	};

      //Retrieve multiple  key-context pairs

      for(rowIndx=0;rowIndx<tmpOffsetList.GetLength();rowIndx++)
	{
	  
	  if ( helpDBTable.GetOffsetCell(tmpOffsetList.GetIndex(rowIndx),0,tmpString) <=0 )
	    {
	      return 0;
	    };
	  
	  if ( helpDBTable.GetOffsetCell(tmpOffsetList.GetIndex(rowIndx),1,tmpString1) <=0 )
	    {
	      return 0;
	    };
	  tmpString1.append(" ");
	  tmpString1.append(tmpString);
	  outSeealsoList.Append(tmpString1);
	  
	};
      
      
      outSeealsoList.Sort(TRUE,TRUE);
    }
  catch( int status )
    {
      return status;
    };

  return 1;

};


/*
** DESC: Creates intermediate web pages for local/internat versions
**       or ahelp files
**
** PRE: inKey key for which files are created
**      inContext context of inKey
**      inContribIndx is 0 if corresponding xml file is in contrib/doc/xml
**                       1 if in doc/xml
**      inAddressList list of addresses (path/url, label) that correspond
**                         to subject inKey in contex inContext
**      outHTMLFileNmae name of html file created for locally based html 
**                      help
**      outInternetHTMLFileName name of html file created for internet based
**                              html help
**
**
**
** POST: inAddressList is scanned and if multiple entries are found html files
**       are created in doc/html.  If url in inAddressList entry starts with 
**       'http://" then outInternetHTMLFileName is assigned a name of 
**       newly created file, otherwise outHTMLFileName is also assigned name
**       of newly created file.  Newly created files are html documents that
**       contain links to actual html help files.
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::CreateWEBPages(const AhelpStrings& inKey, 
			const AhelpStrings& inContext, 
			int inContribIndx,
			AhelpList<Address>& inAddressList,
			AhelpStrings& outHTMLFileName,
			AhelpStrings& outInternetHTMLFileName)
{
  int indx;
  AhelpList<Address> internetAddressList;
  AhelpList<Address> localAddressList;
  AhelpStrings tmpFilePath;


  //Get all internet url links and local file links
  
  AhelpListIterator<Address> addrIter(&inAddressList);
  for ( addrIter.First(); !addrIter.IsDone(); addrIter.Next() )
    {
      if ( addrIter.GetCurrent().url.FindStr("http://",TRUE) >=0)
	{
	  internetAddressList.Append(addrIter.GetCurrent());
	}
      else
	{
	  localAddressList.Append(addrIter.GetCurrent());
	};
    };



  // Create all the intermediate html files

  outHTMLFileName.clear();
  outInternetHTMLFileName.clear();

  indx = localAddressList.GetLength();

  if ( indx == 1)
    {
      outHTMLFileName = localAddressList.GetFirst().url;
    }
  else if ( indx > 1 )
    {
      // This is for local html ahelp files

      outHTMLFileName = "mlocal_";
      outHTMLFileName.append(inKey);
      outHTMLFileName.append("_");
      outHTMLFileName.append(inContext);
      outHTMLFileName.append(".html");
      
      
      tmpFilePath=dbPath;
      
      if ( inContribIndx )
	{
	  tmpFilePath.append(CONTRIBHTMLDIR);
	}
      else
	{
	  tmpFilePath.append(HTMLDIR);
	};
      
      tmpFilePath.append(outHTMLFileName);
      

      
      if ( GenerateHTML(tmpFilePath,inKey,inContext,"",localAddressList) <= 0 )
	{
	  return 0;
	};
    };


  indx = internetAddressList.GetLength();

  if ( indx == 0 )
    {
      return 1;
    }
  else if ( indx == 1)
    {
      outInternetHTMLFileName = internetAddressList.GetFirst().url;
    }
  else
    {
      //This is for internet urls

      outInternetHTMLFileName = "minternet_";
      outInternetHTMLFileName.append(inKey);
      outInternetHTMLFileName.append("_");
      outInternetHTMLFileName.append(inContext);
      outInternetHTMLFileName.append(".html");


      tmpFilePath=dbPath;
      
      if (inContribIndx )
	{
	  tmpFilePath.append(CONTRIBHTMLDIR);
	}
      else
	{
	  tmpFilePath.append(HTMLDIR);
	};
      
      tmpFilePath.append(outInternetHTMLFileName);
      
      
      if ( GenerateHTML(tmpFilePath,inKey,inContext,"",internetAddressList) <=0 )
	{
	  return 0;
	};
    };

  return 1;

};



/*
** DESC: Generates inFilePath that is an html file containting links from
**       inList
**
** PRE: inFilePath path of file to be generated
**      inKey key for which this file is generated
**      inContext context of inKey
**      inHeader header to be generated for the file
**      inList list of links to actual html ahelp files
**
**
** POST: inFilePath is generated with inHeader and list of hypertext links
**       with addressed from inList
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::GenerateHTML(const AhelpStrings& inFilePath,
			  const AhelpStrings& inKey, 
			  const AhelpStrings& inContext,
			  const AhelpStrings& inHeader,
			  AhelpList<Address>& inList)
{
  int indx;
  ofstream htmlFile(inFilePath.chars());
  
  if ( ! htmlFile )
    return 0;
  
  
  htmlFile << "<HTML><HEAD><TITLE>  CXC Help Topics:" << inKey.chars() << "(" << inContext.chars() << "</TITLE>";
  htmlFile << "</HEAD><BODY>";
  htmlFile << "<CENTER><H3>" << "Help Topics for Subject '" << inKey.chars()<< "' in Context '" << inContext.chars() << "'" ;
  
  htmlFile<< "</H3><CENTER>"<<endl;

  for(indx=0;indx<inList.GetLength();indx++)
    {
      
      if (  inList.GetIndex(indx).label.isEmpty()  )
	{
	  htmlFile << "<LI><A href=\""<< inList.GetIndex(indx).url  << "\">"<<inList.GetIndex(indx).url<<"</A><BR><BR></LI>"; 
	}
      else
	{
	  htmlFile << "<LI><A href=\""<< inList.GetIndex(indx).url  << "\">"<<inList.GetIndex(indx).label<<"</A><BR><BR></LI>"; 
	};


    };
  htmlFile << "</BODY></HTML>"<<endl;

  return 1;
};









/*
** DESC: Setsup paths for ahelp DB 
**
**
** PRE: inPath is dir that contains doc, doc/xml doc/html,...
**                              contrib/doc contrib/xml,...
**
**
**
** POST: internal path vars are initialized
**
**
*/


void AhelpDB::SetPaths(const AhelpStrings& inPath)
{


  if ( inPath.isEmpty())
    {
      dbPath=".";
      helpDBPath      = dbPath;
    }
  else
    {
      dbPath=inPath;
      helpDBPath      = dbPath;
    };


  helpDBPath.append(XMLDIR);
  helpDBIndex01Path = helpDBPath;
  helpDBIndex1Path = helpDBPath;
  crossreferenceDBPath = helpDBPath;
  seealsoDBPath = helpDBPath;
  dtdPath = helpDBPath;

  helpDBPath.append(helpDBFileName);

  helpDBIndex01Path.append(helpDBIndex01FileName);
  
  helpDBIndex1Path.append(helpDBIndex1FileName);

  crossreferenceDBPath.append(crossreferenceDBFileName);
  
  seealsoDBPath.append(seealsoDBFileName);
  
  dtdPath.append(DTDFileName);
};



/*
** DESC: Retrieves paths from ahelp DB
**
**
** PRE: inPathType path of XML, XMLCONTIB,HTML or HTMLCONTRIB,or INTERNET
**      inHelpDBTable ahelp table that contains all ahelp DB
**      inRowOffset location of the row from which to retrieve path
**      outPath retrieved path
**
**
** POST: inHelpDBTable is accessed and row with offset inRowOffset is 
**       retrieved then depending on inPathType appropriate column's value 
**       is assigned to outPath after full path is built
**       Returns 1 on success 
**               0 otherwise
**       
**
**
*/


int AhelpDB::GetPath(AhelpPathType inPathType,AhelpDBTable& inHelpDBTable,long inRowOffset,AhelpStrings& outPath)
{
  
  int colNo;
  AhelpStrings tmpString;

  outPath.clear();


  if ((inPathType == XML ) || (inPathType == XMLCONTRIB  ) )
    {
      colNo=3;
    }
  else if (( inPathType == HTML ) || (inPathType == HTMLCONTRIB))
    {
      colNo=4;
    }
  else
    {
      colNo=5;
    };

  if ( inPathType != INTERNET )
    {
      outPath=dbPath;
      if ( inHelpDBTable.GetOffsetCell(inRowOffset,2,tmpString) <=0 )
	{
	  return 0;
	};
      
      if ( tmpString == "0")
	{
	  if ((inPathType == XML ) || (inPathType == XMLCONTRIB  ) )
	    {
	      outPath.append(XMLDIR);
	    }
	  else
	    {
	      outPath.append(HTMLDIR);
	    }
	}
      else
	{
	  if ((inPathType == XML ) || (inPathType == XMLCONTRIB  ) )
	    {
	      outPath.append(CONTRIBXMLDIR);
	    }
	  else
	    {
	      outPath.append(CONTRIBHTMLDIR);
	    }
	};
    };
  
  if ( inHelpDBTable.GetOffsetCell(inRowOffset,colNo,tmpString) <=0 )
    {
      return 0;
    }
  else if (tmpString.isEmpty())
    {
      return 0;
    };
  
  if  ( inPathType == INTERNET )
    {
      if (tmpString.FindStr("http://",TRUE) != 0)
	{
	  outPath=dbPath;
	  outPath.append(HTMLDIR);
	};
    };

  outPath.append(tmpString);

  return 1;
};



/*
** DESC: Finds ahelp files that relate to inKeyword
**
**
** PRE: inPathFlag 1 for contrib dir to be prepended to final path
**                 0 otherwise 
**      inKeyword keyword to be used in search
**      outPathList empty list of Strings
**
**
** POST: Crossreference index is searched for inKeyword and corresponding
**       paths of ahelp files are retrieved from main DB table into 
**       outPathList.  
**       Returns 1 on success 
**               0 otherwise
**
**
*/

  
int AhelpDB::FindFile(AhelpPathType inPathFlag,const AhelpStrings& inKeyword,AhelpStringsList& outPathList)
{

  SortedList<long> tmpOffsetList;
  AhelpStrings tmpPath;
  AhelpList<int> tmpColList;

  try
    {
      
      AhelpDBTable helpDBTable(helpDBPath);  
      AhelpDBTable crossreferenceDBTable(crossreferenceDBPath);
      
      
      outPathList.RemoveAll();

      // Get offsets from index table
      
      if ( crossreferenceDBTable.Find(0,inKeyword,1,tmpOffsetList) <=0 )
	{
	  return 0;
	};

      if ( tmpOffsetList.GetLength() > 1 )
	{
	  tmpOffsetList.Sort(TRUE,TRUE);
	};

      // Get actual paths

      if ( tmpOffsetList.GetLength() == 1 )
	{
	  if ( GetPath(inPathFlag,helpDBTable,tmpOffsetList.GetIndex(0),tmpPath) <= 0 )
	    {
	      return 0;
	    };

	  if ( inPathFlag == XML )
	    {
	      AhelpStrings tmpString;
	      if ( helpDBTable.GetOffsetCell(tmpOffsetList.GetIndex(0),1,tmpString) <=0 )
		{
		  return 0;
		}
	      tmpPath.append(" ");
	      tmpPath.append(tmpString);
	    };
	  


	  outPathList.Append(tmpPath);
	}
      else if ( tmpOffsetList.GetLength() > 1 )
	{
	  
	  // which columns to retrieve from main ahelp DB table

	  tmpColList.Append(0);
	  tmpColList.Append(1);
	  tmpColList.Append(6);
	  if ( helpDBTable.GetOffsetCells(tmpOffsetList,tmpColList,outPathList) <=0 )
	    {
	      return 0;
	    };
	  
	  outPathList.Sort(TRUE,TRUE);      

	};
      
    }
  catch ( int status )
    {
      return status;
    };


  return outPathList.GetLength();

};


/*
** DESC: Finds ahelp files that relate to inKey and inContext
**
**
** PRE: inPathFlag 1 for contrib dir to be prepended to final path
**                 0 otherwise 
**      inKey subject to be used in search
**      inContext context of inKey to be used in search
**      outPathList empty list of Strings
**
**
** POST: Crossreference index is searched for inKey and inContext
**       and corresponding paths of ahelp files are retrieved from 
**       main DB table into outPathList. 
**       Returns 1 on success 
**               0 otherwise 
**
**
*/



int AhelpDB::FindFile(AhelpPathType inPathFlag,const AhelpStrings& inKey, const AhelpStrings& inContext,AhelpStringsList& outPathList)
{
  long tmpOffset;
  AhelpStrings tmpString;
  AhelpStrings tmpPath;
  SortedList<long> tmpOffsetList;
  AhelpList<int> tmpColList;
  regex_t  regExp;
  int regExpFlag=0;

  try
    {
      AhelpDBTable helpDBTable(helpDBPath);  
      AhelpDBTable helpDBIndex01Table(helpDBIndex01Path);  
      AhelpDBTable helpDBIndex1Table(helpDBIndex1Path); 
      
      //Which columns to retrieve

      tmpColList.Append(0);
      tmpColList.Append(1);
      tmpColList.Append(6);
      
      outPathList.RemoveAll();

      //Consider cases when inKey and/or inContext are empty

      if ( (inKey.isEmpty()) && (inContext.isEmpty()))
	{

	  //Get all entries when inKey and inContext are empty
	  
	  if ( helpDBIndex01Table.GetLength() == 1 )
	    { 
	      
	      if ( helpDBIndex01Table.GetCell(0,2,tmpOffset) <= 0 )
		{
		  return 0;
		};
	      
	      if ( GetPath(inPathFlag,helpDBTable,tmpOffset,tmpPath) <= 0 )
		{
		  return 0;
		}

	      if ( inPathFlag == XML )
		{
		  AhelpStrings tmpString;
		  if ( helpDBTable.GetOffsetCell(tmpOffsetList.GetIndex(0),1,tmpString) <=0 )
		    {
		      return 0;
		    };
		  
		  tmpPath.append(" ");
		  tmpPath.append(tmpString);
		};


	      outPathList.Append(tmpPath);
	    }
	  else if ( helpDBIndex01Table.GetLength() > 1 )
	    {
	      if ( helpDBTable.GetCells(tmpColList,outPathList) <=0 )
		{
		  return 0;
		};
	    }
	}
      else
	{

	  if ( (!inKey.isEmpty()) && (!inContext.isEmpty()))
	    {
	      
	      AhelpList<long> tmpOffsetList1;
	      
	      if ( helpDBIndex01Table.Find(0,inKey,2,tmpOffsetList1) <=0 )
		{
		  return 0;
		};
	      
	      
	      
	      if ( (tmpString = GetRegularExpression(inContext) ) != "")
		{
		  // we are searching using regular expression for context
		  
		  if ( regcomp(&regExp,tmpString.chars(),REG_EXTENDED|REG_NOSUB) != 0 )
		    {  
		      return 0;
		    };
		  regExpFlag=1;
		};
	      
	      AhelpListIterator<long> offsetIter(&tmpOffsetList1);
	      for(offsetIter.First();!offsetIter.IsDone();offsetIter.Next())
		{
		  if ( helpDBTable.GetOffsetCell(offsetIter.GetCurrent(),1,tmpString) <= 0 )
		    {
		      if ( regExpFlag )
			{
			  regfree(&regExp);
			};   
		      
		      return 0;
		    };
		  
		  if ( regExpFlag )
		    {
		      // Compare as regular expression
		      
		      if (! regexec(&regExp,tmpString.chars(),0,NULL,0))
			{
			  tmpOffsetList.Append(offsetIter.GetCurrent());
			};
		    }
		  else if ( inContext == tmpString )
		    {
		      tmpOffsetList.Append(offsetIter.GetCurrent());
		    };
		};
	      
	      if ( regExpFlag )
		{
		  regfree(&regExp);
		};
	      
	    }
	  else if ( inKey.isEmpty() )
	    {
	  
	      // Get all entries for inContext if inContext only is not empty
    
	      if ( helpDBIndex1Table.Find(0,inContext,1,tmpOffsetList) <=0 )
		{
		  return 0;
		};
	    }
	  else if ( inContext.isEmpty() )
	    {
	      //Get all entries for inKey if inKey only is not empty

	      if ( helpDBIndex01Table.Find(0,inKey,2,tmpOffsetList) <=0 )
		{
		  return 0;
		};
	    };
	  
	  if ( tmpOffsetList.GetLength() > 1 )
	    {
	      tmpOffsetList.Sort(TRUE,TRUE);
	    };

	  if ( tmpOffsetList.GetLength() == 1 )
	    { 
	      
	      if ( GetPath(inPathFlag,helpDBTable,tmpOffsetList.GetIndex(0),tmpPath) <= 0 )
		{
		  return 0;
		};

	      if ( inPathFlag == XML )
		{
		  AhelpStrings tmpString;
		  if ( helpDBTable.GetOffsetCell(tmpOffsetList.GetIndex(0),1,tmpString) <=0 )
		    {
		      return 0;
		    };
		  
		  tmpPath.append(" ");
		  tmpPath.append(tmpString);
		};
	      
	      outPathList.Append(tmpPath);
	    }
	  else if ( tmpOffsetList.GetLength() > 1 )
	    {
	      if ( helpDBTable.GetOffsetCells(tmpOffsetList,tmpColList,outPathList) <=0 )
		{
		  return 0;
		};
	      outPathList.Sort(TRUE,TRUE);
	    };
	};
      
    }
  catch ( int status )
    {
      return status;
    };

  return outPathList.GetLength();
};




/*
** DESC: Finds inKey.hlp or inKey.doc or inKey.txt in doc/txt
**
**
** PRE: inKey string to be part of searched file's name
**      outPathLIst empty list of Strings
**
**
** POST: dbPath/TEXTDIR is searched for file whose name matches:
**       either inKey.hlp or inKey.doc or inKey.txt.  If found
**       outPathList contains path of found file.
**       Returns 1 on success 
**               0 otherwise
**
*/


int AhelpDB::FindTEXT(const AhelpStrings& inKey,AhelpStringsList& outPathList)
{
  AhelpStrings helpBaseFile,contribBaseFile;
  AhelpStrings helpFile,contribHelpFile;
  ifstream inHelpFile;

  outPathList.RemoveAll();


  helpBaseFile = dbPath;
  helpBaseFile.append(TEXTDIR);

  contribBaseFile = dbPath;
  contribBaseFile.append(CONTRIBTEXTDIR);


  helpFile=helpBaseFile;
  helpFile.append( inKey);
  helpFile.append(".hlp");

  inHelpFile.open(helpFile.chars()) ;
  if ( inHelpFile)
    {
      outPathList.Append(helpFile);
      return 1;
    }

  contribHelpFile=contribBaseFile;
  contribHelpFile.append( inKey);
  contribHelpFile.append(".hlp");

  inHelpFile.open(contribHelpFile.chars());
  if ( inHelpFile)
    {
      outPathList.Append(contribHelpFile);
      return 1;
    }


  helpFile=helpBaseFile;
  helpFile.append( inKey);
  helpFile.append(".doc");

  inHelpFile.open(helpFile.chars());
  if ( inHelpFile)
    {
      outPathList.Append(helpFile);
      return 1;
    }

  contribHelpFile=contribBaseFile;
  contribHelpFile.append( inKey);
  contribHelpFile.append(".doc");

  inHelpFile.open(contribHelpFile.chars());
  if ( inHelpFile)
    {
      outPathList.Append(contribHelpFile);
      return 1;
    }

  
  helpFile=helpBaseFile;
  helpFile.append( inKey);
  helpFile.append(".txt");

  inHelpFile.open(helpFile.chars());
  if ( inHelpFile)
    {
      outPathList.Append(helpFile);
      return 1;
    }

  contribHelpFile=contribBaseFile;
  contribHelpFile.append( inKey);
  contribHelpFile.append(".txt");

  
  inHelpFile.open(contribHelpFile.chars());
  if ( inHelpFile)
    {
      outPathList.Append(contribHelpFile);
      return 1;
    }
  
  return 0;

};



/*
** DESC: Prints file inTablePath to stdout
**
**
** PRE: inTablePath path to ahelp DB table
**
**
**
** POST: table is printed to stdou
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::DumpTable(const AhelpStrings& inTablePath)
{
  try
    {
      AhelpDBTable ahelpTable(inTablePath);  
      cout << ahelpTable<<endl;

    }
  catch(int status )
    {
      return status;
    };

  return 1;
};




/*
** DESC: Retrieves all contexts from database
**
**
** PRE: 
**
**
**
** POST: list of contexts is returned in outContextList variable
**       Returns 1 on success 
**               0 otherwise
**
**
*/


int AhelpDB::GetAllContexts(AhelpStringsList& outContextList)
{
  try
    {
      AhelpList<int> tmpColList;
      tmpColList.Append(0);
      AhelpDBTable ahelpTable(helpDBIndex1Path);  
      ahelpTable.GetCells(tmpColList,outContextList);
      outContextList.Sort(TRUE,TRUE);
    }
  catch(int status )
    {
      return status;
    };

  return 1;
};






const char* const AhelpDB::helpDBFileName="CXCHelpDB";
const char* const AhelpDB::helpDBIndex01FileName="CXCHelpDB.indx01";
const char* const AhelpDB::helpDBIndex1FileName="CXCHelpDB.indx1";
const char* const AhelpDB::crossreferenceDBFileName="CXCHelpCrossreferenceDB";
const char* const AhelpDB::seealsoDBFileName="CXCHelpSeealsoDB";
const char* const AhelpDB::DTDFileName="CXCHelp.dtd";



const char* const AhelpDB::XMLDIR="/doc/xml/";
const char* const AhelpDB::HTMLDIR="/doc/html/";
const char* const AhelpDB::TEXTDIR="/doc/txt/";

const char* const AhelpDB::CONTRIBXMLDIR="/contrib/doc/xml/";
const char* const AhelpDB::CONTRIBHTMLDIR="/contrib/doc/html/";
const char* const AhelpDB::CONTRIBTEXTDIR="/contrib/doc/txt/";

const char AhelpDB::QUERYFIELDSEPARATOR='\x1E';






