// 
//  Copyright (C) 2002,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __AHELPDB_HH__
#define __AHELPDB_HH__

#include "AhelpStrings.hh"
#include "AhelpList.hh"
#include "AhelpDBTable.hh"
#include "AhelpDoc.hh"


class AhelpDB
{
  enum AhelpPathType { XML,XMLCONTRIB,HTML,HTMLCONTRIB,INTERNET};

public:
  AhelpDB();
  AhelpDB(const AhelpStrings& inPath);


  int Reindex(const AhelpStrings& inPath);
  int Reindex();
  const AhelpStrings&  DTDPath(){return dtdPath;};
  int FindXML(const AhelpStrings& inKeyword,AhelpStringsList& outPathList);
  int FindXML(const AhelpStrings& inKey, const AhelpStrings& inContext,AhelpStringsList& outPathList);
  int FindHTML(const AhelpStrings& inKeyword,int localFlag,AhelpStringsList& outPathList);
  int FindHTML(const AhelpStrings& inKey, const AhelpStrings& inContext,int localFlag,AhelpStringsList& outPathList);

  int FindSEEALSO(const AhelpStrings& inGroupName,AhelpStringsList& outSeealsoList);

  int FindTEXT(const AhelpStrings& inKey,AhelpStringsList& outPathList);

  int DumpTable(const AhelpStrings& inTablePath);

  int GetAllContexts(AhelpStringsList& outContextList);

private:

  int CreateWEBPages(const AhelpStrings& inKey, 
		 const AhelpStrings& inContext, 
		 int inContribIndx,
		 AhelpList<Address>& inAddressList,
		 AhelpStrings& outLocalHTMLFileName,
		 AhelpStrings& outInternetHTMLFileName);

  int GenerateHTML(const AhelpStrings& inFIlePath,
		   const AhelpStrings& inKey, 
		   const AhelpStrings& inContext,
		   const AhelpStrings& inHeader,
		   AhelpList<Address>& inList);

  void SetPaths(const AhelpStrings& inPath);

  int GetPath(AhelpPathType inPathFlag,AhelpDBTable& inHelpDBTable,long inRowOffset,AhelpStrings& outPath);
  int FindFile(AhelpPathType inPathFlag,const AhelpStrings& inKeyword,AhelpStringsList& outPathList);
  
  int FindFile(AhelpPathType inPathFlag,const AhelpStrings& inKey, const AhelpStrings& inContext,AhelpStringsList& outPathList);

  

  AhelpStrings dbPath;
  AhelpStrings helpDBPath;
  AhelpStrings helpDBIndex01Path;
  AhelpStrings helpDBIndex1Path;
  AhelpStrings crossreferenceDBPath;
  AhelpStrings seealsoDBPath;
  AhelpStrings dtdPath;


  static const char* const helpDBFileName;
  static const char* const helpDBIndex01FileName;
  static const char* const helpDBIndex1FileName;
  static const char* const crossreferenceDBFileName;
  static const char* const crossreferenceDBIndexFileName;
  static const char* const seealsoDBFileName;
  static const char* const seealsoDBIndexFileName;
  static const char* const DTDFileName;


  static const char* const XMLDIR;
  static const char* const HTMLDIR;
  static const char* const TEXTDIR;

  static const char* const CONTRIBXMLDIR;
  static const char* const CONTRIBHTMLDIR;
  static const char* const CONTRIBTEXTDIR;



public:
  static const char QUERYFIELDSEPARATOR;

};



#endif
