// 
//  Copyright (C) 2002,2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __AHELPDBTABLE_HH__
#define __AHELPDBTABLE_HH__

#include "cxcconfig.h"
#include "AhelpStrings.hh"
#include "AhelpList.hh"

class AhelpDBTable
{
  friend class AhelpDB;
  friend ostream& operator<<(ostream& ostr,AhelpDBTable& inTable);
private:

private:
  AhelpDBTable(const AhelpStrings& inPath,int inCreateFlag=0);

  virtual ~AhelpDBTable();

  int GetOffsetRow(long inRowOffset,AhelpStringsList& outFieldList);
  int GetRow(long inRowNo,AhelpStringsList& outFieldList);
  int GetOffsetCell(long inRowOffset, int inColNo,AhelpStrings& outCell);
  int GetOffsetCell(long inRowOffset, int inColNo,long& outCell);
  int GetOffsetCells(AhelpList<long>& inRowOffsetList,AhelpList<int>& inColList,AhelpStringsList& outRowList);
  int GetCell(long inRowNo, int inColNo,AhelpStrings& outCell);
  int GetCell(long inRowNo, int inColNo,long& outCell);
  int GetCells(AhelpList<int>& inColList,AhelpStringsList& outRowList);
  long FindOffset(int inColNo,const AhelpStrings& inValue);
  long Find(int inColNo,const AhelpStrings& inValue);
  int  Find(int inColNo,const AhelpStrings& inValue,int outColNo,AhelpList<long>& outList);
  int AddRow(AhelpStringsList& inFieldList);
  int AddRow(AhelpStringsList& inFieldList, int inMaxRowSize);
  int Sort(AhelpList<int> uniqueColList);
  int Sort();
  int Clear();
  int Commit();
  int CommitWithMaxRecordWidth();
  int Load();
  long GetEndOffset();
  long GetLength() { return this->rowCount+this->cachedRowCount;};



  //private:
  long CalcCacheOffset(long inRowNo);
  int WriteHeader(long inRowCount,int inColCount,int inRowSize,int inSortedFlag);
  int MoveToOffset(long inRowOffset);
  int MoveToRow(long inRowNo);
  int MakeRow(AhelpStringsList& inFieldList, AhelpStrings& outRow,int inMaxRowSize=0);
  int GetCell(const AhelpStrings& inRow,int inColNo, AhelpStrings& outValue);
  AhelpStrings GetRegularExpression(const AhelpStrings& inString);

  long cachedRowCount;
  long rowCount;
  int colCount;
  int rowSize;
  int sortedFlag;
  fstream file;
  AhelpStrings path;

  long currRowNo;
  long currOffset;
  AhelpStringsList currFieldList;

  AhelpStringsList rows;

  static const char  RECORDSEPARATOR;
  static const int   MAXRECORDSIZE;
  static const char* const FILEID;
  static const int   HEADERSIZE;
  static const char  REGULAREXPRBRACKET;

public:
  static const char  FIELDSEPARATOR;

};




#endif
