// 
//  Copyright (C) 2002,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __AHELPDOC_HH__
#define __AHELPDOC_HH__

#include <sys/types.h>
#include <regex.h>


#include "DTDElement.hh"
#include "TextBlock.hh"
#include "TextBlockTable.hh"


struct Address
{
  AhelpStrings url;
  AhelpStrings label;
  friend ostream& operator <<(ostream& outstream,Address& inAddress) 
  { outstream << inAddress.url<<" " <<inAddress.label<<endl; return outstream;}; 


};


class AhelpDoc: public DTDElement
{
public:
  AhelpDoc();
  ~AhelpDoc();
  
  void SetBlocks(int inHEADERFlag=1,
		 int inSYNOPSISFlag=1,
		 int inSYNTAXFlag=1,
		 int inADDRESSFlag=1,
		 int inDESCFlag=1,
		 int inQEXAMPLELISTFlag=1,
		 int inPARAMLISTFlag=1,
		 const AhelpStrings& inPARAMName="",
		 int inADESCFlag=1,
		 const AhelpStrings& inADESCTitle="",
		 int inBUGSFlag=1,
		 int inSEEALSOFlag=1,
		 int inVERSIONFlag=1,
		 int inLASTMODIFIEDFlag=1);


  void SetWidth(int inWidth=AhelpDoc::DEFAULTOUTPUTWIDTH,float inMarginRatio=AhelpDoc::DEFAULTMARGINRATIO);

  void Reset();

  const AhelpStrings& Refkeywords(int inIndx=0);
  const AhelpStrings& Seealsogroups(int inIndx=0);
  const AhelpStrings& Displayseealsogroups(int inIndx=0);
  const AhelpStrings& Key();
  const AhelpStrings& Context(int inIndx=0);
  int ContextCount(){ return context.GetLength();};
  AhelpList<Address>& AddressList();
  
  int FormattedBlocksCount() { return formattedBlocksCount;};

  void Format(AhelpStrings& outFormattedString,const AhelpStrings& inFormatContextString="");

private:
  virtual int CanCompact(AhelpStrings& inBlockName)
  { if (inBlockName == AhelpDoc::TOK_VERBATIM)  return 0; return 1;};

  void FormatHeader(const AhelpStrings& inKey,const AhelpStrings& inContext, int inWidth,TextBlock& outFormattedBlock);

  void FormatCXCHELPTOPICS(DTDElement& inElement,int inWidth,int inMarginWidth,TextBlock& outFormattedBlock);
  void FormatENTRY(DTDElement& inElement,int inWidth,int inMarginWidth,TextBlock& outFormattedBlock);
  void FormatADDRESS(DTDElement& inElement);


  void FormatSelectedADESC(AhelpList<DTDElement>& inElementList, AhelpStrings inTitle,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  
  void FormatADESCList(AhelpList<DTDElement>& inElementList, int inWidth, int inMargin,TextBlock& outFormattedBlock);
  
  void FormatADESC(DTDElement& inElement, const AhelpStrings& inIndx,int inWidth, int inMargin,TextBlock& outFormattedBlock);

  void FormatBUGS(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  void FormatCAPTION(DTDElement& inElement,int inMaxWidth,TextBlock& outFormattedBlock);
  void FormatDATA(DTDElement& inElement);
  void FormatDESC(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  void FormatEQUATION(DTDElement& inElement,int inMaxWidth,TextBlock& outFormattedBlock);
  void FormatHREF(DTDElement& inElement);
  void FormatITEM(DTDElement& inElement,int inWidth,TextBlock& outFormattedBlock);
  void FormatLABEL(DTDElement& inElement);

  void FormatLASTMODIFIED(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  void FormatLINE(DTDElement& inElement,int inWidth,TextBlock& outFormattedBlock);
  void FormatLIST(DTDElement& inElement,int inWidth,TextBlock& outFormattedBlock);
  int FormatPARA(DTDElement& inElement,const AhelpStrings& inIndx, int inWidth,TextBlock& outFormattedBlock);



  void FormatPARAMLIST(DTDElement& inElement,const char* inHeader,const char* inDetailHeader,const char* inDetailElemHeader,int inWidth, int inMarginWidth, int detailFlag,TextBlock& outFormattedBlock,AhelpStrings& outParamStr);


  void FormatPARAMDetail(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,AhelpList<AhelpStrings>& outAtts,TextBlock& outFormattedBlock,AhelpStrings& outParamString);


  void FormatSelectedPARAM(DTDElement& inElement,const char* inHeader,AhelpStrings inName,int inWidth, int inMarginWidth, TextBlock& outFormattedBlock);

  void FormatPASSTHRU(DTDElement& inElement);
  void FormatROW(DTDElement& inElement,AhelpList<AhelpStrings>& outRow);

  void FormatSEEALSO(const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);

  void FormatQEXAMPLELIST(DTDElement& inElement ,const char* inHeader,const char* inExampleHeader,int inWidth, int inMarginWidth,TextBlock& outFormattedBlock);
  void FormatQEXAMPLE(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);



  void FormatSYNOPSIS(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);

  int FormatSYNTAX(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  
  void FormatSYNTAX(const AhelpStrings& inParamStr, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  void FormatTABLE(DTDElement& inElement,int inWidth,TextBlock& outFormattedBlock);
  void FormatURL(DTDElement& inElement);
  void FormatVERBATIM(DTDElement& inElement,TextBlock& outFormattedBlock);
  void FormatVERSION(DTDElement& inElement, const char* inHeader,int inWidth, int inMargin,TextBlock& outFormattedBlock);
  void FormatXMLONLY(DTDElement& inElement);
  void FormatPCDATA(DTDElement& inElement,int inRemoveLTWspaces=1);
  void FilterContext(DTDElement& inElement);
  void ConcatPCDATAandCONDTEXT(DTDElement& inElement);
  void CompactString(AhelpStrings& inString,int inRemoveLTWspaces=1);

  bool CheckContext(const AhelpStrings& inContextAttributeString);
  bool CheckContext(DTDElement& inElement);
  
  int GetAttribute(DTDElement& inElement,const AhelpStrings& inAttributeString,AhelpStrings& outAttributeValueString);




public:
  static const AhelpStrings TOK_ADDRESS;
  static const AhelpStrings TOK_ADESC;
  static const AhelpStrings TOK_BUGS;
  static const AhelpStrings TOK_DESC;
  static const AhelpStrings TOK_LASTMODIFIED;
  static const AhelpStrings TOK_PARAM;
  static const AhelpStrings TOK_PARAMLIST;
  static const AhelpStrings TOK_QEXAMPLELIST;
  static const AhelpStrings TOK_SYNOPSIS;
  static const AhelpStrings TOK_SYNTAX;
  static const AhelpStrings TOK_VERSION;
  static const AhelpStrings TOK_SEEALSO;


  static const int DEFAULTOUTPUTWIDTH;
  static const float DEFAULTMARGINRATIO;




private:
  static const AhelpStrings TOK_CXCHELPTOPICS;
  static const AhelpStrings TOK_ENTRY;
  static const AhelpStrings TOK_CAPTION;
  static const AhelpStrings TOK_DATA;
  static const AhelpStrings TOK_EQUATION;
  static const AhelpStrings TOK_HREF;
  static const AhelpStrings TOK_ITEM;
  static const AhelpStrings TOK_LABEL;
  static const AhelpStrings TOK_LABELTEXT;
  static const AhelpStrings TOK_LINE;
  static const AhelpStrings TOK_LIST;
  static const AhelpStrings TOK_PARA;
  static const AhelpStrings TOK_PASSTHRU;
  static const AhelpStrings TOK_ROW;
  static const AhelpStrings TOK_QEXAMPLE;
  static const AhelpStrings TOK_TABLE;
  static const AhelpStrings TOK_URL;
  static const AhelpStrings TOK_VERBATIM;
  static const AhelpStrings TOK_XMLONLY;
  static const AhelpStrings TOK_CONDTEXT;
  static const AhelpStrings ATT_KEY;
  static const AhelpStrings ATT_REFKEYWORDS;
  static const AhelpStrings ATT_CONTEXT;
  static const AhelpStrings ATT_SEEALSOGROUPS;
  static const AhelpStrings ATT_DISPLAYSEEALSOGROUPS;
  static const AhelpStrings ATT_NAME;
  static const AhelpStrings ATT_TYPE;
  static const AhelpStrings ATT_FILETYPE;
  static const AhelpStrings ATT_DEF;
  static const AhelpStrings ATT_MIN;
  static const AhelpStrings ATT_MAX;
  static const AhelpStrings ATT_UNITS;
  static const AhelpStrings ATT_REQD;
  static const AhelpStrings ATT_STACKS;
  static const AhelpStrings ATT_AUTONAME;
  static const AhelpStrings ATT_TITLE;
  static const AhelpStrings ATT_PARAGRAPHNUMBERING;
  static const AhelpStrings ATT_LINK;
  
  static const AhelpStrings EMPTY_STRING;

  int outputWidth;
  float marginRatio;

  int HEADERFlag;
  int SYNOPSISFlag;
  int SYNTAXFlag;
  int genSYNTAXFlag;
  int ADDRESSFlag;
  int DESCFlag;
  int QEXAMPLELISTFlag;
  int PARAMLISTFlag;
  int ADESCFlag;
  int BUGSFlag;
  int SEEALSOFlag;
  int VERSIONFlag;
  int LASTMODIFIEDFlag;

  AhelpStrings selectionPARAMName;
  AhelpStrings selectionADESCTitle;


  AhelpStrings key;
  AhelpStringsList context;
  AhelpStringsList refkeywords;
  AhelpStringsList seealsogroups;
  AhelpStringsList displayseealsogroups;
  AhelpList<Address> addressList;
  int formattedBlocksCount;


  AhelpStrings formatContextString;
  int formatContextIndx;

};

#endif
