// 
//  Copyright (C) 2002,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __AhelpInterface_HH__
#define __AhelpInterface_HH__

#include "cxcconfig.h"
#include "AhelpStrings.hh"


class AhelpInterface {
public:

  typedef enum CXCHelpStyle { CXCHELP_SHORT, CXCHELP_MED, 
			      CXCHELP_LONG,  CXCHELP_WEB, 
			      CXCHELP_INTERNET
 };
  

  AhelpInterface (const AhelpStrings& inPath="");

  virtual ~AhelpInterface();

  
  // Help function prototypes


  void SetDefaultContext(const AhelpStrings& inContext);
  const AhelpStrings& GetDefaultContext(){ return defaultContext;};

  int DisplayHelpFromFile(const AhelpStrings& inPath);


  int GetAllContexts(int inOutputWidth,AhelpStrings& outInfo);


  int GetHelpFromFile(const AhelpStrings& inPath,
	    AhelpInterface::CXCHelpStyle style,
	    int inOutputWidth,
	    AhelpStrings& outInfo);


  int GetHelp(AhelpStrings inKey, 
	      AhelpStrings inContext,
	      AhelpInterface::CXCHelpStyle inStyle,
	      int inOutputWidth,
	      AhelpStrings& outInfo);

  
  int GetHelp(AhelpStrings inKeyword, 
	      AhelpInterface::CXCHelpStyle inStyle,
	      int inOutputWidth,
	      AhelpStrings& outInfo);
  
  
  int GetHelp(AhelpStrings inKey, 
	      AhelpStrings inContext,
	      const AhelpStrings& inElementName,
	      const AhelpStrings& inAttributeName,
	      int inOutputWidth,
	      AhelpStrings& outInfo);
  

  static const char* ENVIRON_DEFCONTEXT_VARNAME;
  
protected:
  
  int DisplayURL(const AhelpStrings& inUrl);
  int ReadTEXT(const AhelpStrings& inPath,AhelpStrings& outInfo);
  void CreateSubjectList(AhelpStringsList& inList,int inOutputWidth,AhelpStrings& outInfo);
  int GetFHelp(const AhelpStrings& inKey,AhelpStrings& outInfo);
  void CreateContextList(AhelpStringsList& inList,int inOutputWidth,AhelpStrings& outInfo);

private:
  AhelpStrings installPath;
  AhelpStrings defaultContext;

  static const int MAXTRIES;
  static const int INPUTBUFSIZE;
  static const char* HELPKEY;
  static const char* HELPCONTEXT;

};


class GlobalAhelpInterface 
{
public:
  GlobalAhelpInterface(){Init();};
  virtual ~GlobalAhelpInterface(){;};

  static void SetDefaultContext(const AhelpStrings& inContext)
  { 
    Init(); 
    return globalAhelpInterface->SetDefaultContext(inContext);
    
  };
  
  
  static const AhelpStrings&  GetDefaultContext()
  { 
    Init(); 
    return globalAhelpInterface->GetDefaultContext();
    
  };
  

  static int GetAllContexts(int inOutputWidth,AhelpStrings& outInfo)
  {
    Init(); 
    return globalAhelpInterface->GetAllContexts(inOutputWidth,outInfo);

  };


  static int DisplayHelpFromFile(const AhelpStrings& inPath)
  { 
    Init(); 
    return globalAhelpInterface->DisplayHelpFromFile(inPath);
    
  };
  
  static int GetHelpFromFile(const AhelpStrings& inPath,
			     AhelpInterface::CXCHelpStyle style,
			     int inOutputWidth,
			     AhelpStrings& outInfo)
  { 
    Init(); 
    return globalAhelpInterface->GetHelpFromFile(inPath, style, inOutputWidth, outInfo);
  };
  
  
  static int GetHelp(AhelpStrings inKey, 
		     AhelpStrings inContext,
		     AhelpInterface::CXCHelpStyle inStyle,
		     int inOutputWidth,
		     AhelpStrings& outInfo)
  { 
    Init(); 
    return globalAhelpInterface->GetHelp(inKey, inContext, inStyle,inOutputWidth, outInfo);
    
  };
  
  
  static int GetHelp(const AhelpStrings& inKeyword, 
		     AhelpInterface::CXCHelpStyle inStyle,
		     int inOutputWidth,
		     AhelpStrings& outInfo)
  { 
    Init(); 
    return globalAhelpInterface->GetHelp(inKeyword, inStyle, inOutputWidth, outInfo);
    
  };
  
  
  static int GetHelp(AhelpStrings inKey, 
		     AhelpStrings inContext,
		     const AhelpStrings& inElementName,
		     const AhelpStrings& inAttributeName,
		     int inOutputWidth,
		     AhelpStrings& outInfo)
  { 
    Init(); 
    return globalAhelpInterface->GetHelp(inKey, inContext, inElementName, inAttributeName, inOutputWidth, outInfo);
    
  };
  
  static int GetHelp(AhelpStringsList& inCommandOptions,AhelpStrings& outInfo);

private:
  static void Init() { if ( ! globalAhelpInterface ) globalAhelpInterface = new AhelpInterface();};
  static AhelpInterface* globalAhelpInterface;
  static int optionsCount;
  static const char* const optionsTab[]; 
  static void PrintSynopsis(AhelpStrings& outInfo);
  static bool IsOption(const AhelpStrings& inString);
};

#endif
