// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __AHELPPAGER_HH
#define __AHELPPAGER_HH

#include <unistd.h>
#include <iostream>
#include <signal.h>
#include <errno.h>
#include <math.h>
#include <strstream>


using namespace std;

class AhelpPager 
{
  public:
  enum Toggle { Off, On };

  protected:
  enum { FormatWidth=20 };
  char _formatString[FormatWidth];
  static AhelpPager::Toggle _toggle;
  FILE *_fp;

  private:
  int _width;
  int _precision;
  // Should the number show trailing zeroes, e.g. 1.1 -> 1.100?
  short _showpoint;
  // Should the number be in scientific notation by default?
  short _scientific;
  int _disableCtrlC;

  public:
  static const char* DEFAULT_PAGER;
  
	AhelpPager(int disableCtrlC=0);
	AhelpPager(char *pager,int disableCtrlC=0);
	AhelpPager(FILE *fp);
	virtual ~AhelpPager();
	static void on()  { _toggle = On; }
	static void off() { _toggle = Off; }

  // Notice that the following may not be virtual!  They must be called
  // explicitly by the derived class, which must then cast the
  // pager back to the derived class at the end of the derived operator<<().
  // See src/libdev/ascfitlib/ascfit/data/datapager.C for an example
  // derived class.
  //
  AhelpPager& operator<<(const char *);
  AhelpPager& operator<<(const char );
  AhelpPager& operator<<(const short);
  AhelpPager& operator<<(const unsigned short);
  AhelpPager& operator<<(const int);
  AhelpPager& operator<<(const unsigned int);
  AhelpPager& operator<<(const long);
  AhelpPager& operator<<(const unsigned long);
  AhelpPager& operator<<(const float);
  AhelpPager& operator<<(const double);
  AhelpPager& operator<<(ostream& (*_f)(ostream&));

  void setWidth(int w) { if ( w>=0 ) _width = w; }
  void setPrecision(int p) { if ( p>=0 ) _precision = p; }
  void showPoint(short p) { _showpoint = p; }
  void showScientific(short p) { _scientific = p; }

  int getWidth() const { return _width; }
  int getPrecision() const { return _precision; }
  short getShowPoint() const { return _showpoint; }
  short getShowScientific() const { return _scientific; }

  void removeZeroes(double,short,int*);

  int sigrelease(int signal);
  int sighold(int signal);

};

#endif
