// 
//  Copyright (C) 2007,2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <strstream>
#include <string.h>
#include "AhelpStrings.hh"



ostream& operator<<(ostream &oss,const AhelpDataObject &obj)
{
  obj.Print(oss);
  return oss;
}

istream& operator>>(istream &iss,AhelpDataObject &obj)
{
  obj.Read(iss);
  return iss;
}







void AhelpStrings::Print(ostream &oss) const
{
  oss << theString;
}

void AhelpStrings::Read(istream &iss)
{
  char *temp = new char[1024];
  iss >> temp;
  delete [] theString;
  theString = temp;
}


AhelpStrings::AhelpStrings() : AhelpDataObject()
{
  theString = new char [1];
  theString [0] = '\0';
}

AhelpStrings::AhelpStrings(char character) : AhelpDataObject()
{
  theString = new char [2];
  theString [0] = character;
  theString [1] = '\0';
}

AhelpStrings::AhelpStrings(const char* string) : AhelpDataObject()
{
  if (string != 0)
    {
    theString = new char [strlen (string) + 1];
    strcpy (theString, string);
    }
  else
    {
    theString = new char [1];
    theString [0] = '\0';
    }
}

AhelpStrings::AhelpStrings(const AhelpStrings& string) : AhelpDataObject()
{
  theString = new char [strlen (string.theString) + 1];
  strcpy (theString, string.theString);
}

AhelpStrings::AhelpStrings(const AhelpStringsList &list,char delim) : AhelpDataObject()
{
  theString = new char [1];
  theString [0] = '\0';
  Join(list,delim);
}

AhelpStrings::~AhelpStrings() 
{
  if (theString)
  {
    delete [] theString;
    theString = NULL; 
  }
}

AhelpStrings AhelpStrings::emptyString()
{
  AhelpStrings string;

  return string;
}

unsigned int AhelpStrings::length() const
{
  return strlen (theString); 
}

int
AhelpStrings::count(char theChar) const
{
	char *p=theString;
	int  count=0;
	while ((p = strchr(p,theChar)))
	{
		p++; count++;
	}

	return count;
}

void AhelpStrings::append (char character) 
{
char *ctemp;
char *temp;

  temp = new char [strlen (theString) + 1];
  strcpy (temp, theString);

  ctemp = new char [2];
  ctemp [0] = character;
  ctemp [1] = '\0';
 
  if (theString)
    delete [] theString;
  theString = new char [strlen (temp) + strlen (ctemp) + 1];
  
  strcpy (theString, temp);
  strcat (theString, ctemp);

  delete [] ctemp;
  delete [] temp;
}

void AhelpStrings::append (const char* string) 
{
  if (string != 0)
    {
    char* temp;
    
    temp = new char [strlen (theString) + 1];
    strcpy (temp, theString);

    delete [] theString;
    theString = new char [strlen (temp) + strlen (string) + 1];

    strcpy (theString, temp);
    strcat (theString, string);

    delete [] temp;
    }
}

void AhelpStrings::append (const AhelpStrings& string) 
{
  char* temp;

  temp = new char [strlen (theString) + 1];
  strcpy (temp, theString);

  delete [] theString;
  theString = new char [strlen (temp) + strlen (string.theString) + 1];

  strcpy (theString, temp);
  strcat (theString, string.theString);

  delete [] temp;
}

void AhelpStrings::append(int val) 
{
	append((long)val);
}

void AhelpStrings::append(unsigned long val) 
{
  ostrstream buffer;
  char *ichar;
  buffer.clear();
  buffer << val << '\0';;
  ichar = buffer.str();
  append (ichar);
  delete [] ichar;
}

void AhelpStrings::append(long val) 
{
  ostrstream buffer;
  char *ichar;
  buffer.clear();
  buffer << val << '\0';;
  ichar = buffer.str();
  append (ichar);
  delete [] ichar;
}

void AhelpStrings::append (double real)
{
ostrstream buffer;
char *ichar;

  buffer.clear();
  buffer.setf(ios::fixed, ios::floatfield);
  buffer << real << '\0';;
  ichar = buffer.str();
  append (ichar);
  delete [] ichar;

}

void AhelpStrings::clear() 
{
  delete [] theString;
  theString = new char [1];
  theString [0] = '\0';
}

void AhelpStrings::lowercase() 
{
  for (int i=0; (size_t)i <= strlen (theString); i++)
    theString[i] = tolower ((theString[i]));
}

void AhelpStrings::prepend (char character) 
{
char *ctemp;
char *temp;

  temp = new char [strlen (theString) + 1];
  strcpy (temp, theString);

  ctemp = new char [2];
  ctemp [0] = character;
  ctemp [1] = '\0';
 
  delete [] theString;
  theString = new char [strlen (temp) + strlen (ctemp) + 1];
  
  strcpy (theString, ctemp);
  strcat (theString, temp);

  delete [] ctemp;
  delete [] temp;
}

void AhelpStrings::prepend (const char* string) 
{
  if (string != 0)
    {
    char* temp;
    
    temp = new char [strlen (theString) + 1];
    strcpy (temp, theString);

    delete [] theString;
    theString = new char [strlen (temp) + strlen (string) + 1];

    strcpy (theString, string);
    strcat (theString, temp);

    delete [] temp;
    }
}

void AhelpStrings::prepend (const AhelpStrings& string) 
{
char* temp;

  temp = new char [strlen (theString) + 1];
  strcpy (temp, theString);

  delete [] theString;
  theString = new char [strlen (temp) + strlen (string.theString) + 1];

  strcpy (theString, string.theString);
  strcat (theString, temp);

  delete [] temp;
}

void AhelpStrings::prepend (int integer) 
{
ostrstream buffer;
char *ichar;

  buffer.clear();
  buffer << integer << '\0';;
  ichar = buffer.str();
  prepend (ichar);
  delete [] ichar;

}

void AhelpStrings::uppercase() 
{
  unsigned int length = strlen (theString);

  for (unsigned int ii=0; ii < length; ii++)
    theString[ii] = toupper(theString[ii]);
}

void AhelpStrings::remove(char delim)
{
  if(strchr(theString,delim))
    {
      int len = strlen(theString);
      char *temp = new char[len];
      int count = 0;
      for(int i=0;i<len;i++)
	{
	  if(theString[i] != delim )
	    temp[count++] = theString[i];
	}
      if(count < len)
	temp[count] = '\0';
      delete [] theString;
      theString = temp;      
    }
}

// delete the rest of the string from delim on
void AhelpStrings::remove_rest(char delim)
{
  if(strchr(theString, delim))
    {
      int len = strlen(theString);
      char *temp = new char[len];
      int count = 0;
      while( count <= len )
        {
	  if(theString[count] == delim )
	    break;
	  temp[count] = theString[count];
	  count++;
        }
      temp[count] = '\0';
      delete [] theString;
      theString = temp;
    } 
}

// replaces all multiple spaces with a single space
void AhelpStrings::pack_spaces()
{

  int len = strlen(theString);
  char *temp = new char[len+1];
  int count = 0;
  int first_space = 1;   
   
  for(int i=0;i<len;i++)
  {
    if (theString[i] == ' ')
    {  
       if (first_space){
         temp[count++] = theString[i];
         first_space = 0;  // not first anymore
       }
    } else {  // not a space character
         
      temp[count++] = theString[i]; 

      // back to looking for the 1st ' ' in the rest of the word
      first_space = 1;
    }

  }      
        
  temp[count] = '\0';
  delete [] theString;
  theString = temp;      

}

void AhelpStrings::remove_all_spaces()
{
  char *newstr = new char [strlen(theString)+1];
  char *temp = newstr;
  char *ptr = theString;

  while ( *ptr != '\0' )
  {
    if ( *ptr != ' ' && *ptr != '\t' )
      *temp++ = *ptr++;
    else
      ptr++;
  }

  *temp = '\0';
  
  if ( strlen( newstr) )
  {
    delete [] theString;
    theString = newstr;
  }
}

// removeNewLines defaults to false.
void AhelpStrings::remove_leading_white_space(int removeNewLines) 
{
  char *temp;
  char *ptr = theString;

  while((ptr[0] == ' ') || (ptr[0] == '\t') 
	                || ((ptr[0] == '\n') && removeNewLines) )
  {
      ptr++;
  }

  temp = new char [strlen (ptr) + 1];
  strcpy(temp,ptr);

  delete [] theString;
  theString = new char [strlen(temp) + 1];

  strcpy(theString,temp);

  delete [] temp;
}

// removeNewLines defaults to false.
void AhelpStrings::remove_trailing_white_space(int removeNewLines)
{
  // Work from the end of the string to the begining, and chop
  // any character that is a space, until the first non-space
  // character is encountered.
  for(int i=strlen(theString);i>=0;i--)
    {
      if(theString[i] == '\n' && removeNewLines )
	chop();
      else if(theString[i] == '\n' || theString[i] == '\0')
	continue;
      else if(theString[i] == ' ' || theString[i] == '\t' )
	chop();
      else
	break;
    }
}

Boolean AhelpStrings::isNumeric() const 
{
  unsigned int length = strlen(theString);

  for (unsigned int ii=0; ii < length; ii++)
    {
    if (!isdigit(theString[ii]))
      return FALSE;
    }
  return TRUE;
}

// Only allow "+" or "-" as the first character.
Boolean AhelpStrings::isSignedNumeric() const 
{
   unsigned int length = strlen(theString);

   for (unsigned int ii = 0; ii < length; ii++)
   {
      if (!isdigit(theString[ii]) )
         if ( ii > 0 || theString[ii] != '-' && theString[ii] != '+')
            return FALSE;
   }
   return TRUE;
}

// Check if string is a possible decimal value 
// rules -  1) must contain at least  1 digit (0|1|2|3|4|5|6|7|8|9)
//          2) may contain upto 1 decimal (.)
//          3) may contain upto 1 exponent (e|E)
//          4) may contain upto 2 signs (+|-)
//          5) if sign is not first char it must be preceded by 'e|E'
Boolean AhelpStrings::isDecimal() const 
{
  Boolean rval = FALSE;
  unsigned int length = strlen(theString);

  if (length)
  {
     int d_cnt = 0, p_cnt = 0,      // occurances of digits, decimal point 
         e_cnt = 0, s_cnt = 0;      // occurances of exponent, sign
     int rr;

     for (rr = 0; rr < length; rr++)
     {
        if (theString[rr] == '.') p_cnt++;
           else if ((theString[rr] == '+') || (theString[rr] == '-'))
           {
              s_cnt++;
              if ((rr || (s_cnt == 2)) && (tolower(theString[rr-1]) != 'e'))
                 s_cnt = 99;    // sign not preceded by exponent char
           }
           else if ((theString[rr] == 'e') || (theString[rr] == 'E')) e_cnt++;
              else if (isdigit(theString[rr])) d_cnt++;
                 else p_cnt = 99;   // invalid char
     }
     rval = ((p_cnt < 2) && (d_cnt > 0) &&
        (((e_cnt == 0) && (s_cnt < 2)) || ((e_cnt == 1) && (s_cnt < 3))));
  }
  return rval;
}

Boolean AhelpStrings::isEmpty() const
{
   for (int i = strlen(theString); i--; )
	if (!isspace(theString[i]))
	       return FALSE;
   return TRUE;
}

// Only allow one decimal point.  Also,
// "+" or "-" may only be the first character.
Boolean AhelpStrings::isSignedDecimal() const 
{
  unsigned int length = strlen(theString);
  int decimalHasBeenSeen = 0;

  for (unsigned int ii=0; ii< length;ii++)
    {
      if ( theString[ii] == '.' )
	if ( decimalHasBeenSeen == 1 )
	  return FALSE;
	else
	  decimalHasBeenSeen = 1;
      else
	{
	  if (!isdigit(theString[ii]) )
	    if (  ii != 0 || theString[ii] != '-' && theString[ii] != '+')
	      return FALSE;
	}
    }
  return TRUE;
}

void AhelpStrings::chop()
{
  int len = strlen(theString);
  theString[len-1] = '\0';
}


void AhelpStrings::chopc(AhelpStrings& lastchar)
{
  int len = strlen(theString);
  lastchar = theString[len-1];
  theString[len-1] = '\0';

}


AhelpStringsList AhelpStrings::Split(const char delim, int keepWhiteSpace) const 

{
  AhelpStringsList list;
  if(!strchr(theString,delim))
    {
      if (length() > 0)
        {
          AhelpStrings new_str(theString);
          list.Append(new_str);
        }
      return list;
    }

  char *temp = new char[strlen(theString)+1];
  strcpy(temp,theString);
  istrstream iss(temp,strlen(theString));
  while(iss.good() && !iss.eof())
    {
      iss.getline(temp,strlen(theString),delim);
      AhelpStrings new_str(temp);
      if((delim != ' ')&& !keepWhiteSpace)
	{
	  new_str.remove_leading_white_space();
	  new_str.remove_trailing_white_space();
	}
      list.Append(new_str);
    } 
  delete [] temp;
  return list;
}

// Splits one long string into many short ones.  Each short string
// will be 'len' or less characters, however, we only divide at white
// space, so most of the short strings will be less than 'len' chars long.
// Also, in this context, tabs and newlines do not make sense, so they 
// are removed before the division.
//
// Afterward, the client code may want each string terminated with a newline,
// and may want each string to be prepended, usually with tabs.
//
// appendNewLine defaults to FALSE, and prependStr defaults to the empty string.
//
AhelpStringsList AhelpStrings::Split(int len, Boolean appendNewLine,
			   AhelpStrings prependStr )
{
  AhelpStrings     newStr;
  AhelpStringsList list;
  unsigned int newStart  = 0;
  unsigned int i, newEnd;
  Boolean doPrepend = TRUE;
  if ( prependStr == "" )
    doPrepend = FALSE;

  // Remove any tabs or newlines.
  Substitute(' ','\n');
  Substitute(' ','\t');

  // Loop until we have done the entire string.
  while ( newStart < length() )
    {
      // Loop through, finding one sub string per iteration.
      // Find the last possible character which we could split at.
      newEnd = 0;
      for ( i = newStart; i < newStart + len && i<length(); i++ )
	{
	  if ( theString[i] == ' ' )
	    newEnd = i;
	}

      // If we found nowhere nice to break it, hack it off at the end.
      if ( newEnd == 0 )
	newEnd = newStart + len;

      // Did we make it all the way to the end?
      if ( i == length() )
	newEnd = length();

      newStr = GetSubStr(newStart,newEnd);
      if ( appendNewLine == TRUE )
	newStr.append('\n');
      if ( doPrepend == TRUE )
	newStr.prepend(prependStr);

      list.Append(newStr);
      newStart = newEnd + 1;
    }
  return list;
}

// Substitute one character for another
void AhelpStrings::Substitute(char newchar, char oldchar)
{
   unsigned int length = strlen(theString);

   for(unsigned int  ii=0; ii <= length; ii++)
   {
      if (theString[ii] == oldchar)
         theString[ii] = newchar;
   }  
}

// Substitutes one string with another.
// "allOccurences" defaults to false. Set it to true to replace
// more than just the first match of oldstr.
void AhelpStrings::Substitute(const AhelpStrings &oldstr, const AhelpStrings &newstr, 
			 int allOccurences)
{
  int tempIndex;
  int thisIndex = 0;
  int occurences = 0;
  int newStrLen = newstr.length();
  int oldStrLen = oldstr.length();
  int thisStrLen = length();
  int thisStrLastChar = thisStrLen - 1;
  char *oldStrChars = oldstr.chars();
  char *newStrChars = newstr.chars();
  AhelpStrings searchStr;

  tempIndex = FindStr(oldStrChars);

  // If we don't have any matches, quit now.
  if ( tempIndex < 0 )
    return;

  // How many occurences to change?
  if ( allOccurences == 0 )
    occurences = 1;
  else
    {
      while ( tempIndex > -1 )
	{
	  occurences++;
	  thisIndex = thisIndex + tempIndex + oldStrLen;
	  searchStr = GetSubStr(thisIndex,thisStrLastChar);
	  tempIndex = searchStr.FindStr(oldStrChars);
	}
    }
  
  int len = thisStrLen + occurences * (newStrLen-oldStrLen);
  char *temp = new char[len+1];
  memset(temp,0,len+1);

  // Now make the replacements.
  int tmpCount = 0;
  thisIndex    = 0;
  searchStr    = theString;
  tempIndex    = searchStr.FindStr(oldStrChars);
  while ( tmpCount < occurences )
    {
      tmpCount++;
      strncat(temp, searchStr(),tempIndex);
      strncat(temp,newStrChars, newStrLen);
      thisIndex = thisIndex + tempIndex + oldStrLen;
      searchStr = GetSubStr(thisIndex,thisStrLastChar);
      tempIndex = searchStr.FindStr(oldStrChars);
    }

  strcat(temp, searchStr());

  delete[] theString;
  theString = temp;
}
      
// Returns the substring that occurs between index a and b.
// Returns a empty string if the indexes are invalid.
AhelpStrings AhelpStrings::GetSubStr(int a,int b) const 
{
   int len_val = (int)length(); 
   AhelpStrings substr;
   substr.clear();
   char* tmpstr;

   if((a >= 0 )&& (a <= len_val) && (b >= 0) && (b <= len_val))
   {
      if(a <= b)
      {
	tmpstr = new char[b-a+2];
	strncpy(tmpstr,theString+a,b-a+1);
	tmpstr[b-a+1]=NULL;
	substr = tmpstr;
	delete[] tmpstr;
      }
   }
   return substr;
}


// compares String against another string ignoring case and
// preceeding/trailing whitespace- returns 0 if equivalent 
//
// For consistency with AhelpStrings::remove_leading_white_space() and
// AhelpStrings::remove_trailing_white_space(), we do not disregard newlines
// unless explicitly told to.  (disregardNewLines defaults to zero.)
int AhelpStrings::StrcmpLoose(AhelpStrings other, int disregardNewLines ) const
{
   int result = -1; 
   AhelpStrings *s1, *s2;

   // copy to new AhelpStrings to keep from modifying input
   s1 = new AhelpStrings(theString);
   s2 = new AhelpStrings(other.theString);

   // convert both to lowercase 
   s1->lowercase(); 
   s2->lowercase(); 

   // remove leading/trailing whitespace
   s1->remove_leading_white_space(disregardNewLines); 
   s2->remove_leading_white_space(disregardNewLines); 
   s1->remove_trailing_white_space(disregardNewLines); 
   s2->remove_trailing_white_space(disregardNewLines); 

   result = strcmp(s1->chars(), s2->chars()); 

   delete s1;
   delete s2;

   return (result); 
}

int AhelpStrings::StrcmpLoose(const char* other, int disregardNewLines ) const
{
   int result = -1; 
   AhelpStrings *s1, *s2;

   // copy to new AhelpStrings to keep from modifying input
   s1 = new AhelpStrings(theString);
   s2 = new AhelpStrings(other);

   // convert both to lowercase
   s1->lowercase();
   s2->lowercase();

   // remove leading/trailing whitespace
   s1->remove_leading_white_space(disregardNewLines);
   s2->remove_leading_white_space(disregardNewLines);
   s1->remove_trailing_white_space(disregardNewLines);
   s2->remove_trailing_white_space(disregardNewLines);

   result = strcmp(s1->chars(), s2->chars());

   delete s1;
   delete s2;

   return (result); 
}


void AhelpStrings::Join(const AhelpStringsList &list,char delim)
{
  clear();
  if(((AhelpStringsList)list).GetLength() > 0)
    {
      int first = 1;
      AhelpListIterator<AhelpStrings> it((AhelpStringsList*)&list);
      for(it.First();!it.IsDone();it.Next())
	{
	  if(!first && delim >=0)
	    append(delim);
	  if(first)
	    *this = it.GetCurrent();
	  else
	    append(it.GetCurrent());
	  first = 0;
	}
    }
}

// Finds the first occurance of the character in the strings.
// If there are no occurances, then a -1 is returned.
int AhelpStrings::FindChar(char delim) const 
{
   unsigned int length = strlen(theString); 
   int count = -1;
   for(unsigned int ii = 0; ii < length; ii++)
   {
      if(theString[ii] == delim)
      {
         count = ii;
         break;
      }	  
   }
   return count;
}

// Finds nth occurance (default is first) of the character.
// If the nth instance cannot be found, a -1 is returned.
int AhelpStrings::FindChar(char delim, int nth) const 
{
   unsigned int length = strlen(theString); 
   int count;
   for(unsigned int ii = 0; ii < length; ii++)
   {
      if(theString[ii] == delim)
      {
         count = ii;
         if (--nth == 0) break;
      }	  
   }
   if (nth == 0)
      return count;
   else
      return -1;
}

// Finds the last occurance of the character in the strings.
// If there are no occurances, then a -1 is returned.
int AhelpStrings::FindLastChar(char delim) const 
{
  int count = -1;
  for(int i=strlen(theString);i>=0;i--)
    {
      if(theString[i] == delim)
	{
	  count = i;
	  break;
	}	  
    }
  return count;
}

// Finds the first occurance of the string str in the string.
// If there are no occurances, then a -1 is returned, else the
// offset into the string of the substring
int AhelpStrings::FindStr(const char *str) const 
{
  int offset = 0;
  char *substr;

  if ((substr = strstr(theString, str)))
  {
     offset = strlen(theString) - strlen(substr);
     return offset;

  } else {
    return -1;
  }

}
// Case-insensitive version of FindStr
int AhelpStrings::FindStr(const char *str, Boolean loosely) const 
{
  int offset = 0;
  char *substr;
  AhelpStrings s1 = str;
  AhelpStrings s2 = theString;

  if (loosely == TRUE)
  {
	s1.lowercase();
	s2.lowercase();
  }

  if ((substr = strstr(s2.theString,s1.theString)))
  {
	offset = strlen(s2.theString) - strlen(substr);
	return offset;

  }
  else
	return -1;
}

// Pad the front of the string with the
// given number of the given character.
void AhelpStrings::padL(int count, char padc)
{
  for (int i = 0; i < count; ++i)
    prepend(padc);
}

// Pad the end of the string with the
// given number of the given character.
void AhelpStrings::padR(int count, char padc)
{
  for (int i = 0; i < count; ++i)
    append(padc);
}

// Fill a string, if necessary, to given length using given pad char
void AhelpStrings::fill(int toLength,char padc)
{
  int curr_pos = strlen(theString);

  if (toLength > curr_pos)
  {
    char *temp = new char [toLength + 1];
    strcpy (temp, theString);
    while (curr_pos < toLength) temp[curr_pos++] = padc;
    temp[curr_pos] = 0;

    if (theString)
      delete [] theString;

    theString = temp;
  }
}

char* AhelpStrings::operator()() const
{
  return theString;
}

AhelpStrings& AhelpStrings::operator = (char character)
{
  delete [] theString;
  theString = new char [2];
  theString [0] = character;
  theString [1] = '\0';
  return *this;
}

AhelpStrings& AhelpStrings::operator = (const char* string)
{
  delete [] theString;
  if (string != 0)
    {
      theString = new char [strlen (string) + 1];
      strcpy (theString, string);
    }
  else
    {
      theString = new char [1];
      theString [0] = '\0';
    }
  return *this;
}

AhelpStrings& AhelpStrings::operator = (const long val)
{
	this->clear();
	this->append(val);
	return *this;
}

AhelpStrings& AhelpStrings::operator = (const unsigned long val)
{
	this->clear();
	this->append(val);
	return *this;
}

AhelpStrings& AhelpStrings::operator = (const double val)
{
	this->clear();
	this->append(val);
	return *this;
}

AhelpStrings& AhelpStrings::operator = (const AhelpStrings& string)
{
  if (this == &string) return *this;

  delete [] theString;
  theString = new char [strlen (string.theString) + 1];
  strcpy (theString, string.theString);
  return *this;
}

int AhelpStrings::operator == (const AhelpStrings& string) const
{
  return (strcmp (theString, string.theString) == 0);
}

int AhelpStrings::operator == (const char* string) const
{
  return (strcmp (theString, string) == 0);
}

int AhelpStrings::operator != (const AhelpStrings& string) const
{
  return !(strcmp (theString, string.theString) == 0);
}

int AhelpStrings::operator != (const char* string) const
{
  return !(strcmp (theString, string) == 0);
}

int AhelpStrings::operator < (const AhelpStrings& string) const
{
  return (strcmp (theString, string.theString) < 0);
}

int AhelpStrings::operator < (const char* string) const
{
  return (strcmp (theString, string) < 0);
}

int AhelpStrings::operator > (const AhelpStrings& string) const
{
  return (strcmp (theString, string.theString) > 0);
}

int AhelpStrings::operator > (const char* string) const
{
  return (strcmp (theString, string) > 0);
}

int AhelpStrings::operator <= (const AhelpStrings& string) const
{
  return (strcmp (theString, string.theString) <= 0);
}

int AhelpStrings::operator <= (const char* string) const
{
  return (strcmp (theString, string) <= 0);
}

int AhelpStrings::operator >= (const AhelpStrings& string) const
{
  return (strcmp (theString, string.theString) >= 0);
}

int AhelpStrings::operator >= (const char* string) const
{
  return (strcmp (theString, string) >= 0);
}

AhelpStringsListIterator::AhelpStringsListIterator(AhelpStringsList *lst) : 
AhelpListIterator<AhelpStrings>(lst) { }

// Constructor
AhelpStringsList::AhelpStringsList(char dlim) : AhelpList<AhelpStrings>()
{
  delim = dlim;
}

// Constructor
AhelpStringsList::AhelpStringsList(const AhelpStringsList &list) : AhelpList<AhelpStrings>()
{
  delim = list.delim;
  AhelpStringsListIterator it((AhelpStringsList*)&list);
  for(it.First();!it.IsDone();it.Next())
    Append(it.GetCurrent());
}

AhelpStringsList::~AhelpStringsList()
{
  RemoveAll();
}


// Operator
AhelpStringsList& AhelpStringsList::operator=(const AhelpStringsList &list)
{
  if (this == &list) return *this;

  RemoveAll();
  delim = list.delim;
  AhelpStringsListIterator it((AhelpStringsList*)&list);
  for(it.First();!it.IsDone();it.Next())
    Append(it.GetCurrent());
  return *this;
}

// Converts all of the strings into a char**
char** AhelpStringsList::GetChars()
{
  int count = 0;
  typedef char *charptr;
  charptr *list = new charptr[GetLength()+1];
  AhelpStringsListIterator temp(this);
  for(temp.First();!temp.IsDone();temp.Next())
    {
      list[count] = new char[temp.GetCurrent().length() + 1];
      strcpy(list[count],temp.GetCurrent().chars());
      count++;
    }
  list[count] = 0;
  return list;
}

// Returns True if char* exists in StringList
Boolean AhelpStringsList::Exists(char *match_str)
{
  char    **list = this->GetChars();
  int     count  = this->GetLength();
  Boolean retval = FALSE;
  int     ii;
   
  for (ii=0; ii < count; ii++)
    {
    if (strstr(list[ii],match_str) != 0)
       {
       retval =  TRUE;
       break;
       }
    }
  
  for (ii=0; ii < count; ii++)
     delete list[ii];
  delete list;

  return retval; 
}

// Replace a string with another string
void AhelpStringsList::Replace(char *old_str,char *new_str)
{
  AhelpStrings  str; 
  AhelpStringsListIterator temp(this);

  temp.First();
  if (!temp.IsDone())
    {
    for(;!temp.IsDone();temp.Next())
      {
      str = temp.GetCurrent();

      if (strcmp(str.chars(),old_str) == 0)
        {
	if (new_str)
          temp.ReplaceCurrent(new_str);
        else 
          temp.RemoveCurrent();
        break ;
        }
      }
    }
  return; 
}


void AhelpStringsList::Sort(Boolean order,Boolean unique)
{
  MergeSort(*this,order);

  AhelpStringsListIterator temp(this);
  AhelpStrings  str1;


  if ( unique)
    {
      if ( GetLength() > 1 )
	{
	  str1= GetFirst();
	  temp.First();
	  temp.Next();
	  while(!temp.IsDone())
	    {
	      if ( str1 == temp.GetCurrent() )
		{
		  temp.RemoveCurrent();
		}
	      else
		{
		  str1 = temp.GetCurrent();
		};
	      
	      temp.Next();
	    };
	};
    };
  
}


void AhelpStringsList::MergeSort(AhelpStringsList& inList,Boolean order)
{

  if ( inList.GetLength() <= 1 )
    return; 

  AhelpStringsList lList,rList;
  int mIndex=inList.GetLength()/2;
  AhelpStringsListIterator temp(&inList);
  
  temp.First();
  for(int indx=0;indx<mIndex;indx++)
    {
      lList.Append(temp.GetCurrent());
      temp.Next();
    };
  
  for(int indx=mIndex;indx<inList.GetLength();indx++)
    {
      rList.Append(temp.GetCurrent());
      temp.Next();
    };
  
  
  MergeSort(lList,order);
  MergeSort(rList,order);

  if (( (lList.GetLast() > rList.GetFirst()) &&  order )
      ||
      (( lList.GetLast() < rList.GetFirst()) &&  ! order ))
    {
      MergeLists(lList,rList,order,inList);
    }
  else 
    {
      inList=lList;
      inList.Append(rList);
    };

}


void AhelpStringsList::MergeLists(AhelpStringsList& inList1,AhelpStringsList& inList2, Boolean order,AhelpStringsList& outList)
{
  outList.RemoveAll();

  while (( inList1.GetLength() > 0 ) &&  ( inList2.GetLength() > 0) )
    {
      if (( order && (inList1.GetFirst() < inList2.GetFirst() ))
	  ||
	  ( (! order ) && (inList1.GetFirst() > inList2.GetFirst() ))
	   )
	{
	  outList.Append(inList1.GetFirst());
	  inList1.RemoveTop();
	}
      else
	{
	  outList.Append(inList2.GetFirst());
	  inList2.RemoveTop();
	};
    };

  if ( inList1.GetLength() > 0 )
    {
      outList.Append(inList1);
    }
  else
    {
      outList.Append(inList2);
    };

}



// Prints the contents if the String list to the output stream
void AhelpStringsList::Print(ostream &oss) 
{
  AhelpStrings line(*this,delim);
  oss << line;
}

// Reads the contents of the String list from the input stream
void AhelpStringsList::Read(istream &iss) 
{
  AhelpStrings temp;
  iss >> temp;
  AhelpStringsList dumb = temp.Split(delim,0);

  // Remove all of the current data
  RemoveAll();
  
  // Add all of the new data to the list
  AhelpStringsListIterator it(&dumb);
  for(it.First();!it.IsDone();it.Next())
    Append(it.GetCurrent());
}

// Copyright (c) 1995 D J Supel and ISX Corporation


