// 
//  Copyright (C) 2002,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "DTDElement.hh"

/*
**
** DESC: CONSTRUCTOR
**
**
**
**
** PRE: NONE
**
**
**
**
** POST: object allocated
**
**
**
**
*/

DTDElement::DTDElement()
{
  parent = 0;
};




/*
**
** DESC: Copy Constructor
**
**
**
**
** PRE: inElment is DTDElement
**
**
**
**
** POST: inElement is copied to newly created object
**
**
**
**
*/

DTDElement::DTDElement(const DTDElement& inElement)
{
  name = inElement.name;
  attributes = inElement.attributes;
  data =inElement.data;
  parent = inElement.parent;
  children = inElement.children;

};



/*
**
** DESC: Desctructor
**
**
**
**
** PRE: None
**
**
**
**
** POST: Dealocation
**
**
**
**
*/

DTDElement::~DTDElement()
{  

};




/*
**
** DESC: Formats content of DTDElment and puts formatted string into 
**       outFormattedString
**
**
**
** PRE: outFormattedString is a Strings
**
**
**
**
** POST: Internal structures are traversed and contained text data
**       is formatted and copied to outFormattedString
**
**
**
*/

void DTDElement::Format(AhelpStrings& outFormattedString)
{

  if (name == TOK_PCDATA )
    {
      outFormattedString=data;
      outFormattedString.append(" ");
      data.clear();
    }
  else
    {
      AhelpStrings tmpString;

      // By default just concatenate the content of xml blocks
      // without any formatting

      while(children.GetLength() )
        {
          children.GetFirst().Format(tmpString);
          outFormattedString.append(tmpString);
          children.RemoveTop();
          tmpString.clear();
        };
    };
};



/*
**
** DESC: Clear internal structures
**
**
**
**
** PRE: None
**
**
**
**
** POST: Contained textual data is cleared
**
**
**
**
*/

void DTDElement::Reset()
{
  children.RemoveAll();
  data.clear();

};


/*
**
** DESC: Check if DTDElement is empty
**       (data empty and children empty)
**
**
**
** PRE: None
**
**
**
**
** POST: Return true if data is empty string
**                   and ( no children or children are empty too)
**
**
**
*/

bool DTDElement::IsEmpty()
{
  
  AhelpListIterator<DTDElement> childIter(&(children));
  AhelpListIterator<AhelpStrings> attIter(&(attributes));

  if (! data.isEmpty() )
    return false;

  for ( childIter.First();!childIter.IsDone();childIter.Next())
    {
      if ( !childIter.GetCurrent().IsEmpty())
	{
	  return false;
	};
    };

  for(attIter.First();!attIter.IsDone();attIter.Next())
    {
      attIter.Next();
      if ( !attIter.GetCurrent().isEmpty())
	{
	  return false;
	};
    };

  return true;
};




const AhelpStrings DTDElement::TOK_PCDATA="#PCDATA";

