// 
//  Copyright (C) 2002,2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "AhelpList.hh"


template< class Type >
class SortedList : public AhelpList<Type>
{
public:

  void Sort(Boolean order,Boolean unique=FALSE);

private:

  void MergeSort(AhelpList<Type>& inList,Boolean order);
  void MergeLists(AhelpList<Type>& inList1, AhelpList<Type>& inList2, Boolean order,AhelpList<Type>& outList);
};


/*
** DESC: Sorts the list 
**
**
** PRE: order - TRUE for ascending , FALSE otherwise
**      unique - unigue flag to remove duplicates
**
**
** POST: List is sorted and if specified duplicates are removed
**
**
*/

template < class Type >
void SortedList< Type >::Sort(Boolean order,Boolean unique)
{
  MergeSort(*this,order);

  AhelpListIterator<Type> temp(this);
  Type  str1;


  if ( unique)
    {
      if ( AhelpList<Type>::GetLength() > 1 )
	{
	  str1= AhelpList<Type>::GetFirst();
	  temp.First();
	  temp.Next();
	  while(!temp.IsDone())
	    {
	      if ( str1 == temp.GetCurrent() )
		{
		  temp.RemoveCurrent();
		}
	      else
		{
		  str1 = temp.GetCurrent();
		};
	      
	      temp.Next();
	    };
	};
    };
  
}

template < class Type >
void SortedList< Type >::MergeSort(AhelpList<Type>& inList,Boolean order)
{

  if ( inList.GetLength() <= 1 )
    return; 

  AhelpList<Type> lList,rList;
  int mIndex=inList.GetLength()/2;
  AhelpListIterator<Type> temp(&inList);
  
  temp.First();
  for(int indx=0;indx<mIndex;indx++)
    {
      lList.Append(temp.GetCurrent());
      temp.Next();
    };
  
  for(int indx=mIndex;indx<inList.GetLength();indx++)
    {
      rList.Append(temp.GetCurrent());
      temp.Next();
    };
  
  
  MergeSort(lList,order);
  MergeSort(rList,order);

  if (( (lList.GetLast() > rList.GetFirst()) &&  order )
      ||
      (( lList.GetLast() < rList.GetFirst()) &&  ! order ))
    {
      MergeLists(lList,rList,order,inList);
    }
  else 
    {
      inList=lList;
      inList.Append(rList);
    };

}

template < class Type >
void SortedList< Type >::MergeLists(AhelpList<Type>& inList1,AhelpList<Type>& inList2, Boolean order,AhelpList<Type>& outList)
{
  outList.RemoveAll();

  while (( inList1.GetLength() > 0 ) &&  ( inList2.GetLength() > 0) )
    {
      if (( order && (inList1.GetFirst() < inList2.GetFirst() ))
	  ||
	  ( (! order ) && (inList1.GetFirst() > inList2.GetFirst() ))
	   )
	{
	  outList.Append(inList1.GetFirst());
	  inList1.RemoveTop();
	}
      else
	{
	  outList.Append(inList2.GetFirst());
	  inList2.RemoveTop();
	};
    };

  if ( inList1.GetLength() > 0 )
    {
      outList.Append(inList1);
    }
  else
    {
      outList.Append(inList2);
    };

}

