// 
//  Copyright (C) 2002,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __TEXTBLOCK_HH__
#define __TEXTBLOCK_HH__

#include "AhelpList.hh"
#include "AhelpStrings.hh"

class TextBlock
{

  friend ostream& operator<<(ostream& ostr,TextBlock& inBlock);

public:

  TextBlock(int inWidth=TextBlock::DEFAULTWIDTH);
  TextBlock(const TextBlock& inBlock);
  TextBlock(int inWidth, const char* const inStr);
  TextBlock(const char* const inStr);
  virtual ~TextBlock();
  void Paste(int inCoordX, int inCoordY, const TextBlock& inBlock);
  void Paste(int inCoordX, int inCoordY, const char* const  inStr);
  void Paste(int inCoordX, int inCoordY, char inChar);
  void Set(const TextBlock& inBlock);
  void Set(int Width,const char* const inStr);
  void Set(const char* const inStr);
  void Append(const TextBlock& inBlock);
  void Append(int inEmptyLineCount=1);
  void Append(const char* const inStr,int resizeFlag=0);
  void Prepend(int inEmptyLineCount=1);
  void Clear(int inCoordX, int inCoordY, int inWidth, int inHeight);
  void Clear();
  void Resize(int inWidth);
  void Resize();
  void Compact();
  int  IsEmpty();
  int  Width(){ return width;};
  int  Width(int inCoordY);
  int  Height(){return lines.GetLength();};
  void Str(AhelpStrings& outString);

  TextBlock& operator = (const TextBlock&);
  TextBlock& operator = (const char*);

  static int DEFAULTWIDTH;

protected:

  void Paste(char*& inDstStr, int inDstMaxWidth, char* inSrcStr,int inSrcMaxWidth,int inDstIndx);
  void CreateLines(const char* inStr,int inWidth, AhelpList<char*>& outLines);
  int CreateLines(const char* inStr, AhelpList<char*>& outLines);
  void DeleteLines(AhelpList<char*>& outLines);


private:
  int width;
  AhelpList<char*> lines;



};



#endif
