// 
//  Copyright (C) 2002,2005,2007,2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//




#ifndef __XMLFPARSER_HH__
#define __XMLFPARSER_HH__

#include "xercesc/util/XMLUniDefs.hpp"
#include "xercesc/sax2/Attributes.hpp"
#include "xercesc/sax2/DefaultHandler.hpp"
#include "xercesc/framework/XMLFormatter.hpp"
#include "xercesc/framework/LocalFileInputSource.hpp"
#include "xercesc/util/PlatformUtils.hpp"
#include "xercesc/util/TransService.hpp"
#include "xercesc/sax2/SAX2XMLReader.hpp"
#include "xercesc/sax2/XMLReaderFactory.hpp"

#include "AhelpStrings.hh"
#include "DTDElement.hh"

XERCES_CPP_NAMESPACE_USE

// -----------------------------------------------------------------------
// Your basic element used in creating parse tree for documents
// -----------------------------------------------------------------------


class XMLFParser : public DefaultHandler , private XMLFormatTarget
{
public:
  // -----------------------------------------------------------------------
  //  Constructors
  // -----------------------------------------------------------------------
  XMLFParser() throw(int);
  virtual ~XMLFParser() ;
  
  int parse(const AhelpStrings& inFilePath,
	    const AhelpStrings& inDTDPath,
	    bool inValidateFlag,
	    DTDElement& outElement);

protected:
  // -----------------------------------------------------------------------
  //  Implementations of the format target interface
  // -----------------------------------------------------------------------
  
#ifdef __XERCES_2_8__

  void writeChars(const XMLByte* const toWrite,
		  const unsigned int count,
		  XMLFormatter* const formatter);  

#else

  void writeChars(const   XMLByte* const  toWrite, 
		  XMLSize_t    count, 
		  XMLFormatter* const   formatter);
  
#endif

  
  
  // -----------------------------------------------------------------------
  //  Implementations of the SAX DocumentHandler interface
  // -----------------------------------------------------------------------
  void endDocument();
  
  void endElement( const XMLCh* const uri, 
		   const XMLCh* const localname, 
		   const XMLCh* const qname);
  
#ifdef __XERCES_2_8__

void characters(const     XMLCh* const    chars, 
			    const   unsigned int    length);
#else
  
void characters(const     XMLCh* const    chars, 
			    XMLSize_t length);
#endif

  
  void startDocument();
  
  void startElement(	const   XMLCh* const    uri,
			const   XMLCh* const    localname,
			const   XMLCh* const    qname,
			const   Attributes&		attributes);
  
  
  
  // -----------------------------------------------------------------------
  //  Implementations of the SAX ErrorHandler interface
  // -----------------------------------------------------------------------
  void warning(const SAXParseException& exception) throw(int);
  void error(const SAXParseException& exception) throw(int);
  void fatalError(const SAXParseException& exception) throw(int);
  void formatErrorMsg(const char* inHeader,
		      const char* inFilePath,
		      int  inLineNo,
		      int  inColNo,
		      const char* inMessage,
		      AhelpStrings& outMessage);
  
  // -----------------------------------------------------------------------
  //  Implementations of the SAX DTD location resolver  interface
  // -----------------------------------------------------------------------

  InputSource* resolveEntity (const XMLCh* const publicId, const XMLCh* const systemId );

  DTDElement* top;
  DTDElement* endPtr;

  LocalFileInputSource* DTDSource;

  static SAX2XMLReader* parser;
  static XMLFormatter* fFormatter;

  static int globalEnvInitFlag;
  static int instanceCount;
};


#endif
