/*
 *  /data/udoc2/bin/java/main.java
 * 
 * Converts a CXCDS Tools xml file into LaTeX.
 *
 */

import java.util.*;
import java.io.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.*;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class xml2tex 
{
  // To write to the file.
  static FileWriter      _write;
  static char  DBFIELDSEPARATOR;




  //
  // Reading a stored XML Tools file, and creating LaTeX output.
  //
  public static void main (String argv [])
    {
      InputSource	input;
      XmlDocument	doc;
      String            outDir="";
      String            inFile="";
      boolean           oneOutFile = false;
      String            indexPath="";


      DBFIELDSEPARATOR = '\u001D';

      
      if (argv.length < 1  || argv.length > 4 )
	{
	  System.err.println ("Usage: java xml2tex -i IndexDir [-d <DIR>] [-oneOutFile] "
			      + " <filename> ");
	  System.exit (1);
	}

      // Parse the command line arguments.
      // If it is a '-d', the next argument is the output Directory.
      // Otherwise, it is the input file to use.
      for ( int ci=0; ci < argv.length; ci++ )
	{
	  if ( argv[ci].equals("-d") )
	    {
	      ci++;
	      outDir = argv[ci];

	      if ( outDir.charAt(0) != '/' )
		{
		  System.err.println ("\n \n I feel bad, but must insist that "
				      + "you provide "
				       + "a full path name for the "
				       + "directory.\n");
		  System.exit (1);
		}
	      outDir = outDir.concat("/");
	    }
	  else if ( argv[ci].equals("-i") )
	    {
	      indexPath = argv[++ci];

	    }
	  else if ( argv[ci].equalsIgnoreCase("-oneOutFile") )
	    oneOutFile = true;
	  else
	    inFile = argv[ci];
	}
      
      if (indexPath.equals(""))
	{
	  System.err.println ("Usage: java xml2tex -i IndexDir [-d <DIR>] [-oneOutFile] "
			      + " <filename>");
	  System.out.println ("** You need to specify -i option");
	  
	  System.exit (1);
	};

      try {
	boolean doLongParams = false;
	int i, ii, iii, iv;
	String  keyStr, contextStr, parentStr;
	Node     parentN;
	NodeList rootList;

	Node     currEntryN,    currSynopsisN,    currSyntaxN;
	Element  currEntryE,    currSynopsisE,    currSyntaxE;
	NodeList currEntryList, currSynopsisList, currSyntaxList;

	Node     currDescN,    currParamN,    currDescChildN;
	Element  currDescE,    currParamE,    currDescChildE;
	NodeList currDescList, currParamList, currDescChildList;

	Node     currCaptionN,    currRowN,    currDataN;
	Element  currCaptionE,    currRowE,    currDataE;
	NodeList currCaptionList, currRowList, currDataList;

	Node     currListN,    currItemN,    currLastModN;
	Element  currListE,    currItemE,    currLastModE;
	NodeList currListList, currItemList, currLastModList;
	
	Node     currBugsN,    currVersionN,    currQExampleN;
	Element  currBugsE,    currVersionE,    currQExampleE;
	NodeList currBugsList, currVersionList, currQExampleList;

	Node     currLineN, currAdescN, currHrefN;
	Element  currLineE, currAdescE, currHrefE;
	NodeList currLineList,currAdescList, currHrefList;

	String seealsogroupsStr;


	// turn the filename into an input source
	input = Resolver.createInputSource (new File (inFile));

	// turn it into an in-memory object
	// ... the "true" flag says we want to validate.

	doc = XmlDocument.createXmlDocument (input, false);

	// normalize text representation
	//	doc.getDocumentElement().normalize();

	// Get the root 'cxchelptopics' element.
	rootList = doc.getElementsByTagName( "cxchelptopics" );
	Node rootN = rootList.item( 0 );
	Element rootE = (Element) rootN;

	if ( oneOutFile == true )
	  {
	    if( inFile.endsWith(".xml") )
		{
		  outDir = inFile.substring(0,inFile.length()-4);
		};

	    outDir = outDir+".tex";
	    File tmpFile = new File(outDir);

	    _write = new FileWriter(tmpFile.getAbsolutePath());
	    System.err.println ("Output File: "+tmpFile.getAbsolutePath()+"\n");
	  }



	// Loop for all the 'ENTRY' tags.  
	// We write one file per ENTRY

	currEntryList = rootE.getElementsByTagName("ENTRY");
	for ( i=0; i<currEntryList.getLength(); i++ )
	  {
	    currEntryN = currEntryList.item( i );
	    currEntryE = (Element) currEntryN;
	    keyStr = currEntryE.getAttribute("key");
	    contextStr = currEntryE.getAttribute("context");
	    seealsogroupsStr = currEntryE.getAttribute("seealsogroups");

	    if ( oneOutFile == false )
	      {
		_write = new FileWriter(outDir+keyStr+".tex");
		System.err.println ("Output File: "+outDir+keyStr+".tex. \n");
	      }


	    //	    InitDoc();

	    // write instruction for custom names of created files
	    WriteLatexText("\\HTMLset{mytitle}{", false);
	    WriteLatexText(keyStr, false);
	    WriteLatexText("}\n", false);

	    // Write the section;
	    WriteLatexText("\\section{", false);
            WriteLatexText( keyStr, true );
	    WriteLatexText("}\n", false);
	    // Write the label
	    WriteLatexText("\\label{sec:", false);
	    WriteLatexText( keyStr, true );
	    WriteLatexText("}\n", false);



	    // Write the SYNOPSIS.
	    currSynopsisList = currEntryE.getElementsByTagName( "SYNOPSIS" );
	    for ( ii=0; ii<currSynopsisList.getLength(); ii++)
	      {
		currSynopsisN = currSynopsisList.item( ii );
		currSynopsisE = (Element) currSynopsisN;

		// Remember, we have SYNOPSIS as children of PARAM too,
		// so we must ensure that this is a direct child of ENTRY.
		parentN   = currSynopsisN.getParentNode();
		parentStr = parentN.getNodeName();
		if ( parentStr.equals("ENTRY"))
		  {
		    WriteLatexText("\n\n{\\bf SYNOPSIS}\\\\\n", false);
		    //WriteLatexText("\\cmd{", false);
		    //WriteLatexText(keyStr, true);
		    //WriteLatexText("} \n", false);

		    // Now get any text in the SYNOPSIS element.
		    GetXMLText(currSynopsisE, true,true);
		  }
	      }

	    // Write the SYNTAX.
	    currSyntaxList = currEntryE.getElementsByTagName( "SYNTAX" );
	    for ( ii=0; ii<currSyntaxList.getLength(); ii++)
	      {
		currSyntaxN = currSyntaxList.item( ii );

		// Remember, we have SYNTAX as children of other elements too,
		// so we must ensure that this is a direct child of ENTRY.
		parentN   = currSyntaxN.getParentNode();
		parentStr = parentN.getNodeName();
		if ( parentStr.equals("ENTRY"))
		  {
		    WriteLatexText("\n\n{\\bf SYNTAX}\\\\\n", false);
		    WriteLatexText("\\begin{itemize}\n", false);
		    DoSyntax( currSyntaxN );
		    WriteLatexText("\\end{itemize}\n", false);
		  }
	      }

	    // Write the DESCRIPTION.
	    currDescList = currEntryE.getElementsByTagName( "DESC" );
	    for ( ii=0; ii<currDescList.getLength(); ii++)
	      {
		currDescN = currDescList.item( ii );
		currDescE = (Element) currDescN;

		// Make sure this DESC is a direct child of ENTRY.
		if ( currDescN.getParentNode().getNodeName().equals("ENTRY") )
		  {
		    WriteLatexText("\n\n{\\bf DESCRIPTION}\\\\\n", false);
		    DoDesc( currDescN );
		  }
	      }




	    // Write the QEXAMPLE.
	    currQExampleList = currEntryE.getElementsByTagName( "QEXAMPLE" );
	    if ( currQExampleList.getLength() > 0 )
	      {
		WriteLatexText("\n\n{\\bf EXAMPLES}\\\\\n", false);
		WriteLatexText("\\begin{itemize}\n", false);
		WriteLatexText("\\item\n", false);
	      }
	    for ( ii=0; ii<currQExampleList.getLength(); ii++)
	      {
		currQExampleN = currQExampleList.item( ii );
		currQExampleE = (Element) currQExampleN;

		// Write the SYNTAX.
		currSyntaxList = currQExampleE.getElementsByTagName( "SYNTAX" );
		for ( iii=0; iii<currSyntaxList.getLength(); iii++)
		  {
		    currSyntaxN = currSyntaxList.item( iii );

		    // Remember, we have SYNTAX as children of PARA too,
		    // so we must ensure that this is a direct child of 
		    // QEXAMPLE.  (PARA is a child of DESC is a child of
		    // QEXAMPLE.)
		    parentN   = currSyntaxN.getParentNode();
		    parentStr = parentN.getNodeName();
		    if ( parentStr.equals("QEXAMPLE"))
		      {
			DoSyntax( currSyntaxN );
		      }
		    
		  }

		// Write the DESC.
		currDescList = currQExampleE.getElementsByTagName( "DESC" );
		for ( iii=0; iii<currDescList.getLength(); iii++)
		  {
		    currDescN = currDescList.item( iii );
		    currDescE = (Element) currDescN;
		    DoDesc( currDescN );
		  };

		WriteLatexText("~\\\\~\\\\\n", false);
	      }
	    if ( currQExampleList.getLength() > 0 )
	      WriteLatexText("\\end{itemize}\n", false);


	    // Write the PARAM table.
	    currParamList = currEntryE.getElementsByTagName( "PARAM" );

	    // Write the header data before doing the first PARAM.
	    if ( currParamList.getLength() > 0 )
	      {
		WriteLatexText("\n\n{\\bf PARAMETERS}\\\\\n", false);
		WriteLatexText("The table below describes each of the ", false);
		WriteLatexText("parameters. Parameters listed in ", false);
		WriteLatexText("{\\bfseries bold} are {\\em required}.\n", 
			       false);
		WriteLatexText("\\begin{longtable}{|llllllllll|p", false);
		WriteLatexText("{1.0in}|}\n", false);
		WriteLatexText("\\hline ", false);
		WriteLatexText("name & type & ftype & ", false);
		WriteLatexText("default & min & max & ", false);
		WriteLatexText("units & reqd & stacks & ", false);
		WriteLatexText("autoname & description \\\\\n", false);
		WriteLatexText("\\hline \\hline\n", false);
		WriteLatexText("\\endhead\n", false);
	      }

	    for ( ii=0; ii<currParamList.getLength(); ii++)
	      {
		String reqdStr;

		// Write each PARAM.
		currParamN = currParamList.item( ii );
		currParamE = (Element) currParamN;

		reqdStr     = currParamE.getAttribute("reqd");

		// If the parameter is required, make it bold.
		if  ( reqdStr.equalsIgnoreCase("yes") )
		  WriteLatexText("\n{\\bf ", false);

		WriteLatexText( currParamE.getAttribute("name"), true );

		if  ( reqdStr.equalsIgnoreCase("yes") )
		  WriteLatexText("} ", false);

		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("type"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("filetype"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("def"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("min"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("max"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("units"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("reqd"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("stacks"), true );
		WriteLatexText(" & ", false);
		WriteLatexText( currParamE.getAttribute("autoname"), true );
		WriteLatexText(" & ", false);


		// Now get the SYNOPSIS of the PARAM element.
		currSynopsisList = currParamE.getElementsByTagName("SYNOPSIS");
		if ( currSynopsisList.getLength() > 0 )
		  {
		    currSynopsisN = currSynopsisList.item( 0 );
		    currSynopsisE = (Element) currSynopsisN;
		    GetXMLText(currSynopsisE, true,true);
		  }
		    
		WriteLatexText(" \\\\ \n", false);

		// Finally, do any PARAMs have a long DESC?
		currDescList = currParamE.getElementsByTagName("DESC");
		if ( currDescList.getLength() > 0 )
		  doLongParams = true;
	      }

	    // End the parameter summary table.
	    if ( currParamList.getLength() > 0 )
	      WriteLatexText("\n\\hline\n\\end{longtable}\n", false);
	    
	    // Now write the long descriptions for the parameters.
	    // Write the header data before doing the first PARAM.
	    if ( currParamList.getLength() > 0 && doLongParams == true )
	      {
		WriteLatexText("\n\n{\\bf Detailed Parameter ", false);
		WriteLatexText("Descriptions}\n", false);
		WriteLatexText("\\begin{description}\n", false);
	      }

	    for (ii=0; ii<currParamList.getLength() && doLongParams==true; ii++)
	      {
		// Write each PARAM.
		currParamN = currParamList.item( ii );
		currParamE = (Element) currParamN;

		// If the current PARAM element has a DESC, write it.
		currDescList = currParamE.getElementsByTagName("DESC");
		if ( currDescList.getLength() > 0 )
		  {
		    currDescN = currDescList.item( 0 );
		    currDescE = (Element) currDescN;

		    WriteLatexText("\\item[", false);
		    WriteLatexText( currParamE.getAttribute("name"), true );
		    WriteLatexText("]\n", false);

		    DoDesc( currDescN );
		  }
	      }

	    // End the long parameter summary.
	    if ( currParamList.getLength() > 0  && doLongParams == true )
	      WriteLatexText("\n\\end{description}\n", false);




	    // Write the ADESC.
	    currAdescList = currEntryE.getElementsByTagName( "ADESC" );
	    for ( ii=0; ii<currAdescList.getLength(); ii++)
	      {
		currAdescN = currAdescList.item( ii );
		currAdescE = (Element) currAdescN;

		// Make sure this DESC is a direct child of ENTRY.
		if ( currAdescN.getParentNode().getNodeName().equals("ENTRY") )
		  {
		    String tmpStr = currAdescE.getAttribute("title");
		    
		    tmpStr.trim();

		    if (  tmpStr.length() > 0 )
		      {
			WriteLatexText("\n\n{\\bf "+ tmpStr +"}\\\\\n", false);
		      };

		    DoDesc( currAdescN );
		  }
	      }






	    // Write the BUGS.
	    currBugsList = currEntryE.getElementsByTagName( "BUGS" );
	    for ( ii=0; ii<currBugsList.getLength(); ii++)
	      {
		currBugsN = currBugsList.item( ii );
		currBugsE = (Element) currBugsN;

		WriteLatexText("\n\n{\\bf BUGS}\\\\\n", false);

		// Now get any text in the BUGS element.
		GetXMLText(currBugsE, true,true);
	      }

	    // Write the LASTMODIFIED.
	    currLastModList = currEntryE.getElementsByTagName( "LASTMODIFIED" );
	    for ( ii=0; ii<currLastModList.getLength(); ii++)
	      {
		currLastModN = currLastModList.item( ii );
		currLastModE = (Element) currLastModN;

		WriteLatexText("\n\n{\\bf LAST MODIFIED}\\\\\n", false);

		// Now get any text in the LASTMODIFIED element.
		GetXMLText(currLastModE, true,true);
	      }

	    // Write the VERSION.
	    currVersionList = currEntryE.getElementsByTagName( "VERSION" );
	    for ( ii=0; ii<currVersionList.getLength(); ii++)
	      {
		currVersionN = currVersionList.item( ii );
		currVersionE = (Element) currVersionN;

		WriteLatexText("\n\n{\\bf VERSION}\\\\\n", false);

		// Now get any text in the VERSION element.
		GetXMLText(currVersionE, true,true);
	      }


	    String relativeHtmlPath = GetHtmlRelativePath(currEntryE,keyStr);
	    
	    //Write SEE ALSO 

	    DoSeeAlso(seealsogroupsStr, keyStr,contextStr,indexPath,relativeHtmlPath);
	    

	    //	    FinalizeDoc();


	    // End the document.
	    // WriteLatexText("\n\\end{document}\n", false);

	    WriteLatexText("\n",false);
	    // Make sure everything gets written to the file.
	    _write.flush();
	  }




      } catch (SAXParseException err) {
	System.out.println ("** Parsing error" 
			    + ", line " + err.getLineNumber ()
			    + ", uri " + err.getSystemId ());
	System.out.println("   " + err.getMessage ());
	// print stack trace as below

	err.printStackTrace();
      } catch (SAXException e) {
	Exception	x = e.getException ();

	((x == null) ? e : x).printStackTrace ();

      } catch (Throwable t) {
	t.printStackTrace ();
      }

      System.exit (0);
    }

  // This generates a string for any possible SYNTAX element.
  // It must generate a string, because it can be called from within
  // GetXMLText(), because SYNTAX may occur randomly within PARA.
  // The three direct parents SYNTAX may have are: ENTRY, QEXAMPLE, and PARA.
  private static void DoSyntax( Node currSyntaxN )
    throws java.io.IOException
    {
      int i;
      String   parentStr;
      String   preText = "";
      String   postText = "";
      Node     currLineN, parentN;
      Element  currLineE, parentE;
      NodeList currLineList;

      parentN   = currSyntaxN.getParentNode();
      parentStr = parentN.getNodeName();

      // Whether the parent is ENTRY, QEXAMPLE, or PARA, the latex
      // does vary a little bit.
      if ( parentStr.equals("ENTRY") )
	{
	  preText  = "\\item \\param{";
	  postText = "}\\\\\\\n";
	}
      else if ( parentStr.equals("QEXAMPLE") )
	{
	  preText  = "\\param{";
	  postText = "}\\\\\\\n";
	}
      else if ( parentStr.equals("PARA") )
	{
	  preText  = "\n {\\tt ";
	  postText = " } ~\\\\ \n";
	}
	  
      // Now get any text in the LINES of the SYNTAX element.
      Element  currSyntaxE = (Element) currSyntaxN;
      currLineList = currSyntaxE.getElementsByTagName("LINE");
      for ( i=0; i<currLineList.getLength(); i++)
	{
	  currLineN = currLineList.item(i);
	  currLineE = (Element) currLineN;

	  WriteLatexText(preText, false);
	  GetXMLText(currLineE, true,true);
	  WriteLatexText(postText, false);
	}
      return;
    }

  // This handles any possible DESC element.
  // It is up to the caller to ensure that it is only called for the
  // correct DESC tags that it is interested in.
  private static void DoDesc( Node currDescN )
    throws java.io.IOException
    {
      int      i,ii;
      String   returnStr = "";
      NodeList currList;
      Node     parentN;
      String   parentStr;

      // LaTeX strings.
      String   captionPreText,  captionPostText;
      String      paraPreText,     paraPostText;
     String  passthruPreText, passthruPostText;
      String  verbatimPreText, verbatimPostText;
      String      listPreText,     listPostText;
      String      itemPreText,     itemPostText;
      String       rowPreText,      rowPostText;
      String                      tablePostText;

      Node     currDescChildN,    currListN,    currItemN, currVerbatimN;
      Element  currDescChildE,    currListE,    currItemE, currVerbatimE;
      NodeList currDescChildList, currListList, currItemList, currVerbatimList;;

      Node     currCaptionN,    currRowN,    currDataN;
      Element  currCaptionE,    currRowE,    currDataE;
      NodeList currCaptionList, currRowList, currDataList;
	
      // Currently, the following LaTeX is similar, no matter which
      // element is the parent.
      rowPreText       = "";
      rowPostText      = "\\\\\n";
      paraPreText      = "";
      paraPostText     = "~\\\\\n~\\\\\n";
      passthruPreText  = "\n";
      passthruPostText = "\n";
      verbatimPreText  = "\n\\begin{verbatim}";
      verbatimPostText = "\n\\end{verbatim}";
      listPreText      = "\n\\begin{itemize}\n";
      listPostText     = "\n\\end{itemize}\n";
      itemPreText      = "\\item ";
      itemPostText     = "\n";
      tablePostText    = "\\end{tabular}\\\\\n\\\\\n";
      captionPreText   = "";
      captionPostText  = "~\\\\\n";

      // LaTeX that varies between the different parentages should be
      // handled here.
      parentN   = currDescN.getParentNode();
      parentStr = parentN.getNodeName();
      if ( parentStr.equals("ENTRY") )
	{
	  // Currently, ENTRY->DESC does not require any spefic LaTeX changes.
	}
      else if ( parentStr.equals("QEXAMPLE") )
	{
	  paraPostText = "~\\\\\n";
	}
      else if ( parentStr.equals("PARAM") )
	{
	  // Currently, PARAM->DESC does not require any spefic LaTeX changes.
	}


      // Now get any VERBATIM, PARAs, LISTs or TABLEs in DESC.
      currDescChildN = currDescN.getFirstChild();
      while ( currDescChildN != null )
	{
	  currDescChildE = (Element) currDescChildN;
		

	  // Is it a VERBATIM element?
	  if ( currDescChildN.getNodeName().equals("VERBATIM") )
	    {
	      WriteLatexText( verbatimPreText, false );
	      GetXMLText(currDescChildE, true,false);
	      WriteLatexText( verbatimPostText, false );
	    }

    

	  // Is it a PARA element?
	  if ( currDescChildN.getNodeName().equals("PARA") )
	    {
	      
	      String tmpStr =((Element) currDescChildN).getAttribute("title");
	      
	      tmpStr.trim();
	       
	      if ( tmpStr.length() > 0 )
		{
		  WriteLatexText("\n\n{\\bf "+ tmpStr +"}\\\\\n", false);
		};
	      
	      WriteLatexText( paraPreText, false );
	      GetXMLText(currDescChildE, true,true);
	      WriteLatexText( paraPostText, false );
	    }

	  // Is it a LIST element?
	  if ( currDescChildN.getNodeName().equals("LIST") )
	    {
	      // Write a caption?
	      currCaptionList = currDescChildE.getElementsByTagName("CAPTION");
	      if ( currCaptionList.getLength() > 0 )
		{
		  currCaptionN = currCaptionList.item(0);
		  currCaptionE = (Element) currCaptionN;

		  WriteLatexText( captionPreText, false );
		  GetXMLText(currCaptionE, true,true);
		  WriteLatexText( captionPostText, false );
		}
			    
	      WriteLatexText( listPreText, false );

	      // Iterate through all the ITEM elements.
	      currItemList = currDescChildE.getElementsByTagName("ITEM");
	      for ( i=0; i< currItemList.getLength(); i++ )
		{
		  currItemN = currItemList.item( i );
		  currItemE = (Element) currItemN;

		  WriteLatexText( itemPreText, false );
		  GetXMLText(currItemE, true,true);
		  WriteLatexText( itemPostText, false );
		}

	      WriteLatexText( listPostText, false );
	    }

	  // Is it a TABLE element?
	  if ( currDescChildN.getNodeName().equals("TABLE") )
	    {
	      // Write a caption?
	      currCaptionList = currDescChildE.getElementsByTagName("CAPTION");
	      if ( currCaptionList.getLength() > 0 )
		{
		  currCaptionN = currCaptionList.item(0);
		  currCaptionE = (Element) currCaptionN;
		  
		  WriteLatexText( captionPreText, false );
		  GetXMLText(currCaptionE, true,true);
		  WriteLatexText( captionPostText, false );
		}
			    
	      // We must know how many DATA elements in a ROW!
	      // We assume the user was consistent, thought they
	      // would mess us up if they were not!
	      currDataList = currDescChildE.getElementsByTagName("DATA");
	      currRowList =currDescChildE.getElementsByTagName("ROW");
			
	      WriteLatexText("\n\\begin{tabular}{", false);
	      int tempI = currDataList.getLength() /
		currRowList.getLength();
	      for ( i=0; i<tempI; i++ )
		WriteLatexText("l", false);
	      WriteLatexText("}\n", false);

	      // Iterate through all the ROW elements.
	      for ( i=0; i< currRowList.getLength(); i++ )
		{
		  currRowN = currRowList.item( i );
		  currRowE = (Element) currRowN;

		  WriteLatexText( rowPreText, false );

		  // Iterate through all the DATA elements.
		  currDataList =currRowE.getElementsByTagName("DATA");
		  for ( ii=0; ii< currDataList.getLength(); ii++ )
		    {
		      currDataN = currDataList.item(ii);
		      if ( ii > 0 )
			WriteLatexText(" & ", false);
		      currDataE = (Element) currDataN;
		      GetXMLText(currDataE, true,true);
		    }
		  WriteLatexText( rowPostText, false );
		}
	      WriteLatexText( tablePostText, false );
	    }


	  

	  currDescChildN = currDescChildN.getNextSibling();
	}
    }



  private static void DoSeeAlso(String inSeealsogroupsStr, String inKeyStr, String inContextStr,String inIndexPath, String inRelativeHtmlPath)
       throws java.io.IOException
  {
    String tmpContext;
    String tmpKey;
    String tmpHtml;
    Vector tmpVec = GetSeeAlsoKeys(inSeealsogroupsStr,inIndexPath+"/doc/xml/");

    if ( tmpVec.size() > 1 )
      {
	WriteLatexText("\n\n{\\bf SEE ALSO}\\\\\n", false);
	
	for(int indx=0;indx<tmpVec.size();indx++)
	  {
	    Vector currRecord = Split((String)tmpVec.elementAt(indx));
	    tmpKey = (String)(currRecord.elementAt(0));
	    tmpContext = (String)(currRecord.elementAt(1));

	    if ( tmpKey.equals(inKeyStr) && inContextStr.equals(tmpContext) )
		 continue;

	    tmpHtml = GetHTMLEntry(tmpKey,tmpContext,inIndexPath+"/doc/xml/");

	    if ( tmpHtml != "")
	      {
		tmpHtml = inRelativeHtmlPath + tmpHtml;
		WriteLatexText("\n\\begin{htmlonly}\n\\htmladdnormallink{", false);
		WriteLatexText(tmpKey+"("+tmpContext+")",true);
		WriteLatexText("}{",false);
		WriteLatexText(tmpHtml,true);
		WriteLatexText("}\n\\end{htmlonly}\n", false);  
		
	      }
	    else
	      {
		WriteLatexText(tmpKey+"("+tmpContext+")",true);
	      };

	    if ( indx < (tmpVec.size()-1) )
	      {
		WriteLatexText(" , ",false);
	      }
	  }
	
      };

    /*
    System.out.println(tmpStr);


    WriteLatexText("\n\n{\\bf SEE ALSO}\\\\\n", false);
    
    WriteLatexText("\n\\begin{htmlonly}\n\\htmladdnormallink{", false);
    WriteLatexText("SomeText",true);
    WriteLatexText("}{",false);
    WriteLatexText("http://www.someserver.com/~helppage.html",true);
    WriteLatexText("}\n\\end{htmlonly}\n", false);  
    
    WriteLatexText(" , ",false);
    WriteLatexText("\n\\begin{htmlonly}\n\\htmladdnormallink{", false);
    WriteLatexText("SomeNewText",true);
    WriteLatexText("}{",false);
    WriteLatexText("http://www.someserver.com/~helppage.html",true);
    WriteLatexText("}\n\\end{htmlonly}\n", false);  
    */    
  };



  // Get the text for an element. 
  private static void GetXMLText ( Element elmt, boolean escapeLatex ,boolean compactFlag)
    throws java.io.IOException
    {
      Node    tempN;
      Element tempE;
      String value;



      // Now get any text in the element.
      NodeList list = elmt.getChildNodes();

      for ( int iii=0; iii< list.getLength(); iii++)
	{
	  tempN = list.item(iii);


	  // Remember, EQUATION may be stuck randomly inside PARA.
	  if ( tempN.getNodeName().equals("EQUATION"))
	    {
	      tempE = (Element) tempN;
	      WriteLatexText(" ",true);  

	      // If we have a PASSTHRU child, use it for the equation,
	      // otherwise, just use the ascii equation.
	      NodeList currPassthruList =tempE.getElementsByTagName("PASSTHRU");
	      if ( currPassthruList.getLength() > 0 )
		{
		  Node currPassthruN = currPassthruList.item(0);
		  Element currPassthruE = (Element) currPassthruN;
		  GetXMLText(currPassthruE, false,false);
		}
	      else
		{
		  GetXMLText(tempE, escapeLatex,true);
		}
	      WriteLatexText(" ", true);
	    }
	  // Also, SYNTAX may be stuck randomly inside PARA.
	  else if ( tempN.getNodeName().equals("SYNTAX"))
	    {
	      DoSyntax( tempN );
	      WriteLatexText(" ", true);
	    }
	  // And, PASSTHRU may be stuck randomly inside PARA.
	  else if ( tempN.getNodeName().equals("PASSTHRU"))
	    {
	      tempE = (Element) tempN;
	      WriteLatexText("\n", true);  
	      GetXMLText(tempE, false,false);
	      WriteLatexText("\n", true);  
	    }
	  else if ( tempN.getNodeName().equals("HREF"))
	    {

	      tempE = (Element) tempN;
	      WriteLatexText("\n\\begin{htmlonly}\n\\htmladdnormallink{", false);
	      GetXMLText(tempE, escapeLatex,true);
	      WriteLatexText("}{",false);
	      WriteLatexText(tempE.getAttribute("link"),true);
	      WriteLatexText("}\n\\end{htmlonly}\n", false);  

	    }
	  else if ( ! tempN.getNodeName().equals("XMLONLY"))
	    {


	      value = list.item(iii).getNodeValue();
	      
	      if (compactFlag) 
		{
		  value.trim();// = CompactString(value);
		};

	      if ( value.equals("") || value.equals("\r") )
		continue;
	      else
		{
		  WriteLatexText( value, escapeLatex );
		  //break;
		}
	    }
	}
      return;
    }

  // Write a LaTeX string.  This makes any conversions necessary for LaTex.
  // Notice that to have a backslash in a string within a program, one must 
  // double it up.  This is enforced by Java; there is nothing we can do. 
  private static void WriteLatexText ( String txt, 
				       boolean escapeLatex )
    throws java.io.IOException
    {
      int fromIndex = 0;
      int currIndex;
      String tempTxt = "";

      // Is desired, find any characters which LaTeX does not like, and take
      // the proper steps to get LaTeX to accept them.
      if ( escapeLatex == true )
	{
	  // Carats:  We can't fix them, but we can warn the user.

	  
	  currIndex = txt.indexOf('^', fromIndex);
	  if ( currIndex >= 0 )
	    {
	      System.err.print("Found a carat in the line:\n\t==>  ");
	      System.err.println( txt );
	      System.err.println("");
	    }

	  // Underscores:  Prefix them with '\'.
	  currIndex = txt.indexOf('_',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\_");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('_',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }
	

	  // Left Brackets:  Prefix them with '\'.
	  currIndex = txt.indexOf('{',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\{");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('{',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }
	
	  // Right Brackets:  Prefix them with '\'.
	  currIndex = txt.indexOf('}',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\}");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('}',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }

	  // Pound signs:  Prefix them with '\'.
	  currIndex = txt.indexOf('#',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\#");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('#',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }
	
	  // Dollar signs:  Prefix them with '\'.
	  currIndex = txt.indexOf('$',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\$");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('$',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }
	
	  // Percent signs:  Prefix them with '%'.
	  currIndex = txt.indexOf('%',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\%");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('%',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }

	  // Ampersands:  Prefix them with '\'.
	  currIndex = txt.indexOf('&',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("\\&");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('&',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }




	  currIndex = txt.indexOf('<',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("$<$");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('<',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }



	  currIndex = txt.indexOf('>',fromIndex);
	  if ( currIndex >= 0 )
	    {
	      while ( currIndex >= 0  )
		{
		  tempTxt = tempTxt.concat(txt.substring(fromIndex,currIndex));
		  tempTxt = tempTxt.concat("$>$");
		  fromIndex = currIndex+1;
		  currIndex = txt.indexOf('>',fromIndex);
		}
	      if ( fromIndex < txt.length() )
		tempTxt = tempTxt.concat(txt.substring(fromIndex));

	      txt = tempTxt;
	      tempTxt = "";
	      fromIndex = 0;
	    }



	}
		
      _write.write(txt);
    }





  private static boolean  BinarySearch(RandomAccessFile _file,int inRecordSize, int inRecordCount,String inKey, Vector outInfoVector)
  {
    
    int left=0,right;
    int indx;
    String tmpKey;
    byte tmpOutInfo[] = new byte[inRecordSize];
    int tmpStat;
   

    try
      {
    
	if ( inKey == "")
	  {
	    indx=-1;
	    while ((++indx) < inRecordCount )
	      { 
		_file.seek((indx+1)*inRecordSize);
		if ( _file.read(tmpOutInfo) <= 0 )
		  {
		    return false;
		  };
		
		outInfoVector.addElement(new String(tmpOutInfo));
		
	      };
	    
	    return true;
	  };
	
    
	right = inRecordCount-1;
	indx  = (left+right)/2;  
	
	while (left <= right)
	  {
	    _file.seek((indx+1)*inRecordSize);
	    if ( _file.read(tmpOutInfo) <= 0 )
	      {
		return false;
	      };

	    tmpKey = (String)(Split(new String(tmpOutInfo),DBFIELDSEPARATOR)).elementAt(0);

	    tmpKey.trim();


	    tmpStat = tmpKey.compareTo( inKey) ;

	    if ( tmpStat < 0 )
	      {
		left = indx+1;
	      }
	    else if ( tmpStat > 0)
	      {
		right = indx-1;
	      }
	    else 
	      {
		
		
		
		while ((--indx) >= 0 )
		  {

		    _file.seek((indx+1)*inRecordSize);

		    if ( _file.read(tmpOutInfo) <= 0 )
		      {
			return false;
		      };

		    tmpKey = (String)(Split(new String(tmpOutInfo),DBFIELDSEPARATOR)).elementAt(0);


		    tmpKey.trim();
		    
		    if ( ! tmpKey.equals(inKey) )
		      {
			break;
		      };
		  };
		
		
		while ((++indx) < inRecordCount )
		  { 

		    _file.seek((indx+1)*inRecordSize);
		    if ( _file.read(tmpOutInfo) <= 0 )
		      {
			return false;
		      };

		    tmpKey = (String)(Split(new String(tmpOutInfo),DBFIELDSEPARATOR)).elementAt(0);

		    tmpKey.trim();
		    
		    if ( ! tmpKey.equals(inKey) )
		      {
			break;
		      };


		    outInfoVector.addElement(new String(tmpOutInfo));
		    
		  };
		
		
		return true;
	      };
	    
	    
	    indx = (left+right)/2;
	    
	  };
      }
    catch(Exception e)
      {
	System.out.println("ERROR in BinarySearch method: " + e);
      };
    
    return false;
  };
  
  

  private static Vector Split(String inString,char inDelim)
  {
    Vector tmpVector = new Vector();
    int startIndex=0,endIndex=0;
    String tmpStr;

    while((endIndex=inString.indexOf(inDelim,startIndex)) >=0)
      {
	if ( startIndex == endIndex)
	  {
	    tmpStr="";
	  }
	else
	  {
	    tmpStr = inString.substring(startIndex,endIndex);
	  };

	tmpVector.addElement(tmpStr);
	startIndex=endIndex+1;

      };

    tmpVector.addElement(inString.substring(startIndex,inString.length()));

    return tmpVector;

  };


  private static Vector Split(String inString)
  {

    return Split(inString,' ');
  };


private static Vector GetSeeAlsoKeys(String inSeealsogroups,String inIndexPath) throws java.io.IOException
  {



    Vector seealsogroupsVector= Split(inSeealsogroups);
    int recordSize;
    int recordCount;
    Vector tmpSeealsoVector = new Vector();
    Vector seealsoVector = new Vector();
    Vector currRecordVector = new Vector();
    Vector tmpVector = new Vector();
    String tmpStr;
    RandomAccessFile    _seealsoIndexFile;  
    
    try
      {
	
	_seealsoIndexFile = new RandomAccessFile(inIndexPath+"/CXCHelpSeeAlsoIndex","r");
	
	tmpVector = Split(_seealsoIndexFile.readLine());
	
	recordCount = Integer.parseInt((String)tmpVector.elementAt(0));
	recordSize  = Integer.parseInt((String)tmpVector.elementAt(1));
	
	
	for(int indx=0;indx<seealsogroupsVector.size();indx++)
	  {
	    if ( BinarySearch(_seealsoIndexFile, recordSize , recordCount, (String)(seealsogroupsVector.elementAt(indx)),tmpSeealsoVector) )
	      {
		
		
		for(int indx1=0; indx1<tmpSeealsoVector.size();indx1++)
		  {
		    
		    currRecordVector = Split((String)tmpSeealsoVector.elementAt(indx1),DBFIELDSEPARATOR);
		    
		    if ( ((String)currRecordVector.elementAt(0)).equals((String)seealsogroupsVector.elementAt(indx) ))
		      {
			tmpStr = ((String)currRecordVector.elementAt(1)).trim()+" "+((String)currRecordVector.elementAt(2)).trim();
			
			
			if ( ! seealsoVector.contains(tmpStr.trim()) )
			  {
			    seealsoVector.addElement(tmpStr);
			  };
		      };
		  };
	      };
	  };
	
	_seealsoIndexFile.close();
	
      }
    catch(Exception e)
      {
	System.out.println ("ERROR in GetSeeAlsoKeys method: " + e);
      };
    
    return seealsoVector;
    
  };
  
  private static String GetHTMLEntry(String inKey, String inContext,String inIndexPath) throws java.io.IOException
  {
    
    int recordSize;
    int recordCount;
    Vector tmpAhelpVector = new Vector();
    Vector currRecordVector = new Vector();
    Vector tmpVector = new Vector();
    String tmpStr;
    RandomAccessFile      _ahelpIndexFile;
    
    
    
    try
      {
	_ahelpIndexFile = new RandomAccessFile(inIndexPath+"/CXCHelpIndex","r");
	tmpVector = Split(_ahelpIndexFile.readLine());
	
	recordSize  = Integer.parseInt((String)tmpVector.elementAt(1));

	recordCount = Integer.parseInt((String)tmpVector.elementAt(0));

	
	if ( BinarySearch(_ahelpIndexFile, recordSize , recordCount, inKey,tmpAhelpVector) )
	  {

	    for(int indx=0; indx<tmpAhelpVector.size();indx++)
	      {
		currRecordVector = Split((String)tmpAhelpVector.elementAt(indx),DBFIELDSEPARATOR);
		

		if ( ((String)currRecordVector.elementAt(1)).equals(inContext) )
		  {
		    
		    return (String)currRecordVector.elementAt(3);
		    
		  };
	      };
	    
	  };

	_ahelpIndexFile.close();


      }
    catch( Exception e )
      {
	System.out.println("Error in GetHTMLEntry method: " +e);
      };

    
    return "";

  };
  

  private static  void InitDoc() throws java.io.IOException
  {
   WriteLatexText("\\documentclass{book}",false);

    WriteLatexText("\\usepackage{ASC}",false);
    WriteLatexText("\\usepackage{longtable}",false);
    WriteLatexText("\\usepackage{html}",false);
    WriteLatexText("\\usepackage{epsfig}",false);
    WriteLatexText("\\oddsidemargin=0.0in",false);
    WriteLatexText("\\evensidemargin=0.0in",false);
    WriteLatexText("\\baselineskip=12pt",false);
    WriteLatexText("\\textwidth=6.50in",false);
    WriteLatexText("\\topmargin=-0.25in",false);
    WriteLatexText("\\textheight=8.50in",false);
    WriteLatexText("\\setlength{\\parindent}{0 cm}",false);
    WriteLatexText("\\setlength{\\parskip}{\\baselineskip}",false);
    
    WriteLatexText("\\newcommand{\\rr}[1]{{\\bf #1}} ",false);  
    WriteLatexText("\\newcommand{\\rt}[1]{\\subsubsection{#1}}",false);
    WriteLatexText("\\newcommand{\\ttag}[1]{\\hline \\multicolumn{3}{c}{#1}\\  \\hline}",false);
    WriteLatexText("\\newcommand{\\bold}{\\bfseries}",false);
    WriteLatexText("\\newcommand{\\italic}{\\itshape}",false);
    WriteLatexText("\\newcommand{\\normal}{\\normalfont}",false);
    WriteLatexText("\\begin{document}",false);
    
  };
  
  
  private static  void FinalizeDoc() throws java.io.IOException
  {
    WriteLatexText("\\end{document} ",false);   
  };

  private static String GetHtmlRelativePath(Element inElement, String inKey)
  {
    
    Node tempN;
    Element tempE;
    String tmpString;
    NodeList tempNodeList;
    int tmpTokenCount;
    String tmpReturnString="./";

    tempNodeList = inElement.getElementsByTagName( "ADDRESS" );
    for (int  ii=0; ii<tempNodeList.getLength(); ii++)
      {
	
	NodeList list = ((Element)tempNodeList.item( ii )).getElementsByTagName("URL");
	
	for ( int iii1=0; iii1< list.getLength(); iii1++)
	  {
	    tmpString = list.item(iii1).getChildNodes().item(0).getNodeValue();
	    tmpString = tmpString.trim();

	    if (tmpString.indexOf("/"+inKey+".html",0) > 0 )
	      {
		StringTokenizer tmpTokenizer = new StringTokenizer(tmpString,"/");
		tmpTokenCount = tmpTokenizer.countTokens();
		if ( tmpString.indexOf("/",0) == 1 )
		  tmpTokenCount--;
		for ( int indx=1;indx < tmpTokenCount;indx++)
		  tmpReturnString = tmpReturnString+"../";
		return (tmpReturnString);
	      }
		      
	      }
      }
    System.out.println("ERROR: Could not resolve the path of corresponding "+inKey+".html");
    System.out.println("ERROR: See also links may be incorrect");
    return null;

  }

  
}
  
