// 
//  Copyright (C) 2006-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

//---------------------------------------------------------------------------
//
//  FILE NAME:          TransParam.hh
//
//  DEVELOPMENT:        CXCDS
//
//  DESCRIPTION:
//
//---------------------------------------------------------------------------

#ifndef _TRANSPARAM_HH_
#define _TRANSPARAM_HH_

#include <iostream>
#include <string>
#include <float.h>

#define MAX_LENGTH 128
#define BUFFER_LENGTH 2049

#ifndef DMDEF_H
#ifndef DMNUTILS_H

typedef enum { DT_VOID, DT_INT2, DT_PTR, DT_INT4, DT_REAL4, 
	       DT_REAL8, DT_CHAR, DT_UINT1, DT_LOGICAL, DT_UNK, 
	       DT_UINT2, DT_UINT4, DT_BIT } dmDataType;

#define dmUNKNOWNTYPE DT_UNK
#define dmSHORT       DT_INT2
#define dmLONG        DT_INT4
#define dmFLOAT       DT_REAL4
#define dmDOUBLE      DT_REAL8
#define dmTEXT        DT_CHAR
#define dmBYTE        DT_UINT1
#define dmBOOL        DT_LOGICAL
#define dmUSHORT      DT_UINT2
#define dmULONG       DT_UINT4
#define dmBIT         DT_BIT

#define dmErrCode int
#define dmSUCCESS 1
#define dmFAILURE 0

#endif
#endif

using namespace std;

class TransParam
{

public:

  TransParam();
  TransParam(string in_name, void* in_value, dmDataType in_datatype,
	    string in_desc, long in_nvals=1);
  TransParam(string in_name, void* in_value, dmDataType in_datatype,
	    string in_desc, string in_parent, long in_nvals=1);
  TransParam(TransParam& init);
  ~TransParam();
  TransParam& operator=(TransParam& init);

  void clear();

  // Attribute accessors
  string     get_name();
  void*      get_value();
  string     get_desc();
  string     get_parent();
  dmDataType get_datatype();
  long       get_nvals();

  // Allow user to change parameter value (copy)
  void set_value(void* in_value, dmDataType in_datatype, long in_nvals=1);
  void set_value(bool* in_value, long in_nvals=1);
  void set_value(short* in_value, long in_nvals=1);
  void set_value(long* in_value, long in_nvals=1);
  void set_value(float* in_value, long in_nvals=1);
  void set_value(double* in_value, long in_nvals=1);
  void set_value(unsigned short* in_value, long in_nvals=1);
  void set_value(unsigned long* in_value, long in_nvals=1);
  void set_value(unsigned char* in_value, long in_nvals=1);
  void set_value(string* in_value, long in_nvals=1);

  // This should not be public.. but Transform should be able to invoke it.
  void set_parent(string in_parent);

  string print();

protected:
  // Parameter definition is pretty much immutable by user after instantiation.
  void set_name(string in_name);
  void set_desc(string in_desc);


private:
  string     name;       // Parameter name
  void*      value;      // Parameter value(s)
  dmDataType datatype;   // datatype of value(s)
  string     desc;       // Description 
  string     parent;     // Name of object this parameter is assoc. with.
  long       nvals;      // Number of values at value pointer.


};

#endif
