require ("pysl");
py_import ("pytransform");

private define _transform_push ( arg ) { return arg; }

%Check for return. Push tuples on the list individually
private define check_return ()
{
  variable ret_val=NULL;
  if (_stkdepth () > 0){
     ret_val = ();
     if (py_call(py.type,ret_val) == py.tuple){
        variable len = py_call (py.len (ret_val))-1;
        variable ii;
        _for (0, len, 1){
            ii = ();
           _transform_push (ret_val[ii]);
        }
     }
  }
  return ret_val;
}

%Create valid types for python
private define _transform_check_type (value);
private define _transform_check_type (value)
{
  if (typeof (value) == Char_Type)
     value = int (value);
  return value;
}
%Turn a structure into a list
private define _transform_struct_to_list (s);
private define _transform_struct_to_list (s)
{
  variable sl_list = {};
  variable name, value;
  variable prefix = NULL;
  if (_NARGS > 1)
     prefix = ();
  try{
     %Go through each element in the structure and make it a list
     foreach (get_struct_field_names (s)){
        name = ();
        value = get_struct_field (s, name);
        if (prefix != NULL)
           name = prefix + "." + name;
        %Expand out sub structures
        if (is_struct_type (value)){
           variable tmp_list = _transform_struct_to_list (name,value);
           variable ii;
           %Concatinate the two lists
           _for (length (tmp_list)-1, 0, -1){
              ii = ();
              list_insert (sl_list, tmp_list[ii], 0);
           }
        }
        else{
           value = _transform_check_type (value);
           list_insert (sl_list,value,0);
           list_insert (sl_list,name,0);
        }
     }
  }
  catch RunTimeError:
     %Return NULL if this was not a structure
     return NULL;
  return sl_list;
}
#if (_slang_version >= 20101)
%Return all the qualifiers in a struct
private define _transform_get_qualifiers (kwstruct);
private define _transform_get_qualifiers (kwstruct)
{
  if( kwstruct == NULL )
    return NULL;
  variable ii, name;
  variable kwnames = get_struct_field_names(kwstruct);
  variable kwargs = Struct_Type[2 * length(kwnames)];
  ii = 0;
  foreach (kwnames)
  {
     name = ();
     kwargs[ii] = struct{value=name};
     ii = ii + 1;
     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};
     ii = ii + 1;
   }
 return kwargs;
 }
#endif
%Transform HLUI Functions
define BUFFER_LENGTH () { return py.pytransform.BUFFER_LENGTH; }

define DT_BIT () { return py.pytransform.DT_BIT; }

define DT_CHAR () { return py.pytransform.DT_CHAR; }

define DT_INT2 () { return py.pytransform.DT_INT2; }

define DT_INT4 () { return py.pytransform.DT_INT4; }

define DT_LOGICAL () { return py.pytransform.DT_LOGICAL; }

define DT_PTR () { return py.pytransform.DT_PTR; }

define DT_REAL4 () { return py.pytransform.DT_REAL4; }

define DT_REAL8 () { return py.pytransform.DT_REAL8; }

define DT_UINT1 () { return py.pytransform.DT_UINT1; }

define DT_UINT2 () { return py.pytransform.DT_UINT2; }

define DT_UINT4 () { return py.pytransform.DT_UINT4; }

define DT_UNK () { return py.pytransform.DT_UNK; }

define DT_VOID () { return py.pytransform.DT_VOID; }

define LINEAR2DTransform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.LINEAR2DTransform, __push_args (f_args));
        else
        py_call(py.pytransform.LINEAR2DTransform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.LINEAR2DTransform);
       else
         py_call(py.pytransform.LINEAR2DTransform, PY_KW,  __push_args(kwargs));
     }
}

define MAX_LENGTH () { return py.pytransform.MAX_LENGTH; }

define PySwigIterator ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.PySwigIterator, __push_args (f_args));
        else
        py_call(py.pytransform.PySwigIterator, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.PySwigIterator);
       else
         py_call(py.pytransform.PySwigIterator, PY_KW,  __push_args(kwargs));
     }
}

define TransParam ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.TransParam, __push_args (f_args));
        else
        py_call(py.pytransform.TransParam, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.TransParam);
       else
         py_call(py.pytransform.TransParam, PY_KW,  __push_args(kwargs));
     }
}

define Transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.Transform, __push_args (f_args));
        else
        py_call(py.pytransform.Transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.Transform);
       else
         py_call(py.pytransform.Transform, PY_KW,  __push_args(kwargs));
     }
}

define WCSTANTransform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.WCSTANTransform, __push_args (f_args));
        else
        py_call(py.pytransform.WCSTANTransform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.WCSTANTransform);
       else
         py_call(py.pytransform.WCSTANTransform, PY_KW,  __push_args(kwargs));
     }
}

define apply_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pytransform.apply_transform, __push_args (f_args));
        else
        _py_call(py.pytransform.apply_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pytransform.apply_transform);
       else
         _py_call(py.pytransform.apply_transform, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define copy_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pytransform.copy_transform, __push_args (f_args));
        else
        _py_call(py.pytransform.copy_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pytransform.copy_transform);
       else
         _py_call(py.pytransform.copy_transform, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define dmFAILURE () { return py.pytransform.dmFAILURE; }

define dmSUCCESS () { return py.pytransform.dmSUCCESS; }

define get_dmType ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.get_dmType, __push_args (f_args));
        else
        py_call(py.pytransform.get_dmType, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.get_dmType);
       else
         py_call(py.pytransform.get_dmType, PY_KW,  __push_args(kwargs));
     }
}

define get_dmType_str ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.get_dmType_str, __push_args (f_args));
        else
        py_call(py.pytransform.get_dmType_str, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.get_dmType_str);
       else
         py_call(py.pytransform.get_dmType_str, PY_KW,  __push_args(kwargs));
     }
}

define get_pyType ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pytransform.get_pyType, __push_args (f_args));
        else
        py_call(py.pytransform.get_pyType, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        py_call(py.pytransform.get_pyType);
       else
         py_call(py.pytransform.get_pyType, PY_KW,  __push_args(kwargs));
     }
}

define get_transform_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _transform_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _transform_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _transform_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pytransform.get_transform_type, __push_args (f_args));
        else
        _py_call(py.pytransform.get_transform_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
        _py_call(py.pytransform.get_transform_type);
       else
         _py_call(py.pytransform.get_transform_type, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

$1 = path_concat (path_dirname (__FILE__), "help/transform_hlui.hlp");
if (NULL != stat_file ($1))
  add_doc_file ($1);
provide ("transform")