#!/usr/bin/env slsh
% 
%  Copyright (C) 2007  Smithsonian Astrophysical Observatory
%
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License along
%  with this program; if not, write to the Free Software Foundation, Inc.,
%  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%


require("transform");

%-----------------------------------------------
%  Setup input/output directories
%-----------------------------------------------
variable testin  = getenv("TESTIN");
if (testin == NULL )
{
  variable indir = getenv("PWD");
}
else
{
  variable indir = testin + "/test_transform";
}

variable testout = getenv("TESTOUT");
if ( testout == "" )
{
  variable outdir = getenv("PWD");
}
else 
{
  variable outdir = testout + "/transform";
}

%-----------------------------------------------
% Test and verify copy_transform()
%-----------------------------------------------

% Create and define a WCS Transform 
variable tr = WCSTANTransform("UNIT");
variable param = py_call( tr.get_parameter, "CRPIX");  % <= NO SLANG CAPABILITY
py_call( param.set_value, [50.0, 50.0]);               % <= NO SLANG CAPABILITY
param = py_call( tr.get_parameter, "CRVAL");           % <= NO SLANG CAPABILITY
py_call( param.set_value, [50.0, 50.0]);               % <= NO SLANG CAPABILITY
param = py_call( tr.get_parameter, "CDELT");           % <= NO SLANG CAPABILITY
py_call( param.set_value, [1.0, 1.0]);                 % <= NO SLANG CAPABILITY

% Show it
printf("Initial Transform definition\n");
printf("Transform Type:  %s\n", get_transform_type(tr));
variable rval = py_call( tr.print_parameter_list );    % <= NO SLANG CAPABILITY
printf(rval);

% Copy it
printf("\n");
printf("Copy Transform\n");
printf("  - change name of copy\n");
variable ntr = copy_transform(tr);
py_call( ntr.set_name, "UNIT_COPY");

% Show copy defintion
printf("\n");
printf("Copy Transform definition\n");
printf("Transform Type:  %s\n", get_transform_type(ntr));
rval = py_call( ntr.print_parameter_list );
printf(rval);

% Show values are not linked..
% Modify initial transform attributes and display both
printf("\n");
printf("Modify Initial Transform\n");
printf("  - NAME\n");
printf("  - CRVAL\n");
printf("  - CROTA\n");
py_call( tr.set_name, "UNIT_MOD");
param = py_call( tr.get_parameter, "CRVAL");
py_call( param.set_value, [88.0, 88.0]);
param = py_call( tr.get_parameter, "CROTA");
py_call( param.set_value, 20.0 );

printf("\n");
printf("Initial Transform definition (modified)\n");
printf("Transform Type:  %s\n", get_transform_type(tr));
rval = py_call( tr.print_parameter_list );
printf(rval);

printf("\n");
printf("Copy Transform definition (unchanged)\n");
printf("Transform Type:  %s\n", get_transform_type(ntr));
rval = py_call( ntr.print_parameter_list );
printf(rval);


%-----------------------------------------------
% Test and verify apply_transform()
%  - extracted from low-level interface.
%-----------------------------------------------
% Data to be transformed.
variable sky = [ [ 984.5, 1020.5],
                 [ 994.5, 1021.5],
                 [1004.5, 1022.5],
                 [1014.5, 1023.5],
                 [1024.5, 1024.5],
                 [1034.5, 1025.5],
                 [1044.5, 1026.5],
                 [1054.5, 1027.5],
                 [1064.5, 1028.5],
                 [1074.5, 1029.5] ];
reshape(sky,[10,2]);

printf("\n");
printf("----------------------------------------------------------------------\n");
printf("Define a 'real' WCS transform\n");

% Define Transform (sky2cel)
variable sky2cel = WCSTANTransform();
param = py_call( sky2cel.get_parameter, "CRPIX");
py_call( param.set_value, [1.0245000000000E+03,1.0245000000000E+03]);
param = py_call( sky2cel.get_parameter, "CRVAL");
py_call( param.set_value, [2.0152004149394E+02,-4.2949051379390E+01]);
param = py_call( sky2cel.get_parameter, "CDELT");
py_call( param.set_value, [-0.000585778,0.0005857778]);
param = py_call( sky2cel.get_parameter, "CROTA");
py_call( param.set_value, 0.0 );
param = py_call( sky2cel.get_parameter, "EQUINOX");
py_call( param.set_value, 2000.0);
param = py_call( sky2cel.get_parameter, "EPOCH");
py_call( param.set_value, 2000.0);

printf("Transform Type:  %s\n", get_transform_type(ntr));
rval = py_call( sky2cel.print_parameter_list );
printf(rval);

printf("\n");
printf("Apply Transform to data array..\n");
variable cel = apply_transform(sky2cel, sky);

variable dims, ndims, dtype;
(dims, ndims, dtype) = array_info(sky);

printf("\n");
printf("  Input array length..  %d\n", dims[0] );
printf("  Input array ndim..  %d\n", ndims);
printf("  Input array shape..  (");
variable ii = 0;
for ( ii = 0; ii < ndims; ii++ )
{
  printf("%2d",dims[ii]);
  if ( ii < ndims - 1 )
  {
    printf(",");
  }
}
printf(")\n");
printf("  Input array dtype..  ");
print( dtype );
printf("\n");

(dims, ndims, dtype) = array_info(cel);
printf("  Output cel values are type...  ");
print(typeof(cel));
printf("  Output array length..  %d\n",  dims[0]);
printf("  Output array ndim..  %d\n", ndims);
printf("  Output array shape..  (");
for ( ii = 0; ii < ndims; ii++ )
{
  printf("%2d",dims[ii]);
  if ( ii < ndims - 1 )
  {
    printf(",");
  }
}
printf(")\n");
printf("  Output array dtype..  ");
print(dtype);
printf("\n");
printf("     ( x,y )     ->    ( RA, DEC )\n");

for( ii=0; ii< dims[0]; ii++)
{
  printf("%7.1f, %7.1f -> %7.3f, %7.3f \n",sky[ii,0], sky[ii,1], cel[ii,0], cel[ii,1]);
}
